/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of the Crystal animation and 3D modeling application
 *
 * Authors:   Tommi Ilmonen 2001-2004
 * Copyright: Tommi Ilmonen and/or Tekes 2001-2004
 *
 * firstname.lastname@hut.fi
 *
 */

#include <crystal_bgl.h>

#include <crystal_engine.h>
#include <crystal_gl_data.h>

#include <vee_error.h>

#include <Fluid/Config.hpp>

#include <iostream>
#include <unistd.h>

namespace Crystal { 
  
  Bgl::Bgl(Engine *engine)
    : m_glData(0),
      m_engine(engine)
  {
    VEE_GlRenderer::createPrivateTextures(false);
    m_statistics = fopen("statistics.txt", "w");
  }

  Bgl::~Bgl()
  {
    if(m_statistics)
      fclose(m_statistics);
  }
  
  void Bgl::init()
  {
    const char * fname = "Bgl::init";

    VEE_trace(fname);

    m_engine->init();

    m_frameCount = 0;
    m_lastFrameCount = 0;
    m_lastFrameTime.getTimeOfDay();  

    
    m_updateUSCount = 0;
    m_renderUSCount = 0;
  }

  void Bgl::preFrame()
  {
    m_frameCount++;

    VEE_Time now;
    now.getTimeOfDay();
    
    m_renderUSCount += m_lastFrameEnd.usecsTo(now);

    double sinceFrameUpdate = m_lastFrameTime.secsTo(now);

    if(sinceFrameUpdate >  5.0) {

      long maxRenderTime = m_glData->m_timings[0];
      for(uint j = 1;j < 4; j++)
	maxRenderTime += m_glData->m_timings[j];
      
      double rate = (double) (m_frameCount - m_lastFrameCount) 
	/ sinceFrameUpdate;
      m_lastFrameTime = now;

      VEE_trace("Bgl::preFrame # Frame rate is %f Hz", rate);
      m_lastFrameCount = m_frameCount;

      if(m_statistics && m_glData) {
	GEE_GlRenderer * g = & m_glData->m_geeRenderer;
	GlData * gld = m_glData;
	fprintf(m_statistics, "%u  %f %ld %ld %ld %ld %ld %ld %ld %ld %ld ", 
		m_frameCount, rate, 
		g->vertexCount(), g->triangleCount(), 
		g->particleCount(), g->geometryCount(),
		(long) gld->m_geeLists.size(),
		g->renderDisplayListCount(), 
		g->generateDisplayListCount(),
		m_updateUSCount, m_renderUSCount);

	for(uint i = 0; i < 6; i++)
	  fprintf(m_statistics, " %.2f", 
		  (float) gld->m_timings[i] * 0.000001f);
	
	fprintf(m_statistics, " %u %.2f ", index, (float) maxRenderTime * 0.000001f);
	
	fprintf(m_statistics, "\n");

	fflush(m_statistics);

	m_updateUSCount = 0;
	m_renderUSCount = 0;
	gld->clearTimings();
      }
    }

    m_lastFrameEnd.getTimeOfDay();

    m_updateUSCount += now.usecsTo(m_lastFrameEnd);

    m_engine->update();

    if(m_engine->isDone())
      quit();
  }

  static int contextCount = 0;

  void Bgl::contextInit()
  {
    const char * fname = "Bgl::contextInit";

    contextCount++;

    GlData *glData = new GlData;

    m_glData = glData;

    VEE_trace("%s # %d %p", fname, contextCount, glData);

    glData->contextInit(m_engine, &m_config);

    glData->lightsOn();


    m_frameCount = 0;
    m_lastFrameCount = 0;
    m_lastFrameTime.getTimeOfDay();  
  
    glFlush();

    VEE_trace("%s # EXIT", fname);
  }

  void Bgl::draw()
  {
    m_glData->m_geeRenderer.resetCounters();

    VEE_GlRenderer *veeGl = & m_glData->m_veeRenderer;
    veeGl->setBlend(true);

    glEnable(GL_BLEND);

    glDisable(GL_LIGHTING);

    m_engine->render(m_glData, veeGl, & m_glData->m_solarRenderer);
    
  }

  void Bgl::loadConfig(const char * filename)
  {
    if(!filename) return;

    ConfigReader::readConfig(& m_config, filename);
  }
}
