/* -*- C++ -*- */

/* -*- C++ -*- */

/* -*- C++ -*- */

/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of the Crystal animation and 3D modeling application
 *
 * Authors:   Tommi Ilmonen 2001-2004
 * Copyright: Tommi Ilmonen and/or Tekes 2001-2004
 *
 * firstname.lastname@hut.fi
 *
 */

#include <crystal_action_list.h>

#include <crystal_action.h>
#include <crystal_component_editor.h>
#include <crystal_dom_util.h>
#include <crystal_engine.h>
#include <crystal_recorder.h>
#include <crystal_tools.h>
#include <crystal_visualize.h>

#include <vee_error.h>
#include <vee_scene.h>

#include <typeinfo>

namespace Crystal { 


  ActionList::ActionList
  (VEE_SceneBox *s, 
   Engine *e,
   Mode m, 
   Recorder * r, 
   Fluid::TimeStamp now,
   int groupId,
   int myId)

    : m_mode(m),
      m_recorder(r),
      m_scene(s),
      m_engine(e),
      m_idCount(0),
      m_meshIdCount(0),
      m_graphIdCount(0),
      m_groupId(groupId),
      m_myId(myId),
      m_actionIdCount(0),
      m_host(0),
      m_visualize(0)
  {
    m_nextAction = m_actions.begin();
  
    m_audioAllocator.expand(3);

    if(r) r->prepare(this);
  
    if(m == RECORD && r) {
      continueRecording(now);
      // r->beginRecording(this, now);
    }
  }

  ActionList::~ActionList()
  {
    VEE_trace("ActionList::~ActionLis # %p", this);

    clear();
    /* if(m_recorder) {
       endRecording((Fluid::TimeStamp::stamp_t) -1);
       }*/
    // delete m_recorder;
  }

  /** Clears the action list. Deletes action elements and removes all
      the objects. */

  void ActionList::clear()
  {
    m_veeObjects.clear();

    while(m_actions.begin() != m_actions.end()) {
      delete (*m_actions.begin());
      m_actions.erase(m_actions.begin());
    }

    m_graphs.clear();

    delete m_visualize;
    m_visualize = 0;
  }

  /** Rewinds the action list to the beginning and prepares for
      recording. */
  void ActionList::rewind(const Fluid::TimeStamp &now)
  {
    VEE_trace("ActionList::rewind # %p", this);

    if(m_recorder) endRecording(now);

    // delete m_recorder;
    m_recorder = 0;

    while(m_veeObjects.size())
      deleteObject((*m_veeObjects.begin()).first);

    m_nextAction = m_actions.begin();

    // m_beginTime = now;
    m_currentActions.clear();

    updateTo(Fluid::TimeStamp(0));
  }


  /** Play the actions until the given time stamp. */
  void ActionList::updateTo(const Fluid::TimeStamp &externalTime)
  {
    const char * fname = "ActionList::updateTo";

    // VEE_trace("ActionList::updateTo # %p %lf", this, t.secondsD());

    if(m_mode == MUTE)
      return;

    Fluid::TimeStamp t = externalTime - m_timeOffset;

    m_currentInternalTime = t;

    // if(m_recorder && m_mode == RECORD)
    if(m_recorder)
      m_recorder->record(this, t);

    // VEE_trace("ActionList::updateTo # 1");

    if(m_nextAction == m_actions.end() && !m_recorder)
      return;

    // VEE_trace("ActionList::updateTo # 1,5");

    if(m_nextAction == m_actions.begin()) {
      if(!m_actions.size()) {
	// VEE_trace("%s # EXIT 2", fname);
	return;
      }
      // m_nextAction++;
    }

    // VEE_trace("ActionList::updateTo # 2, %f", t.secondsF());
 
    // Action *a = (*m_nextAction);

    /* VEE_trace("ActionList::updateTo # 2,5 %p %d %d %s", 
       a, a->getTimeStamp().stamp(), t.stamp(), typeid(*a).name()); */

    // Run the stored actions with longer time-span now:
    for(iterator ai = m_currentActions.begin(); 
	ai != m_currentActions.end(); ) {
      
      Action * a = * ai;

      if(a->getEndTimeStamp() >= t) {
	a->act(this);
	ai++;
      }
      else {

	/* VEE_trace("%s # Popping %p (%s) from curr-Actions",
	   fname, a, typeid(*a).name()); */

	if(ai == m_currentActions.begin()) {
	  m_currentActions.erase(ai);
	  ai = m_currentActions.begin();
	}
	else {
	  iterator tmp = ai;
	  tmp--;
	  m_currentActions.erase(ai);
	  tmp++;
	  ai = tmp;
	}
      }
    }
  
    while(m_nextAction != m_actions.end() && 
	  (*m_nextAction)->getTimeStamp() <= t) {

      /* VEE_trace("ActionList::updateTo # act %p %s %lf", 
	 (*m_nextAction), typeid(**m_nextAction).name(),
	 (*m_nextAction)->getTimeStamp().secondsD());  */
    
      Action * a = *m_nextAction;

      a->act(this);

      if(a->getEndTimeStamp() > t) {
	/* VEE_trace("%s # Adding %p (%s) to curr-Actions",
	   fname, a, typeid(*a).name()); */
	m_currentActions.push_back(a);
      }

      m_nextAction++;
    }

    double td = t.secondsD();
  
    for(graph_iterator it = m_graphs.begin(); it != m_graphs.end(); it++) {
      int id = (*it).first;
      
      ValueGraphs::TargetType type = (*it).second.targetType();
      
      if(type == ValueGraphs::PARTICLE) {
	VEE_Object * obj = getObject(id);
	if(obj) {
	  (*it).second.transmitValues(obj, td);
	}
	else {
	  VEE_error("%s # No object %d at %lf", fname, id, t.secondsD());
		    
	}
      }
      else {
	Model * model = getModel(id);
	if(model) {
	  (*it).second.transmitValues(model, td);
	}
      }
    }
    // VEE_trace("ActionList::updateTo # EXIT");
  }

  /** Ends the recording. Can be called at any time. 
   */
  void ActionList::endRecording(const Fluid::TimeStamp &t)
  {
    if(!m_recorder) return;

    VEE_trace("ActionList::endRecording");

    if(m_mode == RECORD) {
      m_mode = PLAY;
      m_recorder->endRecording(this, t);

      // Scan the remaining elements from the list. 
      while(m_nextAction != m_actions.end()) {
	(*m_nextAction)->act(this);
	m_nextAction++;
      }
    }

    m_recorder = 0;

    if(!m_actions.size())
      return;

    /* Now some dirty stuff: Once the recording is done we change the
       time-stamps to match local timing. */

    iterator it = m_actions.begin();

    Fluid::TimeStamp beginTime = (*it)->getTimeStamp();
    m_timeOffset = beginTime;

    for(; it != m_actions.end(); it++)
      (*it)->setTimeStamp((*it)->getTimeStamp() - beginTime);

    double beginTimeD = beginTime.secondsD();
  
    for(graph_iterator it = m_graphs.begin(); it != m_graphs.end(); it++)
      (*it).second.moveTime(-beginTimeD);
  }

  void ActionList::continueRecording(const Fluid::TimeStamp &now)
  {
    if(!m_recorder) return;

    VEE_trace("ActionList::continueRecording");
  
    m_timeOffset = 0;

    m_recorder->beginRecording(this, now);
    updateTo(now);
    m_mode = RECORD;
  }

  /** Get one of the currently active objects. 

  @return This method returns either the corresponding object or
  NULL it the object could not be found.

  */ 

  VEE_Object *ActionList::getObject(int id)
  {
    std::map<int,VEE_Object*>::iterator it = m_veeObjects.find(id);

    if(it == m_veeObjects.end()) {
      // VEE_error("ActionList::getObject # Failed to find object %d", id);
      return 0;
    }

    return (*it).second;
  }

  /** Get one of the currently active models. 

  @return This method returns either the corresponding object or
  NULL it the object could not be found.

  */ 

  Model *ActionList::getModel(int id)
  {
    std::map<int,Model*>::iterator it = m_models.find(id);

    if(it == m_models.end()) {
      // VEE_error("ActionList::getModel # Failed to find object %d", id);
      return 0;
    }
    
    return (*it).second;
  }

  void ActionList::setModel(int id, Model *obj)
  {
    m_models[id] = obj;
  } 
  /** This method adds a new object to the object collection. If there
      exists an object with the same id, then the old object is
      deleted. */

  void ActionList::setObject(int id, VEE_Object *obj)
  {
    VEE_trace("ActionList::setObject # %d %p", id, obj);

    std::map<int,VEE_Object*>::iterator it = m_veeObjects.find(id);

    if(it != m_veeObjects.end()) {
      delete (*it).second;
      (*it).second = obj;
    }
    else
      m_veeObjects[id] = obj;
  }

  /** Forgets one of the current objects. */
  void ActionList::forgetObject(int id)
  {
    std::map<int,VEE_Object*>::iterator it = m_veeObjects.find(id);

    if(it != m_veeObjects.end())
      m_veeObjects.erase(it);
  }

  /** Removes an object from the action list (if it exists). 
   */
  void ActionList::deleteObject(int id)
  {
    VEE_Object *o = getObject(id);

    if(o) {
      if(o->sceneBox()) {

	// VEE_ForceObject *force = dynamic_cast<VEE_ForceObject *> (o);
	/* if(force)
	   force->sceneBox()->removeChildForceObject(force);*/
	// else
	o->sceneBox()->removeChildObject(o);
      }

      delete o;
      forgetObject(id);
    }
  }

  /// Appends an action to the end of list
  void ActionList::appendAction(Action *a)
  {
    VEE_trace("ActionList::appendAction # %p %3.2f %s", 
	      a, a->getTimeStamp().secondsF(), typeid(*a).name());

    // Be careful not to corrupt our list iterator.

    if(m_nextAction == m_actions.end()) {
      m_actions.push_back(a);
      m_nextAction = m_actions.end();
      m_nextAction--;
    }
    else if(m_nextAction == m_actions.begin()) {
      m_actions.push_back(a);
      m_nextAction = m_actions.begin();
    }
    else {
      m_actions.push_back(a);
    }

    if(a->specialId() >= 0)
      m_actionMap[a->specialId()] = a;
  }

  double ActionList::beginTime()
  {
    /* double t = 0.0;

    if(m_actions.size())
    t = (*m_actions.begin())->getTimeStamp().secondsD(); */

    return m_timeOffset.secondsD();
  }

  double ActionList::endTime() 
  {
    double t = 0.0;

    if(m_actions.size()) {

      iterator it = m_actions.end();
      it--;
      t = (*it)->getTimeStamp().secondsD();
    }

    return t + m_timeOffset.secondsD();
  }

  /** Allocates local audio id. */

  int ActionList::allocateAudioId()
  { 
    return m_audioAllocator.allocateChannel(); 
  }

  /** Deallocates local audio id and optionally also the related global
      id. */
  void ActionList::deallocateAudioId(int id, bool alsoGlobalId) 
  {
    m_audioAllocator.deallocateChannel(id);

    std::map<int, int>::iterator it = m_audioIds.find(id);

    if(it != m_audioIds.end()) {

      if(alsoGlobalId) {
	int globalId = (*it).second;
	m_host->engine()->deallocateAudioSource(globalId);

	VEE_trace("ActionList::deallocateAudioId # %d %d", id, globalId);
      }

      m_audioIds.erase(it);
    }
  }

  /** Turns the local id to global id. If the id does not exist, then
      the function tries to allocate a nwe global id. */

  int ActionList::audioIdToChannel(int id)
  {
    std::map<int, int>::iterator it = m_audioIds.find(id);

    if(it == m_audioIds.end()) {
      int globalId = m_host->engine()->allocateAudioSource();
      if(globalId < 0)
	return -1;

      setAudioIdMapping(id, globalId);
      return globalId;
    }

    return (*it).second;
  }

  void ActionList::setAudioIdMapping(int localId, int globalId)
  {
    m_audioIds[localId] = globalId;
  }

  QDomElement ActionList::writeDom(QDomDocument & doc)
  {
    QDomElement elem = doc.createElement("ActionList");

    // Store actions:
    QDomElement actions = doc.createElement("actions");

    for(iterator it = m_actions.begin(); it != m_actions.end(); it++) {
      QDomElement node = (*it)->writeDom(doc);
      int sid = (*it)->specialId();
      if(sid >= 0)
	addTextChild(node, doc, "special_id", sid);
      
      actions.appendChild(node);
    }

    elem.appendChild(actions);

    // Store the value graphs:
    QDomElement graphs = doc.createElement("graphs");

    for(graph_iterator git = m_graphs.begin(); git != m_graphs.end(); git++) {
      QDomElement node = (*git).second.writeDom(doc);
      addTextChild(node, doc, "id", (*git).first);
      graphs.appendChild(node);
    }

    elem.appendChild(graphs);
  
    if(m_visualize) {
      QDomElement visual = doc.createElement("visual");
    
      visual.appendChild(m_visualize->writeDom(doc));
    
      elem.appendChild(visual);
    }

    addTextChild(elem, doc, "my_id", m_myId);
    addTextChild(elem, doc, "offset", m_timeOffset);

    return elem;
  }

  bool ActionList::readDom(QDomElement &elem)
  {
    if(elem.tagName() != "ActionList")
      return false;

    // actions:
    QDomElement actions = elem.namedItem("actions").toElement();

    if(actions.isNull())
      return false;

    QDomNode n = actions.firstChild();
    while ( !n.isNull() ) {
      if ( n.isElement() ) {
	QDomElement e = n.toElement();
	QString tag = e.tagName();
	Action * a = Action::create(tag.ascii());

	if(!a)
	  VEE_error("ActionList::readDom # failed to create \"%s\"",
		    tag.ascii());
	else {
	  if(a->readDom(e)) {
	    int sid = getInt(e, "special_id", -1);
	    if(sid >= 0)
	      a->setSpecialId(sid);
	    appendAction(a);
	  }
	  else
	    delete a;
	}
      }
      n = n.nextSibling();
    }

    QDomElement graphs = elem.namedItem("graphs").toElement();

    if(graphs.isNull())
      return false;

    n = graphs.firstChild();
    while ( !n.isNull() ) {
      if ( n.isElement() ) {
	QDomElement e = n.toElement();
	ValueGraphs g;
	if(g.readDom(e)) {
	  int id = getInt(e, "id", -1);
	  if(id < 0)
	    VEE_error("ActionList::readDom # graphs id = %d", id);
	  m_graphs[id] = g;
	}
      }
      n = n.nextSibling();
    }

    VEE_trace("RYSTAL_ActionList::readDom # %u lists, %u graphs",
	      m_actions.size(), m_graphs.size());

    QDomElement visual = elem.namedItem("visual").toElement();

    if(!visual.isNull()) {

      QDomElement n = visual.firstChild().toElement();
      QString id = n.tagName();
      Visualize * v = Visualize::create(id.ascii());
      if(v) {
	v->readDom(n);
	setVisualize(v);
      }
    }

    m_myId = getInt(elem, "my_id", -1);
    m_timeOffset = getTimeStamp(elem, "offset");

    return true;
  }

  int ActionList::addMappedAction(Action *a)
  {
    m_actionMap[m_actionIdCount] = a;

    int r = m_actionIdCount++;
    a->setSpecialId(r);
    return r;
  }

  Action * ActionList::getMappedAction(int id)
  {
    std::map<int, Action *>::iterator it = m_actionMap.find(id);

    if(it == m_actionMap.end())
      return 0;

    return (*it).second;
  }


  void ActionList::visualize(GlData *gld)
  {
    if(!m_visualize) return;

    m_visualize->visualize(this, gld);
  }

  void ActionList::setVisualize(Visualize *v)
  {
    delete m_visualize;
    m_visualize = v;
    if(v) v->setHost(this);
  }

  void ActionList::moveTimeOffset(const Fluid::TimeStamp &offset) 
  { 
    m_timeOffset += offset;
    if(m_timeOffset <  Fluid::TimeStamp(0))
      m_timeOffset = 0;
  }

  void ActionList::render(GlData *gld)
  {
    if(m_recorder)
      m_recorder->render(m_engine, gld);
  }

  /////////////////////////////////////////////////////////////////////////////
  /////////////////////////////////////////////////////////////////////////////

  ActionListManager::ActionListManager(Engine *engine)
    : m_engine(engine),
      m_recorder(0),
      m_recordList(0),
      m_idCount(0),
      m_groupCount(0),
      m_newGroup(true)
  {}

  ActionListManager::~ActionListManager()
  {
    clear();
  }

  void ActionListManager::updateTo(const Fluid::TimeStamp &t)
  {
    for(iterator i = m_lists.begin(); i != m_lists.end(); i++) {
      (*i)->updateTo(t);
    }
  }

  void ActionListManager::rewind(const Fluid::TimeStamp &now)
  {
    if(m_recordList)
      m_recordList->endRecording(now);

    m_recordList = 0;
  
    // We should delete the list if it is empty. RRR

    VEE_trace("ActionListManager::rewind # %p %u", this, m_lists.size());

    for(iterator i = m_lists.begin(); i != m_lists.end(); i++) {
      (*i)->rewind(now);
    }

    delete m_recorder;
    m_recorder = 0;

    m_newGroup = true;
  }


  void ActionListManager::addActionList(ActionList *list)
  { 
    m_lists.push_back(list); 
    list->setHost(this); 
    m_listMap[list->myId()] = list; 
  }

  ActionList * ActionListManager::getList(int id)
  {
    std::map<int, ActionList *>::iterator it = m_listMap.find(id);

    if(it == m_listMap.end())
      return 0;

    return (*it).second;
  }


  void ActionListManager::pauseRecording(const Fluid::TimeStamp &t)
  {
    if(!m_recorder) 
      return;
 
    VEE_trace("ActionListManager::pauseRecording");

    if(m_recordList) {
      m_recordList->endRecording(t);

      m_recordList = 0;

      m_engine->tools()->componentEditor()->redesignLayout();
    }

    m_recordList = new ActionList(m_engine->scene(),
				  m_engine,
				  ActionList::PLAY,
				  m_recorder,
				  t,
				  m_groupCount - 1,
				  ++m_idCount); // RRR ???

    addActionList(m_recordList);
  } 

  void ActionListManager::continueRecording(const Fluid::TimeStamp &zero,
					    const Fluid::TimeStamp &now)
  {
    VEE_trace("ActionListManager::continueRecording");

    if(!m_recorder) 
      return;

    if(m_recordList)
      m_recordList->endRecording(now);

    if(m_newGroup)
      m_groupCount++;

    m_newGroup = false;

    // if(!m_recordList) {
    m_recordList = new ActionList(m_engine->scene(),
				  m_engine,
				  ActionList::PLAY,
				  m_recorder,
				  now,
				  m_groupCount - 1,
				  ++m_idCount); // RRR ???

    m_timing.m_zeroTime = zero;
    addActionList(m_recordList);
    m_recordList->continueRecording(now);

    m_engine->tools()->componentEditor()->addList(m_recordList);
  }

  void ActionListManager::setRecorder(Recorder *r)
  { 
    delete m_recorder;
    m_recorder = r; 

    pauseRecording(0);
  } 

  QDomElement ActionListManager::writeDom(QDomDocument &doc)
  {
    QDomElement elem = doc.createElement("ActionListManager");

    if(m_lists.size()) {
      QDomElement lists = doc.createElement("lists");
    
      for(iterator it = m_lists.begin(); it != m_lists.end(); it++) {
	QDomElement node = (*it)->writeDom(doc);
	lists.appendChild(node);
      }
  
      elem.appendChild(lists);
    }

    addTextChild(elem, doc, "groups", m_groupCount);

    return elem;
  }

  bool ActionListManager::readDom(QDomElement &elem)
  {
    if(elem.tagName() != "ActionListManager")
      return false;

    QDomElement lists = elem.namedItem("lists").toElement();

    clear();
  
    QDomNode n = lists.firstChild();

    while ( !n.isNull() ) {
      if ( n.isElement() ) {
	QDomElement e = n.toElement();
	ActionList * list =
	  new ActionList(m_engine->scene(),
			 m_engine,
			 ActionList::PLAY, 0,
			 Fluid::TimeStamp(0), 0, -1); // RRR FIX 
	bool ok = list->readDom(e);

	if(ok)
	  addActionList(list);
	else
	  delete list;
      }
      n = n.nextSibling();
    }
  
    VEE_trace("ActionListManager::readDom #  got %u lists",
	      m_lists.size());

    return true;
  }

  void  ActionListManager::clear()
  {
    for(iterator i = m_lists.begin(); i != m_lists.end(); i++) {
      delete (*i);
    }

    m_lists.clear();
    m_listMap.clear();
  }

  void ActionListManager::render(GlData *gld)
  {
    if(m_recordList)
      m_recordList->render(gld);

    if(m_recorder)
      m_recorder->render(m_engine, gld);
  }

  
}
