/* -*- C++ -*- */

/* -*- C++ -*- */

/* -*- C++ -*- */

/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of the Crystal animation and 3D modeling application
 *
 * Authors:   Tommi Ilmonen 2001-2004
 * Copyright: Tommi Ilmonen and/or Tekes 2001-2004
 *
 * firstname.lastname@hut.fi
 *
 */

#ifndef CRYSTAL_ACTION_LIST_H
#define CRYSTAL_ACTION_LIST_H

#include <crystal_audio_allocator.h>
#include <crystal_model.h>
#include <crystal_value_graphs.h>
#include <crystal_timing.h>

#include <Fluid/TimeStamp.hpp>

#include <list>
#include <map>

#include <qdom.h> 

class VEE_Object;
class VEE_SceneBox;

namespace Crystal { 

  class Action;
  class ActionListManager;
  class GlData;
  class Engine;
  class Recorder;
  class Visualize;

  class ActionList
  {
  public:
    enum Mode {
      PLAY,
      RECORD,
      MUTE
    };

    typedef std::list<Action *>::iterator iterator;
    typedef std::map<int, ValueGraphs>::iterator graph_iterator;

    ActionList(VEE_SceneBox *s, 
	       Engine *e,
	       Mode m, 
	       Recorder * r, 
	       Fluid::TimeStamp now,
	       int groupId,
	       int myId);
    ~ActionList();

    void clear();

    void rewind(const Fluid::TimeStamp &now);
    void updateTo(const Fluid::TimeStamp &t);
    void endRecording(const Fluid::TimeStamp &now);
    void continueRecording(const Fluid::TimeStamp &now);

    VEE_Object *getObject(int id);
    void setObject(int id, VEE_Object *obj);

    Model      *getModel(int id);
    void setModel(int id, Model *obj);


    int allocateObjectId() { m_idCount++; return m_idCount; }
    int allocateMeshId() { m_meshIdCount++; return m_meshIdCount; }

    /// Drops pointers to some object without deleting the object.
    void forgetObject(int id);
    void deleteObject(int id);

    void setScene(VEE_SceneBox *s) { m_scene = s; }
    VEE_SceneBox *getScene() { return m_scene; }

    Engine *engine() { return m_engine; }

    void appendAction(Action *);

    ValueGraphs & getGraph(int i) { return m_graphs[i]; }

    void setHost(ActionListManager *host) { m_host = host; }
    ActionListManager *host() { return m_host; }
    const ActionListManager *host() const { return m_host; }

    inline Fluid::TimeStamp realTimeToRecordTime
    (const Fluid::TimeStamp & t) const;

    void setMode(Mode m) { m_mode = m; }

    int groupId() const { return m_groupId; }

    double beginTime();
    double endTime();

    int allocateAudioId();
    void deallocateAudioId(int i, bool alsoGlobalId);

    int audioIdToChannel(int id);
    void setAudioIdMapping(int localId, int globalId);

    QDomElement writeDom(QDomDocument &);
    bool readDom(QDomElement &);

    int  myId() const { return m_myId; }

    int addMappedAction(Action *);
    Action * getMappedAction(int);
  
    void visualize(GlData *gld);
    void setVisualize(Visualize *);
    Visualize * visualization() { return m_visualize; }

    const Fluid::TimeStamp & currentInternalTime() const
    { return m_currentInternalTime; }

    const Fluid::TimeStamp & timeOffset() const { return m_timeOffset; }
    void moveTimeOffset(const Fluid::TimeStamp &offset);

    std::map<int, ValueGraphs> & graphs() { return m_graphs; }

    iterator begin() { return m_actions.begin(); }
    iterator end()   { return m_actions.end(); }

    void render(GlData *);

  protected:

    Mode            m_mode;
    Recorder       *m_recorder;

    VEE_SceneBox   *m_scene;
    Engine         *m_engine;
    int             m_idCount;
    int             m_meshIdCount;
    int             m_graphIdCount;
    int             m_groupId;

    int             m_myId;

    int             m_actionIdCount;

    TimeStamp       m_timeOffset;
    TimeStamp       m_currentInternalTime;

    ActionListManager *m_host;

    std::map<int, VEE_Object*>   m_veeObjects;
    std::map<int, Model*>        m_models;
    std::map<int, Action *>      m_actionMap;
    std::map<int, ValueGraphs>   m_graphs;
  
    std::map<int, int>          m_audioIds;
    AudioAllocator              m_audioAllocator;

    std::list<Action *>         m_actions;
    std::list<Action *>         m_currentActions;
    iterator                    m_nextAction;

    Visualize                  *m_visualize;

  private:
    // Disabled:
    ActionList(const ActionList & ) {}
    ActionList & operator = (const ActionList & ) 
    { return * this; }
  };

  /////////////////////////////////////////////////////////////////////////////
  /////////////////////////////////////////////////////////////////////////////

  class ActionListManager
  {
  public:
    typedef std::list<ActionList *>::iterator iterator;

    ActionListManager(Engine *);
    ~ActionListManager();

    void updateTo(const Fluid::TimeStamp &t);
    void rewind(const Fluid::TimeStamp &now);

    void addActionList(ActionList *list);
    ActionList * getList(int);

    void pauseRecording(const Fluid::TimeStamp &t);
    void continueRecording(const Fluid::TimeStamp &zero,
			   const Fluid::TimeStamp &now);

    void setRecorder(Recorder *r);

    Fluid::TimeStamp realTimeToRecordTime(const Fluid::TimeStamp & t) const
    { return m_timing.realTimeToRecordTime(t); }

    void setTiming(const Timing & t)
    { m_timing = t; }

    Engine     *engine() { return m_engine; }

    QDomElement writeDom(QDomDocument &);
    bool readDom(QDomElement &);

    void clear();

    void render(GlData *);

  protected:
    std::list<ActionList *> m_lists;
    std::map<int, ActionList *> m_listMap;

    Engine     *m_engine;
    Recorder   *m_recorder;
    ActionList *m_recordList;

    Timing    m_timing;

    int               m_idCount;
    int               m_groupCount;
    bool              m_newGroup;
  private:
    // Disabled:
    ActionListManager(const ActionListManager & ) {}
    ActionListManager & operator = (const ActionListManager & ) 
    { return * this; }
  };

  inline Fluid::TimeStamp ActionList::realTimeToRecordTime
  (const Fluid::TimeStamp & t) const
  { 
    return m_host->realTimeToRecordTime(t); 
  }

}

#endif
