/* -*- C++ -*- */

/* -*- C++ -*- */

/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of the Crystal animation and 3D modeling application
 *
 * Authors:   Tommi Ilmonen 2001-2004
 * Copyright: Tommi Ilmonen and/or Tekes 2001-2004
 *
 * firstname.lastname@hut.fi
 *
 */

#include <crystal_audio_action.h>

#include <crystal_action_list.h>
#include <crystal_dom_util.h>
#include <crystal_engine.h>

#include <vee_error.h>
#include <vee_object.h>

#include <ac_control.h>
#include <ac_vr_control.h>

namespace Crystal { 

  AudioActionNoteon::~AudioActionNoteon()
  {}

  /////////////////////////////////////////////////////////////////////////////
  /////////////////////////////////////////////////////////////////////////////

  AudioActionNoteoff::~AudioActionNoteoff()
  {}

  void AudioActionNoteoff::act(ActionList *l)
  {
    Engine *engine = l->engine();
    AC_Control *ac = engine->acControl();

    Action * act = l->getMappedAction(m_noteOnActionId);

    if(!act) return;
  
    AudioActionNoteon *ano = 
      dynamic_cast<AudioActionNoteon *> (act);

    if(ac && ano) 
      ac->noteOff(ano->noteId());

    VEE_trace("AudioActionNoteoff::act # %p %d", 
	      l, ano->noteId());

    if(m_deallocateSource) {
      l->deallocateAudioId(ano->sourceId(), true);
      VEE_trace("AudioActionNoteoff::act # Deallocated source %d",
		ano->sourceId());
    }
  }

  QDomElement AudioActionNoteoff::writeDom(QDomDocument &doc)
  {
    QDomElement elem = doc.createElement("AudioActionNoteoff");
  
    writeDomTimeStamp(elem, doc);


    addTextChild(elem, doc, "note_action_id", m_noteOnActionId);
    addTextChild(elem, doc, "deallocate_source", m_deallocateSource);
  
    return elem;
  }

  bool AudioActionNoteoff::readDom(QDomElement &elem)
  {
    if(elem.tagName() != "AudioActionNoteoff")
      return false;

    readDomTimeStamp(elem);

    m_noteOnActionId   = getInt(elem, "note_action_id", -1);
    m_deallocateSource = getInt(elem, "deallocate_source", 0);

    return true;
  }

  /////////////////////////////////////////////////////////////////////////////
  /////////////////////////////////////////////////////////////////////////////

  AudioActionNoteParameter::~AudioActionNoteParameter()
  {}

  void AudioActionNoteParameter::act(ActionList *l)
  {
    Engine *engine = l->engine();
    AC_Control *ac = engine->acControl();

    if(!ac || !m_noteOn) return;
  
    int noteId = m_noteOn->noteId();

    if(noteId < 0) {
      VEE_error("AudioActionNoteParameter::act # noteId = %d", noteId);
    }

    ac->setNoteParameter(noteId, m_param.c_str(), m_value);
  }

  QDomElement AudioActionNoteParameter::writeDom(QDomDocument &)
  {
    VEE_error("AudioActionNoteParameter::writeDom # UNIMPLEMENTED");
    return QDomElement();
  }

  bool AudioActionNoteParameter::readDom(QDomElement &)
  {
    return false;
  }

  /////////////////////////////////////////////////////////////////////////////
  /////////////////////////////////////////////////////////////////////////////

  AudioActionNoise::~AudioActionNoise()
  {}

  void AudioActionNoise::act(ActionList *l)
  {
    Engine *engine = l->engine();
    AC_Control *ac = engine->acControl();

    if(!ac) return;

    int globalId = l->audioIdToChannel(m_sourceId);
  
    m_noteId = ac->playNoise(globalId, 
			     m_gain * engine->particleVolume(),
			     m_centerHz, m_octaves);

    if(m_noteId < 0) {
      VEE_error("AudioActionNoise::act # noteId < 0");
    }
  }
  
  QDomElement AudioActionNoise::writeDom(QDomDocument &doc)
  {
    QDomElement elem = doc.createElement("AudioActionNoise");
  
    writeDomTimeStamp(elem, doc);

    addTextChild(elem, doc, "gain", QString().setNum(m_gain));
    addTextChild(elem, doc, "center_hz", QString().setNum(m_centerHz));
    addTextChild(elem, doc, "octaves", QString().setNum(m_octaves));
    addTextChild(elem, doc, "source_id", QString().setNum(m_sourceId));

    return elem;
  }

  bool AudioActionNoise::readDom(QDomElement &elem)
  {
    if(elem.tagName() != "AudioActionNoise")
      return false;

    readDomTimeStamp(elem);

    m_gain     = getFloat(elem, "gain", 0.5);
    m_centerHz = getFloat(elem, "center_hz", 200);
    m_octaves  = getFloat(elem, "octaves", 0.5);
    m_sourceId = getInt(elem, "source_id", 0);

    return true;
  }

  /////////////////////////////////////////////////////////////////////////////
  /////////////////////////////////////////////////////////////////////////////

  AudioActionGranular::~AudioActionGranular()
  {}

  void AudioActionGranular::act(ActionList *l)
  {
    Engine *engine = l->engine();
    AC_Control *ac = engine->acControl();

    if(!ac) return;

    int globalId = l->audioIdToChannel(m_sourceId);
  
    m_noteId = ac->playGranular(globalId, m_sample.c_str(), 
				m_gain * engine->particleVolume());

    if(m_noteId < 0) {
      VEE_error("AudioActionGranular::act # noteId < 0");
    }
    VEE_trace("AudioActionGranular::act # DONE %p \"%s\" %d (%d %d)", 
	      this, m_sample.c_str(), m_noteId, m_sourceId, globalId);
  }

  QDomElement AudioActionGranular::writeDom(QDomDocument &doc)
  {
    QDomElement elem = doc.createElement("AudioActionGranular");
  
    writeDomTimeStamp(elem, doc);

    addTextChild(elem, doc, "sample", m_sample.c_str());
    addTextChild(elem, doc, "gain", QString().setNum(m_gain));
    addTextChild(elem, doc, "source_id", QString().setNum(m_sourceId));

    return elem;
  }

  bool AudioActionGranular::readDom(QDomElement &elem)
  {
    if(elem.tagName() != "AudioActionGranular")
      return false;

    readDomTimeStamp(elem);

    m_sample = elem.namedItem("sample").firstChild().toText().data().ascii();
    m_gain = getFloat(elem, "gain", 0.5);
    m_sourceId = getInt(elem, "source_id", 0);

    return true;
  }

  /////////////////////////////////////////////////////////////////////////////
  /////////////////////////////////////////////////////////////////////////////

  AudioActionSample::~AudioActionSample()
  {}

  void AudioActionSample::act(ActionList *l)
  {
    Engine *engine = l->engine();
    AC_VrControl *ac = engine->acVrControl();

    if(!ac) return;

    int globalId = engine->allocateAudioSource();
  
    AC_Vector3 loc(m_location.data());

    ac->playSample(globalId, m_sample.c_str(), 
		   m_gain * engine->particleVolume(), 
		   loc, false);

    engine->deallocateAudioSource(globalId);  
  }

  QDomElement AudioActionSample::writeDom(QDomDocument &doc)
  {
    QDomElement elem = doc.createElement("AudioActionSample");
  
    writeDomTimeStamp(elem, doc);

    addTextChild(elem, doc, "sample", m_sample.c_str());
    addTextChild(elem, doc, "gain", m_gain);
    addTextChild(elem, doc, "source_id", m_sourceId);

    addTextChild(elem, doc, "location", m_location);

    return elem;
  }

  bool AudioActionSample::readDom(QDomElement &elem)
  {
    if(elem.tagName() != "AudioActionSample")
      return false;

    readDomTimeStamp(elem);

    m_sample = elem.namedItem("sample").firstChild().toText().data().ascii();
    m_gain = getFloat(elem, "gain", 0.5);
    m_sourceId = getInt(elem, "source_id", 0);

    m_location = getVector3(elem, "location");
  
    return true;
  }

  /////////////////////////////////////////////////////////////////////////////
  /////////////////////////////////////////////////////////////////////////////

  AudioActionMessage::~AudioActionMessage()
  {}

  void AudioActionMessage::act(ActionList *l)
  {
    Engine *engine = l->engine();
    AC_Control *ac = engine->acControl();

    if(!ac) return;
  
    ac->sendMjMessage(m_holder.id(), m_holder.message());
  }

  void AudioActionMessage::set(const char *address, MJ_Message *m)
  {
    m_holder.setId(address);
    m_holder.setMessage(m);
  }

  QDomElement AudioActionMessage::writeDom(QDomDocument &)
  {
    VEE_error("AudioActionMessage::writeDom # UNIMPLEMENTED");
    return QDomElement();
  }

  bool AudioActionMessage::readDom(QDomElement &)
  {
    return false;
  }


  /////////////////////////////////////////////////////////////////////////////
  /////////////////////////////////////////////////////////////////////////////

  AudioActionSourceLocation::~AudioActionSourceLocation()
  {}

  void AudioActionSourceLocation::act(ActionList *l)
  {
    Engine *engine = l->engine();
    AC_VrControl *avr = engine->acVrControl();

    if(!avr || m_sourceId < 0) return;
  
    AC_Vector3 loc(m_location.data());

    avr->moveSource(l->audioIdToChannel(m_sourceId), 0.1, loc);
  }


  QDomElement AudioActionSourceLocation::writeDom(QDomDocument &)
  {
    VEE_error("AudioActionSourceLocation::writeDom # UNIMPLEMENTED");
    return QDomElement();
  }

  bool AudioActionSourceLocation::readDom(QDomElement &)
  {
    return false;
  }

  /////////////////////////////////////////////////////////////////////////////
  /////////////////////////////////////////////////////////////////////////////

  AudioActionSourceLocation2::~AudioActionSourceLocation2()
  {}

  void AudioActionSourceLocation2::act(ActionList *l)
  {
    Engine *engine = l->engine();
    AC_VrControl *avr = engine->acVrControl();

    if(!avr || m_sourceId < 0) return;

    VEE_Object * obj = l->getObject(m_objectId);
  
    if(!obj) return;

    AC_Vector3 loc(obj->location().data());

    avr->moveSource(l->audioIdToChannel(m_sourceId), 0.1, loc);

    VEE_trace("AudioActionSourceLocation2::act # %p", obj);
  }

  QDomElement AudioActionSourceLocation2::writeDom(QDomDocument &doc)
  {
    QDomElement elem = doc.createElement("AudioActionSourceLocation2");
  
    writeDomTimeStamp(elem, doc);

    addTextChild(elem, doc, "source_id", m_sourceId);
    addTextChild(elem, doc, "object_id", m_objectId);
  
    return elem;
  }

  bool AudioActionSourceLocation2::readDom(QDomElement &elem)
  {
    if(elem.tagName() != "AudioActionSourceLocation2")
      return false;

    readDomTimeStamp(elem);

    m_sourceId = getInt(elem, "source_id", -1);
    m_objectId = getInt(elem, "object_id", -1);
  
    return true;
  }

  /////////////////////////////////////////////////////////////////////////////
  /////////////////////////////////////////////////////////////////////////////

  AudioActionControl::~AudioActionControl()
  {}

  void AudioActionControl::act(ActionList *l)
  {
    VEE_trace("AudioActionControl::act");

    Engine *engine = l->engine();
    AC_Control *ac = engine->acControl();

    // First get a handle to the note
    Action * act = l->getMappedAction(m_noteOnId);

    if(!act || ! ac) {
      VEE_trace("AudioActionControl::act # !act || ! ac");
      return;
    }
    AudioActionNoteon *ano = 
      dynamic_cast<AudioActionNoteon *> (act);

    if(!ano) {
      VEE_trace("AudioActionControl::act # ano = NULL");
    
      return;
    }

    long noteId = ano->noteId();
    
    // Now get the parameter from the object:

    ValueGraphs & vg = l->getGraph(m_objectId);

    ValueGraphBase * vgb = vg.findGraph(m_objectParam);

    ValueGraphFloat * vgf = dynamic_cast<ValueGraphFloat *> (vgb);

    if(!vgf) {
      VEE_trace("AudioActionControl::act # vgf = NULL");
      return;
    }
    
    // VEE_trace("DO 1");

    float time = l->currentInternalTime().secondsF();
    float objParam = vgf->m_graph.getValue(time);

    float noteParam = objParam;
    if(m_mapper.ptr())
      noteParam = m_mapper.ptr()->map(noteParam);

    // Finally set the control value of the note:

    VEE_trace("DO 2 %f %s", noteParam, m_audioParam.c_str());

    ac->sendMjNoteControl(noteId, m_audioParam.c_str(), noteParam);

    // VEE_trace("AudioActionControl::act # EXIT");    
  }
  
  QDomElement AudioActionControl::writeDom(QDomDocument &doc)
  {
    QDomElement elem = doc.createElement("AudioActionControl");
  
    writeDomTimeStamp(elem, doc);

    addTextChild(elem, doc, "note_on_id", m_noteOnId);
    addTextChild(elem, doc, "object_id", m_objectId);

    addTextChild(elem, doc, "object_param", m_objectParam.c_str());
    addTextChild(elem, doc, "note_param",   m_audioParam.c_str());

    if(m_mapper.ptr()) {      
      QDomElement elem2 = doc.createElement("mapper");
      
      elem2.appendChild(m_mapper.ptr()->writeDom(doc));
      elem.appendChild(elem2);
    }

    return elem;
  }

  bool AudioActionControl::readDom(QDomElement &elem)
  {
    if(elem.tagName() != "AudioActionControl")
      return false;

    readDomTimeStamp(elem);

    m_noteOnId = getInt(elem, "note_on_id", -1);
    m_objectId = getInt(elem, "object_id", -1);

    
    m_objectParam = elem.namedItem("object_param").
      firstChild().toText().data().ascii();
    
    m_audioParam = elem.namedItem("note_param").
      firstChild().toText().data().ascii();

    QDomElement mapper = elem.namedItem("mapper").toElement();

    if(mapper.isNull()) {
      m_mapper = 0;
    }
    else {
      QDomElement mdata = mapper.firstChild().toElement();

      QString tag = mdata.tagName();      
      MJ_Mapper * m = MJ_Mapper::create(tag.ascii());

      m->readDom(mdata);

      m_mapper = m;
    }

    VEE_trace("AudioActionControl::readDom # %d %d %s %s %p", 
	      m_noteOnId, m_objectId, 
	      m_objectParam.c_str(), m_audioParam.c_str(), 
	      m_mapper.ptr());

    return true;
  }

}
