/* -*- C++ -*- */

/* -*- C++ -*- */

/* -*- C++ -*- */

/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of the Crystal animation and 3D modeling application
 *
 * Authors:   Tommi Ilmonen 2001-2004
 * Copyright: Tommi Ilmonen and/or Tekes 2001-2004
 *
 * firstname.lastname@hut.fi
 *
 */

#ifndef CRYSTAL_AUDIO_ACTION_H
#define CRYSTAL_AUDIO_ACTION_H

#include <crystal_action.h>

#include <mj_mapper.h>
#include <mj_message_carrier.h>

#include <Fluid/Vector3.hpp>

#include <vee_clonable_pointer.h>

#include <string>

namespace Crystal { 

  /** Base class for note-on actions. This class offers two variables 1)
      "noteId" that is used to identify the created note and 2) sourceId
      that idenfities the source. Most other note control action-objects
      will most probably need to store a pointer to this object to be
      effective since the noteId cannot be determined before the note-on
      object has actually done its work. */

  class AudioActionNoteon : public Action
  {
  public:
    AudioActionNoteon() : m_noteId(-1), m_sourceId(-1) {}
    AudioActionNoteon(const Fluid::TimeStamp &t) 
      : Action(t), m_noteId(-1), m_sourceId(-1) {}

    virtual ~AudioActionNoteon();

    void setNoteId(long id) { m_noteId = id; }
    long noteId() const { return m_noteId; }

    void setSourceId(int id) { m_sourceId = id; }
    int  sourceId() const { return m_sourceId; }
    // virtual QDomElement writeDom(QDomDocument &);
  protected:
    long m_noteId;
    int  m_sourceId;
  
  };

  /////////////////////////////////////////////////////////////////////////////
  /////////////////////////////////////////////////////////////////////////////

  /// Note-off
  class AudioActionNoteoff : public Action
  {
  public:
    AudioActionNoteoff(int noteOnActionId = -1,
		       const Fluid::TimeStamp &t = 0,
		       bool deallocateSource = true) 
      : Action(t), m_noteOnActionId(noteOnActionId), 
	m_deallocateSource(deallocateSource) {}

    virtual ~AudioActionNoteoff();

    virtual void act(ActionList *);
    virtual QDomElement writeDom(QDomDocument &);
    virtual bool readDom(QDomElement &elem);

  protected:
    int   m_noteOnActionId;
    bool  m_deallocateSource;
  };

  /////////////////////////////////////////////////////////////////////////////
  /////////////////////////////////////////////////////////////////////////////
  
  class AudioActionNoteParameter : public Action
  {
  public:
    AudioActionNoteParameter(AudioActionNoteon *noteOn = 0,
			     const char * param = "",
			     float value = 1.0,
			     const Fluid::TimeStamp &t = 0) 
      : Action(t), m_noteOn(noteOn), m_param(param), m_value(value) {}
    virtual ~AudioActionNoteParameter();

    virtual void act(ActionList *);

    void setValue(float v) { m_value = v; }

    virtual QDomElement writeDom(QDomDocument &);
    virtual bool readDom(QDomElement &elem);
  protected:
    AudioActionNoteon *m_noteOn;
    std::string  m_param;
    float        m_value;
  };


  /////////////////////////////////////////////////////////////////////////////
  /////////////////////////////////////////////////////////////////////////////

  class AudioActionNoise : public AudioActionNoteon
  {
  public:
    AudioActionNoise(int sourceId = -1, 
		     float gain = 1.0, float pitch = 200, float octaves = 0.5, 
		     const Fluid::TimeStamp &t = 0) 
      : AudioActionNoteon(t), 
	m_gain(gain), 
	m_centerHz(pitch), 
	m_octaves(octaves) 
    { m_sourceId = sourceId; }

    virtual ~AudioActionNoise();

    virtual void act(ActionList *);

    virtual QDomElement writeDom(QDomDocument &);
    virtual bool readDom(QDomElement &elem);
  protected:
    float       m_gain;
    float       m_centerHz;
    float       m_octaves;
  };

  /////////////////////////////////////////////////////////////////////////////
  /////////////////////////////////////////////////////////////////////////////

  class AudioActionGranular : public AudioActionNoteon
  {
  public:
    AudioActionGranular(const Fluid::TimeStamp &t = 0) 
      : AudioActionNoteon(t), m_gain(1.0) {}
    virtual ~AudioActionGranular();

    virtual void act(ActionList *);

    void set(int sourceId, const char * sample, float gain)
    { m_sourceId = sourceId; m_sample = sample; m_gain = gain; }

    virtual QDomElement writeDom(QDomDocument &);
    virtual bool readDom(QDomElement &elem);
  protected:
    std::string m_sample;
    float       m_gain;
  };

  /////////////////////////////////////////////////////////////////////////////
  /////////////////////////////////////////////////////////////////////////////

  class AudioActionSample : public AudioActionNoteon
  {
  public:
    AudioActionSample(const Fluid::TimeStamp &t = 0) 
      : AudioActionNoteon(t), m_gain(1.0) { m_location.clear(); }
    virtual ~AudioActionSample();

    virtual void act(ActionList *);

    void set(int sourceId, const char * sample, float gain, 
	     const Fluid::Vector3 & location)
    { 
      m_sourceId = sourceId; 
      m_sample = sample; 
      m_gain = gain; 
      m_location = location;
    }

    virtual QDomElement writeDom(QDomDocument &);
    virtual bool readDom(QDomElement &elem);

  protected:
    std::string m_sample;
    float       m_gain;
    Fluid::Vector3 m_location;
  };

  /////////////////////////////////////////////////////////////////////////////
  /////////////////////////////////////////////////////////////////////////////

  /** UNIMPLEMENTED !!! Sends a message to the audio engine. */
  class AudioActionMessage : public Action
  {
  public:
    AudioActionMessage() {}
    virtual ~AudioActionMessage();

    virtual void act(ActionList *);

    void set(const char *address, MJ_Message *);

    virtual QDomElement writeDom(QDomDocument &);
    virtual bool readDom(QDomElement &elem);
  protected:
    MJ_MessageCarrier m_holder;
  };

  /////////////////////////////////////////////////////////////////////////////
  /////////////////////////////////////////////////////////////////////////////

  /** Sends a message to the audio engine. */
  class AudioActionSourceLocation : public Action
  {
  public:
    AudioActionSourceLocation() : m_sourceId(-1) {}
    AudioActionSourceLocation
    (int sourceId, const Fluid::Vector3 &location, const Fluid::TimeStamp &t)
      : Action(t)
    { m_sourceId = sourceId; m_location = location; }

    virtual ~AudioActionSourceLocation();

    virtual void act(ActionList *);

    void set(int sourceId, const Fluid::Vector3 &location)
    { m_sourceId = sourceId; m_location = location; }

    virtual QDomElement writeDom(QDomDocument &);
    virtual bool readDom(QDomElement &elem);

  protected:
    int            m_sourceId;
    Fluid::Vector3 m_location;
  };

  /////////////////////////////////////////////////////////////////////////////
  /////////////////////////////////////////////////////////////////////////////

  /** Sends a message to the audio engine. */
  class AudioActionSourceLocation2 : public Action
  {
  public:
    AudioActionSourceLocation2() : m_sourceId(-1) {}
    AudioActionSourceLocation2
    (int sourceId, int objectId, const Fluid::TimeStamp &t)
      : Action(t)
    { m_sourceId = sourceId; m_objectId = objectId; }

    virtual ~AudioActionSourceLocation2();

    virtual void act(ActionList *);

    void set(int sourceId, int objectId)
    { m_sourceId = sourceId; m_objectId = objectId; }
	   

    virtual QDomElement writeDom(QDomDocument &);
    virtual bool readDom(QDomElement &elem);
  protected:
    int            m_sourceId;
    int            m_objectId;
  };

  /////////////////////////////////////////////////////////////////////////////
  /////////////////////////////////////////////////////////////////////////////

  /** Sends a message to the audio engine. */
  class AudioActionControl : public Action
  {
  public:
    AudioActionControl() : m_noteOnId(-1), m_objectId(-1) {}
    AudioActionControl(int noteOnId, 
		       int objectId,  
		       const std::string & objectParam,
		       const std::string & audioParam, 
		       const Fluid::TimeStamp &t,
		       MJ_Mapper * mapper) 
      : Action(t),
	m_noteOnId(noteOnId), 
	m_objectId(objectId), 
	m_objectParam(objectParam), 
	m_audioParam(audioParam),
	m_mapper(mapper)
    {}

    virtual ~AudioActionControl();

    virtual void act(ActionList *);

    virtual QDomElement writeDom(QDomDocument &);
    virtual bool readDom(QDomElement &elem);
  protected:
    int            m_noteOnId;
    int            m_objectId;

    std::string    m_objectParam;
    std::string    m_audioParam;

    VEE_ClonablePointer<MJ_Mapper> m_mapper;
  };

}

#endif
