/* -*- C++ -*- */

/* -*- C++ -*- */

/* -*- C++ -*- */

/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of the Crystal animation and 3D modeling application
 *
 * Authors:   Tommi Ilmonen 2001-2004
 * Copyright: Tommi Ilmonen and/or Tekes 2001-2004
 *
 * firstname.lastname@hut.fi
 *
 */

#ifndef CRYSTAL_COMPONENT_EDITOR_H
#define CRYSTAL_COMPONENT_EDITOR_H

#include <crystal_editor.h>
#include <crystal_graph_editor.h>
#include <crystal_widget.h>

#include <Fluid/MessageSink.hpp>

#include <vee_reference_object.h>

#include <qawl_file_cache.h>
#include <qawl_gl_renderer.h>

#include <list>
#include <map>
#include <vector>

namespace Crystal { 

  class ActionList;
  class Tools;

  class ComponentEditor : public Editor,
			  public Fluid::MessageSink
  {
  public:
  
    class Track;

    class ComponentData : public Fluid::MessageSink
    {
    public:
      ComponentData();

      virtual ~ComponentData();

      virtual bool msgProcess(const char *id, Fluid::Message *);

      void toggle(ComponentEditor *);
      void setEnabled(ComponentEditor *, bool);
      void setHidden();

      void startEdit(ComponentEditor *);
      void clear() { m_button.clear(); }
      void build(ComponentEditor *);
      void toggleGraphs();

      QDomElement writeDom(QDomDocument &doc);
      bool readDom(QDomElement &elem);

      void toggleVisual() 
      { 
	m_visualize = !m_visualize; 
	m_button.ptr()->setChecked(0, m_visualize); 
      }


      VEE_RefPtr<Menu>        m_button;
      int                     m_plane; // widget plane
      VEE_RefPtr<GraphEditor> m_graphs;
      ActionList            * m_list;
      std::string             m_label;
      int                     m_listId;
      int                     m_graphId; // In menu
      bool                    m_enabled;
      bool                    m_hidden;
      bool                    m_visualize;
      ComponentEditor       * m_host;
    };

    class Track
    {
    public:
      typedef std::vector<VEE_RefPtr<ComponentData> >::iterator iterator;

      Track() : m_id(-1) {}

      void setEnabled(ComponentEditor *editor, bool enable)
      {
	for(iterator it = m_components.begin(); it != m_components.end(); it++)
	  (*it).ptr()->setEnabled(editor, enable);
      }

      void build(ComponentEditor *);
      void clear() { m_components.clear(); }

      int findComponent(int id);
      void removeComponent(int index) 
      { m_components.erase(m_components.begin() + index); }
    
      bool spaceIsOccupied(float left, float right,
			   int plane, int until);

      QDomElement writeDom(QDomDocument &doc);
      bool readDom(QDomElement &elem, ComponentEditor *);

      int                        m_id;
      std::string                m_label;
      std::vector<VEE_RefPtr<ComponentData> > m_components;
      VEE_RefPtr<Menu>     m_menu;

    };

    class Counter 
    {
    public:
      Counter() : m_value(0) {}
      int m_value;
    };

    ComponentEditor(Tools *);
    virtual ~ComponentEditor();
  
    virtual bool msgProcess(const char *id, Fluid::Message *);


    void render(GlData *);
    void postRender(GlData *);

    void update();

    virtual void beforeEvents() { m_widgets.beforeEvents(); }
    void handleEvent(const InEvent *e);

    void addList(ActionList *);
    void startRecording(const std::string &);

    Tools * host() { return m_host; }

    void redesignLayout();

    void clear();

    const ColorGroup & disableColors() const { return m_disableColors; }

    virtual QDomElement writeDom(QDomDocument &doc);
    virtual bool readDom(QDomElement &elem);

    void addWidget(Widget *w) { m_widgets.addWidget(w); }

    VEE_RefPtr<ComponentData> & getComponent(int id) 
    { return m_components[id]; }

    float secsPerUnit() const { return m_secsPerUnit; }

  protected:

    void fixRowMenu(Menu *menu, int id, const std::string &label);

    void recalculateWaveform();

    Tools           * m_host;

    Widgets           m_widgets;

    ColorGroup        m_disableColors;
    ColorGroup        m_moveColors;
    Slider           *m_offset;

    int   m_recordTrack;
    bool  m_newRecords;
    std::string m_recordName;

    float m_topLine;
    float m_trackHeight;
    float m_left;
    float m_right;
    float m_secsPerUnit;

    VEE_Vector3               m_pointerLocation;
    VEE_Vector3               m_moveOrigin;
    Fluid::TimeStamp          m_originalTime;
    ComponentData            *m_moveComponent;

    std::vector<VEE_ReferenceObject<Track> > m_tracks;
    typedef std::vector<VEE_ReferenceObject<Track> >::iterator iterator;

    // Components are mapped with respect to action list id.
    std::map<int, VEE_RefPtr<ComponentData> > m_components;
    typedef std::map<int, VEE_RefPtr<ComponentData> >::iterator component_iterator;

    std::map<std::string, Counter > m_typeCounters;

    std::string    m_audioFile;
    QAWL_FileCache m_waveForm;
    std::vector<QAWL_GlRenderer::QuadData> m_waveFormData;

  };

}

#endif
