/* -*- C++ -*- */

/* -*- C++ -*- */

/* -*- C++ -*- */

/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of the Crystal animation and 3D modeling application
 *
 * Authors:   Tommi Ilmonen 2001-2004
 * Copyright: Tommi Ilmonen and/or Tekes 2001-2004
 *
 * firstname.lastname@hut.fi
 *
 */

#ifndef CRYSTAL_DENSITY_EDITOR_H
#define CRYSTAL_DENSITY_EDITOR_H

#include <crystal_editor.h>
#include <crystal_ui_handler.h>
#include <crystal_widget.h>

#include <vee_meta_balls.h>
#include <vee_time.h>
#include <vee_volume_grid.h>

#include <Fluid/MessageSink.hpp>

#include <Solar/GlMaterial.hpp>
#include <Solar/Model.hpp>

namespace Crystal { 

  class Engine;
  class Model;
  class Tools;
  class DensityRecorder;

  class DensityEditor : public Editor,
			public Fluid::MessageSink,
			public Transformation
  {
  public:

    class Element 
    {
    public:
      Element(); 
      ~Element();

      void createModelFromGrid(Engine *);

      QDomElement writeDom(QDomDocument &doc);
      bool readDom(QDomElement &elem, Engine *engine);
      
      // VEE_MetaBalls       m_metaBalls;
      VEE_VolumeGrid2   m_grid;
      VEE_Vector3       m_gridCenter;

      Solar::Model      m_model;
      VEE_Hsva          m_hsva;
      VEE_Rgba          m_rgba;

      Solar::GlMaterial m_material;
      std::string       m_materialName;
      std::string       m_texture;

      bool              m_dirty;
      uint              m_renderCount;
      float             m_uvScale;
    };

    DensityEditor(Tools *);
    virtual ~DensityEditor();
    virtual bool msgProcess(const char *id, Fluid::Message *);

    void render(GlData *);

    void update();

    virtual void beforeEvents() { m_widgets.beforeEvents(); }
    void handleEvent(const InEvent *e) { m_widgets.handleEvent(e); }

    void startRecording();
  
    DensityRecorder * recorder() { return m_recorder; }

    bool select(int);
    Model * createModel(uint);

    int currentMesh() { return m_selectedEditor; }

    Element * currentElement() { return m_current; }

    static void createVolumeBoxes(const VEE_Vector3 & first,
				  const VEE_Vector3 & span,
				  const VEE_Vector3 & move,
				  uint fields,
				  Fluid::MessageSink *sink,
				  Widgets *widgets,
				  Transformation * trans,
				  Tools * tools,
				  std::vector<CubeWidget*> *,
				  std::vector<Menu*> * m = 0);

    uint volumes() const { return m_cubes.size(); }

    void renderVolumes(GlData *, 
		       std::vector<CubeWidget*> *);

    VEE_Volume * cloneVolume(int);

    virtual QDomElement writeDom(QDomDocument &doc);
    virtual bool readDom(QDomElement &elem);

  protected:

    bool selectTexture(const char *);
    uint expandOne();

    void animate(int index);

    void addElement(Element *);

    void rebuildBoxes();

    Tools                *m_host;
    Widgets               m_widgets;

    DensityRecorder      *m_recorder;
    Slider               *m_uvScale;

    /// Box layout settings:
    Vector3 m_low, m_span, m_move;

    int                      m_selectedEditor;
    Element                 *m_current;
    std::vector<CubeWidget*> m_cubes;
    std::vector<Menu*>       m_menus;
    std::vector<VEE_RefPtr<Element> > m_fields;
  };

  /////////////////////////////////////////////////////////////////////////////
  /////////////////////////////////////////////////////////////////////////////

  class DensityRecorder : public UiHandler
  {
  public:
    DensityRecorder(Tools *);
    ~DensityRecorder();

    virtual void handleInput();
    virtual void update();
    virtual void render(GlData *glData);
    virtual void start();
    virtual void stop();

    void setData(DensityEditor::Element *data)
    { m_data = data; }

  protected:

    Tools * m_host;

    DensityEditor::Element * m_data;

    VEE_Time       m_startTime;
    VEE_Vector3    m_prevCenter;
    VEE_Vector3    m_prevLoc;
    float          m_prevRad;
    bool           m_first;

    long m_samples;
  };

}

#endif
