/* -*- C++ -*- */

/* -*- C++ -*- */

/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of the Crystal animation and 3D modeling application
 *
 * Authors:   Tommi Ilmonen 2001-2004
 * Copyright: Tommi Ilmonen and/or Tekes 2001-2004
 *
 * firstname.lastname@hut.fi
 *
 */

#include <crystal_dom_util.h>

#include <vee_error.h>

namespace Crystal { 

  QString toStr(const Fluid::Vector3 &v)
  {
    QString tmp;

    QTextOStream(&tmp) << v.x << "  " << v.y << "  " << v.z;
  
    return tmp;
  }

  QString toStr(const Fluid::Matrix3 &v)
  {
    QString tmp;

    QTextOStream s(&tmp);
    for(uint i=0; i < 9; i++)
      s << v.data()[i] << "  ";
  
    return tmp;
  }

  QString toStr(const VEE_Rgba &c)
  {
    QString tmp;

    QTextOStream(&tmp) << c.red() 
		       << "  " << c.green() 
		       << "  " << c.blue()
		       << "  " << c.alpha()
		       << "  " << c.theta();

    return tmp;  
  }

  QString toStr(const VEE_Hsva &c)
  {
    QString tmp;

    QTextOStream(&tmp) << c.hue() 
		       << "  " << c.saturation() 
		       << "  " << c.value()
		       << "  " << c.alpha()
		       << "  " << c.theta();

    return tmp;  
  }

  Fluid::Vector3 toVector3(const QString &str)
  {
    Fluid::Vector3 tmp;
    tmp.clear();
  
    QTextIStream s( & str);
    s >> tmp;
  
    return tmp;
  }

  Fluid::Matrix3 toMatrix3(const QString &str)
  {
    Fluid::Matrix3 tmp;
    tmp.identity();

    QTextIStream s( & str);
    s >> tmp[0];
    s >> tmp[1];
    s >> tmp[2];
  
    return tmp;
  }

  VEE_Hsva toHsva(const QString & str)
  {
    float h = 0.5, s = 0.5, v = 1.0, a = 1.0, t = 0.0;

    QTextIStream is( & str);
    is >> h >> s >> v >> a >> t;
  
    return VEE_Hsva(h, s, v, a, t);
  }

  VEE_Rgba toRgba(const QString & str)
  {
    float r = 1, g = 1, b = 1, a = 1.0, t = 0.0;

    QTextIStream is( & str);
    is >> r >> g >> b >> a >> t;
  
    return VEE_Rgba(r, g, b, a, t);
  }

  Fluid::TimeStamp toTimeStamp(const QString &str)
  {
    Fluid::TimeStamp v;
    v.read(str.ascii());
    return v;
  }

  void addTextChild(QDomElement &base, QDomDocument &doc,
		    const QString &tag,  const QString &value)
  {
    QDomElement si = doc.createElement(tag);
    si.appendChild(doc.createTextNode(value));
    base.appendChild(si);
  }

  void addTextChild(QDomElement &base, QDomDocument &doc,
		    const QString &tag,  const char *value)
  {
    addTextChild(base, doc, tag, QString(value));
  }

  void addTextChild(QDomElement &base, QDomDocument &doc,
		    const QString &tag,  int value)
  {
    addTextChild(base, doc, tag, QString().setNum(value));
  }

  void addTextChild(QDomElement &base, QDomDocument &doc,
		    const QString &tag,  float value)
  {
    addTextChild(base, doc, tag, QString().setNum(value));
  }

  void addTextChild(QDomElement &base, QDomDocument &doc,
			    const QString &tag,  const Fluid::Vector3 &v)
  {
    addTextChild(base, doc, tag, toStr(v));
  }

  void addTextChild(QDomElement &base, QDomDocument &doc,
			    const QString &tag,  const Fluid::Matrix3 &v)
  {
    addTextChild(base, doc, tag, toStr(v));
  }

  void addTextChild(QDomElement &base, QDomDocument &doc,
			    const QString &tag,  const Fluid::TimeStamp &v)
  {
    char buf[64];
    v.write(buf);
    addTextChild(base, doc, tag, buf);
  }

  float getFloat(QDomElement &e, const QString &tag, float def)
  {
    QDomNode c = e.namedItem(tag);

    QDomText t = c.toElement().firstChild().toText();

    bool ok;
    QString str = t.data().stripWhiteSpace();
    
    float val = str.toFloat(&ok);

    /* VEE_trace("getFloat # %s \"%s\" %d %f", tag.ascii(), str.ascii(), 
       (int) ok, val); */

    return ok ? val : def;
  }

  int getInt(QDomElement &e, const QString &tag, int def)
  {
    QDomNode c = e.namedItem(tag);

    QDomText t = c.toElement().firstChild().toText();

    bool ok = true;

    int val = t.data().stripWhiteSpace().toInt(&ok);

    return ok ? val : def;
  }

  Fluid::Vector3 getVector3(QDomElement &e, const QString &tag,
			    const Vector3 & def)
  {
    QDomNode c = e.namedItem(tag);

    QString str = c.toElement().firstChild().toText().data();

    return str.isEmpty() ? def : toVector3(str);
  }

  Fluid::Matrix3 getMatrix3(QDomElement &e, const QString &tag,
			    const Matrix3 &def )
  {
    QDomNode c = e.namedItem(tag);

    QString str = c.toElement().firstChild().toText().data();

    if(str.isEmpty())
      return def;
      
    return toMatrix3(str);
  }

  QString getString(QDomElement &e, const QString &tag, const char * def)
  {
    QDomNode c = e.namedItem(tag);
    QString str = c.toElement().firstChild().toText().data();
    return str.isEmpty() ? QString(def) : str;
  }

  Fluid::TimeStamp getTimeStamp(QDomElement &e, const QString &tag)
  {
    return toTimeStamp(getString(e, tag));
  }

}

QTextStream & operator >> (QTextStream &s, Fluid::Vector3 &v)
{
  for(uint i=0; i < 3; i++)
    s >> v[i];

  return s;
}

QTextStream & operator >> (QTextStream &s, Fluid::Matrix3 &v)
{
  for(uint i=0; i < 9; i++)
    s >> v.data()[i];

  return s;
}
