/* -*- C++ -*- */

/* -*- C++ -*- */

/* -*- C++ -*- */

/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of the Crystal animation and 3D modeling application
 *
 * Authors:   Tommi Ilmonen 2001-2004
 * Copyright: Tommi Ilmonen and/or Tekes 2001-2004
 *
 * firstname.lastname@hut.fi
 *
 */

#ifndef CRYSTAL_EMITTER_EDITOR_H
#define CRYSTAL_EMITTER_EDITOR_H

#include <crystal_editor.h>
#include <crystal_particle_editor.h>
#include <crystal_transformation.h>
#include <crystal_widget.h>

#include <vee_bounding_box.h>
#include <vee_color.h>
#include <vee_image.h>
#include <vee_operator_volume.h>
#include <vee_emitter.h>
#include <vee_reference_object.h>

#include <mj_array2d.h>

#include <Fluid/MessageSink.hpp>

#include <vector>

namespace Crystal { 

  class GlData;
  class Tools;

  class EmitterEditor : public Editor,
			public Fluid::MessageSink,
			public Transformation
  {
  public:
    enum Param {
      RATE,
      SPEED,
      SPREAD,
      SLIDERS
    };

    EmitterEditor(Tools *);
    virtual ~EmitterEditor();
    virtual bool msgProcess(const char *id, Fluid::Message *);

    void render(GlData *);

    void update();

    void generateColorTextures(const VEE_Hsva &);

    virtual void beforeEvents() { m_widgets.beforeEvents(); }
    void handleEvent(const InEvent *e) { m_widgets.handleEvent(e); }

    void selectParticle(int);
    void selectEmitter(int);
    uint selectedEmitter() const { return m_selectedEmitter; }
    uint emitterParticle(uint emitter);

    VEE_Emitter * createEmitter(uint);
    void getEmitterParameters(uint, std::vector<float> *);
    VEE_Emitter * createSelectedEmitter()
    { return createEmitter(m_selectedEmitter); }

    Tools *host() { return m_host; }

    void updateSliders();

    bool selectEmissionType(uint);
    void selectVolume(int);

    virtual QDomElement writeDom(QDomDocument &doc);
    virtual bool readDom(QDomElement &elem);

  protected:

    class EmitterData
    {
    public:
      EmitterData();
      ~EmitterData() {}

      void init(EmitterEditor *);
      QDomElement writeDom(QDomDocument &doc);
      bool readDom(QDomElement &elem);

      VEE_Emitter  m_emitter;
      uint                  m_particle;
      int                   m_volume;
      float                 m_params[SLIDERS];
      Widget       *m_button;
      VEE_OperatorVolume    m_volumeOp;
    };
  
    Tools      *m_host;

    MJ_Array2D<EmitterData>       m_emitters;
    MJ_Array2D<ParticleEditor::ParticleData> m_particles;
    VEE_BoundingBox               m_emitterBox;
    EmitterData                  *m_currentEmitter;
    uint                          m_selectedEmitter;

    VEE_BoundingBox               m_particleBox;

    std::vector<Widget*>  m_emissions;

    Widgets               m_widgets;
    Slider               *m_sliders[SLIDERS];

    std::vector<CubeWidget*> m_cubes;
  };

}

#endif
