/* -*- C++ -*- */

/* -*- C++ -*- */

/* -*- C++ -*- */

/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of the Crystal animation and 3D modeling application
 *
 * Authors:   Tommi Ilmonen 2001-2004
 * Copyright: Tommi Ilmonen and/or Tekes 2001-2004
 *
 * firstname.lastname@hut.fi
 *
 */

#ifndef CRYSTAL_ENGINE_H
#define CRYSTAL_ENGINE_H

#include <crystal.h>
#include <crystal_audio_allocator.h>
#include <crystal_timing.h>

#include <Fluid/Matrix3.hpp>
#include <Fluid/MessageSink.hpp>
#include <Fluid/MessageSource.hpp>
#include <Fluid/Processors.hpp>
#include <Fluid/TimeStamp.hpp>

#include <vee_garbage.h>
#include <vee_reference_object.h>
#include <vee_time.h>
#include <vee_value.h>

#include <map>
#include <string>

#include <ConfigReaderEmpty.hpp>

#include <Solar/Garbage.hpp>

#include <qdom.h> 

namespace Fluid {
  class ConfigChunk;
  class Glove;
  class GloveNavigator;
  class Fistiness;
  class Mouse;
  class MouseNavigator;
  class MouseButton;
  class SensorHitDetector;
  class Tracker;
  class TrackerSensorSelector2;

  template <class T> class IONodeBuffer;
}

namespace Solar {
  class Model;
  class Object;
  class Renderer;
}

class GEE_Engine;

class VEE_Blow;
class VEE_Gravity;
class VEE_GlRenderer;
class VEE_GlTexture2D;
class VEE_SceneBox;
class VEE_MagicRain;
class VEE_MagicRainDrop;

class AC_Control;
class AC_VrControl;

namespace Crystal { 

  class ActionListManager;
  class GeeConfig;
  class GeeRecorder;
  class GeeRecorders;
  class GlData;
  class SceneObjs;
  class Tools;

  class Engine : public MessageSink, 
		 public MessageSource
  {
  public:
    enum Mode {
      MODE_TOOLS,
      MODE_QUICK_EDIT,
      MODE_PLAY,
      MODE_SIZEOF
    };

    enum Recorder {
      RECORDER_EMITTER,
      RECORDER_EMITTER_STATIC,
      RECORDER_BLOW,
      RECORDER_BLOW_STATIC,
      RECORDER_SWIRL,
      RECORDER_SWIRL_STATIC,
      RECORDER_MESH,
      RECORDER_EXPLOSION,
      /// Use GEE stuff, forget the old recorders.
      RECORDER_GEE,
      RECORDER_SIZEOF
    };

    Engine(VEE_SceneBox *scene, const std::string & configFile);
    virtual ~Engine();

    static Engine * engine() { return m_engine; }

    void init();

    void update(float dt = 0.0f);
  
    void render(GlData *,
		VEE_GlRenderer *, 
		Solar::Renderer *,
		bool moveCamera = true);
  
    VEE_SceneBox *scene() { return m_scene; }
  
    virtual bool msgProcess(const char *id, Fluid::Message *);

    // const Fluid::Vector3 & getHeadPosition();
  
    Fluid::Tracker    *tracker() { return m_tracker; }
    Fluid::Glove      *glove()   { return m_glove; }
    Fluid::Mouse      *mouse()   { return m_mouse; }
    int                trackerSamples() const { return m_trackerSamples; }
    int                handSensor1() const { return m_handSensor1; }
    int                handSensor2() const { return m_handSensor2; }

    Fluid::Fistiness  *fistiness(){return m_fistiness; }
    Fluid::SensorHitDetector *clapDetector() { return m_clapDetector; }

    Fluid::TrackerSensorSelector2 * strongHandLoc() { return m_strongHandLoc; }
    Fluid::TrackerSensorSelector2 * weakHandLoc() { return m_weakHandLoc; }
    Fluid::TrackerSensorSelector2 * handLoc(int hand) 
    { 
      hand &= 0x1;
      if(hand == LEFT_HAND)
	return m_weakHandLoc;
      return m_strongHandLoc; 
    }
    Fluid::TrackerSensorSelector2 * headLoc() { return m_headLoc; }
  
    void setRealTime(bool isRealTime) { m_isRealTime = isRealTime; }
    void setShowForces(bool show) { m_showForces = show; }
    bool showForces () const { return m_showForces; }
    const Timing & timing() const { return m_timing; }
    void setOffset(double offset) { m_timing.m_offset.setSeconds(offset); }
    Fluid::TimeStamp realTimeToRecordTime(const Fluid::TimeStamp & t) const
    { return m_timing.realTimeToRecordTime(t); }    

    AC_Control        *acControl()   { return m_acControl; }
    AC_VrControl      *acVrControl() { return m_acVrControl; }

    int                allocateAudioSource();
    void               deallocateAudioSource(int s);  

    void setDebug(bool b) { m_debug = b; }

    bool loadScene(const char *name, ConfigReader::Chunk * config);

    void selectScene();

    Tools * tools() { return m_tools; }

    void setPlaySpeed(double s) { m_timing.m_playSpeed = s; }
    void setGravity(double g);

    void skipMode(Mode newMode, bool forceToolsVisible = false);

    void setNextRecorder(Recorder r, 
			 const std::string & rec1 = "", 
			 const std::string & rec2 = "");
    
    const std::string & nextRecorderName(int hand) 
    { return m_nextRecorderName[hand]; }

    ConfigReader::Config * config() { return m_config; }

    void setParticleVolume(float volume) { m_particleVolume = volume; }
    float particleVolume() const { return m_particleVolume; }

    void setSoundTrackVolume(float volume) { m_soundTrackVolume = volume; }
    float soundTrackVolume() const { return m_soundTrackVolume; }
  
    bool writeDomFile(const QString &);
    QDomElement writeDom(QDomDocument &doc);

    bool readDomFile(const QString &);
    bool readDom(QDomElement &);

    void useScene(uint);
    void origin(const Vector3 & loc = Vector3(0, 0, 0));

    std::map<std::string, Solar::Model *> &models() { return m_models; }
    std::vector<SceneObjs *> & scenes() { return m_scenes; }

    ActionListManager * actionLists() { return m_actionLists; }

    void setDone(bool done);
    bool isDone() const { return m_done; }

    void addAniMesh(Solar::Object *);
    void removeAniMesh(Solar::Object *);

    Solar::GarbageBin * deletedGraphics() { return & m_deletedGraphics; }
    VEE_GarbageBin    * veeGarbage() { return & m_veeGarbage; }

    /* void addRecorder(const std::string &name, GeeRecorder *r)
       { m_recorders[name] = r; } */
		     
    GeeConfig * geeConfig() { return m_geeConfig; }

    Fluid::MessageSource * buttonMessages(int hand = 0);

    GEE_Engine  * geeEngine() { return m_geeEngine; }

    void toggleRenderSolars() { m_renderSolars = ! m_renderSolars; }

    void setCursorSize(float s) { m_cursorSize = s; }

    void setRebuildGee(bool rebuild) { m_rebuildGee = rebuild; }

    void setRenderHand(int hand, bool render) 
    { m_handRender[hand] = render; }

    Fluid::Processors & processors() { return m_processors; }

    GeeRecorder  * currentActiveGeeRecorder();

    GeeRecorders * recorders(int hand) { return m_recorders[hand]; }

    const Fluid::Matrix3 & rotation() { return m_rotation; }

  protected:
    void useQuickEditor(int hand);
    
    void initSolarObjects();

    void centralizeTools();

    bool m_done;
    bool m_renderSolars;
    uint m_updates;

    VEE_SceneBox      *m_scene;

    VEE_Gravity       *m_gravity;

    VEE_Time           m_lastUpdate;
    VEE_Time           m_changeTime;

    Timing             m_timing;

    bool               m_isRealTime;
    bool               m_showForces;

    ActionListManager * m_actionLists;

    Fluid::Tracker    *m_tracker;
    int                m_trackerSamples;

    int                m_handSensor1;
    int                m_handSensor2;
    int                m_headSensor;

    int                m_audioTrackIds[2];
    int                m_audioNoteIds[2];
    std::string        m_audioTrack;

    Fluid::Mouse      *m_mouse;
    Fluid::Glove      *m_glove;
    Fluid::Fistiness  *m_fistiness;

    Fluid::Processors  m_processors;
    Fluid::SensorHitDetector *m_clapDetector;
    Fluid::ProcessorNode     *m_navigator;
    Fluid::MouseButton       *m_mouseButtons[2];

    IONodeBuffer<Vector3>    *m_navigatorLoc;
    IONodeBuffer<Matrix3>    *m_navigatorRot;
    IONodeBuffer<float>      *m_navigatorScale;

    Fluid::Matrix3     m_rotation;
    Fluid::Vector3     m_location;
    Fluid::Vector3     m_locationOld;
    float              m_scaleOld;
    float              m_scale;
    float              m_cursorSize;

    Fluid::TrackerSensorSelector2 * m_strongHandLoc;
    Fluid::TrackerSensorSelector2 * m_weakHandLoc;
    Fluid::TrackerSensorSelector2 * m_headLoc;
  
    AC_Control        *m_acControl;
    AC_VrControl      *m_acVrControl;
    AudioAllocator     m_audioAllocator;
    float              m_particleVolume;
    float              m_soundTrackVolume;

    Solar::Model      *m_centralModel;
    Solar::Object     *m_centralObject;

    std::map<std::string, Solar::Model *>  m_models;
    std::vector<Solar::Object *>           m_objects;
    std::vector<Solar::Object *>           m_aniObjects;
    Solar::GarbageBin                      m_deletedGraphics;

    Solar::Object     *m_handPointer[2];
    bool               m_handRender[2];
    Mode               m_mode;
    Recorder           m_nextRecorder;
    std::string        m_nextRecorderName[2];
    Recorder           m_recorder;
    Tools             *m_tools;

    std::vector<SceneObjs *>       m_scenes;

  
    Tools             *m_toxols;
    bool               m_forceToolsVisible;

    std::string        m_configFile;
    ConfigReader::Config * m_config;


    GEE_Engine   * m_geeEngine;

    GeeRecorders * m_recorders[2];
    GeeRecorder  * m_currentGeeRecorder[2];

    GeeConfig    * m_geeConfig;
    VEE_GarbageBin m_veeGarbage;

    bool           m_rebuildGee;
    bool           m_showGroups;

    float          m_curtainTop;
    float          m_curtainBottom;
    float          m_curtainColor;

    static  bool m_debug;
    static  Engine * m_engine;
  };

}

#endif
