/* -*- C++ -*- */

/* -*- C++ -*- */

/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of the Crystal animation and 3D modeling application
 *
 * Authors:   Tommi Ilmonen 2001-2004
 * Copyright: Tommi Ilmonen and/or Tekes 2001-2004
 *
 * firstname.lastname@hut.fi
 *
 */

#include <crystal_explosion_recorder.h>

#include <crystal_action_list.h>
#include <crystal_audio_action.h>
#include <crystal_dom_util.h>
#include <crystal_engine.h>
#include <crystal_force_editor.h>
#include <crystal_gl_data.h>
#include <crystal_location_action.h>
#include <crystal_remove_action.h>
#include <crystal_tools.h>
#include <crystal_value_action.h>

#include <vee_explosion.h>
#include <vee_error.h>
#include <vee_scene.h>
#include <vee_strs.h>

#include <Fluid/SensorHitDetector.hpp>
#include <Fluid/TrackerSensorSelector.hpp>

namespace Crystal { 

  static bool debugExplosion = false;

  /** Creates a explosion-object to the action list.  */

  void CreateExplosionAction::act(ActionList *al)
  {
    
    VEE_Explosion *explosion = 
      new VEE_Explosion(m_location, m_range, m_magnitude, 0.5);
    
    al->getScene()->addNewForce(explosion);
    
    /* if(debugExplosion)
       VEE_trace("CreateExplosionAction::act # DONE %d", 
       al->getScene()->totalForces());*/
  }

  QDomElement CreateExplosionAction::writeDom(QDomDocument &doc)
  {
    QDomElement elem = doc.createElement("CreateExplosionAction");

    writeDomTimeStamp(elem, doc);

    QDomText loc = doc.createTextNode("location");
    loc.setData(toStr(m_location));
    elem.appendChild(loc);

    QDomText id = doc.createTextNode("range");
    id.setData(QString ().setNum(m_range));
    elem.appendChild(id);

    id = doc.createTextNode("magnitude");
    id.setData(QString ().setNum(m_magnitude));
    elem.appendChild(id);

    return elem;
  }

  bool CreateExplosionAction::readDom(QDomElement &elem) 
  { 
    m_range     = getFloat(elem, "range", 1.0);
    m_magnitude = getFloat(elem, "magnitude", 1.0);
    
    m_location  = getVector3(elem, "location");

    return false; 
  }


  /////////////////////////////////////////////////////////////////////////////
  /////////////////////////////////////////////////////////////////////////////

  ExplosionRecorder::ExplosionRecorder()
    : m_explodeNow(false),
      m_diameter(1.0)
  {
    m_center.clear();
  }

  ExplosionRecorder::~ExplosionRecorder()
  {}

  void ExplosionRecorder::beginRecording(ActionList *al, 
					 const Fluid::TimeStamp &)
  {
    Engine *engine = al->engine();

    if(debugExplosion)
      VEE_trace("ExplosionRecorder::beginRecording");

    if(engine->clapDetector()) {
      // engine->clapDetector()->addEventListener(this);
      engine->clapDetector()->msgAddSink("hit", "clap", this);
    }

  }

  void ExplosionRecorder::endRecording(ActionList *al, 
				       const Fluid::TimeStamp &)
  {
    Engine *engine = al->engine();

    if(engine->clapDetector()) {
      engine->clapDetector()->msgRemoveSink(this);
    }
  }

  void ExplosionRecorder::record(ActionList *al, 
				 const Fluid::TimeStamp &ts)
  {
    Engine *engine = al->engine();

    Vector3 left  = engine->strongHandLoc()->locations().getNewest();
    Vector3 right = engine->weakHandLoc()->locations().getNewest();

    m_center = 0.5f * (left + right);
    m_diameter = (left - right).length();

    if(!m_explodeNow) return;

    m_explodeNow = false;

    Fluid::SensorHitDetector *shd = al->engine()->clapDetector();

    float magnitude = engine->tools()->forceEditor()->current()->magnitude();
    float sensitivity = 
      engine->tools()->forceEditor()->current()->sensitivity();

    float range = sqrtf(shd->hitEnergy()) * 0.6;
    float emagnitude = range * sensitivity * magnitude;

    VEE_trace("ExplosionRecorder::record # EXPLODE %f", range);

    Vector3 loc = shd->hitLocation();

    al->appendAction(new CreateExplosionAction(loc, range, emagnitude, ts));

    AudioActionSample * aas = new AudioActionSample();

    aas->set(-1, 
	     al->engine()->config()->get("Sounds").
	     get("explosion").getString().c_str(),
	     0.5, loc);
    
    al->appendAction(aas);
  }

  void ExplosionRecorder::render(Engine *, GlData *gld)
  {
    VEE_Explosion bang;

    bang.setLocation(m_center);
    bang.setCurrentShockRange(m_diameter * 0.5);

    gld->m_veeRenderer.initRenderState();
    gld->m_veeRenderer.setForceFrontColor(& VEE_Rgba::Green);
    bang.render(& gld->m_veeRenderer);
    gld->m_veeRenderer.setForceFrontColor(0);
    gld->m_veeRenderer.closeRenderState();
  }

  bool ExplosionRecorder::msgProcess(const char *, Fluid::Message *)
  {
    m_explodeNow = true;
    return true;
  }

}
