/* -*- C++ -*- */

/* -*- C++ -*- */

/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of the Crystal animation and 3D modeling application
 *
 * Authors:   Tommi Ilmonen 2001-2004
 * Copyright: Tommi Ilmonen and/or Tekes 2001-2004
 *
 * firstname.lastname@hut.fi
 *
 */

#include <crystal_force_editor.h>

#include <crystal_engine.h>
#include <crystal_tools.h>

#include <vee_error.h>

#include <Fluid/MessageFloat.hpp>

#include <mj_misc.h>

namespace Crystal { 

  static const char * forceLabels[] = {
    "Vortex",
    "Vortex / static",
    "Blow",
    "Blow / static",
    "Explosion",
    0
  };

  static const int forceIds[] = {
    Engine::RECORDER_SWIRL,
    Engine::RECORDER_SWIRL_STATIC,
    Engine::RECORDER_BLOW,
    Engine::RECORDER_BLOW_STATIC,
    Engine::RECORDER_EXPLOSION
  };

  static float forceMagnitudes[] = {
    2.0,
    2.0,
    3.0,
    3.0,
    1.0
  };

  static const char * paramLabels[] = {
    "Magnitude",
    "Sensitivity"
  };


  ForceEditor::ForceEditor(Tools *host)
    : m_host(host),
      m_selectedForce(-1)
  {
    int i;
    char buf[64];
  
    m_widgets.setGrabGroup(host->grabs());
  
    Vector2 b1, b2, b3; // Force type selectors

    b1.make(0.0, 0.0);
    b2.make(0.0, 0.6);
    b3.make(1.8, 0.4);

    Vector2 sl1(-1.5, 0.0), sl2(0.4, 1.5), sl3(0.6, 0.0); // Slider locations

    for(i=0; forceLabels[i]; i++) {

      TextureButton * txb = new TextureButton(m_host);

      txb->setArea(b1, b3);
    
      txb->setLabel(forceLabels[i]);

      sprintf(buf, "force-%d", i);
      txb->msgAddSink("press", buf, this);
      txb->setSounds(m_host->buttonSounds());

      m_widgets.addWidget(txb);

      
      Item it;
      it.m_button      = txb;
      it.m_magnitude   = forceMagnitudes[i];
      it.m_sensitivity = 0.5;
      it.m_id          = forceLabels[i];

      m_forces.push_back(it);
    
      b1 += b2;    
    }

    for(i=0; i < Params; i++) {
      Slider * s = new Slider(m_host, 0.0, 0.0, 10.0);
    
      m_widgets.addWidget(s);

      s->setArea(sl1, sl2);
      s->setOrientation(Slider::VERTICAL);
      s->setLabel(paramLabels[i]);
      s->msgAddSink("value", paramLabels[i], this);

      m_params[i] = s;      
      sl1 += sl3;
    }

  }

  ForceEditor::~ForceEditor()
  {}

  bool ForceEditor::msgProcess(const char *id, Fluid::Message *m)
  {
    bool ok = false;
  
    Fluid::MessageFloat * mf = dynamic_cast<Fluid::MessageFloat *> (m);

    if(strncmp(id, "force-", 6) == 0) {
      int forceId = atoi(id + 6);

      selectForce(forceId);

      ok = true;
    }
    else if(strcmp(id, paramLabels[MAGNITUDE]) == 0 && 
	    m_selectedForce >= 0 && mf) {
      m_forces[m_selectedForce].m_magnitude = mf->value();
      ok = true;
    }
    else if(strcmp(id, paramLabels[SENSITIVITY]) == 0 && 
	    m_selectedForce >= 0 && mf) {
      m_forces[m_selectedForce].m_sensitivity = mf->value();
      ok = true;
    }

    return ok;
  }

  void ForceEditor::render(GlData *gld)
  {
    m_widgets.renderGl(gld);
  }

  void ForceEditor::update()
  {}

  void ForceEditor::selectForce(int index)
  {
    for(uint i=0; i < m_forces.size(); i++)
      m_forces[i].m_button->setColors(m_host->usualColor());

    m_selectedForce = index;

    if(m_selectedForce >= 0 && m_selectedForce < (int) m_forces.size()) {

      m_forces[m_selectedForce].m_button->setColors(m_host->selectedColor());

      m_params[MAGNITUDE]->setValue(m_forces[m_selectedForce].m_magnitude);
      m_params[SENSITIVITY]->setValue(m_forces[m_selectedForce].m_sensitivity);
      
      m_host->engine()->setNextRecorder
	((Engine::Recorder) forceIds[index], forceLabels[index]);
    }
  }

  QDomElement ForceEditor::writeDom(QDomDocument &doc)
  {
    QDomElement elem = doc.createElement("ForceEditor");

    for(uint i=0; i < m_forces.size(); i++) {
      
      QDomElement felem = m_forces[i].writeDom(doc);

      elem.appendChild(felem);
    }


    return elem;
  }

  bool ForceEditor::readDom(QDomElement &elem)
  {
    if(elem.tagName() != "ForceEditor")
      return false;

    QDomNode n = elem.firstChild();

    while( !n.isNull() ) {
      QDomElement e = n.toElement(); // try to convert the node to an element.

      QString id = e.namedItem("id").toText().data();
      
      int index = MJ_findStr(id.ascii(), forceLabels, m_forces.size());

      if(index >= 0)
	m_forces[index].readDom(e);

      n = n.nextSibling();
    }

    // RRR Now we need to update sliders.
    selectForce(-1);

    VEE_trace("ForceEditor::readDom # EXIT OK");

    return true;
  }

}
