/* -*- C++ -*- */

/* -*- C++ -*- */

/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of the Crystal animation and 3D modeling application
 *
 * Authors:   Tommi Ilmonen 2001-2004
 * Copyright: Tommi Ilmonen and/or Tekes 2001-2004
 *
 * firstname.lastname@hut.fi
 *
 */

#include <crystal_gee_mesh.h>

#include <crystal_engine.h>
#include <crystal_gee_config.h>
#include <crystal_gl_data.h>

#include <gee_color_around_path.h>
#include <gee_copy_geometry.h>
#include <gee_engine.h>
#include <gee_erase_around_path.h>
#include <gee_group.h>
#include <gee_group_around_path.h>
#include <gee_group_geos.h>
#include <gee_insert_particle.h>
#include <gee_move_around_path.h>
#include <gee_particle_rotate.h>
#include <gee_path_insert.h>
#include <gee_path_multiply.h>
#include <gee_path_to_lines.h>
#include <gee_path_to_mesh.h>
#include <gee_path_to_particles.h>
#include <gee_profile.h>
#include <gee_scale_around_path.h>
#include <gee_sphere.h>
#include <gee_texture_around_path.h>
#include <gee_triangle_mesh.h>
#include <gee_turn_around_path.h>
#include <gee_ungroup_geos.h>

#include <vee_algorithm.h>
#include <vee_error.h>
#include <vee_math.h>

#include <Fluid/Tracker.hpp>
#include <Fluid/TrackerSensorSelector.hpp>

#include <typeinfo>

#include <GL/gl.h>

namespace Crystal { 

  
  GeeRecorders::GeeRecorders()
  {}

  GeeRecorders::~GeeRecorders()
  {}
    
  void GeeRecorders::init(Crystal::Engine *ce, GEE_Engine *ge)
  {    
    m_recorders["Color"]     = new GeeColor(ce, ge, true);
    m_recorders["ColorGeo"]  = new GeeColor(ce, ge, false);
    m_recorders["Erase"]     = new GeeErase(ce, ge);
    m_recorders["EraseGeo"]  = new GeeEraseGeo(ce, ge);
    m_recorders["Group"]     = new GeeGroup(ce, ge);
    m_recorders["GroupGeo"]  = new GeeGroupGeo(ce, ge);
    m_recorders["Insert"]    = new GeeInsertGeo(ce, ge);
    m_recorders["InsertQuad"]= new GeeInsertGeo(ce, ge, GeeInsertGeo::QUAD);
    m_recorders["Lines"]     = new GeeLines(ce, ge);
    m_recorders["Mesh"]      = new GeeMesh(ce,  ge);
    m_recorders["Magnet"]    = new GeeMove(ce,  ge);
    m_recorders["Move"]      = new GeeMoveGeo(ce, ge, false);
    m_recorders["MoveRot"]   = new GeeMoveGeo(ce, ge, true);
    m_recorders["MultiBrush"]= new GeeMultiBrush(ce, ge);
    m_recorders["Particles"] = new GeeParticles(ce, ge);
    m_recorders["ParticleRotate"] = new GeeParticleRotate(ce, ge);
    m_recorders["Retexture"] = new GeeRetexture(ce, ge);
    m_recorders["Scale"]     = new GeeMoveGeo(ce, ge, false, true);
    m_recorders["ScaleAxis"] = new GeeMoveGeo(ce, ge, false, true, false);
    m_recorders["SelectGeo"] = new GeeSelectGeo(ce, ge);
    m_recorders["TurnGeo"]   = new GeeMoveGeo(ce, ge, false, true, false,true);
    m_recorders["UnGroupGeo"]= new GeeUnGroupGeo(ce, ge);
  }
  
  void GeeRecorders::setHand(Hand h)
  {
    for(iterator it = m_recorders.begin(); it != m_recorders.end(); it++)
      (*it).second->setHand(h);
  }

  GeePathToX * GeeRecorders::getRecorder(const std::string &id)
  {
    if(m_recorders.find(id) == m_recorders.end())
       return 0;
    return m_recorders[id];
  }
    

  /////////////////////////////////////////////////////////////////////////////
  /////////////////////////////////////////////////////////////////////////////

  GeePathToX::GeePathToX(Crystal::Engine *h, GEE_Engine *e)
    : GeeRecorder(h, e),
      m_ignoreNapra(false),
      m_napra(0),
      m_pathA1(0),
      m_pathA2(0),
      m_pathB1(0),
      m_pathB2(0),
      m_pathToX(0),
      m_trackerCount(0),
      m_pathAId(-1),
      m_pathBId(-1),
      m_collectData(true),
      m_needTwoPaths(false),
      m_needPaths(true),
      m_layer(-1),
      m_toolSphere(0),
      m_useToolSphere(false),
      m_toolSphereColor(VEE_Rgba::White),
      m_highligthSphereArea(false),
      m_useNapraRadius(false),
      m_radius(1.0f),
      m_radiusNorm(1.0f),
      m_radiusBoost(0.0f)
  {
    bzero(& m_napraSample, sizeof(m_napraSample));
    m_correctionRotation.rotateZ(VEE_HALF_PI);    
  }

  GeePathToX::~GeePathToX()
  {
    delete m_toolSphere;
  }

  void GeePathToX::activate(const TimeStamp &now)
  {
    const char * fname = "GeePathToX::activate";

    GeeRecorder::activate(now);

    GeeConfig * geeConfig = m_host->geeConfig(); 

    m_layer = geeConfig->getValueInt("layer", -1); 

    if(m_useToolSphere)
      useToolSphere(geeConfig->getValueFloat(m_toolName + "/radius", 1.0));

    if(m_ignoreNapra)
      m_napra = 0;
    else
      m_napra = InputDevices::getNapra("napra");
    

    VEE_trace("%s # m_napra = %p", fname, m_napra);
    
    char buf[32];
    char xyz[] = "xyz";

    Vector3 angles(1.07, 0.0, 0.0);

    for(int i = 0; i < 3; i++) {
      sprintf(buf, "cal-%c%d", xyz[i], (int) m_hand + 1);
      angles[i] = geeConfig->getValueFloat(buf, angles[i]);
      VEE_trace("%s # %s = %f", fname, buf, angles[i]);
    }

    Matrix3 tmp, tmp2, tmp3, tmp4;
    // tmp2.rotateZ(VEE_HALF_PI);
    tmp2.rotateZ(angles.z);
    // tmp.rotateX(VEE_HALF_PI);
    tmp.rotateX(angles.x);
    tmp4.rotateY(angles.y);
    
    tmp3.identity();
    // tmp3[1][1] = -1.0f; // Invert y
    tmp3[0][0] = -1.0f; // Invert x
    
    tmp = tmp4 * tmp * tmp2 * tmp3;
   
    m_napramatrix.identity();
    m_napramatrix.setRotation(tmp);
    m_napramatrix *= 20.0f;

    m_radiusNorm = 5.0f;
    m_radiusBoost = geeConfig->getValueFloat("radius_boost", 0.0);
    m_radius = geeConfig->getValueFloat(m_toolName + "/radius", 1.0);
  }

  void GeePathToX::start(const TimeStamp &now)
  {
    const char * fname = "GeePathToX::start";

    VEE_trace("%s # %lf ", fname, now.secondsD());

    m_trackerCount = (long) m_host->tracker()->outputBuffer().getSampleCount();
    
    updateOperatorGroup(now);
    updateBeginTime();

    // Now we build the "A" path data structures.
    m_pathAId = GEE_IdContainer::allocateId();
    GEE_PathInsert * ap = new GEE_PathInsert(m_pathAId);

    ap->interval().set(realTimeSinceBeginning());
    ap->setLayer(m_layer);
    ap->setGroup(m_operatorGroup.ptr()->id());
    m_engine->addOperator(ap);
    ap->act(& m_engine->world(), 0);

    if(m_needPaths) {
    
      m_pathA1 = & ap->path();
      m_pathA2 = dynamic_cast<GEE_Path3D *> 
	(m_engine->world().geometry(m_pathAId));

      if(!m_pathA1 || !m_pathA2) {
	VEE_error("%s # m_pathA1 = %p, m_pathA2 = %p", 
		  fname, m_pathA1, m_pathA2);
	return;
      }
      
      m_pathA2->setVisible(false);
      
      // Now we build the "B" path data structures.
      if(m_needTwoPaths) {
	m_pathBId = GEE_IdContainer::allocateId();
	GEE_PathInsert * ap = new GEE_PathInsert(m_pathBId);
	
	ap->interval().set(realTimeSinceBeginning());
	ap->setLayer(m_layer);
	ap->setGroup(m_operatorGroup.ptr()->id());
	m_engine->addOperator(ap);
	ap->act(& m_engine->world(), 0);
	
	m_pathB1 = & ap->path();
	m_pathB2 = dynamic_cast<GEE_Path3D *> 
	  (m_engine->world().geometry(m_pathBId));
	
	if(!m_pathB1 || !m_pathB2) {
	  VEE_error("%s # m_pathB1 = %p, m_pathB2 = %p", 
		    fname, m_pathB1, m_pathB2);
	  return;
	}
	m_pathB2->setVisible(false);
      }
    }

    // Now try to find the Napra:

    if(m_napra) {
      if(m_pathA1)
	m_pathA1->setActiveNapra(true);
      if(m_pathA2)      
	m_pathA2->setActiveNapra(true);
    }

    // Next we build the conversion action.
    
    configureConversionAction();

    if(!m_pathToX) {
      VEE_error("%s # Conversion = NULL (!)", fname);
    }
    else {
      m_pathToX->setLayer(m_layer);
      m_pathToX->setGroup(m_operatorGroup.ptr()->id());
      m_pathToX->interval().setBegin(realTimeSinceBeginning());
      m_pathToX->setKeepAlive(true);
      m_engine->addOperator(m_pathToX);
    }

    m_recording = true;

    VEE_trace("%s # EXIT OK", fname);
  }
  
  void GeePathToX::record(const TimeStamp &)
  {
    const char * fname = "GeePathToX::record";

    // VEE_trace("%s # %lf ", fname, now.secondsD());

    if(m_collectData && (m_needPaths && (!m_pathA1 || !m_pathA2))) {
      VEE_error("%s # m_pathA1 = %p, m_pathA2 = %p", 
		fname, m_pathA1, m_pathA2);
      return;
    }

    IONodeBuffer<Vector3> * handLocA = & m_host->handLoc(m_hand)->locations();
    IONodeBuffer<Matrix3> * handRotA = & m_host->handLoc(m_hand)->rotations();
    
    int other = (m_hand + 1) & 0x1;

    IONodeBuffer<Vector3> * handLocB = & m_host->handLoc(other)->locations();
    IONodeBuffer<Matrix3> * handRotB = & m_host->handLoc(other)->rotations();
    
    for(;m_trackerCount < (long) handLocA->getSampleCount(); m_trackerCount++){
      
      TimeStamp sampleTime = handLocA->getTimeStamp(m_trackerCount);
      //TimeStamp ts = m_host->realTimeToRecordTime(sampleTime - m_beginTime);
      TimeStamp ts = sampleTime - m_beginRealTime;
      double t = ts.secondsD();

      m_locationA = handLocA->getIndex(m_trackerCount);
      m_rotationA = handRotA->getIndex(m_trackerCount) * m_correctionRotation;
            
      m_locationB = handLocB->getIndex(m_trackerCount);
      m_rotationB = handRotB->getIndex(m_trackerCount) * m_correctionRotation;
      
      if(m_napra) {
	m_napraSample = m_napra->outputBuffer().getNewest();
      }

      GEE_PathVertex3D pv(m_locationA, m_rotationA, t);
      
      if(m_napra) {
	for(uint k=0; k < 3; k++)
	  pv.m_tips[k] = m_napramatrix * m_napraSample.fingerTips[m_hand][k];
      }
      
      m_pvA = pv;

      if(m_collectData && m_needPaths) {

	m_pathA1->addVertex(pv);
	m_pathA2->addVertex(pv);
	
	if(m_needTwoPaths) {

	  if(!m_pathB1 || !m_pathB2)
	    VEE_fatal("%s # B paths missing", fname);

	  pv.m_location = m_locationB;
	  pv.m_rotation = m_rotationB;
	  
	  if(m_napra) {
	    for(uint k=0; k < 3; k++)
	      pv.m_tips[k] = 
		m_napramatrix * m_napraSample.fingerTips[other][k];
	  }

	  m_pvB = pv;

	  m_pathB1->addVertex(pv);
	  m_pathB2->addVertex(pv);
	}
	
      }

      newSample();
    }
  }

  void GeePathToX::stop(const TimeStamp &now)
  {
    const char * fname = "GeePathToX::stop";

    TimeStamp ts = realTimeSinceBeginning();
    
    if(m_pathToX) {
      VEE_trace("%s # %s %lf ", fname, typeid(*m_pathToX).name(), 
		now.secondsD());
      
      // Finalize the actions that are necessary.
      
      m_pathToX->interval().setEnd(ts);
      m_pathToX->setKeepAlive(false);
    }

    m_operatorGroup.ptr()->calculateEndTime();

    m_pathA1 = 0;
    m_pathA2 = 0;

    m_pathB1 = 0;
    m_pathB2 = 0;
    m_recording = false;
  }

  void GeePathToX::update(const TimeStamp &now)
  {
    GeeRecorder::update(now);
    
    if(!m_recording) {

      IONodeBuffer<Vector3> * handLoc = & m_host->handLoc(m_hand)->locations();
      IONodeBuffer<Matrix3> * handRot = & m_host->handLoc(m_hand)->rotations();

      m_locationA = handLoc->getNewest();
      m_rotationA = handRot->getNewest() * m_correctionRotation;

      if(m_needTwoPaths) {

	int other = m_hand + 1;

	IONodeBuffer<Vector3> * handLocB =&m_host->handLoc(other)->locations();
	IONodeBuffer<Matrix3> * handRotB =&m_host->handLoc(other)->rotations();

	m_locationB = handLocB->getNewest();
	m_rotationB = handRotB->getNewest() * m_correctionRotation;
      }

      if(m_napra)
	m_napraSample = m_napra->outputBuffer().getNewest();

      GEE_PathVertex3D pv(m_locationA, m_rotationA, 0.0f);
      
      if(m_napra)
	for(uint k = 0; k < 3; k++)
	  pv.m_tips[k] = m_napramatrix * m_napraSample.fingerTips[m_hand][k];
      
      m_pvA = pv;

      newNonrecordingSample();
    }

    if(m_toolSphere && m_useToolSphere) {
      /* Vector3 l = m_locationA;
	 VEE_trace("GeePathToX::update # Display sphere %f %f %f", 
	 l.x, l.y, l.z); */
      Vector3 loc = m_locationA;
      float radius = m_toolSphere->radius();
      
      if(m_useNapraRadius && m_napra) {
	loc = m_pvA.calculateToolCenter();
	radius = 
	  m_pvA.calculateToolRadius(m_radius, m_radiusNorm, m_radiusBoost);
      }

      calculateToolSphereParams( & loc, & radius);

      m_toolSphere->setRadius(radius);
      m_toolSphere->transformation().setColumn3(3, loc);
      m_toolSphere->render(&m_engine->collector());
    }

  }

  
  void GeePathToX::render(GlData *)
  {
    //VEE_trace("GeePathToX::render # %p", m_napra);

  }


  void GeePathToX::newSample()
  {}

  void GeePathToX::newNonrecordingSample()
  {
    if(!m_highligthSphereArea || !m_useToolSphere)
      return;

    GEE_SphereGroup sg;
    calculateSphereGroup(&sg);

    GEE_World * w = & m_engine->world();

    GEE_Geometry * g = 0;

    for(GEE_LayerGeometryIterator it(w, m_layer); (g = *it); it.nextVisible()){
      if(g->intersects( & sg)) {
	g->setNearlySelected(true);
	VEE_trace("GeePathToX::newNonrecordingSample (%s) # hit %p (%s)", 
		  typeid(*this).name(), g, typeid(*g).name());
      }
    }
  }

  void GeePathToX::calculateToolSphereParams(Vector3 *, float *)
  {}

  void GeePathToX::useToolSphere(float radius)
  {
    if(!m_toolSphere)
      m_toolSphere = new GEE_Sphere();

    m_toolSphere->setRadius(radius);
    GEE_Material m = m_toolSphere->material();
    
    Vector4 color(m_host->geeEngine()->world().backgroundColor().data());
    for(int c=0; c < 3; c++)
      color[c] = color[c] > 0.5 ? 0.0 : 1.0;

    m.setColor(color);
    m_toolSphere->setMaterial(m);

    VEE_trace("GeePathToX::useToolSphere # %p %f", m_toolSphere, radius);
  }

  void GeePathToX::calculateSphereGroup(GEE_SphereGroup *sg)
  {
    Vector3 loc;
    float   radius;

    if(m_useNapraRadius && m_napra) {
      loc = m_pvA.calculateToolCenter();
      radius = 
	m_pvA.calculateToolRadius(m_radius, m_radiusNorm, m_radiusBoost);
    }
    else {
      loc = m_locationA;
      radius = m_radius;
    }

    sg->setRadius(radius);
    sg->addSphere(loc, radius);

  }

  void GeePathToX::updateOperatorGroup(const TimeStamp &now)
  {
    GeeRecorder * activeRec = m_host->currentActiveGeeRecorder();
    
    if(activeRec) {
      m_operatorGroup = activeRec->operatorGroup();
    }
    else {
      
      GEE_OperatorGroup *gr = new GEE_OperatorGroup();

      m_layer = m_host->geeConfig()->getValueInt("layer", -1); 
      double tscale = m_host->geeConfig()->getValueFloat("timescale", 1.0);
      
      gr->autoId();
      gr->setLayer(m_layer);
      gr->setScale(tscale);
      gr->setOffset(now);
      gr->setKeepAlive(true);

      m_operatorGroup = gr;

      m_engine->actions().addOperatorGroup2(m_operatorGroup);
    }
  }

  void GeePathToX::updateBeginTime()
  {
    GeeRecorder * activeRec = m_host->currentActiveGeeRecorder();
    
    if(activeRec)
      m_beginRealTime = activeRec->beginRealTime();
    else
      m_beginRealTime = 
	m_host->tracker()->outputBuffer().getTimeStamp(m_trackerCount);
  }

  TimeStamp GeePathToX::realTimeSinceBeginning()
  {
    TimeStamp realTime = 
      m_host->tracker()->outputBuffer().getLatestTimeStamp();
    
    return realTime - m_beginRealTime;
  }
  
  /////////////////////////////////////////////////////////////////////////////
  /////////////////////////////////////////////////////////////////////////////

  GeeNapraRender::GeeNapraRender(Crystal::Engine *host)
    : m_host(host),
      m_profile(new GEE_Profile),
      m_buildGlTextures(false),
      m_minDim(0.0f)
  {
    char buf[32];
    sprintf(buf, "%p-NapraRender", this);
    m_texName = buf;
  }

  GeeNapraRender::~GeeNapraRender()
  {
    delete m_profile;
  }
    
  void GeeNapraRender::setProfile(const GEE_Profile *prof)
  {
    *m_profile = *prof;

    m_buildGlTextures = 3;

    calculateGlTextures();
  }

  void GeeNapraRender::render(GlData *gld, bool withSpheres)
  {
    if(m_buildGlTextures)
      buildGlTextures(gld);      

    glPushAttrib(~0);
    
    glDisable(GL_TEXTURE_2D);

    VEE_GlTexture2D * tex = gld->m_textures.getTexture(m_texName);

    if(tex) 
      tex->use1D();
    else
      glDisable(GL_TEXTURE_1D);
    
    glDisable(GL_BLEND);
    // glPointSize(16);
    
    // glColor3fv(m_profile->material().ambient().data());
    glColor3f(1, 1, 1);
    
    glLineWidth(5);
    glBegin(GL_LINES);

    uint k, n = m_vertices.size();

    float tscale = 3.0f;

    for(k = 0; k < n; k++) {

      Vertex & v = m_vertices[k];

      if(!v.m_continues)
	continue;

      Vertex & v2 = m_vertices[(k + 1) % n];

      glTexCoord1f(0.0f);
      glVertex3fv(v.m_location.data());
      glTexCoord1f(tscale);
      glVertex3fv(v2.m_location.data());
    }

    glEnd();

    if(withSpheres) {

      float size = m_minDim * 0.2;

      GLUquadric * q = gld->m_quadric;

      gluQuadricDrawStyle(q, GLU_FILL);

      for(k = 0; k < n; k++) {
	glPushMatrix();
	Vector3 l = m_vertices[k].m_location;
	glTranslatef(l.x, l.y, l.z);
	gluSphere(q, size, 6, 6);
	glPopMatrix();
      }
      
    }


    glDisable(GL_TEXTURE_1D);

    // Teeth:

    /* 
    glBegin(GL_LINES);

    for(k = 0; k < n; k++) {
      const Vector3 & loc = m_vertices[k].m_location;

      glTexCoord1f(0);
      glVertex3fv(loc.data());
      glTexCoord1f(tscale);
      glVertex3fv((loc + m_napraNormal).data());
    }

    glEnd();
    */


    glPopAttrib();
  }
  
  void GeeNapraRender::update(const GEE_PathVertex3D &pv, float radius, 
			      const Vector3 & extension)
  {
    VEE_Matrix4 m = GEE_PathToMesh::createMatrix(pv, radius);
    
    uint k, n = m_profile->size();

    if(m_vertices.size() != n)
      m_vertices.resize(n);
    
    Vector3 ext = pv.m_rotation * extension;

    for(k = 0; k < n; k++) {
      const GEE_ProfileVertex2D & pv = m_profile->getVertex(k);
      
      Vertex & v = m_vertices[k];
      
      v.m_location  = m * Vector3(pv.m_location, 0.0f) + ext;
      v.m_continues = pv.m_continues;
    }
    
    float maxDim = 0.0f;
    m_minDim = 1.0e+5f;

    for(k = 0; k < n; k++) {
      for(uint h = k + 1; h < n; h++) {
	float d = (m_vertices[k].m_location - m_vertices[h].m_location).
	  length();
	if(d > maxDim)
	  maxDim = d;
	if(m_minDim > d)
	  m_minDim = d;
      }
    }
    
    Vector3 e1 = m_vertices[1].m_location - m_vertices[0].m_location;
    Vector3 e2 = m_vertices[2].m_location - m_vertices[1].m_location;
    Vector3 norm = cross(e1, e2);
    norm.normalize(maxDim * 0.5);
    m_napraNormal = norm;
  }


  /** Transfers the relevant textures to the OpenGL engine. */

  void GeeNapraRender::buildGlTextures(GlData *gld)
  {
    VEE_GlTexture2D * tex = gld->m_textures.getTexture(m_texName);

    GLuint id;
    
    if(!tex) {
      VEE_GlTexture2D tmp;
      tmp.allocateId();
      id = tmp.id();
      gld->m_textures.addTexture(m_texName, tmp);
      VEE_trace("GeeNapraRender::buildGlTextures # New texture (%u)",
		(uint) tmp.id());
    }
    else
      id = tex->id();
    
    glBindTexture(GL_TEXTURE_1D, id);

    glTexImage1D(GL_TEXTURE_1D, 0, GL_RGB, m_texture.size(), 0, 
		 GL_RGB, GL_UNSIGNED_BYTE, m_texture[0].data());
    
    glTexParameteri(GL_TEXTURE_1D, GL_TEXTURE_WRAP_S, GL_REPEAT);
    glTexParameteri(GL_TEXTURE_1D, GL_TEXTURE_WRAP_T, GL_REPEAT);
    glTexParameteri(GL_TEXTURE_1D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
    glTexParameteri(GL_TEXTURE_1D, GL_TEXTURE_MIN_FILTER, GL_LINEAR );
    glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
    
  }

  void GeeNapraRender::calculateGlTextures()
  {
    // Counter-colors of the current background and drawing color:

    VEE_Rgba c1 = m_host->geeConfig()->constructRgba(m_tool).farColor();
    VEE_Rgba c2 = m_host->geeEngine()->world().backgroundColor().farColor();
    
    uint i, size = 64;
    
    if(m_texture.size() != size)
      m_texture.resize(size);

    Rgb rgb1, rgb2;

    rgb1.copy(c1.data());
    rgb2.copy(c2.data());

    for(i = 0; i < size; i++)
      m_texture[i] = i > (size / 2) ? rgb1 : rgb2;
  }
  

  /////////////////////////////////////////////////////////////////////////////
  /////////////////////////////////////////////////////////////////////////////

  GeeMesh::GeeMesh(Crystal::Engine *host, GEE_Engine *e)
    : GeePathToX(host, e),
      m_napraRender(host)
  {
    m_toolName = "Mesh";
    m_napraRender.setToolName(m_toolName);
  }

  GeeMesh::~GeeMesh()
  {
  }

  void GeeMesh::activate(const TimeStamp &now)
  {
    GeePathToX::activate(now);

    GeeConfig * geeConfig = m_host->geeConfig(); 

    m_napraRender.setProfile( & geeConfig->profile(m_toolName));

    if(m_napra)
      m_host->setRenderHand(m_hand, false);
  }

  void GeeMesh::update(const TimeStamp &now)
  {
    GeePathToX::update(now);
    m_napraRender.update();
  }

  void GeeMesh::deactivate(const TimeStamp &now)
  {
    GeePathToX::deactivate(now);

     if(m_napra)
       m_host->setRenderHand(m_hand, true);
  }

  void GeeMesh::render(GlData * gld)
  {
    if(!m_napra)
      return;

    m_napraRender.render(gld, false);
  }
  
  GEE_PathToMesh * GeeMesh::createPathToMesh(GEE_id_t pathId, 
					     int align,
					     bool useNapra)
  {    
    GeeConfig * geeConfig = m_host->geeConfig(); 

    GEE_PathToMesh * apm = new GEE_PathToMesh(pathId, -1);

    apm->setProfile(geeConfig->profile(m_toolName));
    apm->setMaterial(geeConfig->material(m_toolName));

    float threshold = geeConfig->getValueFloat("threshold", 0.2);
    int relativeThreshold = geeConfig->getValueInt("threshold-relative", 1);
    
    if(relativeThreshold)
      threshold = VEE_max(m_radius, 0.1f) * threshold;

    apm->setScale(m_radius);
    apm->setThreshold(threshold);
    apm->setAlignment((GEE_AlignMode) align);
    apm->useNapra(useNapra);

    VEE_trace("GeeMesh::createPathToMesh # scale = %f %u", 
	      m_radius, apm->profile().size());
    
    return apm;
  }

  void GeeMesh::newSample()
  {
    newNonrecordingSample();
  }

  void GeeMesh::newNonrecordingSample()
  {
    if(m_napra)
      m_napraRender.update(m_pvA, m_radius, Vector3(0, 0, 0));
  }

  void GeeMesh::configureConversionAction()
  {
    GeeConfig * geeConfig = m_host->geeConfig(); 
    int align = geeConfig->getValueInt("align", GEE_ALIGN_MOTION);
    bool useNapra = m_napra != 0;

    GEE_PathToMesh * p2m = createPathToMesh(m_pathAId, align, useNapra);

    m_finger = geeConfig->getValueInt(m_toolName + "/finger", -1);

    if(m_finger >= 0) {
      float radius2 = geeConfig->getValueFloat(m_toolName + "/span", -1);
      
      p2m->setSpecialFinger(m_finger);
      p2m->setRadius(radius2);
    }

    m_pathToX = p2m;

    /* if(m_napra)
       m_host->setRenderHand(m_hand, false); */
  }


  /////////////////////////////////////////////////////////////////////////////
  /////////////////////////////////////////////////////////////////////////////

  GeeLines::GeeLines(Crystal::Engine *host, GEE_Engine *e)
    : GeePathToX(host, e)
  {
    m_toolName = "Lines";
  }

  GeeLines::~GeeLines()
  {}

  void GeeLines::activate(const TimeStamp &now)
  {
    GeePathToX::activate(now);
    m_color = m_host->geeEngine()->world().backgroundColor().farColor();
  }

  void GeeLines::render(GlData *)
  {
    if(!m_napra)
      return;

    VEE_Matrix4 m = GEE_PathToMesh::createMatrix(m_pvA, m_radius);
    Vector3 loc = m * Vector3(0, 0.5, 0);
    
    glDisable(GL_TEXTURE_2D);

    glPointSize(6);
    glColor3fv(m_color.data());

    glBegin(GL_POINTS);
    glVertex3fv(loc.data());
    glEnd();
  }

  void GeeLines::configureConversionAction()
  {
    GeeConfig * geeConfig = m_host->geeConfig(); 

    GEE_PathToLines * apm = new GEE_PathToLines(m_pathAId);

    apm->setMaterial(geeConfig->material(m_toolName));
    apm->setWidth(geeConfig->getValueFloat("linewidth", 2));

    if(m_napra) {
      apm->setUseNapra(true);
      apm->setNapraRadius(m_radius);
    }

    m_pathToX = apm;
  }

  /////////////////////////////////////////////////////////////////////////////
  /////////////////////////////////////////////////////////////////////////////

  GeeParticles::GeeParticles(Crystal::Engine *host, GEE_Engine *e)
    : GeePathToX(host, e),
      m_singleShots(false),
      m_particleObj(new GEE_ParticleObject),
      m_motionAlign(VEE_ALIGN_BILLBOARD),
      m_size(1.0)
  {
    GEE_ParticleData tmp;
    tmp.m_particle = new VEE_Particle();
    m_particleObj->addParticle(tmp);

    GEE_ParticleObject::iterator it = m_particleObj->begin();
    m_particle = & GEE_ParticleObject::getParticleData(it);
    
    m_useToolSphere = true;
    m_useNapraRadius = true;
    m_toolName = "Particles";
  }

  GeeParticles::~GeeParticles()
  {
    delete m_particle;
  }

  void GeeParticles::activate(const TimeStamp &now)
  {
    GeePathToX::activate(now);

    m_size = m_host->geeConfig()->getValueFloat(m_toolName + "/size", 0.5);

    m_singleShots = 
      m_host->geeConfig()->getValueInt(m_toolName + "/singleshots");

    m_useToolSphere = !m_singleShots;
    m_useNapraRadius = m_useToolSphere;

    configureParticle(m_particle);
  }

  void GeeParticles::update(const TimeStamp &now)
  {
    GeePathToX::update(now);

    if(m_singleShots || !m_recording)
      m_particleObj->render(&m_engine->collector());
  }

  void GeeParticles::start(const TimeStamp &now)
  {
    if(m_singleShots) {
      
      updateOperatorGroup(now);

      GEE_InsertParticle * ip = new GEE_InsertParticle();

      ip->particle() = * m_particle;

      GEE_OperatorGroup * gr = m_operatorGroup.ptr();
    
      ip->setGroup(gr->id());
      ip->setLayer(m_layer);
      ip->interval().set(realTimeSinceBeginning());

      m_engine->addOperator(ip);
    }
    else GeePathToX::start(now);
  }

  void GeeParticles::stop(const TimeStamp &now)
  {
    if(!m_singleShots)
      GeePathToX::stop(now);
  }

  GEE_PathToParticles * GeeParticles::createPathToParticles(GEE_id_t pathId)
  {
    GeeConfig * geeConfig = m_host->geeConfig(); 

    GEE_PathToParticles * apm = new GEE_PathToParticles(pathId);

    float size = geeConfig->getValueFloat(m_toolName + "/size", 0.5);
    float rate = geeConfig->getValueFloat
      (m_toolName + "/pressure", 0.5) * 5 / size;

    apm->setRate(rate);
    apm->setRadius(m_radius);
    apm->setRadiusExaggerate(m_radiusBoost);
    apm->setRadiusNorm(m_radiusNorm);
    apm->setUseNapra(m_napra != 0);

    configureParticle( & apm->particle());
    
    return apm;
  }
  
  void GeeParticles::newNonrecordingSample()
  {
    newSample();
  }
  
  void GeeParticles::newSample()
  {
    Matrix4 m = GEE_PathToMesh::createMatrix(m_pvA, m_size);
    
    // Transformed origin etc:
    Vector3 to = m.column(3).vector3();
    Vector3 hand = m_singleShots ? m_pvA.m_location : to;

    m_particle->setLocation(hand);
    
    if(m_singleShots) {
      Vector3 tx = m * Vector3(1, 0, 0);
      Vector3 ty = m * Vector3(0, 1, 0);
      
      Vector3 dx = tx - to;
      Vector3 dy = ty - to;

      float size = (dx.length() + dy.length()) * 0.5;
      m_particle->setSize(size);
    }

    if(m_motionAlign == VEE_ALIGN_BILLBOARD) {

      // Head to hand vector
      Vector3 hth = m_host->headLoc()->locations().getNewest() - hand;
      
      Matrix3 rot;
      rot.rotateY(-atan2(hth.x, hth.z));
      
      Vector3 v = rot * (m_pvA.m_rotation * VEE_Vector3_UNIT_Z);
      
      float angle = -atan2(v.x, v.y);
      
      m_particle->setRotation(angle);
    }
    else {

      if(m_motionAlign == VEE_ALIGN_MOTION_TOTAL) {

	Vector3 v = m_pvA.m_rotation * VEE_Vector3_UNIT_Y;
	m_particle->setVelocity(v);
	m_particle->setRotation(0.0);
      }
      else if(m_motionAlign == VEE_ALIGN_FIXED) {

	Matrix3 rot = m.getRotation();
	Vector3 axis;
	float radians;
	rot.getRotateAroundAxis(axis, radians);

	m_particle->setVelocity(axis);
	m_particle->setRotation(radians);
      }
    }
  }

  void GeeParticles::configureConversionAction()
  {
    if(m_singleShots)
      return;

    GEE_PathToParticles * apm = createPathToParticles(m_pathAId);
    
    m_pathToX = apm;
  }

  void GeeParticles::configureParticle(GEE_ParticleData *p)
  {
    GeeConfig * geeConfig = m_host->geeConfig(); 

    std::string texName = geeConfig->getValueString("texture", "noise-1");
    float size = geeConfig->getValueFloat(m_toolName + "/size", 0.5);

    p->setColor(geeConfig->constructRgba(m_toolName));
    p->setTexture(texName, GEE_World::textureId(texName));
    p->setSize(size);

    std::string renderClass = 
      geeConfig->getValueString(m_toolName + "renderclass");

    VEE_RenderClass * rc = GEE_World::renderClass(renderClass);

    m_motionAlign = rc ? rc->align() : VEE_ALIGN_BILLBOARD;
    
    p->setRenderClass(renderClass, rc);
  }

  /////////////////////////////////////////////////////////////////////////////
  /////////////////////////////////////////////////////////////////////////////

  
  GeeMultiBrush::GeeMultiBrush(Crystal::Engine * host, GEE_Engine * e)
    : GeePathToX(host, e),
      m_napraRender(host)
  {
    m_toolName = "MultiBrush";
    m_napraRender.setToolName(m_toolName);
  }
  
  GeeMultiBrush::~GeeMultiBrush()
  {}

  void GeeMultiBrush::activate(const TimeStamp &now)
  {
    GeePathToX::activate(now);

    GeeConfig * geeConfig = m_host->geeConfig(); 

    m_napraRender.setProfile( & geeConfig->profile(m_toolName));
    
    float extrudeZ = geeConfig->getValueFloat(m_toolName + "/extrude");
    m_extension.make(0, 0, extrudeZ);

    if(m_napra)
      m_host->setRenderHand(m_hand, false); 
  }

  void GeeMultiBrush::update(const TimeStamp &now)
  {
    GeePathToX::update(now);
    m_napraRender.update();
  }

  void GeeMultiBrush::deactivate(const TimeStamp &now)
  {
    GeePathToX::deactivate(now);

    if(m_napra)
      m_host->setRenderHand(m_hand, true);
  }

  void GeeMultiBrush::render(GlData * gld)
  {
    if(!m_napra)
      return;

    m_napraRender.render(gld, true);
  }
  
  void GeeMultiBrush::stop(const TimeStamp &now)
  {
    GeePathToX::stop(now);
    
    TimeStamp ts = realTimeSinceBeginning();
    
    for(uint i = 0; i < m_conversions.size(); i++) {
      GEE_Operator * conv = m_conversions[i];
      conv->interval().setEnd(ts);
      conv->setKeepAlive(false);
    }

    m_conversions.clear();
  }

  void GeeMultiBrush::configureConversionAction()
  {
    const char * fname = "GeeMultiBrush::configureConversionAction";

    GeeConfig * geeConfig = m_host->geeConfig(); 
    
    GEE_PathMultiply * multiply = new GEE_PathMultiply(m_pathAId);
    multiply->autoId();

    float threshold = geeConfig->getValueFloat(m_toolName + "/threshold", 0.2);
    int align = geeConfig->getValueInt(m_toolName+ "/align", GEE_ALIGN_MOTION);
    int relativeThreshold = 
      geeConfig->getValueInt(m_toolName + "/threshold-relative", 1);
    const GEE_Profile & profile = geeConfig->profile(m_toolName);
    std::string geoType = 
      geeConfig->getValueString(m_toolName + "/geotype", "Mesh");
    float toolradius = geeConfig->getValueFloat(geoType + "/radius");
    VEE_Rgba color = geeConfig->constructRgba(m_toolName);
    GEE_Material material = geeConfig->material(m_toolName);

    if(relativeThreshold)
      threshold = VEE_max(m_radius, 0.1f) * threshold;

    multiply->setScale(m_radius);
    multiply->setThreshold(threshold);
    multiply->setAlignment((GEE_AlignMode) align);
    multiply->setProfile(profile);
    multiply->setKeepAlive(true);
    multiply->setExtrude(m_extension);

    // Now create the mesh generators:
    
    GeeRecorders * recs = m_host->recorders(m_hand);

    GeeMesh * tomesher = 
      dynamic_cast<GeeMesh *> (recs->getRecorder("Mesh"));
    GeeParticles * toparticles = 
      dynamic_cast<GeeParticles *> (recs->getRecorder("Particles"));
    
    if(!tomesher)
      VEE_fatal("%s # mesher = NULL", fname);
    if(!toparticles)
      VEE_fatal("%s # toparticles = NULL", fname);

    /* 
    tomesher->activate(0);
    toparticles->activate(0);

    tomesher->deactivate(0);
    toparticles->deactivate(0);
    */

    m_conversions.clear();

    TimeStamp ts = realTimeSinceBeginning();
    GEE_id_t gid = m_operatorGroup.ptr()->id();

    for(uint i = 0; i < profile.size(); i++) {
      
      GEE_id_t pathId = multiply->pathId(i);

      GEE_Operator * conv = 0;

      if(geoType == "Mesh") {
      
	GEE_PathToMesh * ptm = 
	  tomesher->createPathToMesh(pathId, GEE_ALIGN_MOTION, false);
	ptm->setMaterial(material);
	ptm->useNapra(false);
	ptm->setScale(toolradius);

	VEE_trace("%s # Mesh with scale = %f", fname, ptm->scale());

	conv = ptm;
      }
      else if(geoType == "Lines") {
	
	GeeConfig * geeConfig = m_host->geeConfig(); 

	GEE_PathToLines * apm = new GEE_PathToLines(pathId);
	
	apm->setWidth(geeConfig->getValueFloat("linewidth", 2));
	apm->setMaterial(material);

	conv = apm;
      }
      else if(geoType == "Particles") {
	GEE_PathToParticles * ptp = toparticles->createPathToParticles(pathId);
	
	ptp->particle().setColor(color);
	ptp->setUseNapra(false);
	ptp->setRadius(toolradius);

	conv = ptp;
      }
      else {
	VEE_error("%s # Unsupported geo type %s", fname, geoType.c_str());
      }

      if(conv) {
	conv->setKeepAlive(true);
	conv->setGroup(gid);
	conv->interval().setBegin(ts);

	m_engine->addOperator(conv);
	m_conversions.push_back(conv);
      }
    }

    m_pathToX = multiply;
  }
  
  void GeeMultiBrush::newSample()
  {
    newNonrecordingSample();
  }

  void GeeMultiBrush::newNonrecordingSample()
  {
    if(m_napra)
      m_napraRender.update(m_pvA, m_radius, m_extension );
  }


  /////////////////////////////////////////////////////////////////////////////
  /////////////////////////////////////////////////////////////////////////////

  GeeMove::GeeMove(Crystal::Engine *host, GEE_Engine *e)
    : GeePathToX(host, e)
  {
    m_useToolSphere = true;
    m_useNapraRadius = true;
    m_toolName = "Magnet";
  }

  GeeMove::~GeeMove()
  {}

  void GeeMove::activate(const TimeStamp &now)
  {
    GeePathToX::activate(now);
  }

  void GeeMove::configureConversionAction()
  {
    GeeConfig * geeConfig = m_host->geeConfig(); 

    float pressure = geeConfig->getValueFloat(m_toolName + "/pressure", 1.0);

    VEE_trace("GeeMove::configureConversionAction # pressure = %f", pressure);

    GEE_MoveAroundPath * apm = 
      new GEE_MoveAroundPath(m_pathAId, -1, false, m_napra != 0, 
			     m_radiusNorm, m_radiusBoost);    

    apm->setParams(m_radius, pressure);
    
    useToolSphere(m_radius);

    m_pathToX = apm;
  }

  /////////////////////////////////////////////////////////////////////////////
  /////////////////////////////////////////////////////////////////////////////

  GeeColor::GeeColor(Crystal::Engine *host, GEE_Engine *e, bool vertexColors)
    : GeePathToX(host, e),
      m_vertexColors(vertexColors)
  {
    m_useToolSphere = true;
    m_useNapraRadius = true;
    m_highligthSphereArea = !m_vertexColors;
    m_toolName = vertexColors ? "Color" : "ColorGeo";
  }

  GeeColor::~GeeColor()
  {}

  void GeeColor::activate(const TimeStamp &now)
  {
    GeePathToX::activate(now);
  }

  void GeeColor::configureConversionAction()
  {
    GeeConfig * geeConfig = m_host->geeConfig(); 

    GEE_ColorAroundPath * apm = 
      new GEE_ColorAroundPath(m_pathAId, 
			      m_vertexColors, 
			      m_radiusNorm, 
			      m_radiusBoost,
			      m_napra != 0);
    
    apm->setColor(geeConfig->constructRgba(m_toolName));
    apm->setRadius(m_radius);
    apm->setPressure(geeConfig->getValueFloat(m_toolName + "/pressure", 1.0));

    m_pathToX = apm;
  }

  /////////////////////////////////////////////////////////////////////////////
  /////////////////////////////////////////////////////////////////////////////

  GeeRetexture::GeeRetexture(Crystal::Engine *host, GEE_Engine *e)
    : GeePathToX(host, e)
  {    
    m_useToolSphere = true;
    m_useNapraRadius = true;
    m_toolName = "Retexture";
  }

  GeeRetexture::~GeeRetexture()
  {}

  void GeeRetexture::configureConversionAction()
  {
    GeeConfig * geeConfig = m_host->geeConfig(); 

    GEE_TextureAroundPath * apm = 
      new GEE_TextureAroundPath(m_pathAId, m_radiusNorm, m_radiusBoost, 
				m_napra != 0);

    apm->setTextureName(geeConfig->getValueString("texture", "noise-1"));
    apm->setRadius(m_radius);
    
    m_pathToX = apm;
  }

  /////////////////////////////////////////////////////////////////////////////
  /////////////////////////////////////////////////////////////////////////////

  GeeGroup::GeeGroup(Crystal::Engine *host, GEE_Engine *e)
    : GeePathToX(host, e)
  {
    m_useToolSphere = true;
    m_useNapraRadius = true;
    m_highligthSphereArea = true;
    m_toolName = "Group";
  }

  GeeGroup::~GeeGroup()
  {}

  void GeeGroup::activate(const TimeStamp &now)
  {
    GeePathToX::activate(now);
  }

  void GeeGroup::configureConversionAction()
  {
    GEE_GroupAroundPath * apm = 
      new GEE_GroupAroundPath(m_pathAId);
    
    apm->setRadius(m_radius);
    
    m_pathToX = apm;
  }

  /////////////////////////////////////////////////////////////////////////////
  /////////////////////////////////////////////////////////////////////////////

  GeeErase::GeeErase(Crystal::Engine *host, GEE_Engine *e)
    : GeePathToX(host, e)
  {
    m_useToolSphere = true;
    m_useNapraRadius = true;
    m_toolSphereColor = VEE_Rgba::Red;
    m_toolName = "Erase";
  }

  GeeErase::~GeeErase()
  {}

  void GeeErase::configureConversionAction()
  {
    GEE_EraseAroundPath * apm = 
      new GEE_EraseAroundPath(m_pathAId, true, 
			      m_radiusNorm, m_radiusBoost,
			      m_napra != 0);
    
    apm->setRadius(m_radius);
    useToolSphere(m_radius);
    
    m_pathToX = apm;
  }

  /////////////////////////////////////////////////////////////////////////////
  /////////////////////////////////////////////////////////////////////////////

  GeeParticleRotate::GeeParticleRotate(Crystal::Engine *host, GEE_Engine *e)
    : GeePathToX(host, e)
  {
    m_useToolSphere = true;
    m_useNapraRadius = true;
    m_toolSphereColor = VEE_Rgba::Red;
    m_toolName = "ParticleRotate";
  }

  GeeParticleRotate::~GeeParticleRotate()
  {}

  void GeeParticleRotate::configureConversionAction()
  {
    VEE_trace("GeeParticleRotate::configureConversionAction");

    GeeConfig * geeConfig = m_host->geeConfig(); 

    GEE_ParticleRotate * pr = 
      new GEE_ParticleRotate(m_pathAId);

    pr->setRadius(m_radius);
    pr->setPressure(geeConfig->getValueFloat(m_toolName + "/pressure", 1.0f));
    pr->setRadiusBoost(m_radiusBoost);
    pr->setRadiusNorm(m_radiusNorm);
    pr->setUseNapra(m_napra != 0);

    useToolSphere(m_radius);
    
    m_pathToX = pr;
  }


  /////////////////////////////////////////////////////////////////////////////
  /////////////////////////////////////////////////////////////////////////////

  GeeEraseGeo::GeeEraseGeo(Crystal::Engine *host, GEE_Engine *e)
    : GeePathToX(host, e)
  {
    m_useToolSphere = true;
    m_useNapraRadius = true;
    m_highligthSphereArea = true;

    m_toolSphereColor = VEE_Rgba::Red;
    m_toolName = "EraseGeo";
  }

  GeeEraseGeo::~GeeEraseGeo()
  {}

  void GeeEraseGeo::activate(const TimeStamp &now)
  {
    GeePathToX::activate(now);
    useToolSphere(m_radius);
  }

  void GeeEraseGeo::configureConversionAction()
  {
    GEE_EraseAroundPath * apm = 
      new GEE_EraseAroundPath(m_pathAId, false, 
			      m_radiusNorm, m_radiusBoost, m_napra != 0);
    
    apm->setRadius(m_radius);
    
    m_pathToX = apm;
  }

  /////////////////////////////////////////////////////////////////////////////
  /////////////////////////////////////////////////////////////////////////////

  GeeMoveGeo::GeeMoveGeo(Crystal::Engine *host, GEE_Engine *e, 
			 bool rotate, bool scale, bool uniformScale,
			 bool onlyTurn)
    : GeePathToX(host, e),
      m_includeRotation(rotate),
      m_scale(scale),
      m_uniformScale(uniformScale),
      m_onlyTurn(onlyTurn),
      m_moveRecord(false)
  {
    m_needTwoPaths = scale;
    m_useToolSphere = true;
    m_useNapraRadius = true;
    m_toolSphereColor = scale ? VEE_Rgba::Green : VEE_Rgba::Cyan;

    if(rotate == false && scale == true && 
       uniformScale == false && onlyTurn && true)
      m_toolName = "TurnGeo";
    else if(rotate == false && scale == true && 
	    uniformScale == false)
      m_toolName = "ScaleAxis";
    else if(rotate == false && scale == true)
      m_toolName = "Scale";
    else if(rotate == false)
      m_toolName = "Move";
    else
      m_toolName = "MoveRot";
  }

  GeeMoveGeo::~GeeMoveGeo()
  {}

  void GeeMoveGeo::activate(const TimeStamp &now)
  {
    GeePathToX::activate(now);
    useToolSphere(m_radius);
    m_moveRecord = false;
  }

  void GeeMoveGeo::start(const TimeStamp &)
  {
    m_moveRecord = true;
    m_collectData = false;
  }

  void GeeMoveGeo::stop(const TimeStamp &now)
  {
    if(!m_collectData)
      return;

    GeePathToX::stop(now);
    
    m_moveRecord = false;
  }

  void GeeMoveGeo::configureConversionAction()
  {
    GeeConfig * geeConfig = m_host->geeConfig(); 

    if(m_onlyTurn) {

      // Find the geometry to turn:

      GEE_SphereGroup sg;
      calculateSphereGroup(&sg);
      
      GEE_World * w = & m_engine->world();
      
      GEE_Geometry * g = 0;
      GEE_id_t id = -1;
      
      for(GEE_LayerGeometryIterator it(w, m_layer); 
	  (g = *it) && id < 0; it.nextVisible())
	if(g->intersects( & sg))
	  id = g->id();
      
      if(id < 0)
	return;
      
      float omega = geeConfig->getValueFloat("omega", 0.01);

      m_pathToX = 
	new GEE_TurnAroundPath(m_pathAId, id, omega);
    }
    else if(m_scale) {
      m_pathToX = 
	new GEE_ScaleAroundPaths(m_pathAId, m_pathBId, -1, m_uniformScale);
    }
    else {
      m_pathToX = 
	new GEE_MoveAroundPath(m_pathAId, false, m_includeRotation);
    }

    m_collectData = false;
  }

  void GeeMoveGeo::newSample()
  {
    if(m_collectData)
      return;
    
    GEE_World * w = & m_engine->world();

    GEE_Geometry * g = 0;

    int c = 0;

    GEE_SphereGroup sg;
    calculateSphereGroup(&sg);
    
    for(GEE_LayerGeometryIterator it(w, m_layer); (g = *it); it.nextVisible()){

      VEE_trace("GeeMoveGeo::newSample # %s", typeid(*g).name());

      if(g->intersects( & sg)) {
	
	if(m_scale) {
	  
	  GEE_ScaleAroundPaths * scale = 
	    dynamic_cast<GEE_ScaleAroundPaths *> (m_pathToX);
	  
	  m_collectData = true;
	  scale->addGeometryId(g->masterGroup());
	}
	else {

	  GEE_MoveAroundPath * move = 
	    dynamic_cast<GEE_MoveAroundPath *> (m_pathToX);
	  
	  if(!move)
	    VEE_fatal("GeeMoveGeo::newSample");
	  
	  m_collectData = true;
	  move->addGeometryId(g->masterGroup());
	}

	// return;
      }
      c++;
    }
    VEE_trace("GeeMoveGeo::newSample # %d", c);
  }  

  void GeeMoveGeo::newNonrecordingSample()
  {
    const char * fname = "GeeMoveGeo::newNonrecordingSample";

    // VEE_trace("%s # ENTER %f %f %f %f", fname, m_locationA.x, m_locationA.y, m_locationA.z, m_radius);

    GEE_SphereGroup sg;
    calculateSphereGroup(&sg);
    
    GEE_World * w = & m_engine->world();

    GEE_Geometry * g = 0;

    for(GEE_LayerGeometryIterator it(w, m_layer); (g = *it); it.nextVisible()){
      if(g->intersects( & sg)) {
	g->setNearlySelected(true);
	VEE_trace("%s # hit %p (%s)", fname, g, typeid(*g).name());

	if(m_moveRecord) {
	  VEE_trace("%s # Starting real recording", fname);
	  GeePathToX::start(m_time);
	  return;
	}
      }
    }
  }

  /////////////////////////////////////////////////////////////////////////////
  /////////////////////////////////////////////////////////////////////////////

  
  GeeInsertGeo::GeeInsertGeo(Crystal::Engine *ce, GEE_Engine *ge, Source s)
    : GeePathToX(ce, ge),
      m_source(s)
  {
    m_modelCenter.clear();
    m_matrix.identity();
    m_needPaths = false;
    m_toolName = (s == QUAD) ? "InsertQuad" : "Insert";
  }

  GeeInsertGeo::~GeeInsertGeo()
  {}

  void GeeInsertGeo::activate(const TimeStamp &now)
  {
    GeePathToX::activate(now);
  }

  void GeeInsertGeo::update(const TimeStamp &now)
  {
    GeePathToX::update(now);

    if(m_recording && m_model.ptr()) {
      calculateTransformation();

      GEE_CollectorRenderer * r = & m_host->geeEngine()->collector();
          
      r->pushTransform(m_matrix);
      m_model.ptr()->render(r);
      r->popTransform();
    }
  }
  
  void GeeInsertGeo::stop(const TimeStamp &now)
  {
    VEE_trace("GeeInsertGeo::stop");

    GeePathToX::stop(now);

    calculateTransformation();

    GEE_CopyGeometry * insert = new GEE_CopyGeometry(m_modelId, false);

    updateOperatorGroup(now);

    GEE_OperatorGroup *gr = m_operatorGroup.ptr();
    
    insert->setTransformation(m_matrix.data());
    insert->setLayer(m_layer);
    insert->interval().set(realTimeSinceBeginning());
    insert->setGroup(gr->id());

    if(m_source != GALLERY && m_model.ptr()) {
      insert->geometries() = * m_model.ptr();
    }

    m_engine->addOperator(insert);
  }

  /** In this method we take current parameters from the config and
      initialize the matrices for the model we are about to insert
      into the world. */
  void GeeInsertGeo::configureConversionAction()
  {
    const char * fname = "GeeInsertGeo::configureConversionAction";

    GeeConfig * geeConfig = m_host->geeConfig(); 

    m_model.breakLink();

    VEE_RefPtr<GEE_World> * model = 0;
    VEE_RefPtr<GEE_World> tmp;

    if(m_source == GALLERY) {
    
      m_modelId = geeConfig->getValueInt("insert_source_id");
      
      model = m_host->geeEngine()->gallery().item2(m_modelId);
      
      if(!model) {
	VEE_error("%s # NULL model %ld", fname, (long) m_modelId);
	return;
      }
    }
    else if(m_source == QUAD) {
      int xdiv = geeConfig->getValueInt("xdiv");
      int ydiv = geeConfig->getValueInt("ydiv");

      float width  = geeConfig->getValueFloat("squarewidth");
      float height = geeConfig->getValueFloat("squareheight");
      
      std::string tex = geeConfig->getValueString("texturename");

      VEE_trace("%s # texture is \"%s\"", fname, tex.c_str());

      GEE_World * w = new GEE_World();

      GEE_TriangleMesh * trimesh = new GEE_TriangleMesh();

      Vector4 color(geeConfig->constructRgba(m_toolName).data());
      
      GEE_Material mat(0.0, color);
      mat.setTexture(tex);

      trimesh->setMaterial(mat);

      trimesh->createRectangularGrid(xdiv, ydiv, width, height);

      trimesh->autoId();
      
      w->addGeometry(trimesh);

      tmp = w;
      model = & tmp;
    }

    m_model = *model;

    VEE_BoundingBox bb;

    m_model.ptr()->getBoundingBox(&bb);

    m_modelCenter = bb.center();
  }

  void GeeInsertGeo::calculateTransformation()
  {
    Matrix4 m1, m2;
    m1.identity();
    m2 = m1;

    m1.setColumn3(3, -m_modelCenter);

    m2.setRotation(m_rotationA);
    
    m1 = m2 * m1;
    m2.identity();
    m2.setColumn3(3, m_locationA);

    m_matrix = m2 * m1;
  }


  /////////////////////////////////////////////////////////////////////////////
  /////////////////////////////////////////////////////////////////////////////

  GeeSelectGeo::GeeSelectGeo(Crystal::Engine *ce, GEE_Engine *ge)
    : GeePathToX(ce, ge)
  {
    m_useToolSphere = true;
    m_needPaths = false;
    m_useNapraRadius = true;
    m_toolName = "SelectGeo";
  }

  GeeSelectGeo::~GeeSelectGeo()
  {}

  void GeeSelectGeo::activate(const TimeStamp &now)
  {
    GeePathToX::activate(now);

    GeeConfig * geeConfig = m_host->geeConfig(); 

    m_layer  = geeConfig->getValueInt("layer", -1);
  }

  void GeeSelectGeo::update(const TimeStamp &now)
  {
    GeePathToX::update(now);
    select(m_recording);
  }
    
  void GeeSelectGeo::configureConversionAction()
  {
    m_ids.clear();
    select(true);
  }

  void GeeSelectGeo::select(bool reallySelect)
  {
    const char * fname = "GeeSelectGeo::select";

    GEE_SphereGroup sg;
    calculateSphereGroup(&sg);
    
    GEE_World * w = & m_engine->world();

    GEE_Geometry * g = 0;

    for(GEE_LayerGeometryIterator it(w, m_layer); (g = *it); it.nextVisible()){

      if(reallySelect && m_ids.find(g->id()) != m_ids.end())
	continue;

      if(g->intersects( & sg)) {
	if(reallySelect) {
	  // Toggle the selection bit:
	  g->setSelectionState(g->selectionState() ^ GEE_SELECT_BIT);
	  m_ids.insert(g->id());
	}
	else
	  g->setNearlySelected(true);

	VEE_trace("%s # hit %p (%s)", fname, g, typeid(*g).name());
      }
    }
  }

  /////////////////////////////////////////////////////////////////////////////
  /////////////////////////////////////////////////////////////////////////////

  GeeGroupGeo::GeeGroupGeo(Crystal::Engine *ce, GEE_Engine *ge)
    : GeePathToX(ce, ge),
      m_layer(-1),
      m_group(0)
  {
    m_useToolSphere = true;
    m_needPaths = false;
    m_useNapraRadius = true;
    m_toolName = "GroupGeo";
  }

  GeeGroupGeo::~GeeGroupGeo()
  {}

  void GeeGroupGeo::activate(const TimeStamp &now)
  {
    GeePathToX::activate(now);

    GeeConfig * geeConfig = m_host->geeConfig(); 

    m_layer  = geeConfig->getValueInt("layer", -1);
  }

  void GeeGroupGeo::update(const TimeStamp &now)
  {
    const char * fname = "GeeGroupGeo::update";

    GeePathToX::update(now);

    GEE_World * w = & m_engine->world();

    GEE_SphereGroup sg;
    calculateSphereGroup(&sg);
        
    GEE_Geometry * g = 0;
    
    // Nearly select things we want to group:
    for(GEE_LayerGeometryIterator it(w, m_layer); 
	(g = *it); it.nextVisible()){
      
      GEE_id_t id = g->id();

      if(m_ids.find(id) != m_ids.end())
	continue;
      
      if(g->intersects( & sg)) {

	VEE_trace("%s # hit %p (%s)", fname, g, typeid(*g).name());
	
	if(!m_recording)
	  g->setNearlySelected(true);
	else {
	  m_ids.insert(id);
	  m_group->addChild(id);
	}
      }
    }
  }
  
  void GeeGroupGeo::stop(const TimeStamp &now)
  {
    GeePathToX::stop(now);

    if(!m_group)
      return;
    
    GEE_id_t gid = m_group->id();
    m_group->breakGroup();
    m_engine->world().eraseGeometry(gid);
    m_group = 0;

    if(m_ids.size() < 2)
      return;

    // Now we build the operator that does the work we want:
    
    updateOperatorGroup(now);

    GEE_GroupGeos *gg = new GEE_GroupGeos(gid);

    GEE_OperatorGroup *gr = m_operatorGroup.ptr();

    gg->interval().set(realTimeSinceBeginning());
    gg->setGroup(gr->id());

    for(std::set<GEE_id_t>::iterator it = m_ids.begin(); 
	it != m_ids.end(); it++)
      gg->addId(*it);

    m_engine->addOperator(gg);

  }
  
  void GeeGroupGeo::configureConversionAction()
  {
    m_group = new GEE_Group();
    m_engine->world().addGeometry(m_group);
    m_ids.clear();
  }

  /////////////////////////////////////////////////////////////////////////////
  /////////////////////////////////////////////////////////////////////////////

  
  GeeUnGroupGeo::GeeUnGroupGeo(Crystal::Engine *ce, GEE_Engine *ge)
    : GeePathToX(ce, ge),
      m_layer(-1)
  {
    m_useToolSphere = true;
    m_needPaths = false;
    m_useNapraRadius = true;
    m_toolName = "UnGroupGeo";
  }

  GeeUnGroupGeo::~GeeUnGroupGeo()
  {}

  void GeeUnGroupGeo::activate(const TimeStamp &now)
  {
    GeePathToX::activate(now);

    GeeConfig * geeConfig = m_host->geeConfig(); 

    m_layer  = geeConfig->getValueInt("layer", -1);
  }

  void GeeUnGroupGeo::update(const TimeStamp &now)
  {
    GeePathToX::update(now);
    
    GEE_World * w = & m_engine->world();
    
    GEE_SphereGroup sg;
    calculateSphereGroup(&sg);

    GEE_Geometry * g = 0;
    
    // Nearly select things we want to group:
    for(GEE_LayerGeometryIterator it(w, m_layer); 
	(g = *it); it.nextVisible()) {
      
      if(!dynamic_cast<GEE_Group *> (g))
	continue;
      
      if(g->intersects( & sg)) {
	
	if(m_recording) {
	  
	  updateOperatorGroup(now);

	  GEE_OperatorGroup *gr = m_operatorGroup.ptr();

	  GEE_UnGroupGeos * ungroup = new GEE_UnGroupGeos(g->id());

	  ungroup->autoId();
	  ungroup->setLayer(m_layer);
	  ungroup->interval().set(realTimeSinceBeginning());
	  ungroup->setGroup(gr->id());

	  m_engine->actions().addOperator(ungroup);
	}
	else
	  g->setNearlySelected(true);
      }
    }

  }

  void GeeUnGroupGeo::configureConversionAction()
  {}

}
