/* -*- C++ -*- */

/* -*- C++ -*- */

/* -*- C++ -*- */

/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of the Crystal animation and 3D modeling application
 *
 * Authors:   Tommi Ilmonen 2001-2004
 * Copyright: Tommi Ilmonen and/or Tekes 2001-2004
 *
 * firstname.lastname@hut.fi
 *
 */

#ifndef CRYSTAL_GEE_MESH_H
#define CRYSTAL_GEE_MESH_H

#include <crystal_gee_recorder.h>

#include <gee_config.h>
#include <gee_containment.h>
#include <gee_path.h>
#include <gee_world.h>

#include <vee_color.h>
#include <vee_reference_object.h>

#include <Fluid/Matrix3.hpp>
#include <Fluid/Napra.hpp>

#include <map>
#include <string>
#include <vector>

class GEE_Geometry;
class GEE_Group;
class GEE_Operator;
class GEE_ParticleData;
class GEE_ParticleObject;
class GEE_Path3D;
class GEE_PathToMesh;
class GEE_PathToParticles;
class GEE_Profile;
class GEE_Sphere;
class GEE_SphereGroup;

namespace Fluid {
  class Napra;
}

namespace Crystal { 

  class GeePathToX;

  class GeeRecorders
  {
  public:
    GeeRecorders();
    ~GeeRecorders();
    
    void init(Crystal::Engine *, GEE_Engine *);

    void setHand(Hand);

    GeePathToX * getRecorder(const std::string &);
    
  protected:

    typedef std::map<std::string, GeePathToX *> container;
    typedef container::iterator iterator;
    
    container m_recorders;
  };

  /////////////////////////////////////////////////////////////////////////////
  /////////////////////////////////////////////////////////////////////////////

  /** Takes motion and and creates the GEE_Actions that turn the
      motion data into a something else. */

  class GeePathToX : public GeeRecorder
  {
  public:
    GeePathToX(Crystal::Engine *, GEE_Engine *);
    virtual ~GeePathToX();

    virtual void activate(const TimeStamp &now);
    virtual void start(const TimeStamp &now);
    virtual void record(const TimeStamp &now);
    virtual void stop(const TimeStamp &now);
    
    virtual void update(const TimeStamp &now);

    virtual void render(GlData *); 
    
    GEE_Path3D   *pathA1() { return m_pathA1; }
    GEE_Path3D   *pathA2() { return m_pathA2; }
    GEE_Path3D   *pathB1() { return m_pathB1; }
    GEE_Path3D   *pathB2() { return m_pathB2; }
    GEE_Operator *pathToX() { return m_pathToX; }

    void setIgnoreNapra(bool ignore) { m_ignoreNapra = ignore; }

  protected:
    
    virtual void configureConversionAction() = 0;

    virtual void newSample();
    virtual void newNonrecordingSample();

    virtual void calculateToolSphereParams(Vector3 * loc, float * radius);
    void useToolSphere(float radius);

    void calculateSphereGroup(GEE_SphereGroup *);
    void updateOperatorGroup(const TimeStamp &now);
    void updateBeginTime();

    TimeStamp realTimeSinceBeginning();

    bool          m_ignoreNapra;
    Napra        *m_napra;
    NapraSample   m_napraSample;
    Matrix4       m_napramatrix;

    GEE_Path3D   *m_pathA1;
    GEE_Path3D   *m_pathA2;
    GEE_Path3D   *m_pathB1;
    GEE_Path3D   *m_pathB2;
    
    GEE_PathVertex3D m_pvA;
    GEE_PathVertex3D m_pvB;

    GEE_Operator *m_pathToX;
    long          m_trackerCount;
    GEE_id_t      m_pathAId;
    GEE_id_t      m_pathBId;
    bool          m_collectData;
    bool          m_needTwoPaths;
    bool          m_needPaths;
    GEE_id_t      m_layer;

    Matrix3     m_correctionRotation;

    Vector3     m_locationA;
    Matrix3     m_rotationA;

    Vector3     m_locationB;
    Matrix3     m_rotationB;

    GEE_Sphere *m_toolSphere;

    std::string m_toolName;
    bool        m_useToolSphere;
    VEE_Rgba    m_toolSphereColor;
    bool        m_highligthSphereArea;

    /// Set to false by this class, child classes can override it to true.
    bool  m_useNapraRadius;
    /** The following variables are initialized at each "activate"
	-call. */
    float m_radius;
    float m_radiusNorm;
    float m_radiusBoost;
  };

  /////////////////////////////////////////////////////////////////////////////
  /////////////////////////////////////////////////////////////////////////////

  class GeeNapraRender
  {
  public:
    GeeNapraRender(Crystal::Engine * host);
    ~GeeNapraRender();
    
    void setProfile(const GEE_Profile *);
    void render(GlData *, bool withSpheres = false);  
    void update(const GEE_PathVertex3D &, float, const Vector3 &);

    void buildGlTextures(GlData *);
    void calculateGlTextures();

    void setToolName(const std::string &tool) { m_tool = tool; }

    void update()
    {
      if(m_buildGlTextures > 0)
	m_buildGlTextures--;
    }

  protected:
    Crystal::Engine * m_host;
    GEE_Profile * m_profile;

    class Vertex
    {
    public:
      Vector3 m_location;
      bool    m_continues;
    };

    std::vector<Vertex> m_vertices;

    class Rgb 
    { 
    public:
      void copy(const float * rgb)
      {
	for(int i = 0; i < 3; i++) {
	  float tmp = rgb[i] * 255.5f;
	  m_rgb[i] = Math::Clamp((int) tmp, 0, 255);
	}
      }

      uchar * data() { return m_rgb; }
      const uchar * data() const { return m_rgb; }

      uchar m_rgb[3]; 
    };

    std::vector<Rgb> m_texture;

    int              m_buildGlTextures;

    // Normal against the napra plane
    Vector3             m_napraNormal;
    float               m_minDim;

    Matrix4             m_matrix;

    std::string m_texName;
    std::string m_tool;

  };

  /////////////////////////////////////////////////////////////////////////////
  /////////////////////////////////////////////////////////////////////////////

  class GeeMesh : public GeePathToX
  {
  public:
    GeeMesh(Crystal::Engine *, GEE_Engine *);
    virtual ~GeeMesh();

    virtual void activate(const TimeStamp &now);
    virtual void update(const TimeStamp &now);
    virtual void deactivate(const TimeStamp &now);

    virtual void render(GlData *);    

    GEE_PathToMesh * createPathToMesh(GEE_id_t pathId, 
				      int align,
				      bool useNapra);
    
  protected:
    virtual void newNonrecordingSample();
    virtual void newSample();

    virtual void configureConversionAction();

    GeeNapraRender m_napraRender;
    int m_finger;
  };

  /////////////////////////////////////////////////////////////////////////////
  /////////////////////////////////////////////////////////////////////////////

  class GeeLines : public GeePathToX
  {
  public:
    GeeLines(Crystal::Engine *, GEE_Engine *);
    virtual ~GeeLines();

    virtual void activate(const TimeStamp &now);
    virtual void render(GlData *);   

  protected:

    virtual void configureConversionAction();
    VEE_Rgba m_color;
  };

  /////////////////////////////////////////////////////////////////////////////
  /////////////////////////////////////////////////////////////////////////////

  class GeeParticles : public GeePathToX
  {
  public:
    GeeParticles(Crystal::Engine *, GEE_Engine *);
    virtual ~GeeParticles();

    virtual void activate(const TimeStamp &now);
    virtual void update(const TimeStamp &now);
    virtual void start(const TimeStamp &now);
    virtual void stop(const TimeStamp &now);

    GEE_PathToParticles * createPathToParticles(GEE_id_t pathId);
    
  protected:
    virtual void newNonrecordingSample();
    virtual void newSample();

    virtual void configureConversionAction();

    void configureParticle(GEE_ParticleData *);

    bool m_singleShots;

    GEE_ParticleData   * m_particle;
    GEE_ParticleObject * m_particleObj;

    int m_motionAlign; // Is VEE_MotionAlign
    float m_size;
  };

  /////////////////////////////////////////////////////////////////////////////
  /////////////////////////////////////////////////////////////////////////////

  /** Converts a single path to multiple paths. Used to get brush
      effects. */

  class GeeMultiBrush : public GeePathToX
  {
  public:
    GeeMultiBrush(Crystal::Engine *, GEE_Engine *);
    virtual ~GeeMultiBrush();

    virtual void activate(const TimeStamp &now);
    virtual void update(const TimeStamp &now);
    virtual void deactivate(const TimeStamp &now);

    virtual void render(GlData *);    

    virtual void stop(const TimeStamp &now);
  protected:

    virtual void configureConversionAction();

    virtual void newNonrecordingSample();
    virtual void newSample();

    std::vector<GEE_Operator *> m_conversions;

    GeeNapraRender m_napraRender;
    Vector3        m_extension;
  };

  /////////////////////////////////////////////////////////////////////////////
  /////////////////////////////////////////////////////////////////////////////

  class GeeMove : public GeePathToX
  {
  public:
    GeeMove(Crystal::Engine *, GEE_Engine *);
    virtual ~GeeMove();

    virtual void activate(const TimeStamp &now);
  protected:

    virtual void configureConversionAction();

  };

  /////////////////////////////////////////////////////////////////////////////
  /////////////////////////////////////////////////////////////////////////////

  class GeeColor : public GeePathToX
  {
  public:
    GeeColor(Crystal::Engine *, GEE_Engine *, bool vertexColors);
    virtual ~GeeColor();
    virtual void activate(const TimeStamp &now);

  protected:

    virtual void configureConversionAction();
    bool m_vertexColors;
  };

  /////////////////////////////////////////////////////////////////////////////
  /////////////////////////////////////////////////////////////////////////////

  class GeeRetexture : public GeePathToX
  {
  public:
    GeeRetexture(Crystal::Engine *, GEE_Engine *);
    virtual ~GeeRetexture();

  protected:

    virtual void configureConversionAction();

  };

  /////////////////////////////////////////////////////////////////////////////
  /////////////////////////////////////////////////////////////////////////////

  class GeeGroup : public GeePathToX
  {
  public:
    GeeGroup(Crystal::Engine *, GEE_Engine *);
    virtual ~GeeGroup();

    virtual void activate(const TimeStamp &now);

  protected:

    virtual void configureConversionAction();
  };

  /////////////////////////////////////////////////////////////////////////////
  /////////////////////////////////////////////////////////////////////////////

  class GeeErase : public GeePathToX
  {
  public:
    GeeErase(Crystal::Engine *, GEE_Engine *);
    virtual ~GeeErase();

  protected:

    virtual void configureConversionAction();
  };

  /////////////////////////////////////////////////////////////////////////////
  /////////////////////////////////////////////////////////////////////////////

  class GeeParticleRotate : public GeePathToX
  {
  public:
    GeeParticleRotate(Crystal::Engine *, GEE_Engine *);
    virtual ~GeeParticleRotate();

  protected:

    virtual void configureConversionAction();
  };

  /////////////////////////////////////////////////////////////////////////////
  /////////////////////////////////////////////////////////////////////////////

  class GeeEraseGeo : public GeePathToX
  {
  public:
    GeeEraseGeo(Crystal::Engine *, GEE_Engine *);
    virtual ~GeeEraseGeo();

    virtual void activate(const TimeStamp &now);

  protected:

    virtual void configureConversionAction();
    // virtual void newNonrecordingSample();

  };

  /////////////////////////////////////////////////////////////////////////////
  /////////////////////////////////////////////////////////////////////////////

  class GeeMoveGeo : public GeePathToX
  {
  public:
    GeeMoveGeo(Crystal::Engine *, GEE_Engine *, 
	       bool rotate, bool scale = false, bool uniformScale = true,
	       bool onlyTurn = false);
    virtual ~GeeMoveGeo();

    virtual void activate(const TimeStamp &now);
    virtual void start(const TimeStamp &now);
    virtual void stop(const TimeStamp &now);

  protected:

    virtual void configureConversionAction();

    virtual void newSample();
    virtual void newNonrecordingSample();

    bool  m_includeRotation;
    bool  m_scale;
    bool  m_uniformScale;
    bool  m_onlyTurn;
    bool  m_moveRecord;
  };

  /////////////////////////////////////////////////////////////////////////////
  /////////////////////////////////////////////////////////////////////////////

  class GeeInsertGeo : public GeePathToX
  {
  public:

    enum Source {
      GALLERY,
      QUAD
    };

    GeeInsertGeo(Crystal::Engine *, GEE_Engine *, Source s = GALLERY);
    virtual ~GeeInsertGeo();

    virtual void activate(const TimeStamp &now);
    virtual void update(const TimeStamp &now);
    virtual void stop(const TimeStamp &now);
    
  protected:

    virtual void configureConversionAction();

    void calculateTransformation();

    Vector3 m_modelCenter;

    Matrix4 m_matrix;

    VEE_RefPtr<GEE_World> m_model;
    GEE_id_t              m_modelId;
    Source                m_source;
  };

  /////////////////////////////////////////////////////////////////////////////
  /////////////////////////////////////////////////////////////////////////////

  class GeeSelectGeo : public GeePathToX
  {
  public:
    GeeSelectGeo(Crystal::Engine *, GEE_Engine *);
    virtual ~GeeSelectGeo();

    virtual void activate(const TimeStamp &now);
    virtual void update(const TimeStamp &now);
    
  protected:

    virtual void configureConversionAction();

    void select(bool reallySelect);

    GEE_id_t m_layer;

    std::set<GEE_id_t> m_ids;
  };

  /////////////////////////////////////////////////////////////////////////////
  /////////////////////////////////////////////////////////////////////////////

  /** Each stroke creates a group. */

  class GeeGroupGeo : public GeePathToX
  {
  public:
    GeeGroupGeo(Crystal::Engine *, GEE_Engine *);
    virtual ~GeeGroupGeo();

    virtual void activate(const TimeStamp &now);
    virtual void update(const TimeStamp &now);
    virtual void stop(const TimeStamp &now);
    
  protected:
    
    virtual void configureConversionAction();

    GEE_id_t m_layer;

    GEE_Group * m_group;

    std::set<GEE_id_t> m_ids;
  };

  /////////////////////////////////////////////////////////////////////////////
  /////////////////////////////////////////////////////////////////////////////

  /** Each stroke bkreaks all the groups it intersects. */
  class GeeUnGroupGeo : public GeePathToX
  {
  public:
    GeeUnGroupGeo(Crystal::Engine *, GEE_Engine *);
    virtual ~GeeUnGroupGeo();

    virtual void activate(const TimeStamp &now);
    virtual void update(const TimeStamp &now);
    
  protected:
    
    virtual void configureConversionAction();

    GEE_id_t m_layer;
  };

}

#endif
