/* -*- C++ -*- */

/* -*- C++ -*- */

/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of the Crystal animation and 3D modeling application
 *
 * Authors:   Tommi Ilmonen 2001-2004
 * Copyright: Tommi Ilmonen and/or Tekes 2001-2004
 *
 * firstname.lastname@hut.fi
 *
 */

#include <crystal_gee_recorder.h>

#include <crystal_engine.h>

#include <vee_error.h>

#include <Fluid/Message.hpp>
#include <Fluid/MessageMouse.hpp>
#include <Fluid/MessageSource.hpp>

#include <typeinfo>

namespace Crystal { 

  GeeRecorder::GeeRecorder(Crystal::Engine *host, 
			   GEE_Engine *engine)
    : m_host(host),
      m_engine(engine),
      m_recording(false),
      m_hand(RIGHT_HAND)
  {}

  void GeeRecorder::activate(const TimeStamp &now)
  {
    m_time = now;
    m_host->buttonMessages(m_hand)->msgAddSink("", "mouse", this);
  }

  void GeeRecorder::deactivate(const TimeStamp &now)
  {
    if(m_recording)
      stop(now);

    m_time = now;
    m_host->buttonMessages(m_hand)->msgRemoveSink(this);
  }

  GeeRecorder::~GeeRecorder()
  {}

  void GeeRecorder::start(const TimeStamp &)
  {
    m_recording = true;
  }

  void GeeRecorder::record(const TimeStamp &)
  {}

  void GeeRecorder::stop(const TimeStamp &)
  {
    m_recording = false;
  }
  
  void GeeRecorder::update(const TimeStamp &now)
  {
    m_time = now;

    if(m_recording)
      record(now);
  }

  bool GeeRecorder::msgProcess(const char *id, Fluid::Message *m)
  {
    const char * fname = "GeeRecorder::msgProcess";

    assert(id && m);

    bool ok = false;
    
    VEE_trace("%s # \"%s\" %s", fname, id, typeid(*m).name());

    const Fluid::MessageMouse *mm = 
      dynamic_cast<const Fluid::MessageMouse *> (m);
    
    if(mm) {
      double td = m_time.secondsD();

      if(mm->pressed(0)) {
	VEE_trace("%s # Start recording %lf", fname, td);
	start(m_time);
	// m_recording = true;
	ok = true;
      }
      else if(mm->released(0)) {
	VEE_trace("%s # Stop recording %lf", fname, td);
	stop(m_time);
	// m_recording = false;
	ok = true;
      }
    }    
    
    return ok;
  }

  void GeeRecorder::render(GlData *)
  {}


}
