/* -*- C++ -*- */

/* -*- C++ -*- */

/* -*- C++ -*- */

/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of the Crystal animation and 3D modeling application
 *
 * Authors:   Tommi Ilmonen 2001-2004
 * Copyright: Tommi Ilmonen and/or Tekes 2001-2004
 *
 * firstname.lastname@hut.fi
 *
 */

#ifndef CRYSTAL_GL_DATA_H
#define CRYSTAL_GL_DATA_H

#include <crystal.h>

#include <vee_gl_renderer.h>
#include <vee_gl_texture.h>

#include <gee_gl_renderer.h>
#include <gee_gl_material.h>
#include <gee_display_lists.h>

#include <Solar/GlDisplayLists.hpp>
#include <Solar/GlMaterial.hpp>
#include <Solar/GlRenderer.hpp>

#include <ConfigReader.hpp>

#include <Fluid/Vector4.hpp>

#include <plib/fnt.h>

#include <map>

#include <strings.h>

namespace Solar {
  class Model;
}

namespace Crystal { 

  class Engine;

  /** OpenGL data and stuff related to the engine. */
  class GlData
  {
  public:
    GlData();
    ~GlData();

    void contextInit(Engine *, ConfigReader::Config * config = 0);

    void lightsOn();

    static void applyGlTransform(const Fluid::Matrix3 &rot,
				 const Fluid::Vector3 &loc);

    Solar::Model * model(const std::string &id)
    {
      std::map<std::string, Solar::Model *>::iterator it = m_models->find(id);
    
      if(it == m_models->end())
	return 0;

      return (*it).second;
    }

    GEE_GlRenderer        m_geeRenderer;
    GEE_GlMaterialBank    m_geeMaterials;
    GEE_GlDisplayLists    m_geeLists;
    GEE_GlDisplayLists    m_geeHighlightLists;

    VEE_GlRenderer        m_veeRenderer;
    VEE_GlTexture2D       m_blobTex;
    VEE_GlTexture2D       m_triTex;
    VEE_GlTexture2D       m_polyTex;
    VEE_GlTexture2DBank   m_textures;

    Solar::GlRenderer     m_solarRenderer;
    Solar::GlMaterialBank m_materials;
    Solar::GlDisplayLists m_displayLists;
  
    fntRenderer           m_fntRenderer;
    fntTexFont            m_font;

    GLUquadric           *m_quadric;

    std::map<std::string, Solar::Model *> * m_models;

    class Light {
    public:
      Light();

      void apply(GLenum id);

      void set(ConfigReader::Chunk *values);

      Vector4 m_location;
      Vector4 m_ambient;
      Vector4 m_diffuse;
      Vector4 m_specular;
    };

    std::vector<Light> m_lights;

    void clearTimings() { bzero(m_timings, sizeof(m_timings)); }

    long m_timings[10];
    uint m_timingCount;

    static std::vector<GlData *> m_instances;
  };

}


#endif
