/* -*- C++ -*- */

/* -*- C++ -*- */

/* -*- C++ -*- */

/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of the Crystal animation and 3D modeling application
 *
 * Authors:   Tommi Ilmonen 2001-2004
 * Copyright: Tommi Ilmonen and/or Tekes 2001-2004
 *
 * firstname.lastname@hut.fi
 *
 */

#ifndef CRYSTAL_GRAPH_EDITOR_H
#define CRYSTAL_GRAPH_EDITOR_H

#include <crystal_widget.h>

// #include <Fluid/MessageSink.hpp>
#include <Fluid/Vector2.hpp>

namespace Crystal { 

  class ValueGraphBase;

  class GraphEditor : public TextureButton
  {
  public:

    class Graph
    {
    public:
      Graph(float *times = 0, 
	    float * values = 0, 
	    uint stride = 0, 
	    uint size = 0) 
	: m_times(times), 
	  m_values(values), 
	  m_stride(stride), 
	  m_size(size), 
	  m_limitMin(false),
	  m_limitMax(false),
	  m_color(VEE_Rgba::White)
      { m_low.clear(); m_high = m_low; }

      ~Graph();

      void renderGl(GlData *);

      void init();

      float renderScale() const 
      {
	float scale = m_max - m_min;
	
	if(scale > 1.0e-7)
	  scale = 1.0 / scale;
	else
	  scale = 1.0;

	return scale;
      }

      float span() const { return m_max - m_min; }

      void setPoint(uint i, float v);
      int findIndex(float time);

      bool inside(const Vector2 &loc)
      { return loc.x > m_low.x && loc.y > m_low.y &&
	  loc.x < m_high.x && loc.y < m_high.y; }

      void setLimits(float minimum, float maximum, 
		     bool limitMin, bool limitMax)
      {
	m_min = minimum;
	m_max = maximum;
	m_limitMin = limitMin;
	m_limitMax = limitMax;
      }

      float * m_times;
      float * m_values;
      uint    m_stride;
      uint    m_size;
      
      float m_min, m_max;
      bool m_limitMin, m_limitMax;

      std::string m_label;

      VEE_Rgba m_color;

      Vector2  m_low, m_high;
    };

    GraphEditor();
    virtual ~GraphEditor();
    
    // virtual bool msgProcess(const char *id, Fluid::Message *);

    virtual void handleEvent(const InEvent *);
    virtual void renderGl(GlData *, bool transform = true);
    virtual void update();

    void addGraph(const char * label, ValueGraphBase *graph);

    void setTimePerUnit(float scale) { m_timePerUnit = scale; }

    void clear() { m_graphs.clear(); }

  protected:
    Vector2 determineTextEntrySize(GlData *);

    int m_selected;
    int m_prevIndex;
    float m_prevValue;
    float m_timePerUnit;

    std::vector<Graph> m_graphs;
    typedef std::vector<Graph>::iterator iterator;
  };
}

#endif
