/* -*- C++ -*- */

/* -*- C++ -*- */

/* -*- C++ -*- */

/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of the Crystal animation and 3D modeling application
 *
 * Authors:   Tommi Ilmonen 2001-2004
 * Copyright: Tommi Ilmonen and/or Tekes 2001-2004
 *
 * firstname.lastname@hut.fi
 *
 */

#ifndef CRYSTAL_HELMA_EDITOR_H
#define CRYSTAL_HELMA_EDITOR_H

#include <crystal_editor.h>
#include <crystal_widget.h>
#include <Fluid/MessageSink.hpp>

#include <gee_particle_object.h>

#include <mj_array2d.h>

#include <set>

class GEE_CollectorRenderer;
class GEE_Engine;

namespace Crystal { 

  class Engine;
  class GeeConfig;
  class GeeRecorders;
  class GlData;
  class HelmaEditor;
  class HelmaTab;
  class Tools;

  class WidgetGroup : public Transformation,
		      public MessageSource
  {
  public:

    class Connection
    {
    public:

      QDomElement writeDom(QDomDocument &doc);
      bool readDom(QDomElement &);

      std::string m_sourceName, m_targetName, m_targetId;
    };

    class Object {
    public:
      typedef std::list<Connection> container;
      typedef container::iterator iterator;

      Object() : m_rows(1), m_cols(1) {}


      QDomElement writeConnections(QDomElement &, QDomDocument &doc);
      bool readConnections(QDomElement &);

      QDomElement writeSize(QDomElement &e, QDomDocument &doc);
      bool readSize(QDomElement &);

      void applyConnections(HelmaEditor *, HelmaTab *tab);

      std::list<Connection> m_connections;
      VEE_RefPtr<Widget>    m_widget;
      int                   m_rows;
      int                   m_cols;
    };
    
    typedef std::list<Object> container;
    typedef container::iterator iterator;

    WidgetGroup(Transformation * host = 0, HelmaTab * tab = 0);
    virtual ~WidgetGroup();

    QDomElement writeDom(QDomDocument &doc);
    bool readDom(QDomElement &);

    void calculateLayout(bool signal = true);

    void clear() { m_grid.resize(0, 0); m_objects.clear(); }
    void resizeGrid(uint rows, uint cols) { m_grid.resize(rows, cols); }

    void addObject(Object &o);

    void activate();
    void update() { m_widgets.update(); }
    void renderGl(GlData * gld);
    void beforeEvents();
    void handleEvent(const InEvent *e);

    const Vector3 & size() const { return m_size; }
    void setSize(const Vector3 & s) { m_size = s; }

    void calculatePreferredSize(uint widgets, float widgetAspectRatio,
				uint * rows, uint * cols);

    void calculateBoundingBox(VEE_BoundingBox *);

    Widgets *widgets() { return & m_widgets; }

    const std::string & id() const { return m_id; }

  protected:

    enum MoveStyle {
      STATIONARY,
      MOVE,
      STRETCH_X,
      STRETCH_Y
    };

    void checkMove(const InEvent *e);
    void doMove(const InEvent *e);

    Widgets    m_widgets;

    Vector3    m_prevMove;
    Vector3    m_size;
    float      m_border;
    float      m_widgetSpace;
    MoveStyle  m_move;
    
    MJ_Array2D<bool> m_grid;

    container  m_objects;
    HelmaTab * m_tab;
    int        m_trackHand;
    /// Identifier for grabbing the group for real-time use.
    std::string m_id;
  };

  /////////////////////////////////////////////////////////////////////////////
  /////////////////////////////////////////////////////////////////////////////

  class Demo : public MessageSink
  {
  public:
    Demo(Crystal::Engine *);
    virtual ~Demo();

    virtual bool msgProcess(const char *id, Fluid::Message *);
    void setTool(const std::string &tool)
    { m_recorderName = tool; updateDemo(); }

    void updateDemo();
    void retexture();
    void recolor();
    void render(GEE_Renderer *);
    void setGeometryTransformation(const Matrix4 &);

  protected:
    
    std::set<std::string> m_coolDemos;
    bool           m_coolDemo;
    std::string    m_recorderName;
    GeeRecorders * m_recorders;
    GeeConfig    * m_config;
    GEE_Engine   * m_engine;
    Crystal::Engine * m_cengine;
  };

  /////////////////////////////////////////////////////////////////////////////
  /////////////////////////////////////////////////////////////////////////////

  class HelmaTab : public MessageSink
  {
  public:

    typedef std::list<VEE_RefPtr<WidgetGroup> > container;
    typedef container::iterator iterator;

    HelmaTab(Transformation *, HelmaEditor *);
    virtual ~HelmaTab();
    
    virtual bool msgProcess(const char *id, Fluid::Message *);

    virtual QDomElement writeDom(QDomDocument &doc);
    virtual bool readDom(QDomElement &elem);

    const std::string & name() const { return m_name; }
    const std::string & tool() const { return m_tool; }

    HelmaEditor    * host() { return m_host; }

    virtual void activate();
    virtual void update();
    virtual void renderGl(GlData * gld);
    virtual void beforeEvents();
    virtual void handleEvent(const InEvent *e);

    virtual void addGroup(VEE_RefPtr<WidgetGroup> & gr) 
    { m_groups.push_back(gr); }

    Demo *demo() { return & m_demo; }

    WidgetGroup * widgetGroup(const std::string &id, bool alwaysGetOne = true);
    
  protected:

    std::string m_name;
    std::string m_tool;

    container m_groups;
    Transformation * m_transform;
    HelmaEditor    * m_host;

    Demo             m_demo;
  };

  /////////////////////////////////////////////////////////////////////////////
  /////////////////////////////////////////////////////////////////////////////

  /** A special kind of tab that offers access to the gallery. */

  class Gallery : public HelmaTab
  {
  public:
    Gallery(Transformation *, HelmaEditor *);
    virtual ~Gallery();
    
    virtual void activate();
    virtual void update();

    virtual bool msgProcess(const char *, Message *);

    virtual QDomElement writeDom(QDomDocument &doc);
    virtual bool readDom(QDomElement &elem);
    virtual void handleEvent(const InEvent *e);

    void takeSnapshot(bool);
    void rebuild();
    void recalculateTransforms();

    void merge(const char * file);
    
  protected:

    void erase(int index);
    void insert(int index);
    
    class Item {
    public:
      VEE_RefPtr<GEE_World> m_model;
      VEE_RefPtr<Widget>    m_cube;
      VEE_RefPtr<Menu>      m_menu;
      Matrix4               m_transform;
    };

    WidgetGroup       * m_group;

    std::vector<Item> m_items;

    int m_delete;
  };

  /////////////////////////////////////////////////////////////////////////////
  /////////////////////////////////////////////////////////////////////////////

  class HelmaEditor : public Editor,
		      public Fluid::MessageSink
  {
  public:

    typedef std::map<std::string, VEE_RefPtr<HelmaTab> > container;
    typedef container::iterator iterator;    

    typedef std::list<VEE_RefPtr<HelmaTab> > container2;
    typedef container2::iterator iterator2;    

    HelmaEditor(Tools *, const char * uiFile);
    virtual ~HelmaEditor();

    virtual bool msgProcess(const char *id, Fluid::Message *);

    virtual void start();
    virtual void render(GlData *);

    virtual void update();
    virtual void beforeEvents();
    virtual void handleEvent(const InEvent *e);
    
    virtual QDomElement writeDom(QDomDocument &doc);
    virtual bool readDom(QDomElement &elem);

    Tools *host() { return m_host; }

    bool selectTool(const char * toolName, int hand);

    HelmaTab * tab(const std::string &tool);

  protected:

    bool buildTabs();
    bool saveTabs(const char * filename);

    Tools      *m_host;

    QDomDocument m_uiSetup;
    
    container    m_tabs;
    container2   m_tabList;
    HelmaTab    *m_currentTab;
    std::string  m_currentTabName;

    std::string  m_currentTools[2];

    HelmaTab     m_tabArea;
    VEE_RefPtr<WidgetGroup>  m_tabButtons;
    Widgets      m_widgets;
  };

  /////////////////////////////////////////////////////////////////////////////
  /////////////////////////////////////////////////////////////////////////////

  /** Enable quick editing of some parameters while drawing. 

  @author Tommi Ilmonen
  */

  class HelmaQuickEdit : public Editor
  {
  public:
    HelmaQuickEdit(Tools * host);
    virtual ~HelmaQuickEdit();

    virtual void start();
    virtual void stop();

    virtual void render(GlData *);
    virtual void update();
    virtual void beforeEvents();
    virtual void handleEvent(const InEvent *e);

    virtual QDomElement writeDom(QDomDocument &doc);
    virtual bool readDom(QDomElement &elem);

    void init(HelmaEditor * editor,
	      int hand,
	      const std::string & tool,
	      const std::string & group = "quick");

  protected:
    Tools          * m_host;
    WidgetGroup    * m_widgets;
    Transformation * m_origTransform;

    Vector3          m_origLoc;
    Matrix3          m_origRot;
  };

}

#endif
