/* -*- C++ -*- */

/* -*- C++ -*- */

/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of the Crystal animation and 3D modeling application
 *
 * Authors:   Tommi Ilmonen 2001-2004
 * Copyright: Tommi Ilmonen and/or Tekes 2001-2004
 *
 * firstname.lastname@hut.fi
 *
 */

#include <crystal_log.h>

#include <qdatetime.h> 

#include <stdarg.h>
#include <stdio.h>

namespace Crystal { 
  
  static FILE * __file = 0;

  void Log::init(const char * file)
  {
    quit();

    QString file2;

    if(file)
      file2 = file;
    else {
      QDateTime dt = QDateTime::currentDateTime ();
      QDate date = dt.date();
      QTime time = dt.time();
      
      file2.sprintf("logs/crystal-%d-%.2d-%.2d-%.2d:%.2d.log", 
		    date.year(), date.month(), date.day(),
		    time.hour(), time.minute());
    }
    
    __file = fopen(file2.ascii(), "w");
  }

  void Log::quit()
  {
    if(__file) {
      fclose(__file);
      __file = 0;
    }
  }

  /** Each log file line begins with the current time that is
      described twice. 
      
      First comes a the timestamp since midnight in milliseconds. Then
      comes the time in hour:minute:second:millisecond format.

      After the time information comes the actual action description.
  */
  void Log::log(const char * format, ...)
  {
    if(!__file) 
      return;

    char buf[8196];
    va_list args;
    va_start(args, format);
    vsprintf(buf, format, args);

    QTime midnight(0, 0, 0, 0);

    QTime time = QTime::currentTime();
    fprintf(__file, "%d %.2d:%2.d:%.2d:%.3d %s\n", midnight.msecsTo(time),
	    time.hour(), time.minute(), time.second(), time.msec(),
	    buf);

    fflush(__file);
  }

}
