/* -*- C++ -*- */

/* -*- C++ -*- */

/* -*- C++ -*- */

/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of the Crystal animation and 3D modeling application
 *
 * Authors:   Tommi Ilmonen 2001-2004
 * Copyright: Tommi Ilmonen and/or Tekes 2001-2004
 *
 * firstname.lastname@hut.fi
 *
 */

#ifndef CRYSTAL_PARTICLE_EDITOR_H
#define CRYSTAL_PARTICLE_EDITOR_H

#include <crystal_editor.h>
#include <crystal_transformation.h>
#include <crystal_value_graph.h>
#include <crystal_widget.h>

#include <vee_bounding_box.h>
#include <vee_color.h>
#include <vee_looks_color_table.h>
#include <vee_image.h>
#include <vee_matrix3.h>
#include <vee_particle2.h>
#include <vee_reference_object.h>
#include <vee_value_curve.h>

#include <mj_array2d.h>

#include <Fluid/MessageSink.hpp>

#include <vector>
#include <map>

namespace Crystal { 

  class GlData;
  class GraphEditor;
  class Tools;
  class Transformation;

  class ParticleEditor : public Editor,
			 public Fluid::MessageSink,
			 public Transformation
  {
  public:
    class ParticleData
    {
    public:
      enum Graps {
	ALPHA,
	THETA,
	SIZE,
	GRAPHS
      };

      ParticleData() : m_button(0) {}
      ~ParticleData() {}

      void init();
      void transmitColors();
      void update();

      QDomElement writeDom(QDomDocument &doc);
      bool readDom(QDomElement &elem);

      TextureButton         * m_button;
      VEE_LooksColorTable     m_colors;
      VEE_ReferenceObject<VEE_Particle2>   m_particle;
      VEE_ValueCurve2         m_sizeCurve;

      std::vector<VEE_Hsva>   m_colorsHsva;

      std::map<std::string, ValueGraphFloat> m_graphs;
      typedef std::map<std::string, ValueGraphFloat>::iterator giterator;
    };

    enum ParticleParam {
      AGE_LIMIT,
      MASS,
      AREA,
      SLIDERS
    };

    ParticleEditor(Tools *);
    ~ParticleEditor();
    virtual bool msgProcess(const char *id, Fluid::Message *);

    void render(GlData *);

    void update();

    virtual void beforeEvents() { m_widgets.beforeEvents(); }
    void handleEvent(const InEvent *e) { m_widgets.handleEvent(e); }

    void selectTexture(int);
    void selectParticle(int);

    VEE_Object * createParticle(uint);

    static void createParticleSelectors(Widgets *,
					const Transformation &,
					Transformation *,
					Tools *,
					Fluid::MessageSink *,
					VEE_BoundingBox *,
					MJ_Array2D<ParticleData> *particles);
				 
    static void renderParticleSelectors(MJ_Array2D<ParticleData> *particles,
					GlData *);

    MJ_Array2D<ParticleData> & particles() { return m_particles; }

    virtual QDomElement writeDom(QDomDocument &doc);
    virtual bool readDom(QDomElement &elem);

  protected:

    void updateSliders();

    Tools *     m_host;

    VEE_BoundingBox     m_texBox;
    VEE_BoundingBox     m_colorBox;
    VEE_BoundingBox     m_particleBox;

    Widgets     m_widgets;

    int                 m_selectedTexture;
    int                 m_selectedParticle;
    int                 m_colorPoint;
    ColorCube * m_colorCube;

    Slider   *  m_sliders[SLIDERS];

    MJ_Array2D<Widget *>  m_textureButtons;
    std::vector<Widget *> m_colorPointButtons;

    MJ_Array2D<ParticleData>      m_particles;
    ParticleData                 *m_currentParticle;

    GraphEditor * m_graphs;
  };

}

#endif
