/* -*- C++ -*- */

/* -*- C++ -*- */

/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of the Crystal animation and 3D modeling application
 *
 * Authors:   Tommi Ilmonen 2001-2004
 * Copyright: Tommi Ilmonen and/or Tekes 2001-2004
 *
 * firstname.lastname@hut.fi
 *
 */

#include <crystal_scene_objs.h>

#include <vee_error.h>
#include <vee_grid_force.h>
#include <vee_math.h>

#include <Solar/Loaders.hpp>
#include <Solar/Model.hpp>
#include <Solar/Object.hpp>

#include <ConfigReader.hpp>

namespace Crystal { 

  SceneObjs::SceneObjs()
    : m_model(0),
      m_object(0),
      m_grid(0)
  {}

  SceneObjs::~SceneObjs()
  {}
  
  bool SceneObjs::loadScene(const char *name, 
			    ConfigReader::Chunk * config)
  {
    const char * fname = "SceneObjs::loadScene";

    if(!config)
      VEE_fatal("%s # NULL config chunk", fname);

    m_name = name;

    std::string modelFile = config->get("file").getString("default.lwo");
    std::string fieldFile = config->get("field").getString("");
    float scale           = config->get("scale").getDouble(1.0);
    std::string locationStr = config->get("location").getString("0 0 0");
    std::string rotationStr = config->get("rotation").getString("0 0 0");
  
    Solar::Model *m = new Solar::Model;

    bool ok = Solar::Loaders::load(m, modelFile.c_str());

    if(!ok) {
      VEE_error("%s # Could not load file \"%s\"", fname, modelFile.c_str());
      delete m;
      return false;
    }
  
    m->centralize();
    m->scale(scale);
  
    Fluid::Vector3 location;
    Fluid::Matrix3 rotation;
    float rx = 0, ry = 0, rz = 0;
    float degToRadScale = VEE_PI / 180.0;

    location.clear();
    sscanf(locationStr.c_str(), "%f %f %f", 
	   & location[0], & location[1], & location[2]);
  
    sscanf(rotationStr.c_str(), "%f %f %f", 
	   & rx, & ry, &rz);
  
    {
      Fluid::Matrix3 mx, my, mz;
      mx.rotateX(rx * degToRadScale);
      my.rotateY(ry * degToRadScale);
      mz.rotateZ(rz * degToRadScale);
      rotation = mx * my * mz;
    }
  
    Solar::Object *obj = new Solar::Object(m);

    obj->setLocation(location);
    obj->setRotation(rotation);

    m_model = m;
    m_object = obj;

    // Now load the force field that represents the scene:

    if(!fieldFile.length()) {
      fieldFile = modelFile + ".vfi";
    }
  
    VEE_GridForce * grid = new VEE_GridForce;
  
    ok = grid->grid().readGridFromFile(fieldFile.c_str());
  
    if(!ok) {
      VEE_error("%s # Could not load field file %s", fname, fieldFile.c_str());
      delete grid;
      return true;
    }
  
    grid->setLocation(location);
    grid->setRotation(rotation);
    grid->setValueFloat("scale", scale);
    grid->setValueFloat("magnitude", 400);
    grid->setValueFloat("bouncedistance", 0.5);
    grid->setValueFloat("bounce", 0.3);
    grid->setVisible(false);

    m_grid = grid;

    return true;
  }

}
