/* -*- C++ -*- */

/* -*- C++ -*- */

/* -*- C++ -*- */

/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of the Crystal animation and 3D modeling application
 *
 * Authors:   Tommi Ilmonen 2001-2004
 * Copyright: Tommi Ilmonen and/or Tekes 2001-2004
 *
 * firstname.lastname@hut.fi
 *
 */

#ifndef CRYSTAL_TOOLS_H
#define CRYSTAL_TOOLS_H

#include <crystal_transformation.h>
#include <crystal_ui_handler.h>
#include <crystal_widget.h>

#include <Fluid/MessageSink.hpp>
#include <Fluid/Mouse.hpp>
#include <Fluid/Processors.hpp>
#include <Fluid/Vector3.hpp>

#include <map>

#include <qdom.h>

namespace Crystal { 

  class ComponentEditor;
  class DensityEditor;
  class UiHandler;
  class Editor;
  class Engine;
  class ForceEditor;
  class GeneralEditor;
  class ParticleEditor;
  class EmitterEditor;
  class GeeEditor;
  class HelmaEditor;
  class HelmaQuickEdit;

  class Tools : public Fluid::MessageSink,
		public Fluid::MessageSource,
		public UiHandler,
		public Transformation
  {
  public:

    enum Tool {
      GENERAL_EDITOR,
      HELMA_EDITOR2,
      HELMA_QUICK_EDIT,
      PARTICLE_EDITOR,
      EMITTER_EDITOR,
      COMPONENT_EDITOR,
      DENSITY_EDITOR,
      FORCE_EDITOR,
      EDITORS
    };

    Tools(Engine * e);
    virtual ~Tools();

    virtual bool msgProcess(const char *id, Fluid::Message *);
  
    virtual void handleInput();
    virtual void update();
    virtual void render(GlData *glData);
    virtual void start();
    virtual void stop();

    void nextEditor();
    void toggleDensityRecord();
    void useQuickEditor(int hand);

    ParticleEditor * particleEditor()  { return m_particleEditor; }
    EmitterEditor  * emitterEditor()   { return m_emitterEditor; }
    ComponentEditor* componentEditor() { return m_componentEditor; }
    DensityEditor  * densityEditor()   { return m_densityEditor; }
    ForceEditor    * forceEditor()     { return m_forceEditor; }

    const ColorGroup & selectedColor() const { return m_selectedColor; }
    const ColorGroup & usualColor() const { return m_usualColor; }

    Engine         * engine() { return m_engine; }

    void selectEditor(Tool e);

    GrabGroup * grabs() { return & m_grabs; }

    SoundGroupRef & buttonSounds() { return m_buttonSounds; }

    UiHandler *  uiHandler() { return m_uiRecorder; }
    void         setUiHandler(UiHandler *r);
    
    Tool         activeEditor() const { return m_activeEditor; }
    bool         runningRecord() const { return m_runningRecord; }

    void setRenderingTransforms(bool use) { m_renderingTransforms = use; }

    QDomElement writeDom(QDomDocument &doc);
    bool readDom(QDomElement &);

  protected:

    Engine         * m_engine;

    ParticleEditor * m_particleEditor;
    EmitterEditor  * m_emitterEditor;
    GeneralEditor  * m_generalEditor;
    DensityEditor  * m_densityEditor;
    ComponentEditor* m_componentEditor;
    ForceEditor    * m_forceEditor;
    // GeeEditor      * m_geeEditor;
    HelmaEditor    * m_helmaEditor;
    HelmaQuickEdit * m_helmaQuickEdit;

    UiHandler      * m_uiRecorder;

    Editor         * m_editors2[EDITORS];

    std::map<std::string, Editor *> m_editors;
    typedef std::map<std::string, Editor *>::iterator iterator;

    /* The basic transformation in the EVE coordinates: */
    Fluid::Vector3 m_cursorLoc;
    Fluid::Vector3 m_cursorRot;

    IONodeBuffer<MouseSample> *m_mouse[2];
    long           m_mouseCount[2];
    int            m_pastState[2];
    long           m_trackerCount;

    Tool           m_activeEditor;
    Tool           m_oldEditor;
    bool           m_runningRecord;

    ColorGroup    m_selectedColor;
    ColorGroup    m_usualColor;
    SoundGroupRef m_buttonSounds;

    std::string         m_bgImage;
    Widgets             m_widgets;

    bool                m_moveTools;
    bool                m_renderingTransforms;

    // Processing location/rotation in world coordinates
    Fluid::Vector3      m_currentLoc[2];
    Fluid::Matrix3      m_currentRot[2];

    Fluid::Vector3      m_dragBeginLoc;
    Fluid::Vector3      m_dragBeginLocRel;
    Fluid::Matrix3      m_dragBeginRot;
    Fluid::Matrix3      m_dragBeginRot2;
    int                 m_dragHand;
    int                 m_handMask;

    GrabGroup   m_grabs;

    TextureButton * m_headLine;
  };

}

#endif
