/* -*- C++ -*- */

/* -*- C++ -*- */

/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of the Crystal animation and 3D modeling application
 *
 * Authors:   Tommi Ilmonen 2001-2004
 * Copyright: Tommi Ilmonen and/or Tekes 2001-2004
 *
 * firstname.lastname@hut.fi
 *
 */

#include <crystal_transformation.h>

namespace Crystal { 


  Transformation::Transformation(Transformation * host)
    : m_scale(1.0f),
      m_host(host)
  {
    m_rotation.identity();
    m_location.clear();
  }

  Transformation::~Transformation()
  {}

  /** Accumulates the transformation matrices to a matrix that tells the
      rotation in world coordinates. */

  Matrix3 Transformation::worldRotation() const
  {
    const Transformation * trans = this;

    Matrix3 m;
    m.identity();

    while(trans) {
      m = trans->m_rotation * m;
      trans = trans->m_host;
    }
  
    return m;
  }

  /** Accumulates the transformation matrices to a matrix that tells the
      rotation in world coordinates. */
  Matrix3 Transformation::worldRotation
  (const Matrix3 &r) const
  {
    const Transformation * trans = this;

    Matrix3 m(r);

    while(trans) {
      m = trans->m_rotation * m;
      trans = trans->m_host;
    }
  
    return m;
  }

  /** Accumulates the transformations and tells the location of this
      object in world coordinates. */
  Vector3 Transformation::worldLocation() const
  {
    const Transformation * trans = this;

    Vector3 loc;
    loc.clear();

    while(trans) {
      loc = trans->m_rotation * (loc * trans->m_scale) + trans->m_location;
      trans = trans->m_host;
    }
  
    return loc;
  }

  /** Accumulates the transformations and tells the location of this
      object in world coordinates. */
  Vector3 Transformation::worldLocation
  (const Vector3 &l) const
  {
    const Transformation * trans = this;

    Vector3 loc(l);

    while(trans) {

      loc = trans->m_rotation * (loc * trans->m_scale) + trans->m_location;
    
      trans = trans->m_host;
    }
  
    return loc;
  }

  Vector3 Transformation::applyMyTransform(const Vector3 &v) const
  {
    return ((v - m_location) / m_scale) * m_rotation;
  }

  Vector3 Transformation::unapplyMyTransform(const Vector3 &v) const
  {
    return m_rotation * (v * m_scale) + m_location;
  }

  Matrix3 Transformation::applyMyTransform(const Matrix3 &m) const
  {
    return m * m_rotation;
  }
  
  Matrix3 Transformation::unapplyMyTransform(const Matrix3 &m) const
  {
    return m_rotation * m;
  }

  /** Calculates the total scaling factor in world coordinates. */
  float Transformation::totalScale() const
  {
    float s = m_scale;
    const Transformation * trans = m_host;
    
    while(trans) {

      s *= trans->m_scale;
    
      trans = trans->m_host;
    }
  
    return s;
  }
  

}
