/* -*- C++ -*- */

/* -*- C++ -*- */

/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of the Crystal animation and 3D modeling application
 *
 * Authors:   Tommi Ilmonen 2001-2004
 * Copyright: Tommi Ilmonen and/or Tekes 2001-2004
 *
 * firstname.lastname@hut.fi
 *
 */

#include <crystal_value_action.h>

#include <crystal_action_list.h>

#include <vee_error.h>
#include <vee_object.h>
#include <vee_values.h>

namespace Crystal { 

  ValueAction::ValueAction(const ValueAction &a)
    : Action(a),
      m_objId(a.m_objId), 
      m_msgId(a.m_msgId),
      m_msg(a.m_msg ? a.m_msg->clone() : 0)
  {}

  /// Construct a float message.
  ValueAction::ValueAction(int objId, 
					   const std::string &msgId,
					   float v,
					   const Fluid::TimeStamp &ts)
    : Action(ts),
      m_objId(objId),
      m_msgId(msgId),
      m_msg(new VEE_ValueFloat(v))
  {}

  /// Construct an integer message.
  ValueAction::ValueAction(int objId, 
					   const std::string &msgId,
					   int v,
					   const Fluid::TimeStamp &ts)
    : Action(ts),
      m_objId(objId),
      m_msgId(msgId),
      m_msg(new VEE_ValueInt(v))
  {}


  /// Construct a vector message.
  ValueAction::ValueAction(int objId, 
					   const std::string &msgId,
					   const Fluid::Vector3 & v,
					   const Fluid::TimeStamp &ts)
    : Action(ts),
      m_objId(objId),
      m_msgId(msgId),
      m_msg(new VEE_ValueVector3(v))
  {
    // std::cout << "Vector value: " << v << std::endl;
  }

  /// Construct a matrix message.
  ValueAction::ValueAction(int objId, 
					   const std::string &msgId,
					   const Fluid::Matrix3 & m,
					   const Fluid::TimeStamp &ts)
    : Action(ts),
      m_objId(objId),
      m_msgId(msgId),
      m_msg(new VEE_ValueMatrix3(m))
  {}

  /// Deletes the message.
  ValueAction::~ValueAction()
  {
    delete m_msg;
  }

  /// Delivers the message to the object.
  void ValueAction::act(ActionList *al)
  {
    // VEE_trace("ValueAction::act # %p %d %s", this, m_objId, m_msgId.c_str());

    VEE_Object *o = al->getObject(m_objId);

    if(!o || !m_msg) {
      VEE_error("ValueAction::act # o = %p, m_msg = %p", o, m_msg);
      return;
    }

    o->setValue(m_msgId.c_str(), m_msg);
  }

  void ValueAction::setValue(VEE_Value *m)
  {
    delete m_msg;
    m_msg = m;
  }

  ValueAction & ValueAction::operator = 
  (const ValueAction &a)
  {
    m_objId = a.m_objId;
    m_msgId = a.m_msgId;
    m_msg   = a.m_msg ? a.m_msg->clone() : 0;

    return *this;
  }

}

