/* -*- C++ -*- */

/* -*- C++ -*- */

/* -*- C++ -*- */

/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of the Crystal animation and 3D modeling application
 *
 * Authors:   Tommi Ilmonen 2001-2004
 * Copyright: Tommi Ilmonen and/or Tekes 2001-2004
 *
 * firstname.lastname@hut.fi
 *
 */

#ifndef CRYSTAL_VALUE_GRAPH_H
#define CRYSTAL_VALUE_GRAPH_H

#include <crystal.h>

#include <vee_interpolators.h>
#include <vee_matrix3.h>
#include <vee_vector3.h>

#include <qdom.h>

class VEE_HasValues;

namespace Crystal { 

  class Model;

  class ValueGraphBase
  {
  public:
    ValueGraphBase() {}
    virtual ~ValueGraphBase();
    virtual bool deliverValue(VEE_HasValues *, const char *, double t) = 0;

    virtual QDomElement writeDom(QDomDocument &doc) = 0;
    virtual bool readDom(QDomElement &elem) = 0;

    virtual void moveTime(double seconds) = 0;

    virtual bool getParameters(uint index,
			       float ** times, float ** values,
			       uint * stride, uint * size, 
			       const char ** label,
			       float ** min, float ** max) = 0;
  };

  /////////////////////////////////////////////////////////////////////////////
  /////////////////////////////////////////////////////////////////////////////

  template <typename T>
  class ValueGraphT : public ValueGraphBase
  {
  public:
    ValueGraphT();
    virtual ~ValueGraphT();

    virtual bool deliverValue(VEE_HasValues *, const char *, double t);

    virtual QDomElement writeDom(QDomDocument &doc);
    virtual bool readDom(QDomElement &elem);
    virtual void moveTime(double seconds);

    virtual bool getParameters(uint index,
			       float ** times, float ** values,
			       uint * stride, uint * size, 
			       const char ** label,
			       float ** min, float ** max);

    const char * classType();

    void copy(const T * data, uint n);

    VEE_LinearInterpolation<T> m_graph;
    
    void setLimits(T minimum, T maximum, bool limitMin, bool limitMax)
    {
      m_min = minimum;
      m_max = maximum;
      m_limitMin = limitMin;
      m_limitMax = limitMax;
    }

    T m_min, m_max;
    bool m_limitMin, m_limitMax;
  };

  typedef ValueGraphT<float> ValueGraphFloat;
  typedef ValueGraphT<VEE_Vector3> ValueGraphVector3;
  typedef ValueGraphT<VEE_Matrix3> ValueGraphMatrix3;

}


#endif
