/* -*- C++ -*- */

/* -*- C++ -*- */

/* -*- C++ -*- */

/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of the Crystal animation and 3D modeling application
 *
 * Authors:   Tommi Ilmonen 2001-2004
 * Copyright: Tommi Ilmonen and/or Tekes 2001-2004
 *
 * firstname.lastname@hut.fi
 *
 */

#ifndef CRYSTAL_VALUE_GRAPH_TMPL_H
#define CRYSTAL_VALUE_GRAPH_TMPL_H

#include <crystal_value_graph.h>

#include <crystal.h>
#include <crystal_dom_util.h>

#include <vee_error.h>
#include <vee_object.h>

#include <typeinfo>

namespace Crystal { 

  template <typename T>
  ValueGraphT<T>::ValueGraphT()
    : m_limitMin(false),
      m_limitMax(false)
  {}

  template <typename T>
  ValueGraphT<T>::~ValueGraphT()
  {}

  template <typename T>
  bool ValueGraphT<T>::deliverValue(VEE_HasValues *, const char *, double)
  {
    return false;
  }

  template <typename T>
  const char * ValueGraphT<T>::classType()
  {
    return "UNDEFINED";
  }

  template <typename T>
  void ValueGraphT<T>::copy(const T * data, uint n)
  {
    m_graph.clear();

    for(uint i=0; i < n; i++)
      m_graph.appendPoint(*data++, (float) i);
  }

  template<>
  bool ValueGraphT<float>::deliverValue
  (VEE_HasValues *o, const char *id, double t)
  {
    if(!m_graph.size()) {
      VEE_trace("ValueGraphT<float>::deliverValue # Premature %s %s", 
		id, typeid(*o).name());
      return false;
    }

    float v = m_graph.getValue(t);
    bool ok = o->setValueFloat(id, v);
    if(!ok)
      VEE_error("ValueGraphT<float>::deliverValue # %s %f %lf", 
		id, v, t); 

    /* VEE_trace("ValueGraphT<float>::deliverValue # %s %f %s", 
       id, v, typeid(*o).name());*/
    return ok;
  }

  template<>
  bool ValueGraphT<VEE_Vector3>::deliverValue
  (VEE_HasValues *o, const char *id, double t)
  {
    if(!m_graph.size()) return false;

    VEE_Vector3 v = m_graph.getValue(t);
    /* VEE_trace("ValueGraphT<VEE_Vector3> # %s %f %f %f %lf",
       id, v.x, v.y, v.z, t); */
    return o->setValueVector3(id, v);
  }

  template<>
  bool ValueGraphT<VEE_Matrix3>::deliverValue
  (VEE_HasValues *o, const char *id, double t)
  {
    if(!m_graph.size()) return false;

    // Do not interpolate matrices = use "fast" method
    VEE_Matrix3 v = m_graph.getValueFast(t);
    return o->setValueMatrix3(id, v);
  } 


  template<>
  const char * ValueGraphT<float>::classType()
  {
    return "ValueGraphFloat";
  }

  template<>
  const char * ValueGraphT<VEE_Vector3>::classType()
  {
    return "ValueGraphVector3";
  }

  template<>
  const char * ValueGraphT<VEE_Matrix3>::classType()
  {
    return "ValueGraphMatrix3";
  }

  template<>
  bool ValueGraphT<float>::getParameters(uint index,
					 float ** times, float ** values,
					 uint * stride, uint * size, 
					 const char ** label,
					 float ** min, float ** max)
  {
    if(index >= 1)
      return false;

    VEE_InterpolationPoint<float> *p = & m_graph.getPoint(0);

    *times  = & p->m_time;
    *values = & p->m_value;
    *stride = sizeof(VEE_InterpolationPoint<float>);
    *size   = m_graph.size();
    *label  = "";

    *min = m_limitMin ? & m_min : 0;
    *max = m_limitMax ? & m_max : 0;

    return true;
  }

  template<>
  bool ValueGraphT<Vector3>::getParameters(uint index,
					   float ** times, float ** values,
					   uint * stride, uint * size, 
					   const char ** label,
					   float ** min, float ** max)
  {
    static const char * labels[3] = { "x", "y", "z" };

    if(index >= 3)
      return false;

    VEE_InterpolationPoint<Vector3> *p = & m_graph.getPoint(0);

    *times  = & p->m_time;
    *values = & p->m_value[index];
    *stride = sizeof(VEE_InterpolationPoint<Vector3>);
    *size   = m_graph.size();
    *label  = labels[index];

    *min = m_limitMin ? & m_min[index] : 0;
    *max = m_limitMax ? & m_max[index] : 0;

    return true;
  }

  template<>
  bool ValueGraphT<Matrix3>::getParameters(uint ,
					   float ** , float ** ,
					   uint *, uint *, 
					   const char **,
					   float ** , float **)
  {
    return false;
  }

  template <typename T>
  QDomElement ValueGraphT<T>::writeDom(QDomDocument &doc)
  {
    QDomElement elem = doc.createElement(classType());
  
    QString str;

    {
      QTextOStream s(&str);
      s.precision(10);
    
      s << m_graph.size() << " ";
    
      for(uint i=0; i < m_graph.size(); i++) {
	VEE_InterpolationPoint<T> p = m_graph.getPoint(i);
	s << p.m_time << " " << p.m_value << "  "; 
      }
    }

    addTextChild(elem, doc, "data", str);

    return elem;
  }

  template <typename T>
  bool ValueGraphT<T>::readDom(QDomElement &elem)
  {
    if(elem.tagName() != classType())
      return false;

    QString str = elem.namedItem("data").toElement().firstChild().toText().data();

    QTextIStream s(&str);

    uint n;

    s >> n;

    for(uint i=0; i < n; i++) {
      float t;
      T v;
      s >> t >> v;

      m_graph.appendPoint(v, t);
    }

    VEE_trace("%s::readDom # got %u values", typeid(*this).name(), n);

    return true;
  }

  template <typename T> 
  void ValueGraphT<T>::moveTime(double seconds)
  {
    m_graph.moveTime(seconds);
  }

}

#endif
