
/* -*- C++ -*- */

/* -*- C++ -*- */

/* -*- C++ -*- */

/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of the Crystal animation and 3D modeling application
 *
 * Authors:   Tommi Ilmonen 2001-2004
 * Copyright: Tommi Ilmonen and/or Tekes 2001-2004
 *
 * firstname.lastname@hut.fi
 *
 */

#include <crystal_value_graphs.h>

#include <crystal_dom_util.h>
#include <crystal_model.h>

#include <vee_error.h>

#include <qregexp.h> 

namespace Crystal { 

  ValueGraphs::ValueGraphs(TargetType type)
    : m_targetType(type)
  {}

  ValueGraphs::~ValueGraphs()
  {
    // deleteAll();
  }

  ValueGraphBase * ValueGraphs::create(const char * type)
  {
    if(!type) return 0;

    if(strcmp(type, "ValueGraphFloat") == 0)
      return new ValueGraphFloat;
    if(strcmp(type, "ValueGraphVector3") == 0)
      return new ValueGraphVector3;
    if(strcmp(type, "ValueGraphMatrix3") == 0)
      return new ValueGraphMatrix3;

    return 0;
  }

  void ValueGraphs::appendFloat(const std::string &s, double t, float v)
  {
    ValueGraphFloat * graph = 0;

    iterator it = m_graphs.find(s);
    if(it == m_graphs.end()) {
      graph = new ValueGraphFloat;
      m_graphs[s] = graph;
    }
    else {
      graph = dynamic_cast<ValueGraphFloat *> ((*it).second.ptr());
      if(!graph)
	VEE_fatal("ValueGraphs::appendFloat # %s", 
		  typeid(*(*it).second.ptr()).name());
    }
  
    graph->m_graph.appendPoint(v, t);
  }

  void ValueGraphs::appendVector3
  (const std::string &s, double t, const Fluid::Vector3 &v)
  {
    ValueGraphVector3 * graph = 0;

    /* VEE_trace("ValueGraphs::appendVector3 # %s %f %f %f,  %lf",
       s.c_str(), v.x, v.y, v.z, t); */

    iterator it = m_graphs.find(s);
    if(it == m_graphs.end()) {
      graph = new ValueGraphVector3;
      m_graphs[s] = graph;
    }
    else {
      graph = dynamic_cast<ValueGraphVector3 *> ((*it).second.ptr());
      if(!graph)
	VEE_fatal("ValueGraphs::appendVector3 # %s", 
		  typeid(*(*it).second.ptr()).name());
    }
  
    graph->m_graph.appendPoint(v, t);
  }

  void ValueGraphs::appendMatrix3
  (const std::string &s, double t, const Fluid::Matrix3 &v)
  {
    ValueGraphMatrix3 * graph = 0;

    iterator it = m_graphs.find(s);
    if(it == m_graphs.end()) {
      graph = new ValueGraphMatrix3;
      m_graphs[s] = graph;
    }
    else {
      graph = dynamic_cast<ValueGraphMatrix3 *> ((*it).second.ptr());
      if(!graph)
	VEE_fatal("ValueGraphs::appendMatrix3 # %s", 
		  typeid(*(*it).second.ptr()).name());
    }
  
    graph->m_graph.appendPoint(v, t);
  }

  void ValueGraphs::deleteAll()
  {
    m_graphs.clear();
  }

  void ValueGraphs::transmitValues(VEE_HasValues *o, double t)
  {
    for(iterator it = m_graphs.begin(); it != m_graphs.end(); it++) {
      /* VEE_trace("ValueGraphs::transmitValues \"%s\" %p %s -> %p", 
	 (*it).first.c_str(), (*it).second, typeid(*(*it).second).name(), o); */
      (*it).second.ptr()->deliverValue(o, (*it).first.c_str(), t);
    }
  }


  QDomElement ValueGraphs::writeDom(QDomDocument &doc)
  {
    QDomElement elem = doc.createElement("ValueGraphs");

    addTextChild(elem, doc, "targetType", (int) m_targetType);

    QRegExp re("/"); // Cannot use "/" in XML :-|, swap it to "_"

    for(iterator it = m_graphs.begin(); it != m_graphs.end(); it++) {
      QString tmp((*it).first.c_str());
      QDomElement e2 = doc.createElement(tmp.replace(re, "_"));
      e2.appendChild((*it).second.ptr()->writeDom(doc));
      elem.appendChild(e2);
    }
    return elem;
  }

  bool ValueGraphs::readDom(QDomElement &elem)
  {
    if(elem.tagName() != "ValueGraphs")
      return false;

    void deleteAll();

    m_targetType = (TargetType) getInt(elem, "targetType");

    QDomNode n = elem.firstChild();
    while ( !n.isNull() ) {
      if ( n.isElement() ) {
	QDomElement e = n.toElement();
	QString id = e.tagName();

	id = id.replace(QRegExp("_"), "/"); // Swap back from XML

	QDomElement e2 = e.firstChild().toElement();
	QString type = e2.tagName();

	ValueGraphBase * graph = create(type.ascii());
	if(graph) {
	  if(graph->readDom(e2)) {
	    m_graphs[id.ascii()] = graph;
	    VEE_trace("ValueGraphs::readDom # got graph \"%s\"",
		      id.ascii());
	  }
	  else
	    delete graph;
	}
      }
      n = n.nextSibling();
    }
  
    return true;
  }

  void ValueGraphs::moveTime(double offset)
  {
    for(iterator it = m_graphs.begin(); it != m_graphs.end(); it++)
      (*it).second.ptr()->moveTime(offset);
  }

  ValueGraphBase * ValueGraphs::findGraph(const std::string &id)
  {
    iterator it = m_graphs.find(id);
    if(it == m_graphs.end())
      return 0;

    return (*it).second.ptr();
  }

}
