/* -*- C++ -*- */

/* -*- C++ -*- */

/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of the Crystal animation and 3D modeling application
 *
 * Authors:   Tommi Ilmonen 2001-2004
 * Copyright: Tommi Ilmonen and/or Tekes 2001-2004
 *
 * firstname.lastname@hut.fi
 *
 */

#include <crystal_visualize_location.h>

#include <crystal_action_list.h>
#include <crystal_dom_util.h>
#include <crystal_engine.h>
#include <crystal_gl_data.h>
#include <crystal_value_graphs.h>

#include <vee_error.h>
#include <vee_math.h>

#include <Fluid/Mouse.hpp>
#include <Fluid/TrackerSensorSelector.hpp>

#include <GL/gl.h>
#include <GL/glu.h>

namespace Crystal { 

  VisualizeLocation::VisualizeLocation()
    : m_objectId(-1),
      m_color(VEE_Rgba::Blue),
      m_color2(VEE_Rgba::Cyan * 0.7),
      m_samples(0),
      m_mouseSamples(0),
      m_selection(-1),
      m_nextSelection(-1),
      m_onDrag(false)
  {
    m_prevLoc.clear();
    m_color2.setAlpha(1.0);
    m_color2.setTheta(1.0);
  }

  VisualizeLocation::~VisualizeLocation()
  {}
  
  void VisualizeLocation::handleInput()
  {
    if(!m_host) return;
    
    Mouse * mouse = m_host->engine()->mouse();
    TrackerSensorSelector2 *left = m_host->engine()->strongHandLoc();
    TrackerSensorSelector2 *right = m_host->engine()->weakHandLoc();
    
    ValueGraphs & vg = m_host->getGraph(m_objectId);

    ValueGraphVector3 *vgv = 
      dynamic_cast<ValueGraphVector3 *> (vg.findGraph(m_parameterName));

    if(!vgv)
      return;

    uint n = vgv->m_graph.size();
    VEE_InterpolationPoint<Vector3> * ip = & vgv->m_graph.getPoint(0);

    for(; m_mouseSamples <= (long) mouse->outputBuffer().getSampleCount(); 
	m_mouseSamples++) {

      MouseSample ms = mouse->outputBuffer().getIndex(m_mouseSamples);
      bool press = ms.buttonPressed(0);

      int selection = -1;

      Vector3 handLoc = left->locations().getIndex
	(left->locations().getSampleCount() - 1); 
	
      float closest = 1.0;

      // Find the best match:
      for(uint i=0; i < n; i++) {
	  
	float distance = (ip[i].m_value - handLoc).length();

	if(closest > distance) {
	  closest = distance;
	  selection = i;
	}
      }

      if(press && (!m_onDrag) && (selection >= 0)) { 
	m_onDrag = true;
	m_selection = selection;
	m_samples = (long) left->locations().getSampleCount() - 1;
	m_prevLoc = left->locations().getIndex(m_samples); 
      }
      else if(!press && m_onDrag) {
	m_onDrag = false;
	m_selection = -1;
      }
    }

    VEE_trace("VisualizeLocation::handleInput # %d %d", 
	      (int) m_onDrag, m_selection);
    m_radius = -1;

    for(; m_samples <= (long) left->locations().getSampleCount(); 
	m_samples++) {
      
      Vector3 handLoc = left->locations().getIndex(m_samples); 
      Vector3 hondLoc2 = right->locations().getIndex(m_samples); 

      float closest = 1.0;
      m_nextSelection = -1;

      // Find the best match:
      for(uint i=0; i < n; i++) {
	  
	float distance = (ip[i].m_value - handLoc).length();

	if(closest > distance) {
	  closest = distance;
	  m_nextSelection = i;
	}
      }

      // Now move the stuff:
      
      int radius = (int) ((handLoc - hondLoc2).length() * 10.0);
      m_radius = radius;
      // VEE_clamp(radius, 1, 50);
      
      Vector3 diff = handLoc - m_prevLoc;
	
      for(int j = VEE_max(m_selection - radius, 0);
	  (j < VEE_min((int) n - 1, m_selection + radius)) && m_onDrag; 
	  j++) {

	float d = j - m_selection;
	float weight = cos(d / radius * VEE_HALF_PI);
	weight *= weight;
	Vector3 move = diff * weight;
	ip[j].m_value += move;
      }
	
      m_prevLoc = handLoc;
    }
  }  
    
  void VisualizeLocation::update()
  {}

  void VisualizeLocation::render(GlData *gld)
  {
    if(m_host) visualize(m_host, gld);
  }

  void VisualizeLocation::start()
  {
    TrackerSensorSelector2 *left = m_host->engine()->strongHandLoc();
    
    m_samples = (long) left->locations().getSampleCount();

    m_prevLoc = left->locations().getIndex(m_samples);

    m_mouseSamples = (long) m_host->engine()->mouse()->outputBuffer().
      getSampleCount() + 1;

    m_selection = -1;
    m_nextSelection = -1;
    m_radius = -1;
    m_onDrag = false;
  }

  void VisualizeLocation::stop()
  {}
  
  void VisualizeLocation::visualize(ActionList *al, GlData *gld)
  {
    m_host = al;

    ValueGraphs & vg = al->getGraph(m_objectId);

    ValueGraphVector3 *vgv = 
      dynamic_cast<ValueGraphVector3 *> (vg.findGraph(m_parameterName));

    if(!vgv)
      return;

    glPushAttrib(~0);
    glDisable(GL_LIGHTING);
    glLineWidth(4);
    glDisable(GL_TEXTURE_2D);

    uint i, n = vgv->m_graph.size();

    if(!n) return;

    VEE_InterpolationPoint<VEE_Vector3> * ip = & vgv->m_graph.getPoint(0);

    int index = (m_selection >= 0) ? m_selection : m_nextSelection;
    uint low = n + 1;
    uint high = low;

    if(m_radius > 0 && index >= 0) {

      low = VEE_max(0, index - m_radius);
      high = VEE_min((int) n - 1, index + m_radius);

      glColor4fv(VEE_Rgba::Green.data());
      glBegin(GL_LINE_STRIP);
  
      for(i=low; i <= high; i++)
	glVertex3fv(ip[i].m_value.data());
      glEnd();
    }

    glColor4fv(m_color.data());
    glBegin(GL_LINES);
  
    for(i=1; i < n; i += 2) {
      if(i <= low || i > high) {
	glVertex3fv(ip[i-1].m_value.data());
	glVertex3fv(ip[i].m_value.data());
      }
    }
    glEnd();

    glColor4fv(m_color2.data());
    glBegin(GL_LINES);
  
    for(i=2; i < n; i += 2) {
      if(i <= low || i > high) {
	glVertex3fv(ip[i-1].m_value.data());
	glVertex3fv(ip[i].m_value.data());
      }
    }
  
    glEnd();


    // Render the location balls
    if(m_selection >= 0 || m_nextSelection >= 0) {

      Vector3 loc = ip[index].m_value;

      glPushMatrix();
      glTranslatef(loc.x, loc.y, loc.z);
      glRotatef(90.0, 1.0, 0.0, 0.0);
      
      gluQuadricDrawStyle(gld->m_quadric, GLU_FILL);
      glColor4fv(m_selection >= 0 ? 
		 VEE_Rgba::Green.data() : VEE_Rgba::Yellow.data());
      gluSphere(gld->m_quadric, 0.05, 6, 6);
      
      glPopMatrix();
    }    

    glPopAttrib();

    // VEE_trace("VisualizeLocation::visualize # %u points", n);
  }

  QDomElement VisualizeLocation::writeDom(QDomDocument &doc)
  {
    QDomElement elem = doc.createElement("VisualizeLocation");
  
    addTextChild(elem, doc, "object_id", m_objectId);
    addTextChild(elem, doc, "param_name", m_parameterName.c_str());
    addTextChild(elem, doc, "color", toStr(m_color));
  
    return elem;
  }

  bool VisualizeLocation::readDom(QDomElement &elem)
  {
    if(elem.tagName() != "VisualizeLocation")
      return false;

    m_objectId = getInt(elem, "object_id", -1);
    m_parameterName = getString(elem, "param_name").ascii();
    m_color = toRgba(getString(elem, "color"));
    return true;
  }

}

