/* -*- C++ -*- */

/* -*- C++ -*- */

/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of the Crystal animation and 3D modeling application
 *
 * Authors:   Tommi Ilmonen 2001-2004
 * Copyright: Tommi Ilmonen and/or Tekes 2001-2004
 *
 * firstname.lastname@hut.fi
 *
 */

#include <crystal_visualize_particles.h>

#include <crystal_action_creator.h>
#include <crystal_action_list.h>
#include <crystal_dom_util.h>
#include <crystal_engine.h>
#include <crystal_gl_data.h>

#include <vee_math.h>

#include <Fluid/Mouse.hpp>
#include <Fluid/TrackerSensorSelector.hpp>

#include <map>

namespace Crystal { 
  
  VisualizeParticles::VisualizeParticles()
    : m_samples(0),
      m_mouseSamples(0),
      m_radius(0.0),
      m_onDrag(false)
  {
    m_prevLoc.clear();
  }

  VisualizeParticles::~VisualizeParticles()
  {}

  void VisualizeParticles::handleInput()
  {
    // if(!m_host) return;
    
    Mouse * mouse = m_host->engine()->mouse();
    TrackerSensorSelector2 *left = m_host->engine()->strongHandLoc();
    TrackerSensorSelector2 *right = m_host->engine()->weakHandLoc();
    
    for(; m_samples <= (long) left->locations().getSampleCount(); 
	m_samples++) {
      
      Vector3 handLoc = left->locations().getIndex(m_samples); 
      Vector3 handLoc2 = right->locations().getIndex(m_samples); 
      
      m_radius = (handLoc - handLoc2).length();

      Vector3 diff = handLoc - m_prevLoc;

      TimeStamp ts = left->locations().getTimeStamp(m_samples);
      long mouseIndex = mouse->outputBuffer().timeStampToIndex(ts);
      MouseSample ms = mouse->getSample(mouseIndex);

      if(ms.buttonPressed(0))
	move(diff, m_prevLoc, m_radius);

      m_prevLoc = handLoc;
    }
  }

  void VisualizeParticles::update()
  {}

  void VisualizeParticles::render(GlData *gld)
  {
    if(m_host) visualize(m_host, gld);
  }
    
  void VisualizeParticles::start()
  {
    TrackerSensorSelector2 *left = m_host->engine()->strongHandLoc();
    
    m_samples = (long) left->locations().getSampleCount();

    m_prevLoc = left->locations().getIndex(m_samples);

    m_mouseSamples = (long) m_host->engine()->mouse()->outputBuffer().
      getSampleCount() + 1;
    
    m_radius = 0.0;
    m_onDrag = false;
  }

  void VisualizeParticles::stop()
  {}

  void VisualizeParticles::visualize(ActionList *al, GlData *gld)
  {
    m_host = al;

    typedef std::multimap<float, VEE_ParticleVisualInfo *> Store;

    Store store;

    VEE_GlRenderer * renderer = & gld->m_veeRenderer;

    Vector3 loc = renderer->camPos();

    for(ActionList::iterator it = al->begin(); it != al->end(); it++) {

      Action * a = (*it);
      ActionCreator * ac = dynamic_cast<ActionCreator *> (a);

      if(ac) {

	VEE_ParticleVisualInfo *pvi = & ac->visual();
	float distance = (loc - pvi->location()).length();

	store.insert
	  (std::pair<float, VEE_ParticleVisualInfo *>(distance, pvi));
      }
    }

    Store::iterator it = store.end();

    if(it != store.begin()) {

      glPushAttrib(~0);
      renderer->prepareGlMode();

      do {
	renderer->renderParticleVisual((*it).second);
	it--;
      } while(it != store.begin());

      renderer->endGlMode();
      glPopAttrib();
    }    
  }

  QDomElement VisualizeParticles::writeDom(QDomDocument &doc)
  {
    QDomElement elem = doc.createElement("VisualizeParticles");
    
    addTextChild(elem, doc, "foo", 1);

    return elem;
  }

  bool VisualizeParticles::readDom(QDomElement &elem)
  {
    if(elem.tagName() != "VisualizeParticles")
      return false;

    return true;
  }

  void VisualizeParticles::move
  (Vector3 direction, Vector3 center, float radius)
  {
    if(!m_host) return;

    for(ActionList::iterator it = m_host->begin(); it != m_host->end(); it++) {

      Action * a = (*it);
      ActionCreator * ac = dynamic_cast<ActionCreator *> (a);

      if(ac) {
	Vector3 loc = ac->visual().location();

	float dist = (loc - center).length();

	if(dist < radius) {
	  float w = cosf(VEE_HALF_PI * dist / radius) * 0.5;
	  loc += (direction * w);
	  ac->visual().setLocation(loc);
	}
      }      
    }
  }

}
