/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of the Crystal animation and 3D modeling application
 *
 * Authors:   Tommi Ilmonen 2001-2004
 * Copyright: Tommi Ilmonen and/or Tekes 2001-2004
 *
 * firstname.lastname@hut.fi
 *
 */

#include <crystal_qsettings.h>

#include <vee_error.h>

#include <mj_gui_numeric.h>
#include <mj_gui_int.h>
#include <mj_gui_string.h>

#include <qlabel.h>
#include <qlayout.h>

CRYSTAL_QSettings::CRYSTAL_QSettings(QWidget * parent,
				     const char * name)
  : QWidget(parent, name),
    m_frameName("Wind-Frame"),
    m_ribName("Wind"),
    m_framesPerFile(100),
    m_frameRate(25)
{
  QGridLayout *layout = new QGridLayout(this);

  int currentRow = 1;

  {
    // Create the frame
    MJ_GuiString * fileView = new MJ_GuiString(this);

    fileView->createObjs(this, "Frame file name", m_frameName);
    fileView->toGridHorizontal(layout, currentRow++, 0);
    
    connect(fileView, SIGNAL(textEmit(const QString &)), 
	    this, SLOT(setFrameFileName(const QString &)));
  }

  {
    // Create the RIB file name entry
    MJ_GuiString * fileView = new MJ_GuiString(this);

    fileView->createObjs(this, "RIB file name", m_ribName);
    fileView->toGridHorizontal(layout, currentRow++, 0);
    
    connect(fileView, SIGNAL(textEmit(const QString &)), 
	    this, SLOT(setRibFileName(const QString &)));
  }

  {
    // Create the entry to get the number of frames per RIB file
    MJ_GuiIntSlider *frames = new MJ_GuiIntSlider(this);

    frames->createObjs(this);
    frames->connectObjs();
    frames->set(m_framesPerFile, 1, 200);
    frames->limMinSizes();
    frames->label()->setText("Frames / RIB");
    
    connect(frames, SIGNAL(emitValue(int)), 
	    this, SLOT(setFramesPerFile(int)));
    frames->toGridHorizontal(layout, currentRow++, 0);
  }

  {
    // Create the entry to get the number of frames per RIB file
    MJ_GuiDoubleSlider *fps = new MJ_GuiDoubleSlider(this);

    fps->createObjs(this);
    fps->connectObjs();
    fps->set(m_frameRate, 5, 100);
    fps->limMinSizes();
    fps->label()->setText("Frames per second");
    
    connect(fps, SIGNAL(emitValue(double)), 
	    this, SLOT(setFrameRate(double)));
    fps->toGridHorizontal(layout, currentRow++, 0);
  }
}

CRYSTAL_QSettings::~CRYSTAL_QSettings()
{}

void CRYSTAL_QSettings::setFrameFileName(const QString &str)
{
  VEE_trace("CRYSTAL_QSettings::setFrameFileName # \"%s\"", str.ascii());

  m_frameName = str;
}

void CRYSTAL_QSettings::setRibFileName(const QString &str)
{
  VEE_trace("CRYSTAL_QSettings::setRibFileName # \"%s\"", str.ascii());

  m_ribName = str;
}

void CRYSTAL_QSettings::setFramesPerFile(int i)
{
  if(i > 10000) i = 10000;
  if(i < 1) i = 1;

  m_framesPerFile = i;
}

void CRYSTAL_QSettings::setFrameRate(double d)
{
  if(d > 100.0) d = 100.0;
  if(d < 5.0) d = 5.0;

  m_frameRate = d;
}
