/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of the Crystal animation and 3D modeling application
 *
 * Authors:   Tommi Ilmonen 2001-2004
 * Copyright: Tommi Ilmonen and/or Tekes 2001-2004
 *
 * firstname.lastname@hut.fi
 *
 */

#include <crystal_engine.h>
#include <crystal_qmain.h>
#include <crystal_tools.h>

#include <vee_error.h>
#include <vee_magic_rain.h>
#include <vee_scene.h>

#include <Fluid/Config.hpp>
#include <Fluid/InputDevices.hpp>

#include <di_sleep.h>

int main(int argc, char **argv)
{  
  QApplication a( argc, argv );

  std::string imagePath("./images");
  std::string textureName("Pyramid");
  const char * fluidConfig = "fluid-config";
  const char * animaConfig = "anima-config";
  const char * animationFile = 0;
  
  int processedArgs = 1;
  bool rain = false;
  bool onlyInit = false;
  int firstTools = -1;

  while(processedArgs < argc && argv[processedArgs][0] == '-') {
    const char *arg = argv[processedArgs];

    if(strcmp(arg, "--image_path") == 0 && processedArgs + 1 < argc) {
      imagePath   = argv[++processedArgs];
      processedArgs++;
    }
    else if(strcmp(arg, "--texnames") == 0 && processedArgs + 1 < argc) {
      textureName = argv[++processedArgs];
      processedArgs++;
    }
    else if(strcmp(arg, "--fluid") == 0 && processedArgs + 1 < argc) {
      fluidConfig = argv[++processedArgs];
      processedArgs++;
      VEE_trace("New FLUID config \"%s\"", fluidConfig);
    }
    else if(strcmp(arg, "--config") == 0 && processedArgs + 1 < argc) {
      animaConfig = argv[++processedArgs];
      processedArgs++;
      VEE_trace("New ANIMA config \"%s\"", animaConfig);
    }
    else if(strcmp(arg, "--file") == 0 && processedArgs + 1 < argc) {
      animationFile = argv[++processedArgs];
      processedArgs++;
      VEE_trace("New animation file \"%s\"", animationFile);
    }
    else if(strcmp(arg, "--rain") == 0) {
      rain = true;
      processedArgs++;      
    }
    else if(strcmp(arg, "--tools") == 0) {
      firstTools = atoi(argv[++processedArgs]);
      processedArgs++;      
    }
    else if(strcmp(arg, "--onlyinit") == 0) {
      onlyInit = true;
      processedArgs++;      
    }
    else {
      VEE_error("Unrecognized option \"%s\"", arg);
      return 0;
    }
  }
  
  Fluid::DeviceConfig::init(fluidConfig);
  /* Load more configurations. */

  Fluid::InputDevices::init();

  VEE_SceneBox *scene = new VEE_SceneBox(0, 
					 Fluid::Vector3(-50.0, -50.0, -50.0),
					 Fluid::Vector3(50.0, 50.0, 50.0));
  //int particles = 100000;
  


  if(rain) {
    VEE_MagicRain *rain = 
      new VEE_MagicRain(Fluid::Vector3(0.0, 10.1, -2.0), 
			Fluid::Vector3(0.0, 0.0, 0.0), 
			1000.0, 
			5.0,
			0.0,
			1.0);
    scene->addObject(rain);
  }

  VEE_trace("Creating CRYSTAL engine");

  Crystal::Engine engine(scene, animaConfig);

  engine.init();

  if(onlyInit)
    return 0;

  if(firstTools >= 0) 
    engine.tools()->selectEditor((Crystal::Tools::Tool) firstTools);

  VEE_Qt *q = new VEE_Qt(engine.scene(), VEE_GL_SL, 0, 0, false);

  CRYSTAL_QMain *q2 = new CRYSTAL_QMain(q, &engine, q);

  if(animaConfig) {
    VEE_trace("Reading AnimaLand config \"%s\"", animaConfig);
    q2->setConfigFile(animaConfig);
  }
  else {
    VEE_trace("No AnimaLand configuration specified");
  }

  q->setGlWidget(q2);

  int kp = q->addKeyPoint2(-10.144, 3.300, 1.597, 1.542, 0);
  kp = q->addKeyPoint2(1.392, 5.267, 7.695, 3.142, 0.000);
  q->selectKeyPoint2(kp);
  
  a.setMainWidget( q );

  Fluid::InputDevices::update();
  DI_Sleep::sleepMs(100);
  Fluid::InputDevices::update();

  if(animationFile)
    engine.readDomFile(animationFile);

  q->show();

  int r = a.exec();

  delete q;

  return r;
}
