/* COPYRIGHT
 *
 * This file is part of FLUID
 *
 * Author: Tommi Ilmonen, Janne Kontkanen 1999-2002.
 * Tommi.Ilmonen@hut.fi, Janne.Kontkanen@hut.fi
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. 
 *    
 */

#ifndef FLUID_DEVICE_FACTORY_HPP
#define FLUID_DEVICE_FACTORY_HPP

#include <string>

#include <Fluid/ConfigReaderEmpty.hpp>

namespace Fluid
{

class InputDeviceBase;

  /**
   * Device factory class
   *
   * This class is of intrest only for someone interested
   * in implementing new device drivers.
   *
   * Plugin-device drivers communicate with the base library
   * by handing DeviceFactory objects to InputDevices collection.
   *
   * This is all a device driver must do to subscribe into the system
   */

  class DeviceFactory
  {
  public:
    /**
     * Virtual destructor
     */
    virtual ~DeviceFactory();
    /**
     * Returns the driver name (actual class) of the device
     * that this factory instance is able to create.
     */
    virtual std::string getDriverName() const = 0;
    /**
     * Abstract factory method to create a input device
     *
     * Actual implementations of this function reside in 
     * device driver plugins
     */
    virtual InputDeviceBase* create(Chunk* chunk) = 0;
  };

} // namespace

#endif
