/* COPYRIGHT
 *
 * This file is part of FLUID
 *
 * Author: Tommi Ilmonen, Janne Kontkanen 1999-2002.
 * Tommi.Ilmonen@hut.fi, Janne.Kontkanen@hut.fi
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. 
 *    
 */

#include <Fluid/Directory.hpp>

#include <assert.h>
#include <dirent.h>

namespace Fluid
{

  /**
   * Construct a directory listing
   * 
   * Creating a Directory object immediately scans the
   * contents of the director. Also subdirectories and
   * special directories such as ".." "." are included.
   * 
   *  @param pathname = directory path
   */

Directory::Directory(const std::string& pathname)
  :namelist(0)
{
  n = scandir(pathname.c_str(),&namelist,0,0);
}

/**
 * Deallocates the directory list
 */
  
Directory::~Directory()
{
  if(namelist != 0)
    {
      while(n-- > 0)
	{
	  free(namelist[n]);
	}
      free(namelist);
    }
}

/**
 * Returns the number of files in the directory.
 * 
 * This includes subdirectories and special directories
 * such as ".." "."
 * 
 * @return integer
 */


int Directory::getFileCount() const
{
  assert(n >= 0);
  return n;
}

/**
 * Returns given filename in directory
 * 
 * @param i = integer index of file   
 * @return filename
 */

std::string Directory::getFileName(int i) const
{
  assert(i >= 0 && i < n);
  return namelist[i]->d_name;
}

} // namespace
