/* COPYRIGHT
 *
 * This file is part of FLUID
 *
 * Author: Tommi Ilmonen, 1999-2001.
 * Tommi.Ilmonen@hut.fi
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. 
 *    
 */

#include <Fluid/Error.hpp>

#include <stdarg.h>
#include <stdio.h>

namespace Fluid {


/// Print trace information to the user
extern void Trace(const char *msg, ...)
{
  char buf[8196];
  va_list args;
  va_start(args, msg);
  vsprintf(buf, msg, args);
  printf("FLUID TRACE # %s\n", buf);
  fflush(0);
  va_end(args);
}

/// Print error information to the user
extern void Error(const char *msg, ...)
{
  char buf[8196];
  va_list args;
  va_start(args, msg);
  vsprintf(buf, msg, args);
  fprintf(stderr, "FLUID ERROR # %s\n", buf);
  fflush(0);
  va_end(args);
}


/** Print fatal error information to the user. Once the information
    has been written the application will be killed with segmentation
    fault. */
extern void Fatal(const char *msg, ...)
{
  char buf[8196];
  va_list args;
  va_start(args, msg);
  vsprintf(buf, msg, args);
  fprintf(stderr, 
	  "FLUID FATAL ERROR # %s\nINTENTIONAL SEGMENTATION FAULT !!\n",
	  buf);

  fflush(0);

  int *badAddress = 0;
  *badAddress = 1; // Segmentation fault
}


} // namespace Fluid

