/* COPYRIGHT
 *
 * This file is part of FLUID
 *
 * Author: Tommi Ilmonen, 1999-2001.
 * Tommi.Ilmonen@hut.fi
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. 
 *    
 */
#ifndef FLUID_GLOVE_HPP
#define FLUID_GLOVE_HPP

#include <Fluid/InputDevice.hpp>

class DI_DataBuffer2;

namespace Fluid {

class GloveSample
{
public:
  void       clear();
  void       read(DI_DataBuffer2 &buffer);
  void       write(DI_DataBuffer2 &buffer) const;
  int        getMaxSerializedSize() const;

  float getFingerJoint(int finger, int joint) const
  { return fingerJoints[finger][joint]; }

  float getAbduction(int index) { return abductionAngles[index]; }
  
  float fingerJoints[5][3];
  float abductionAngles[4];
};

/**
 * Data glove API
 *
 * This class provides raw access to the input data from a
 * glove
 *
 */

class Glove : public InputDevice<GloveSample>
{
public:
  // typedef GloveSample Sample;

  /// finger enumeration
  enum Finger
    {
        FINGER_THUMB,  
        FINGER_INDEX, 
        FINGER_MIDDLE,  
        FINGER_RING,  
        FINGER_LITTLE,  
        FINGERS
    };
  
  /// finger joint enumeration
  enum FingerJoint
    {
        FINGERJOINT_BASE,  
        FINGERJOINT_SECOND,  
        FINGERJOINT_THIRD,  
        FINGERJOINTS 
    };

  /// enumeration for abduction angle between fingers
  enum AbductionAngle
    {
      ABDUCTION_THUMB_INDEX,
      ABDUCTION_INDEX_MIDDLE,
      ABDUCTION_MIDDLE_RING,
      ABDUCTION_RING_LITTLE,
      ABDUCTION_ANGLES
    };

  Glove(Chunk* chunk, DataSource<GloveSample>* source) 
    : InputDevice<GloveSample>(chunk, source) {}

  float         getJointAngle(Finger finger, FingerJoint joint);
  float         getJointAngle(Finger finger, FingerJoint joint, ulong n);
  float         getAbductionAngle(AbductionAngle s);
  float         getAbductionAngle(AbductionAngle s, ulong n);
  std::string   getClassName() const;
};

} // namespace

#endif
