/* COPYRIGHT
 *
 * This file is part of FLUID
 *
 * Author: Tommi Ilmonen, Janne Kontkanen 1999-2002.
 * Tommi.Ilmonen@hut.fi, Janne.Kontkanen@hut.fi
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. 
 *    
 */

#include "Fluid/InputDevice.hpp"
/** @file */

/** @fn  Fluid::InputDevice::InputDevice(DataSource<Sample> *dataSource)
 * Constructs an InputDevice
 *
 * Binds the input device into given datasource. To start reading input
 * data from the device, start() must be called (internally this spawns 
 * the reader thread for the data source)
 *
 *  @param dataSource = pointer to DataSource   
 */

/** @fn Fluid::InputDevice::getLatestIndex() const
 * Returns the number of latest sample available for query
 *
 * @note if update() is not called frequently enough, the
 *       index returned by this function does no longer
 *       reflect the real index of the sample read from
 *       the device
 *
 * @returns an integer
 */

/** @fn Fluid::InputDevice::getLatestTimeStamp() const
 * Returns the timestamp of the latest sample.
 *
 * Latest sample is means the sample that is available
 * for reading by query functions of the device. Calling
 * update() might bring fresh data available and thus 
 * change the value this function returns
 * 
 * @returns timestamp of type long long
 * @see TimeStamp
 */

/** @fn Fluid::InputDevice::requireHistory(unsigned int n)
 * Requests the input device to keep track of history
 *
 * This function causes given amount of history samples
 * to be available at any time. Calling this function can
 * cause the device driver to enlarge it's buffers, but
 * buffers are never diminished. 
 *
 * So, after calling requireHistory() it's safe to assume
 * that desired amount of history is available even if
 * the function is called with smaller parameters in between.
 *
 * @param nsamples = required number of history samples
 *
 */

/** @fn Fluid::InputDevice::update()
 * Takes a local snapshot from internal device buffer
 * 
 * Data access functions of this class operate on date
 * that is read into local buffer. Thus nothing is got
 * unless this function is called first and furthermore
 * this function should be called frequently if data
 * should not be lost.
 *
 * Both, latestIndex and the timeStamp will change if
 * new samples were copied from the data source.
 *
 */

/** @fn Fluid::InputDevice<Sample>::start()
 * Starts the sampling
 *
 * This function must be called before any
 * data is queried from the device. Starting
 * and stopping is not done automatically by
 * the library because sampling consumes CPU
 * power and memory. Thus sampling unused 
 * data would waste of resources.
 * 
 * Note that this function waits until data is
 * available from the DataSource. Maximum time
 * to wait is three seconds, after that the 
 * function just returns. If data became available
 * update() function is called once.
 */

/** @fn Fluid::InputDevice<Sample>::getSample(int n)
 *
 * @param n = index of sample
 *
 * Fetches the given sample from input
 * 
 * Samples are indexed beginning from zero to the index
 * of latest sample (getLatestIndex()). Zero denotes the
 * first sample ever read from source.
 *
 * The samples that really are available range from
 * [getLatestIndex()-requiredHistory,getLatestIndex]
 */

/** @fn Fluid::InputDevice<Sample>::getLatestSample()
 *
 * Fetches the latest available sample 
 * 
 * This function is shorthand to getSample(getLatestIndex())
 */

/**
 * @fn Fluid::InputDevice<Sample>::getDelayedSample(int delay)
 *
 * @param delay = the age of sample
 *
 * Fetches a history sample. Given delay means the age of sample.
 * Index of zero denotes the current sample. This function
 * equals to: 
 * 
 * getSample(getLatestIndex()-delay)
 * 
 */
