/* COPYRIGHT
 *
 * This file is part of FLUID
 *
 * Author: Tommi Ilmonen, Janne Kontkanen 1999-2002.
 * Tommi.Ilmonen@hut.fi, Janne.Kontkanen@hut.fi
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. 
 *    
 */

#ifndef FLUID_INPUTDEVICEBASE_HPP
#define FLUID_INPUTDEVICEBASE_HPP

#include <string>

#include <Fluid/IONode.hpp>

namespace Fluid {

  /**
   * Base class for all the InputDevices. This class
   * defines abstract interface common to all the 
   * devices, independent of sample type.
   */

  class InputDeviceBase : public IOSource
{
public:
  /// Destroys the device
  virtual             ~InputDeviceBase() {}
  /// Makes fresh data available
  virtual void        update() = 0;
  /// Starts reading the input
  virtual void        start() = 0;
  /// Stops reading the input
  virtual void        stop() = 0;
  /// Returns true if the device has been started
  virtual bool        isRunning() const = 0;
  /// Possibly resizes the history buffer
  virtual void        requireHistory(unsigned int n) = 0;
  /// Returns the name of the instance
  virtual std::string getName() const = 0;
  /// Returns the class name of the InputDevice 
  virtual std::string getClassName() const = 0;
  /// Returns the driver name of the DataSource corresponding to this device
  virtual std::string getDriverName() const = 0;
  /// Returns sample rate in hertz
  virtual float       getSampleRate() const = 0;
  /// Sets the file for dumping the input data
  virtual void        setDumpFile(const std::string& fname) = 0;
};

} // namespace

#endif // INPUTDEVICEBASE_HPP
