/* COPYRIGHT
 *
 * This file is part of FLUID
 *
 * Author: Tommi Ilmonen, Janne Kontkanen 1999-2002.
 * Tommi.Ilmonen@hut.fi, Janne.Kontkanen@hut.fi
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. 
 *    
 */

#ifndef FLUID_INPUT_DEVICES_HPP
#define FLUID_INPUT_DEVICES_HPP

#include <Fluid/InputDevice.hpp>

#include <di_condition.h>

#include <map>
#include <string>

namespace Fluid {

class DeviceFactory;
class Glove;
class Mouse;
class Napra;
class Tracker;

  /**
   * This class provides central collection of InputDevice objects
   *
   * All of the methods of the class are static and internally
   * the class is implemented as singleton object, which is
   * created by init() function call and deleted by cleanup()
   * function.
   *
   */

class InputDevices
{
public:

  static void             init();
  static void             cleanup();
  static InputDeviceBase* getInputDevice(const std::string& name);
  static Glove*           getGlove(const std::string& name);
  static Mouse*           getMouse(const std::string& name);
  static Napra*           getNapra(const std::string& name);
  static Tracker*         getTracker(const std::string& name); 
  static void             addDevices(const std::string& driver, DeviceFactory *factory);
  static void             start(const TimeStamp *ts = 0);
  static void             stop();
  static void             update(const TimeStamp *ts = 0);

  static void             threadWaitUntil(const TimeStamp &);

  static TimeStamp        latestTimeStamp() 
  { return instance ? instance->m_latestStamp : 0; }

  static TimeStamp        timeShift() 
  { return instance ? instance->m_timeShift : 0; }

  static void             setTimeShift(const TimeStamp & ts)
  { if(instance) instance->m_timeShift = ts; }

private:
  static void ensure() { if(!instance) instance = new InputDevices; }
  InputDevices();
  ~InputDevices() {}
  static InputDevices *instance;
  std::map<std::string,InputDeviceBase*> devices;

  TimeStamp    m_latestStamp;
  TimeStamp    m_timeShift;
  DI_MutexAuto m_mutex;
  DI_Condition m_condition;
};

}

#endif
