/* COPYRIGHT
 *
 * This file is part of FLUID
 *
 * Author: Tommi Ilmonen, 1999-2001.
 * Tommi.Ilmonen@hut.fi
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. 
 *    
 */

#include <Fluid/Keyboard.hpp>

#include <ctype.h>

#include <mj_ringbuffertmpl.h>

namespace Fluid {


  char KeyToChar(Key k, int modifiers)
  {
    char res = 0;

    int keymap[] = {
      KEY_A, 'a',
      KEY_B, 'b',
      KEY_C, 'c',
      KEY_D, 'd',
      KEY_E, 'e',
      KEY_F, 'f',
      KEY_G, 'g',
      KEY_H, 'h',
      KEY_I, 'i',
      KEY_J, 'j',
      KEY_K, 'k',
      KEY_L, 'l',
      KEY_M, 'm',
      KEY_N, 'n',
      KEY_O, 'o',
      KEY_P, 'p',
      KEY_Q, 'q',
      KEY_R, 'r',
      KEY_S, 's',
      KEY_T, 't',
      KEY_U, 'u',
      KEY_V, 'v',
      KEY_W, 'w',
      KEY_X, 'x',
      KEY_Y, 'y',
      KEY_Z, 'z',

      KEY_0, '0',
      KEY_1, '1',
      KEY_2, '2',
      KEY_3, '3',
      KEY_4, '4',
      KEY_5, '5',
      KEY_6, '6',
      KEY_7, '7',
      KEY_8, '8',
      KEY_9, '9',

      KEY_SPACE, ' ',
      KEY_TAB,   '\t',
      KEY_ENTER, '\n',

      KEY_LEFT_PAREN, '(',
      KEY_RIGHT_PAREN, ')',
      KEY_LEFT_BRACE, '{',
      KEY_RIGHT_BRACE, '}',
      KEY_LEFT_BRACKET, '[',
      KEY_RIGHT_BRACKET, ']',

      KEY_LESS, '<',
      KEY_EQUAL, '=',
      KEY_GREATER, '>',
      KEY_PLUS, '+',
      KEY_MINUS, '-',
      
      KEY_ASTERISK,   '*',
      KEY_SLASH,      '/',
      KEY_BACKSLASH,  '\\',
      KEY_QUESTION,   '?',
      KEY_EXCLAM,     '!',
      KEY_COMMA,      ',',
      KEY_PERIOD,     '.',
      KEY_COLON,      ':',
      KEY_SEMICOLON,  ';',
      KEY_UNDERSCORE, '_',
      KEY_QUOTE,      '\"',
      KEY_DOLLAR,     '$',
      KEY_AT,         '@',
      KEY_TILDE,      '~',
      KEY_PERCENT,    '%',
      KEY_AMPERSAND,  '&',
      KEY_APOSTROPHE, '\'',
      KEY_NUMBERSIGN, '#',
      KEY_CIRCUM,     '^',
      KEY_BAR,        '|',
      -1, -1
    };

    for(int i=0; keymap[i] > 0 && res == 0; i += 2)
      if(keymap[i] == k)
	res = keymap[i + 1];


    if((modifiers & MOD_SHIFT) && !(modifiers & MOD_CAPS_LOCK))
      res = toupper(res);
    if(!(modifiers & MOD_SHIFT) && (modifiers & MOD_CAPS_LOCK))
      res = toupper(res);
    
    return res;
  }
  

  void KeyboardSample::read(DI_DataBuffer2 &buffer)
  {
    time = buffer.readInt64();
    key  = (Key) buffer.readInt32();
    modifiers =  buffer.readInt32();
  }
  
  void KeyboardSample::write(DI_DataBuffer2 &buffer) const
  {
    buffer.writeInt64(time.stamp());
    buffer.writeInt32(key);
    buffer.writeInt32(modifiers);
  }

  Keyboard::~Keyboard() 
  {}
  
  std::string Keyboard::getClassName() const
  {
    return "Keyboard";
  }

}
template class MJ_RingBuffer<Fluid::KeyboardSample>;

