/* COPYRIGHT
 *
 * This file is part of FLUID
 *
 * Author: Tommi Ilmonen, Janne Kontkanen 1999-2002.
 * Tommi.Ilmonen@hut.fi, Janne.Kontkanen@hut.fi
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. 
 *    
 */

#ifndef FLUID_MATH_HPP
#define FLUID_MATH_HPP

#include <math.h>

namespace Fluid {

  namespace Math
  {
    // Constants:

    const double PI         = 3.1415926535897931;
    const double TWO_PI     = 6.2831853071795862;
    const double HALF_PI    = 1.57079632679489660;
    const double QUARTER_PI = 0.78539816339744828;

    const double SQRT2      = 1.41421356237309514547;
    const double SQRT2_PER2 = 0.70710678118654757273;

    // float & double inlines:

    inline float Cos(float v)  { return cosf(v); }
    inline float Sin(float v)  { return sinf(v); }  
    inline float Tan(float v)  { return tanf(v); }  
    inline float Sqrt(float v) { return sqrtf(v); }
    inline float Exp(float v)  { return expf(v); }
    inline float Log(float v)  { return logf(v); }
    inline float Pow(float x, float y)    { return powf(x, y); }

    inline float ACos(float v)  { return acosf(v); }
    inline float ASin(float v)  { return asinf(v); }
    inline float ATan(float v)  { return atanf(v); }
    inline float ATan2(float x, float y)  { return atan2f(x, y); }
  
    inline double Cos(double v)  { return cos(v); }
    inline double Sin(double v)  { return sin(v); } 
    inline double Tan(double v)  { return tan(v); }  
    inline double Sqrt(double v) { return sqrt(v); }
    inline double Exp(double v)  { return exp(v); }
    inline double Log(double v)  { return log(v); }
    inline double Pow(double x, double y)    { return pow(x, y); }

    inline double ACos(double v)  { return acos(v); }
    inline double ASin(double v)  { return asin(v); }
    inline double ATan(double v)  { return atan(v); }
    inline double ATan2(double x, double y)  { return atan2(x, y); }

    // Min & Max inlines:
  
    inline short Max(short x, short y) { return x > y ? x : y; }
    inline short Min(short x, short y) { return x < y ? x : y; }
    inline short Abs(short x) { return x > 0 ? x : -x; }
  
    inline int Max(int x, int y) { return x > y ? x : y; }
    inline int Min(int x, int y) { return x < y ? x : y; }
    inline int Abs(int x) { return x > 0 ? x : -x; }
  
    inline unsigned Max(unsigned x, unsigned y) { return x > y ? x : y; }
    inline unsigned Min(unsigned x, unsigned y) { return x < y ? x : y; }

    inline long Max(long x, long y) { return x > y ? x : y; }
    inline long Min(long x, long y) { return x < y ? x : y; }
    inline long Abs(long x) { return x > 0 ? x : -x; }

    inline unsigned long Max(unsigned long x, unsigned long y) { return x > y ? x : y; }
    inline unsigned long Min(unsigned long x, unsigned long y) { return x < y ? x : y; }

    inline float Max(float x, float y) { return x > y ? x : y; }
    inline float Min(float x, float y) { return x < y ? x : y; }
    inline float Abs(float x) { return x > 0.0f ? x : -x; }

    inline double Max(double x, double y) { return x > y ? x : y; }
    inline double Min(double x, double y) { return x < y ? x : y; }
    inline double Abs(double x) { return x > 0.0 ? x : -x; }

    template <class T>
    inline T Clamp(T x, T low, T high) 
    { 
      if(x < low) return low;
      if(x > high) return high;
      return x;
    }

  }

}

#endif
