/* COPYRIGHT
 *
 * This file is part of FLUID
 *
 * Author: Tommi Ilmonen, Janne Kontkanen 1999-2002.
 * Tommi.Ilmonen@hut.fi, Janne.Kontkanen@hut.fi
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. 
 *    
 */

#include <Fluid/Mouse.hpp>

#include <Fluid/Error.hpp>
#include <Fluid/TimeStamp.hpp>
#include <di_data_buffer.h>

#include <mj_ringbuffertmpl.h>

#include <iostream>

namespace Fluid {

void MouseSample::clear()
{
  state = 0;
  x = y = 0;
  xmotion = ymotion = 0;
}

/**
 * Serializes the mouse sample into data buffer. This
 * function is used by SocketDataSource template to
 * fetch samples from network sockets
 *
 * @param buffer = buffer from which to read
 * @note DI_XXX classes are part of Diva-library, check the
 *       documentation of Diva for details
 */

void  MouseSample::read(DI_DataBuffer2 &buffer)
{
  state = buffer.readInt32();
  x = buffer.readInt32();
  y = buffer.readInt32();

  // Trace("MouseSample::read %d %d %d", b, x, y);
}

/**
 * Reads the mouse sample from data buffer. This
 * function is used by SocketDataSource template to
 * write samples to network socket
 *
 * @param buffer = buffer in which to write
 * @note DI_XXX classes are part of Diva-library, check the
 *       documentation of Diva for details
 */
void  MouseSample::write(DI_DataBuffer2& buffer) const
{
  buffer.writeInt32(state);
  buffer.writeInt32(x);
  buffer.writeInt32(y);

  // Trace("MouseSample::write %d", b);
}

/**
 * Returns the bytes consumed by single sample serialized
 * by write(DI_DataBuffer2& buffer)
 *
 */
int MouseSample::getMaxSerializedSize() const
{
  return 4*3+4; // 4 for padding
}

Mouse::Mouse(Chunk* chunk, DataSource<MouseSample>* dataSource)
  :InputDevice<MouseSample>(chunk,dataSource)
{
}

bool Mouse::getButtonState(Button button, ulong n)
{
  return getSample(n).state & (1 << button);
}

int Mouse::getX(ulong n)
{
  return getSample(n).x;  
}

int Mouse::getY(ulong n)
{
  return getSample(n).y;
}

std::string Mouse::getClassName() const 
{ 
  return "Mouse"; 
}

} // namespace

template class MJ_RingBuffer<Fluid::MouseSample>;
