/* COPYRIGHT
 *
 * This file is part of FLUID
 *
 * Author: Tommi Ilmonen, 1999-2001.
 * Tommi.Ilmonen@hut.fi
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. 
 *    
 */

#include <Fluid/Napra.hpp>

#include <di_data_buffer.h>

#include <mj_ringbuffertmpl.h>

#include <strings.h>

namespace Fluid
{

  void NapraSample::clear()
  {
    bzero(this, sizeof(*this));
  }

  /**
   * De-serializes the napra sample from data buffer. This
   * function is used by SocketDataSource template to
   * fetch samples from network sockets
   *
   * @param buffer = buffer from which to read
   * @note DI_XXX classes are part of Diva-library, check the
   *       documentation of Diva for details
   */

  void NapraSample::read(DI_DataBuffer2 &buffer)
  {
    int v = buffer.readInt32();
    if(v != 1)
      return;

    for(int n = 0; n < 2; n++) {

      switches[n] = buffer.readInt32();
      for(int i=0; i<3; i++)
	for(int j=0; j<3; j++)
	  fingerTips[n][i][j] = buffer.readFloat32();
    }
  }

  /**
   * Writes the napra sample into data buffer. This
   * function is used by SocketDataSource template to
   * write samples to network socket
   *
   * @param buffer = buffer in which to write
   * @note DI_XXX classes are part of Diva-library, check the
   *       documentation of Diva for details
   */
  void  NapraSample::write(DI_DataBuffer2& buffer) const
  {
    buffer.writeInt32(1); // Version
    for(int n = 0; n < 2; n++) {

      buffer.writeInt32(switches[n]);
      for(int i=0; i<3; i++)
	for(int j=0; j<3; j++)
	  buffer.writeFloat32(fingerTips[n][i][j]);
    }
  }

  std::string   Napra::getClassName() const
  {
    return "Napra";
  }

}

template class MJ_RingBuffer<Fluid::NapraSample>;
