/* COPYRIGHT
 *
 * This file is part of FLUID
 *
 * Author: Tommi Ilmonen, 1999-2001.
 * Tommi.Ilmonen@hut.fi
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. 
 *    
 */
#ifndef FLUID_NAPRA_HPP
#define FLUID_NAPRA_HPP

#include <Fluid/InputDevice.hpp>
#include <Fluid/Vector3.hpp>

class DI_DataBuffer2;

namespace Fluid {

  class NapraSample
  {
  public:

    enum {
      SWITCH_LITTLE_FINGER    = 0x1,
      SWITCH_RING_FINGER_NEAR = 0x2,
      SWITCH_RING_FINGER_FAR  = 0x4,
      SWITCH_MIDDLE_THUMB     = 0x8,
      SWITCH_INDEX_THUMB      = 0x10      
    };

    void       clear();
    void       read(DI_DataBuffer2 &buffer);
    void       write(DI_DataBuffer2 &buffer) const;
  
    Vector3 fingerTips[2][3];
    uint    switches[2];
  };

  /**
   * Napra API
   *
   * This class provides raw access to the input data from a
   * napra. Napra (or Naeprae, or Npr), is device for finger tip
   * tracking. Napra tracks the motion of three finger tips. In
   * addition to this location information there are some buttons and
   * contact between finger tips provides us with triggering
   * information.
   *
   */

  class Napra : public InputDevice<NapraSample>
  {
  public:

    Napra(Chunk* chunk, DataSource<NapraSample>* source) 
      : InputDevice<NapraSample>(chunk, source) {}

    std::string   getClassName() const;
  };

} // namespace

#endif
