%{
#include "Config.hpp"
#include "ConfigChunk.hpp"
#include "Debug.hpp"
#include <iostream>
#include <assert.h>
using namespace Fluid;
ConfigChunk* currentChunk = 0;  
std::string currentVariable;

// #define YY_SKIP_YYWRAP 1

#ifndef __sgi
extern "C" int FluidParser_wrap()
{
  // return yywrap();
  return 0;
}
#else
#define  FluidParser_wrap yywrap
#endif


//  using std;
%}
SYMCHAR [^\n{}=/ \\]
VALCHAR [^\n{}=/]
VALUE   [^"\n\t{}=]*[^"\t\n{}= ]
PVALUE  ["].*["]
BLANK [ \n\t]

%x comment
%x chunkstart
%x inchunk
%x variable
%x value
%option stack
%%
<*>"/*"         yy_push_state(comment);
<comment>[^*\n]*        /* eat anything that's not a '*' */
<comment>"*"+[^*/\n]*   /* eat up '*'s not followed by '/'s */
<comment>\n             /* if line number were counted ... */
<comment>"*"+"/"        yy_pop_state();

<INITIAL>{BLANK}*                          /* eat spaces */
<INITIAL>{SYMCHAR}+ {
              BEGIN(chunkstart); 
	      currentChunk = new ConfigChunk(yytext);
                }
<chunkstart>{BLANK}*"{" BEGIN(inchunk);
<inchunk>"}" BEGIN(INITIAL); if(currentChunk) Config::addConfigChunk(currentChunk);
<inchunk>{BLANK}*
<inchunk>{SYMCHAR}+ {
  BEGIN(variable);
  currentVariable = yytext;
  }
<variable>{BLANK}*={BLANK}* BEGIN(value);
<value>{VALUE} { BEGIN(inchunk); if(currentChunk) currentChunk->set(currentVariable,Variant(yytext));}
<value>{PVALUE} { 
  BEGIN(inchunk); 
  assert(yytext[0] == '"' && yytext[strlen(yytext)-1] == '"');
  std::string text = yytext+1;
  text.resize(text.size()-1);
  if(currentChunk) 
    currentChunk->set(currentVariable,Variant(text)); 
}

<variable>.   errStream << "Parse error: Expected \"=\", but found "<<yytext<<" instead\n";
<value>.      errStream << "Parse error: " << yytext << " is illegal value for variable " << currentVariable << '\n';
<inchunk>.    errStream << "Parse error: Expected a variable name in chunk " << currentChunk->getName() << ", but found " << yytext << " instead\n";
<chunkstart>.  errStream << "Parser error in chunk: " << currentChunk->getName() << '\n';

%%
bool parse_configuration(const char *filename)
{
  assert(filename);
  yyin = fopen(filename,"rb");
  if(yyin)
    {
      yylex();
      fclose(yyin);
      return true;
    }
  return false;
}
