/* COPYRIGHT
 *
 * This file is part of FLUID
 *
 * Author: Tommi Ilmonen, Janne Kontkanen 1999-2002.
 * Tommi.Ilmonen@hut.fi, Janne.Kontkanen@hut.fi
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. 
 *    
 */

#ifndef FLUID_SERIAL_PORT_HPP
#define FLUID_SERIAL_PORT_HPP

#include <unistd.h>

namespace Fluid
{

  /** A serial port handler. This class is for binary IO with serial
      ports.

      @author Tommi Ilmonen
  */

  class SerialPort
  {
  public:
    SerialPort();
    ~SerialPort();
  
    bool open(const char * device, 
	      /// Use stop bit
	      bool stopBit, 
	      /// Use parity bit
	      bool parityBit,
	      /// The baud rate
	      int baud, 
	      /// The number of data bits
	      int bits, 
	      /// The number of bytes to be read before returning
	      int waitBytes, 
	      /// Time to wait in microseconds
	      int waitTimeUS);
    bool close();

    int write(const void * buf, int bytes) { return ::write(m_fd, buf, bytes);}
    int read(void * buf, int bytes) { return ::read(m_fd, buf, bytes); }

    bool isOpen() const { return m_fd >= 0; }
  
  protected:
  
    int m_fd;
  };

}

#endif


