/* COPYRIGHT
 *
 * This file is part of FLUID
 *
 * Author: Tommi Ilmonen, 1999-2001.
 * Tommi.Ilmonen@hut.fi
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. 
 *    
 */

#ifndef FLUID_SPEECH_HPP
#define FLUID_SPEECH_HPP

#include <Fluid/InputDevice.hpp>
#include <Fluid/TimeStamp.hpp>

#include <string>

namespace Fluid {

class Phrase
{
public:
  Phrase();
  ~Phrase();

  void       clear() { time = 0; phrase.resize(0); }
  void       read(DI_DataBuffer2 &buffer);
  void       write(DI_DataBuffer2 &buffer) const;
  int        getMaxSerializedSize() const { return 100; }  

  TimeStamp   time;

  std::string phrase;
};


class Speech : public InputDevice<Phrase>
{
public:

  Speech(Chunk* chunk, DataSource<Phrase>* source) 
    : InputDevice<Phrase>(chunk, source){}

  virtual ~Speech();

  virtual std::string   getClassName() const;

};

template <>
TimeStamp IONodeBuffer<Phrase>::getTimeStamp(long i) const
{
  return getIndex(i).time;
}

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////


template <>
inline void InputDevice<Phrase>::start()
{
  assert(dataSource);

  if(running) return;

  history.resize(dataSource->bufferSize());

  history.setSampleCount(0);
  history.setLatestTimeStamp(0);

  dataSource->start();

  running = true;

  history.setSampleRate(dataSource->getSampleRate());
}

} // namespace Fluid


#endif
