/* COPYRIGHT
 *
 * This file is part of the Geometry Engine - GEE
 *
 * Read the "GEE-LICENSE" file for the license.
 *
 * Authors & Copyright:   
 *
 * Tommi Ilmonen 2003
 *
 * firstname.lastname@hut.fi
 *
 */

#include <BglApp.hpp>

#include <Fluid/InputDevices.hpp>
#include <Fluid/Tracker.hpp>

#include <di_error.h>

#include <bgl.h>

using namespace Fluid;

BglApp::BglApp()
  : m_run(false)
{}

BglApp::~BglApp()
{}

bool BglApp::loadConfigs(const char * caveProjFile,
			 const char * bglConfigFile)
{
  return m_projections.init(caveProjFile, bglConfigFile);
}

bool BglApp::exec()
{
  childLoop();

  return true;
}

bool BglApp::quit()
{
  m_run = false;
  return true;
}

void BglApp::childLoop()
{
  m_run = true;

  contextInit();

  while(m_run) {

    // printf(",");

    preFrame();

    setHeadPosition();

    // printf("[");

    m_projections.cave().calculateProjections();
    // printf("]");
    m_projections.setProjectionMatrices();

    // printf("]");

    draw();

    bglSwapBuffers();
  }

  contextCleanup();

  bglQuit();
}

void BglApp::contextInit()
{}

void BglApp::contextCleanup()
{}

void BglApp::draw()
{}

void BglApp::preFrame()
{}

void BglApp::setHeadPosition()
{
  m_projections.cave().setHeadPosition(Vector3(0, 0, 0), Vector3(1, 0, 0));
}

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

BglFluidApp::BglFluidApp()
  : m_tracker(0),
    m_headSensor(0)
{}

BglFluidApp::~BglFluidApp()
{}

bool BglFluidApp::exec()
{
  const char * fname = "BglFluidApp::exec";

  m_tracker = InputDevices::getTracker("tracker");

  if(!m_tracker) {
    DI_error2(DI_ERR_UNINITIALIZED, "%s # No tracker", fname);
    return false;
  }
  
  m_headSensor = m_tracker->findSensor("head");

  if(m_headSensor < 0) {
    DI_error2(DI_ERR_UNINITIALIZED, "%s # No head sensor", fname);
    return false;
  }
  return BglApp::exec();
}

void BglFluidApp::setHeadPosition()
{
  InputDevices::update();
  TrackerSample & sample = m_tracker->outputBuffer().getNewest();
  
  if(m_headSensor < sample.getSensorCount()) {
    const Matrix3 & r = sample.getRotation(m_headSensor);
    const Vector3 & l = sample.getLocation(m_headSensor);
    Vector3 right = r * Vector3(0, 0, 1);
    m_projections.cave().setHeadPosition(l, right);
    // m_cave.setHeadPosition(Vector3(0, 0, 0), Vector3(1, 0, 1));
  }
  else {
    DI_trace2("BglFluidApp::setHeadPosition # "
	      "head sensor %d not present in tracker sample", 
	      m_headSensor);
  }
}
