/* COPYRIGHT
 *
 * This file is part of the Geometry Engine - GEE
 *
 * Read the "GEE-LICENSE" file for the license.
 *
 * Authors & Copyright:   
 *
 * Tommi Ilmonen 2003
 *
 * firstname.lastname@hut.fi
 *
 */

#ifndef _FLUID_BGL_APP_HPP_
#define _FLUID_BGL_APP_HPP_

#include <BglProjections.hpp>

namespace Fluid {
  class Tracker;
}

class BglApp
{
public:
  BglApp();
  virtual ~BglApp();

  bool loadConfigs(const char * caveProjFile,
		   const char * bglConfigFile);
  virtual bool exec();
  bool quit();

protected:
  virtual void childLoop();

  /// Should be overridden if you want to initialize things
  virtual void contextInit();

  /// Should be overridden if you need to clean up things
  virtual void contextCleanup();

  /// Should be overridden to draw something. 
  virtual void draw();

  /// Should be overridden to do per-frame calculations
  virtual void preFrame();

  virtual void setHeadPosition();

  BglProjections   m_projections;

  bool m_run;
};

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

class BglFluidApp : public BglApp
{
public:
  BglFluidApp();
  virtual ~BglFluidApp();

  virtual bool exec();

protected:

  virtual void setHeadPosition();

private:

  Fluid::Tracker * m_tracker;
  int              m_headSensor;

};

#endif
