/* COPYRIGHT
 *
 * This file is part of the Geometry Engine - GEE
 *
 * Read the "GEE-LICENSE" file for the license.
 *
 * Authors & Copyright:   
 *
 * Tommi Ilmonen 2003
 *
 * firstname.lastname@hut.fi
 *
 */

/* Two test cases for this library:
   
   1) basicTest - a simple test to load a cave setup and dump its
   contents to the screen.

   2) heavyTest - runs a minimal VR test application in a cave
   environment.

*/

#include <BglApp.hpp>
#include <EveTestEngine.hpp>

#include <Fluid/Config.hpp>
#include <Fluid/InputDevices.hpp>

class TestApp : public BglFluidApp
{
public:
  TestApp() {}
  virtual ~TestApp() {}

  void init(int models = 50, int tris = 10)
  {
    m_engine.init(models, tris);
  }
  
  void report()
  {
    m_engine.report();
  }

protected:
  
  virtual void contextInit()
  {
    DI_trace2("TestApp::contextInit");
  }

  virtual void draw()
  {
    m_engine.draw();
  }

  virtual void preFrame()
  {
    m_engine.update();
  }  

  EveTestEngine m_engine;
};


void heavyTest(const char * projfile, const char * bglfile)
{
  TestApp app;

  if(!app.loadConfigs(projfile, bglfile))
    return;

  app.init(200);
  app.exec();
  app.report();
}

void basicTest(const char * projfile, const char * bglfile)
{
  BglProjections bglProj;

  bglProj.init(projfile, bglfile);
}

int main(int argc, char **argv)
{
  const char * projFile  = "../caveproj/eve.vrjfluid.config";
  const char * fluid = "../caveproj/fluid-config";
  const char * bglFile  = "bgl-config";

  bool testRun = false;

  for(int i = 1; i < argc; i++) {

    const char * arg = argv[i];

    if(strcmp(arg, "--vrs") == 0 && i + 1 < argc)
      projFile = argv[++i];
    if(strcmp(arg, "--fluid") == 0 && i + 1 < argc)
      fluid = argv[++i];
    if(strcmp(arg, "--bgl") == 0 && i + 1 < argc)
      bglFile = argv[++i];
    else if(strcmp(arg, "--run") == 0)
      testRun = true;
  }

  if(!testRun)
    basicTest(projFile, bglFile);
  else {
    Fluid::DeviceConfig::init(fluid);
    Fluid::InputDevices::init();
    Fluid::InputDevices::start();
    heavyTest(projFile, bglFile);
  }

  return 0;
}
