/* COPYRIGHT
 *
 * This file is part of FLUID
 *
 * Author: Tommi Ilmonen, Janne Kontkanen 1999-2002.
 * Tommi.Ilmonen@hut.fi, Janne.Kontkanen@hut.fi
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. 
 *    
 */

#include <Fluid/CMatrix3.h>

#include <Fluid/Matrix3.hpp>

using namespace Fluid;

FLUID_CMatrix3 FLUID_CMatrix3_multiplyMM(const FLUID_CMatrix3 *m1,
					 const FLUID_CMatrix3 *m2)
{
  const Matrix3 *fm1 = (const Matrix3 *) m1;
  const Matrix3 *fm2 = (const Matrix3 *) m2;

  Matrix3 r((*fm1) * (*fm2));

  return * (FLUID_CMatrix3 *) &r;
}

FLUID_CVector3 FLUID_CMatrix3_multiplyMV(const FLUID_CMatrix3 *m,
					 const FLUID_CVector3 *v)
{
  const Matrix3 *fm = (const Matrix3 *) m;
  const Vector3 *fv = (const Vector3 *) v;

  Vector3 r((*fm) * (*fv));

  return * (FLUID_CVector3 *) &r;
}


void FLUID_CMatrix3_transpose(FLUID_CMatrix3 *m)
{
  ((Matrix3 *) m)->transpose();
}

void FLUID_CMatrix3_clear(FLUID_CMatrix3 *m)
{
  ((Matrix3 *) m)->clear();
}

void FLUID_CMatrix3_identity(FLUID_CMatrix3 *m)
{
  ((Matrix3 *) m)->identity();
}

void FLUID_CMatrix3_printf(const FLUID_CMatrix3 *m, FILE *f)
{
  fprintf(f, 
	  "[%4.4f   %4.4f   %4.4f\n"
	  " %4.4f   %4.4f   %4.4f\n"
	  " %4.4f   %4.4f   %4.4f]", 
	    m->m[0][0],m->m[0][1],m->m[0][2],
	    m->m[1][0],m->m[1][1],m->m[1][2],
	    m->m[2][0],m->m[2][1],m->m[2][2]);
}
