/* COPYRIGHT
 *
 * This file is part of FLUID
 *
 * Author: Tommi Ilmonen, 1999-2002.
 * Tommi.Ilmonen@hut.fi
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. 
 *    
 */

#include <Fluid/CConfig.h>
#include <Fluid/CInputDevices.h>
#include <Fluid/CTracker.h>

#include <Fluid/Error.hpp>

#include <di_sleep.h>

/** This is a test application that uses the C-API to access Fluid
    functionality. */

int main(int, char **)
{
  const char * trackerName = "tracker";
  const char * sensorName  = "left-hand";

  puts("Loading Fluid configuration data");
  FLUID_CConfig_init(0);

  puts("Initializing input devices");
  FLUID_CInputDevices_init();

  // Get the tracker object
  FLUID_CTracker *tracker = FLUID_CInputDevices_getTracker(trackerName);

  if(!tracker)
    Fluid::Fatal("No tracker \"%s\"");

  // Find a sensor that matches given name
  int sensorId = FLUID_CTracker_findSensor(tracker, sensorName);

  puts("Calling FLUID_CInputDevices_start");

  // Start the device drivers
  FLUID_CInputDevices_start();

  puts("Preparing to run");

  for(uint i=0; i < 1000; i++) {
    
    // Sleep for 0.5 seconds.
    DI_Sleep::sleepMs(500);

    // Update all input devices
    FLUID_CInputDevices_update();

    // Get the latest sample from tracker
    FLUID_CTrackerSample *sample = FLUID_CTracker_getLatestSample(tracker);

    // Get the current rotation matrix
    const FLUID_CMatrix3 *rot = 
      FLUID_CTrackerSample_getRotation(sample, sensorId);

    // Get the current location vector
    const FLUID_CVector3 *loc = 
      FLUID_CTrackerSample_getLocation(sample, sensorId);
    
    // Print current values
    printf("\n\nSample count is %d\nNew rotation matrix:\n",
	   (int) FLUID_CTracker_getLatestIndex(tracker));

    FLUID_CMatrix3_printf(rot, stdout);
    printf("\nNew location vector:\n");
    FLUID_CVector3_printf(loc, stdout);
    fflush(0);
  }
  
  // Stop the devices.
  FLUID_CInputDevices_stop();
}
