/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of DIVA base library.
 *
 * Author(s): Jarmo Hiipakka and Tommi Ilmonen, 2000-2001.
 * Jarmo.Hiipakka@hut.fi Tommi.Ilmonen@hut.fi
 *
 * DIVA - Digital Interactive Virtual Acoustics group 
 * Telecommunications Software and Multimedia Laboratory
 * Helsinki University of Technology
 * Finland
 *
 * http://www.tml.hut.fi/Research/DIVA/
 *
 * DIVA base library is licensed under GNU Lesser General Public
 * License (LGPL). The LGPL conditions can be found in file "LGPL"
 * that is distributed with this source package or obtained from the
 * GNU organization (www.gnu.org).
 * 
 */

#ifndef DI_CONDITION_H
#define DI_CONDITION_H

#include <di_mutex.h>

/// Condition for threads.
class DI_Condition
{
public:
  DI_Condition() 
  { pthread_cond_init(&m_cond, 0); }

  ~DI_Condition() { pthread_cond_destroy(&m_cond); }

  int wait(DI_Mutex &mutex)
  { return pthread_cond_wait(& m_cond, & mutex.pthreadMutex()); }

  int broadcast() 
  { return pthread_cond_broadcast(& m_cond); }

  int broadcast2(DI_Mutex &mutex) 
  { 
    mutex.lock();
    int r = pthread_cond_broadcast(& m_cond); 
    mutex.unlock();
    return r;
  }

  int signal() 
  { return pthread_cond_signal(& m_cond); }

  int signal2(DI_Mutex &mutex) 
  {
    mutex.lock();
    int r = signal();
    mutex.unlock();
    return r;
  }

protected:
  pthread_cond_t m_cond;
};

#endif
