/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of DIVA base library.
 *
 * Author(s): Jarmo Hiipakka and Tommi Ilmonen, 2000-2001.
 * Jarmo.Hiipakka@hut.fi Tommi.Ilmonen@hut.fi
 *
 * DIVA - Digital Interactive Virtual Acoustics group 
 * Telecommunications Software and Multimedia Laboratory
 * Helsinki University of Technology
 * Finland
 *
 * http://www.tml.hut.fi/Research/DIVA/
 *
 * DIVA base library is licensed under GNU Lesser General Public
 * License (LGPL). The LGPL conditions can be found in file "LGPL"
 * that is distributed with this source package or obtained from the
 * GNU organization (www.gnu.org).
 * 
 */
#ifndef DI_ERROR_ENUMS_H
#define DI_ERROR_ENUMS_H

///
enum DI_ErrorLevel {
  /// Program execution trace
  DI_ERRL_TRACE,
  DI_ERRL_INFORMATION,
  DI_ERRL_WARNING,
  DI_ERRL_ERROR,
  DI_ERRL_FATAL,
  DI_ERRL_SIZEOF
};

extern const char *DI_errorLevelStr(DI_ErrorLevel);

extern const char *DI_ErrorLevels[DI_ERRL_SIZEOF];

/// Error codes.
enum DI_ErrorCode {
  /// No error
  DI_ERR_NONE,
  /// Uknown error
  DI_ERR_UNKNOWN,
  /// Out of memory
  DI_ERR_MEMORY,
  /// Data corrupted
  DI_ERR_DATA_CORRUPTED,
  /// Internal error
  DI_ERR_INTERNAL,
  /// System error
  DI_ERR_SYSTEM,
  /// Array size exceeded
  DI_ERR_ARRAY_EXCEED,
  /// List size exceeded
  DI_ERR_LIST_EXCEED,
  /// Bad argument
  DI_ERR_ARGUMENT,
  /// Bad argument
  DI_ERR_NULL_ARGUMENT,
  /// Bad match
  DI_ERR_MATCH,
  /// End of file
  DI_ERR_EOF,
  /// Resource denied
  DI_ERR_RC_DENIED,
  /// Only virtual
  DI_ERR_VIRTUAL,
  /// Do not call this function
  DI_ERR_DO_NOT_CALL,
  /// Not implemented yet
  DI_ERR_UNIMPLEMENTED,
  /// Not initialized yet
  DI_ERR_UNINITIALIZED,
  /// Real-time performance failure
  DI_ERR_REALTIME,
  /// Floating point error
  DI_ERR_FPE,
  /// CPU load too high
  DI_ERR_CPU_LOAD,
  /// Numeric overflow
  DI_ERR_OVERFLOW,
  /// Input problem
  DI_ERR_INPUT,
  /// Output problem
  DI_ERR_OUTPUT,
  /// Number of error codes
  DI_ERR_SIZEOF
};

extern const char *DI_errorStr(DI_ErrorLevel);

/// Character strings for error codes:
extern const char *DI_ErrorCodes[DI_ERR_SIZEOF];

#endif  // DI_ERROR_ENUMS_H
