/* COPYRIGHT
 *
 * This file is part of Mustajuuri.
 *
 * Author: Tommi Ilmonen, 1999-2001.
 * Tommi.Ilmonen@hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * Mustajuuri is licensed under GNU Lesser General Public License
 * (LGPL). The LGPL conditions can be found in file "LGPL" that is
 * distributed with this source package. The Mustajuuri license
 * interpretation that further clarifes LGPL can be found in file
 * "MUSTAJUURI_LICENSE" that is distributed with the Mustajuuri source code.
 *    
 */

// #include <iostream.h>

#include <string.h>

#include "mj_ringbuffer.h"

template <class TElem>
bool MJ_RingBuffer<TElem>::resize(ulong nBufSize)
{
  ulong j = 1;
  while (j < nBufSize)
    j = j << 1;
  
  if(j == m_size) return true;

  delete []m_line;

  if(!nBufSize) {
    m_line = 0;
    m_size = 0;
    m_mask = 0;
    return true;
  }

  m_size = j;
  m_mask = m_size - 1;
  m_line = new TElem[m_size];
  if(!m_line) {
    m_size = 0;
    m_mask = 0;
    return false;
  }
  return true;
}

template <class TElem>
void MJ_RingBuffer<TElem>::setAll(TElem xVal) 
{ 
  TElem *xp1=m_line, *xp2=&m_line[m_size];

  if(m_size < 4)
    while(xp1 < xp2) *xp1++ = xVal; 
  else {
    // Manual loop unrolling. 
    // At any rate the memory will bottleneck us...
    for(;xp1 < xp2; xp1 += 4) {
      *xp1   = xVal;
      xp1[1] = xVal;
      xp1[2] = xVal;
      xp1[3] = xVal;
    }
  }
}

template <class TElem>
MJ_RingBuffer<TElem> &
MJ_RingBuffer<TElem>::operator = (const MJ_RingBuffer &xrBuffer)
{
  if(resize(xrBuffer.m_size, false))
    memcpy(m_line, xrBuffer.m_line, m_size * sizeof(TElem));
  return *this;
}

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

template <class TElem>
TElem MJ_RingBuffer2<TElem>::getMax(ulong nTime) const
{
  TElem xRes = getNewestConst();
  for(ulong nI=1; nI < nTime; nI++) {
    const TElem *xpTmp = &getNewestConst(nI);
    if(*xpTmp > xRes)
      xRes = *xpTmp;
  }
  return xRes;
}

template <class TElem>
TElem MJ_RingBuffer2<TElem>::getMin(ulong nTime) const
{
  TElem xRes = getNewestConst();
  for(ulong nI=1; nI < nTime; nI++) {
    const TElem *xpTmp = &getNewestConst(nI);
    if(*xpTmp < xRes)
      xRes = *xpTmp;
  }
  return xRes;
}
