/* COPYRIGHT
 *
 * This file is part of FLUID
 *
 * Author: Tommi Ilmonen, 1999-2001.
 * Tommi.Ilmonen@hut.fi
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. 
 *    
 */

#include "Fluid/InputDevices.hpp"
#include "Fluid/Config.hpp"
#include "Fluid/Tracker.hpp"
#include "Fluid/DataSource.hpp"
#include "Fluid/DeviceFactory.hpp"

using namespace Fluid;

class EmulatedTrackerSource : public StreamDataSource<TrackerSample>
{
public:
  EmulatedTrackerSource()
    :StreamDataSource<TrackerSample>(20),sampleRate(20),sensorCount(3)
  {
  }

  virtual std::string getDriverName() const { return "EmulatedTracker"; }
  virtual float getSampleRate() const { return sampleRate; }
  virtual void get(TrackerSample& s)
  {
    assert(sampleRate > 1.0f);
    s.setSensorCount(sensorCount);
    for(int i = 0; i < sensorCount; i++)
      {
	Matrix3 m;
	Vector3 v(1+2.0f*i,2+4.0f*i,3+8.0f*i);
	m.identity();
	s.setLocation(i,v);
	s.setRotation(i,m);
      }

    usleep((unsigned int) (1000000.0f/sampleRate));
  }

  float sampleRate;
  int sensorCount;
};

class EmulatedTrackerFactory : public DeviceFactory
{
public:
  virtual std::string      getDriverName () const { return "EmulatedTracker"; }
  virtual InputDeviceBase* create        (ConfigChunk* chunk)
  {
    std::cout << "Creating an emulated tracker" << std::endl;
    return new Tracker(chunk,new EmulatedTrackerSource);
  }
};

EmulatedTrackerFactory factory;

int main(int argc, char **argv)
{
  Config::init();
  InputDevices::addDevices(factory.getDriverName(),&factory);
  InputDevices::init();
  Tracker *tracker = InputDevices::getTracker("emuTracker");
  if(tracker != NULL)
    {
      tracker->start();
      while(1)
	{
	  std::cout << tracker->getLocation(0) << std::endl;
	  usleep(500000);
	}
      tracker->stop();
    }
  else
    {
      std::cout << "emuTracker not found!" << std::endl;
    }


  InputDevices::cleanup();
  Config::cleanup();
  return 0;
}
