/* COPYRIGHT
 *
 * This file is part of FLUID
 *
 * Author: Tommi Ilmonen, 1999-2001.
 * Tommi.Ilmonen@hut.fi
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. 
 *    
 */

#include <di_data_buffer.h>

#include <app.hpp>
#include <Math/vjCoord.h>
#include <Fluid/Vector3.hpp>
#include <Fluid/Tracker.hpp>
#include <Fluid/InputDevices.hpp>

void drawSensor();
extern bool requestExit;

//----------------------------------------------
//  Draw the scene. 
//----------------------------------------------
void App::draw()
{
   mWand->updateData();
   vjMatrix wandPos = *((*mWand).getData());
  
   if(mKeyboard->keyPressed(VJKEY_Q))
     {
       static bool yet = false;
       if(!yet)
	 {
	   std::cout << "EXIT REQUESTED" << std::endl;
	   yet = true;
	 }
       requestExit = true;
     }

   glClearColor(0.0, 0.0, 0.0, 0.0);
   glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

   // Set light position
   GLfloat light0_position[] = {6.0, 6,0, 6.0, 1.0};
   glLightfv(GL_LIGHT0, GL_POSITION,  light0_position);

   //// Draw a doughnut on the wand
   GLfloat mat_specular[] = { 1.0, 1.0, 1.0, 1.0};
   GLfloat mat_shininess[] = { 50.0};

   glEnable(GL_COLOR_MATERIAL);
   glMaterialfv( GL_FRONT, GL_SPECULAR, mat_specular );
   glMaterialfv( GL_FRONT,  GL_SHININESS, mat_shininess );
   glColorMaterial(GL_FRONT,GL_AMBIENT_AND_DIFFUSE);

   glPushMatrix();
   glLoadIdentity();
   glTranslatef(0.0f,0.0f,-30.0f);
   drawSensor();
   glTranslatef(0.0f,0.0f,30.0f);
   glLoadMatrixf(wandPos.getFloatPtr());
   drawSensor();
   glPopMatrix();
}

void drawSensor()
{
  glPushMatrix();
  GLUquadricObj *q = gluNewQuadric();
  glColor3f(0.0f,1.0f,0.5f);
  gluCylinder(q,0.1,0.1,1.0,16,4); //z axis
  glRotatef(90.0f,1.0f,0.0f,0.0f);
  glColor3f(0.0f,0.5f,1.0f);
  gluCylinder(q,0.1,0.1,1.0,16,4); //y axis
  glRotatef(90.0f,0.0f,1.0f,0.0f);
  glColor3f(1.0f,0.5f,0.0f);
  gluCylinder(q,0.1,0.1,1.0,16,4); //z axis
  gluDeleteQuadric(q);
  glPopMatrix();
}

void App::initGLState()
{
  std::cout << "initGLState()" << std::endl;
   GLfloat light0_ambient[] = { .4,  .4,  .4,  1.0};
   GLfloat light0_diffuse[] = { 1.0,  1.0,  1.0,  1.0};
   GLfloat light0_specular[] = { 1.0,  1.0,  1.0,  1.0};
   GLfloat light0_position[] = {6.0, 6,0, 6.0, 1.0};

   glLightfv(GL_LIGHT0, GL_AMBIENT,  light0_ambient);
   glLightfv(GL_LIGHT0, GL_DIFFUSE,  light0_diffuse);
   glLightfv(GL_LIGHT0, GL_SPECULAR,  light0_specular);
   glLightfv(GL_LIGHT0, GL_POSITION,  light0_position);

   glEnable(GL_DEPTH_TEST);
   glEnable(GL_NORMALIZE);
   glEnable(GL_LIGHTING);
   glEnable(GL_LIGHT0);
   glDisable(GL_COLOR_MATERIAL);
   glShadeModel(GL_SMOOTH);

   glMatrixMode(GL_MODELVIEW);
}

