/* COPYRIGHT
 *
 * This file is part of FLUID
 *
 * Author: Tommi Ilmonen, 1999-2001.
 * Tommi.Ilmonen@hut.fi
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. 
 *    
 */

#ifndef _APP
#define _APP

#include <vjConfig.h>

#include <iostream>
#include <math.h>
#include <GL/gl.h>
#include <GL/glu.h>

#include <Kernel/GL/vjGlApp.h>
#include <Math/vjMatrix.h>
#include <Math/vjVec3.h>
#include <Math/vjQuat.h>
#include <Kernel/vjDebug.h>

#include <Input/InputManager/vjPosInterface.h>
#include <Input/InputManager/vjAnalogInterface.h>
#include <Input/InputManager/vjDigitalInterface.h>
#include <Input/InputManager/vjKeyboardInterface.h>

class App : public vjGlApp
{
public:
   App(vjKernel* kern) : vjGlApp(kern)
   {;}

   virtual ~App (void) {
      /* Do nothing. */ ;
   }

   // Execute any initialization needed before the API is started.  Put device
   // initialization here.
   virtual void init()
   {
     std::cout << "App::init()" << std::endl;
     mWand.init("VJWand");
      mKeyboard.init("VJKeyboard");
      //mDigital.init("VJButton0");
      //mTrackerPos.init("VJLeftHand");
     std::cout << "App::init() finished" << std::endl;
   }

   // Execute any initialization needed <b>after</b> API is started
   //  but before the drawManager starts the drawing loops.
   virtual void apiInit()
   {;}

   // Called immediately upon opening a new OpenGL context.  This is called
   // once for every display window that is opened.  Put OpenGL resource
   // allocation here.
   virtual void contextInit()
   {
      initGLState();
   }

   // Function called after tracker update but before start of drawing.  Do
   // calculations and state modifications here.
   virtual void preFrame()
   {;}

   // Function to draw the scene.  Put OpenGL draw functions here.
   //
   // PRE: OpenGL state has correct transformation and buffer selected
   // POST: The current scene has been drawn
   virtual void draw();

   // Function called after drawing has been triggered but BEFORE it completes
   virtual void intraFrame()
   {;}

   // Function called before updating trackers but after the frame is drawn.
   // Do calculations here.
   virtual void postFrame()
   {;}

private:
   void initGLState();

private:
   vjPosInterface    mWand;
   vjKeyboardInterface mKeyboard;
  //vjDigitalInterface mDigital;
  //vjPosInterface mTrackerPos;
   
};

#endif
