/* COPYRIGHT
 *
 * This file is part of FLUID
 *
 * Author: Tommi Ilmonen, 1999-2001.
 * Tommi.Ilmonen@hut.fi
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. 
 *    
 */

//----------------------------------------------------
// This program tests VRJuggler and Fluid co-operation
//----------------------------------------------------
#include <di_data_buffer.h>

#include <app.hpp>
#include <string>
// --- Lib Stuff --- //
#include <Kernel/vjKernel.h>
#include <Kernel/vjProjection.h>
#include <Fluid/Config.hpp>
#include <Fluid/InputDevices.hpp>
#include <Fluid/VrjKeyboardExport.hpp>
#include <Fluid/VrjTrackerExport.hpp>
#include <Fluid/Tracker.hpp>
//#include <unistd.h>

#ifndef WIN32
#include <sched.h>
#endif

bool requestExit = false;

int main(int argc, char* argv[])
{
  bool useFluid = true;

  Fluid::Config::init();
  Fluid::InputDevices::init();
  
   vjProjection::setNearFar(0.01f, 10000.0f);
   
   vjKernel* kernel = vjKernel::instance();        // Get the kernel
   App* application = new App(kernel);   // Declare an instance of the app
   
   if (argc <= 1)
     {
       // display some usage info (holding the user by the hand stuff)
       //  this will probably go away once the kernel becomes separate 
       //  and can load application plugins.
       std::cout<<"\n\n\n";
       std::cout<<"Usage: "<<argv[0]
		<<" vjconfigfile[0] vjconfigfile[1] ... vjconfigfile[n]\n\n\n"
	      <<std::flush;
       exit(1);
     }
  
   if(useFluid)
     {
       Fluid::VrjKeyboardExport::configInit();
       Fluid::VrjTrackerExport::configInit();
       Fluid::Tracker *t = Fluid::InputDevices::getTracker("tracker");
       if(t)
	 {
	   //t->start();
	 }
       else
	 {
	   std::cout << "NOOOOO TRACKER " << std::endl;
	 }

       //Fluid::InputDevices::start();
     }

   std::cout << "Loading VRJ config files" << std::endl;
  
   // Load any config files specified on the command line
   for(int i=1;i<argc;i++)
      kernel->loadConfigFile(argv[i]);

   std::cout << "Starting VRJ kernel" << std::endl;

   kernel->start();
      //- Kernel load global config  -- Environment variable
      //- App KernelConfig function
      //- Load user local config

   std::cout << "Setting VRJ application" << std::endl;

   kernel->setApplication(application);         // Set application

   std::cout << "Runnign the VRJ application" << std::endl;

   while(!requestExit)
     usleep(500000);

   std::cout << "Trying to quit..." << std::endl;
   kernel->setApplication(0);
   
   if(useFluid)
     {
       Fluid::InputDevices::cleanup();
       Fluid::Config::cleanup();
     }
   
   system((std::string("killall ")+argv[0]).c_str());

   return 1;
}




