/* COPYRIGHT
 *
 * This file is part of FLUID
 *
 * Author: Tommi Ilmonen, 1999-2001.
 * Tommi.Ilmonen@hut.fi
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. 
 *    
 */

#include <5DTGlove.hpp>

using namespace Fluid;

FDTGloveSource::FDTGloveSource(const std::string& device, 
			       int sampleRate, 
			       int bufferSize)
  :DataSource<GloveSample>(bufferSize),
   device(device),
   sampleRate(sampleRate),
   scaled(0),
   sensorCount(0)
{
}

void FDTGloveSource::start()
{
  bool ok = glove.init((char*) device.c_str());
  if(!ok) 
    {
      errStream << "Couldn't open device: " << device.c_str() << '\n';
      return;
    }

  sensorCount = glove.sensorCount();
  scaled = new float[sensorCount];
  m_continue = true;
  run();
}

void FDTGloveSource::stop()
{
  m_continue = false;

  waitEnd();
  
  glove.shutdown();
  delete [] scaled;
  scaled = 0;
}

void FDTGloveSource::childLoop()
{
  if(sensorCount)
    {
      while(m_continue)
	{
	  glove.flexures(scaled);
	  GloveSample sample;
	  if(sensorCount == 5) {
	    
	    for(uint i=0; i < 5; i++) {

	      sample.fingerJoints[i][Glove::FINGERJOINT_BASE] = 0.0f; 
	      sample.fingerJoints[i][Glove::FINGERJOINT_SECOND] = scaled[i];
	      sample.fingerJoints[i][Glove::FINGERJOINT_THIRD] = scaled[i];
	    }
	  }

	  // puts("FDTGloveSource::childLoop");

	  write(sample);
	  usleep(1000000/sampleRate);
	}
    }
}


FDTGloveFactory factory;

extern "C" DeviceFactory* getFactory()
{
  return &factory;
}

