/* COPYRIGHT
 *
 * This file is part of FLUID
 *
 * Author: Tommi Ilmonen, 1999-2001.
 * Tommi.Ilmonen@hut.fi
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. 
 *    
 */

#ifndef _5DTGLOVE_HPP
#define _5DTGLOVE_HPP

#include <Fluid/DataSource.hpp>
#include <Fluid/Glove.hpp>
#include <Fluid/Debug.hpp>
#include <Fluid/Config.hpp>
#include <Fluid/ConfigReader.hpp>
#include <Fluid/DeviceFactory.hpp>

#include <Serial5DT.hpp>

#include <unistd.h>

using namespace Fluid;

class FDTGloveSource : public DataSource<GloveSample>, public DI_Thread
{
public:
  FDTGloveSource(const std::string& device, int sampleRate, int bufferSize);
  void start();
  void stop();
  void childLoop();

  std::string getDriverName() const { return "5DTGlove"; }
private:
  std::string device;
  int      sampleRate;
  Serial5DT glove;
  float   *scaled;
  int      sensorCount;

  bool     m_continue;
};

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

class FDTGloveFactory : public DeviceFactory
{
public:
  virtual std::string      getDriverName() const { return "5DTGlove"; }
  FDTGloveSource*          createDataSource(Fluid::Chunk* chunk)
  {
    std::string device;
    int sampleRate,bufferSize;
    
    sampleRate = chunk->get("sampleRate").getInt(60);
    
    
    device = chunk->get("device").getString("/dev/fglove0");
    
    bufferSize = chunk->get("bufferSize").getInt(50);

    FDTGloveSource *source = new FDTGloveSource(device,sampleRate,bufferSize);
    return source;
  }

  virtual InputDeviceBase* create(Chunk* chunk)
  {

    Glove * g = new Glove(chunk,createDataSource(chunk));
    g->outputBuffer().setSampleRate(chunk->get("sampleRate").getDouble(60));
    return g;
  }
};

extern FDTGloveFactory factory;


#endif
