/* COPYRIGHT
 *
 * This file is part of FLUID
 *
 * Author: Tommi Ilmonen, 1999-2001.
 * Tommi.Ilmonen@hut.fi
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. 
 *    
 */

#include "Serial5DT.hpp"

#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/stat.h>

#include <fcntl.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <termios.h>

Serial5DT::Serial5DT()
  : m_fd(-1), 
    m_times(0),
    m_sensorCount(0),
    m_autoCalibrate(true) 
{}

Serial5DT::~Serial5DT() 
{ 
  shutdown(); 
}
  
bool Serial5DT::init(const char * device)
{
  m_times = 0;
  m_fd = open(device, O_RDWR);
  
  printf("m_fd = %d\n", m_fd);
  
  if(m_fd <= 0)
    return false;

  if(1) {
    struct termio sttyArgs;
    ioctl(m_fd, TCGETA , &sttyArgs);    

    sttyArgs.c_cflag &= ~CSTOPB;	/* One stop bit */
    sttyArgs.c_cflag &= ~PARENB;	/* No parity */

    sttyArgs.c_cflag &= ~CBAUD;
    sttyArgs.c_cflag |=  B19200; 

    /* sttyArgs.c_cflag &= ~CBAUD;
       sttyArgs.c_cflag |=  B1200; */
    sttyArgs.c_cflag &= ~CSIZE;
    sttyArgs.c_cflag |= CSIZE & CS8;
    
    sttyArgs.c_cflag |= CREAD;	/* Allow reading */
    sttyArgs.c_cflag |= CLOCAL;	/* No modem between us and device */
    
    sttyArgs.c_cc[VMIN] = 0;	/* Return read even if no chars */
    sttyArgs.c_cc[VTIME] = 0;	/* Return without waiting */
    
    sttyArgs.c_iflag = (IGNBRK | IGNPAR);  /* Ignore break & parity errs */
    sttyArgs.c_oflag = 0;                  /* Raw output, leave tabs alone */
    sttyArgs.c_lflag = 0;              /* Raw input (no KILL, etc.), no echo */

    if(ioctl(m_fd, TCSETA, &sttyArgs) < 0) {
      puts("Failed to set TTY parameters");
    }
  }

  unsigned char command;
  unsigned char res[32];
  bzero(res, 32);

  command = RESET;

  usleep(20000);

  write(m_fd, & command, 1);
  usleep(20000);

  read(m_fd, res, 1);
    
  printf("com = %x, res = %x\n", (int) command, (int) res[0]);
    
  command = REQUEST_GLOVE_INFORMATION;
  write(m_fd, & command, 1);
    
  usleep(20000);
    
  int n = 0;
  
  while(n < 32 && n >= 0) {
    printf(" Have read %d", n); fflush(0);
    n += read(m_fd, &res[n], 32 - n);
    if(n != 32)
      usleep(20000);
  }
    
  printf(" read %d bytes\n Header = %x %x\n version = %d.%d\n"
	 " caplow = %x, caphigh = %x\n id = \"%s\"\n", n,
	 (int) res[0], (int) res[1],
	 (int) res[2], (int) res[3],
	 (int) res[4], (int) res[5],
	 &res[20]);
    
  m_gloveId = (const char *) &res[20];

  if(m_gloveId == "DataGlove5")
    m_sensorCount = 5;

  m_rightHanded = res[4] & 0x1;

  command = TEST_SERIAL_IO;
  write(m_fd, & command, 1);
  usleep(20000);
  command = 77;
  write(m_fd, & command, 1);
  usleep(20000);

  int bytes = 0;
  while(bytes == 0)
    bytes += read(m_fd, res, 1);

  printf("res = %d (should be 77)\n", res[0]);

  for(int i=0; i < 5; i++) {
    m_min[i] = 0;
    m_max[i] = 0xff;
  }
      
  

  return res[0] == 77;
}

bool Serial5DT::shutdown()
{
  if(m_fd <= 0)
    return false;

  close(m_fd);

  m_fd = -1;
  m_sensorCount = 0;

  return true;
}

bool Serial5DT::reset()
{
  char command = RESET;

  write(m_fd, & command, 1);

  read(m_fd, &command, 1);

  return command == RECEIVED;
}

void Serial5DT::flexures(float * data)
{
  // reset();
    
  char command = SET_REPORT_DATA;
  // char command = SET_DATA_STREAM_MODE;
  // usleep(50000);
  int bytes = write(m_fd, & command, 1);
  /* bytes = write(m_fd, & command, 1);
     bytes = write(m_fd, & command, 1);
     bytes = write(m_fd, & command, 1);
     bytes = write(m_fd, & command, 1);
     bytes = write(m_fd, & command, 1);
     bytes = write(m_fd, & command, 1); */

  if(bytes != 1)
    puts("Failed to write command");

  int n = 0;
  unsigned char res[9];
    
  while(n < 9) {
    // usleep(50000);
    n += read(m_fd, res + n, 9 - n);
    /* printf("Reading yet %d", 9 - n);
       fflush(0); */
  }

  if(!m_rightHanded) {
    unsigned char tmp = res[5];
    res[5] = res[1];
    res[1] = tmp;
    tmp = res[4];
    res[4] = res[2];
    res[2] = tmp;
  }

  int i;

  if(!m_times && m_autoCalibrate) {
    for(i=0; i < 5; i++) {
      m_min[i] = res[i+1];
      m_max[i] = res[i+1];
    }
  }
    
  if(m_autoCalibrate) {
    for(i=0; i < 5; i++) {
      unsigned char tmp = res[i+1];
      if(tmp < m_min[i]) m_min[i] = tmp;
      if(tmp > m_max[i]) m_max[i] = tmp;
    }
  }

  for(i=0; i < 5; i++) {
    float range = m_max[i] - m_min[i];
    if(range == 0.0f)
      range = 1.0f;
      
    float tmp = (res[i + 1] - m_min[i]) / range;

    data[i] = tmp;
  }

  m_times++;
}
