/* COPYRIGHT
 *
 * This file is part of FLUID
 *
 * Author: Tommi Ilmonen, 1999-2001.
 * Tommi.Ilmonen@hut.fi
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. 
 *    
 */

#include <5DTGlove.hpp>

#include <di_socket.h>

#include <string>

using namespace Fluid;

int main(int argc, char **argv)
{
  unsigned repeat = 1;

  std::cout << "FLUID 5DT Data Glove Server starting up...\n";
  std::string configuration;

  for(int argn=1; argn < argc; argn++) {

    const char * arg = argv[argn];

    if(strcmp(arg, "--repeat") == 0) {
      if(argn + 1 <= argc) {
	repeat = atoi(argv[++argn]);
      }
    }
    else if(arg) {
      configuration = arg;
    }
    else {
      std::cout << "USAGE: " << argv[0] 
		<< " <configuration file> <--repeat n>\n";
      exit(0);
    }
  }

  DeviceConfig::init(configuration);
  Chunk *chunk = DeviceConfig::getChunk("5DTGlove");

  if(!chunk) {
    std::cerr << "No 5DTGlove conviguration found" << std::endl;
    return 0;
  }

  DataSource<GloveSample> *source = factory.createDataSource(chunk);
  std::string port = chunk->get("port").getString("2199");

  int sampleRate = chunk->get("sampleRate").getInt(60);


  DI_Socket socket;
  GloveSample sample;
  DI_DataBuffer2 buffer(sample.getMaxSerializedSize());
  socket.open(port.c_str(),true); // server mode    

  if(!socket)  {
    std::cout << "Error creating server socket" << std::endl;
    exit(0);
  }
  else {
    std::cout << "Created server socket into port: " << port << std::endl;
  }
  
  while(repeat--) {
  
    std::cout << repeat  << " repetitions left" << std::endl;
      
    
    DI_Socket *connection = 0;
    std::cout << "Waiting for client to connect" << std::endl;

    while(connection == 0)
      connection = socket.listen(); 

    std::cout << "Got a client connection" << std::endl;
    std::cout << "Starting glove source" << std::endl;
    source->start();
    sleep(1);

    while(connection->isOK())
      {
	source->lock();
	int avail = source->getAvailable();
	while(avail--)
	  {
	    buffer.rewind();
	    source->read(sample);
	    sample.write(buffer);
	    DI_writeToSocket(connection,&buffer);
	    // puts("One round");
	  }
	source->release();
	usleep(1000000/sampleRate);
      }

    source->stop();
    delete source;
    connection->close();
    delete connection;
  }
  socket.close();

  std::cout << "5DTGlove Server shutting down\n" << std::endl;
  DeviceConfig::cleanup();
  return 0;
}
