/* COPYRIGHT
 *
 * This file is part of FLUID
 *
 * Author: Tommi Ilmonen, 1999-2001.
 * Tommi.Ilmonen@hut.fi
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. 
 *    
 */

#include <Fluid/DeviceFactory.hpp>
#include <Fluid/Debug.hpp>
#include <Fluid/FileDataSource.hpp>
#include <Fluid/InputDevices.hpp>
#include <Fluid/ConfigReader.hpp>
#include <Fluid/Glove.hpp>

#include <stdlib.h>

using namespace Fluid;

class FileGloveFactory : public DeviceFactory
{
public:
  virtual std::string getDriverName() const { return "FileGlove"; } 
  virtual InputDeviceBase* create(Chunk* chunk)
  {
    std::string filename;
    int bufferSize;
    double sampleRate;

    bufferSize = chunk->get("bufferSize").getInt(50);
    
    filename = chunk->get("filename").getString("undefined");
    
    sampleRate = chunk->get("sampleRate").getDouble(105.8);
    
    FileDataSource<GloveSample> *source = new FileDataSource<GloveSample>(filename,bufferSize, sampleRate);
    return new Glove(chunk,source);
  }
};

FileGloveFactory factory;

extern "C" DeviceFactory* getFactory()
{
  return &factory;
}
