/* COPYRIGHT
 *
 * This file is part of FLUID
 *
 * Author: Tommi Ilmonen, 1999-2001.
 * Tommi.Ilmonen@hut.fi
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. 
 *    
 */

#include <Fluid/DeviceFactory.hpp>
#include <Fluid/Debug.hpp>
#include <Fluid/FileDataSource.hpp>
#include <Fluid/InputDevices.hpp>
#include <Fluid/ConfigReader.hpp>
#include <Fluid/Mouse.hpp>

#include <stdlib.h>

using namespace Fluid;

class FileMouseFactory : public DeviceFactory
{
public:
  virtual std::string getDriverName() const { return "FileMouse"; } 
  virtual InputDeviceBase* create(Chunk* chunk)
  {
    std::string filename;
    int bufferSize;
    double sampleRate;


    bufferSize = chunk->get("bufferSize").getInt(50);

    filename = chunk->get("filename").getString("undefined");
    
    sampleRate = chunk->get("sampleRate").getDouble(40.0);
    
    FileDataSource<MouseSample> *source = new FileDataSource<MouseSample>(filename,bufferSize, sampleRate);
    return new Mouse(chunk,source);
  }
};

FileMouseFactory factory;

extern "C" DeviceFactory* getFactory()
{
  return &factory;
}
