/* COPYRIGHT
 *
 * This file is part of FLUID
 *
 * Author: Tommi Ilmonen, 1999-2001.
 * Tommi.Ilmonen@hut.fi
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. 
 *    
 */

#include <Fluid/DeviceFactory.hpp>
#include <Fluid/Variant.hpp>
#include <Fluid/Debug.hpp>
#include <Fluid/FileDataSource.hpp>
#include <Fluid/InputDevices.hpp>
#include <Fluid/ConfigReader.hpp>
#include <Fluid/Speech.hpp>

#include <stdlib.h>

using namespace Fluid;

class FileSpeechFactory : public DeviceFactory
{
public:
  virtual std::string getDriverName() const { return "FileSpeech"; } 
  virtual InputDeviceBase* create(Chunk* chunk)
  {
    std::string filename;
    int bufferSize;
    double sampleRate;


    Variant v = chunk->get("bufferSize");
    if(v.getType() != Variant::INVALID)
      bufferSize = v;
    else
      bufferSize = 50;
    
    v = chunk->get("filename");
    if(v.getType() != Variant::INVALID)
      filename = std::string(v);
    else
      filename = "undefined";
    
    v = chunk->get("sampleRate");
    if( v.getType() != Variant::INVALID)
      sampleRate = v;
    else 
      sampleRate = 40.0;
    
    FileDataSource<Phrase> *source = new FileDataSource<Phrase>(filename,bufferSize, sampleRate);
    return new Speech(chunk,source);
  }
};

FileSpeechFactory factory;

extern "C" DeviceFactory* getFactory()
{
  return &factory;
}
