/* COPYRIGHT
 *
 * This file is part of FLUID
 *
 * Author: Tommi Ilmonen, 1999-2001.
 * Tommi.Ilmonen@hut.fi
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. 
 *    
 */

#include <Fluid/DeviceFactory.hpp>
#include <Fluid/Debug.hpp>
#include <Fluid/FileDataSource.hpp>
#include <Fluid/InputDevices.hpp>
#include <Fluid/ConfigReader.hpp>
#include <Fluid/Tracker.hpp>

#include <stdlib.h>

using namespace Fluid;

/* class FileTracker : public FileDataSource<TrackerSample>
{
public:
  FileTracker(const std::string& filename, int bufferSize)
    : FileDataSource(filename, bufferSize),
      m_sensors(0)
  {}
  
  virtual void start()
  {
    FileDataSource<TrackerSample>::start();
    
    if(isRunning()) {
      if(getLatestIndex() >= 0) {

	m_sensors = getSample(0).getSensorCount();

	Trace("FileTracker::start # resizing all sensors to %d", m_sensors);
	
	for(uint i=0; i < historySize(); i++) {
	  getSample(i).setSensorCount(m_sensors);
	}
      }
    }
  }

protected:
  uint m_sensors;
  };*/

class FileTrackerFactory : public DeviceFactory
{
public:
  virtual std::string getDriverName() const { return "FileTracker"; } 
  virtual InputDeviceBase* create(Chunk* chunk)
  {
    std::string filename;
    int bufferSize;
    double sampleRate;

    bufferSize = chunk->get("bufferSize").getInt(50);
    
    filename = chunk->get("filename").getString("undefined");
    
    sampleRate = chunk->get("sampleRate").getDouble(105.8);
    
    FileDataSource<TrackerSample> *source = 
      new FileDataSource<TrackerSample>(filename,bufferSize, sampleRate);

    return new Tracker(chunk,source);
  }
};

FileTrackerFactory factory;

extern "C" DeviceFactory* getFactory()
{
  return &factory;
}
