/* COPYRIGHT
 *
 * This file is part of FLUID
 *
 * Author: Tommi Ilmonen, 1999-2001.
 * Tommi.Ilmonen@hut.fi
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. 
 *    
 */

#include <Fluid/DeviceFactory.hpp>
#include <Fluid/Debug.hpp>
#include <Fluid/SocketDataSource.hpp>
#include <Fluid/InputDevices.hpp>
#include <Fluid/ConfigReader.hpp>
#include <Fluid/Glove.hpp>

using namespace Fluid;

class RemoteGloveFactory : public DeviceFactory
{
public:
  virtual std::string getDriverName() const { return "RemoteGlove"; } 
  virtual InputDeviceBase* create(Chunk* chunk)
  {
    std::string server;
    int bufferSize;

    bufferSize = chunk->get("bufferSize").getInt(50);
    
    server = chunk->get("server").getString("127.0.0.1:2300");
    
    float sr = chunk->get("samplerate").getDouble(60.0);

    // Trace("RemoteGloveFactory::create # %f", sr);
    
    std::string script = chunk->get("launcher").getString();

    SocketDataSource<GloveSample> *source = 
      new SocketDataSource<GloveSample>(server, script, bufferSize, sr);
    Glove * g = new Glove(chunk, source);
    g->outputBuffer().setSampleRate(sr);    
    return g;
  }
};

RemoteGloveFactory factory;

extern "C" DeviceFactory* getFactory()
{
  return &factory;
}
