/* COPYRIGHT
 *
 * This file is part of FLUID
 *
 * Author: Tommi Ilmonen, 1999-2001.
 * Tommi.Ilmonen@hut.fi
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. 
 *    
 */

#include "mostar/WL/Mostar.h"

#include <iostream>
#include <string>

/* A simple program to reset the Ascension to a state that even VRJ
   can use. */

int main()
{
  using namespace std;
  
  std::string server("motion.tml.hut.fi");
  float sampleRate = 33.0;

  wlMostar mostar(server.c_str());

  //
  // CONNECT
  //
  cerr << endl << "Attempting to connect to motionstar..." << endl;
  if (mostar.ConnectTCP() != 0)
    {
      cerr << "FLUID # ERROR - Error while connecting to: " << server << endl;
      cerr << "FLUID # ERROR - ... " << mostar.ErrorString() << endl;
      return 0;
    }
  cerr << "...Connected." << endl;

  //
  // WAKEUP
  //
  if (mostar.Wakeup() != 0)
    {
      cerr << "FLUIF # ERROR - Error During Wakeup(). " 
	   << mostar.ErrorString() << endl;
      mostar.Disconnect();
      return 0;
    }
  //
  // GET GENERAL STATUS
  //
  wlMostarSys status;
  if (mostar.GetSystemStatus(status) != 0)
    {
      cerr << "FLUID # ERROR: Error During GetSystemStatus(). "
           << mostar.ErrorString() << endl;
      mostar.Shutdown();
      mostar.Disconnect();
      return 0;
    }

  //
  // SET GENERAL STATUS
  //
  status.SetMeasurementRate((int)(sampleRate*1000)); 
  if (mostar.SetSystemStatus(status) != 0)
    {
      cerr << "FLUID # ERROR: Error During SetSystemStatus(). "
           << mostar.ErrorString() << endl;     
      mostar.Shutdown();
      mostar.Disconnect();
      return 0;
    }

  //
  // VERIFY GENERAL STATUS
  //
  if (mostar.GetSystemStatus(status) != 0)
    {
      cerr << "ERROR: During GetSystemStatus(). "
           << mostar.ErrorString() << endl;
      mostar.Shutdown();
      mostar.Disconnect();
      return 0;
    }
  status.Print(cerr);

  //
  // GET/SET DEVICE STATUS
  //
  wlMostarDev dev_status;
  for (int i = 1; i <= status.FlockCount(); i++)
    {
      if (mostar.GetDeviceStatus(dev_status, i) != 0)
        {
          cerr << "FLUID # ERROR: During GetDeviceStatus(). "
               << mostar.ErrorString() << endl;
          exit(1);
        }
      dev_status.SetHemisphere(wlMostar::FRONT_HEMISPHERE);
      //dev_status.SetDataFormat(wlMostar::FORMAT_POS_QUAT);
      // dev_status.SetDataFormat(wlMostar::FORMAT_POS_MAT);
      dev_status.SetDataFormat(wlMostar::FORMAT_POS_ANG);
      // dev_status.SetDataFormat(wlMostar::FORMAT_POS);
      dev_status.SetReportRate(1);
      if (mostar.SetDeviceStatus(dev_status, i) != 0)
        {
          cerr << "FLUID # ERROR: During SetDeviceStatus(). "
               << mostar.ErrorString() << endl;
	  return 0;
        }
    }

  //
  // VERIFY DEVICE STATUS
  //
  for (int i = 1; i <= status.FlockCount(); i++)
    {
      if (mostar.GetDeviceStatus(dev_status, i) != 0)
        {
          cerr << "FLUID # ERROR: During GetDeviceStatus(). "
               << mostar.ErrorString() << endl;
          return 0;
        }
      dev_status.Print(cerr);
    }

  //
  // RUN CONTINUOUS
  //
  if (mostar.RunContinuous() != 0)
    {
      cerr << "FLUID # ERROR: During RunContinuous(). "
           << mostar.ErrorString() << endl;
      mostar.Shutdown();
      mostar.Disconnect();
      return 0;
    }
  cerr << "FLUID # MoStarTracker is cleaned" << endl;
}
