
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/param.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <errno.h>

#include <WL/Mostar.h>

wlMostar::wlMostar(const char *cpAddress)
{
  seq_num = 0;
  socket_id = -1;
  hostname = strdup(cpAddress);
  connected = 0;
  error_code = 0;
}

wlMostar::~wlMostar()
{
  Disconnect();
  delete [] hostname;
}

int
wlMostar::ConnectTCP()
{
  error_code = ERR_NONE;

  if (connected)
    Disconnect();

  struct hostent *hp = gethostbyname(hostname);
  if (!hp)
    return error_code = ERR_BAD_SERVER;

  // Create a socket
  //
  socket_id = socket(AF_INET, SOCK_STREAM, 0);
  if (socket_id < 0)
    return error_code = ERR_BAD_SOCKET;

  // Connect to socket
  //
  memset(&server, 0, sizeof(struct sockaddr_in));
  server.sin_family = AF_INET;
  memcpy(&(server.sin_addr.s_addr), hp->h_addr, hp->h_length);
  server.sin_port = htons(6000);
  if (connect(socket_id, (const sockaddr*) &server, sizeof(server)) < 0)
    {
      close(socket_id); socket_id = -1;
      return error_code = ERR_BAD_TCP_CONNECT;
    }

  usingUDP = 0;
  return error_code;
}

int
wlMostar::ConnectUDP()
{
  struct sockaddr_in client;
  char localhostname[MAXHOSTNAMELEN];
  struct hostent *localhost_hp;
  char *localhostaddr;
  struct linger lin = {0, 0};
  int one = 1;
  error_code = ERR_NONE;
  extern int errno;
  int tryPort = IPPORT_RESERVED + 1;
  int portOk = 0;

  if (connected)
    Disconnect();
  
  // Get local lost IP addr
  if (gethostname(localhostname, MAXHOSTNAMELEN) < 0)
    return ERR_BAD_CLIENT;
  localhost_hp = gethostbyname(localhostname);
  if (localhost_hp == NULL)
    return ERR_BAD_CLIENT;
  localhostaddr = inet_ntoa(*(struct in_addr *) localhost_hp->h_addr);
  
  struct hostent *hp = gethostbyname(hostname);
  if (!hp)
    return error_code = ERR_BAD_SERVER;
  
  // Create a socket
  //
  socket_id = socket(AF_INET, SOCK_DGRAM, 0);
  if (socket_id < 0)
    return error_code = ERR_BAD_SOCKET;
  if (setsockopt(socket_id, SOL_SOCKET, SO_REUSEADDR, &one, sizeof(one)) == -1)
    return error_code = ERR_BAD_SOCKET;
  if (setsockopt(socket_id, SOL_SOCKET, SO_LINGER, &lin, sizeof(lin)) == -1)
    return error_code = ERR_BAD_SOCKET;

  // Create client socket, init server structs
  //
  memset(&client, 0, sizeof(struct sockaddr_in));
  client.sin_family = AF_INET;
#if 0
  client.sin_port = htons(4000);
#endif
  client.sin_addr.s_addr=inet_addr(localhostaddr);

#if 0
  if (bind(socket_id, (struct sockaddr *) &client, sizeof (client)) < 0)
    return error_code = ERR_BAD_SOCKET;
#endif

  do {
    client.sin_port = htons(tryPort);
    if (bind(socket_id, (struct sockaddr *) &client, sizeof (client)) < 0)
      {
	if ((errno == EADDRINUSE || errno == EADDRNOTAVAIL))
	  tryPort++;
	else
	  return error_code = ERR_BAD_SOCKET;
      }
    else
      portOk = 1;
  } while (!portOk);

  memset(&server, 0, sizeof(struct sockaddr_in));
  server.sin_family = AF_INET;
  memcpy(&(server.sin_addr.s_addr), hp->h_addr, hp->h_length);
  server.sin_port = htons(5000);

  usingUDP = 1;

  return error_code;
}

int 
wlMostar::Disconnect()
{
  error_code = ERR_NONE;

  shutdown(socket_id, 2);
  close(socket_id); socket_id = -1;
  connected = 0;

  return error_code;
}

int 
wlMostar::Wakeup(short seq_prime)
{
  error_code = ERR_NONE;
  seq_num = seq_prime;

  PacketHeader message(seq_num, WAKEUP);

  if (!Send(socket_id, message))
    return error_code = ERR_BAD_SEND;

  PacketHeader response;
  if (!Recv(socket_id, response))
    return error_code = ERR_BAD_RECEIVE;

  if (response.Type() != R_WAKEUP)
    {
      cerr << "WAKEUP RESPONSE TYPE = " << (int)response.Type() << endl;
      return error_code = ERR_BAD_RESPONSE;
    }

  return error_code = response.ErrorCode();
}

int 
wlMostar::Shutdown()
{
  error_code = ERR_NONE;

  PacketHeader message(seq_num++, SHUTDOWN);
    
  if (!Send(socket_id, message))
    return error_code = ERR_BAD_SEND;

#if 0
  /* Ascension should reply. It does not. */
  PacketHeader response;
  if (!Recv(socket_id, response))
    return error_code = ERR_BAD_RECEIVE;

  if (response.Type() != R_SHUTDOWN)
    return error_code = ERR_BAD_RESPONSE;

  return error_code = response.ErrorCode();
#endif

  return error_code;
}

int
wlMostar::GetSystemStatus(wlMostarSys& status)
{
  error_code = ERR_NONE;

  PacketHeader message(seq_num++, GET_STATUS, 0);

  if (!Send(socket_id, message))
    return error_code = ERR_BAD_SEND;
  PacketHeader response;
  if (!Recv(socket_id, response))
    return error_code = ERR_BAD_RECEIVE;
  if (response.Type() != R_GET_STATUS)
    return error_code = ERR_BAD_RESPONSE;

  if (!RecvData())
    return error_code = ERR_BAD_DATA;

  status.FromRaw(RecvData());

  return error_code = response.ErrorCode();
}

int
wlMostar::SetSystemStatus(wlMostarSys& status)
{
  error_code = ERR_NONE;

  PacketHeader message(seq_num++, SEND_SETUP, 0);

  unsigned char raw[16];
  status.ToRaw(raw);
  if (!Send(socket_id, message, 16, raw))
    return error_code = ERR_BAD_SEND;

  PacketHeader response;
  if (!Recv(socket_id, response))
    return error_code = ERR_BAD_RECEIVE;

  if (response.Type() != R_SEND_SETUP)
    return error_code = ERR_BAD_RESPONSE;

  return error_code = response.ErrorCode();
}

int
wlMostar::GetDeviceStatus(wlMostarDev& status, int addr)
{
  error_code = ERR_NONE;

  PacketHeader message(seq_num++, GET_STATUS, addr);

  if (!Send(socket_id, message))
    return error_code = ERR_BAD_SEND;

  PacketHeader response;
  if (!Recv(socket_id, response))
    return error_code = ERR_BAD_RECEIVE;

  if (response.Type() != R_GET_STATUS)
    return error_code = ERR_BAD_RESPONSE;

  if (!RecvData())
    return error_code = ERR_BAD_DATA;

  status.FromRaw(RecvData());

  return error_code = response.ErrorCode();
}

int
wlMostar::SetDeviceStatus(wlMostarDev& status, int addr)
{
  error_code = ERR_NONE;

  PacketHeader message(seq_num++, SEND_SETUP, addr);

  unsigned char raw[70];
  status.ToRaw(raw);

  if (!Send(socket_id, message, 70, raw))
    return error_code = ERR_BAD_SEND;

  PacketHeader response;
  if (!Recv(socket_id, response))
    return error_code = ERR_BAD_RECEIVE;

  if (response.Type() != R_SEND_SETUP)
    return error_code = ERR_BAD_RESPONSE;

  return error_code = response.ErrorCode();
}

int
wlMostar::RunContinuous()
{
  error_code = ERR_NONE;

  PacketHeader message(seq_num++, RUN_CONTINUOUS);

  if (!Send(socket_id, message))
    return error_code = ERR_BAD_SEND;

  PacketHeader response;
  if (!Recv(socket_id, response))
    return error_code = ERR_BAD_RECEIVE;

  if (response.Type() != R_RUN_CONTINUOUS)
    return error_code = ERR_BAD_RESPONSE;

  return error_code = response.ErrorCode();
}

int
wlMostar::SingleShot()
{
  error_code = ERR_NONE;

  PacketHeader message(seq_num++, SINGLE_SHOT);
    
  if (!Send(socket_id, message))
    return error_code = ERR_BAD_SEND;

  PacketHeader response;
  if (!Recv(socket_id, response))
    return error_code = ERR_BAD_RECEIVE;

  if (response.Type() != R_SINGLE_SHOT)
    return error_code = ERR_BAD_RESPONSE;

  return error_code = response.ErrorCode();
}

int
wlMostar::StopData()
{
  error_code = ERR_NONE;

  PacketHeader message(seq_num++, STOP_DATA);
    
  if (!Send(socket_id, message))
    return error_code = ERR_BAD_SEND;

  PacketHeader response;

  int cnt = 0;
  do
    {
      cnt++;
      if (!Recv(socket_id, response))
	return error_code = ERR_BAD_RECEIVE;
    } while (response.Type() == DATA_PACKET);

  if (cnt > 1)
    cerr << "Skipped " << cnt-1 << " packets." << endl;

  if (response.Type() != R_STOP_DATA)
    return error_code = ERR_BAD_RESPONSE;

  return error_code = response.ErrorCode();
}

int
wlMostar::GetDataPacket(wlMostarPack& packet)
{
  error_code = ERR_NONE;

  PacketHeader header;
  int status = Recv(socket_id, header);

  /*
    if (!Recv(socket_id, header))
    return error_code = ERR_BAD_RECEIVE;
    */

  if (status)
    {
      if (header.Type() != DATA_PACKET)
	{
	  cerr << "Bad type = " << (int) header.Type() << endl;
	  return error_code = ERR_BAD_RESPONSE;
	}

      if (header.NumDataBytes() > 0 && !RecvData())
	return error_code = ERR_BAD_DATA;
    }

  seq_num = header.Sequence();
  packet.FromRaw(header, RecvData());

  PacketHeader message(seq_num, R_DATA_PACKET);
  if (!Send(socket_id, message))
    return error_code = ERR_BAD_SEND;
    
  return error_code;
}

const char*
wlMostar::ErrorString()
{
  const char* hdr_err = StringFromErrorCode(error_code);
  if (hdr_err)
    return hdr_err;

  switch (error_code)
    {
    case ERR_BAD_SERVER:
      return "Failed to find MotionStar server name.";
    case ERR_BAD_SOCKET:
      return "Failed to create socket.";
    case ERR_BAD_TCP_CONNECT:
      return "Failed to create TCP connection.";
    case ERR_BAD_UDP_CONNECT:
      return "Failed to create UDP connection.";
    case ERR_BAD_SEND:
      return "Failed sending message to server.";
    case ERR_BAD_RECEIVE:
      return "Failed receiving message from server.";
    case ERR_BAD_RESPONSE:
      return "Received response of wrong type from server.";
    case ERR_BAD_DATA:
      return "Received bad data from server.";
    case ERR_BAD_CLIENT:
      return "Failed to get local host info.";
    }

  return 0;
}

// For the wlMostar::Test
void
print_matrix(float m[4][4], const char* ind)
{
    for (int i = 0; i < 4; i++)
    {
	fprintf(stderr, ind);
	for (int j = 0; j < 4; j++)
	    fprintf(stderr, "%6.2f    ", m[i][j]);
	fprintf(stderr, "\n");
    }
}

void 
wlMostar::Test(const char *cpAddress, const char *birdno)
{

  puts("NEW MOSTAR TEST");

  wlMostar mostar;

  //
  // CONNECT
  //
  cerr << endl << "Attempting to connect to motionstar..." << endl;
  // if (mostar.ConnectUDP() != 0)
  if (mostar.ConnectTCP() != 0)
    {
      cerr << "ERROR: While connecting. " << mostar.ErrorString() << endl;
      exit(1);
    }
  cerr << "...Connected." << endl;

  //
  // WAKEUP
  //
  if (mostar.Wakeup() != 0)
    {
      cerr << "ERROR: During Wakeup(). " << mostar.ErrorString() << endl;
      mostar.Disconnect();
      exit(1);
    }

  //
  // GET GENERAL STATUS
  //
  wlMostarSys status;
  if (mostar.GetSystemStatus(status) != 0)
    {
      cerr << "ERROR: During GetSystemStatus(). "
	   << mostar.ErrorString() << endl;
      mostar.Shutdown();
      mostar.Disconnect();
      exit(1);
    }

  //
  // SET GENERAL STATUS
  //
  // status.SetMeasurementRate(105800); // 100 Hz.
  // status.SetMeasurementRate(105800); // 100 Hz.
  status.SetMeasurementRate(68599); // 68.6 Hz
  if (mostar.SetSystemStatus(status) != 0)
    {
      cerr << "ERROR: During SetSystemStatus(). "
	   << mostar.ErrorString() << endl;	
      mostar.Shutdown();
      mostar.Disconnect();
      exit(1);
    }

  //
  // VERIFY GENERAL STATUS
  //
  if (mostar.GetSystemStatus(status) != 0)
    {
      cerr << "ERROR: During GetSystemStatus(). "
	   << mostar.ErrorString() << endl;
      mostar.Shutdown();
      mostar.Disconnect();
      exit(1);
    }
  status.Print(cerr);

  //
  // GET/SET DEVICE STATUS
  //
  wlMostarDev dev_status;
  for (int i = 1; i <= status.FlockCount(); i++)
    {
      if (mostar.GetDeviceStatus(dev_status, i) != 0)
	{
	  cerr << "ERROR: During GetDeviceStatus(). "
	       << mostar.ErrorString() << endl;
	  exit(1);
	}
      dev_status.SetHemisphere(wlMostar::FRONT_HEMISPHERE);
      //dev_status.SetDataFormat(wlMostar::FORMAT_POS_QUAT);
      dev_status.SetDataFormat(wlMostar::FORMAT_POS_MAT);
      //dev_status.SetDataFormat(wlMostar::FORMAT_POS);
      dev_status.SetReportRate(1);
      if (mostar.SetDeviceStatus(dev_status, i) != 0)
	{
	  cerr << "ERROR: During SetDeviceStatus(). "
	       << mostar.ErrorString() << endl;
	  exit(1);
	}
    }

  //
  // VERIFY DEVICE STATUS
  //
  for (int i = 1; i <= status.FlockCount(); i++)
    {
      if (mostar.GetDeviceStatus(dev_status, i) != 0)
	{
	  cerr << "ERROR: During GetDeviceStatus(). "
	       << mostar.ErrorString() << endl;
	  exit(1);
	}
      dev_status.Print(cerr);
    }

  //
  // RUN CONTINUOUS
  //
  cerr << "Run Continuous" << endl;
  if (mostar.RunContinuous() != 0)
    {
      cerr << "ERROR: During RunContinuous(). "
	   << mostar.ErrorString() << endl;
      mostar.Shutdown();
      mostar.Disconnect();
      exit(1);
    }
  cerr << "Running" << endl;

  //
  // RETRIEVE PACKETS
  //
  wlMostarPack packet;
  for (int j = 0; j <= 1500; j++)
    {
      if (mostar.GetDataPacket(packet) != 0)
	{
	  cerr << "ERROR: During GetDataPacket(). "
	       << mostar.ErrorString() << endl;
	  mostar.Shutdown();
	  mostar.Disconnect();
	  exit(1);
	}

      if (j % 100 == 0 || j == 1 || j == 2)
	{
	  cerr << "sample = " << j << " seq = " << packet.Sequence()
	       << " time = " << packet.Time() << endl;
	}

      if (j % 200 == 0)
	{
	  wlMostarRec record;
	  int k = atoi(birdno);
	  //for (k = 0; k < 1; k++)
	  //{
	      packet.GetRecord(k, record);
	      float m[4][4];
	      record.GetXform(m);
	      print_matrix(m, "    ");
	      cerr << endl;
	      //}
	}
    }

  //
  // SHUTDOWN
  //
  cerr << "Stopping..." << endl;
  if (mostar.StopData() != 0)
    {
      cerr << "ERROR: During StopData(). " << mostar.ErrorString() << endl;
      exit(1);
    }

  cerr << "Shutdown..." << endl;
  if (mostar.Shutdown() != 0)
    {
      cerr << "ERROR: During Shutdown(). " << mostar.ErrorString() << endl;
      exit(1);
    }

  cerr << "Disconnecting..." << endl;
  if (mostar.Disconnect() != 0)
    {
      cerr << "ERROR: During Disconnect(). " << mostar.ErrorString() << endl;
      exit(1);
    }

  cerr << "Disconnected." << endl;
}

