
#include <string.h>
#include <unistd.h>
#include <strings.h>

#include <sys/time.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/ioctl.h>

#include <WL/MostarDefs.h>

unsigned char wlMostarDefs::databuf[PACKET_HEADER_SIZE + MAX_DATA_SIZE];
unsigned int wlMostarDefs::databuf_count = 0;

int wlMostarDefs::usingUDP = 0;
struct sockaddr_in wlMostarDefs::server;

inline static float
convert_float(unsigned char buf[2])
{
/*
    short a, b, c;
    a = buf[0] & 0x7f; b = buf[1] & 0x7f;
    c = ((a << 1) | (b << 8)) << 1;
    return (float)c/32768.0;
*/

    unsigned int xraw = (buf[0] << 8) | buf[1];
    int	xint = (int) xraw;
    if (xint > 32767)
	xint = xint - 65536;
    double xreal = (double)(xint) / 32768;
    return (float) xreal;
}

void
wlMostarDefs::CvtAng(float ang[3], unsigned char data[6])
{
    ang[0] = convert_float(&data[0]);
    ang[1] = convert_float(&data[2]);
    ang[2] = convert_float(&data[4]);
}

void
wlMostarDefs::CvtMat(float mat[3][3], unsigned char data[18])
{
    mat[0][0] = convert_float(&data[0]);
    mat[1][0] = convert_float(&data[2]);
    mat[2][0] = convert_float(&data[4]);
    mat[0][1] = convert_float(&data[6]);
    mat[1][1] = convert_float(&data[8]);
    mat[2][1] = convert_float(&data[10]);
    mat[0][2] = convert_float(&data[12]);
    mat[1][2] = convert_float(&data[14]);
    mat[2][2] = convert_float(&data[16]);
}

void
wlMostarDefs::CvtPos(float pos[3], unsigned char data[6], float factor)
{
    pos[0] = factor*convert_float(&data[0]);
    pos[1] = factor*convert_float(&data[2]);
    pos[2] = factor*convert_float(&data[4]);
}

void
wlMostarDefs::CvtPosAng(float pos[3], float ang[3], unsigned char data[12],
		      float factor)
{
    pos[0] = factor*convert_float(&data[0]);
    pos[1] = factor*convert_float(&data[2]);
    pos[2] = factor*convert_float(&data[4]);

    ang[0] = convert_float(&data[6]);
    ang[1] = convert_float(&data[8]);
    ang[2] = convert_float(&data[10]);
}

void
wlMostarDefs::CvtPosMat(float pos[3], float mat[3][3], unsigned char data[24],
		      float factor)
{
    pos[0] = factor*convert_float(&data[0]);
    pos[1] = factor*convert_float(&data[2]);
    pos[2] = factor*convert_float(&data[4]);

    mat[0][0] = convert_float(&data[6]);
    mat[1][0] = convert_float(&data[8]);
    mat[2][0] = convert_float(&data[10]);
    mat[0][1] = convert_float(&data[12]);
    mat[1][1] = convert_float(&data[14]);
    mat[2][1] = convert_float(&data[16]);
    mat[0][2] = convert_float(&data[18]);
    mat[1][2] = convert_float(&data[20]);
    mat[2][2] = convert_float(&data[22]);
}

void
wlMostarDefs::CvtPosQuat(float pos[3], float q[4], unsigned char data[14],
		       float factor)
{
    pos[0] = factor*convert_float(&data[0]);
    pos[1] = factor*convert_float(&data[2]);
    pos[2] = factor*convert_float(&data[4]);

    q[0] = convert_float(&data[6]);
    q[1] = convert_float(&data[8]);
    q[2] = convert_float(&data[10]);
    q[3] = convert_float(&data[12]);
}

void
wlMostarDefs::CvtQuat(float q[4], unsigned char data[8])
{
    q[0] = convert_float(&data[0]);
    q[1] = convert_float(&data[2]);
    q[2] = convert_float(&data[4]);
    q[3] = convert_float(&data[6]);
}

int
wlMostarDefs::Send(int socket_id, PacketHeader& hdr, int n,
		   unsigned char* data)
{
    databuf_count = 0;

    hdr.SetNumDataBytes(n);
    hdr.ToRaw(databuf);
    if (n > 0 && data != 0)
	memcpy(&databuf[PACKET_HEADER_SIZE], data, n);

    int total = PACKET_HEADER_SIZE + n;
    int send_n;
    if (usingUDP)
      {
	send_n = sendto(socket_id, databuf, total, 0,
			(const sockaddr*) &server, sizeof(server));
      }
    else
      send_n = send(socket_id, databuf, total, 0);

    if (send_n < total)
	return 0;
    
    return 1;
}

static struct timeval recv_timeout = { 20L, 0L};

inline int 
wlMostarDefs::recvall(int s, unsigned char *buf, int len)
{
  int total = 0, n, stat;
  if (usingUDP)
    {
      int fromlen;
      fromlen = sizeof(server);
      n = recvfrom(s, &buf[total], len, 0,
		   (sockaddr*) &server, (socklen_t*) &fromlen);
      total += n;
    }
  else
    {
      fd_set select_set;
      FD_ZERO(&select_set);
      FD_SET(s, &select_set);
      do
	{
	  stat = select(FD_SETSIZE, &select_set, 0, 0, &recv_timeout);
	  if (stat < 1)
	    {
	      cerr << "Total " << total << endl;
	      return stat;
	    }
	  n = recv(s, &buf[total], len-total, 0);
	  total += n;
	} while (total < len);
      
    }
    return 1;
}

int
wlMostarDefs::Recv(int socket_id, PacketHeader& hdr)
{
    databuf_count = 0;

    if (usingUDP)
      {
	int stat = recvall(socket_id, databuf, PACKET_HEADER_SIZE + MAX_DATA_SIZE);
	if (stat < 1)
	  {
	    cerr << "Recv() UDP error." << endl;
	    return 0;
	  }
	hdr.FromRaw(databuf);
	databuf_count = hdr.NumDataBytes();
	if (databuf_count < 1)
	  return 1;
      }
    else
      {
	int stat = recvall(socket_id, databuf, PACKET_HEADER_SIZE);
	if (stat < 1)
	  {
	    if (stat == 0)
	      cerr << "Recv() header timed-out." << endl;
	    else 
	      cerr << "Recv() header error." << endl;
	    return 0;
	  }
	
	hdr.FromRaw(databuf);
	databuf_count = hdr.NumDataBytes();
	
	if (databuf_count < 1)
	  return 1;

	printf("wlMostarDefs::Recv # Reading %d extra bytes\n", 
	       (int) databuf_count);
	fflush(0);

	stat = recvall(socket_id, &databuf[PACKET_HEADER_SIZE], databuf_count);
	if (stat < 1)
	  {
	    if (stat == 0)
	      cerr << "Recv() data timed-out." << endl;
	    else 
	      cerr << "Recv() data error." << endl;
	    return 0;
	  }
      }

    return 1;
}

unsigned char*
wlMostarDefs::RecvData()
{
    return (databuf_count > 0) ? &databuf[PACKET_HEADER_SIZE] : 0;
}

const char*
wlMostarDefs::StringFromErrorCode(int errcode)
{
    switch (errcode)
    {
    case ERR_ONE_MISSING_PACK:
	return "Single packet missing.";
    case ERR_FEW_MISSING_PACK:
	return "Several packets missing.";
    case ERR_MANY_MISSING_PACK:
	return "Many packets missing.";
    case ERR_REPEAT_SEQ_NUM:
	return "Sequence number recently repeated.";
    case ERR_UNEXPECTED_PACK:
	return "Unexpected packet received.";
    case ERR_BAD_PACK:
	return "Bad or corrupt packet received.";
    case ERR_BAD_STATUS:
	return "Illegal status requested.";
    case ERR_BAD_SETUP:
	return "Illegal setup attepted.";
    case ERR_SYSTEM_NOT_READY:
	return "System not ready.";
    default:
	return 0;
    }
}

int
wlMostarDefs::HemisphereFromString(const char* string)
{
    if (strcasecmp("FRONT", string) == 0)
	return FRONT_HEMISPHERE;
    if (strcasecmp("REAR", string) == 0)
	return REAR_HEMISPHERE;
    if (strcasecmp("UPPER", string) == 0)
	return UPPER_HEMISPHERE;
    if (strcasecmp("LOWER", string) == 0)
	return LOWER_HEMISPHERE;
    if (strcasecmp("LEFT", string) == 0)
	return LEFT_HEMISPHERE;
    if (strcasecmp("RIGHT", string) == 0)
	return RIGHT_HEMISPHERE;

    // Error Condition
    return FRONT_HEMISPHERE;
}

const char*
wlMostarDefs::StringFromHemisphere(int hemi)
{
    switch (hemi)
    {
    case FRONT_HEMISPHERE:
	return "FRONT";
    case REAR_HEMISPHERE:
	return "REAR";
    case UPPER_HEMISPHERE:
	return "UPPER";
    case LOWER_HEMISPHERE:
	return "LOWER";
    case LEFT_HEMISPHERE:
	return "LEFT";
    case RIGHT_HEMISPHERE:
	return "RIGHT";
    default:
	return "UNKNOWN";
    }
}

int
wlMostarDefs::DataFormatFromString(const char* string)
{
    if (strcasecmp("NONE", string) == 0)
	return FORMAT_NONE;
    if (strcasecmp("POS", string) == 0)
	return FORMAT_POS;
    if (strcasecmp("ANG", string) == 0)
	return FORMAT_ANG;
    if (strcasecmp("MAT", string) == 0)
	return FORMAT_MAT;
    if (strcasecmp("POS_ANG", string) == 0)
	return FORMAT_POS_ANG;
    if (strcasecmp("POS_MAT", string) == 0)
	return FORMAT_POS_MAT;
    if (strcasecmp("QUAT", string) == 0)
	return FORMAT_QUAT;
    if (strcasecmp("POS_QUAT", string) == 0)
	return FORMAT_POS_QUAT;

    // Error Condition
    return FORMAT_POS_ANG;
}

const char*
wlMostarDefs::StringFromDataFormat(int fmt)
{
    switch (fmt)
    {
    case FORMAT_NONE:
	return "NONE";
    case FORMAT_POS:
	return "POS";
    case FORMAT_ANG:
	return "ANG";
    case FORMAT_MAT:
	return "MAT";
    case FORMAT_POS_ANG:
	return "POS_ANG";
    case FORMAT_POS_MAT:
	return "POS_MAT";
    case FORMAT_QUAT:
	return "QUAT";
    case FORMAT_POS_QUAT:
	return "POS_QUAT";
    default:
	return "UNKNOWN";
    }
}


//-----------------------------------------------------------------------------
// wlMostarDefs::PacketHeader

wlMostarDefs::PacketHeader::PacketHeader(unsigned short seq, unsigned char t,
					 unsigned char xt)
{
    sequence = seq;
    time = 0; 
    milleseconds = 0;
    type = t;
    xtype = xt;
    protocol = 1;
    error_code = 0;
    error_code_ext = 0;
    num_data_bytes = 0;
}

wlMostarDefs::PacketHeader::PacketHeader(unsigned char raw[])
{
    memcpy(&sequence, &raw[0], 2);
    memcpy(&time, &raw[2], 4);
    memcpy(&milleseconds, &raw[6], 2);
    memcpy(&type, &raw[8], 1);
    memcpy(&xtype, &raw[9], 1);
    memcpy(&protocol, &raw[10], 1);
    memcpy(&error_code, &raw[11], 1);
    memcpy(&error_code_ext, &raw[12], 2);
    memcpy(&num_data_bytes, &raw[14], 2);
}

void
wlMostarDefs::PacketHeader::FromRaw(unsigned char raw[])
{
    memcpy(&sequence, &raw[0], 2);
    memcpy(&time, &raw[2], 4);
    memcpy(&milleseconds, &raw[6], 2);
    memcpy(&type, &raw[8], 1);
    memcpy(&xtype, &raw[9], 1);
    memcpy(&protocol, &raw[10], 1);
    memcpy(&error_code, &raw[11], 1);
    memcpy(&error_code_ext, &raw[12], 2);
    memcpy(&num_data_bytes, &raw[14], 2);
}

void
wlMostarDefs::PacketHeader::ToRaw(unsigned char raw[])
{
    memcpy(&raw[0], &sequence, 2);
    memcpy(&raw[2], &time, 4);
    memcpy(&raw[6], &milleseconds, 2);
    memcpy(&raw[8], &type, 1);
    memcpy(&raw[9], &xtype, 1);
    memcpy(&raw[10], &protocol, 1);
    memcpy(&raw[11], &error_code, 1);
    memcpy(&raw[12], &error_code_ext, 2);
    memcpy(&raw[14], &num_data_bytes, 2);    
}

int 
wlMostarDefs::PacketHeader::Verify()
{
    switch (type)
    {
    case WAKEUP:
    case SHUTDOWN:
    case GET_STATUS:
    case SEND_SETUP:
    case SINGLE_SHOT:
    case RUN_CONTINUOUS:
    case STOP_DATA:
    case DATA_PACKET:
    case R_WAKEUP:
    case R_SHUTDOWN:
    case R_GET_STATUS:
    case R_SEND_SETUP:
    case R_SINGLE_SHOT:
    case R_RUN_CONTINUOUS:
    case R_STOP_DATA:
    case R_DATA_PACKET:
    case R_ILLEGAL:
    case R_UNKNOWN:
	break;
    default:
	cerr << "BAD TYPE" << endl;
	return 0;
    }

    if (protocol != 1)
    {
	cerr << "BAD PROTOCOL" << endl;
	return 0;
    }

    return 1;
}



