
#include <WL/MostarDev.h>

wlMostarDev::wlMostarDev()
{
    status = 0;
    id = 0;
    soft_rev = 0;
    error_code = 0;
    setup = 0;
    data_format = 0;
    report_rate = 0;
    scaling = 0;
    hemisphere = 0;
    fbb_addr = 0;
    for (int i = 0; i < 7; i++)
	alpha_min[i] = alpha_max[i] = vm[i] = 0;
    for (int i = 0; i < 3; i++)
	xyz_ref_frame[i] = xyz_ang_align[i] = 0;
}

wlMostarDev::wlMostarDev(unsigned char* raw)
{
    memcpy(&status, &raw[0], 1);
    memcpy(&id, &raw[1], 1);
    memcpy(&soft_rev, &raw[2], 2);
    memcpy(&error_code, &raw[4], 1);
    memcpy(&setup, &raw[5], 1);
    memcpy(&data_format, &raw[6], 1);
    memcpy(&report_rate, &raw[7], 1);
    memcpy(&scaling, &raw[8], 2);
    memcpy(&hemisphere, &raw[10], 1);
    memcpy(&fbb_addr, &raw[11], 1);
    memcpy(alpha_min, &raw[16], 14);
    memcpy(alpha_max, &raw[30], 14);
    memcpy(vm, &raw[44], 14);
    memcpy(xyz_ref_frame, &raw[58], 6);
    memcpy(xyz_ang_align, &raw[64], 6);
}

void
wlMostarDev::FromRaw(unsigned char* raw)
{
    memcpy(&status, &raw[0], 1);
    memcpy(&id, &raw[1], 1);
    memcpy(&soft_rev, &raw[2], 2);
    memcpy(&error_code, &raw[4], 1);
    memcpy(&setup, &raw[5], 1);
    memcpy(&data_format, &raw[6], 1);
    memcpy(&report_rate, &raw[7], 1);
    memcpy(&scaling, &raw[8], 2);
    memcpy(&hemisphere, &raw[10], 1);
    memcpy(&fbb_addr, &raw[11], 1);

    memcpy(alpha_min, &raw[16], 14);
    memcpy(alpha_max, &raw[30], 14);
    memcpy(vm, &raw[44], 14);
    memcpy(xyz_ref_frame, &raw[58], 6);
    memcpy(xyz_ang_align, &raw[64], 6);
}

void
wlMostarDev::ToRaw(unsigned char* raw)
{
    memcpy(&raw[0], &status, 1);
    memcpy(&raw[1], &id, 1);
    memcpy(&raw[2], &soft_rev, 2);
    memcpy(&raw[4], &error_code, 1);
    memcpy(&raw[5], &setup, 1);
    memcpy(&raw[6], &data_format, 1);
    memcpy(&raw[7], &report_rate, 1);
    memcpy(&raw[8], &scaling, 2);
    memcpy(&raw[10], &hemisphere, 1);
    memcpy(&raw[11], &fbb_addr, 1);
    raw[12] = raw[13] = raw[14] = raw[15] = 0;
    memcpy(&raw[16], alpha_min, 14);
    memcpy(&raw[30], alpha_max, 14);
    memcpy(&raw[44], vm, 14);
    memcpy(&raw[58], xyz_ref_frame, 6);
    memcpy(&raw[64], xyz_ang_align, 6);
}

void
wlMostarDev::AlphaMinTable(unsigned short tbl[7])
{
    memcpy(tbl, alpha_min, sizeof(unsigned short)*7);
}

void
wlMostarDev::AlphaMaxTable(unsigned short tbl[7])
{
    memcpy(tbl, alpha_max, sizeof(unsigned short)*7);
}

void
wlMostarDev::VmTable(unsigned short tbl[7])
{
    memcpy(tbl, vm, sizeof(unsigned short)*7);
}

void
wlMostarDev::SetUseACNarrowFilter(int flag)
{
    if (flag)
	setup |= DEV_AC_NARROW;
    else
	setup &= ~(DEV_AC_NARROW);
}

void
wlMostarDev::SetUseACWideFilter(int flag)
{
    if (flag)
	setup |= DEV_AC_WIDE;
    else
	setup &= ~(DEV_AC_WIDE);
}

void
wlMostarDev::SetUseDCFilter(int flag)
{
    if (flag)
	setup |= DEV_DC_FILT;
    else
	setup &= ~(DEV_DC_FILT);
}

void
wlMostarDev::SetDataFormat(unsigned char fmt)
{
    unsigned char data_size;

    switch (fmt)
    {
    case FORMAT_POS:
    case FORMAT_ANG:
	data_size = 3;
	break;
    case FORMAT_MAT:
	data_size = 9;
	break;
    case FORMAT_POS_ANG:
	data_size = 6;
	break;
    case FORMAT_POS_MAT:
	data_size = 12;
	break;
    case FORMAT_QUAT:
	data_size = 4;
	break;
    case FORMAT_POS_QUAT:
	data_size = 7;
	break;
    case FORMAT_NONE:
    default:
	data_size = 0;
	break;
    }

    data_format = ((data_size << 4) & 0xF0) | (fmt & 0x0F);
}

void
wlMostarDev::SetScale(unsigned short s, unsigned char unit)
{
    scaling = ((unit << 12) & 0xF000) | (s & 0x0FFF);
}

void
wlMostarDev::SetHemisphere(unsigned char hemi)
{
    hemisphere = hemi;
}

void
wlMostarDev::Print(ostream& pout)
{
    pout << endl;
    pout << "Device Status (" << (int) FBBAddress() << ")" << endl;
    pout << "  Flags =";
        if (HasError()) pout << " ERROR";
        if (IsRunning()) pout << " RUNNING";
        if (HasButton()) pout << " BUTTON";
        if (HasReceiver()) pout << " RECEIVER";
	if (HasTransmitter()) pout << " TRANSMITTER("
	    << (HasRunningTransmitter() ? "ON" : "OFF") << ")";
        if (UseACNarrowFilter()) pout << " NARROW";
        if (UseACWideFilter()) pout << " WIDE";
	if (UseDCFilter()) pout << " DC";
    pout << endl;
    pout << "  ID = " << (int) ID() << endl;
    pout << "  Software Revision = " << SoftwareRevision() << endl;
    pout << "  Error Code = " << (int) ErrorCode() << endl;
    pout << "  Report Rate = " << (int) ReportRate() << endl;
    pout << "  Hemisphere = " << StringFromHemisphere(Hemisphere()) << endl;
    pout << "  Scale = " << Scale()
	<< ", Scale Unit = " << (int) ScaleUnit() << endl;
    pout << "  Data Format = " << StringFromDataFormat(DataFormat()) << endl;
    pout << "  Data Format Size = " << (int) DataFormatSize() << endl;
    pout << "  Alpha Min =";
        for (int i = 0; i < 7; i++)
	    pout << " " << FractionFromShort(alpha_min[i]);
    pout << endl;
    pout << "  Alpha Max =";
        for (int i = 0; i < 7; i++)
	    pout << " " << FractionFromShort(alpha_max[i]);
    pout << endl;
    pout << "  Vm =";
        for (int i = 0; i < 7; i++)
	    pout << " " << vm[i];
    pout << endl;
    pout << "  XYZ Reference Frame =";
        for (int i = 0; i < 3; i++)
	    pout << " " << xyz_ref_frame[i];
    pout << endl;
    pout << "  XYZ Angle Align =";
        for (int i = 0; i < 3; i++)
	    pout << " " << xyz_ang_align[i];
    pout << endl;
}




