#include <stdio.h>
#include <WL/MostarRec.h>

inline void q2m(float q[4], float m[3][3])
{
    float q0xq0 = q[0]*q[0], q1xq1 = q[1]*q[1];
    float q2xq2 = q[2]*q[2], q3xq3 = q[3]*q[3];

    m[0][0] = q0xq0 + q1xq1 - q2xq2 - q3xq3;
    m[0][1] = 2*(q[1]*q[2] - q[0]*q[3]);
    m[0][2] = 2*(q[1]*q[3] + q[0]*q[2]);
    m[1][0] = 2*(q[3]*q[0] + q[1]*q[2]);
    m[1][1] = q0xq0 - q1xq1 + q2xq2 - q3xq3;
    m[1][2] = 2*(q[2]*q[3] - q[1]*q[0]);
    m[2][0] = 2*(q[1]*q[3] - q[2]*q[0]);
    m[2][1] = 2*(q[1]*q[0] + q[3]*q[2]);
    m[2][2] = q0xq0 - q1xq1 - q2xq2 + q3xq3;
}

wlMostarRec::wlMostarRec()
{
    fbb_addr = 0;
    btn_flag = 0;
    data_size = 0;
    data_fmt = 0;
    memset(data, 0, 24);
    btn_data[0] = btn_data[1] = 0;
    good_data = 0;
}

wlMostarRec::wlMostarRec(unsigned char* raw)
{
    btn_flag = ((raw[0] & 0x80) == 0x80);
    fbb_addr = (raw[0] & 0x7f);
    data_size = 2*(raw[1] &  0x0F);
    data_fmt = (raw[1] &  0xF0) >> 4;
    memcpy(data, &raw[2], data_size);
    good_data = 1;
    if (data_fmt == 15)
    {
	good_data = 0;
	cerr << "PACKET DATA ERROR " << fbb_addr << " fmt=" << data_fmt
	    << " sz=" << data_size << endl;
	fprintf(stderr, "bytes = ");
	for (int i = 0; i < data_size; i++)
	    fprintf(stderr, "%x", data[i]);
	fprintf(stderr, "\n");
    }
}

void
wlMostarRec::FromRaw(unsigned char* raw)
{
    btn_flag = ((raw[0] & 0x80) == 0x80);
    fbb_addr = (raw[0] & 0x7f);
    data_size = 2*(raw[1] &  0x0F);
    data_fmt = (raw[1] &  0xF0) >> 4;
    memcpy(data, &raw[2], data_size);
    good_data = 1;
    if (data_fmt == 15)
    {
	good_data = 0;
	cerr << "PACKET DATA ERROR " << fbb_addr << " fmt=" << data_fmt
	    << " sz=" << data_size << endl;
	fprintf(stderr, "bytes = ");
	for (int i = 0; i < data_size; i++)
	    fprintf(stderr, "%x ", data[i]);
	fprintf(stderr, "\n");
    }
}

void
wlMostarRec::GetPos(float pos[3], float scaling)
{
    if (!GoodData())
    {
	pos[0] = pos[1] = pos[2] = 0.0;
	return;
    }

    switch (data_fmt)
    {
    case FORMAT_POS:
    case FORMAT_POS_ANG:
    case FORMAT_POS_MAT:
    case FORMAT_POS_QUAT:
	CvtPos(pos, data, scaling);
	break;
    case FORMAT_ANG:
    case FORMAT_MAT:
    case FORMAT_QUAT:
    case FORMAT_NONE:
    default:
	pos[0] = pos[1] = pos[2] = 0.0;
	break;
    }
}

void
wlMostarRec::GetAng(float ang[3])
{
    if (!GoodData())
    {
	ang[0] = ang[1] = ang[2] = 0.0;
	return;
    }

    switch (data_fmt)
    {
    case FORMAT_ANG:
	CvtAng(ang, data);
	break;
    case FORMAT_POS_ANG:
	CvtAng(ang, &data[6]);
	break;
    case FORMAT_MAT:
	// CONVERT POSSIBLE
	ang[0] = ang[1] = ang[2] = 0.0;
	break;
    case FORMAT_QUAT:
	// CONVERT POSSIBLE
	ang[0] = ang[1] = ang[2] = 0.0;
	break;
    case FORMAT_POS_MAT:
	// CONVERT POSSIBLE
	ang[0] = ang[1] = ang[2] = 0.0;
	break;
    case FORMAT_POS_QUAT:
	// CONVERT POSSIBLE
	ang[0] = ang[1] = ang[2] = 0.0;
	break;
    case FORMAT_POS:
    case FORMAT_NONE:
    default:
	ang[0] = ang[1] = ang[2] = 0.0;
	break;
    }
}

void
wlMostarRec::GetQuat(float quat[4])
{
    if (!GoodData())
    {
	quat[0] = quat[1] = quat[2] = quat[3] = 0.0;
	return;
    }

    switch (data_fmt)
    {
    case FORMAT_QUAT:
	CvtQuat(quat, data);
	break;
    case FORMAT_POS_QUAT:
	CvtQuat(quat, &data[6]);
	break;
    case FORMAT_POS_ANG:
	// CONVERT POSSIBLE
	quat[0] = quat[1] = quat[2] = quat[3] = 0.0;
	break;
    case FORMAT_POS_MAT:
	// CONVERT POSSIBLE
	quat[0] = quat[1] = quat[2] = quat[3] = 0.0;
	break;
    case FORMAT_ANG:
	// CONVERT POSSIBLE
	quat[0] = quat[1] = quat[2] = quat[3] = 0.0;
	break;
    case FORMAT_MAT:
	// CONVERT POSSIBLE
	quat[0] = quat[1] = quat[2] = quat[3] = 0.0;
	break;
    case FORMAT_POS:
    case FORMAT_NONE:
    default:
	quat[0] = quat[1] = quat[2] = quat[3] = 0.0;
	break;
    }
}

void
wlMostarRec::GetMat(float m[3][3])
{
    if (!GoodData())
    {
	m[0][0] = m[1][1] = m[2][2] = 1.0;
	m[0][1] = m[0][2] = m[1][0] = m[1][2] = m[2][0] = m[2][1];
	return;
    }
    
    float q[4];

    switch (data_fmt)
    {
    case FORMAT_MAT:
	CvtMat(m, data);
	break;
    case FORMAT_POS_MAT:
	CvtMat(m, &data[6]);
	break;
    case FORMAT_QUAT:
	CvtQuat(q, data);
	q2m(q, m);
	break;
    case FORMAT_POS_QUAT:
	CvtQuat(q, &data[6]);
	q2m(q, m);
	break;
    case FORMAT_ANG:
	m[0][0] = m[1][1] = m[2][2] = 1.0;
	m[0][1] = m[0][2] = m[1][0] = m[1][2] = m[2][0] = m[2][1];
	break;
    case FORMAT_POS_ANG:
	m[0][0] = m[1][1] = m[2][2] = 1.0;
	m[0][1] = m[0][2] = m[1][0] = m[1][2] = m[2][0] = m[2][1];
	break;
    case FORMAT_POS:
    case FORMAT_NONE:
    default:
	m[0][0] = m[1][1] = m[2][2] = 1.0;
	m[0][1] = m[0][2] = m[1][0] = m[1][2] = m[2][0] = m[2][1];
	break;
    }
}

void
wlMostarRec::GetXform(float mat[4][4], float scaling)
{
    float p[3], m[3][3];

    GetPos(p, scaling);
    GetMat(m);

    mat[0][0] = m[0][0]; mat[1][0] = m[1][0]; mat[2][0] = m[2][0];
    mat[0][1] = m[0][1]; mat[1][1] = m[1][1]; mat[2][1] = m[2][1];
    mat[0][2] = m[0][2]; mat[1][2] = m[1][2]; mat[2][2] = m[2][2];

    mat[3][0] = p[0]; mat[3][1] = p[1]; mat[3][2] = p[2];

    mat[0][3] = mat[1][3] = mat[2][3] = 0.0;
    mat[3][3] = 1.0;
}

void
wlMostarRec::GetRawData(float* d)
{
    if (data_size > 0)
	memcpy(d, data, data_size);
}

int
wlMostarRec::RawRecordSize(unsigned char* raw)
{
    int btn_data = ((raw[0] & 0x80) == 0x80);
    int fmt_sz = (raw[1] &  0x0F);
    int rec_sz = 1 + 1 + 2*fmt_sz;
    if (btn_data)
	rec_sz += 2;

    return rec_sz;
}


