
#include <stdlib.h>
#include <stdio.h>

#include <WL/MostarSys.h>

wlMostarSys::wlMostarSys()
{
    all = 0;
    fbb_error = 0;
    flock_count = 0;
    server_count = 0;
    xmitter_addr = 0;
    chassis_id = 0;
    chassis_count = 0;
    first_addr = 0;
}

wlMostarSys::wlMostarSys(unsigned char* raw)
{
    memcpy(&all, &raw[0], 1);
    memcpy(&fbb_error, &raw[1], 1);
    memcpy(&flock_count, &raw[2], 1);
    memcpy(&server_count, &raw[3], 1);
    unsigned char temp;
    memcpy(&temp, &raw[4], 1);
    xmitter_addr = (temp >> 4) & 0x0f;
    memcpy(measurement_rate, &raw[5], 6);
    memcpy(&chassis_id, &raw[11], 1);
    memcpy(&chassis_count, &raw[12], 1);
    memcpy(&first_addr, &raw[13], 1);
    memcpy(fbb_status, &raw[16], 120);    
}

void
wlMostarSys::FromRaw(unsigned char* raw)
{
    memcpy(&all, &raw[0], 1);
    memcpy(&fbb_error, &raw[1], 1);
    memcpy(&flock_count, &raw[2], 1);
    memcpy(&server_count, &raw[3], 1);
    unsigned char temp;
    memcpy(&temp, &raw[4], 1);
    xmitter_addr = (temp >> 4) & 0x0f;
    memcpy(measurement_rate, &raw[5], 6);
    memcpy(&chassis_id, &raw[11], 1);
    memcpy(&chassis_count, &raw[12], 1);
    memcpy(&first_addr, &raw[13], 1);
    memcpy(fbb_status, &raw[16], 120);
}

void
wlMostarSys::ToRaw(unsigned char* raw)
{
    memcpy(&raw[0], &all, 1);
    memcpy(&raw[1], &fbb_error, 1);
    memcpy(&raw[2], &flock_count, 1);
    memcpy(&raw[3], &server_count, 1);
    unsigned char temp;
    temp = (xmitter_addr << 4) & 0xf0;
    memcpy(&raw[4], &temp, 1);
    memcpy(&raw[5], measurement_rate, 6);
    memcpy(&raw[11], &chassis_id, 1);
    memcpy(&raw[12], &chassis_count, 1);
    memcpy(&raw[13], &first_addr, 1);
    raw[14] = raw[15] = 0;
    // NOTE - fbb status array not used in this direction
}

int
wlMostarSys::MeasurementRate()
{
    char temp[7];
    memcpy(temp, measurement_rate, 6); temp[6] = '\0';
    return atoi(temp);
}

void
wlMostarSys::SetMeasurementRate(int rate)
{
    if (rate < 0)
	rate = 0;
    else if (rate > 999999)
	rate = 999999;
    char temp[7];
    sprintf(temp, "%d", rate);
    memcpy(measurement_rate, temp, 6); temp[6] = '\0';
}

void
wlMostarSys::SetTransmitterAddress(unsigned char addr)
{
    xmitter_addr = addr;
}

int
wlMostarSys::DeviceIsAccessible(unsigned char which)
{
    which--;
    if (which < flock_count)
	return (fbb_status[which] & FBB_ACCESSIBLE) == FBB_ACCESSIBLE;
    return 0;
}

int
wlMostarSys::DeviceIsRunning(unsigned char which)
{
    which--;
    if (which < flock_count)
	return (fbb_status[which] & FBB_RUNNING) == FBB_RUNNING;
    return 0;
}

int
wlMostarSys::DeviceHasReceiver(unsigned char which)
{
    which--;
    if (which < flock_count)
	return (fbb_status[which] & FBB_HAS_RCVR) == FBB_HAS_RCVR;
    return 0;
}

int
wlMostarSys::DeviceIsAnERC(unsigned char which)
{
    which--;
    if (which < flock_count)
	return (fbb_status[which] & FBB_IS_ERC) == FBB_IS_ERC;
    return 0;
}

void
wlMostarSys::Print(ostream& pout)
{
    pout << endl;
    pout << "System Status" << endl;
    pout << "  Flags =";
	if (IsRunning()) pout << " RUNNING";
	if (HasError()) pout << " ERROR";
	if (HasFBBError()) pout << " FBB_ERROR";
	if (IsMaster()) pout << " MASTER";
	if (HasHostSync()) pout << " HOST_SYNC";
	if (HasCRTSync()) pout << " CRT_SYNC";
    pout << endl;
    pout << "  FBB Error = " << (int) FBBError() << endl;
    pout << "  Flock Count = " << (int) FlockCount() << endl;
    pout << "  Server Count = " << (int) ServerCount() << endl;
    pout << "  Transmitter Address = " << (int) TransmitterAddress() << endl;
    pout << "  Measurement Rate = ";
        for (int i = 0; i < 6; i++) pout << measurement_rate[i];
    pout << endl;
    pout << "  Chassis ID = " << (int) ChassisID() << endl;
    pout << "  Chassis Count = " << (int) ChassisCount() << endl;
    pout << "  First Address = " << (int) FirstAddress() << endl;
    for (int i = 1; i <= 120; i++)
    {
	if (DeviceIsAccessible(i))
	{
	    pout << "  Device[" << i << "] = ";
	    if (DeviceIsRunning(i)) pout << " RUNNING";
	    if (DeviceHasReceiver(i)) pout << " RECEIVER";
	    if (DeviceIsAnERC(i)) pout << " ERC";
	    pout << endl;
	}
    }
}
