/* -*- C++ -*- */
#ifndef __MOSTAR_H__
#define __MOSTAR_H__

#include <iostream>
using namespace std;
#include <unistd.h>
#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>

#include <WL/MostarDefs.h>
#include <WL/MostarSys.h>
#include <WL/MostarDev.h>
#include <WL/MostarPack.h>

/// Connection to Ascension Motionstar
/** {\large NOTE !!!} that many functions return value
  telling of their success. {\bf Zero = success !!}.
  
  For examples of how to use see the code of
  {\em wlMostar::Test}. */
class wlMostar : public wlMostarDefs
{
public:
  ///
  enum
  {
    ///
    ERR_BAD_SERVER = 	1000,
    ///
    ERR_BAD_SOCKET =	1001,
    ///
    ERR_BAD_TCP_CONNECT = 	1002,
    ///
    ERR_BAD_UDP_CONNECT =	1003,
    ///
    ERR_BAD_SEND =		1004,
    ///
    ERR_BAD_RECEIVE =	1005,
    ///
    ERR_BAD_RESPONSE = 	1006,
    ///
    ERR_BAD_DATA =		1007,
    ///
    ERR_BAD_CLIENT = 1008,
  };

private:
  short		     seq_num;
  int		     socket_id;
  char*		     hostname;
  int		     connected;
  int		     error_code;

public:		
  ///
  wlMostar(const char *cpAddress = "motion.tcm.hut.fi");
  ///
  ~wlMostar();
  ///
  int			ConnectTCP();
  ///
  int			ConnectUDP();
  ///
  int			Disconnect();
  ///
  int			Wakeup(short seq_prime = 0);
  ///
  int			Shutdown();
  ///
  int			GetSystemStatus(wlMostarSys&);
  ///
  int			SetSystemStatus(wlMostarSys&);
  ///
  int			GetDeviceStatus(wlMostarDev&, int addr);
  ///
  int			SetDeviceStatus(wlMostarDev&, int addr);
  ///
  int			RunContinuous();
  ///
  int			SingleShot();
  ///
  int			StopData();
  ///
  int			GetDataPacket(wlMostarPack&);
  ///
  int			ErrorCode() { return error_code; }
  ///
  const char*		ErrorString();
  /// Drive self-test.
  static void Test(const char *cpAddress = "130.233.45.15", const char *birdno = "0");
};




#endif
