/* -*- C++ -*- */

#ifndef __MOSTARDEFS_H__
#define __MOSTARDEFS_H__

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <iostream>

using namespace std;
/// Defines & constants for Asc. Motionstar.
class wlMostarDefs
{
public:
  ///
  enum
  {
    // Flock Hemisphere Codes
    ///
    FRONT_HEMISPHERE = 0,
    ///
    REAR_HEMISPHERE = 1,
    ///
    UPPER_HEMISPHERE = 2,
    ///
    LOWER_HEMISPHERE = 3,
    ///
    LEFT_HEMISPHERE = 4,
    ///
    RIGHT_HEMISPHERE = 5,
  };

  ///
  enum
  {
    // Data Format Codes (bits 0-3)
    ///
    FORMAT_NONE = 0,
    ///
    FORMAT_POS = 1,
    ///
    FORMAT_ANG = 2,
    ///
    FORMAT_MAT = 3,
    ///
    FORMAT_POS_ANG = 4,
    ///
    FORMAT_POS_MAT = 5,
    ///
    FORMAT_QUAT = 7,
    ///
    FORMAT_POS_QUAT = 8,
  };

  // Enumerations

  // Command/Response Codes (use in DataPacket::type field)
  ///
  enum	
  {
    ///
    WAKEUP = 10,
    ///
    R_WAKEUP = 20,
    ///
    SHUTDOWN = 11,
    ///
    R_SHUTDOWN = 21,
    ///
    GET_STATUS = 101,
    ///
    R_GET_STATUS = 201,
    ///
    SEND_SETUP = 102,
    ///
    R_SEND_SETUP = 202,
    ///
    SINGLE_SHOT = 103,	
    ///
    R_SINGLE_SHOT = 203,
    ///
    RUN_CONTINUOUS = 104,	
    ///
    R_RUN_CONTINUOUS = 204,
    ///
    STOP_DATA = 105,	
    ///
    R_STOP_DATA = 205,
    ///
    DATA_PACKET = 210,	
    ///
    R_DATA_PACKET = 211,
    ///
    R_ILLEGAL = 40,
    ///
    R_UNKNOWN = 50
  };

  ///
  enum
  {
    ///
    FLOCK_XYZREFERENCE = 1 << 12,
    // UNUSED = 1 << 11,
    ///
    FLOCK_ACNARROWNOTCHFILTER = 1 << 10,
    ///
    FLOCK_ACWIDENOTCHFILTER = 1 << 9,
    ///
    FLOCK_DCFILTER = 1 << 8,

    // Flock Scaling Codes  (bits 4-7)
    ///
    FLOCK_SCALE_036 = 0 << 4,
    ///
    FLOCK_SCALE_072 = 1 << 4,
    ///
    FLOCK_SCALE_144 = 2 << 4,



    // System Status Flags
    ///
    SYS_RUNNING = 1 << 7,
    ///
    SYS_ERROR = 1 << 6,
    ///
    SYS_FBB_ERROR = 1 << 5,
    ///
    SYS_UNDEFINED1 = 1 << 4,
    ///
    SYS_UNDEFINED2 = 1 << 3,
    ///
    SYS_MASTER = 1 << 2,
    ///
    SYS_HOSTSYNC = 1 << 1,
    ///
    SYS_CRTSYNC = 1 << 0,

    // FBB Status Flags
    ///
    FBB_ACCESSIBLE = 1 << 7,
    ///
    FBB_RUNNING = 1 << 6,
    ///
    FBB_HAS_RCVR = 1 << 5,
    ///
    FBB_IS_ERC = 1 << 4,
    ///
    FBB_ERT3 = 1 << 3,
    ///
    FBB_ERT2 = 1 << 2,
    ///
    FBB_ERT1 = 1 << 1,
    ///
    FBB_ERT0 = 1 << 0
  };

  ///
  enum
  {
    ///
    PACKET_HEADER_SIZE = 16,
    ///
    MAX_DATA_SIZE = 2048,
  };

  ///
  enum
  {
    ///
    ERR_NONE =		0,			

    // Packet Header Errors //
    ///
    ERR_ONE_MISSING_PACK = 	1,
    ///
    ERR_FEW_MISSING_PACK =	2,
    ///
    ERR_MANY_MISSING_PACK =     3,
    ///
    ERR_REPEAT_SEQ_NUM =	4,
    ///
    ERR_UNEXPECTED_PACK =	6,
    ///
    ERR_BAD_PACK =		7,
    ///
    ERR_BAD_STATUS = 	        8,
    ///
    ERR_BAD_SETUP =		9,
    ///
    ERR_SYSTEM_NOT_READY =	100,
  };

  // Structures
  /// Header of the packets.
  class PacketHeader
  {
    unsigned short	sequence;
    unsigned int	time;
    unsigned short	milleseconds;
    unsigned char	type;
    unsigned char	xtype;
    unsigned char	protocol;
    unsigned char	error_code;	
    unsigned short	error_code_ext;
    unsigned short	num_data_bytes;

  public:		
    ///
    PacketHeader(unsigned short seq=0,
		 unsigned char t=R_UNKNOWN,
		 unsigned char xt=0);
    ///
    PacketHeader(unsigned char raw[]);
    ///
    void		FromRaw(unsigned char raw[]);
    ///
    void		ToRaw(unsigned char raw[]);
    ///
    void		SetSequence(unsigned short seq) { sequence = seq; }
    ///
    unsigned short	Sequence() { return ntohs(sequence); }
    ///
    void		SetTime(int t) { time = htonl(t); }
    ///
    unsigned int	Time() { return ntohl(time); }
    ///
    void		SetMilleseconds(unsigned short ms) 
    { milleseconds = htonl(ms); }
    unsigned short	Milleseconds() { return ntohl(milleseconds); }
    ///
    void		SetType(unsigned char t, unsigned char xt=0)
    { type = t; xtype = xt; }
    ///
    unsigned char	Type() { return type; }
    ///
    unsigned char	XType() { return xtype; }
    ///
    unsigned char	Protocol() { return protocol; }
    ///
    unsigned char	ErrorCode() { return error_code; }
    /// 
    unsigned char	ErrorCodeExt() { return error_code_ext; }
    ///
    unsigned short 	NumDataBytes() { return ntohs(num_data_bytes); }
    ///
    void 	        SetNumDataBytes(unsigned short n) 
    { num_data_bytes = htons(n); }
    ///
    int		Verify();
    ///
    void		Print(ostream&);
  };
    
private:
  static unsigned char databuf[PACKET_HEADER_SIZE + MAX_DATA_SIZE];
  static unsigned int  databuf_count;
  static inline int recvall(int s, unsigned char *buf, int len);

protected:

  ///
  static short 	ShortFromFraction(float f);
  ///
  static float 	FractionFromShort(short s);
  ///
  static void  	CvtAng(float a[3], unsigned char data[6]);
  ///
  static void  	CvtMat(float m[3][3], unsigned char data[18]);
  ///
  static void  	CvtPos(float p[3], unsigned char data[6],float factor);
  ///
  static void  	CvtPosAng(float p[3], float a[3],
			  unsigned char data[24], float factor);
  ///
  static void  	CvtPosMat(float p[3], float m[3][3],
			  unsigned char data[24], float factor);
  static void  	CvtPosQuat(float p[3], float q[4],
			   unsigned char data[14], float factor);
  ///
  static void  	CvtQuat(float q[4], unsigned char data[8]);

  ///
  static int	 	Send(int, PacketHeader&, int n=0,
			     unsigned char* data=0);
  ///
  static int	 	Recv(int, PacketHeader&);
  ///
  static unsigned char* RecvData();

  ///
  static const char*	StringFromErrorCode(int errcode);

  ///
  static int		HemisphereFromString(const char*);
  ///
  static const char*	StringFromHemisphere(int);

  ///
  static int		DataFormatFromString(const char*);
  ///
  static const char*	StringFromDataFormat(int);


  static struct sockaddr_in server;
  static int                usingUDP;
};

inline short
wlMostarDefs::ShortFromFraction(float f)
{
  int i = (int)(f*32768.0);
  return (i > 32767) ? 32767 : (i < -32768) ? -32768 : (short) i;
}

inline float
wlMostarDefs::FractionFromShort(short s)
{
  return (s == 32767) ? 1.0 : ((float) s)/32768.0;
}

inline void
wlMostarDefs::PacketHeader::Print(ostream& pout)
{
  pout << "Packet Header" << endl;
  pout << "    Sequence " << sequence << endl;
  pout << "    Time " << time << endl;
  pout << "    Milleseconds " << milleseconds << endl;    
  pout << "    Type " << (int) type << endl;
  pout << "    XType " << (int) xtype << endl;
  pout << "    Protocol " << (int) protocol << endl;
  pout << "    Error " << (int) error_code << endl;
  pout << "    XError " << error_code_ext << endl;
  pout << "    Data " << num_data_bytes << endl;
}

#endif
