/* -*- C++ -*- */

#ifndef __MOSARTDEV_H__
#define __MOSARTDEV_H__

#include <WL/MostarDefs.h>

///
class wlMostarDev : public wlMostarDefs
{
  ///
  enum // status flags
  {
    ///
    DEV_ERROR = 1 << 7,
    ///
    DEV_RUNNING = 1 << 6,
    ///
    DEV_HAS_BTN = 1 << 3,
    ///
    DEV_HAS_RCVR = 1 << 2,
    ///
    DEV_HAS_XMTR = 1 << 1,
    ///
    DEV_XMTR_ON = 1 << 0
  };
  
  ///
  enum // setup flags
  {
    ///
    DEV_XYZ_REF = 1 << 4,
    ///
    DEV_HAS_BTN_DATA = 1 << 3,
    ///
    DEV_AC_NARROW = 1 << 2,
    ///
    DEV_AC_WIDE= 1 << 1,
    ///
    DEV_DC_FILT = 1 << 0
  };
  
  

  unsigned char	status;
  unsigned char	id;
  unsigned short	soft_rev;
  unsigned char	error_code;
  unsigned char	setup;
  unsigned char	data_format;
  unsigned char	report_rate;
  unsigned short	scaling;
  unsigned char	hemisphere;
  unsigned char	fbb_addr;
  unsigned short	alpha_min[7];
  unsigned short	alpha_max[7];
  unsigned short	vm[7];
  unsigned short	xyz_ref_frame[3];
  unsigned short	xyz_ang_align[3];

public:	
  ///		
  wlMostarDev();
  ///
  wlMostarDev(unsigned char* raw);
  ///	
  void		FromRaw(unsigned char* raw);
  ///	
  void		ToRaw(unsigned char* raw); 
  ///	   
  unsigned char	FBBAddress() { return fbb_addr; }
  ///	
  int			HasError()
  { return (status & DEV_ERROR) == DEV_ERROR; }
  ///	
  int			IsRunning()
  { return (status & DEV_RUNNING) == DEV_RUNNING; }
  ///	
  int			HasButton()
  { return (status & DEV_HAS_BTN) == DEV_HAS_BTN; }
  ///	
  int			HasReceiver()
  { return (status & DEV_HAS_RCVR) == DEV_HAS_RCVR; }
  ///	
  int			HasTransmitter()
  { return (status & DEV_HAS_XMTR) == DEV_HAS_XMTR; }
  ///	
  int			HasRunningTransmitter()
  { return (status & DEV_XMTR_ON) == DEV_XMTR_ON; }
  ///  
  int			UseACNarrowFilter()
  { return (setup & DEV_AC_NARROW)==DEV_AC_NARROW;}
  ///	
  int			UseACWideFilter()
  { return (setup & DEV_AC_WIDE) == DEV_AC_WIDE;}
  ///	
  int			UseDCFilter()
  { return (setup & DEV_DC_FILT) == DEV_DC_FILT;}
  ///	
  unsigned char	ID() { return id; }
  ///	
  unsigned short	SoftwareRevision() { return soft_rev; }
  ///	
  unsigned char	ErrorCode() { return error_code; }
  ///	
  unsigned char	ReportRate() { return report_rate; }
  ///	
  unsigned char	Hemisphere() { return hemisphere; }
  ///	
  unsigned char	DataFormat() { return data_format & 0x0F; }
  ///	
  unsigned char	DataFormatSize() 
  { return (data_format & 0xF0) >> 4; }
  ///	
  unsigned short	Scale() { return scaling & 0x0FFF; }
  ///	
  unsigned char	ScaleUnit() { return (scaling & 0xF000) >> 12; }
  ///	
  void		AlphaMinTable(unsigned short tbl[7]);
  ///	
  void		AlphaMaxTable(unsigned short tbl[7]);
  ///	
  void		VmTable(unsigned short tbl[7]);
  ///	
  void		SetUseACNarrowFilter(int flag);
  ///	
  void		SetUseACWideFilter(int flag);
  ///	
  void		SetUseDCFilter(int flag);
  ///	
  void		SetDataFormat(unsigned char fmt);
  ///	
  void		SetScale(unsigned short, unsigned char unit=0);
  ///	
  void		SetHemisphere(unsigned char hemi);
  ///	
  void                SetReportRate(unsigned char rate)
  { report_rate = rate; }
  ///	
  void		Print(ostream&);
};



#endif
