/* COPYRIGHT
 *
 * This file is part of FLUID
 *
 * Author: Tommi Ilmonen, 1999-2001.
 * Tommi.Ilmonen@hut.fi
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. 
 *    
 */
#include <Fluid/DeviceFactory.hpp>
#include <Fluid/Debug.hpp>
#include <Fluid/SocketDataSource.hpp>
#include <Fluid/InputDevices.hpp>
#include <Fluid/ConfigReader.hpp>
#include <Fluid/Mouse.hpp>

#include <stdlib.h>

using namespace Fluid;

class RemoteMouseFactory : public DeviceFactory
{
public:
  virtual std::string getDriverName() const { return "RemoteMouse"; } 
  virtual InputDeviceBase* create(Chunk* chunk)
  {
    std::string server;
    int bufferSize;

    bufferSize = chunk->get("bufferSize").getInt(50);
        
    server = chunk->get("server").getString("127.0.0.1:2300");
    
    std::string script = chunk->get("launcher").getString();

    SocketDataSource<MouseSample> *source = 
      new SocketDataSource<MouseSample>(server, script, bufferSize,
					chunk->get("samplerate").getDouble(50.0));
    return new Mouse(chunk,source);
  }
};

RemoteMouseFactory factory;

extern "C" DeviceFactory* getFactory()
{
  return &factory;
}
