/* COPYRIGHT
 *
 * This file is part of FLUID
 *
 * Author: Tommi Ilmonen, 1999-2001.
 * Tommi.Ilmonen@hut.fi
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. 
 *    
 */

#include <HelmaNapra.hpp>

#include <SignalCleaner.hpp>

#include <helma.h>

#include <strings.h>

using namespace Fluid;

SignalCleaner<float> foo;

NapraSource::NapraSource(const std::string& device, 
			 int bufferSize,
			 int offset,
			 int filtering)
  :DataSource<NapraSample>(bufferSize),
   m_device(device),
   m_offset(offset),
   m_filtering(filtering)
{
}

void NapraSource::start()
{
  bool ok = m_port.open(m_device.c_str(), true, false, 115200, 8, 1, 0);
  if(!ok) {
    errStream << "Could not open device: " << m_device.c_str() << '\n';
    return;
  }

  m_continue = true;
  run();
}

void NapraSource::stop()
{
  m_continue = false;

  waitEnd();
  
  m_port.close();
}

void NapraSource::childLoop()
{
  NapraSample sample;

  char byte;

  helma_state state;

  helma_init(&state);

  state.filtering = (HELMA_Filter) m_filtering;

  bool hasFirst = false;

  while(m_continue) {
    
    if(m_port.read(&byte, 1) == 1) {
    
      int r = helma_parse(byte, &state);

      if(r == 1)
	hasFirst = true;

      if(r == 2 && hasFirst) {
	
	for(int n=0; n < 2; n++) {
	  for(int i=0; i < 3; i++) {
	    int ind = n * 3 + i;
	    ind = (ind + 6 + m_offset) % 6;
	    sample.fingerTips[n][i].x = state.x[ind];
	    sample.fingerTips[n][i].y = state.y[ind];
	    sample.fingerTips[n][i].z = state.z[ind];
	  }
	  sample.switches[n] = state.switches[n];
	}

	write(sample);
      }

      // usleep(1000000/sampleRate);

    }
    else
      usleep(10000);
  }
}


NapraSourceFactory factory;

extern "C" DeviceFactory* getFactory()
{
  return &factory;
}

