/* COPYRIGHT
 *
 * This file is part of FLUID
 *
 * Author: Tommi Ilmonen, 1999-2001.
 * Tommi.Ilmonen@hut.fi
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. 
 *    
 */

#ifndef _HELMA_NAPRA_HPP
#define _HELMA_NAPRA_HPP

#include <Fluid/DataSource.hpp>
#include <Fluid/Config.hpp>
#include <Fluid/Debug.hpp>
#include <Fluid/DeviceFactory.hpp>
#include <Fluid/Napra.hpp>
#include <Fluid/SerialPort.hpp>

#include <unistd.h>

using namespace Fluid;

class NapraSource : public DataSource<NapraSample>, public DI_Thread
{
public:
  NapraSource(const std::string& device, 
	      int bufferSize, 
	      int offset,
	      int filtering);
  void start();
  void stop();
  void childLoop();

  std::string getDriverName() const { return "Napra"; }
private:
  std::string m_device;
  SerialPort  m_port;

  bool        m_continue;

  /** Used to correct the incorrect sensor ordering/numbering. Range: 0-5.*/
  int         m_offset;
  int         m_filtering;
};

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

class NapraSourceFactory : public DeviceFactory
{
public:
  virtual std::string      getDriverName() const { return "Napra"; }
  NapraSource*          createDataSource(Fluid::Chunk* chunk)
  {
    std::string device;
    
    device = chunk->get("device").getString("/dev/ttyS5");
    
    int bufferSize = chunk->get("bufferSize").getInt(30);

    int offset = chunk->get("offset").getInt(0);
    int filter = chunk->get("filter").getInt(1);

    NapraSource *source = new NapraSource(device, bufferSize, offset, filter);
    return source;
  }

  virtual InputDeviceBase* create(Chunk* chunk)
  {
    
    Napra * g = new Napra(chunk, createDataSource(chunk));
    g->outputBuffer().setSampleRate(chunk->get("sampleRate").getDouble(60));
    return g;
  }
};

extern NapraSourceFactory factory;


#endif
