/* COPYRIGHT
 *
 * This file is part of FLUID
 *
 * Author: Tommi Ilmonen, Janne Kontkanen 1999-2002.
 * Tommi.Ilmonen@hut.fi, Janne.Kontkanen@hut.fi
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. 
 *    
 */

#ifndef FLUID_IIR1_HPP
#define FLUID_IIR1_HPP

#include <Fluid/Math.hpp>

namespace Fluid {

  /** First order IIR filter. */

  template <class T>
  class Iir1
  {
  public:
    Iir1() {}
    ~Iir1() {}

    void set(T value, T a) { m_value = value; m_a = a; }

    /// Sets the feedback coefficient.
    void setA(T a) { m_a = a; }

    void put(T v)
    {
      m_value = m_value * m_a + v * (1.0f - m_a);
    }
    
    /** Puts a sample to the filter. If the sample value exceeds
	current value, then it uses coefficient a2 (different
	filtering for different directions. . */
    void putRaiser(T v, T a2)
    {
      if(v > m_value)
	m_value = m_value * a2 + v * (1.0f - a2);
      else
	m_value = m_value * m_a + v * (1.0f - m_a);
    }

    const T & get() const { return m_value; }

  protected:
    T m_value;
    T m_a;
  };
  
}

#endif
