/* Copyright
 *
 * This file is part of FLUID
 *
 * Authors: Karri Palovuori, Tommi Ilmonen, 2004.
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. 
 *    
 */

/*
  
g++ -I../../../include -L../../../lib/$FLUID_PLATFORM helma-test.c helma.c -o helma-test -lFluid -lm


*/

#include <Fluid/Error.hpp>
#include <Fluid/SerialPort.hpp>

#include "helma.h"

#include <stdio.h>
#include <memory.h>
#include <malloc.h>

char testdata[37]={0x83,
                   0x04,0x00,0x04,0x00,
                   0x04,0x00,0x04,0x00,
                   0x04,0x00,0x04,0x00,
                   0x04,0x00,0x04,0x00,
                   0x04,0x00,0x04,0x00,
                   0x04,0x00,0x04,0x00,
                   0x04,0x00,0x04,0x00,
                   0x04,0x00,0x04,0x00,
                   0x04,0x00,0x04,0x00};


void test1()
{
  int i;

  helma_state teststate;
  
  bzero(&teststate,sizeof(helma_state));

  for(i = 0; i < 37; i++) {
    helma_parse(testdata[i], &teststate);
    printf("%d: %02x %0.5lf %0.5lf %0.5lf\n",
	   i,teststate.switches[0], teststate.x[0], teststate.y[0],
	   teststate.z[0]);
    fflush(0);
  }
  
}

void test2(const char * dev)
{
  Fluid::SerialPort serial;
  helma_state teststate;
  char byte;
  
  if(!serial.open(dev, true, false, 115200, 8, 1, 0)) {
    Fluid::Error("Could not open %s", dev);
    return;
  }

  bzero(&teststate,sizeof(helma_state));

  int i = 0;

  Fluid::Trace("Beginning Naeprae");

  bool first = true;


  byte = 0;
  while(!(byte & 0x80))
    serial.read(&byte, 1);

  Fluid::Trace("Getting first package!!");

  while(true) {

    if(!first) {
      int n = serial.read(&byte, 1);
      
      // Fluid::Trace("Bytes = %d", n);
      
      if(n != 1) {
	Fluid::Error("Read failed (%d)", n);
	return;
      }
    }

    int res = helma_parse(byte, &teststate);
    
    if(res > 0 && (i % 10 == 0)) {
      int id = (res - 1) * 3;
      printf("%d %d: %02x", i, res, teststate.switches[res - 1]); 
      for(int k=0; k < 3; k++)
	printf(" # %0.3lf %0.3lf %0.3lf",
	       teststate.x[id + k], teststate.y[id + k], teststate.z[id + k]);
      printf("\n");

      if(res == 2)
	fflush(0);
    }
    if(res == 1)
      i++;
    
    first = false;

  }

}

main(int argc, char* argv[])
{
  int test = 1;
  const char * dev = "/dev/tty0";

  for(int i = 1; i < argc; i++) {
    if(strcmp(argv[i], "-d") == 0 && i + 1 < argc) {
      dev = argv[++i];
      test = 2;
    }
    else if(strcmp(argv[i], "-tty") == 0) {
      test = 2;
    }
    else
      Fluid::Error("%s: Unknown argument %s\n", argv[0], argv[i]);
  }

  if(test == 1)
    test1();
  else
    test2(dev);
}

