/* Copyright
 *
 * This file is part of FLUID
 *
 * Authors: Karri Palovuori, Tommi Ilmonen, 2004.
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. 
 *    
 */

#include "math.h"
#include "helma.h"

float helma_readtime(int index, struct helma_state *state);

int helma_parse(char newdata, struct helma_state *state)
{
  float timeA, timeB;
  float distance[9];
  int index;
  float a, b, c, a2, b2, c2, det, x, y, z, bx, cx, cy, bx2, cx2, cy2;
  int i, j;

  /* protection against transmission errors */
  if (state->messagecounter > 36) state->messagecounter = 0; 
  if (newdata & 0x80) state->messagecounter = 0;

  state->messagebuffer[state->messagecounter] = newdata;
  state->messagecounter++;

  if (state->messagecounter != 37) return(0); // not a complete record yet

  /* at this point we have a complete, new record */

  index = (state->messagebuffer[0] & 0x20) ? 1 : 0; /* 0 = master, 1 = slave */

  /* Negate the LSB. */
  state->switches[index] = (state->messagebuffer[0] & 0x1f) ^ 1;

  for(i = 0; i < 9; i++) {

    timeA = helma_readtime(i * 2, state)   - 12.5 - 21.8; /* calibration */
    timeB = helma_readtime(i * 2 + 1, state) - 12.5 - 21.8; /* same here */

    distance[i] = HELMA_SPEEDOFSOUND * timeB/1000000.0;

    j = i + 9 * index;

    state->distances[j][0] = HELMA_SPEEDOFSOUND * timeA/1000000.0;
    state->distances[j][1] = HELMA_SPEEDOFSOUND * timeB/1000000.0;

    /*if ((timeA>0.0) && (timeB>0.0) && (fabs(timeA-timeB)<20.0))
      distance[i] = HELMA_SPEEDOFSOUND * timeB/1000000.0;
    else
    distance[i] = 0.0f;*/
  }
  
  bx = HELMA_BX;
  cx = HELMA_CX;
  cy = HELMA_CY;

  bx2 = bx * bx;
  cx2 = cx * cx;
  cy2 = cy * cy;

  for(i = 0; i < 3; i++) {

    if ((distance[i]!=0.0) && (distance[i+3]!=0.0) && (distance[i+6]!=0.0)) {

      a = distance[i];
      b = distance[i + 3];
      c = distance[i + 6];

      a2 = a * a;
      b2 = b * b;
      c2 = c * c;

      x = (a2 - b2 + bx2) / (2 * bx);
      y = (a2 - c2 - 2 * x * cx + cx2 + cy2) / (2 * cy);
      det = a2 - x * x - y * y;

      if (det > 0.0) { /* and it should be! */
	z = sqrt(det);
	j = i + 3 * index;
	state->x[j] = x;
	state->y[j] = y;
	state->z[j] = z;
      }
    }
  }
  return (index+1);
}

/* return time records in microseconds */
float helma_readtime(int index, struct helma_state *state)
{
  float time;
  time =  4 * (state->messagebuffer[index * 2 + 1] * 128 + 
	       state->messagebuffer[index * 2 + 2]);
  time /= 7.3728;
  return(time);
}
