/* Copyright
 *
 * This file is part of FLUID
 *
 * Authors: Karri Palovuori, Tommi Ilmonen, 2004.
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. 
 *    
 */

#ifndef HELMA_H
#define HELMA_H

#include <SignalCleaner.hpp>
#include <Fluid/Vector3.hpp>

#define HELMA_SPEEDOFSOUND 340.0
#define HELMA_BX 0.045
#define HELMA_CX 0.0225
#define HELMA_CY 0.038971

enum HELMA_Filter {
  HELMA_RAW,
  HELMA_FILTERED1,
  HELMA_FILTERED2

};

/* the first half of the results (x[0..2],y[0..2],z[0..2],switches[0])
   are for the master device, the second half for the slave device
   the switches bits are  0 - little finger
                          1 - ring finger switch A
                          2 - ring finger switch B
                          3 - middle finger touches thumb
                          4 - index finger touches thumb 
*/
class helma_state
{
public:
  char  messagebuffer[37];
  int   messagecounter;
  float x[6];
  float y[6];
  float z[6];
  float distances[18][2];
  int   switches[2];
  int   verbose;
  
  HELMA_Filter filtering;

  /** Indexing:
      1: hand
      2: finger
      (3: flight-time-number (emitter number))
      4: A or B time
  */
  Fluid::SignalCleaner3<float> cleaner[2][3][2];
  Fluid::SignalCleaner<float>::Params cleanerParams;
  Fluid::Vector3 cleaned[2][3];
};


#ifdef __cplusplus
extern "C" {
#endif

  void helma_init(helma_state *state);
  /* helma_parse receives one byte at a time and returns current status
     0=no new record yet, 1=master record completed, 2=slave record
     completed the helma_state struct must have been externally
     allocated and zeroed before use
  */
  int helma_parse(char newdata, helma_state *state); 

#ifdef __cplusplus
}
#endif

#endif
