/* COPYRIGHT
 *
 * This file is part of FLUID
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * Tommi.Ilmonen@hut.fi
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. 
 *    
 */
#ifndef NAPRAGL_HPP
#define NAPRAGL_HPP

#include <Fluid/Vector3.hpp>

#include <qgl.h>

#include <di_thread.h>

class NapraGl : public QGLWidget, public DI_Thread
{
  Q_OBJECT

  enum {
    FPS = 50,
    SAMPLES = 512
  };

  enum RenderGraph {
    RENDER_XYZ,
    RENDER_DIS_A,
    RENDER_DIS_FA, // Filtered
    RENDER_DIS_FA2, // Filtered
    RENDER_DIS_B,
    RENDER_DIS_FB,
    RENDER_DIS_FB2,
    RENDER_DIS_CLEAN,
    RENDER_SIZEOF
  };

public:
  NapraGl(QWidget * parent, const char * dev);
  virtual ~NapraGl();

  static void exec( int & argc, char ** argv, const char * dev);

public slots:

  void setGraph(int);
  void setMode(int);

protected:
  void keyPressEvent ( QKeyEvent * e );
  virtual void resizeGL ( int w, int h ) ;
    
  virtual void paintGL () ;

  virtual void childLoop();

  void renderGraph(int sensor, RenderGraph);
  void squares(float ymin, float ymax, int n, bool isAll);
  void seconds();

  Fluid::Vector3 m_buffers[6][SAMPLES];
  Fluid::Vector3 m_distances[7][6][SAMPLES];
  int            m_finger;
  const char    *m_dev;
  uint           m_index;
  RenderGraph    m_mode;
  bool           m_run;
};

#endif

