/* COPYRIGHT
 *
 * This file is part of FLUID
 *
 * Author: Tommi Ilmonen, 1999-2001.
 * Tommi.Ilmonen@hut.fi
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. 
 *    
 */

#include <QTKeyboard.hpp>

#include <Fluid/DeviceFactory.hpp>
#include <Fluid/Debug.hpp>

#include <Fluid/Error.hpp>

using namespace Fluid;

QTKeyboardSource::QTKeyboardSource(int sampleRate, int bufferSize, bool grab)
  :DataSource<KeyboardSample>(bufferSize),sampleRate(sampleRate),grab(grab)
{   
}

void QTKeyboardSource::start()
{
  logStream << "QTKeyboardSource started" << std::endl;
  logStream << "Spawning device thread for QTKeyboardSource" << std::endl;
  run();
}

void QTKeyboardSource::stop()
{
  logStream << "QTKeyboardSource stop()" << std::endl; 
  logStream << "QTKeyboardSource - killing device thread - ";
  //kill(); 
  widget->close();
  waitEnd();
}

QTKeyboardSource::~QTKeyboardSource()
{
  logStream << "QTKeyboardSource - destructing" << std::endl;
}

float QTKeyboardSource::getSampleRate() const
{
  return 0.0f;
}

void QTKeyboardSource::notify()
{
  // Trace("QTMouseSource::notify # %d", (int) s.buttons[Mouse::BUTTON_LEFT] );

  KeyboardSample s;

  while(widget->keyCount()) {
    s = widget->popKey();
    write(s);
  }
}


void QTKeyboardSource::childLoop()
{
  logStream << "THREAD2: Device thread succesfully started" << std::endl;
  int argc = 0;
  QApplication a(argc,(char**)NULL);
  app = &a;
  KeyboardTrackerWidget w(this,sampleRate,grab);
  widget = &w;
  widget->resize( 200, 100 );
  app->setMainWidget( widget );
  widget->show();
  logStream << "THREAD2: Widget shown" << std::endl;
  app->exec();
  logStream << "THREAD2: Closing" << std::endl;
}

class QTKeyboardFactory : public DeviceFactory
{
public:
  QTKeyboardFactory()
  {
  }
  
  virtual std::string      getDriverName() const { return "QTKeyboard"; }
  virtual InputDeviceBase* create(Chunk* chunk)
  {
    int sampleRate;
    bool grab;
    int bufferSize;
    
    sampleRate = chunk->get("sampleRate").getInt(60);
    
    std::string s = chunk->get("grab").getString();
    if( s.size())
      grab = (s == "true") ? true : false;
    else grab = false;
    
    bufferSize = chunk->get("bufferSize").getInt(50);
    
    logStream << "Creating QTKeyboardSource\n";
    logStream << "****************\n";
    chunk->dump(logStream);
    logStream << '\n';

    QTKeyboardSource *qtsource = new QTKeyboardSource(sampleRate,bufferSize,grab);
    return new Keyboard(chunk,qtsource);
  }
};

QTKeyboardFactory factory;
extern "C" DeviceFactory* getFactory()
{
  return &factory;
}

