/* COPYRIGHT
 *
 * This file is part of FLUID
 *
 * Author: Tommi Ilmonen, 1999-2001.
 * Tommi.Ilmonen@hut.fi
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. 
 *    
 */

#include <QTMouse.hpp>

#include <Fluid/DeviceFactory.hpp>
#include <Fluid/Debug.hpp>

#include <Fluid/Error.hpp>

using namespace Fluid;

QTMouseSource::QTMouseSource(int sampleRate, int bufferSize, bool grab)
  :DataSource<MouseSample>(bufferSize),sampleRate(sampleRate),grab(grab)
{   
}

void QTMouseSource::start()
{
  logStream << "QTMouseSource started" << std::endl;
  logStream << "Spawning device thread for QTMouseSource" << std::endl;
  run();
}

void QTMouseSource::stop()
{
  logStream << "QTMouseSource stop()" << std::endl; 
  logStream << "QTMouseSource - killing device thread - ";
  //kill(); 
  widget->close();
  waitEnd();
}

QTMouseSource::~QTMouseSource()
{
  logStream << "QTMouseSource - destructing" << std::endl;

}

float QTMouseSource::getSampleRate() const
{
  return widget->getSampleRate();
}

void QTMouseSource::notify()
{
  MouseSample s;
  s.x = widget->getCurrentX();
  s.y = widget->getCurrentY();
  s.state = widget->getButtonState();

  // Trace("QTMouseSource::notify # %d", (int) s.buttons[Mouse::BUTTON_LEFT] );

  write(s);
}


void QTMouseSource::childLoop()
{
  logStream << "THREAD2: Device thread succesfully started" << std::endl;
  int argc = 0;
  QApplication a(argc,(char**)NULL);
  app = &a;
  MouseTrackerWidget w(this,sampleRate,grab);
  widget = &w;
  widget->resize( 200, 100 );
  app->setMainWidget( widget );
  widget->show();
  logStream << "THREAD2: Widget shown" << std::endl;
  app->exec();
  logStream << "THREAD2: Closing" << std::endl;
}

class QTMouseFactory : public DeviceFactory
{
public:
  QTMouseFactory()
  {
  }
  
  virtual std::string      getDriverName() const { return "QTMouse"; }
  virtual InputDeviceBase* create(Chunk* chunk)
  {
    int sampleRate;
    bool grab;
    int bufferSize;
    sampleRate = chunk->get("sampleRate").getInt(60);
    
    Variant v = chunk->get("grab");
    if(!v.isEmpty())
      grab = (std::string(v) == "true") ? true : false;
    else grab = false;
    
    bufferSize = chunk->get("bufferSize").getInt(30);

    logStream << "Creating QTMouseSource\n";
    logStream << "****************\n";
    chunk->dump(logStream);
    logStream << '\n';

    QTMouseSource *qtsource = new QTMouseSource(sampleRate,bufferSize,grab);
    return new Mouse(chunk,qtsource);
  }
};

QTMouseFactory factory;
extern "C" DeviceFactory* getFactory()
{
  return &factory;
}

