/* COPYRIGHT
 *
 * This file is part of FLUID
 *
 * Author: Tommi Ilmonen, 1999-2001.
 * Tommi.Ilmonen@hut.fi
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. 
 *    
 */
#include "QTMouse.hpp"

#include <Fluid/Config.hpp>

#include <di_socket.h>
#include <di_data_buffer.h>

int main(int argc, char **argv)
{
  // Infinite loop:
  unsigned repeat = (unsigned) 1;

  std::cout << "Starting QTMouse server..." << std::endl;
  std::string configuration;

  for(int argn=1; argn < argc; argn++) {

    const char * arg = argv[argn];

    if(strcmp(arg, "--repeat") == 0) {
      if(argn + 1 <= argc) {
	repeat = atoi(argv[++argn]);
      }
    }
    else if(arg) {
      configuration = arg;
    }
    else {
      std::cout << "USAGE: " << argv[0] 
		<< " <configuration file> <--repeat n>\n";
      exit(0);
    }
  }

  DeviceConfig::init(configuration);

  Chunk *chunk = DeviceConfig::getChunk("QTMouse");

  if(!chunk) {
    std::cout << "No QTMouse configuration defined" << std::endl;
    DeviceConfig::cleanup();
    return 0;
  }

  int sampleRate;
  bool grab;
  int bufferSize;
  std::string address;
  
  sampleRate = chunk->get("sampleRate").getInt(60);
  
  
  Variant v = chunk->get("grab");
  if(!v.isEmpty())
    grab = (std::string(v) == "true") ? true : false;
  else grab = false;
  
  bufferSize = chunk->get("bufferSize").getInt(30);
  
  std::string port;
  port = chunk->get("port").getString("2199");
            
  std::cout << "Creating QTMouseSource\n";
  std::cout << "**********************\n";
  chunk->dump(std::cout);
  std::cout << '\n';
  MouseSample sample;      
  DI_Socket socket;
  DI_DataBuffer2 buffer(sample.getMaxSerializedSize());

  std::cout << "Opening listener to port: " << port.c_str() << std::endl;
  socket.open(port.c_str(),true); // server mode    

  if(!socket) {
    std::cout << "Error creating server socket" << std::endl;
    exit(0);
  }
  else {
    std::cout << "Created server socket into port: " << port << std::endl;
  }
  while(repeat--) {
    std::cout << repeat  << " repetitions left" << std::endl;

    DI_Socket *connection = 0;
    std::cout << "Waiting for client to connect" << std::endl;

    while(connection == 0)
      connection = socket.listen(); 

    std::cout << "Got a client connection" << std::endl;
    QTMouseSource source(sampleRate,bufferSize,grab);
    std::cout << "Starting mouse source" << std::endl;
    source.start();
    sleep(1);

    while(connection->isOK())  {

      source.lock();
      int avail = source.getAvailable();

      while(avail--) {
	buffer.rewind();
	source.read(sample);
	sample.write(buffer);
	DI_writeToSocket(connection,&buffer);
      }
      source.release();
      usleep(1000000/sampleRate);
    }

    source.stop();
    connection->close();
    delete connection;
  }
  socket.close();

  std::cout << "Shutting down QTMouse server" << std::endl;
  DeviceConfig::cleanup();
  return 0;
}

