/* COPYRIGHT
 *
 * This file is part of FLUID
 *
 * Author: Tommi Ilmonen, Janne Kontkanen 1999-2002.
 * Tommi.Ilmonen@hut.fi, Janne.Kontkanen@hut.fi
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. 
 *    
 */

#include "Fluid/DeviceFactory.hpp"
#include "Fluid/Variant.hpp"
#include "Fluid/Debug.hpp"
#include "Fluid/SocketDataSource.hpp"
#include "Fluid/InputDevices.hpp"
#include "Fluid/ConfigReader.hpp"
#include "Fluid/Tracker.hpp"

using namespace Fluid;

class RemoteTrackerFactory : public DeviceFactory
{
public:
  virtual std::string getDriverName() const { return "RemoteTracker"; } 
  virtual InputDeviceBase* create(Chunk* chunk)
  {
    std::string server;
    int bufferSize;

    Variant v = chunk->get("bufferSize");
    if(v.getType() != Variant::INVALID)
      bufferSize = v;
    else
      bufferSize = 50;
    
    v = chunk->get("server");
    if(v.getType() != Variant::INVALID)
      server = std::string(v);
    else
      server = "127.0.0.1:2299";
    
    SocketDataSource<TrackerSample> *source = new SocketDataSource<TrackerSample>(server,bufferSize);
    return new Tracker(chunk,source);
  }
};

RemoteTrackerFactory factory;

extern "C" DeviceFactory* getFactory()
{
  return &factory;
}
