/* -*- C++ -*- */

//
// DIVA Library. Copyright (C) Jarmo Hiipakka, 1999-2000.
//
// $Log: di_HUTcave_TTS.h,v $
// Revision 1.1  2002/10/16 08:45:54  tilmonen
// Wonderful Speech recognition system by
//  - IBM: ViaVoice.
//  - Jarmo Hiipakka: C++ wrapping
//  - Tommi Ilmonen: Fluid wrapping
//

#ifndef DI_HUTCAVE_TTS_H
#define DI_HUTCAVE_TTS_H

#include "di_socket.h"
#include "di_TTS_base.h"

enum DI_TTSMessageType {
  /// Add text to synthesis input buffer.
  DI_TTS_MSG_ADD_TEXT,
  /// Pause or unpause synthesis.
  DI_TTS_MSG_PAUSE,
  /// Stop synthesis.
  DI_TTS_MSG_STOP,
  /// Sync synthesizer (Block and do the actual synthesis).
  DI_TTS_MSG_SYNC,
  /// Copy voice.
  DI_TTS_MSG_COPY_VOICE,
  /// Set a voice parameter.
  DI_TTS_MSG_SET_VOICE_PARAM,
  /// Set a synthesis state parameter.
  DI_TTS_MSG_SET_STATE_PARAM,
  /// Number of message types.
  DI_TTS_MSG_SIZEOF
};

enum DI_TTSVoiceParamType {
  /// Set the breathiness of the voice.
  DI_TTS_VPT_BREATHINESS,
  /// Set gender (0 = male, 1 = female).
  DI_TTS_VPT_GENDER,
  /// Set head size.
  DI_TTS_VPT_HEAD_SIZE,
  /// Set pitch baseline.
  DI_TTS_VPT_PITCH_BASELINE,
  /// Set pitch fluctuation.
  DI_TTS_VPT_PITCH_FLUCTUATION,
  /// Set roughness.
  DI_TTS_VPT_ROUGHNESS,
  /// Set speed.
  DI_TTS_VPT_SPEED,
  /// Set volume.
  DI_TTS_VPT_VOLUME,
  /// Number of voice parameters.
  DI_TTS_VPT_SIZEOF
};

enum DI_TTSStateParamType {
  /// Use (or don't use) abbreviations dictionaries.
  DI_TTS_SPT_ABBR_DICT,
  /// Set input type (plain text = 0, annotated text = 1).
  DI_TTS_SPT_INPUT_TYPE,
  /// Set language.
  DI_TTS_SPT_LANGUAGE,
  /// Set number mode.
  DI_TTS_SPT_NUMBER_MODE,
  /// Set the usage (use = 1, don't use = 0) of real world units.
  DI_TTS_SPT_RWU,
  /// Set sample rate.
  DI_TTS_SPT_SAMPLERATE,
  /// Set the synthesis mode (sentence = 0, manual = 1).
  DI_TTS_SPT_SYNTH_MODE,
  /// Set the current text mode.
  DI_TTS_SPT_TEXT_MODE,
  /// Set the usage of the phoneme indices.
  DI_TTS_SPT_WANT_PHONEMES,
  /// Number of state parameters.
  DI_TTS_SPT_SIZEOF
};

enum DI_TTSInputType {
  DI_TTS_INPUT_PLAIN = 0,
  DI_TTS_INPUT_ANNOTATED = 1,
};

enum DI_TTSNumberMode {
  DI_TTS_NM_NONYEARS = 0,
  DI_TTS_NM_YEARS = 1
};

enum DI_TTSUnits {
  DI_TTS_UNITS_ECI = 0,
  DI_TTS_UNITS_RWU = 1
};

enum DI_TTSTextMode {
  DI_TTS_TM_DEFAULT = 0,
  DI_TTS_TM_ALPHA_SPELL = 1,
  DI_TTS_TM_ALL_SPELL = 2,
  DI_TTS_IRC_SPELL = 3
};

class DI_HUTCaveTTS : public DI_TTSBase
{
public:

  /**name Constructors */

  //@{
  
  ///
  
  DI_HUTCaveTTS();

  ///
  
  ~DI_HUTCaveTTS();

  //@}

  /**name Access */

  //@{

  ///

  DI_Socket &socket() { return m_socket; }

  //@}
  
  /**name Message-related functions. */

  //@{

  ///
  
  ECICallbackReturn waveformBufferMsg(long nSamples);

  ///
  
  ECICallbackReturn phonemeBufferMsg(long nBytes);

  ///
  
  ECICallbackReturn indexReplyMsg(long index);

  ///
  
  ECICallbackReturn phonemeIndexReplyMsg(ECIMouthData *mouthData);

  ///
  
  
  //@}

  /**name Synthesis state */

  //@{

  ///

  bool running() const { return m_running; }
  
  //@}

private:

  DI_Socket  m_socket;
  char      *m_buffer;
  bool       m_running;
};

#endif  // DI_HUTCAVE_TTS_H
