/* -*- C++ -*- */

//
// DIVA Library. Copyright (C) Jarmo Hiipakka, 1999-2000.
//
// $Log: di_HUTcave_navig.C,v $
// Revision 1.2  2002/10/17 08:25:47  tilmonen
// Bug fixes.
//
// Revision 1.1  2002/10/16 08:45:54  tilmonen
// Wonderful Speech recognition system by
//  - IBM: ViaVoice.
//  - Jarmo Hiipakka: C++ wrapping
//  - Tommi Ilmonen: Fluid wrapping
//


#include <iostream>
using namespace std;

#include <string.h>
#include "di_error.h"
#include "di_HUTcave_navig.h"

static const int BUFFER_LENGTH(100);

DI_HUTCaveNavig::DI_HUTCaveNavig() :
  m_buffer(NULL),
  m_running(true)
{
  m_buffer = new char[BUFFER_LENGTH];
}

DI_HUTCaveNavig::~DI_HUTCaveNavig()
{
}

SmHandler DI_HUTCaveNavig::recoWordReply(SM_MSG *, void *)
{
  // Just tell the recognition engine to continue recognition
  recognizeNextWord();
  return SM_RC_OK;
}

SmHandler DI_HUTCaveNavig::recoPhraseReply(SM_MSG *reply, void * call_data)
{
  int rc(SmReturnRc(*reply));

  if (rc != SM_RC_OK)
  {
    DI_error(DI_ERR_UNKNOWN,"Error in phrase recognition",
	     SmReturnRcDescription(rc));
    return ( recognizeNextWord() ? SM_RC_OK : -1 );
  }

  unsigned long flags(0);

  rc = SmGetPhraseState(*reply, &flags);

  if (rc != SM_RC_OK)
  {
    DI_error(DI_ERR_UNKNOWN,"Error in phrase recognition",
	     SmReturnRcDescription(rc));
    return ( recognizeNextWord() ? SM_RC_OK : -1 );
  }
  
  unsigned long nWords(0);
  SM_WORD *words(NULL);

  rc = SmGetFirmWords(*reply, &nWords, &words);
  
  if (rc != SM_RC_OK)
  {
    DI_error(DI_ERR_UNKNOWN,"Error in phrase recognition",
	     SmReturnRcDescription(rc));
    return ( recognizeNextWord() ? SM_RC_OK : -1 );
  }

  int len(0);

  for (unsigned long i(0); i<nWords; i++)
  {
    strncpy(m_buffer+len, words[i].spelling, BUFFER_LENGTH-len);
    len += (int)(words[i].spelling_size);
    if (len > BUFFER_LENGTH-1) len = BUFFER_LENGTH-1;
    m_buffer[len-1] = ' ';
  }
  m_buffer[len] = '\0';

  if (flags & SM_PHRASE_ACCEPTED)
    cout << "Acc: ";
  else
    cout << "Rej: ";
  cout << m_buffer << endl;

  char *translation(NULL);
  if (m_trans_fsg && (flags & SM_PHRASE_ACCEPTED))
  {
    char **phraseWords = new char *[nWords+1];

    for (unsigned long i(0); i<nWords; i++)
      phraseWords[i] = words[i].spelling;

    phraseWords[nWords] = NULL;
    VtGetTranslation(m_trans_fsg, phraseWords, &translation);
    cout << "Translation: " << translation << endl;

    if (strcmp("shutdown",translation) == 0)
      m_running = false;
  }

  if (strcmp("shutdown",m_buffer) == 0)
    m_running = false;
  
  if(flags & SM_PHRASE_ACCEPTED)
  {
    if (m_trans_fsg)
      deliver(translation);
    else
      deliver(m_buffer);
  }

  return ( recognizeNextWord() ? SM_RC_OK : -1 );
}

void DI_HUTCaveNavig::deliver(const char *str)
{
  DI_trace2("DI_HUTCaveNavig::deliver # \"%s\"", str);
  
  if(m_socket) {
    m_socket.write(str, strlen(str)+1);
    m_socket.read(m_buffer,BUFFER_LENGTH);  
  }
}
