/* -*- C++ -*- */

//
// DIVA Library. Copyright (C) Jarmo Hiipakka, 1999-2000.
//
// $Log: di_speech_reco_base.h,v $
// Revision 1.1  2002/10/16 08:45:55  tilmonen
// Wonderful Speech recognition system by
//  - IBM: ViaVoice.
//  - Jarmo Hiipakka: C++ wrapping
//  - Tommi Ilmonen: Fluid wrapping
//

#ifndef DI_SPEECH_RECO_BASE_H
#define DI_SPEECH_RECO_BASE_H

#include <smapi.h>
#include <vtbnfc.h>

class DI_SpeechRecoBase
{
public:

  /**name Constructors */

  //@{
  
  ///
  
  DI_SpeechRecoBase();

  ///
  
  virtual ~DI_SpeechRecoBase();

  //@}

  /**name Initialization */

  //@{
  
  ///

  bool open(const char *applicationName, int connectID);

  ///

  bool close();

  //@}

  /**name Vocabularies and Grammars */
  
  //@{
  
  ///

  bool defineVocab(/// Vocabulary name
		   const char *name,
		   /// Number of words
		   int numWords,
		   /// An array of words
		   const char **words);
  
  ///

  bool undefineVocab(/// Vocabulary name
		     const char *name);

  ///

  bool addToVocab(/// Vocabulary name
		  const char *name,
		   /// Number of words
		  int numWords,
		   /// An array of words
		  const char **words);
  
  ///

  bool removeFromVocab(/// Vocabulary name
		       const char *name,
		       /// Number of words
		       int numWords,
		       /// An array of words
		       const char **words);

  ///

  bool defineGrammar(/// Vocabulary name
		     const char *name,
		     /// Grammar file name (.fsg)
		     const char *filename);

  ///

  bool loadTranslationGrammar(const char *name);

  ///

  bool unloadTranslationGrammar();

  ///

  bool enableVocab(/// Vocabulary name
		   const char *name);

  ///

  bool disableVocab(/// Vocabulary name
		   const char *name);

  //@}

  /**name Operation */

  //@{

  ///

  bool requestFocus();

  ///

  bool releaseFocus();
  
  ///

  bool micOn();

  ///

  bool micOff();
  
  ///

  bool recognizeNextWord();

  /// Block and dispatch the next message.

  bool dispatch();

  /// Get a socket handle that can be used, e.g., for select() or poll().

  int getSocketHandle() { return m_smapi_socket; }
  
  //@}

  /**@name Reply functions */

  //@{

  virtual SmHandler connectReply(SM_MSG *reply, void * call_data);
  virtual SmHandler disconnectReply(SM_MSG *reply, void * call_data);
  virtual SmHandler setReply(SM_MSG *reply, void * call_data);
  virtual SmHandler requestFocusReply(SM_MSG *reply, void * call_data);
  virtual SmHandler focusGrantedReply(SM_MSG *reply, void * call_data);
  virtual SmHandler micOnReply(SM_MSG *reply, void * call_data);
  virtual SmHandler micOffReply(SM_MSG *reply, void * call_data);
  virtual SmHandler defineGramReply(SM_MSG *reply, void * call_data);
  virtual SmHandler defineVocReply(SM_MSG *reply, void * call_data);
  virtual SmHandler enableVocReply(SM_MSG *reply, void * call_data);
  virtual SmHandler nextWordReply(SM_MSG *reply, void * call_data);
  virtual SmHandler recoWordReply(SM_MSG *reply, void * call_data);
  virtual SmHandler recoPhraseReply(SM_MSG *reply, void * call_data);
  virtual SmHandler recoTextReply(SM_MSG *reply, void * call_data);
  virtual SmHandler utteranceReply(SM_MSG *reply, void * call_data);

  //@}

protected:

  int m_smapi_socket;
  int ( * m_smapi_fn ) ( ... );
  void * m_smapi_data;

  void *m_trans_fsg;

private:

  static int notifier(int socket_handle, int ( * recv_fn ) ( ... ),
		      void * recv_data, void * client_data );

  static SmHandler connectCB(SM_MSG reply, void * client, void * call_d)
  { return ((DI_SpeechRecoBase *)client)->connectReply(&reply, call_d); }

  static SmHandler disconnectCB(SM_MSG reply, void * client, void * call_d)
  { return ((DI_SpeechRecoBase *)client)->disconnectReply(&reply, call_d); }

  static SmHandler setCB(SM_MSG reply, void * client, void * call_d)
  { return ((DI_SpeechRecoBase *)client)->setReply(&reply, call_d); }

  static SmHandler requestFocusCB(SM_MSG reply, void * client, void * call_d)
  { return ((DI_SpeechRecoBase *)client)->requestFocusReply(&reply, call_d); }

  static SmHandler focusGrantedCB(SM_MSG reply, void * client, void * call_d)
  { return ((DI_SpeechRecoBase *)client)->focusGrantedReply(&reply, call_d); }

  static SmHandler micOnCB(SM_MSG reply, void * client, void * call_d)
  { return ((DI_SpeechRecoBase *)client)->micOnReply(&reply,call_d); }

  static SmHandler micOffCB(SM_MSG reply, void *client, void *call_d)
  { return ((DI_SpeechRecoBase *)client)->micOffReply(&reply,call_d);}

  static SmHandler defineGramCB(SM_MSG reply, void * client, void * call_d)
  { return ((DI_SpeechRecoBase *)client)->defineGramReply(&reply, call_d); }

  static SmHandler defineVocCB(SM_MSG reply, void * client, void * call_d)
  { return ((DI_SpeechRecoBase *)client)->defineVocReply(&reply, call_d); }

  static SmHandler enableVocCB(SM_MSG reply, void * client, void * call_d)
  { return ((DI_SpeechRecoBase *)client)->enableVocReply(&reply, call_d); }

  static SmHandler nextWordCB(SM_MSG reply, void * client, void * call_d)
  { return ((DI_SpeechRecoBase *)client)->nextWordReply(&reply, call_d); }

  static SmHandler recoWordCB(SM_MSG reply, void * client, void * call_d)
  { return ((DI_SpeechRecoBase *)client)->recoWordReply(&reply, call_d); }

  static SmHandler recoPhraseCB(SM_MSG reply, void * client, void * call_d)
  { return ((DI_SpeechRecoBase *)client)->recoPhraseReply(&reply, call_d); }

  static SmHandler recoTextCB(SM_MSG reply, void * client, void * call_d)
  { return ((DI_SpeechRecoBase *)client)->recoTextReply(&reply, call_d); }

  static SmHandler utteranceCB(SM_MSG reply, void * client, void * call_d)
  { return ((DI_SpeechRecoBase *)client)->utteranceReply(&reply, call_d); }

  bool m_initialized;
  bool m_connected;

  bool disconnect();
};

#endif  // DI_SPEECH_RECO_BASE_H
