/* COPYRIGHT
 *
 * This file is part of FLUID
 *
 * Author: Tommi Ilmonen, 1999-2001.
 * Tommi.Ilmonen@hut.fi
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. 
 *    
 */

#include <SpeechViaVoice.hpp>

#include <Fluid/Config.hpp>
#include <Fluid/ConfigReader.hpp>

#include <di_sleep.h>
#include <di_socket.h>

#include <string>

using namespace Fluid;

int main(int argc, char **argv)
{
  unsigned repeat = (unsigned) -1;

  std::cout << "FLUID SpeechViaVoice Server starting up...\n";
  std::string configuration;

  for(int argn=1; argn < argc; argn++) {

    const char * arg = argv[argn];

    if(strcmp(arg, "--repeat") == 0) {
      if(argn + 1 <= argc) {
	repeat = atoi(argv[++argn]);
      }
    }
    else if(arg) {
      configuration = arg;
    }
    else {
      std::cout << "USAGE: " << argv[0] 
		<< " <configuration file> <--repeat n>\n";
      exit(0);
    }
  }

  Config::init(configuration);
  Chunk *chunk = Config::getChunk("SpeechViaVoice");

  if(!chunk) {
    std::cerr << "No SpeechViaVoice configuration found" << std::endl;
    return 0;
  }

  DataSource<Phrase> *source = SpeechViaVoice::create(chunk);

  std::string port;
  Variant v = chunk->get("port");
  if( v.getType() != Variant::INVALID )
    port = std::string(v);
  else 
    port = "2201";

  DI_Socket socket;
  Phrase sample;
  DI_DataBuffer2 buffer;

  socket.open(port.c_str(),true); // server mode    

  if(!socket)  {
    Error("Error creating server socket to \"%s\"", port.c_str());
    exit(0);
  }
  else {
    std::cout << "Created server socket into port: " << port << std::endl;
  }
  
  while(repeat--) {
    std::cout << repeat  << " repetitions left" << std::endl;
      
    DI_Socket *connection = 0;
    std::cout << "Waiting for client to connect" << std::endl;

    while(connection == 0)
      connection = socket.listen(); 

    std::cout << "Got a client connection" << std::endl;
    std::cout << "Starting glove source" << std::endl;

    source->start();

    sleep(1);

    while(connection->isOK()) {

      source->lock();
      int avail = source->getAvailable();

      while(avail--) {
	buffer.rewind();
	source->read(sample);
	sample.write(buffer);
	DI_writeToSocket(connection,&buffer);
      }

      source->release();
      DI_Sleep::sleepMs(10);
    }

    source->stop();
    delete source;
    connection->close();
    delete connection;
  }
  socket.close();

  
  std::cout << "5DTGlove Server shutting down\n";
  Config::cleanup();
  return 0;
}
