/* -*- C++ -*- */

//
// DIVA Library. Copyright (C) Jarmo Hiipakka, 1999-2000.
//
// $Log: main_old.C,v $
// Revision 1.1  2002/10/17 08:25:48  tilmonen
// Bug fixes.
//
// Revision 1.1  2002/10/16 08:45:55  tilmonen
// Wonderful Speech recognition system by
//  - IBM: ViaVoice.
//  - Jarmo Hiipakka: C++ wrapping
//  - Tommi Ilmonen: Fluid wrapping
//

#include <iostream>
using namespace std;

#include <stdlib.h>
#include <unistd.h>
#include <stdio.h>

#include "di_error.h"
#include "di_HUTcave_navig.h"

void usage(const char *name);

int main(int argc, char **argv)
{
  int option(0);
  
  char *enGrammarIn("cave_grammar_en.fsg");
  char *trGrammarIn("cave_grammar_tr.fsg");
  char *socketAddr(NULL);
  
  char  enGrammarFile[512];
  char  trGrammarFile[512];
  char  currDir[512];

  for (option = 1; option < argc && argv[option][0] == '-'; option++)
  {
    switch(argv[option][1])
    {
     case 'e':
      enGrammarIn = argv[++option];
      break;
     case 'h':
      usage(argv[0]);
      return EXIT_SUCCESS;
      break;
     case 's':
      socketAddr = argv[++option];
      break;
     case 't':
      trGrammarIn = argv[++option];
      break;
    }
  }

  if (argc-option != 0)
  {
    usage(argv[0]);
    return EXIT_FAILURE;
  }

  DI_HUTCaveNavig reco;

  if (!reco.open("testReco",0))
  {
    DI_error(DI_ERR_UNKNOWN,
	     "Could not open connection to the recognition engine");
    return EXIT_FAILURE;
  }

  if (socketAddr)
  {
    if (!reco.socket().open(socketAddr,false,true))
    {
      DI_error(DI_ERR_SYSTEM, "Could not open socket connection");
      return EXIT_FAILURE;
    }
    reco.socket().setBlocking(DI_BLOCK_ONE);
  }
  
  // Grammar
  if (getcwd(currDir, sizeof(currDir)-1) == NULL)
  {
    DI_error(DI_ERR_SYSTEM,"Could not get the current working directory");
    return EXIT_FAILURE;
  }
  if (enGrammarIn[0] == '/')  // If using an absolute path
    snprintf(enGrammarFile, sizeof(enGrammarFile), "%s", enGrammarIn);
  else
    snprintf(enGrammarFile, sizeof(enGrammarFile), "%s/%s", currDir,
	     enGrammarIn);
  if (trGrammarIn[0] == '/')
    snprintf(trGrammarFile, sizeof(trGrammarFile), "%s", trGrammarIn);
  else
    snprintf(trGrammarFile, sizeof(trGrammarFile), "%s/%s", currDir,
	     trGrammarIn);

  if (!reco.defineGrammar("cave_grammar",enGrammarFile))
  {
    DI_error(DI_ERR_UNKNOWN,"Error defining grammar");
    return EXIT_FAILURE;
  }

  if (!reco.enableVocab("cave_grammar"))
  {
    DI_error(DI_ERR_UNKNOWN,"Error enabling vocalulary");
    return EXIT_FAILURE;
  }
    
  if (!reco.loadTranslationGrammar(trGrammarFile))
  {
    DI_error(DI_ERR_UNKNOWN,"Error loading translation grammar");
    return EXIT_FAILURE;
  }

  reco.requestFocus();
  reco.micOn();
  reco.recognizeNextWord();

  DI_trace("Preparing to run the recognition engine");
  
  while (reco.running()) {
    reco.dispatch();
    DI_trace("One round");
  }

  reco.micOff();
  reco.releaseFocus();
  reco.close();

  return 0;
}

void usage(const char *name)
{
  cout << "\nUsage:\n\n"
       << name << " [options]\n\n"
       << "  -e en_name   : recognition engine grammar name\n"
       << "  -h           : show this message\n"
       << "  -s sock_addr : socket address\n"
       << "  -t tr_name   : translation grammar name\n"
       << endl;
}
