

#include <VWand.hpp>

using namespace Fluid;

VWandSource::VWandSource(const std::string& device, int sampleRate, int bufferSize)
  :DataSource<MouseSample>(bufferSize),
   device(device),
   sampleRate(sampleRate)
{
}

void VWandSource::start()
{
  bool ok = wand.init((char*) device.c_str());
  if(!ok)
    {
      errStream << "Couldn't open device: " << device.c_str() << '\n';
      return;
    }

  m_continue = true;
  run();
}

void VWandSource::stop()
{
  m_continue = false;
  
  waitEnd();

  wand.shutdown();
}

void VWandSource::childLoop()
{
  while(m_continue)
    {
      SerialMouseState state;
      MouseSample sample;

      if(wand.nextState(&state)) {
	stateToSample(&state, &sample);
	write(sample);
	// usleep(1000000/sampleRate);
      }
    }
}

void VWandSource::stateToSample(SerialMouseState *state, MouseSample *sample)
{
  sample->x = state->m_xMotion;
  sample->y = state->m_yMotion;

  sample->state = state->m_buttonStates;
}


VWandFactory factory;

extern "C" DeviceFactory* getFactory()
{
  return &factory;
}

