
#ifndef _VWAND_HPP_
#define _VWAND_HPP_

#include <Fluid/DataSource.hpp>
#include <Fluid/Mouse.hpp>
#include <Fluid/Debug.hpp>
#include <Fluid/Config.hpp>
#include <Fluid/ConfigReader.hpp>
#include <Fluid/DeviceFactory.hpp>

#include <SerialMouse.hpp>

#include <unistd.h>

using namespace Fluid;

class VWandSource : public DataSource<MouseSample>, public DI_Thread
{
public:
  VWandSource(const std::string& device, int sampleRate, int buffersize);
  void start();
  void stop();
  void childLoop();

  std::string getDriverName() const { return "VWand"; }

  static void stateToSample(SerialMouseState *state, MouseSample *sample);

private:
  std::string device;
  int sampleRate;
  SerialMouse wand;

  bool m_continue;
};

////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////

class VWandFactory : public DeviceFactory
{
public:
  virtual std::string getDriverName() const { return "VWand"; }
  VWandSource*        createDataSource(Fluid::Chunk* chunk)
  {
    std::string device;
    int sampleRate, bufferSize;
    sampleRate = chunk->get("sampleRate").getInt(60);
    
    
    device = chunk->get("device").getString("/dev/ttys1");
    
    bufferSize = chunk->get("bufferSize").getInt(50);

    VWandSource *source = new VWandSource(device,sampleRate,bufferSize);
    return source;
  }
    
  virtual InputDeviceBase* create(Chunk* chunk)
  {

    Mouse * g = new Mouse(chunk,createDataSource(chunk));
    g->outputBuffer().setSampleRate(chunk->get("sampleRate").getDouble(60));
    return g;
  }
};

extern VWandFactory factory;


#endif
