/* COPYRIGHT
 *
 * This file is part of FLUID
 *
 * Author: Tommi Ilmonen, 1999-2001.
 * Tommi.Ilmonen@hut.fi
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. 
 *    
 */

#include <VWand.hpp>

#include <di_socket.h>

#include <string>

using namespace Fluid;

int main(int argc, char **argv)
{
  unsigned repeat = 1;

  std::cout << "FLUID V-Wand Server starting up...\n";
  std::string configuration;

  for(int argn=1; argn < argc; argn++) {

    const char * arg = argv[argn];

    if(strcmp(arg, "--repeat") == 0) {
      if(argn + 1 <= argc) {
	repeat = atoi(argv[++argn]);
      }
    }
    else if(arg) {
      configuration = arg;
    }
    else {
      std::cout << "USAGE: " << argv[0] 
		<< " <configuration file> <--repeat n>\n";
      exit(0);
    }
  }

  DeviceConfig::init(configuration);
  Chunk *chunk = DeviceConfig::getChunk("VWand");

  if(!chunk) {
    std::cerr << "No VWand configuration found" << std::endl;
    return 0;
  }

  DataSource<MouseSample> *source = factory.createDataSource(chunk);
  std::string port = chunk->get("port").getString("2199");

  int sampleRate = chunk->get("sampleRate").getInt(60);
    
  DI_Socket socket;
  MouseSample sample;
  DI_DataBuffer2 buffer(sample.getMaxSerializedSize());
  socket.open(port.c_str(),true); // server mode    
  if(!socket)  {
    std::cout << "Error creating server socket" << std::endl;
    exit(0);
  }
  else {
    std::cout << "Created server socket into port: " << port << std::endl;
  }
  
  while(repeat--) {

    std::cout << repeat  << " repetitions left" << std::endl;
    
    DI_Socket *connection = 0;
    std::cout << "Waiting for client to connect" << std::endl;
    while(connection == 0)
      connection = socket.listen(); 
    std::cout << "Got a client connection" << std::endl;
    std::cout << "Starting wand source" << std::endl;
    source->start();
    sleep(1);

    connection->setBlocking(DI_BLOCK_NOT);

    while(connection->isOK()) {

      // puts("Enter loop");
      
      source->lock();
      int avail = source->getAvailable();
      if(!avail) {
        /* puts("Check socket");
	connection->isPendingInput(0);
	connection->write(0, 0);
	*/
      }
      while(avail--) {
	buffer.rewind();
	source->read(sample);
	sample.write(buffer);
	DI_writeToSocket(connection,&buffer);

	// printf("state = %x\n", sample.state);

	// puts("One round");
      }
      source->release();
      usleep(1000000/sampleRate);

      // puts("Foo read");
      char foo;
      connection->read(&foo, 1); // Perform empty read to check socket status
      // puts("Foo read 2");
    }
    puts("Done one loop");

    if(!repeat)
      return 0; // dirty hack

    source->stop();
    delete source;
    puts("Source done");
    connection->close();
    delete connection;

    puts("Connection done");
  }
  socket.close();
  
  std::cout << "V-Wand Server shutting down\n" << std::endl;

  DeviceConfig::cleanup();
  return 0;
}
