/* COPYRIGHT
 *
 * This file is part of FLUID
 *
 * Author: Tommi Ilmonen, 1999-2001.
 * Tommi.Ilmonen@hut.fi
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. 
 *    
 */

#ifndef FLUID_FISTINESS_HPP
#define FLUID_FISTINESS_HPP

#include <Fluid/ClonablePointer.hpp>
#include <Fluid/Glove.hpp>
#include <Fluid/ProcessorNode.hpp>
#include <Fluid/IONodeBuffer.hpp>

#include <iostream>

namespace Fluid {

  /** Recognize the fistiness of the data glove. */

  class Fistiness : public ProcessorNodeContinuous, public IOSource
  {
  public:

    friend class FistinessRequest;

    Fistiness(Processors * host, 
	      const char * id,
	      const std::string &sourceId, 
	      float memorySeconds,
	      ProcessorNodeRequest *req = 0);
    virtual ~Fistiness();

    virtual bool start();
    virtual bool stop();
    
    virtual void update();

    virtual IONode *ioGetOutput(const char *);
    virtual std::string ioGetOutputName(unsigned);

    bool isOk() { return m_input != 0; }

    ulong processedSamples() const 
    { return (long) m_buffer.getSampleCount(); }

    /* float getSample(long index) const { return m_buffer.getIndexConst(index); }

    float getDelayedSample(long delay) const 
    { return m_buffer.getNewestConst(delay); }*/

    const std::string & sourceId() const { return m_sourceId; }

    IONodeBuffer<float> & buffer() { return m_buffer; }

  protected:

    IONodeBuffer<GloveSample> *m_input;
    IONodeBuffer<float> m_buffer;

    std::string m_sourceId;
    float m_memorySeconds;

    ClonablePointer<ProcessorNodeRequest> m_sourceRequest;
  };

  /////////////////////////////////////////////////////////////////////////////
  /////////////////////////////////////////////////////////////////////////////
  
  /** A query-object to assist you to get handle to a Fistiness object. */
  class FistinessRequest : public ProcessorNodeRequest
  {
  public:
    FistinessRequest(const std::string &id, 
		     const std::string &sourceId, 
		     float memorySeconds,
		     ProcessorNodeRequest * req = 0);
    virtual ~FistinessRequest();
  
    virtual bool isEqual(ProcessorNode *);
    virtual ProcessorNode *createNew(Processors *);
    virtual ProcessorNodeRequest *clone() const;

  protected:

    std::string m_id;
    std::string m_sourceId;
    float m_memorySeconds;

    ClonablePointer<ProcessorNodeRequest> m_sourceRequest;
  };


} // namespace Fluid

#endif
