/* COPYRIGHT
 *
 * This file is part of FLUID
 *
 * Author: Tommi Ilmonen, 1999-2001.
 * Tommi.Ilmonen@hut.fi
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. 
 *    
 */

#ifndef FLUID_GLOVE_NAVIGATOR_HPP
#define FLUID_GLOVE_NAVIGATOR_HPP

#include <Fluid/IONodeBuffer.hpp>
#include <Fluid/Matrix3.hpp>
#include <Fluid/MessageSink.hpp>
#include <Fluid/MessageSource.hpp>
#include <Fluid/ProcessorNode.hpp>

// #include <Fluid/Glove.hpp>

//#include <iostream>

namespace Fluid {
  
  class GloveNavigator : public IOSource, 
			 public MessageSink,
			 public ProcessorNodeContinuous
  {
  public:
    enum FingerGesture {
      FG_NONE,
      FG_POINTING_INDEX,
      FG_PALM_OPEN,
      FG_PALM_HALF_CLOSED,
      FG_FIST,
      FG_SIZEOF
    };

    enum Direction {
      DIR_UNDEFINED,
      DIR_PALM_IN,
      DIR_PALM_OUT,
      DIR_PALM_LEFT,
      DIR_PALM_RIGHT,
      DIR_SIZEOF
    };

    GloveNavigator(Processors * host, 
		   const std::string &id, 
		   const std::string & headLocationStr,
		   const std::string & sensorLocationStr,
		   const std::string & sensorRotationStr,
		   const std::string & gloveSourceStr);

    virtual ~GloveNavigator();

    virtual IONode *ioGetOutput(const char *);
    virtual std::string ioGetOutputName(unsigned);
 
    virtual bool msgProcess(const char *id, Message *);

    virtual bool start();
    virtual bool stop();
    virtual void update();
    
    IONodeBuffer<Vector3> & location() { return m_location; }
    IONodeBuffer<Matrix3> & rotation() { return m_rotation; }
    IONodeBuffer<float>   & scale() { return m_scale; }

    bool enabled() const { return m_enabled; }
    void setEnabled(bool enabled) { m_enabled = enabled; }
  protected:

    void flapMotion();

    void dragMotion();

    bool palmFlap(FingerGesture g1, FingerGesture g2, TimeStamp delta);

    IONodeBuffer<Vector3> * m_headLocation;
    IONodeBuffer<Vector3> * m_sensorLocation;
    IONodeBuffer<Matrix3> * m_sensorRotation;

    std::string             m_headLocationStr;
    std::string             m_sensorLocationStr;
    std::string             m_sensorRotationStr;
    std::string             m_gloveSourceStr;

    // Output buffers:
    IONodeBuffer<Vector3> m_location;
    IONodeBuffer<Matrix3> m_rotation;
    IONodeBuffer<float>   m_scale;

    uint m_count;
    bool m_enabled;
    float m_accelerationScale;
    float m_rotationScale;

    float m_accelerationDescale;
    float m_rotationDescale;

    int           m_flaps;

    Vector3       m_vel;
    Vector3       m_loc;
    float         m_aziVel;
    float         m_azi;

    Matrix3       m_worldRot;
    Matrix3       m_dragStartRot;
    Vector3       m_dragStartLoc;
    bool          m_onDrag;

    FingerGesture m_currentGesture;
    FingerGesture m_prevGesture;
    TimeStamp     m_currentTime;
    TimeStamp     m_deltaTime;
  };
  
}

#endif
