%{
#include <Fluid/GloveRecognizer.hpp>
using namespace Fluid;
static GloveGesture currentGesture;  
static GloveRecognizer *currentRecognizer = 0;
static std::string currentVariable;
static FingerTest currentTest;
static bool validGesture = false;

/*
#ifndef __sgi

 extern "C" {
   int FluidGlovewrap() 
   {
     return yywrap();
   }
 }
#else
#define FluidGlovewrap yywrap
#endif
*/
%}

SYMCHAR [^\n{}=/ \\]
VALCHAR [^\n{}=/]
VALUE   [^\n\t{}=]*[^\t\n{}= ]
BLANK [ \n\t]

DIGIT    [0-9]
INT {DIGIT}+
FLOAT    {DIGIT}+"."{DIGIT}*

%x comment      
%x chunkstart   
%x inchunk      
%x def1
%x def2
%x def3
%x def4
%x defend
%option stack
%%
<*>"/*"         yy_push_state(comment);
<comment>[^*\n]*        /* eat anything that's not a '*' */
<comment>"*"+[^*/\n]*   /* eat up '*'s not followed by '/'s */
<comment>\n             /* if line number were counted ... */
<comment>"*"+"/"        yy_pop_state();

<INITIAL>{BLANK}*                          /* eat spaces */
<INITIAL>{SYMCHAR}+ {
  BEGIN(chunkstart); 
  currentGesture.setName(yytext);
  currentGesture.clearTests();
  printf("A new gesture was began: \"%s\"\n", yytext);
}
<chunkstart>{BLANK}*"{" BEGIN(inchunk);
<inchunk>{BLANK}*"}" {
  BEGIN(INITIAL); 
  currentRecognizer->addGesture(currentGesture);
  printf("Added gesture \"%s\"\n", (const char *) & currentGesture.name()[0]);
}
<inchunk>{BLANK}*"{" {
  BEGIN(def1);
  validGesture = false;
}
<def1>{BLANK}*{INT} {
  BEGIN(def2);
  currentTest.setFinger((Glove::Finger) atoi(yytext));
}
<def2>{BLANK}*{INT} {
  BEGIN(def3);
  currentTest.setJoint((Glove::FingerJoint) atoi(yytext));
}
<def3>{BLANK}*({INT}|{FLOAT}) {
  BEGIN(def4);
  currentTest.setThreshold(atof(yytext));
}
<def4>{BLANK}* /* Skip blanks. */
<def4>(greater|smaller) {
  BEGIN(defend);
  currentTest.setMode(yytext);
}

<defend>{BLANK}*"}" {
  BEGIN(inchunk);
  currentGesture.addEntryTest(currentTest);

  printf("Added new test: %d %d %f %d\n", 
	 currentTest.finger(), currentTest.joint(),
	 currentTest.threshold(), (int) currentTest.greaterThan());
}

<inchunk>. fprintf(stderr, "Unexpected \"%s\" in middle of chunk", yytext);
<def1>. fprintf(stderr, "Unexpected \"%s\" instead of finger id", yytext);
<def2>. fprintf(stderr, "Unexpected \"%s\" instead of joint id", yytext);
<def3>. fprintf(stderr, "Unexpected \"%s\" instead of angle limit", yytext);
<def4>. fprintf(stderr, "Unexpected \"%s\" instead of \"greater|smaller\"", yytext);
<defend>. fprintf(stderr, "Unexpected \"%s\" instead of \"}\"", yytext);

%%

bool Fluid::GloveRecognizer::readConfiguration(const std::string & filename)
{
  currentRecognizer = this;

  assert(filename.c_str());
  yyin = fopen(filename.c_str(),"rb");

  if(yyin) {
    m_gestures.resize(0);
    m_lastGesture = -1;
    yylex();

    /* 
    logStream << "GloveRecognizer::readConfigurationcreateNew " 
	      << nofGestures() << endl;
    */

    fclose(yyin);
    m_configFile = filename;

    currentRecognizer = 0;
    return m_gestures.size() != 0;
  }

  m_configFile.resize(0);

  return false;
}
