/* COPYRIGHT
 *
 * This file is part of FLUID
 *
 * Author: Tommi Ilmonen, 1999-2001.
 * Tommi.Ilmonen@hut.fi
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. 
 *    
 */

#ifndef FLUID_MOTION_NAVIGATOR_HPP
#define FLUID_MOTION_NAVIGATOR_HPP

#include <Fluid/Matrix3.hpp>
#include <Fluid/MessageSink.hpp>
#include <Fluid/MessageSource.hpp>
#include <Fluid/ProcessorNode.hpp>

#include <Fluid/Mouse.hpp>

#include <iostream>

namespace Fluid {
  
  class MotionNavigator : public IOSource, 
			  public ProcessorNodeContinuous
  {
  public:

    MotionNavigator(Processors * host, 
		    const std::string &id, 
		    const std::string & trackerSensor,
		    const Vector3 & center,
		    float forwardAzi,
		    const Vector3 & ellipsoidDimensions,
		    const std::string & style);

    ~MotionNavigator();

    virtual IONode *ioGetOutput(const char *);
    virtual std::string ioGetOutputName(unsigned);
 
    virtual bool start();
    virtual bool stop();
    virtual void update();
    
    IONodeBuffer<Vector3> & location() { return m_location; }
    IONodeBuffer<Matrix3> & rotation() { return m_rotation; }
 
    bool enabled() const { return m_enabled; }
    void setEnabled(bool enabled) { m_enabled = enabled; }

    void moveTo(const Vector3 &loc, float azi);

  protected:


    std::string             m_trackerSensor;
    Vector3                 m_center;
    float                   m_forwardAzi;
    Vector3                 m_ellipsoid;
    std::string             m_style;
    
    IONodeBuffer<Vector3> * m_sensorLocation;

    // Output buffers:
    IONodeBuffer<Vector3> m_location;
    IONodeBuffer<Matrix3> m_rotation;
    IONodeBuffer<float>   m_scale;

    long m_count;
    bool m_enabled;
    /* float m_accelerationScale;
       float m_rotationScale; */

    float         m_azi;
    // Vector3       m_vel;
    Vector3       m_loc;
    Matrix3       m_basicRot;
  };
  
}

#endif
