/* COPYRIGHT
 *
 * This file is part of FLUID
 *
 * Author: Tommi Ilmonen, 1999-2001.
 * Tommi.Ilmonen@hut.fi
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. 
 *    
 */

#include <Fluid/MouseButtons.hpp>

#include <Fluid/Error.hpp>
#include <Fluid/MessageMouse.hpp>
#include <Fluid/Mouse.hpp>
#include <Fluid/Processors.hpp>

namespace Fluid {
  
  
  MouseButton::MouseButton(Processors *p, const char * id,
			   const std::string &nodeName)
    : ProcessorNodeContinuous(p, id),
      m_input(0),
      m_count(0),
      m_buttonState(0),
      m_nodeName(nodeName)
  {}

  MouseButton::~MouseButton()
  {}
    
  bool MouseButton::start()
  {
    IONode * node = m_host->findIONode(m_nodeName);

    m_input = dynamic_cast<IONodeBuffer<MouseSample> *> (node);
    
    if(!m_input) {
      Error("MouseButton::MouseButton # No mouse \"%s\" (node = %p, %s)", 
	    m_nodeName.c_str(), node, node ? typeid(*node).name() : "nil");
      return false;
    }
    
    m_count = m_input->getSampleCount();

    return true;
  }

  bool MouseButton::stop()
  {
    m_input = 0;
    return true;
  }
    
  void MouseButton::update()
  {
    assert(m_input);

    // Trace("MouseButton::update");

    for(; m_count <= m_input->getSampleCount(); m_count++) {
      
      const MouseSample &sample = m_input->getIndex(m_count);

      int state = sample.state;

      /* for(int i=0; i < MouseSample::BUTTON_NUMBER; i++) {
	if(sample.buttons[i]) 
	  state = state | (1 << i);
	  }*/

      // Trace("MouseButton::update %d %d", (int) m_count, state);

      if(state != m_buttonState) {
	// The status was changed !!
	
	MessageMouse e(state, m_buttonState, sample.x, sample.y, 
		       m_input->getTimeStamp((long) m_count));
	
	msgSend("mouse", &e);

	/* Trace("MouseButton::update # Sent message %x -> %x", 
	   m_buttonState, state); */

	m_buttonState = state;
      }
    }
  }


}
