/* COPYRIGHT
 *
 * This file is part of FLUID
 *
 * Author: Tommi Ilmonen, 1999-2001.
 * Tommi.Ilmonen@hut.fi
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. 
 *    
 */

#ifndef FLUID_MOUSE_NAVIGATOR_HPP
#define FLUID_MOUSE_NAVIGATOR_HPP

#include <Fluid/Matrix3.hpp>
#include <Fluid/MessageSink.hpp>
#include <Fluid/MessageSource.hpp>
#include <Fluid/ProcessorNode.hpp>

#include <Fluid/Mouse.hpp>

#include <iostream>

namespace Fluid {
  
  class MouseNavigator : public IOSource, 
			 public ProcessorNodeContinuous,
			 public MessageSink
			 
  {
  public:

    enum Direction {
      DIR_UNDEFINED,
      DIR_PALM_IN,
      DIR_PALM_OUT,
      DIR_PALM_LEFT,
      DIR_PALM_RIGHT,
      DIR_SIZEOF
    };

    MouseNavigator(Processors * host, 
		   const std::string &id, 
		   const std::string & sensorLocationStr,
		   const std::string & sensorLocation2Str,
		   const std::string & sensorRotationStr,
		   const std::string & mouseSourceStr,
		   int button);

    ~MouseNavigator();

    virtual IONode *ioGetOutput(const char *);
    virtual std::string ioGetOutputName(unsigned);
 
    virtual bool msgProcess(const char *id, Message *);

    virtual bool start();
    virtual bool stop();
    virtual void update();
    
    IONodeBuffer<Vector3> & location() { return m_location; }
    IONodeBuffer<Matrix3> & rotation() { return m_rotation; }
    IONodeBuffer<float>   & scale() { return m_scale; }

    bool enabled() const { return m_enabled; }
    void setEnabled(bool enabled) { m_enabled = enabled; }

    void moveTo(const Vector3 &loc, float azi);

  protected:

    void dragMotion();

    IONodeBuffer<Vector3> * m_sensorLocation;
    IONodeBuffer<Vector3> * m_sensorLocation2;
    IONodeBuffer<Matrix3> * m_sensorRotation;

    std::string             m_sensorLocationStr;
    std::string             m_sensorLocation2Str;
    std::string             m_sensorRotationStr;
    std::string             m_mouseSourceStr;
    int                     m_button;

    // Output buffers:
    IONodeBuffer<Vector3> m_location;
    IONodeBuffer<Matrix3> m_rotation;
    IONodeBuffer<float>   m_scale;

    long m_count;
    bool m_enabled;
    /* float m_accelerationScale;
       float m_rotationScale; */

    Vector3       m_vel;
    Vector3       m_loc;
    float         m_aziVel;
    float         m_azi;
    float         m_scaleVal;
    Matrix3       m_worldRot;
    Matrix3       m_dragStartRot;
    Vector3       m_dragStartLoc;
    bool          m_onDrag;
    bool          m_onScaling;
    bool          m_startDrag;
  };
  
}

#endif
