/* COPYRIGHT
 *
 * This file is part of FLUID
 *
 * Author: Tommi Ilmonen, 1999-2001.
 * Tommi.Ilmonen@hut.fi
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. 
 *    
 */

#include <NapraToMouse.hpp>

#include <Fluid/Error.hpp>
#include <Fluid/Processors.hpp>

namespace Fluid {


  NapraToMouse::NapraToMouse(Processors * host, 
			     const std::string &id,
			     const std::string & napraBufferName,
			     int hand)
    : ProcessorNodeContinuous(host, id),
      m_napraName(napraBufferName),
      m_hand(hand),
      m_source(0)
  {}

  NapraToMouse::~NapraToMouse()
  {}
  
  bool NapraToMouse::start()
  {
    // IONode * source = m_host->findIONode(m_napraName);

    IONode * source = m_host->findIONode(m_napraName);

    m_source = dynamic_cast<IONodeBuffer<NapraSample> *> (source);
    
    if(!m_source) {
      Error("NapraToMouse::start # No Napra buffer \"%s\"", m_napraName.c_str());
      return false;
    }
    
    m_buffer.copyParameters(*m_source);

    return true;
  }

  bool NapraToMouse::stop()
  {
    m_source = 0;
    return true;
  }
    
  void NapraToMouse::update()
  {
    assert(m_source);


    MouseSample ms;
    ms.clear();

    // for(; m_count < m_source->getSampleCount(); m_count++) {
    while(m_buffer.getSampleCount() < m_source->getSampleCount()) {

      int switches = m_source->getIndex(m_buffer.getSampleCount()).
	switches[m_hand];
      int state = 0;
      
      if(switches & NapraSample::SWITCH_LITTLE_FINGER)
	state |= 0x1; 
      if(switches & NapraSample::SWITCH_RING_FINGER_NEAR)
	state |= 0x2;
      if(switches & NapraSample::SWITCH_RING_FINGER_FAR)
	state |= 0x4;
      if(switches & NapraSample::SWITCH_INDEX_THUMB)
	state |= 0x8;
      if(switches & NapraSample::SWITCH_MIDDLE_THUMB)
	state |= 0x10;
      
      ms.state = state;
      
      // Trace("NapraToMouse::update # %x", state);

      m_buffer.put(ms);
    }
    
    m_buffer.copyParameters(*m_source);
  }
    
  IONode *NapraToMouse::ioGetOutput(const char *id)
  {
    if(strcmp(id, "output") == 0)
      return & m_buffer;

    return 0;
  }

  std::string NapraToMouse::ioGetOutputName(unsigned i)
  {
    return i ? "" : "output";
  }

}
