/* COPYRIGHT
 *
 * This file is part of FLUID
 *
 * Author: Tommi Ilmonen, 1999-2001.
 * Tommi.Ilmonen@hut.fi
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. 
 *    
 */

#ifndef FLUID_SENSOR_CONTAINMENT_HPP
#define FLUID_SENSOR_CONTAINMENT_HPP

#include <Fluid/ClonablePointer.hpp>
#include <Fluid/IONodeBuffer.hpp>
#include <Fluid/MessageSource.hpp>
#include <Fluid/ProcessorNode.hpp>
#include <Fluid/TimeStamp.hpp>
#include <Fluid/Vector3.hpp>

namespace Fluid {

  class Tracker;

  /** A gesture detector that detects if a particular sensor is within
      an axis-aligned cube. 

      This class emits integer events that can have two values:
      <OL>
      <LI> 1: Enter the cube
      <LI> 2: Exit the cube
      </OL>
  */
  
  class SensorBoxContainment : public MessageSource,
			       public ProcessorNodeContinuous
  {
  public:
    SensorBoxContainment(Processors *host,
			 const std::string &id,
			 const std::string &nodeId,
			 const Vector3 &low,
			 const Vector3 &high,
			 float blockSeconds,
			 ProcessorNodeRequest *req = 0);

    virtual ~SensorBoxContainment();

    virtual bool start();
    virtual bool stop();
    
    virtual void update();

    bool ok() const { return m_input; }
    
    // const std::string & trackerName() const { return m_trackerName; }
    const std::string & nodeId() const { return m_nodeId; }
    
    const Vector3 & low() const { return m_low; }
    const Vector3 & high() const { return m_high; }

  protected:
    
    // Fluid::Tracker *m_tracker;

    // int m_sensorId;
    IONodeBuffer<Vector3> *m_input;

    Vector3 m_low;
    Vector3 m_high;

    ulong   m_count;

    bool    m_within;

    float   m_blockSeconds;

    TimeStamp m_releaseTime;

    std::string m_nodeId;

    ClonablePointer<ProcessorNodeRequest> m_sourceRequest;
    // std::string m_sensorName;
  };

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

  class SensorBoxContainmentRequest : public ProcessorNodeRequest
  {
  public:
    SensorBoxContainmentRequest(const std::string &nodeId,
				const std::string &id,
				const Vector3 &low,
				const Vector3 &high,
				float blockSeconds,
				ProcessorNodeRequest *req = 0);

    virtual ~SensorBoxContainmentRequest();

    virtual bool isEqual(ProcessorNode *);
    virtual ProcessorNode *createNew(Processors *);
    virtual ProcessorNodeRequest *clone() const;

  protected:
    std::string m_id;
    std::string m_nodeId;
    Vector3 m_low;
    Vector3 m_high;
    
    float   m_blockSeconds;

    ClonablePointer<ProcessorNodeRequest> m_sourceRequest;
  };

}

#endif
