/* COPYRIGHT
 *
 * This file is part of FLUID
 *
 * Author: Tommi Ilmonen, 1999-2001.
 * Tommi.Ilmonen@hut.fi
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. 
 *    
 */

#ifndef FLUID_TRACKER_SENSOR_SELECTOR_HPP
#define FLUID_TRACKER_SENSOR_SELECTOR_HPP

#include <Fluid/ProcessorNode.hpp>
#include <Fluid/Tracker.hpp>

namespace Fluid {


  /** Selects sensor data coming from the motion tracker. */
  class TrackerSensorSelector : public IOSource,
				public ProcessorNodeContinuous
				
  {
  public:
    TrackerSensorSelector(Processors * host, 
			  const std::string &id,
			  const std::string &trackerName,
			  const std::string &sensorName);

    virtual ~TrackerSensorSelector();

    virtual bool start();
    virtual bool stop();
    
    virtual void update();
    
    virtual IONode *ioGetOutput(const char *);
    virtual std::string ioGetOutputName(unsigned);

    bool ok () const { return m_input != 0; }

  protected:

    IONodeBuffer<TrackerSample> *m_input;
    
    int m_index;

    std::string m_trackerName;
    std::string m_sensorName;

    IONodeBuffer<Vector3> m_locations;
    IONodeBuffer<Matrix3> m_rotations;
  };

  /** Selects tracker sensors and transforms them according to current
      transformation. */

  class TrackerSensorSelector2 : public IOSource,
				 public ProcessorNodeContinuous
				 
  {
  public:
    TrackerSensorSelector2(Processors * host,
			   const std::string &id,
			   const std::string &trackerName,
			   const std::string &sensorName,
			   const std::string &rotationName,
			   const std::string &locationName,
			   const std::string &scaleName,
			   const Vector3 & offset = Vector3(0.0, 0.0, 0.0));
    
    virtual ~TrackerSensorSelector2();

    virtual bool start();
    virtual bool stop();
    
    virtual void update();
    
    virtual IONode *ioGetOutput(const char *);
    virtual std::string ioGetOutputName(unsigned);

    bool ok () const { return m_input && m_wrotation && m_wlocation; }

    IONodeBuffer<Vector3> & locations() { return m_locations; }
    IONodeBuffer<Matrix3> & rotations() { return m_rotations; }

  protected:

    IONodeBuffer<TrackerSample> *m_input;
    // Worlds rotation and location
    IONodeBuffer<Matrix3>       *m_wrotation;
    IONodeBuffer<Vector3>       *m_wlocation;
    IONodeBuffer<float>         *m_wscale;
    
    int m_index;

    std::string m_trackerName;
    std::string m_sensorName;

    std::string m_rotationName;
    std::string m_locationName;
    std::string m_scaleName;

    Vector3     m_offset;

    IONodeBuffer<Vector3> m_locations;
    IONodeBuffer<Matrix3> m_rotations;
  };


}

#endif
