/* COPYRIGHT
 *
 * This file is part of FLUID
 *
 * Author: Tommi Ilmonen, 1999-2001.
 * Tommi.Ilmonen@hut.fi
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. 
 *    
 */

#ifndef FLUID_MESSAGE_MOUSE_H
#define FLUID_MESSAGE_MOUSE_H

#include <Fluid/Message.hpp>

namespace Fluid {
  
  class MessageMouse : public Message
  {
  public:
    MessageMouse() {}
    MessageMouse(int currentState,
		 int pastState,
		 int x,
		 int y,
		 const TimeStamp &time)
      : Message(time), 
	m_currentState(currentState), 
	m_pastState(pastState), 
	m_x(x), m_y(y) {}
    
    virtual ~MessageMouse() {}
    
    int currentState() const { return m_currentState; }
    int pastState() const { return m_pastState; }

    bool pressedCurrentButton(int i) const
    { return (m_currentState & (1 << i)); }

    bool pressedPastButton(int i) const
    { return (m_pastState & (1 << i)); }

    bool pressed(int i) const
    { return pressedCurrentButton(i) && !pressedPastButton(i); }

    bool released(int i) const
    { return !pressedCurrentButton(i) && pressedPastButton(i); }

    bool buttonChange(int i) const
    { return pressedCurrentButton(i) != pressedPastButton(i); }

    int x() const { return m_x; }
    int y() const { return m_y; }

  private:
    int m_currentState;
    int m_pastState;
    int m_x, m_y;
  };

}

#endif
