/* COPYRIGHT
 *
 * This file is part of FLUID
 *
 * Author: Tommi Ilmonen, 1999-2001.
 * Tommi.Ilmonen@hut.fi
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. 
 *    
 */

#include <Fluid/MessageSource.hpp>
#include <Fluid/MessageSink.hpp>

#include <assert.h>

namespace Fluid {
  
  MessageSource::MessageSource()
  {}

  MessageSource::~MessageSource()
  {
    for(iterator it = m_targets.begin(); it != m_targets.end(); it++) {
      MessageSink * sink = (*it).m_target;
      sink->msgForgetSource(this);      
    }
    m_targets.clear();
  }

  void MessageSource::msgAddSink(const std::string &sourceId, 
				 const std::string &targetId, 
				 MessageSink *target)
  {
    Remap remap(sourceId, targetId, target);
    assert(target);
    m_targets.push_back(remap);
    target->msgAddSource(this);
  }
  
  void MessageSource::msgAddSink(MessageSink *target)
  {
    Remap remap;
    assert(target);
    remap.m_target = target;
    m_targets.push_back(remap);
    target->msgAddSource(this);
  }
  
  void MessageSource::msgRemoveSink(MessageSink *sink)
  {
    iterator it = m_targets.begin();

    while(it != m_targets.end()) {
      
      Remap *remap = & (*it);

      if(remap->m_target == sink) {
	iterator tmp(it);

	tmp++;
	m_targets.erase(it);
	it = tmp;
      } 
      else
	++it;
    }    
  }

  void MessageSource::msgClearSinks()
  {
    m_targets.clear();
  }

  void MessageSource::msgSend(const char *id, Message *msg)
  {
    iterator it = m_targets.begin();

    while(it != m_targets.end()) {
      
      Remap *remap = & (*it);
      assert(remap->m_target);

      if(!id)
	remap->m_target->msgProcess(remap->m_targetId.c_str(), msg);
      else if(remap->m_sourceId.size() == 0) {
	  remap->m_target->msgProcess(remap->m_targetId.c_str(), msg);
      }
      else if(remap->m_sourceId == id) {
	if(!remap->m_targetId.size())
	  remap->m_target->msgProcess(remap->m_sourceId.c_str(), msg);
	else
	  remap->m_target->msgProcess(remap->m_targetId.c_str(), msg);
      }
      
      ++it;
    }
    
  }
    

}
