/* COPYRIGHT
 *
 * This file is part of FLUID
 *
 * Author: Tommi Ilmonen, 1999-2001.
 * Tommi.Ilmonen@hut.fi
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. 
 *    
 */

#ifndef FLUID_MESSAGE_SOURCE_HPP
#define FLUID_MESSAGE_SOURCE_HPP

#include <string>
#include <list>

namespace Fluid {

  class MessageSink;
  class Message;

  class MessageSource
  {
  public:
    MessageSource();
    virtual ~MessageSource();
    
    void msgAddSink(const std::string &sourceId, 
		    const std::string &targetId, 
		    MessageSink *);

    void msgAddSink(MessageSink *);

    void msgRemoveSink(MessageSink *);

    void msgClearSinks();

  protected:
    
    void msgSend(const char *, Message *);
    
    class Remap
    {
    public:
      Remap() : m_target(0) {}
      Remap(const std::string &sourceId, 
	    const std::string &targetId, 
	    MessageSink *target) 
	: m_sourceId(sourceId), m_targetId(targetId), m_target(target) {}

      std::string  m_sourceId;
      std::string  m_targetId;
      MessageSink *m_target;
    };
    
    std::list<Remap> m_targets;
    typedef std::list<Remap>::iterator iterator;
  };
  
}

#endif
