/* COPYRIGHT
 *
 * This file is part of FLUID
 *
 * Author: Tommi Ilmonen, 1999-2001.
 * Tommi.Ilmonen@hut.fi
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. 
 *    
 */

#include <Fluid/ProcessorNode.hpp>

#include <Fluid/Processors.hpp>

namespace Fluid {

  ProcessorNode::ProcessorNode(Processors * host)
    : m_host(host)
  {
    if(host) host->addNode(this);
  }

  ProcessorNode::ProcessorNode(Processors * host, const char * id) 
    : m_host(host), m_id(id)
  {
    if(host) host->addNode(this);
  }

  ProcessorNode::ProcessorNode(Processors * host, const std::string & id) 
    : m_host(host), m_id(id)
  {
    if(host) host->addNode(this);
  }

  ProcessorNode::~ProcessorNode()
  {
    if(m_host)
      m_host->removeNode(this);
  }

  bool ProcessorNode::start()
  {
    return true;
  }

  bool ProcessorNode::stop()
  {
    return true;
  }

  /////////////////////////////////////////////////////////////////////////////
  /////////////////////////////////////////////////////////////////////////////

  
  ProcessorNodeContinuous::ProcessorNodeContinuous(Processors * host)
  {
    if(host) {
      m_host = host;
      host->addNode(this);
    }
  }

  ProcessorNodeContinuous::ProcessorNodeContinuous
  (Processors * host, const char * id) 
    : ProcessorNode(0, id)
  {
    if(host) {
      m_host = host;
      host->addNode(this);
    }
  }

  ProcessorNodeContinuous::ProcessorNodeContinuous
  (Processors * host, const std::string & id) 
    : ProcessorNode(0, id)
  {
    if(host) {
      m_host = host;
      host->addNode(this);
    }
  }

  ProcessorNodeContinuous::~ProcessorNodeContinuous()
  {
    /* Note that while this function looks a lot like
       ProcessorNode::~ProcessorNode they do not have the same
       results. The reason is that while this method is called the
       object is still of type ProcessorNodeContinuous rather than
       ProcessorNode. Due to this the call m_host->removeNode has
       different end results. */
    
    if(m_host)
      m_host->removeNode(this);
    
    m_host = 0;
  }
  
}
