/* COPYRIGHT
 *
 * This file is part of FLUID
 *
 * Author: Tommi Ilmonen, 1999-2001.
 * Tommi.Ilmonen@hut.fi
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. 
 *    
 */

#ifndef FLUID_PROCESSOR_NODE_HPP
#define FLUID_PROCESSOR_NODE_HPP

#include <string>
#include <vector>

namespace Fluid {

  class Processors;

  /** Base class for all processor nodes. If you want to put your
      object within the processor colledtion, then any new classes
      must inherit this base class. */

  class ProcessorNode
  {
  public:
    ProcessorNode(Processors * host = 0);
    ProcessorNode(Processors * host, const char * id);
    ProcessorNode(Processors * host, const std::string & id);

    virtual ~ProcessorNode();

    virtual bool start();
    virtual bool stop();

    virtual std::vector<ProcessorNode *> dependsOn()
    { std::vector<ProcessorNode *> tmp; return tmp; }

    const std::string & id() const     { return m_id;}
    void setId(const std::string & id) { m_id = id; }
    void setId(const char * id)        { m_id = id; }

    void setHost(Processors * host) { m_host = host; }
    Processors * host() { return m_host; }

  protected:
    Processors * m_host;
    std::string  m_id;
  };

  /** Base class for processor nodes that must be frequently updated. */

  class ProcessorNodeContinuous : public ProcessorNode
  {
  public:
    ProcessorNodeContinuous(Processors * host = 0);
    ProcessorNodeContinuous(Processors * host, const char * id);
    ProcessorNodeContinuous(Processors * host, const std::string & id);

    virtual ~ProcessorNodeContinuous();

    virtual void update() = 0;
  };

  /** Request a processing node. */

  class ProcessorNodeRequest
  {
  public:
    ProcessorNodeRequest() {}
    virtual ~ProcessorNodeRequest() {}
  
    /** Test if a node equals the request. This method is used by the
	host system to check if some existing node would already do the
	stuff reqquired. */
    virtual bool isEqual(ProcessorNode *) = 0;

    /** This method produces a new processor node that matches the
	request. If node cannot be created, then NULL is returned.*/
    virtual ProcessorNode *createNew(Processors *) = 0;

    /** This method clones this request. */
    virtual ProcessorNodeRequest *clone() const = 0;
  };

} // namespace Fluid

#endif
