/* COPYRIGHT
 *
 * This file is part of FLUID
 *
 * Author: Tommi Ilmonen, 1999-2001.
 * Tommi.Ilmonen@hut.fi
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. 
 *    
 */

#ifndef FLUID_PROCESSORS_HPP
#define FLUID_PROCESSORS_HPP

#include <Fluid/ProcessorNode.hpp>

#include <vector>

namespace Fluid {

  class IONode;
  class MessageSource;
  class ProcessorNode;
  class ProcessorNodeRequest;
  
  /** Class to manage all the data processor/event generator nodes. */
  
  class Processors
  {
  public:
    Processors();
    ~Processors();
  
    void clear(bool deleteChildren);

    void addNode(ProcessorNode *);

    bool removeNode(ProcessorNode *);

    IONode * findIONode(const char * sourceId);
    IONode * findIONode(const std::string & sourceId)
    { return findIONode(sourceId.c_str()); }

    ProcessorNode *findNode(const char *id);
    ProcessorNode *findNode(const std::string & sourceId)
    { return findNode(sourceId.c_str()); }

    MessageSource *findMessageSource(const char *id);
    MessageSource *findMessageSource(const std::string & sourceId)
    { return findMessageSource(sourceId.c_str()); }

    ProcessorNode *requestNode(ProcessorNodeRequest *);
  
    bool start();
    void update();
    bool stop();

  private:

    std::vector<ProcessorNode *> m_nodes;
    std::vector<ProcessorNodeContinuous *> m_cnodes;

    bool m_running;
  };

} // namespace Fluid

#endif
