/* COPYRIGHT
 *
 * This file is part of the Geometry Engine - GEE
 *
 * Read the "GEE-LICENSE" file for the license.
 *
 * Authors & Copyright:   
 *
 * Tommi Ilmonen 2003
 *
 * firstname.lastname@hut.fi
 *
 */

#ifndef _EVE_TEST_ENGINE_HPP
#define _EVE_TEST_ENGINE_HPP

#include <Fluid/Vector3.hpp>
#include <Fluid/TimeStamp.hpp>

#include <list>

#include <GL/gl.h>

#include <di_error.h>

class EveTestEngine
{
public:

  class Vertex
  {
  public:
    Fluid::Vector3 m_location;
    Fluid::Vector3 m_normal;
  };
  
  class Model
  {
  public:
    
    void draw(uint)
    {
      glPushMatrix();
      
      // float degrees = 360.0f * (frame % 120) / 120.0f;
      
      glTranslatef(m_location.x, m_location.y, m_location.z);
      // glRotatef(degrees, m_location.x, m_location.y, m_location.z);
      
      glBegin(GL_TRIANGLES);
      
      glColor3fv(m_color.data());
      
      for(uint i = 0; i < m_vertices.size(); i++) {
	Vertex & v = m_vertices[i];
	glNormal3fv(v.m_normal.data());
	glVertex3fv(v.m_location.data());
      }
      
      glEnd();
      
      glPopMatrix();
    }
    
    std::vector<Vertex> m_vertices;
    Fluid::Vector3 m_location;
    Fluid::Vector3 m_color;
  };

  // Begin the local stuff.

  EveTestEngine() : m_counter(0) {}
  ~EveTestEngine() {}

  void init(int models = 50, int tris = 10)
  {
    Fluid::Vector3 v;

    for(int m = 0; m < models; m++) {
      Model mod;
	
      Vertex vert;
      
      for(int t = 0; t < tris * 3; t++) {
	v.x = rand() / (float) RAND_MAX - 0.5;
	v.y = rand() / (float) RAND_MAX - 0.5;
	v.z = rand() / (float) RAND_MAX - 0.5;
	
	vert.m_location = v;
	
	v.x = rand() / (float) RAND_MAX - 0.5;
	v.y = rand() / (float) RAND_MAX - 0.5;
	v.z = rand() / (float) RAND_MAX - 0.5;
	
	vert.m_normal = v;

	mod.m_vertices.push_back(vert);
      }

      mod.m_location = v * 20.0f;
      mod.m_color = v + 0.5f;

      m_models.push_back(mod);
    }

    m_startTime.getTimeOfDay();
  }
  
  void report()
  {
    Fluid::TimeStamp ts;
    ts.getTimeOfDay();
    float secs = (ts - m_startTime).secondsD();
    
    DI_trace2("TestApp::report # %u frames, %f fps", 
	      m_counter, (float) (m_counter / secs));
    m_counter++;
  }

  void draw()
  {
    GLfloat light_ambient[]  = { 0.6, 0.6, 0.6, 1.0 };
    GLfloat light_diffuse[]  = { 0.9, 0.9, 0.9, 1.0 };
    GLfloat light_specular[] = { 1.0, 1.0, 1.0, 1.0 };
    GLfloat light_position[] = { 2.0, 10.0, 4.0, 0.0 };

    glEnable (GL_LIGHTING);
    glEnable (GL_LIGHT0); 
    
    glLightfv(GL_LIGHT0, GL_AMBIENT, light_ambient);
    glLightfv(GL_LIGHT0, GL_DIFFUSE, light_diffuse);
    glLightfv(GL_LIGHT0, GL_SPECULAR, light_specular);
    glLightfv(GL_LIGHT0, GL_POSITION, light_position);
    
    glLightModeli(GL_LIGHT_MODEL_TWO_SIDE, GL_TRUE); 

    glShadeModel (GL_SMOOTH);

    glEnable(GL_COLOR_MATERIAL);
    glColorMaterial(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE);
    
    float w = (m_counter % 100) * 0.01f;
    glClearColor(0.2f * w, 0.5f * w, 1.0 * w, 1.0f);
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

    for(std::list<Model>::iterator it = m_models.begin(); 
	it != m_models.end(); it++) {
      (*it).draw(m_counter);
      // DI_trace("draw");
    }
  }

  void update()
  {
    m_counter++;
  }
  
protected:

  Fluid::TimeStamp m_startTime;

  uint m_counter;
  std::list<Model> m_models;

};

#endif
