/*
  23.8.2004 Marq    Siistitty kommentteja
  18.8.2004 Marq    Eka versio
*/

#include "glx_wrapper.h"

#include <stdio.h>


#include <X11/keysym.h>

#include <GL/gl.h>
#include <GL/glx.h>

#include <string.h>
#include <strings.h>
#include <stdlib.h>

struct glx_t
{
  char        *displayname;
  Display     *display;
  XVisualInfo *vinfo;
  XSetWindowAttributes swattr;
  Window      wid;
  int         attrList[20];
  GLXContext  ctx;
};

GLX * GLX_Create()
{
  GLX * glx = (GLX *) malloc(sizeof(GLX));

  bzero(glx, sizeof(GLX));

  return glx;
}

void GLX_Free(GLX * glx)
{
  free(glx);
}

int GLX_Open(GLX *glx,const char *display,int x,int y,int xs,int ys,int flags)
{
  XWindowAttributes winattrs;
  int indx=0, dummy;

  /* GL-kontekstin oletusarvot */
  glx->attrList[indx++]=GLX_USE_GL;
  glx->attrList[indx++]=GLX_DOUBLEBUFFER;
  glx->attrList[indx++]=GLX_DEPTH_SIZE;
  glx->attrList[indx++]=16;
  glx->attrList[indx++]=GLX_STENCIL_SIZE;
  glx->attrList[indx++]=8;
  glx->attrList[indx++]=GLX_RGBA;
  glx->attrList[indx++]=GLX_RED_SIZE;
  glx->attrList[indx++]=4;
  glx->attrList[indx++]=GLX_GREEN_SIZE;
  glx->attrList[indx++]=4;
  glx->attrList[indx++]=GLX_BLUE_SIZE;
  glx->attrList[indx++]=4;
  
  if(flags & GLX_USE_STEREO)
    glx->attrList[indx++]=GLX_STEREO;

  glx->attrList[indx++]=None;

  glx->displayname = strdup(display);

  /* Display auki */
  if(!(glx->display=XOpenDisplay(display)))
  {
    fprintf(stderr,"Display %s cannot be opened\n",display);
    return(-1);
  }

  /* Onko GLX olemassa? */
  if(!glXQueryExtension(glx->display,&dummy,&dummy))
  {
    fprintf(stderr,"GLX not available in %s\n",display);
    return(-1);
  }

  /* Visuaali ja konteksti */
  glx->vinfo=glXChooseVisual(glx->display,DefaultScreen(glx->display),
                             glx->attrList);
  if(!glx->vinfo)
  {
    fprintf(stderr,"Visual error in %s\n",display);
    return(-1);
  }
    
  if(!(glx->ctx=glXCreateContext(glx->display,glx->vinfo,0,
                                 (flags&GLX_LOCAL)?True:False)))
  {
    fprintf(stderr,"Cannot create context in %s\n",display);
    return(-1);
  }

  /* Ikkunan parametrit ja aukaisu */
  glx->swattr.colormap=XCreateColormap(glx->display,
                                       RootWindow(glx->display,
						  glx->vinfo->screen),
                                       glx->vinfo->visual, AllocNone);
  glx->swattr.border_pixel=0;
  glx->swattr.event_mask=ExposureMask|KeyPressMask|ButtonPressMask|
                         StructureNotifyMask;
  
  XGetWindowAttributes(glx->display, 
		       RootWindow (glx->display,glx->vinfo->screen), 
		       &winattrs);

  glx->wid=XCreateWindow(glx->display,
                         RootWindow(glx->display,glx->vinfo->screen),
                         x, winattrs.height - y - ys, xs, ys,
			 0,glx->vinfo->depth,InputOutput,
                         glx->vinfo->visual,
                         CWBorderPixel|CWColormap|CWEventMask,&glx->swattr);
  if(!glx->wid)
  {
    fprintf(stderr,"Cannot open window in %s\n",display);
    return(-1);
  }

  /* Heitetn kehykset mkeen */
  if(flags&GLX_NODECOR)
  {
    glx->swattr.override_redirect=True;
    XChangeWindowAttributes(glx->display,glx->wid,
                            CWOverrideRedirect,&glx->swattr);
  }

  XMapRaised(glx->display,glx->wid);
  XSync(glx->display,False);

  /* Fokus ikkunalle, jotta Esc menee perille */
  if(flags&GLX_NODECOR)
  {
    XSetInputFocus(glx->display,glx->wid,RevertToNone,CurrentTime);
    XSync(glx->display,False);
  }
  
  return(0);
}

void GLX_SetActive(GLX *glx)
{
  glXMakeCurrent(glx->display,glx->wid,glx->ctx);
}

int GLX_GetEvents(GLX *glx)
{
  XEvent e;
  int retval=0;

  /* X:n eventtej on luettava tai hukka perii */
  while(XPending(glx->display)>0)
  {
    XNextEvent(glx->display,&e);
    switch(e.type)
    {
      case KeyPress: if(XLookupKeysym(&e.xkey,0)==XK_Escape)
                       retval=1; break;
      default: ;
    }
  }
  
  return(retval);
}

void GLX_PreSwap(GLX *glx)
{

  /* Bufferin vaihto sek X:n ja GL:n yhteisymmrrykseen saattaminen */
  glXMakeCurrent(glx->display,glx->wid,glx->ctx);
  glFinish();
  glXWaitGL(); /* Tmnhn _piti_ riitt ilman Finishi, mutta katin kontit */
}

void GLX_Swap(GLX *glx)
{
  GLX_PreSwap(glx);

  glXSwapBuffers(glx->display,glx->wid);
  /* glXWaitX(); Pistetn takas jos tarvii */
}

void GLX_Close(GLX *glx)
{
  glXMakeCurrent(glx->display,glx->wid,glx->ctx);
  glXDestroyContext(glx->display,glx->ctx);
  XCloseDisplay(glx->display);
  free(glx->displayname);
  glx->displayname = 0;
}
