/* COPYRIGHT
 *
 * This file is part of the Geometry Engine - GEE
 *
 * Read the "GEE-LICENSE" file for the license.
 *
 * Authors & Copyright:   
 *
 * Tommi Ilmonen 2003
 *
 * firstname.lastname@hut.fi
 *
 */

/* Two test cases for this library:
   
   1) basicTest - a simple test to load a cave setup and dump its
   contents to the screen.

   2) heavyTest - runs a minimal VR test application in a cave
   environment.

*/

#include <CaveProj.hpp>
#include <EveTestEngine.hpp>
#include <VrSmuggler.hpp>

#include <Fluid/Config.hpp>
#include <Fluid/InputDevices.hpp>

#include <unistd.h>

using namespace CaveProj;

class TestApp : public VRS_FluidGlApp
{
public:
  TestApp() {}
  virtual ~TestApp() {}

  void init(int models = 50, int tris = 10)
  {
    m_engine.init(models, tris);
  }
  
  void report()
  {
    m_engine.report();
  }

protected:
  
  virtual void contextInit(VRS_Window * win)
  {
    DI_trace2("TestApp::contextInit # %p", win);
  }

  virtual void draw(VRS_Window * , int)
  {
    m_engine.draw();
  }

  virtual void preFrame()
  {
    m_engine.update();
  }  

  EveTestEngine m_engine;
};

void heavyTest(const char * file)
{
  TestApp app;

  app.loadVrConfig(file);
  app.init(200);
  app.exec(false);
  
  // sleep(10);

  app.quit();
  app.cave().dump(std::cout);
  app.report();
}

void basicTest(const char * file)
{
  Cave cave;

  cave.init(file);

  // cave.setInterocularDistance(0.0f);
  cave.setHeadPosition(Vector3(0.08, 4.0, 0), Vector3(1, 0, -1));

  cave.calculateProjections();

  cave.dump(std::cout);
  
  std::cout << std::flush;

  puts("CAVEPROJ DONE");
}

int main(int argc, char **argv)
{
  const char * file  = "eve.vrjfluid.config";
  const char * fluid = "fluid-config";
  
  bool testRun = false;

  for(int i = 1; i < argc; i++) {

    const char * arg = argv[i];

    if(strcmp(arg, "--vrs") == 0 && i + 1 < argc)
      file = argv[++i];
    if(strcmp(arg, "--fluid") == 0 && i + 1 < argc)
      fluid = argv[++i];
    else if(strcmp(arg, "--run") == 0)
      testRun = true;
  }

  if(!testRun)
    basicTest(file);
  else {
    Fluid::DeviceConfig::init(fluid);
    Fluid::InputDevices::init();
    Fluid::InputDevices::start();
    heavyTest(file);
  }

  return 0;
}
