/* COPYRIGHT
 *
 * This file is part of FLUID
 *
 * Author: Tommi Ilmonen, 1999-2001.
 * Tommi.Ilmonen@hut.fi
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. 
 *    
 */

#include <Fluid/VrjKeyboardExport.hpp>

#include <Fluid/InputDevices.hpp>
#include <Fluid/Error.hpp>
#include <Fluid/Mouse.hpp>

#include <Input/InputManager/vjDeviceFactory.h>
// #include <vjDeviceFactory.cpp>

#include <di_sleep.h>

#include <strings.h>

namespace Fluid {

  VrjKeyboardExport::VrjKeyboardExport()
    : m_keyboard(0),
      m_mouse(0)
  {
    Trace("VrjKeyboardExport::VrjKeyboardExport # 0x%p", this);
    bzero(m_curKeys, sizeof(m_curKeys));
  }

  VrjKeyboardExport::~VrjKeyboardExport()
  {
    stopSampling();
    Trace("VrjKeyboardExport::~VrjKeyboardExport # 0x%p", this);
  }
    
  bool VrjKeyboardExport::config(vjConfigChunk *c)
  {
    const char * fname = "VrjKeyboardExport::config";

    bool ok = vjKeyboard::config(c);

    Trace("%s #  %p %d", fname, c, (int) ok);

    std::string name = static_cast<std::string>(c->getProperty("sourcename"));

    Trace("%s #  \"%s\"", fname, name.c_str());

    m_mouse = Fluid::InputDevices::getMouse(name);

    if(m_mouse) {
      Trace("%s # Got mouse", fname);
      return true;
    }
    else {
      Trace("%s # Could not get mouse", fname);
      return false;
    }
  }

  int VrjKeyboardExport::startSampling()
  {
    if(m_mouse) {
      Fluid::InputDevices::start();
      // m_tracker->start();
    }
    
    Trace("VrjKeyboardExport::startSampling # 0x%p", this);

    return 1;
  }

  int VrjKeyboardExport::stopSampling()
  {
    Trace("VrjKeyboardExport::stopSampling # 0x%p", this);
    return 1;
  }

  int VrjKeyboardExport::sample()
  {
    // Trace("VrjTrackerExport::sample # 0x%p", this);
    return 1;
  }

  void VrjKeyboardExport::updateData()
  {
    // Clear all key stroke records.
    bzero(m_curKeys, sizeof(m_curKeys));

    if(m_mouse) {
      for(; m_mouseSamples <= m_mouse->getLatestIndex(); m_mouseSamples++) {

	MouseSample s = m_mouse->getSample(m_mouseSamples);
	
	m_curKeys[VJMBUTTON1] += (s.state & 0x01 != 0);
	m_curKeys[VJMBUTTON2] += (s.state & 0x02 != 0);
	m_curKeys[VJMBUTTON3] += (s.state & 0x04 != 0);

	if(s.xmotion > 0) m_curKeys[VJMOUSE_POSX] += s.xmotion;
	if(s.xmotion < 0) m_curKeys[VJMOUSE_NEGX] -= s.xmotion;

	if(s.ymotion > 0) m_curKeys[VJMOUSE_POSY] += s.ymotion;
	if(s.ymotion < 0) m_curKeys[VJMOUSE_NEGY] -= s.ymotion;

      }
    }
  }
    
  void VrjKeyboardExport::configInit()
  {
    Trace("VrjKeyboardExport::configInit");
    // vjDeviceConstructor<VrjTrackerExport>* this_ptr_not_used = 
    new vjDeviceConstructor<VrjKeyboardExport>;
  }
  
} // namespace Fluid

template class vjDeviceConstructor<Fluid::VrjKeyboardExport>;
