/* COPYRIGHT
 *
 * This file is part of FLUID
 *
 * Author: Tommi Ilmonen, 1999-2001.
 * Tommi.Ilmonen@hut.fi
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. 
 *    
 */

#ifndef FLUID_VRJ_KEYBOARD_EXPORT_H
#define FLUID_VRJ_KEYBOARD_EXPORT_H


#include <di_data_buffer.h>

#include <Input/vjInput/vjKeyboard.h>

namespace Fluid {

  class Keyboard;
  class Mouse;
  
  /** Class that hides the FLUID tracker behind the VRJ tracker
      interface. */ 

  class VrjKeyboardExport : public vjKeyboard
  {
  public:
    VrjKeyboardExport();

    virtual ~VrjKeyboardExport();

    virtual bool config(vjConfigChunk *);

    void controlLoop(void* nullParam);

    int startSampling();
    int stopSampling();
    int sample();
    void updateData();

    char* getDeviceName (void) {
      return "FluidVrjKeyboardExport";
    }

    static std::string getChunkType()
    { return "FluidVrjKeyboard"; }

    int isKeyPressed(int vjKey)
    { return m_curKeys[vjKey];}

    virtual int keyPressed(int keyId)
    { return isKeyPressed(keyId); }
  
    virtual bool modifierOnly(int modKey) { return false; }

    static void configInit();

  private:
    int onlyModifier(int);

    void updKeys();

  protected:
    Keyboard   *m_keyboard;
    Mouse      *m_mouse;

    int         m_curKeys[256];

    long        m_mouseSamples;

    //uint      m_sensors;
  
  };

} // namespace Fluid

#endif

