/* COPYRIGHT
 *
 * This file is part of FLUID
 *
 * Author: Tommi Ilmonen, 1999-2001.
 * Tommi.Ilmonen@hut.fi
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. 
 *    
 */

#include <Fluid/VrjTrackerExport.hpp>

#include <Fluid/InputDevices.hpp>
#include <Fluid/Error.hpp>
#include <Fluid/Tracker.hpp>

#include <Input/InputManager/vjDeviceFactory.h>
// #include <vjDeviceFactory.cpp>

#include <di_sleep.h>

namespace Fluid {

VrjTrackerExport::VrjTrackerExport()
  : m_tracker(0),
    m_sensors(0)
{
  Trace("VrjTrackerExport::VrjTrackerExport # 0x%p", this);
}

VrjTrackerExport::~VrjTrackerExport()
{
  Trace("VrjTrackerExport::~VrjTrackerExport # 0x%p", this);
}
    
bool VrjTrackerExport::config(vjConfigChunk *c)
{
  const char * fname = "VrjTrackerExport::config";

  bool ok = vjPosition::config(c);

  Trace("%s #  %p %d", fname, c, (int) ok);

  std::string name = static_cast<std::string>(c->getProperty("sourcename"));

  Trace("%s #  \"%s\"", fname, name.c_str());

  m_tracker = Fluid::InputDevices::getTracker(name);

  if(m_tracker) {
    Trace("%s # Got tracker", fname);
    return true;
  }
  else {
    Trace("%s # Could not get tracker", fname);
    return false;
  }
}

int VrjTrackerExport::startSampling()
{
  if(m_tracker) {
    Fluid::InputDevices::start();
    // m_tracker->start();
  }

  while(!m_tracker->getLatestIndex()) {
    DI_Sleep::sleepUs(10000);
    Fluid::InputDevices::update();
    // m_tracker->update();
  }

  m_sensors = m_tracker->getLatestSample().getSensorCount();
  theData = new vjMatrix[m_sensors * 3];

  resetIndexes();

  Trace("VrjTrackerExport::startSampling # 0x%p %d sensors", this, m_sensors);

  return 1;
}

int VrjTrackerExport::stopSampling()
{
  Trace("VrjTrackerExport::stopSampling # 0x%p", this);
  return 1;
}

int VrjTrackerExport::sample()
{
  // Trace("VrjTrackerExport::sample # 0x%p", this);
  return 1;
}

void VrjTrackerExport::updateData()
{
  m_tracker->update();

  TrackerSample ts = m_tracker->getLatestSample();

  for(uint i=0; i < m_sensors; i++) {

    vjMatrix &vjm = theData[i];

    Matrix3 flm = ts.getRotation(i);
    
    /* Counter the final transpose in MoStarTracker.cpp here. */
    flm.transpose();

    for(int k = 0; k < 3; k++)
      for(int j = 0; j < 3; j++)
	vjm(k,j) = flm[j][k];

    Vector3 flv = ts.getLocation(i);
    vjm.setTrans(* (vjVec3 *) & flv);

  }

}

vjMatrix* VrjTrackerExport::getPosData(int dev)
{
  // Trace("VrjTrackerExport::getPosData # 0x%p  %d", this, dev);

  if((uint) dev >= m_sensors)
    return 0;

  return theData + dev;
}

vjTimeStamp* VrjTrackerExport::getPosUpdateTime(int)
{
  // Trace("VrjTrackerExport::getPosUpdateTime # 0x%p", this);
  return 0;
}
void VrjTrackerExport::configInit()
{
  Trace("VrjTrackerExport::configInit");
  // vjDeviceConstructor<VrjTrackerExport>* this_ptr_not_used = 
  new vjDeviceConstructor<VrjTrackerExport>;
}

} // namespace Fluid

template class vjDeviceConstructor<Fluid::VrjTrackerExport>;
