
/* COPYRIGHT
 *
 * This file is part of Mustajuuri LGPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. Extra clauses can be found from the file
 * MUSTAJUURI_LGPL and further explanations from the file
 * MUSTAJUURI_LICENSE. The license text can be found in file
 * "LGPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *    
 */

#include <ac_attenuation.h>

#include <math.h>

AC_AttenuationPolynomial::AC_AttenuationPolynomial()
  : m_maxDist(50.0),
    m_minDist(0.5),
    m_c1(0.0),
    m_c2(1.0)
{}

AC_AttenuationPolynomial::~AC_AttenuationPolynomial()
{}

float AC_AttenuationPolynomial::metersToGain(float meters)
{
  if(meters > m_maxDist) meters = m_maxDist;
  else if(meters < m_minDist) meters = m_minDist;

  float tmp = 1.0f / (meters * m_c1 + meters * meters * m_c2);

  return tmp;
}

AC_Attenuation * AC_AttenuationPolynomial::clone() const
{
  return new AC_AttenuationPolynomial(*this);
}

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

AC_AttenuationPower::AC_AttenuationPower(float power)
  : m_maxDist(50.0),
    m_minDist(0.5),
    m_power(power)
{}

AC_AttenuationPower::~AC_AttenuationPower()
{}

float AC_AttenuationPower::metersToGain(float meters)
{
  if(meters > m_maxDist) meters = m_maxDist;
  else if(meters < m_minDist) meters = m_minDist;

  return 1.0 / powf(meters, m_power);
}

AC_Attenuation * AC_AttenuationPower::clone() const
{
  return new AC_AttenuationPower(*this);
}
