/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of Mustajuuri LGPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. Extra clauses can be found from the file
 * MUSTAJUURI_LGPL and further explanations from the file
 * MUSTAJUURI_LICENSE. The license text can be found in file
 * "LGPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *    
 */
#ifndef AC_ERROR_ENUMS_H
#define AC_ERROR_ENUMS_H

///
enum AC_ErrorLevel {
  /// Program execution trace
  AC_ERRL_TRACE,
  AC_ERRL_INFORMATION,
  AC_ERRL_WARNING,
  AC_ERRL_ERROR,
  AC_ERRL_FATAL,
  AC_ERRL_SIZEOF
};

extern const char *AC_errorLevelStr(AC_ErrorLevel);

extern const char *AC_ErrorLevels[AC_ERRL_SIZEOF];

/// Error codes.
enum AC_ErrorCode {
  /// No error
  AC_ERR_NONE,
  /// Uknown error
  AC_ERR_UNKNOWN,
  /// Out of memory
  AC_ERR_MEMORY,
  /// Data corrupted
  AC_ERR_DATA_CORRUPTED,
  /// Internal error
  AC_ERR_INTERNAL,
  /// System error
  AC_ERR_SYSTEM,
  /// Array size exceeded
  AC_ERR_ARRAY_EXCEED,
  /// List size exceeded
  AC_ERR_LIST_EXCEED,
  /// Bad argument
  AC_ERR_ARGUMENT,
  /// Bad argument
  AC_ERR_NULL_ARGUMENT,
  /// Bad match
  AC_ERR_MATCH,
  /// End of file
  AC_ERR_EOF,
  /// Resource denied
  AC_ERR_RC_DENIED,
  /// Only virtual
  AC_ERR_VIRTUAL,
  /// Do not call this function
  AC_ERR_DO_NOT_CALL,
  /// Not implemented yet
  AC_ERR_UNIMPLEMENTED,
  /// Not initialized yet
  AC_ERR_UNINITIALIZED,
  /// Real-time performance failure
  AC_ERR_REALTIME,
  /// Floating point error
  AC_ERR_FPE,
  /// CPU load too high
  AC_ERR_CPU_LOAD,
  /// Numeric overflow
  AC_ERR_OVERFLOW,
  /// Input problem
  AC_ERR_INPUT,
  /// Output problem
  AC_ERR_OUTPUT,
  /// Number of error codes
  AC_ERR_SIZEOF
};

extern const char *AC_errorStr(AC_ErrorLevel);

/// Character strings for error codes:
extern const char *AC_ErrorCodes[AC_ERR_SIZEOF];

#endif  // AC_ERROR_ENUMS_H
