/* COPYRIGHT
 *
 * This file is part of Mustajuuri LGPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. Extra clauses can be found from the file
 * MUSTAJUURI_LGPL and further explanations from the file
 * MUSTAJUURI_LICENSE. The license text can be found in file
 * "LGPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *    
 */


#include <ac_error.h>
#include <ac_stk.h>
#include <ac_vr_control.h>

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>


void AC_defaultTest(AC_Control &ac)
{
  AC_trace("AC Default test");

  sleep(1);

  // long id2 = 
  // ac.playInput(2, 0, 0.2, -170, 40, 0.0, 1.0);

  ac.playSample(0, "test.wav", 0.1, 11.0, 22.0, 0.0, 1.0, true, 0.5, 0.5);
  AC_trace("AC play a sample");

  sleep(2);
  // Two silent samples:
  ac.playSample(2, "test.wav", 0.0, -44.0, -63.0, 0.0, 1.0, false);
  ac.playSample(3, "test.wav", 0.0, -45.5, -64.0, 0.0, 1.0, false);
  /* long noteId = 
     ac.playSamplePitched(1, "test.wav", 0.2, 3.0, -45.0, -64.0, false);*/

  sleep(2);

  puts("Setting the gain");
  // ac.setNoteGain(noteId, 1.0);

  // long id = ac.playSample(1, "test.wav", 0.1, 45.0, -63.0, true);
  // AC_trace2("AC play a sample # %d", (int) id);

  // ac.playInput(4, 0, 0.2, -150, -20);

  sleep(2);
  /* puts("Initializing a granular voice");
  
  long granularId = ac.playGranular(1, "groove.wav", 10.5);

  sleep(1);
  ac.setNoteParameter(granularId, "gain", 0.1);
  sleep(1);
  ac.setNoteParameter(granularId, "gain", 0.03);
  sleep(1);
  ac.setNoteParameter(granularId, "rate", 10);*/

  long noiseId = ac.playNoise(0, 0.02, 200.0, 0.5);
  puts("noise");

  sleep(1);
  
  puts("noise-control");
  ac.beginContiguous();
  ac.sendMjNoteControl(noiseId, "pitch", 1000.0);
  ac.sendMjNoteControl(noiseId, "octaves", 1.0);
  ac.endContiguous();

  sleep(1);
  ac.noteOff(noiseId);
  
  ac.playShaker(3, AC_STK_SHAKER_CABASA, 64.0);
  puts("Cabasa");
  sleep(1);
  ac.playShaker(3, AC_STK_SHAKER_PENNY_MUG, 64.0);
  puts("Penny mug");
  sleep(1);
  ac.playShaker(3, AC_STK_SHAKER_STICKS, 64.0);
  puts("Stick");
  sleep(1);
  ac.playShaker(3, AC_STK_SHAKER_TAMBOURINE, 64.0);
  puts("Tambourine");
  sleep(1);

  long shakerId = ac.playShaker2(3, AC_STK_SHAKER_BAMBOO_CHIMES, 64.0); 
  puts("Long bamboo");

  ac.setStkNoteParameter(shakerId,1,90.0);
  ac.setStkNoteParameter(shakerId,11,90.0);
  sleep(3);
  ac.setStkNoteParameter(shakerId,1,32.0);
  ac.setStkNoteParameter(shakerId,11,32.0);
 
  sleep(1);
  
  puts("Bamboo off");

  ac.noteOff(shakerId);

  
  shakerId = ac.playShaker2(1, AC_STK_SHAKER_WATER_DROPS, 64.0,10);
  puts("Water drops");
  ac.setStkNoteParameter(shakerId,1,32.0);
  ac.setStkNoteParameter(shakerId,11,32.0);
  sleep(3);
  ac.noteOff(shakerId);
  puts("Water drops off");

  // ac.noteOff(granularId);

  // ac.noteOff(id);
  // ac.noteOff(id2);

  puts("Now 5 seconds quiet sleep");
  sleep(5);
}

void AC_vrTest(AC_Control &ac)
{
  AC_trace("VR test");

  AC_VrControl avr(&ac);
  
  float listenerY = 1.7; // The height of the listener

  AC_Vector3 location(0.0, listenerY, 0.0), tmp;

  tmp.zero();

  avr.moveListener(location, 0.01);

  // Forward
  sleep(1);
  location.set(0.0, listenerY, -5.0);
  avr.playSample(0, "hit0.aiff", 0.2, location, false);

  // 45 degrees right
  sleep(1);
  location.set(5.0, listenerY, -5.0);
  avr.playSample(1, "hit0.aiff", 0.2, location, false);

  // 90 degrees right
  sleep(1);
  location.set(5.0, listenerY, 0.0);
  avr.playSample(2, "hit0.aiff", 0.2, location, false);

  // Forward and up
  sleep(1);
  location.set(0.0, 6.7, -5.0);
  avr.playSample(3, "hit0.aiff", 0.2, location, false);

  // Back and up
  sleep(1);
  location.set(0.0, 6.7, 5.0);
  avr.playSample(4, "hit0.aiff", 0.2, location, false);

  // Left and down
  sleep(1);
  location.set(-5.0, -3.7, 0.0);
  avr.playSample(5, "hit0.aiff", 0.2, location, false);

  // Play a longer sample
  avr.playSample(0, "test.wav", 1.0, location, true);

  // Pan horizontally
  sleep(1);  
  location.set(5.0, -3.7, 0.0);
  avr.moveSource(0, 0.4, location);

  // Pan horizontally
  sleep(1);  
  location.set(5.0, -3.7, 10.0);
  avr.moveSource(0, 0.4, location);

  // Pan horizontally
  sleep(1);
  location.set(-2.0, -3.7, -1.0);
  avr.moveSource(0, 0.4, location);

  // Move further (doppler etc)
  sleep(2);  
  location.set(35.0, -3.7, 0.0);
  avr.moveSource(0, 0.4, location);

  sleep(3);

}

void AC_vrTest2(AC_Control &ac)
{
  AC_trace("VR test 2");

  AC_VrControl avr(&ac);
  
  float listenerY = 1.7; // The height of the listener

  AC_Vector3 location(0.0, listenerY, 0.0), tmp;

  tmp.zero();

  avr.moveListener(location, 0.01);

  avr.playInput(0, 0, 0.5, tmp);
  
  float y = -1.0;
  float yvel = 0.5;

  float phase = 0.0;
  float phasevel = 2.0;

  float interval = 0.1;

  float endtime = 30.0;

  uint microsecs = (uint) (1000000.0 * interval);

  for(float i=0; i < endtime; i += interval) {

    usleep(microsecs);
    
    float radius = cos(phase * 0.25) * 10.0;
    if(radius < 0.0) radius  *= -1.0;

    location[0] = cos(phase) * radius;
    location[2] = sin(phase) * radius;
    location[1] = y;

    avr.moveSource(0, interval * 1.3, location);

    y += yvel * interval;
    phase += phasevel * interval;
  }
}

enum AC_Test {
  AC_TEST_DEFAULT,
  AC_TEST_VR,
  AC_TEST_VR2
};

int main(int argc, char **argv)
{
  AC_Control ac;

  AC_Test test = AC_TEST_DEFAULT;

  const char *addr = "10030";

  for(int i = 1; i < argc; i++) {
    if(strcmp(argv[i], "-a") == 0 && i + 1 < argc)
      addr = argv[++i];
    else if(strcmp(argv[i], "-vr") == 0)
      test = AC_TEST_VR;
    else if(strcmp(argv[i], "-vr2") == 0)
      test = AC_TEST_VR2;
    else if(strcmp(argv[i], "-g") == 0)
      AC_VrControl::setDebug(true);
  }

  if(ac.init(addr)) {
    AC_trace("AC initialized");
  }
  else {
    AC_trace("INIT failed");
    return 0;
  }
  
  if(test == AC_TEST_DEFAULT)
    AC_defaultTest(ac);
  else if(test == AC_TEST_VR)
    AC_vrTest(ac);
  else
    AC_vrTest2(ac);

  return 0;
}
