

#include "ac_buffer.h"
#include "di_socket.h"

#include <stdio.h>
#include <stdlib.h>

int main(int argc, char **argv)
{
  AC_Buffer acb;
  DI_Socket sock;
  char *socketName(NULL);

  if (argc != 3)
  {
    printf("Usage: ./smain s port | c socket:port\n");
    exit(1);
  }
  bool server((argv[1][0] == 's'));
  socketName = argv[2];

  int msgID1 = 111, msgID2;
  long long ts1 = 111222333, ts2;
  int srcID1 = 222, srcID2;
  float f1 = 0.42, f2 = 99.0;
  int i1 = 18, i2 = 20;
  char * text = "Lauri poika lappalainen\nPaha poika pikkarainen";

  ts1 *= 123456;

  if (!server)
  {
    acb.writeHeader(ts1, msgID1, AC_MT_CHANGE_SOURCE_LOCATION, srcID1);
    acb.writeFloat32(f1);
    acb.writeFloat32(f2);
    acb.writeInt32(i1);
    acb.writeInt32(i2);
    acb.writeInt32(i2);
    acb.writeInt64(ts1);
    acb.writeString(text);

    printf("MESSAGE CREATED, size = %d\n", acb.msgLength());
    fflush(0);

    sock.open(socketName,false,false);
    // sock << acb;
    printf("%d bytes written to the socket\n",DI_writeToSocket(&sock,&acb));
    sock.close();
  }
  
  // END WRITE; BEGIN READ

  if (server)
  {
    sock.open(socketName,true,false);
    sock.setBlocking(DI_BLOCK_FULL);
    // sock >> acb;
    printf("%d bytes read from the socket\n",DI_readFromSocket(&sock,&acb));
    sock.close();
  
    AC_MsgType type;
    uint size;
    
    size = acb.msgLength();
    acb.readHeader(&ts2, &msgID2, &type, &srcID2);
    
    if(type >= AC_MT_SIZEOF) {
      puts("BAD TYPE");
      exit(0);
    }

    if(ts1 != ts2) {
      puts("TIMESTAMP LOST");
      printf("ts1 = %llx, ts2 = %llx\n",ts1,ts2);
    }
    if(msgID1 != msgID2) {
      puts("MESSAGE ID LOST");
    }
    if(srcID1 != srcID2) {
      puts("SOURCE ID LOST");
    }
    
    printf("Type = \"%s\", size = %d\n", AC_MsgTypeKeys[type], size);
    
    float ef1 = acb.readFloat32() - f1;
    float ef2 = acb.readFloat32() - f2;
    
    int e1 = acb.readInt32() - i1;
    int e2 = acb.readInt32() - i2;
    int e3 = acb.readInt32() - i2;
    
    long long tserr = acb.readInt64() - ts1;
    
    printf("ef1 = %f, ef2 = %f, e1 = %d, e2 = %d, e3 = %d, tserr = %d \n",
	   ef1, ef2, e1, e2, e3, (int) tserr);
    
    fflush(0);
    
    char buf[DI_MAX_DATA_STRING];
    acb.readString(buf);
    puts(buf);
  }
    
  return 0;
}
