/* COPYRIGHT
 *
 * This file is part of the DIVA IO library.
 *
 * Author(s): Jarmo Hiipakka and Tommi Ilmonen, 2000.
 * Jarmo.Hiipakka@hut.fi Tommi.Ilmonen@hut.fi
 *
 * DIVA - Digital Interactive Virtual Acoustics group 
 * Telecommunications Software and Multimedia Laboratory
 * Helsinki University of Technology
 * Finland
 *
 * http://www.tcm.hut.fi/Research/DIVA/
 *
 * DIVA IO library is licensed under GNU Lesser General Public
 * License (LGPL). The LGPL conditions can be found in file "LGPL"
 * that is distributed with this source package or obtained from the
 * GNU organization (www.gnu.org).
 * 
 */


#include <stdlib.h>

#include <math.h>
#include <stdio.h>
#include <unistd.h>

#include "di_config.h"
#include "di_error.h"
#include "di_io_defs.h"
#include "di_sleep.h"
#include "di_audio_file_threaded.h"

#define DI_pi 3.14159
#define DI_2pi 3.14159 * 2

int DI_audioFileThreadedTest()
{
  const char * fname = "DI_audioFileThreadedTest";

  DI_AudioFileThreaded aft;

  const int blockSize = 32;
  const char * filename = "foo.wav";
  const int channels = 2;
  const int sr = 44100;
  const int mul = sr / blockSize + 15;
  const uint secs = 7;
  const uint loops = (secs * sr) / blockSize;
  double phase = 0.0;
  const double phaseVel = 3.0 * DI_2pi / (double) sr; // VERY LOW FREQ

  DI_Sample buf[channels * blockSize];

  DI_SleepSync ss;

  aft.startWrite(filename, DI_ASF_SAMPLE, channels, blockSize * mul);
  DI_trace(fname, "THREAD STARTED", ss.sleepSynchroUs(0));


  ss.sleepMs(500);
  ss.resetTiming();
  aft.userPrepare();
  
  DI_trace(fname, "THREAD PREPARED", ss.sleepSynchroUs(0));

  uint nextSleep = 0;

  for(uint i=0; i < loops; i++) {
    
    for(uint j=0; j < blockSize; j++) {
      MJ_Sample *base = & buf[j * channels];
      base[0] = sin(phase);
      base[1] = cos(phase);
      phase += phaseVel;
      if(phase > DI_2pi) phase -= DI_2pi;
    }

    if(aft.userFrames() > nextSleep) {
      ss.sleepMs(100);
      nextSleep = aft.userFrames() + sr / 4;
    }
    ss.resetTiming();
    aft.writeFrames(buf, blockSize);
    DI_trace(fname, "WROTE FRAME", ss.sleepSynchroUs(0));    
  }

  
  ss.resetTiming();
  aft.requestEnd();
  
  DI_trace(fname, "END REQUESTED", ss.sleepSynchroUs(0));

  ss.sleepMs(200);

  ss.resetTiming();
  aft.waitEnd();
 
  DI_trace(fname, "DONE", ss.sleepSynchroUs(0));

  return EXIT_SUCCESS;
}


extern int main(int argc, char **argv)
{
  return DI_audioFileThreadedTest();
}


