/* COPYRIGHT
 *
 * This file is part of the DIVA IO library.
 *
 * Author(s): Jarmo Hiipakka and Tommi Ilmonen, 2000.
 * Jarmo.Hiipakka@hut.fi Tommi.Ilmonen@hut.fi
 *
 * DIVA - Digital Interactive Virtual Acoustics group 
 * Telecommunications Software and Multimedia Laboratory
 * Helsinki University of Technology
 * Finland
 *
 * http://www.tcm.hut.fi/Research/DIVA/
 *
 * DIVA IO library is licensed under GNU Lesser General Public
 * License (LGPL). The LGPL conditions can be found in file "LGPL"
 * that is distributed with this source package or obtained from the
 * GNU organization (www.gnu.org).
 * 
 */


#include <stdlib.h>

#include <math.h>
#include <stdio.h>
#include <unistd.h>

#include "di_config.h"
#include "di_error.h"
#include "di_io_defs.h"
#include "di_sleep.h"
#include "di_audio_file_threaded.h"

#define DI_pi 3.14159
#define DI_2pi 3.14159 * 2

int DI_audioFileThreadedReadTest()
{
  const char * fname = "DI_audioFileThreadedReadTest";

  DI_AudioFileThreaded aft;

  const int blockSize = 89;
  const char * filename = "foo.wav";
  const char * filename2 = "foo2.wav";
  int channels = 2;
  int sr = 44100;
  const int mul = 10;
  const uint secs = 12;
  const uint loops = (secs * sr) / blockSize;

  DI_Sample buf[channels * blockSize];

  DI_SleepSync ss;

  aft.startRead(filename, 
		DI_ASF_SAMPLE, 
		blockSize * mul, 
		0, 1000000000, false);

  DI_trace(fname, "THREAD STARTED", ss.sleepSynchroUs(0));

  ss.sleepMs(500);
  ss.resetTiming();
  aft.userPrepare();
  
  DI_trace(fname, "THREAD PREPARED", ss.sleepSynchroUs(0));

  uint nextSleep = 0;

  DI_AudioFile af;
  
  af.setChannels(2);
  af.setSampleFormat(DI_AFSF_TWOSCOMP);
  af.setByteOrder(DI_AFBO_LITTLEENDIAN);
  af.open(filename2, DI_IO_OUTPUT, DI_ASF_SAMPLE, blockSize);

  for(uint i=0; i < loops; i++) {

    if(aft.userFrames() >= nextSleep) {
      ss.sleepMs(100);
      nextSleep = aft.userFrames() + sr / 4;
    }
    ss.resetTiming();
    aft.readFrames(buf, blockSize);
    DI_trace(fname, "READ FRAME", ss.sleepSynchroUs(0));    

    af.writeFrames(buf, blockSize);
    
  }

  
  ss.resetTiming();
  aft.requestEnd();
  
  DI_trace(fname, "END REQUESTED", ss.sleepSynchroUs(0));

  ss.sleepMs(200);

  ss.resetTiming();
  aft.waitEnd();
 
  DI_trace(fname, "DONE", ss.sleepSynchroUs(0));

  af.close();

  return EXIT_SUCCESS;
}


extern int main(int argc, char **argv)
{
  return DI_audioFileThreadedReadTest();
}


