/* COPYRIGHT
 *
 * This file is part of the DIVA IO library.
 *
 * Author(s): Jarmo Hiipakka and Tommi Ilmonen, 2000.
 * Jarmo.Hiipakka@hut.fi Tommi.Ilmonen@hut.fi
 *
 * DIVA - Digital Interactive Virtual Acoustics group 
 * Telecommunications Software and Multimedia Laboratory
 * Helsinki University of Technology
 * Finland
 *
 * http://www.tcm.hut.fi/Research/DIVA/
 *
 * DIVA IO library is licensed under GNU Lesser General Public
 * License (LGPL). The LGPL conditions can be found in file "LGPL"
 * that is distributed with this source package or obtained from the
 * GNU organization (www.gnu.org).
 * 
 */


#include <stdlib.h>

#include <math.h>
#include <stdio.h>
#include <unistd.h>

#include "di_config.h"
#include "di_error.h"
#include "di_io_defs.h"
#include "di_sleep.h"
#include "di_audio_port.h"

#define DI_pi 3.14159
#define DI_2pi 3.14159 * 2

int DI_audioTest()
{
  DI_AudioPort p;

  const int bufLen = 512;
  const int sr = 44100;
  const int chans = 2;
  const int times = (4 * chans * sr) / bufLen + 1;
  float phase1 = 0.0;
  float omega1 = DI_2pi * 440.0 / (float) sr;
  float phase2 = 0.0;
  float omega2 = omega1 * 1.7;
  
  float data[bufLen * 2];

  int i;

  char buf[1024];

#if defined(DI_AUDIO_IS_ALSA) || defined(DI_AUDIO_IS_IRIX)
  
  int inDevs = DI_AudioDevice::getNumberOfInputDevices();
  int outDevs = DI_AudioDevice::getNumberOfOutputDevices();

  char name[DI_AUDIO_MAX_NAME_LENGTH], 
    nameLong[DI_AUDIO_MAX_NAME_LENGTH];

  for(i=0; i < inDevs; i++) {
    DI_AudioDevice::getInputDeviceName(i, name);
    DI_AudioDevice::getDeviceLabel(i, nameLong);
    printf("Indevice %d - name = \"%s\" - label = \"%s\"\n",
	   i, name, nameLong);
  }
  for(i=0; i < outDevs; i++) {
    DI_AudioDevice::getOutputDeviceName(i, name);
    DI_AudioDevice::getDeviceLabel(i, nameLong);
    printf("Outdevice %d - name = \"%s\" - label = \"%s\"\n",
	   i, name, nameLong);
  }
  fflush(0);

#endif

  if(!p.open("foo", DI_IO_INPUT_OUTPUT, DI_ASF_FLOAT32, bufLen * 6, sr, chans, 
	     bufLen)) {
    DI_error(DI_ERR_INTERNAL, "int main()");
    return 0;
  }
  
  if(!p.start())
    DI_error(DI_ERR_INTERNAL, "int main() 2");

#if defined(DI_AUDIO_IS_ALSA)
  p.alsaDescribePlaybackSetup(buf);

  DI_trace(buf);
  
#endif

  for(i=0; i < times; i++) {

    for(int j=0; j < bufLen; j++) {
      phase1 += omega1;
      phase2 += omega2;

      if(phase1 > DI_2pi) phase1 -= DI_2pi;
      if(phase2 > DI_2pi) phase2 -= DI_2pi;
      
      if(chans == 2) {
	data[j*2] = sinf(phase1) * 0.6;
	data[j*2+1] = sinf(phase2) * 0.1;
      }
      else
	data[j] = sinf(phase2) * 0.1;

    }

    p.writeFrames(data, bufLen);
    // int fill = p.outFilledFrames();
    int fill = p.waitOutputFilled(bufLen * 2, 1000);

    if(i < 300)
      printf(" fill = %d ", fill);
  }
  fflush(0);

  DI_Sleep s;
  s.sleepMs(1000);

  return EXIT_SUCCESS;
}


extern int main(int argc, char **argv)
{
  return DI_audioTest();
}


