#! /bin/sh 

#
# Configuration / early installation script for Mustajuuri
#
# Copyright: Tommi Ilmonen (Tommi.Ilmonen@hut.fi) 2001
#

export OS=`uname -s`
export PWD_ORIG=`pwd`

if test $OS = IRIX64  ; then
  export OS=IRIX
fi

ln -s -f confbase_IRIX.sh confbase_IRIX64.sh

echo ""
echo "Platform base type is $OS"
export OSTYPE=`./scripts/mj_uname.sh` 

echo "Platform specification is $OSTYPE"

if test $OS = IRIX  ; then  
  export CONFDIR=IRIX-CC
  export SOURCEDIR=IRIX
  export SOURCEAUDIO=IRIX
  export SOURCEMIDI=IRIX
else
  source confbase_$OS.sh
fi



if test ! -e $OSTYPE ; then
    mkdir $OSTYPE
fi

cd $OSTYPE

ln -sf ../$CONFDIR/*.in ../$CONFDIR/Makelibs  ../$CONFDIR/Makeplugin  .

cd $PWD_ORIG

# Set up diva stuff:
if test ! -e divabase/$OSTYPE ; then
    mkdir divabase/$OSTYPE
fi
if test ! -e divaio/$OSTYPE ; then
    mkdir divaio/$OSTYPE
fi

cd divaio/$OSTYPE
ln -sf ../$SOURCEMIDI/di_midi*.[hC] .
ln -sf ../$SOURCEAUDIO/di_audio*.[hC] .

cd $PWD_ORIG

# MJ base directory

if test ! -e mj/$OSTYPE ; then
    mkdir mj/$OSTYPE
fi
cd mj/$OSTYPE/
ln -sf ../$SOURCEDIR/*.[hC] .

# Create a dir for the libraries
cd $MUSTAJUURI_PATH/lib/

if test ! -e $OSTYPE ; then
    mkdir $OSTYPE
fi

# Create a dir for the headers

if test ! -e $MUSTAJUURI_PATH/include/ ; then
    mkdir $MUSTAJUURI_PATH/include
fi

cd $MUSTAJUURI_PATH/include/

if test ! -e $OSTYPE ; then
    echo ""
    echo "Creating include directory for platform $OSTYPE"
    mkdir $OSTYPE
fi

if test ! -e $MUSTAJUURI_PATH/bin ; then
    echo "Creating $MUSTAJUURI_PATH/bin"
    mkdir $MUSTAJUURI_PATH/bin
fi

if test ! -e $MUSTAJUURI_PATH/bin/$OSTYPE ; then
    echo "Creating $MUSTAJUURI_PATH/bin/$OSTYPE"
    mkdir $MUSTAJUURI_PATH/bin/$OSTYPE
fi

#if test ! -e $MUSTAJUURI_PATH/bin/$OSTYPE/mustajuuri; then
#    cd $MUSTAJUURI_PATH/bin/$OSTYPE
#    ln -sf ../../src/mj/$OSTYPE/dmain mustajuuri
#fi

# if test ! -e $MUSTAJUURI_PATH/bin/$OSTYPE/mustajuuri; then
#     cd $MUSTAJUURI_PATH/bin/$OSTYPE
#     ln -sf ../../src/mj/$OSTYPE/dmain mustajuuri-bin
# fi

# Create a dir for the plugins
cd $MUSTAJUURI_PATH/plugins/

if test ! -e $OSTYPE ; then
    mkdir $OSTYPE
fi

cd $PWD_ORIG

# Set up the permission scripts:

TMLHOST=`hostname|grep tml.hut.fi`

if test ! -z $TMLHOST ; then
  echo ""

  echo "HUT TML environment, will use group vsop and group write permissions"
  echo "MAKE WILL NOW CHANGE GROUP AND PERMISSIONS OF BINARIES"

  echo "You may want to give vsop group the relevant permissions:"
  echo "chgrp -R vsop ../"
  echo "chmod -R g+w ../"
  cp give_group_permissions_tml.sh give_normal_permissions.sh
  cp give_group_permissionsx_tml.sh give_exec_permissions.sh
  # chgrp -R vsop $MUSTAJUURI_PATH
  # chmod -R g+w $MUSTAJUURI_PATH
  chmod +x *.sh
else
  echo "Normal environment"
  cp blank.sh give_normal_permissions.sh
  cp blank.sh give_exec_permissions.sh
  chmod +x *.sh
fi

# Help the user to get the path variable right:
cd $PWD_ORIG/..
export CURR_PWD=`pwd`
echo ""
echo "You may want to set your MUSTAJUURI_PATH to $CURR_PWD"
echo "Use: 'export MUSTAJUURI_PATH=$CURR_PWD'"

# Return to the original dir. Necessary if the user simply "source"d
# this file.

cd $PWD_ORIG

# Remove old config crap and give a helpful hint to the user:

if test -e config.cache; then
  echo ""
  echo "Removing old config.cache"
  rm -rf config.cache
fi

if test -z $QTDIR ; then
  echo ""
  echo "Environment variable QTDIR is not set!!!"
  echo ""
  echo "Please set QTDIR if you want full Mustajuuri installation."
  echo "Otherwise you will not be able to compile all Mustajuuri libraries."
  echo "You will be able to compile the divabase, divaio and mjdsp libraries."
  echo "Typically QTDIR is '/usr/lib/qt2', '/usr/src/qt3' or something similar."
  echo ""
fi

echo ""
echo "You can now configure Mustajuuri with: './configure'"
echo ""
