/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of DIVA base library.
 *
 * Author(s): Jarmo Hiipakka and Tommi Ilmonen, 2000-2001.
 * Jarmo.Hiipakka@hut.fi Tommi.Ilmonen@hut.fi
 *
 * DIVA - Digital Interactive Virtual Acoustics group 
 * Telecommunications Software and Multimedia Laboratory
 * Helsinki University of Technology
 * Finland
 *
 * http://www.tml.hut.fi/Research/DIVA/
 *
 * DIVA base library is licensed under GNU Lesser General Public
 * License (LGPL). The LGPL conditions can be found in file "LGPL"
 * that is distributed with this source package or obtained from the
 * GNU organization (www.gnu.org).
 * 
 */

#ifndef DI_SLEEP_H
#define DI_SLEEP_H
#include <di_types.h>

#define DI_BILLION 1000000000
#define DI_MILLION 1000000

class DI_Mutex;
class DI_TimeStamp;

enum DI_Timing {
  DI_TIMING_GOOD,
  DI_TIMING_LATE,
  DI_TIMING_EARLY
};


/** Sleeping services. This class contains only static member
    functions. The constructor and destructor are included to prevent
    compiler warnings.*/
class DI_Sleep
{
public:

  DI_Sleep() { }

  ~DI_Sleep() {}

  /// Sleep for n seconds.
  static bool sleepS(uint secs); 

  /** Sleep for n milliseconds. You cannot sleep more than one second
      with this function. */
  static bool sleepMs(uint msecs);

  /** Sleep for n microseconds. You cannot sleep more than one second
      with this function. */
  static bool sleepUs(uint usecs);
};

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
/*
/// Synchronized sleeping.
class DI_SleepSync
{
public:
  /// The constructor resets the timing.
  DI_SleepSync() { resetTiming(); }

  /// Resets the reference time to current time.
  void resetTiming() 
  { gettimeofday(& m_timing, 0); m_initial = m_timing; }
  
  long sleepSynchroUs(long us);

  long sleepTo(const DI_TimeStamp *stamp, DI_Mutex *mutex = 0);

protected:  
  struct timeval m_initial, m_timing;
};
*/
#endif
