/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of DIVA base library.
 *
 * Author(s): Jarmo Hiipakka and Tommi Ilmonen, 2000-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * DIVA - Digital Interactive Virtual Acoustics group 
 * Telecommunications Software and Multimedia Laboratory
 * Helsinki University of Technology
 * Finland
 *
 * http://www.tml.hut.fi/Research/DIVA/
 *
 * DIVA base library is licensed under GNU Lesser General Public
 * License (LGPL). The LGPL conditions can be found in file "LGPL"
 * that is distributed with this source package or obtained from the
 * GNU organization (www.gnu.org).
 * 
 */

#ifndef DI_ENDIAN_H
#define DI_ENDIAN_H

#include "di_types.h"

/* This file contains code to determine the endianness of the
   operating system. If the system is big-endian, then macro
   WORDS_BIGENDIAN will be defined. Otherwise macro
   WORDS_LITTLEENDIAN will be defined. 

   This file also defines DIVA versions of a few common macros. This
   is needed to to bypass VR Juggler (or Performer ?) bugs.
*/

#if defined(__sgi)
// All SGI is currently big-endian
#define WORDS_BIGENDIAN 1
#define DI_ntohl(x) x
#define DI_htonl(x) x

#elif defined(__linux__) 

// This endian check only works on Linux (?)

#include <endian.h>
#include <netinet/in.h>


#define DI_ntohl(x) ntohl(x)
#define DI_htonl(x) htonl(x)
#define DI_ntohs(x) ntohs(x)
#define DI_htons(x) htons(x)

#if __BYTE_ORDER == __LITTLE_ENDIAN

#undef WORDS_BIGENDIAN
#define WORDS_LITTLEENDIAN 1

#else

#define WORDS_BIGENDIAN 1

#endif   // __BYTE_ORDER == __LITTLE_ENDIAN

#elif defined(WIN32) //WIN32

#define WORDS_LITTLEENDIAN 1

#else

#error "PLATFORM NOT (YET) SUPPORTED"

#endif   // defined(__sgi)

inline short DI_byteSwap2(short i)
{ 
  return (i << 8) + (i >> 8);
}

inline int DI_byteSwap4(uint i)
{ 
  return ((i&0xff) << 24) + ((i&0xff00) << 8) + 
    ((i&0xff0000) >> 8) + ((i&0xff000000) >> 24);
}

inline ullong DI_byteSwap8(ullong x)
{ 
#ifndef WIN32
  return ((x << 56) | ((x & 0xff00ll) << 40) | ((x & 0xff0000ll) << 24) |
	  ((x & 0xff000000ll) << 8) | ((x >> 8) & 0xff000000ll) |
	  ((x >> 24) & 0xff0000ll) | ((x >> 40) & 0xff00ll) | (x >> 56));
#else
  return ((x << 56) | ((x & 0xff00) << 40) | ((x & 0xff0000) << 24) |
	  ((x & 0xff000000) << 8) | ((x >> 8) & 0xff000000) |
	  ((x >> 24) & 0xff0000) | ((x >> 40) & 0xff00) | (x >> 56));
#endif
}

#endif   // DI_ENDIAN_H
