/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of DIVA base library.
 *
 * Author(s): Jarmo Hiipakka and Tommi Ilmonen, 2000-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * DIVA - Digital Interactive Virtual Acoustics group 
 * Telecommunications Software and Multimedia Laboratory
 * Helsinki University of Technology
 * Finland
 *
 * http://www.tml.hut.fi/Research/DIVA/
 *
 * DIVA base library is licensed under GNU Lesser General Public
 * License (LGPL). The LGPL conditions can be found in file "LGPL"
 * that is distributed with this source package or obtained from the
 * GNU organization (www.gnu.org).
 * 
 */

#ifndef DI_TIME_H
#define DI_TIME_H

#ifndef WIN32
#include <sys/time.h>
#else
#include <winsock.h>
#include <di_types.h>
#endif

/** Time measurement. */
class DI_Time
{
public:
  DI_Time() {}
  ~DI_Time() {}

  /// Gets the current time.
  void getTimeOfDay() { 
#ifndef WIN32
	  gettimeofday(&m_time, 0); 
#else
	int temp=timeGetTime();
	m_time.tv_sec=temp/1000;
	m_time.tv_usec=temp*1000;
#endif
  }

  /// Compare two times (returns seconds)
  double secsTo(const DI_Time &t) const
  { return (t.m_time.tv_sec - m_time.tv_sec) + 
      (t.m_time.tv_usec - m_time.tv_usec) * 0.000001; }

  /// Compare two times (returns microseconds)
  double usecsTo(const DI_Time &t) const
  { return (t.m_time.tv_sec - m_time.tv_sec) * 1000000.0+ 
      (t.m_time.tv_usec - m_time.tv_usec); }

  double secsSince() 
  { DI_Time tmp; tmp.getTimeOfDay(); return secsTo(tmp); }

protected:
  struct timeval m_time;
};


#endif

