x/* COPYRIGHT
 *
 * This file is part of DIVA base library.
 *
 * Author(s): Jarmo Hiipakka and Tommi Ilmonen, 2000-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * DIVA - Digital Interactive Virtual Acoustics group 
 * Telecommunications Software and Multimedia Laboratory
 * Helsinki University of Technology
 * Finland
 *
 * http://www.tml.hut.fi/Research/DIVA/
 *
 * DIVA base library is licensed under GNU Lesser General Public
 * License (LGPL). The LGPL conditions can be found in file "LGPL"
 * that is distributed with this source package or obtained from the
 * GNU organization (www.gnu.org).
 * 
 */

#include "di_condition.h"
#include "di_data_buffer.h"
#include "di_error.h"
#include "di_sleep.h"
#include "di_socket.h"
#include "di_thread.h"

// #include "di_thread_data.h"


#include <stdio.h>

void DI_threadTest()
{
  DI_ThreadTest *thread;
  DI_Sleep sleep;
  
  thread = new DI_ThreadTest();
  thread->run();
  puts("Sleep for 10 seconds");
  sleep.sleepS(10);
  printf("%f", (float)thread->count());
  puts("Sleep for 1 second");
  sleep.sleepS(1);
  printf("%f", (float)thread->count());
  thread->requestEnd();
}

// static const char * addr = "127.0.0.1:10025";
static const char * addr = "10025";
// static const char * addr = "130.234.46.56:10025";

void DI_listenTest()
{
  DI_Sleep sleep;

  DI_Socket socket;
  bool ok = socket.open(addr, true, true);

  if(!ok) {
    puts("socket.open FAILED");
    return;
  }

  while(true) {

    if(socket.isPendingInput()) {

      puts("Enter listening mode");
      DI_Socket *newSocket = socket.listen();

      if(newSocket) {

	puts("Got new socket");
	DI_DataBuffer2 buf;
	std::string rstr;
	
	DI_readFromSocket(newSocket, &buf);
	buf.rewind();

	int r1 = buf.readInt32();
	char c = buf.readByte();
	llong r2 = buf.readInt64();
	float r3 = buf.readFloat32();
	buf.readString(&rstr);
	
	short r5 = buf.readInt16();
	
	DI_trace2("Read following data: %d %c %d %f \"%s\" %d", 
		  r1, c, (int) r2, r3,
		  rstr.c_str(), (int) r5);
	
	
	DI_trace2("Socket was hung up = %d", (int) newSocket->isHungUp());

	sleep.sleepS(3);
	puts("DONE");

	DI_trace2("Socket was hung up = %d", (int) newSocket->isHungUp());

      }
      else 
	puts("Socket listen failed");

      return;
    }
    puts("One Loop");
    sleep.sleepS(1);
  }
}

void DI_clientTest()
{
  DI_Sleep sleep;
  
  DI_Socket socket;
  bool ok = socket.open(addr, false, true);
  if(ok)
    puts("Client socket well opened");
  else
    puts("Client socket could NOT BE OPENED");

  DI_DataBuffer2 buf;
  
  std::string str("Arhippa Perttunen");

  buf.writeInt32(7);
  buf.writeByte('X');
  buf.writeInt64(77);
  buf.writeFloat32(1.0 / 3.0);

  buf.writeString(&str);
  buf.writeInt16(9999);

  DI_trace2("Buffer has now %d bytes of data", buf.position());
  DI_writeToSocket(&socket, & buf);
  socket.flush();
  DI_trace("Wrote to socket, now sleeping");
  DI_Sleep::sleepS(2);
  DI_trace("DONE");  
}

void DI_strTest()
{
  std::string str("1234");

  printf("String \"%s\" takes %d bytes\n", str.c_str(), str.size());
  fflush(0);
}

void DI_bufferTest()
{
  // DI_DataBuffer2 *buf = new DI_DataBuffer2;

  std::string rstr, str("test.wav");
  
  DI_DataBuffer2 buf;

  buf.writeInt32(7);
  buf.writeByte('b');
  buf.writeInt64(77);
  buf.writeFloat32(0.33333);

  buf.writeString(&str);
  buf.writeInt16(9999);

  DI_trace2("Buffer has now %d bytes of data", buf.position());
  
  buf.rewind();

  int r1 = buf.readInt32();
  buf.readByte();
  llong r2 = buf.readInt64();

  float r3 = buf.readFloat32();
  buf.readString(&rstr);

  short r5 = buf.readInt16();
  
  DI_trace2("Read following data: %d %d %f \"%s\" %d", r1, (int) r2, r3,
	    rstr.c_str(), (int) r5);
}

int main(int argc, char **argv)
{
  const char * mode = "-t";

  if(argc > 1) {
    mode = argv[1];
  }

  if(!strcmp(mode, "-t"))
    DI_threadTest();
  if(!strcmp(mode, "-t2"))
    DI_Thread::test();
  else if(!strcmp(mode, "-l"))
    DI_listenTest();
  else if(!strcmp(mode, "-c"))
    DI_clientTest();
  else if(!strcmp(mode, "-s"))
    DI_strTest();
  else if(!strcmp(mode, "-b"))
    DI_bufferTest();
  else 
    puts("Unrecognized argument");

  return 0;
}
