/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of DIVA base library.
 *
 * Author(s): Jarmo Hiipakka and Tommi Ilmonen, 2000-2001.
 * Jarmo.Hiipakka@hut.fi Tommi.Ilmonen@hut.fi
 *
 * DIVA - Digital Interactive Virtual Acoustics group 
 * Telecommunications Software and Multimedia Laboratory
 * Helsinki University of Technology
 * Finland
 *
 * http://www.tml.hut.fi/Research/DIVA/
 *
 * DIVA base library is licensed under GNU Lesser General Public
 * License (LGPL). The LGPL conditions can be found in file "LGPL"
 * that is distributed with this source package or obtained from the
 * GNU organization (www.gnu.org).
 * 
 */

#include "di_mutex.h"
#include "di_error.h"

#include <stdio.h>


DI_Mutex::DI_Mutex()
  : m_mutex(0)
{}

DI_Mutex::~DI_Mutex()
{
  delete m_mutex;
}


bool DI_Mutex::close()
{
  if(m_mutex) {
    delete m_mutex;
    m_mutex = 0;
    return true;
  }
  return false; // Nothing to close
}

bool DI_Mutex::lock()
{ 
  if(!m_mutex) return false;

  m_mutex->lock();

  return true;
}

bool DI_Mutex::lock(bool block)
{
  if(!block) return tryLock();
  else return lock();
}

bool DI_Mutex::tryLock() 
{ 
#if (QT_VERSION < 300)
  DI_trace2("QMutex::tryLock not implemented prior to QT3\n");
  return false;
#else
  if(!m_mutex) return false;

  return m_mutex->tryLock();
#endif
}

bool DI_Mutex::unlock()
{
  if(!m_mutex) return false;

  m_mutex->unlock();

  return true;
}

void DI_Mutex::test()
{
  puts("DI_Mutex::test # UNIMPLEMENTED IN QTHREADS");
}

DI_Mutex &DI_Mutex::operator = (const DI_Mutex &foo)
{
  (void) foo;
  return *this;
}
