/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of DIVA base library.
 *
 * Author(s): Jarmo Hiipakka and Tommi Ilmonen, 2000.
 * Jarmo.Hiipakka@hut.fi Tommi.Ilmonen@hut.fi
 *
 * DIVA - Digital Interactive Virtual Acoustics group 
 * Telecommunications Software and Multimedia Laboratory
 * Helsinki University of Technology
 * Finland
 *
 * http://www.tml.hut.fi/Research/DIVA/
 *
 * DIVA base library is licensed under GNU Lesser General Public
 * License (LGPL). The LGPL conditions can be found in file "LGPL"
 * that is distributed with this source package or obtained from the
 * GNU organization (www.gnu.org).
 * 
 */

#ifndef DI_MUTEX_H
#define DI_MUTEX_H

#include <qglobal.h>

#if (QT_VERSION==230)
#include <qthread.h>
#else
#include <qmutex.h>
#endif

/// Mutex class.
class DI_Mutex
{
public:
  ///
  DI_Mutex();

  ///
  virtual ~DI_Mutex();
  
  /** Initialize the mutex. 
   
      Setting all values to false gives usually the best performance.

      The boolean arguments refer to POSIX-functionality with the same
      name. Not all features work on all platforms how-ever: My Linux
      box does not know anything about inheriting priorities. 

      Recursion should work on all platforms. */
  bool init(bool shared = false, 
	    bool prio_inherit = true,
	    bool recursive = true)
  { 
    (void) shared; 
    (void) prio_inherit; 
    delete m_mutex; 
    m_mutex = new QMutex(recursive); 
    return m_mutex;
  }
  
  /// Close the mutex.
  bool close();

  /** Locks the mutex. Blocks until mutex is available. */
  bool lock(); 

  /// Lock the mutex, optionally blocking.
  bool lock(bool block);

  /** Tries to lock the mutex. Does not block. */
  bool tryLock();

  /// Unlocks the mutex.
  bool unlock();

  /// Self test.
  static void test();

  
  QMutex * qmutex() { return m_mutex; }

protected:
  /// Disabled
  DI_Mutex(const DI_Mutex &foo)
  { (void) foo; } 
  /// Disabled
  DI_Mutex &operator = (const DI_Mutex &);

  QMutex         *m_mutex;

};

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

/// Mutex that initializes automatically.
class DI_MutexAuto : public DI_Mutex
{
public:
  /// Calls init.
  DI_MutexAuto(bool shared = false, 
	       bool prio_inherit = true,
	       bool recursive = true) 
  { init(shared, prio_inherit, recursive); }
  ///
  ~DI_MutexAuto() {}
};


#endif
