/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of the DIVA IO library.
 *
 * Author(s): Jarmo Hiipakka and Tommi Ilmonen, 2000-2001.
 * Jarmo.Hiipakka@hut.fi Tommi.Ilmonen@hut.fi
 *
 * DIVA - Digital Interactive Virtual Acoustics group 
 * Telecommunications Software and Multimedia Laboratory
 * Helsinki University of Technology
 * Finland
 *
 * http://www.tml.hut.fi/Research/DIVA/
 *
 * DIVA IO library is licensed under GNU Lesser General Public
 * License (LGPL). The LGPL conditions can be found in file "LGPL"
 * that is distributed with this source package or obtained from the
 * GNU organization (www.gnu.org).
 * 
 */

#ifndef DI_MIDI_IO_H
#define DI_MIDI_IO_H

#include "di_midi_event.h"

#include "di_io_defs.h"

#include <dmedia/midi.h>
#include <poll.h>

#define DI_MIDI_IS_IRIX 1

extern const char *DI_defaultMidiDev();

bool DI_irixMdToMjMidi(DI_MidiEvent *, const MDevent *);

/** Midi input/output. */
class DI_MidiIo
{
public:
  DI_MidiIo();
  ~DI_MidiIo();

  /** Returns the name of the port. */
  const char *portName() const { return m_portName; }

  /// Open the port.
  bool open(const char * port,
	    DI_IoMode);

  /// Close the port.
  bool close();

  /// Returns true if the port is open.
  bool isInput() const { return m_inPort != 0; }
  bool isOutput() const { return m_outPort != 0; }
  void resetTiming();

  /** Returns true is there is a MIDI event available for reading. */
  bool eventPending();

  /** Reads a MIDI event. Blocks until the event is received. If you
      want to avoid blocking, use the method "eventPending" to check
      there really is an event tto be read. */
  bool readEvent(DI_MidiEvent *);
  
  /** Write a MIDI event to the port. The time stamp of the event is
      used determine when it will be passed to the port.*/
  bool writeEvent(DI_MidiEvent *);

protected:
  char         *m_portName;
  DI_IoMode     m_mode;

  MDport        m_inPort;
  MDport        m_outPort;
  pollfd        m_pollFd;
  
  MDevent       m_event;
};



#endif
