/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of the DIVA IO library.
 *
 * Author(s): Jarmo Hiipakka and Tommi Ilmonen, 2000-2001.
 * Jarmo.Hiipakka@hut.fi Tommi.Ilmonen@hut.fi
 *
 * DIVA - Digital Interactive Virtual Acoustics group 
 * Telecommunications Software and Multimedia Laboratory
 * Helsinki University of Technology
 * Finland
 *
 * http://www.tml.hut.fi/Research/DIVA/
 *
 * DIVA IO library is licensed under GNU Lesser General Public
 * License (LGPL). The LGPL conditions can be found in file "LGPL"
 * that is distributed with this source package or obtained from the
 * GNU organization (www.gnu.org).
 * 
 */


#ifndef DI_MIDI_IO_H
#define DI_MIDI_IO_H

#include <di_io_defs.h>
#include <di_midi_event.h>

#include <alsa/asoundlib.h>

#include <string>

#define DI_MIDI_IS_ALSA 1

extern const char *DI_defaultMidiDev();

/** MIDI IO class for ALSA 0.9 and greater. */
class DI_MidiIo
{
public:
  DI_MidiIo();
  ~DI_MidiIo();

  /** Returns the name of the port. */
  const char *portName() const { return m_portName.c_str(); }

  bool open(const char * port, DI_IoMode);
  bool close();
  bool isInput() const { return m_mode & DI_IO_INPUT; }
  bool isOutput() const { return m_mode & DI_IO_OUTPUT; }
  void resetTiming();
  bool eventPending();
  bool readEvent(DI_MidiEvent *);
  bool writeEvent(DI_MidiEvent *);

protected:
  std::string m_portName;

  DI_IoMode      m_mode;

  snd_seq_t    * m_seq;
};

#endif
