/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of the DIVA IO library.
 *
 * Author(s): Jarmo Hiipakka and Tommi Ilmonen, 2000-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * DIVA - Digital Interactive Virtual Acoustics group 
 * Telecommunications Software and Multimedia Laboratory
 * Helsinki University of Technology
 * Finland
 *
 * http://www.tml.hut.fi/Research/DIVA/
 *
 * DIVA IO library is licensed under GNU Lesser General Public
 * License (LGPL). The LGPL conditions can be found in file "LGPL"
 * that is distributed with this source package or obtained from the
 * GNU organization (www.gnu.org).
 * 
 */

//
// DIVA Library. Copyright (C) Jarmo Hiipakka and Tommi Ilmonen, 1999-2001.
//
// $Log: di_audio_file.C,v $
// Revision 1.14  2004/11/30 13:51:46  tilmonen
// Copyright upodates...
//
// Revision 1.13  2004/11/30 13:14:38  tilmonen
// Copyright updates.
//
// Revision 1.12  2004/11/30 12:58:00  tilmonen
// Updated license text.
//
// Revision 1.11  2004/02/06 15:47:53  tilmonen
// Moved many data types from "ulong" to "uint".
//
// Revision 1.10  2002/09/16 06:28:36  tilmonen
// Bug fixes and minor improvements.
//
// Revision 1.9  2002/09/04 13:24:02  tilmonen
// Bug fixes.
//
// Revision 1.8  2002/08/26 12:54:32  tilmonen
// Bug fixes.
//
// Revision 1.7  2002/08/26 10:56:38  tilmonen
// Bug fixes.
//
// Revision 1.6  2002/08/19 12:45:59  tilmonen
// Merged old changes and fixed a few bugs shown by MipsPro compiler.
//
// Revision 1.5  2002/08/19 11:34:36  tilmonen
// Simplified the audio file class operations. Now much more usable.
//
// Revision 1.4  2002/08/09 07:52:37  tilmonen
// Fixes / additions.
//
// Revision 1.3  2001/07/27 08:23:26  tilmonen
// License text updates (=cosmetic changes).
//
// Revision 1.2  2001/06/06 10:34:58  tilmonen
// Updates for new glibc include strategies and libaudiofile-relaed fixes.
//
// Revision 1.1.1.1  2001/06/04 09:04:20  tilmonen
// Mustajuuri 4.6.2001, initial CVS cersion
//

#include <stdlib.h>
#include <string.h>
#include <stdio.h>

#include <di_error.h>
#include <di_audio_file.h>
// #include "di_numconv.h"

DI_AudioFile::DI_AudioFile() :
  m_totFrames(0),
  m_fHandle(AF_NULL_FILEHANDLE),
  m_fileSFormat(AF_SAMPFMT_TWOSCOMP),
  m_fileSWidth(16)
{
  m_fSetup = afNewFileSetup();
}

DI_AudioFile::~DI_AudioFile(void)
{
  if(m_mode != DI_IO_NONE)
    close();

  afFreeFileSetup(m_fSetup);
}

DI_AudioFileFormat DI_AudioFile::getFileFormat() const
{
  int version(0);
  if (m_fHandle != AF_NULL_FILEHANDLE)
    return (DI_AudioFileFormat)afGetFileFormat(m_fHandle,&version);
  else
    return DI_AFF_UNKNOWN;
}

bool DI_AudioFile::open(const char *file,
			DI_IoMode mode,
			DI_AudioSampleFormat userFormat,
			ulong maxBlock)
{
  (void) maxBlock;

  if (m_fHandle != AF_NULL_FILEHANDLE)
    close();

  if (!file)
  {
    DI_error(DI_ERR_NULL_ARGUMENT, "DI_AudioFile::open", "NULL file name");
    return false;
  }

  m_name = strdup(file);
  
  if (mode == DI_IO_INPUT)
  {
    m_fHandle = afOpenFile(file, "r", m_fSetup);
    // m_totFrames = (long)afGetFrameCount(m_fHandle,AF_DEFAULT_TRACK);
  }
  else if (mode == DI_IO_OUTPUT)
  {
    m_fHandle = afOpenFile(file, "w", m_fSetup);
  }
  else
  {
    DI_error(DI_ERR_ARGUMENT,"DI_AudioFile::open", "I/O mode not supported");
    return false;
  }
  m_mode = mode;

  double rate(0);
  int ch(0);

  if ((rate = afGetRate(m_fHandle,AF_DEFAULT_TRACK)) < 0)
  {
    m_inRate = DI_AUDIO_UNSPECIFIED_RATE;
    m_outRate = DI_AUDIO_UNSPECIFIED_RATE;
  }
  else
  {
    m_inRate = (m_mode == DI_IO_INPUT) ? (uint)rate : 0;
    m_outRate = (m_mode == DI_IO_OUTPUT) ? (uint)rate : 0;
  }

  if ((ch = afGetChannels(m_fHandle,AF_DEFAULT_TRACK)) < 0)
  {
    DI_error(DI_ERR_INTERNAL,"DI_AudioFile::open",
	     "Could not determine the number of channels");
    close();
    return false;
  }
  else
  {
    m_inChannels = (m_mode == DI_IO_INPUT) ? (uint)ch : 0;
    m_outChannels = (m_mode == DI_IO_OUTPUT) ? (uint)ch : 0;
  }
  afGetSampleFormat(m_fHandle,AF_DEFAULT_TRACK,&m_fileSFormat,&m_fileSWidth);
  
#ifdef WORDS_BIGENDIAN
  afSetVirtualByteOrder(m_fHandle,AF_DEFAULT_TRACK,AF_BYTEORDER_BIGENDIAN);
#else
  afSetVirtualByteOrder(m_fHandle,AF_DEFAULT_TRACK,AF_BYTEORDER_LITTLEENDIAN);
#endif

  setUserSampleFormat(userFormat);

  m_inFormat = userFormat;
  m_outFormat = userFormat;

  return true;
}

bool DI_AudioFile::close(void)
{
  const char * fname = "DI_AudioFile::close";

  if (afCloseFile(m_fHandle))
  {
    DI_error(DI_ERR_INTERNAL,fname,"Error closing audio file");
    return false;
  }
  
  // DI_trace(fname, "file closed");

  if (m_name)
  {
    free(m_name);
    m_name = NULL;
  }
  
  // DI_trace(fname, "name freed");

  m_mode = DI_IO_NONE;
  m_inFormat = DI_ASF_SAMPLE;
  m_outFormat = DI_ASF_SAMPLE;
  m_inChannels = 0;
  m_outChannels = 0;
  m_inRate = DI_AUDIO_UNSPECIFIED_RATE;
  m_outRate = DI_AUDIO_UNSPECIFIED_RATE;
  m_fHandle = AF_NULL_FILEHANDLE;
  return true;
}

int DI_AudioFile::readFrames(void *buffer, uint frames)
{
  if(m_mode != DI_IO_INPUT)
    return -1;

  return afReadFrames(m_fHandle, AF_DEFAULT_TRACK, buffer, (int)frames);
}

int DI_AudioFile::writeFrames(const void *buffer, uint frames)
{
  if (m_mode != DI_IO_OUTPUT)
    return -1;

  return afWriteFrames(m_fHandle,AF_DEFAULT_TRACK,(void *)buffer,(int)frames);

}

bool DI_AudioFile::isOK(void) const
{
  if (m_fHandle != AF_NULL_FILEHANDLE)
    if ((m_mode == DI_IO_INPUT) && !isEOF())
      return true;
    else if (m_mode == DI_IO_OUTPUT)
      return true;
  return false;
}

bool DI_AudioFile::isEOF(void) const
{
  if (m_mode == DI_IO_INPUT)
    return (currentFrame() >= frameCount());
  else
    return false;
}

/** Seeks the peak amplitude of the file. Returns a double between
    zero and one.
    
    If the \b verbose argument is true, then dots will be printed to
    the screen so that the user knows that something is happening.
*/

double DI_AudioFile::filePeak(const char * fileName, bool verbose)
{
  const char * fname = "DI_AudioFile::filePeak";

  const int blockSize = 8192;

  DI_AudioFile file;
  if(!file.open(fileName, DI_IO_INPUT, DI_ASF_FLOAT64, blockSize)) {
    DI_error(DI_ERR_ARGUMENT, fname, "Could not open file");
    return 1.0;
  }

  int chans = (int) file.inChannels();

  double * buf = new double[chans * blockSize];

  uint frames = file.frameCount();

  uint loops = frames / blockSize;
  uint blockSamples = chans * blockSize;

  double peak = 0.0;

  uint informInterval = loops / 10;

  for(uint i=0; i < loops; i++) {
    file.readFrames(buf, blockSize);

    double *data = buf;

    for(uint j=0; j < blockSamples; j++) {
      double tmp = *data++;
      if(tmp < 0.0) tmp *= -1.0;

      if(tmp > peak) peak = tmp;
    }
    if((i+1) % informInterval == 0 && verbose) {
      printf("."); 
      fflush(0);
    }
  }
  delete [] buf;
  return peak;
}

static const char * DI_AFFormatKeys[] =  {
  "raw",
    "aifc",
    "aiff",
    "next",
    "wav",
};

int DI_AudioFile::determineFormat(const char *formatName)
{
  DI_trace("DI_AudioFile::determineFormat", formatName);

  if(!formatName) return AF_FILE_UNKNOWN;

  int ftype = AF_FILE_UNKNOWN;

  const int sentinel = 1000;

  for(int i=0; i < sentinel; i++) {
    if(strlen(DI_AFFormatKeys[i]) < 1) {
      i = sentinel;
    }
    else if(!strcmp(formatName, DI_AFFormatKeys[i])) {
      ftype = i;
      i = sentinel;      
    }
  }
  
  if(ftype == AF_FILE_UNKNOWN) {
    if(!strcmp(formatName, "aif"))
      return AF_FILE_AIFF;
  }
  
  DI_trace("DI_AudioFile::determineFormat", "EXIT");

  return ftype;
}


const char *DI_AudioFile::formatStr(DI_AudioFileFormat f)
{
  int ff = (int) f;

  if(ff <= DI_AFF_WAVE) return DI_AFFormatKeys[ff];

  return 0;
}

