/* COPYRIGHT
 *
 * This file is part of the DIVA IO library.
 *
 * Author(s): Jarmo Hiipakka and Tommi Ilmonen, 2000-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * DIVA - Digital Interactive Virtual Acoustics group 
 * Telecommunications Software and Multimedia Laboratory
 * Helsinki University of Technology
 * Finland
 *
 * http://www.tml.hut.fi/Research/DIVA/
 *
 * DIVA IO library is licensed under GNU Lesser General Public
 * License (LGPL). The LGPL conditions can be found in file "LGPL"
 * that is distributed with this source package or obtained from the
 * GNU organization (www.gnu.org).
 * 
 */

#include <di_image_conversion.h>

#include <di_types.h>

#include <string.h>

bool DI_ImageConversion::convert(const DI_Image * source, DI_Image * target)
{
  bool ok = true;
  
  DI_ImageFormat sourceFmt = source->m_format;
  DI_ImageFormat targetFmt = target->m_format;

  if(sourceFmt == DI_IMAGE_GRAYSCALE) {
    if(targetFmt == DI_IMAGE_RGB)
      grayscaleToRGB(source, target);
    else 
      ok = false;
  }
  else if(sourceFmt == DI_IMAGE_YUV_411) {
    if(targetFmt == DI_IMAGE_GRAYSCALE)
      YUV411ToGrayscale(source, target);
    else if(targetFmt == DI_IMAGE_RGB)
      YUV411ToRGB(source, target);
    else 
      ok = false;
  }
  else if(sourceFmt == DI_IMAGE_YUV_420P) {
    if(targetFmt == DI_IMAGE_GRAYSCALE)
      YUV420PToGrayscale(source, target);
    else 
      ok = false;    
  }
  else 
    ok = false;

  return ok;
}

// color conversion functions from Bart Nabbe.
// corrected by Damien: bad coeficients in YUV2RGB
/** These macros were copied from the Coriander. */
#define YUV2RGB(y, u, v, r, g, b)\
  r = y + ((v*1436) >> 10);\
  g = y - ((u*352 + v*731) >> 10);\
  b = y + ((u*1814) >> 10);\
  r = r < 0 ? 0 : r;\
  g = g < 0 ? 0 : g;\
  b = b < 0 ? 0 : b;\
  r = r > 255 ? 255 : r;\
  g = g > 255 ? 255 : g;\
  b = b > 255 ? 255 : b

#define RGB2YUV(r, g, b, y, u, v)\
  y = (306*r + 601*g + 117*b)  >> 10;\
  u = ((-172*r - 340*g + 512*b) >> 10)  + 128;\
  v = ((512*r - 429*g - 83*b) >> 10) + 128;\
  y = y < 0 ? 0 : y;\
  u = u < 0 ? 0 : u;\
  v = v < 0 ? 0 : v;\
  y = y > 255 ? 255 : y;\
  u = u > 255 ? 255 : u;\
  v = v > 255 ? 255 : v

/* This function contains code copied from the Coriander. */
void DI_ImageConversion::YUV411ToRGB
(const DI_Image * , DI_Image *)
{
/*  long numpixels = (long) width * (long) height;
  
  long i = numpixels + ( numpixels >> 1 ) -1;
  long j = numpixels * 3 - 1;
  int y0, y1, y2, y3, u, v, r, g, b;

  while (i > 0)
  {
    y3 = (unsigned char) src[i--];
    y2 = (unsigned char) src[i--];
    v  = (unsigned char) src[i--] - 128;
    y1 = (unsigned char) src[i--];
    y0 = (unsigned char) src[i--];
    u  = (unsigned char) src[i--] - 128;

    YUV2RGB (y3, u, v, r, g, b);
    dest[j--] = b;
    dest[j--] = g;
    dest[j--] = r;
    YUV2RGB (y2, u, v, r, g, b);
    dest[j--] = b;
    dest[j--] = g;
    dest[j--] = r;
    YUV2RGB (y1, u, v, r, g, b);
    dest[j--] = b;
    dest[j--] = g;
    dest[j--] = r;
    YUV2RGB (y0, u, v, r, g, b);
    dest[j--] = b;
    dest[j--] = g;
    dest[j--] = r;
    }*/
}

/* This function contains code copied from the Coriander. */

void DI_ImageConversion::YUV411ToGrayscale
(const DI_Image *, DI_Image *)
{
/*  long w = source->m_width;
  long h = source->m_height;

  uchar * src = source->m_planes[0].m_data;
  uchar * dest = target->m_planes[0].m_data;

  for(long y = 0; y < h; y++) {


  {
    dest[j++] = src[i++];
    dest[j++] = src[i++];
    i++;
    dest[j++] = src[i++];
    dest[j++] = src[i++];
    i++;
    }*/
}

void DI_ImageConversion::YUV420PToGrayscale
(const DI_Image * source, DI_Image * target)
{
  long lw = source->m_planes[0].m_linesize;
  long w = source->m_width;
  long h = source->m_height;

  target->m_width  = w;
  target->m_height = h;
  target->m_format = DI_IMAGE_GRAYSCALE;

  const uchar * src = source->m_planes[0].m_data;
  uchar * dest = target->m_planes[0].m_data;
  
  for(long y = 0; y < h; y++) {
    memcpy(dest, src, w);
    dest += w;
    src += lw;
  }
}

void DI_ImageConversion::grayscaleToRGB
(const DI_Image * source, DI_Image * target)
{
  long w = source->m_width;
  long h = source->m_height;

  target->m_width  = w;
  target->m_height = h;
  target->m_format = DI_IMAGE_RGB;

  const uchar * src  = source->m_planes[0].m_data;
  uchar * dest = target->m_planes[0].m_data;

  uchar * sentinel = dest + w * h * 3;

  while(dest < sentinel) {
    
    uchar tmp = * src;

    dest[0] = tmp;
    dest[1] = tmp;
    dest[2] = tmp;

    src ++;
    dest += 3;
  }
}
