/* COPYRIGHT
 *
 * This file is part of the DIVA IO library.
 *
 * Author(s): Jarmo Hiipakka and Tommi Ilmonen, 2000-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * DIVA - Digital Interactive Virtual Acoustics group 
 * Telecommunications Software and Multimedia Laboratory
 * Helsinki University of Technology
 * Finland
 *
 * http://www.tml.hut.fi/Research/DIVA/
 *
 * DIVA IO library is licensed under GNU Lesser General Public
 * License (LGPL). The LGPL conditions can be found in file "LGPL"
 * that is distributed with this source package or obtained from the
 * GNU organization (www.gnu.org).
 * 
 */

#include <di_video_avcodec.h>

#include <di_error.h>
#include <di_types.h>

#include <avcodec.h>
#include <avformat.h>

#include <strings.h>

DI_VideoAvcodecInput::DI_VideoAvcodecInput()
  : m_vcodec(0),
    m_vindex(-1),
    m_vcontext(0),
    m_frame(0),
    m_ic(0)
{
  static bool once = false;

  if(!false) {
    av_register_all();
    once = true;
  }

}

DI_VideoAvcodecInput::~DI_VideoAvcodecInput()
{}

const DI_Image * DI_VideoAvcodecInput::captureImage()
{
  // const char * fname = "DI_VideoAvcodecInput::captureImage";

  AVPacket pkt;
  
  int got = false;

  int len1, got_picture;

  av_init_packet( & pkt);

  while(!got) {

    // printf(","); fflush(0);

    int ret = av_read_frame(m_ic, & pkt);

    if(ret < 0)
      return 0;

    // DI_trace2("%s # %d %d", fname, ret, pkt.stream_index);

    if (pkt.stream_index == m_vindex) {
      len1 = avcodec_decode_video(m_vcontext, 
				  m_frame, & got_picture, 
				  pkt.data, pkt.size);
      if (got_picture) {
	got = true;
	// printf(" ls = %d", m_frame->linesize[0]);
      }
    }
  }

  for(int i = 0; i < 3; i++) {
    m_image.m_planes[i].m_data = m_frame->data[i];
    m_image.m_planes[i].m_linesize = m_frame->linesize[i];
  }
  av_free_packet( & pkt);

  return & m_image;
}

int DI_VideoAvcodecInput::width()
{
  return m_vcontext ? m_vcontext->width : 0;
}

int DI_VideoAvcodecInput::height()
{
  return m_vcontext ? m_vcontext->height : 0;
}

float DI_VideoAvcodecInput::fps()
{
  return 0;
}

DI_ImageFormat DI_VideoAvcodecInput::imageFormat()
{
  return m_image.m_format;
}

bool DI_VideoAvcodecInput::open(const char * filename, 
				const char * codecname)
{
  const char * fname = "DI_VideoAvcodecInput::open";

  int i;

  AVInputFormat * iformat = 0;
  AVFormatParameters params, *ap = &params;  
  
  int err = av_open_input_file( & m_ic, filename, iformat, 0, ap);
  
  if(err < 0) {
    DI_error2(DI_ERR_UNKNOWN, "%s # Could not open file \"%s\"", 
	      fname, filename);
    return false;
  }
  
  av_read_play(m_ic);

  for(i = 0; i < m_ic->nb_streams; i++) {
    AVCodecContext *enc = &m_ic->streams[i]->codec;
    /* switch(enc->codec_type) {
       case CODEC_TYPE_AUDIO:
       if (audio_index < 0 && !audio_disable)
       audio_index = i;
       break;
       case CODEC_TYPE_VIDEO:
       if (video_index < 0 && !video_disable)
       video_index = i;
       break;
       default:
       break;
       }*/

    if(enc->codec_type == CODEC_TYPE_VIDEO) {

      m_vindex = i;
      m_vcodec = avcodec_find_decoder(enc->codec_id);
      m_vcontext = enc;

      if(!m_vcodec || avcodec_open(enc, m_vcodec) < 0)
	return false;
    }
  }

  m_frame = avcodec_alloc_frame();
  
  PixelFormat avcfmt = m_vcontext->pix_fmt;

  if(avcfmt == PIX_FMT_YUV420P) {
    m_image.setFormatYUV420P();
  }

  m_image.m_width = width();
  m_image.m_height = height();


  DI_trace2("%s # Opened file %s,  (%d x %d %s) %d (%d)", 
	    fname, filename, width(), height(), m_vcodec->name,
	    (int) m_image.m_format, (int) avcfmt);

  return true;
}

bool DI_VideoAvcodecInput::close()
{
  if(!m_vcodec)
    return false;
  
  av_close_input_file(m_ic);
  
  m_vcodec = 0;
  m_ic = 0;

  return true;
}
