/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of the DIVA IO library.
 *
 * Author(s): Jarmo Hiipakka and Tommi Ilmonen, 2000-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * DIVA - Digital Interactive Virtual Acoustics group 
 * Telecommunications Software and Multimedia Laboratory
 * Helsinki University of Technology
 * Finland
 *
 * http://www.tml.hut.fi/Research/DIVA/
 *
 * DIVA IO library is licensed under GNU Lesser General Public
 * License (LGPL). The LGPL conditions can be found in file "LGPL"
 * that is distributed with this source package or obtained from the
 * GNU organization (www.gnu.org).
 * 
 */

#ifndef DI_VIDEO_INPUT_H
#define DI_VIDEO_INPUT_H

#include <di_image_conversion.h>

/** Base class for video input classes. Potential child classes are:
    FireWire video input, USB video input and movie video input. */

class DI_VideoInput
{
public:
  
  virtual ~DI_VideoInput();

  /** This method captures an image from the video source. The image
      is returned in the native format of the device, for conversions
      into application video formats see #DI_ImageConversion. */

  virtual const DI_Image * captureImage() = 0;

  /// The width of the video stream images.
  virtual int width() = 0;
  /// The height of the video stream images.
  virtual int height() = 0;
  /// The frame rate of the video stream.
  virtual float fps() = 0;
  /// Native image format of the stream.
  virtual DI_ImageFormat imageFormat() = 0;
  

protected:
  DI_VideoInput() {}
  DI_VideoInput(const DI_VideoInput &) {}

};

#endif


