/* COPYRIGHT
 *
 * This file is part of the DIVA IO library.
 *
 * Author(s): Jarmo Hiipakka and Tommi Ilmonen, 2000-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * DIVA - Digital Interactive Virtual Acoustics group 
 * Telecommunications Software and Multimedia Laboratory
 * Helsinki University of Technology
 * Finland
 *
 * http://www.tml.hut.fi/Research/DIVA/
 *
 * DIVA IO library is licensed under GNU Lesser General Public
 * License (LGPL). The LGPL conditions can be found in file "LGPL"
 * that is distributed with this source package or obtained from the
 * GNU organization (www.gnu.org).
 * 
 */


#include <di_audio_port.h>
#include <di_audio_file_threaded.h>
#include <di_audio_file_handler.h>
#include <di_config.h>
#include <di_error.h>
#include <di_io_defs.h>
#include <di_midi_io.h>
#include <di_sleep.h>

#include <stdlib.h>
#include <strings.h>
#include <math.h>
#include <stdio.h>
#include <unistd.h>

#include <vector>

#define DI_pi 3.14159
#define DI_2pi 3.14159 * 2

#define DI_TEST_FLOAT32 1

#ifdef DI_TEST_FLOAT32
typedef float DI_testSample;
DI_AudioSampleFormat format = DI_ASF_FLOAT32;
#else
typedef double DI_testSample;
DI_AudioSampleFormat format = DI_ASF_FLOAT64;
#endif

int DI_audioTest(const char * device, 
		 int chans, 
		 int sr, 
		 int bufLen,
		 bool duplex)
{
  DI_AudioPort p;

  const DI_IoMode mode = duplex ? DI_IO_INPUT_OUTPUT : DI_IO_OUTPUT;

  const int times = (7 * sr) / bufLen + 2;
  float phase1 = 0.0;
  float omega1 = DI_2pi * 220.0 / (float) sr;
  float phase2 = 0.0;
  float omega2 = omega1 * 1.7;
  float phase3 = 0.0;
  float omega3 = DI_2pi * 1.0 / (float) sr;
  
  DI_testSample * data  = new DI_testSample[bufLen * chans];
  DI_testSample * rdata = new DI_testSample[bufLen * chans];

  DI_AudioFileThreaded af;

  int i, j, f;

  for(i = 0; i < bufLen * chans; i++) {
    data[i] = 0.0;
    rdata[i] = 0.0;
  }

#if defined(DI_AUDIO_IS_IRIX)
  
  int inDevs = DI_AudioDevice::getNumberOfInputDevices();
  int outDevs = DI_AudioDevice::getNumberOfOutputDevices();

  char name[DI_AUDIO_MAX_NAME_LENGTH], 
    nameLong[DI_AUDIO_MAX_NAME_LENGTH];
  
  for(i=0; i < inDevs; i++) {
    DI_AudioDevice::getInputDeviceName(i, name);
    DI_AudioDevice::getDeviceLabel(i, nameLong);
    printf("Indevice %d - name = \"%s\" - label = \"%s\"\n",
	   i, name, nameLong);
  }
  for(i=0; i < outDevs; i++) {
    DI_AudioDevice::getOutputDeviceName(i, name);
    DI_AudioDevice::getDeviceLabel(i, nameLong);
    printf("Outdevice %d - name = \"%s\" - label = \"%s\"\n",
	   i, name, nameLong);
  }
  fflush(0);

#endif

  DI_AudioPort::setDebug(true);

  DI_trace2("Opening device \"%s\"", device);


  if(!p.open(device, 
	     mode, format, bufLen * 4, sr, chans, 
	     bufLen)) {
    DI_error(DI_ERR_INTERNAL, "int main()");
    return 0;
  }

  DI_trace2("Opened device %s in mode %d, bufLen = %d", 
	    device, (int) mode, bufLen);
  
  /* af.startWrite("data.wav", format, sr, chans, bufLen, 50, DI_AFF_WAVE, 16);
  if(!af.userPrepare())
  return 0;*/

  DI_trace("TO START");

  if(!p.start())
    DI_error(DI_ERR_INTERNAL, "int main() 2");

  DI_trace("Enter loop", times);

  for(i=0; i < times; i++) {
    
    for(j=0; j < bufLen; j++) {

      phase1 += omega1;
      phase2 += omega2;
      phase3 += omega3;

      if(phase1 > DI_2pi) phase1 -= DI_2pi;
      if(phase2 > DI_2pi) phase2 -= DI_2pi;
      if(phase3 > DI_2pi) phase3 -= DI_2pi;

      float scale = cosf(phase3);
      float s1 = sinf(phase1);
      float s2 = sinf(phase2);
      
      for(int k=0; k < chans; k++) {
	if(k != 2 || chans < 3) {
	  float s = (k == 1) ? s1 : s2;
	  /* data[j * chans + k] = (float) random() / (float) RAND_MAX / 
	     (float) (k+2); */
	  data[j * chans + k] = scale * s * 0.9 / (float) (k+1);
	  // data[j * chans + k] = 0.0;
	}
	else
	  data[j * chans + k] = 0.0;
      }
    }
    
    // DI_trace("Loop", i);

    int fill = 0;
    int ca = -2;

    
    for(j = 0; j < bufLen * chans; j++) 
      rdata[j] = 0.0;

    // int ca = p.inFilledFrames();
    
#ifdef ALSA_IS_0_9
    while((f = p.outEmptyFrames()) < (bufLen * 2) && !p.xrun()) {
      // printf("."); fflush(0);
      ;
    }
    if(f < 0)
      DI_error(DI_ERR_UNKNOWN, "f < 0");      
    
    // printf("f = %d...", f); fflush(0);

    printf("round %d  ", i); DI_trace("outEmptyFrames", f);
    
    if(i && (mode & DI_IO_INPUT))
      ca = p.inFilledFrames();
#else
    // if(i > 1)
    if(mode & DI_IO_INPUT)
      ca = p.inFilledFrames();
#endif
    
    DI_trace("ca", ca);

    printf("YY "); fflush(0);
    fill = p.writeFrames(data, bufLen);
    printf("XX "); fflush(0);
  
    if(!fill) {
      DI_error(DI_ERR_UNKNOWN, "p.writeFrames",  "fill = 0");
    }

    if(ca > bufLen && i && (mode & DI_IO_INPUT)){
      p.readFrames(rdata, bufLen); 
      /* printf("#"); fflush(0);
	 af.writeFrames(rdata, bufLen);*/
    }

    // if(i < times / 2) {
    if (0) {

      fill = p.outFilledFrames();

      int nonZeros = 0;
      int firstZero = -1;
      
      for(j = 0; j < bufLen * chans; j++) {
	if(rdata[j] > 0.001 || rdata[j] < -0.001) {
	  nonZeros++; 
	}
	else if(firstZero < 0)
	  firstZero = j;
      }
      
      printf("ca = %d fill = %d rdata[0] = %f data[%d] = %f nonZeros = %d, firstZero = %d\n", 
	     ca, fill, rdata[0], chans / 2, data[chans / 2], 
	     nonZeros, firstZero);
      
      // if((i + 1) % 5 == 0)
      fflush(0);
    }
  }
  printf("\nExit loop\n");
  fflush(0);

  // DI_fatal(DI_ERR_UNKNOWN, "Won't close audio devices !!");

  p.close();

  af.requestEnd();
  af.waitEnd();
  // p2.close();
  // p3.close();

  /* DI_Sleep s;
     s.sleepMs(1000); */

  return EXIT_SUCCESS;
}

void MJ_midiReadTest()
{
  DI_MidiIo midi;

  const char * device = DI_defaultMidiDev();

  bool ok = midi.open(device, DI_IO_INPUT);

  if(!ok) {
    DI_error2(DI_ERR_UNKNOWN, "Could not open MIDI device \"%s\" (%s)", 
	      device, strerror(errno));
    return;
  }
  
  DI_trace2("Opened MIDI device \"%s\"", device);

  uint maxEvents = 128;

  DI_MidiEvent e;

  for(uint i=0; i < maxEvents; i++) {
    midi.readEvent(&e);

    DI_trace2("Got new MIDI event: %x %x %x", 
	      (int) e.a(), (int) e.b(), (int) e.c());

    if(e.isSysex()) {
      for(uint j=0; j < e.sysexSize(); j++) {
	printf(" %x ", (int) (e.sysexDataConst()[j]));
      }
      fflush(0);
    }
    
  }
  midi.close();
}

static void fileTest(const char * filename)
{
  const int bufSize = 1000;

  DI_AudioFile file;

  file.open(filename, DI_IO_INPUT, DI_ASF_FLOAT32, bufSize);

  int channels = file.inChannels();

  std::vector<float> buffer;

  buffer.resize(channels * bufSize);

  int frames = 0;

  while(frames < file.frameCount()) {

    bzero(& buffer[0], 4 * channels * bufSize);
    file.readFrames(& buffer[0], bufSize);
    
    for(int i = 0; i < bufSize * channels; i++)
      printf(" %f ", buffer[i]);

    frames += bufSize;
  }
  
  fflush(0);
}

static void fileTest2(const char * filename)
{
  puts("Threaded audio file test");

  const int bufSize = 1000;

  DI_AudioFileThreaded file;

  file.startRead(filename, DI_ASF_FLOAT32, bufSize, 0, 1000000000,
		 false);

  bool ok = file.userPrepare();

  if(!ok) {
    DI_error2(DI_ERR_UNKNOWN, "Could not open file \"%s\"", filename);
    return;
  }

  int channels = file.file().inChannels();

  std::vector<float> buffer;

  buffer.resize(channels * bufSize);

  int frames = 0;

  while(frames < file.file().frameCount()) {

    bzero(& buffer[0], 4 * channels * bufSize);
    file.readFrames(& buffer[0], bufSize);
    
    for(int i = 0; i < bufSize * channels; i++)
      printf(" %f ", buffer[i]);

    frames += bufSize;
  }
  
  fflush(0);
}

static void fileTest3(const char * filename)
{
  puts("Threaded audio handler test");

  DI_AudioFileHandler handler;
  
  handler.start();

  DI_AudioFileHandler::Handle * file = handler.readFile(filename, 0);

  bool ok = file->waitOpen();

  DI_trace2("Opened file %s (%s)\n", filename, ok ? "success" : "failed");
  
  int channels = file->channels();

  const int bufSize = 1000;

  std::vector<DI_Sample> buffer;
  buffer.resize(channels * bufSize);

  int frames = 0;

  while(frames < file->frames()) {
    
    bzero(& buffer[0], sizeof(DI_Sample) * channels * bufSize);
    file->readFrames(& buffer[0], bufSize);
    
    for(int i = 0; i < bufSize * channels; i++)
      printf(" %f ", buffer[i]);

    frames += bufSize;
  }
  
  puts("READ");
  handler.stop();
  puts("STOPPED");
}

static void fileTest4(const char * filename)
{
  puts("Threaded audio handler write test");

  DI_AudioFileHandler handler;
  
  handler.start();

  DI_AudioFileHandler::Handle * file = handler.writeFile
    (filename, 1, 44100, SF_FORMAT_WAV | SF_FORMAT_PCM_16);

  bool ok = file->waitOpen();

  DI_trace2("Opened file %s (%s)\n", filename, ok ? "success" : "failed");

  if(!ok)
    return;
  
  int channels = file->channels();

  const int bufSize = 987;

  std::vector<DI_Sample> buffer;
  buffer.resize(channels * bufSize);

  float w = 0.0f;
  float omega = 0.03;

  for(int i = 0; i < 100000; i++) {
    
    for(int j=0; j < bufSize; j++) {
      w += omega;

      if (w > 3.1415926535897931)
	w -= 6.2831853071795862;

      buffer[j] = sinf(w) * 0.7;
      i++;
    }      
    
    file->writeFrames(& buffer[0], bufSize);
    
  }
  
  handler.done(file);
  
  puts("WRITTEN");
  handler.stop();
  puts("STOPPED");
}

void helper(const char *progName, const char * badArg = 0)
{
  if(badArg)
    printf("%s: ERROR Bad argument \"%s\"\n", progName, badArg);
  
  printf("Usage:\n%s [options]", progName);
  printf("\nOptions:\n"
	 "\t-b +int  - Buffer (fragment) size\n"
	 "\t-c +int  - Channels\n"
	 "\t-d +dev  - Device name\n"
	 "\t-f +file - Load a file and print contents\n"
	 "\t-F +file - Load a file and print contents (threaded)\n"
	 "\t-D       - Duplex\n"
	 "\t-g       - Debug on\n"
	 "\t-h       - This help\n"
	 "\t-m       - MIDI test\n"
	 "\t-H +file - Load a file and print contents (multi-file handler)\n"
	 "\t-X +file - Write a test file (multi-file handler)\n"
	 "\t-r +int  - Sample rate\n");
  
  fflush(0);
}

#ifdef HAVE_AVCODEC

#include <di_video_avcodec.h>

static void DI_avdecode(const char * file)
{
  DI_trace2("AVDECODE %s", file);

  DI_VideoAvcodecInput video;

  video.open(file);

  video.close();
}

#endif

/* 
   For example (second audio card on an ALSA 0.9 system):

   ./Linux-i686/dmain hw:1,0

*/

int main(int argc, char **argv)
{
  const char * device = 0;
  
  bool testAvcodec = false;
  bool testMidi = false;
  int testFile = 0;
  bool duplex = false;

  int chans = 2;
  int sr = 44100;
  int i;
  int bufLen = 512;

  for(i = 1; i < argc && argv[i][0] == '-'; i++) {
    switch(argv[i][1]) {
    case 'A':
      testAvcodec = true;
      break;
    case 'b':
      bufLen = atoi(argv[++i]);
      break;
    case 'c':
      chans = atoi(argv[++i]);
      break;
    case 'D':
      duplex = true;
      break;
    case 'd':
      device = argv[++i];
      break;
    case 'f':
      testFile = 1;
      device = argv[++i];
      break;
    case 'F':
      testFile = 2;
      device = argv[++i];
      break;
    case 'H':
      testFile = 3;
      device = argv[++i];
      break;
    case 'X':
      testFile = 4;
      device = argv[++i];
      break;
    case 'g':
      DI_AudioPort::setDebug(true);
      DI_MidiIo::setDebug(true);
      break;
    case 'h':
      helper(argv[0]);
      exit(EXIT_SUCCESS);
    case 'm':
      testMidi = true;
      break;
    case 'r':
      sr = atoi(argv[++i]);
      break;      
    default:
      helper(argv[0], argv[i]);
      exit(EXIT_FAILURE);
    }
  }

  if(i < argc) {
    helper(argv[0], argv[i]);
    exit(EXIT_FAILURE);
  }

  /* if(argc >= 2) {
    if(!strcmp(argv[1], "MIDI"))
      testMidi = true;
    else 
      device = argv[1];
      }*/
  if(testFile == 1) 
    fileTest(device);
  else if(testFile == 2) 
    fileTest2(device);
  else if(testFile == 3) 
    fileTest3(device);
  else if(testFile == 4) 
    fileTest4(device);
#ifdef HAVE_AVCODEC
  else if(testAvcodec)
    DI_avdecode(device);
#endif
  else if(testMidi)
    MJ_midiReadTest();
  else {
    DI_trace2("Testing audio with %d channels and sample rate of %d",
	      chans, sr);

    bool ok   = DI_audioTest(device, chans, sr, bufLen, duplex) == EXIT_SUCCESS;
    if(ok) ok = DI_audioTest(device, chans, sr, bufLen, duplex) == EXIT_SUCCESS;
    if(ok) ok = DI_audioTest(device, chans, sr, bufLen, duplex) == EXIT_SUCCESS;
  }

  return 0;
}
