/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of the DIVA IO library.
 *
 * Author(s): Jarmo Hiipakka and Tommi Ilmonen, 2000-2001.
 * Jarmo.Hiipakka@hut.fi Tommi.Ilmonen@hut.fi
 *
 * DIVA - Digital Interactive Virtual Acoustics group 
 * Telecommunications Software and Multimedia Laboratory
 * Helsinki University of Technology
 * Finland
 *
 * http://www.tml.hut.fi/Research/DIVA/
 *
 * DIVA IO library is licensed under GNU Lesser General Public
 * License (LGPL). The LGPL conditions can be found in file "LGPL"
 * that is distributed with this source package or obtained from the
 * GNU organization (www.gnu.org).
 * 
 */

#ifndef DI_MIDI_IO_H
#define DI_MIDI_IO_H

#include <di_io_defs.h>
#include <di_midi_event.h>

#include <sys/time.h>
#include <di_types.h>

#define DI_MIDI_IS_OSS 1

extern const char *DI_defaultMidiDev();


/** MIDI IO class for OSS. */
class DI_MidiIo
{
public:
  DI_MidiIo();
  ~DI_MidiIo();

  /** Returns the name of the port. */
  const char *portName() const { return m_portName; }

  /// Open the port.
  bool open(const char * port,
	    DI_IoMode);

  /// Close the port.
  bool close();

  /// Returns true if the port is open for input
  bool isInput() const { return m_mode & DI_IO_INPUT; }
  /// Returns true if the port is open for output
  bool isOutput() const { return m_mode & DI_IO_OUTPUT; }

  /// Resets the base timing against which events are compared.
  void resetTiming();

  /// Returns true if there is an incoming event pending.
  bool eventPending();

  /** Reads a MIDI event. Blocks until the event can be read. */
  bool readEvent(DI_MidiEvent *);
  /** Writes the event to the port. On Linux/OSS we use raw MIDI
      interface -> The message is written immediately rather than at
      the correct time. */
  bool writeEvent(DI_MidiEvent *);

  static void setDebug(bool debug) { m_debug = debug; }

protected:

  void readSysex(DI_MidiEvent *);

  int read(int, void *, int);

  char          *m_portName;

  bool           m_open;
 
  /// MIDI line status (For missing headers...)
  unsigned char  m_status1, m_status2;

  struct timeval m_tv, m_time, m_origin;
  fd_set         m_rfds;

  DI_IoMode      m_mode;

  int            m_midiFd;

  static bool    m_debug;

};



#endif
