/* COPYRIGHT
 *
 * This file is part of Mustajuuri.
 *
 * Author: Tommi Ilmonen, 2001
 * Tommi.Ilmonen@hut.fi
 *
 * http://www.tcm.hut.fi/~tilmonen/mustajuuri/
 *
 * Mustajuuri is licensed under GNU Lesser General Public License
 * (LGPL). The LGPL conditions can be found in file "LGPL" that is
 * distributed with this source package. The Mustajuuri license
 * interpretation that further clarifes LGPL can be found in file
 * "MUSTAJUURI_LICENSE" that is distributed with the Mustajuuri source code.
 *    
 */

#ifndef MJ_LADSPA_EXPORT
#define MJ_LADSPA_EXPORT

#include <ladspa.h>

#include <mj_module_dsp.h>
#include <mj_parameter.h>

class MJ_PluginDbeDsp;
class MJ_LadspaExport;

class MJ_LadspaPlugin : public MJ_ModuleDsp
{
public:

  MJ_LadspaPlugin(MJ_ModuleDsp *, MJ_LadspaExport *);
  ~MJ_LadspaPlugin();

  static void activate(LADSPA_Handle instance);

  static void run(LADSPA_Handle instance,
		  unsigned long sampleCount);

  static void deactivate(LADSPA_Handle instance);

  static void cleanup(LADSPA_Handle instance);

  static void connectPort(LADSPA_Handle instance,
			  unsigned long port,
			  LADSPA_Data * dataLocation);

protected:

  MJ_Array<LADSPA_Data *> m_inMessagesNew;
  MJ_Array<LADSPA_Data>   m_inMessagesOld;

  MJ_ModuleDsp *m_module;
  MJ_TimeStamp  m_timeStamp;
  ulong m_lastSampleCount;
  MJ_LadspaExport *m_ladspaSource;
};

enum MJ_IoType {
  MJ_IOT_SIGNAL_INPUT,
  MJ_IOT_SIGNAL_OUTPUT,
  MJ_IOT_MESSAGE_INPUT,
  MJ_IOT_MESSAGE_OUTPUT
};

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

/** Remap LADSPA ports to Mustajuuri io. */
class MJ_LadspaPortMapping
{
public:
  MJ_LadspaPortMapping()
    : m_index((uint)-1), m_type((MJ_IoType)-1) {}

  inline uint index() const { return m_index; }
  inline uint type() const { return m_type; }

  void set(uint index, MJ_IoType type)
  { m_index = index; m_type = type; }
  
protected:

  uint m_index;
  MJ_IoType m_type;
};

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

/** This will eventually allow Mustajuuri plugins to work under
    LADSPA. */
class MJ_LadspaExport
{
public:
  MJ_LadspaExport(MJ_PluginDbeDsp *dspPlugin, 
		  LADSPA_Descriptor *lad, 
		  uint index);
 
  static LADSPA_Handle instantiate(const LADSPA_Descriptor * descriptor,
				   unsigned long             sampleRate);

  static LADSPA_Descriptor *getLadspaDescriptor(MJ_PluginDbeDsp *p, uint i);

  MJ_Array<MJ_LadspaPortMapping> & mappings() {return m_mappings; }
 
  MJ_Parameters & params() { return m_params; }

protected:

  void constructMappings(MJ_ModuleDsp *);

  static LADSPA_PortRangeHint parameterToHint(MJ_Parameter *);

  MJ_ModuleDsp *newModule();

  MJ_Array<MJ_LadspaPortMapping>  m_mappings;
  MJ_Array<LADSPA_PortDescriptor> m_portDescriptors;
  MJ_Array<LADSPA_PortRangeHint>  m_portRangeHints;
  MJ_Array<QCString>              m_portNames;
  MJ_Array<const char *>          m_portNamePtrs;

  MJ_Parameters      m_params;

  MJ_PluginDbeDsp   *m_dspPlugin;
  uint               m_index;

  LADSPA_Descriptor *m_ladspaDescriptor;
};

#endif
