/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of Mustajuuri LGPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2001.
 * Tommi.Ilmonen@hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. Extra clauses can be found from the file
 * MUSTAJUURI_LGPL and further explanations from the file
 * MUSTAJUURI_LICENSE. The license text can be found in file
 * "LGPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *    
 */

#ifndef MJ_RTC_HZ_H
#define MJ_RTC_HZ_H

/** Sets the frequency of the real-time clock (RTC). This class is
    used for short-period sleeps on Linux.

    Only useful on Linux. 

    For more documentation see file "rtc.txt" in linux kernel
    documentation.

    @author Tommi Ilmonen */

class MJ_RtcHz 
{
public:
  MJ_RtcHz();
  ~MJ_RtcHz();
  
  bool open(const char *device = 0);

  bool startPeriodic();

  bool close();


  bool isOpen() { return (m_fd != -1); }

  bool setHz(int Hz);

  int getHz();

  bool waitTicks(int n);

  static bool isImplemented() { return true; }
  
  static MJ_RtcHz *rtc() { return m_rtc; }

  static MJ_RtcHz *newRtc(const char *device = 0, int Hz = 2048);
  
protected:
  int m_fd;

  /** Since the RTC can only be used by one user at a time there needs
      to be a "lock" variable that controls access to it. As the RTC
      object is "opened" it sets this variable to point to itself. 

      Yes, this is not too compatible when there are multiple
      devices in the same box... Fix it when you must. */
  static MJ_RtcHz * m_rtc;
};


#endif
