/* COPYRIGHT
 *
 * This file is part of Mustajuuri LGPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. Extra clauses can be found from the file
 * MUSTAJUURI_LGPL and further explanations from the file
 * MUSTAJUURI_LICENSE. The license text can be found in file
 * "LGPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *    
 */

#include <mj_app_actions.h>

#include <mj_error.h>
#include <mj_global.h>
#include <mj_module_dsp_gui.h>
#include <mj_kernel.h>
#include <mj_satellite.h>

MJ_AppActionStartDsp::~MJ_AppActionStartDsp()
{}
  
void MJ_AppActionStartDsp::operate()
{
  const char * fname = "MJ_AppActionStartDsp::operate";

  MJ_trace(fname);

  MJ_ModuleDsp *tmp;
  
  while((tmp = m_module->parentModule())) {
    m_module = tmp;
  }
  
  MJ_ModuleDspUi *mg = 0;
  
  mg = MJ_ModuleDspUi::findUi(m_module);

  if(mg) {
    MJ_trace(fname, "Starting threaded operation");
    MJ_kernel->playModule(mg, MJ_KERNEL_THREADED_PLAY);
  }
  else 
    MJ_trace(fname, "Could not find corresponding UI");

  m_done = true;
}

bool MJ_AppActionStartDsp::isIdentical(MJ_AppAction *a)
{
  return dynamic_cast<MJ_AppActionStartDsp *> (a);
}

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

MJ_AppActionStopDsp::~MJ_AppActionStopDsp()
{}
  
void MJ_AppActionStopDsp::operate()
{
  if(m_done) return;

  MJ_trace("MJ_AppActionStopDsp::operate");
  MJ_kernel->stopPlaying();
  m_done = true;
}

bool MJ_AppActionStopDsp::isIdentical(MJ_AppAction *a)
{
  return dynamic_cast<MJ_AppActionStopDsp *> (a);
}


/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

MJ_AppActionModuleModified::MJ_AppActionModuleModified
(MJ_ModuleDsp *module, const char *parameter)
  : m_module(module), m_parameter(parameter), m_ioflags(0)
{
  connectSignals();
}


MJ_AppActionModuleModified::MJ_AppActionModuleModified
(MJ_ModuleDsp *module, uint ioChangeFlags)
  : m_module(module), m_ioflags(ioChangeFlags)
{
  connectSignals();
}

MJ_AppActionModuleModified::~MJ_AppActionModuleModified()
{}
  
void MJ_AppActionModuleModified::operate()
{
  if(m_done) return;

  if(m_parameter.size())
    m_module->satellite()->modified(m_parameter.c_str());
  if(m_ioflags)
    m_module->satellite()->ioModified(m_ioflags);

  m_done = true;
}

bool MJ_AppActionModuleModified::isIdentical(MJ_AppAction *a)
{
  MJ_AppActionModuleModified *amm = 
    dynamic_cast<MJ_AppActionModuleModified *> (a);

  if(!amm) return false;

  if(amm->m_module != m_module) return false;

  if(amm->m_parameter != m_parameter)
    return false;

  return m_ioflags ? (amm->m_ioflags == m_ioflags) : false;
}

void MJ_AppActionModuleModified::moduleDeleted()
{
  // MJ_trace("MJ_AppActionModuleModified::moduleDeleted");
  m_done = true;
}

void MJ_AppActionModuleModified::connectSignals()
{
  if(m_module->satellite())
    connect(m_module->satellite(), SIGNAL(destroyed()), 
	    this, SLOT(moduleDeleted()));
}

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
  
void MJ_AppActionErrorStr::operate()
{
  if(m_done) return;

  MJ_KernelGui::addError(m_str);

  m_done = true;
}
  

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
  
void MJ_AppActionCheckDsp::operate()
{
  if(MJ_kernel) {
    if(MJ_kernel->status() == MJ_KERNEL_STOP &&
       MJ_kernel->rootUi())
      MJ_kernel->stopPlaying();
  }
}
