/* COPYRIGHT
 *
 * This file is part of Mustajuuri LGPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. Extra clauses can be found from the file
 * MUSTAJUURI_LGPL and further explanations from the file
 * MUSTAJUURI_LICENSE. The license text can be found in file
 * "LGPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *    
 */

#include <mj_audio_io_gui.h>

#include <mj_audio_conf.h>

#include <mj_strs.h>
#include <mj_tr.h>

MJ_AudioIoGui::MJ_AudioIoGui(QWidget      *parent, 
			     const char   *name)
  : MJ_IoConfGui(parent, name)
{
  draw();

  if(!parent) {
    QString label(MJ_Tr::tr("Audio IO"));
    
    setCaption(label);
    setIconText(label);
  }
}

MJ_AudioIoGui::~MJ_AudioIoGui()
{}

QString MJ_AudioIoGui::ioTypeName(bool input)
{
  return MJ_Tr::tr(input ? MJ_Strs::AudioInput : MJ_Strs::AudioOutput);
}

QString MJ_AudioIoGui::ioName(bool input, int index)
{
  MJ_AudioDeviceType adt = (MJ_AudioDeviceType) ! input;

  int devices = MJ_AudioConf::audioDevices(adt);

  if(index >= devices) return QString::null;

  return MJ_Tr::tr("%1 %2 (%3  - %4 channels)")
    .arg(MJ_AudioDeviceTypeKeys2[adt])
    .arg(index + 1)
    .arg(MJ_AudioConf::audioDevice(adt, index))
    .arg(MJ_AudioConf::deviceChannels(adt, index));
}
