/* COPYRIGHT
 *
 * This file is part of Mustajuuri LGPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. Extra clauses can be found from the file
 * MUSTAJUURI_LGPL and further explanations from the file
 * MUSTAJUURI_LICENSE. The license text can be found in file
 * "LGPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *    
 */
#include <mj_debug.h>
#include <mj_error.h>
#include <mj_global.h>

// Files that introduce debugging symbols:
#include <mj_dsp_plugin.h>
#include <mj_dsp_plugin_loader.h>
#include <mj_kernel.h>
#include <mj_message.h>
#include <mj_module_dsp.h>
#include <mj_module_dsp_gui_default.h>
#include <mj_rt_driver.h>
#include <mj_rt_thread.h>
#include <mj_so_ref_count.h>

#include <qvaluelist.h>


const char *MJ_debugTargetKeys[MJ_DEBUG_SIZEOF] = {
  "all",
  "dso",
  "kernel",
  "kernel_gui",
  "message",
  "message_array",
  "message_list",
  "module_dsp",
  "module_dsp_gui",
  "module_dsp_gui_postcard",
  "module_dsp_ui",
  "plugin_base",
  "plugin_dsp",
  "plugin_loader",
  "rt_driver",
  "rt_thread",
  "test_menus",
  "what",
  "?"
};

static QValueList<QCString> MJ_debugStrs;

int MJ_Debug::m_debugAllv = 0;
bool MJ_Debug::m_allowTestMenus = false;

bool MJ_Debug::debugName(const char *str)
{
  int t = MJ_findStr(str, MJ_debugTargetKeys, MJ_DEBUG_SIZEOF);

  bool ok = MJ_Debug::debugType((MJ_DebugTarget) t);
  if(!ok) MJ_debugStrs.append(str);
  return ok;
}

bool MJ_Debug::debugType(MJ_DebugTarget t)
{
  if(m_debugAllv) {
    MJ_trace("MJ_Debug::debugType", (int) t);
  }

  switch(t) {

  case MJ_DEBUG_ALL:
    MJ_Debug::setDebugAll(2);
    break;
    
  case MJ_DEBUG_DSO:
    MJ_SoRefCount::setDebug(true);
    break;

  case MJ_DEBUG_KERNEL:
    MJ_Kernel::setDebug(true);
    break;

  case MJ_DEBUG_KERNEL_GUI:
    MJ_KernelGui::setDebug(true);
    break;

  case MJ_DEBUG_MODULE_DSP:
    MJ_ModuleDsp::setDspDebug(true);
    break;

  case MJ_DEBUG_MESSAGE:
    MJ_Message::setDebug(true);
    break;

  case MJ_DEBUG_MESSAGE_ARRAY:
    MJ_MessageArray::setDebug(true);
    break;

  case MJ_DEBUG_MESSAGE_LIST:
    MJ_MessageList::setDebug(true);
    break;

  case MJ_DEBUG_MODULE_DSP_GUI:
    MJ_ModuleDspGui::setDspGuiDebug(true);
    break;

  case MJ_DEBUG_MODULE_DSP_GUI_POSTCARD:
    MJ_ModuleDspGuiPostCard::setDebug(true);
    break;

  case MJ_DEBUG_MODULE_DSP_UI:
    MJ_ModuleDspUi::setUiDebug(true);
    break;

  case MJ_DEBUG_PLUGIN_BASE:
    MJ_PluginDbeBase::setDebug(true);
    break;

  case MJ_DEBUG_PLUGIN_DSP:
    MJ_PluginDbeDsp::setDebug(true);
    break;

  case MJ_DEBUG_PLUGIN_LOADER:
    MJ_PluginDbeDspLoader::setDebug(true);
    break;

  case MJ_DEBUG_RT_DRIVER:
    MJ_RtDriver::setDebug(true);
    break;

  case MJ_DEBUG_RT_THREAD:
    MJ_RtThread::setDebug(true);
    break;

  case MJ_DEBUG_TEST_MENUS:
    m_allowTestMenus = true;
    break;

  case MJ_DEBUG_WHAT:
  case MJ_DEBUG_QMARK:
    writeDebugStrs();
    break;

  case MJ_DEBUG_SIZEOF:
  default:
    MJ_error(MJ_ERR_ARGUMENT, "mj_debugType", (int) t);
    return false;
  }
  return true;
}

bool MJ_Debug::debugNeeded(const char *c)
{
  return MJ_debugStrs.findIndex(c) >= 0;
}

void MJ_Debug::setDebugAll(int debugAll)
{ 
  m_debugAllv = debugAll;

  for(uint i=1; i < MJ_DEBUG_SIZEOF; i++)
    MJ_Debug::debugType((MJ_DebugTarget) i);
}

void MJ_Debug::writeDebugStrs()
{
  
  printf("MJ_Debug: Debugging strings: \n");

  for(uint i=0; i < MJ_DEBUG_SIZEOF; i++) {
    printf("\t%s\n", MJ_debugTargetKeys[i]);
  }
  fflush(0);
}
