/* COPYRIGHT
 *
 * This file is part of Mustajuuri LGPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. Extra clauses can be found from the file
 * MUSTAJUURI_LGPL and further explanations from the file
 * MUSTAJUURI_LICENSE. The license text can be found in file
 * "LGPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *    
 */

#include <mj_io_conf_gui.h>

#include <mj_error.h>
#include <mj_gui_utils.h>
#include <mj_int_remap.h>
#include <mj_math.h>
#include <mj_misc.h>
#include <mj_tr.h>

#include <qcheckbox.h>
#include <qlabel.h>
#include <qlayout.h>

MJ_IoConfGui::MJ_IoConfGui(QWidget      *parent, 
			       const char   *name)
  : QWidget(parent, name),
    m_layout(0)
{
 m_inputs.setAutoDelete(true);
 m_outputs.setAutoDelete(true);
 m_remaps.setAutoDelete(true);
}

MJ_IoConfGui::~MJ_IoConfGui()
{}

void MJ_IoConfGui::uncheckAll()
{
  uint i;

  uint count = m_inputs.count();
  for(i = 0; i < count; i++) 
    ((QCheckBox *) m_inputs.at(i))->setChecked(false);

  count = m_outputs.count();
  for(i = 0; i < count; i++)  
    ((QCheckBox *) m_outputs.at(i))->setChecked(false);
}

void MJ_IoConfGui::setChecked(bool input, int index, bool check)
{
  qDebug("MJ_IoConfGui::setChecked %d %d %d", (int) input, index, (int) check);

  QList<QWidget> *l;
  if(input) l = & m_inputs;
  else l = & m_outputs;

  if((uint) index >= l->count()) qDebug("MJ_IoConfGui::setChecked");
  else ((QCheckBox *) l->at(index))->setChecked(check);
}

void MJ_IoConfGui::setEnabled(bool ena)
{
  uint i;

  uint count = m_inputs.count();
  for(i = 0; i < count; i++) 
    m_inputs.at(i)->setEnabled(ena);

  count = m_outputs.count();
  for(i = 0; i < count; i++) 
    m_outputs.at(i)->setEnabled(ena);

  QWidget::setEnabled(ena);
}

void MJ_IoConfGui::enableInput(int index,bool ena)
{
  qDebug("MJ_IoConfGui::enableInput %d %d", index, (int) ena);
  enableInOrOut(true, index, ena);
}

void MJ_IoConfGui::enableOutput(int index,bool ena)
{
  qDebug("MJ_IoConfGui::enableOutput %d %d", index, (int) ena);  
  enableInOrOut(false, index, ena);
}

void MJ_IoConfGui::draw()
{
  uint i;
  m_layout = new QGridLayout(this, 1, 2, 5, 5);

  /* Label for the input and ouput columns. */
  for(i = 0; i < 2; i++) {
    QLabel *label = new QLabel((ioTypeName(!i)), this);
    MJ_limMinSize(label);
    m_layout->addWidget(label, 0, i);
  }

  for(i = 0; i < 2; i++)
    appendTypeButtons(i);

  MJ_equalStretch(m_layout);
  int tmp = m_layout->numRows();
  m_layout->expand(tmp + 1, 2);
  m_layout->setRowStretch(tmp, 50);
}

void MJ_IoConfGui::appendTypeButtons(bool input)
{
  const char * fname = "MJ_IoConfGui::appendTypeButtons";

  int colIndex;
  QList<QWidget> *buttonList;
  QString text;

  if(input) {
    colIndex = 0;
    buttonList = & m_inputs;
  } 
  else {
    colIndex = 1;
    buttonList = & m_outputs;
  }

  for(int i=0; true ; i++) {

    text = ioName(input, i);
    
    if(text.isEmpty()) 
      break;

    QCheckBox *cb = new QCheckBox(text, this);
    if(!cb) {
      MJ_fatal(MJ_ERR_MEMORY, fname);
    }
    MJ_limSize(cb);
    buttonList->append(cb);

    m_layout->expand(i + 2, 2);

    m_layout->addWidget(cb, buttonList->count(), colIndex);

    MJ_IntRemap *rm = new MJ_IntRemap(this, buttonList->count() - 1);
    connect(cb, SIGNAL(toggled(bool)), rm, SLOT(callBool(bool)));

    if(input)
      connect(rm, SIGNAL(emitCallBool(int,bool)), 
	      this, SLOT(enableInput(int,bool)));
    else
      connect(rm, SIGNAL(emitCallBool(int,bool)), 
	      this, SLOT(enableOutput(int,bool)));
  }
}


void MJ_IoConfGui::enableInOrOut(bool input, int index, bool ena)
{
  QCheckBox * cb;

  if(input) cb = (QCheckBox *) m_inputs.at(index);
  else cb = (QCheckBox *) m_outputs.at(index);

  bool blocked = cb->signalsBlocked();
  cb->blockSignals(true);
  cb->setChecked(ena);

  emit enableDevice(input, index, ena);

  cb->blockSignals(blocked);
}
