/* COPYRIGHT
 *
 * This file is part of Mustajuuri LGPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. Extra clauses can be found from the file
 * MUSTAJUURI_LGPL and further explanations from the file
 * MUSTAJUURI_LICENSE. The license text can be found in file
 * "LGPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *    
 */

#include <mj_iostream.h>
#include <qfile.h>
#include <stdio.h>

static QTextStream *_MJ_cin = 0;

static QTextStream *_MJ_cout = 0;
static QTextStream *_MJ_cerr = 0;

// static QByteArray a_cin,a_cout,a_cerr;

static void MJ_iosInit()
{
#ifndef WIN32
  _MJ_cin = new QTextStream(stdin, IO_ReadOnly);
  _MJ_cout = new QTextStream(stdout, IO_WriteOnly);
  _MJ_cerr = new QTextStream(stderr, IO_WriteOnly);
#else
  //for some reason the Win32 QT 2.3 causes access violations when operating with stderr,stdin and stdout
  //that's why we use files.
  static QFile fstdin("mj_stdin.txt");
  static QFile fstdout("mj_stdout.txt");
  static QFile fstderr("mj_stderr.txt");
  fstdin.open(IO_ReadOnly);
  fstdout.open(IO_WriteOnly);
  fstderr.open(IO_WriteOnly);

  _MJ_cin = new QTextStream(&fstdin);
  _MJ_cout = new QTextStream(&fstdout);
  _MJ_cerr = new QTextStream(&fstderr);
#endif //WIN32
}

QTextStream &MJ_cin()
{
  if(!_MJ_cin) MJ_iosInit();

  return *_MJ_cin;
}

QTextStream &MJ_cout()
{
  if(!_MJ_cout) MJ_iosInit();
	
  return *_MJ_cout;
}

QTextStream &MJ_cerr()
{
  if(!_MJ_cerr) MJ_iosInit();
  
  return *_MJ_cerr;
}
