/* COPYRIGHT
 *
 * This file is part of Mustajuuri LGPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. Extra clauses can be found from the file
 * MUSTAJUURI_LGPL and further explanations from the file
 * MUSTAJUURI_LICENSE. The license text can be found in file
 * "LGPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *    
 */

#include <mj_latency_gui.h>

#include <mj_gui_utils.h>
#include <mj_slider.h> 
#include <mj_tr.h> 

#include <qlabel.h> 
#include <qlayout.h> 

MJ_LatengyGui::MJ_LatengyGui(float minLat,
			     float maxLat,
			     QWidget      *parent,
			     const char   *name)
  : QWidget(parent, name)
{
  if(!parent) {
    QString caption(MJ_Tr::tr("Latency"));
    setCaption(caption);
    setIconText(caption);
  }

  QGridLayout *layout = new QGridLayout(this, 2, 3, 5, 5);

  MJ_equalStretch(layout);
  layout->setColStretch(1, 80); // Slider takes the space

  // Sliders:
  m_minSlider = new MJ_Slider(1.0, 150.0, minLat, this);
  connect(m_minSlider, SIGNAL(valueChangedDouble(double)),
	  this, SLOT(setMin(double)));
  layout->addWidget(m_minSlider, 0, 1);
	  
  m_maxSlider = new MJ_Slider(1.0, 150.0, maxLat, this);
  connect(m_maxSlider, SIGNAL(valueChangedDouble(double)),
	  this, SLOT(setMax(double)));
  layout->addWidget(m_maxSlider, 1, 1);

  // Param labels:
  layout->addWidget(new QLabel(MJ_Tr::tr("Minimum buffer (ms)"), this), 0, 0);
  layout->addWidget(new QLabel(MJ_Tr::tr("Maximum buffer (ms)"), this), 1, 0);

  // Numeric labels:
  m_minStr = new QLabel(this);
  layout->addWidget(m_minStr, 0, 2);
  m_maxStr = new QLabel(this);
  layout->addWidget(m_maxStr, 1, 2);

  updateTexts();
}

MJ_LatengyGui::~MJ_LatengyGui()
{}

void MJ_LatengyGui::setMin(double d)
{
  double maxLat;
  m_maxSlider->getValueDouble(&maxLat);

  if(d >= maxLat) {
    m_maxSlider->setValueDouble(d);
  }
  updateTexts();
  emitConf();
}

void MJ_LatengyGui::setMax(double d)
{
  double minLat;
  m_minSlider->getValueDouble(&minLat);

  if(d <= minLat) {
    m_minSlider->setValueDouble(d);
  }
  updateTexts();
  emitConf();
}

void MJ_LatengyGui::updateTexts()
{
  double minLat;
  m_minSlider->getValueDouble(&minLat);

  QString text;
  text.sprintf("%4.1f", minLat);
  m_minStr->setText(text);

  double maxLat;
  m_maxSlider->getValueDouble(&maxLat);

  text.sprintf("%4.1f", maxLat);
  m_maxStr->setText(text);
}

void MJ_LatengyGui::emitConf()
{
  double minLat;
  m_minSlider->getValueDouble(&minLat);

  double maxLat;
  m_maxSlider->getValueDouble(&maxLat);

  emit newConf(minLat, maxLat);
}
