/* COPYRIGHT
 *
 * This file is part of Mustajuuri LGPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. Extra clauses can be found from the file
 * MUSTAJUURI_LGPL and further explanations from the file
 * MUSTAJUURI_LICENSE. The license text can be found in file
 * "LGPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *    
 */

#include <mj_message_string.h>

#include <di_data_buffer.h>

MJ_MessageString::MJ_MessageString(const MJ_MessageString &s)
  : MJ_Message(s)
{
  m_string = s.m_string.copy();
}

MJ_MessageString::MJ_MessageString(const QString &s)
  : m_string(s.copy())
{}


MJ_MessageType MJ_MessageString::messageType() const
{
  return MJ_MSG_STRING;
}

MJ_Message *MJ_MessageString::duplicate()
{
  return new MJ_MessageString(*this);
}

double MJ_MessageString::toDouble(bool *ok)
{
  return m_string.toDouble(ok);
}

int MJ_MessageString::toInt(bool *ok)
{
  return m_string.toInt(ok);
}

bool MJ_MessageString::write2(DI_DataBuffer2 *b)
{
  if(!MJ_Message::write2(b)) return false;
  
  uint l = m_string.length();

  b->writeInt32(l);

  for(uint i=0; i < l; i++)
    b->writeInt16(m_string.unicode()[i]);

  return true;
}

bool MJ_MessageString::read2(DI_DataBuffer2 *b, int readFields)
{
  if(!MJ_Message::read2(b, readFields)) return false;

  uint l = b->readInt32();
  
  for(uint i=0; i < l; i++)
    m_string[i] = b->readInt16();
  
  return true;
}

std::string MJ_MessageString::describe()
{
  const char * str = m_string.ascii();
  if(!str) str = "";

  return str;
}

MJ_Message *MJ_MessageString::createNew()
{
  return new MJ_MessageString;
}
