/* COPYRIGHT
 *
 * This file is part of Mustajuuri LGPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. Extra clauses can be found from the file
 * MUSTAJUURI_LGPL and further explanations from the file
 * MUSTAJUURI_LICENSE. The license text can be found in file
 * "LGPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *    
 */

#include <mj_error.h>
#include <mj_iostream.h>
#include <mj_midi.h>

#include <qbuffer.h> 
#include <qfile.h> 
#include <qtextstream.h> 

QTextStream &operator << (QTextStream &s, const MJ_MidiEvent &e)
{
  QString str;
  
  str.sprintf(" %d %d %d ", (int) e.a(), (int) e.b(), (int) e.c());

  s << MJ_MidiEvent::m_label << " " << e.ticks() << " " << e.time() 
    << str << e.sysexSize() << " ";
  
  if(e.sysexSize()) {
    s.writeRawBytes((const char *) e.sysexDataConst(), e.sysexSize());
  }
  return s;
}

QTextStream &operator >> ( QTextStream &s, MJ_MidiEvent &e)
{
  const char *fname = "operator >> ( QTextStream &s, const MJ_MidiEvent)";

  e.clear();

  QString str;
  uint tmp;

  bool ok = true;

  s >> str;
  if(str != MJ_MidiEvent::m_label) MJ_error(MJ_ERR_ARGUMENT, fname);

  s >> str;
  e.setTicks(str.toInt(&ok));

  s >> str;
  e.setTime(str.toInt(&ok));
  
  s >> str;
  e.setMessage(str.toInt(&ok));

  s >> str;
  tmp = (uint) str.toInt(&ok);

  if(tmp) {
    char c;
    s >> c; // Skip one.

    if(e.setSysex(0, tmp))
      s.readRawBytes((char *) e.sysexData(), tmp);
    else ok = false;
  }

  if(!ok) MJ_error(MJ_ERR_ARGUMENT, fname);

  return s;
}

void writeCout(const MJ_MidiEvent &e)
{
  QBuffer b;
  b.open(IO_ReadWrite);
  QTextStream s(&b);
  s << e;
  
  QByteArray ba = b.buffer();
  MJ_cout().device()->writeBlock(ba, ba.size());
  MJ_cout().device()->flush();
}
