/* COPYRIGHT
 *
 * This file is part of Mustajuuri LGPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. Extra clauses can be found from the file
 * MUSTAJUURI_LGPL and further explanations from the file
 * MUSTAJUURI_LICENSE. The license text can be found in file
 * "LGPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *    
 */

#include <mj_midi_event.h>

#include <math.h>
#include <string.h>
#include <stdio.h>
// #include <iostream.h>

const char *MJ_MidiEvent::m_label= "MJ_MIDI_EVENT";

MJ_MidiEvent::MJ_MidiEvent(const MJ_MidiEvent &e)
  : m_sysexData(0),
    m_sysexSize(0)
{
  *this = e;
}

bool MJ_MidiEvent::setSysex(const char *data, unsigned l)
{
  if(m_sysexSize != l) {
    if(m_sysexData) 
      delete [] m_sysexData;
  
    m_sysexData = (unsigned char *)(new char[l]);
    m_sysexSize = l;
    if(!m_sysexData) return false;
  }

  if(data) memcpy(m_sysexData, data, l);
  return true;
}



void MJ_MidiEvent::clear()
{
  m_msg = 0;

  if(m_sysexData) {
    delete [] m_sysexData;
    m_sysexData = 0;
    m_sysexSize = 0;
  }
}


MJ_MidiEvent& MJ_MidiEvent::operator = (const MJ_MidiEvent &e)
{
  if(m_sysexData) {
    delete [] m_sysexData;
    m_sysexData = 0;
    m_sysexSize = 0;
  }

  m_ticks = e.m_ticks;
  m_time  = e.m_time;
  m_msg   = e.m_msg;

  if(e.m_sysexSize) {
    m_sysexData = (unsigned char *)(new char[e.m_sysexSize]);
    if(!m_sysexData) return *this;
    memcpy(m_sysexData, e.m_sysexData, e.m_sysexSize);
    m_sysexSize = e.m_sysexSize;
  }
  return *this;
}

void MJ_MidiEvent::noteOn
(unsigned char key, unsigned char velocity, unsigned char channel, unsigned ticks)
{ 
  m_ticks = ticks; 
  set(0, 0x90 | channel); 
  set(1, key); 
  set(2, velocity); 
}

void MJ_MidiEvent::noteOff
(unsigned char key, unsigned char velocity, unsigned char channel, unsigned ticks)
{ 
  m_ticks = ticks; 
  set(0, 0x80 | channel); 
  set(1, key); 
  set(2, velocity); 
}

double MJ_MidiEvent::keyNumberToHz(int key)
{
  double dOff = (double) (key - 64);
  double hz = pow(2.0, dOff / 12.0) * 440.0;
  return hz;
}

int MJ_MidiEvent::messageBytes()
{
  unsigned char tmp = a() & 0xF0;
  
  if((tmp == 0x80) ||  // Note off.
     (tmp == 0x90) ||  // Note on.
     (tmp == 0xA0) ||  // Polyphonic Key
     (tmp == 0xB0) ||  // Controller Change 
     (tmp == 0xE0))    // Pitch Bend
    return 3;
  else if((tmp == 0xC0) ||  // Program Change
	  (tmp == 0xD0))    // Channel Key Pressure 
    return 2;
  return 0;
}

/*
void MJ_MidiEvent::write(ostream *s) const
{
  char buf[128];

  sprintf(buf, "0x %x %x %x %x", a(), b(), c(), d());

  (*s) << m_label << " " << m_ticks << " " << m_time << " " << buf;
}
*/
