/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of Mustajuuri LGPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. Extra clauses can be found from the file
 * MUSTAJUURI_LGPL and further explanations from the file
 * MUSTAJUURI_LICENSE. The license text can be found in file
 * "LGPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *    
 */

#ifndef MJ_MISC_H
#define MJ_MISC_H

#include <qstring.h>

// #include <math.h>

/** The string that says "Mustajuuri". */
extern const char *MJ_mustajuuriStr;

/** Measurement units. Most of these are self-explanatory. 

    @see MJ_UnitKeys MJ_UnitKeysShort
*/
enum MJ_Unit {
  /// Blank unit
  MJ_UNIT_BLANK,
  /// Angle in radians.
  MJ_UNIT_ANGLE_RADIANS,
  /// Angle in degrees.
  MJ_UNIT_ANGLE_DEGREES,
  /// Beats per minute
  MJ_UNIT_BPM,
  MJ_UNIT_DECIBEL,
  MJ_UNIT_HERTZ,
  MJ_UNIT_METERS,
  /// Radians per second
  MJ_UNIT_OMEGA,
  /// Quarter notes.
  MJ_UNIT_QUARTERS,
  MJ_UNIT_SAMPLES,
  MJ_UNIT_SECONDS,
  MJ_UNIT_MILLISECONDS,
  /// The number of unit types known by the system.
  MJ_UNIT_SIZEOF
};

/** String names for the measurement units. Use these to get unit labels. 
    
    @see MJ_UnitKeysShort MJ_Unit 
*/
extern const char *MJ_UnitKeys[MJ_UNIT_SIZEOF];

/** Short string names for the measurement units. Use these to get
    unit labels.

    @see MJ_UnitKeys MJ_Unit */
extern const char *MJ_UnitKeysShort[MJ_UNIT_SIZEOF];

/** Stereo channel definitions. */
enum MJ_Stereo {
  MJ_STEREO_LEFT,
  MJ_STEREO_RIGHT,
  MJ_STEREO_SIZEOF
};

/** String names for stereo channels. */
extern const char *MJ_StereoKeys[MJ_STEREO_SIZEOF];

/** 5+1 channel definitions. The first two are stereo-compatible. */
enum MJ_51_Channel {
  MJ_51_LEFT,
  MJ_51_RIGHT,
  MJ_51_CENTER,
  MJ_51_REARLEFT,
  MJ_51_REARRIGHT,
  MJ_51_SUBSONIC,
  MJ_51_SIZEOF
};


/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

/** Check Pointer Fatal. Check pointer and exit if null. */
void MJ_cpf(void *);
void MJ_cpf(void *, void *);
void MJ_cpf(void *, void *, void *);

void MJ_cpf(void *, const char *);
void MJ_cpf(void *, void *,const char *);
void MJ_cpf(void *, void *, void *,const char *);

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

/** @name string utilities */
//@{

int MJ_findStr(const char *str, const char **strs, int nofStrs);
int MJ_findStr2(const char *str, const char **strs, int nofStrs);

/** Delete old string and duplicate the new one. */
void MJ_delAndDup(char **target, const char *str);

/** Copy character string to QCString if the integer argument is zero. */
QCString MJ_copyIfZero(const char *, int);

int MJ_fields(const QString &, const char *separators = 0);
// int MJ_fields(const char *, const char *separators = 0);
QString MJ_field(const QString &, int field, const char *separators = 0);
QCString MJ_fieldC(const QCString &s, int field, const char *separators);
/* const char *MJ_fieldStart(const char *s, int field, 
   const char * separators = 0); */
QString MJ_lastField(const QString &, const char *separators = 0);
void MJ_fieldTest();

QString MJ_stripLastField(const QString &, const char * separator);

QString MJ_shorten(const QString &, uint);

QString MJ_appendUnit(const QString &, MJ_Unit);

QString MJ_absToRelPath(const QString &absPath,
			       const QString &dotPath);

void MJ_shiftBack(int startInd, int skip, int *size, char ** argv);

extern const char * MJ_pluginSeparator;

//@}

void MJ_memDebugReport();

void MJ_allocFree(uint bytes = 32768);

QString MJ_describeVersion();

#endif
