/* COPYRIGHT
 *
 * This file is part of Mustajuuri LGPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. Extra clauses can be found from the file
 * MUSTAJUURI_LGPL and further explanations from the file
 * MUSTAJUURI_LICENSE. The license text can be found in file
 * "LGPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *    
 */

#include <mj_module_stand_alone.h>

#include <mj_dsp_plugin.h>
#include <mj_global.h>
#include <mj_kernel.h>
#include <mj_math.h>
#include <mj_misc.h>

MJ_ModuleStandAlone::MJ_ModuleStandAlone()
  : m_audioIn(0),
  m_audioOut(0),
  m_module(0)
{}

MJ_ModuleStandAlone::~MJ_ModuleStandAlone()
{
  if(m_audioIn)  delete m_audioIn;
  if(m_module)   delete m_module;
  if(m_audioOut) delete m_audioOut;
}

bool MJ_ModuleStandAlone::init(const QString & id)
{ 
  const char * fname = "MJ_ModuleStandAlone::init";

  MJ_cpf(MJ_kernel, fname);

  MJ_ModuleDsp *m = MJ_Kernel::newDspModule(id, MJ_PP_IDENTIFIER);
  int i;

  if(!m) {
    error(MJ_ERR_UNKNOWN, fname, "Could not create module");
    return false;
  }

  m_module = m;

  int ins = m->dspInChannels();
  int outs = m->dspOutChannels();
  /* This is the number of channels to use in input and output
     ports. This is done since it is common that the number of
     channels needs to be same to both directions. The only notable
     exception to this rule IRIX. */
  int chans = MJ_max(ins, outs);

  if(!chans) error(MJ_ERR_UNKNOWN, fname, "No audio I/O ports (!)");

  // Create and connect audio input
  if(ins) {
    m_audioIn = MJ_kernel->newDspModule
      (MJ_DspPluginFunctionalityKeys[MJ_DPF_AUDIO_IN], MJ_PP_FUNCTIONALITY);
    
    MJ_cpf(m_audioIn, fname);

    m_audioIn->dspSetOutChannels(chans);
    m_audioIn->setId("in");
    
    for(i = 0; i < ins; i++) {
      m->dspConnectInput(m_audioIn->id(), i, i);
    }

    appendModule(m_audioIn);
  }

  appendModule(m);

  // Create and connect audio output
  if(outs) {
    m_audioOut = MJ_kernel->newDspModule
      (MJ_DspPluginFunctionalityKeys[MJ_DPF_AUDIO_OUT], MJ_PP_FUNCTIONALITY);

    MJ_cpf(m_audioOut, fname);

    m_audioOut->dspSetInChannels(chans);
    m_audioOut->setId("out");
    
    for(i = 0; i < chans; i++) {
      m_audioOut->dspConnectInput(m->id(), i % outs, i);
    }
    appendModule(m_audioOut);
  }

  return true;
}
