/* COPYRIGHT
 *
 * This file is part of Mustajuuri LGPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. Extra clauses can be found from the file
 * MUSTAJUURI_LGPL and further explanations from the file
 * MUSTAJUURI_LICENSE. The license text can be found in file
 * "LGPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *    
 */

#include <mj_parameters.h>

#include <mj_messages.h>
#include <mj_message_string.h>
#include <mj_message_carrier.h>

#include <qstringlist.h> 

MJ_ParameterMidi::~MJ_ParameterMidi()
{}

MJ_MessageType MJ_ParameterMidi::type() const
{
  return MJ_MSG_MIDI_EVENT;
}

MJ_MessageCarrier *MJ_ParameterMidi::valueMessage()
{
  MJ_MessageCarrier *mi = new MJ_MessageCarrier;
  MJ_MessageMidiEvent *md = new MJ_MessageMidiEvent;
  
  md->midiMsg() = m_event;
  mi->setMessage(md);
  mi->setId(m_id);
  return mi;
}


/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

MJ_ParameterString::MJ_ParameterString() 
  : m_allowedStrs(0) 
{}

MJ_ParameterString::MJ_ParameterString(const QString &str) 
  : m_string(str.copy()), 
  m_allowedStrs(0)  
{}

MJ_ParameterString::MJ_ParameterString(const QString &str, 
				       const QString &name,
				       const char    *id) 
  : MJ_Parameter(name, id),
    m_string(str.copy()),
    m_allowedStrs(0)
{}

MJ_ParameterString::MJ_ParameterString(const QString &str, 
				       const char    *name,
				       const char    *id) 
  : MJ_Parameter(name, id),
    m_string(str.copy()),
    m_allowedStrs(0)
{}

MJ_ParameterString::~MJ_ParameterString()
{
  if(m_allowedStrs) delete m_allowedStrs;
}

MJ_MessageType MJ_ParameterString::type() const
{
  return MJ_MSG_STRING;
}

MJ_MessageCarrier *MJ_ParameterString::valueMessage()
{
  MJ_MessageString *ms = new MJ_MessageString(m_string);

  if(!ms) return 0;

  MJ_MessageCarrier *mi = new MJ_MessageCarrier(m_id);

  if(mi) mi->setMessage(ms);

  return mi;
}

QStringList MJ_ParameterString::writeClean()
{
  QStringList l = MJ_Parameter::writeClean();
  l.append(MJ_appendUnit(name(), m_unit));
  l.append(m_string);

  return l;
}

/** Adds a new string to the list of allowed strings. */
void MJ_ParameterString::addAllowedStr(const QString &str)
{
  if(!m_allowedStrs) {
    m_allowedStrs = new QStringList;
    MJ_cpf(m_allowedStrs, "MJ_ParameterString::addAllowedStr");
  }

  m_allowedStrs->append(str.copy());
}

void MJ_ParameterString::addAllowedStr(const char *str)
{
  if(!m_allowedStrs) {
    m_allowedStrs = new QStringList;
    MJ_cpf(m_allowedStrs, "MJ_ParameterString::addAllowedStr");
  }

  m_allowedStrs->append(str);
}

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////


MJ_ParameterFilename::MJ_ParameterFilename(const QString & string,
					   const QString & name,
					   const char * id,
					   const QString &filter, 
					   int mode)
  : MJ_ParameterString(string, name, id),
    m_filter(filter.copy()),
    m_mode(mode)
{}

MJ_ParameterFilename::MJ_ParameterFilename(const QString & string,
					   const char *  name,
					   const char * id,
					   const QString &filter, 
					   int mode)
  : MJ_ParameterString(string, name, id),
    m_filter(filter.copy()),
    m_mode(mode)
{}

MJ_ParameterFilename::~MJ_ParameterFilename()
{}

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

MJ_ParameterBool::MJ_ParameterBool(bool value,
				   const QString & name,
				   const char * id)
  : MJ_Parameter(name, id), 
  m_value(value)
{}

MJ_ParameterBool::MJ_ParameterBool(bool value,
				   const char * name,
				   const char * id)
  : MJ_Parameter(name, id), 
  m_value(value)
{}

MJ_ParameterBool::~MJ_ParameterBool()
{}

MJ_MessageType MJ_ParameterBool::type() const
{
  return MJ_MSG_BOOL;
}

MJ_MessageCarrier *MJ_ParameterBool::valueMessage()
{
  MJ_MessageCarrier *mi = new MJ_MessageCarrier;
  MJ_MessageBool *mb = new MJ_MessageBool(m_value);
  
  mi->setMessage(mb);
  mi->setId(m_id);
  return mi;
}
