/* COPYRIGHT
 *
 * This file is part of Mustajuuri LGPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. Extra clauses can be found from the file
 * MUSTAJUURI_LGPL and further explanations from the file
 * MUSTAJUURI_LICENSE. The license text can be found in file
 * "LGPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *    
 */

#include <mj_error.h>
#include <mj_misc.h>
#include <mj_paths.h>

#include <qfileinfo.h> 

#include <stdlib.h>

extern const char * MJ_PathKeys[MJ_PATH_SIZEOF] = {
  "MUSTAJUURI_PATH",
  "MUSTAJUURI_PLUGIN_PATH",
  "PRIOGIVE",
  "RTCHZ"
};

MJ_Paths::MJ_Paths()
{}

MJ_Paths::~MJ_Paths()
{}

bool isEnvVariable(const QString &varName)
{
  return getenv(varName) != 0;
}


bool MJ_Paths::addPathVariable(const QString &varName)
{
  const char *p = getenv(varName);

  if(p) {
    addPath(p);
    return true;
  }
  else {
    // MJ_error(MJ_ERR_ARGUMENT, "MJ_Paths::addPathVariable", varName);
    return false;
  }
}

bool MJ_Paths::addPathVariableN(MJ_Path v)
{
  if((uint) v >= (uint) MJ_PATH_SIZEOF) return false;
  else return addPathVariable(MJ_PathKeys[v]);
}

void MJ_Paths::addPath(const QString &p)
{
  if(!p.isEmpty()) m_path += ":" + p;
}

QString MJ_Paths::path(uint field)
{
  return MJ_field(m_path, field, ":");
}

uint MJ_Paths::paths()
{
  return MJ_fields(m_path, ":");
}


QString MJ_Paths::findFile(const QString &fileName)
{
  QString tmp;
  QFileInfo info;

  int number = paths();

  for(int i=0; i < number; i++) {
    tmp = path((uint) i) + fileName;
    info.setFile(tmp);
    if(info.exists()) return tmp;
  }
  // Failed
  tmp = "";
  return tmp;
}

QString MJ_Paths::homeDir()
{
  QString r;
  const char *d = getenv("HOME");
  if(d) r = d;
  return r;
}
