/* COPYRIGHT
 *
 * This file is part of Mustajuuri LGPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. Extra clauses can be found from the file
 * MUSTAJUURI_LGPL and further explanations from the file
 * MUSTAJUURI_LICENSE. The license text can be found in file
 * "LGPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *    
 */

#include <mj_plugin_db_tree.h>

#include <mj_error.h>
#include <mj_misc.h>
#include <mj_plugin_filter.h>

#include <mj_iostream.h>

QTextStream & MJ_PluginDbTreeNodeName::print(QTextStream &out, 
					     int indentLevel, 
					     bool html)
{
  for(int i=0; i < indentLevel; i++) {
    out << "  ";
  }
  if(html) out << "<LI> ";
  
  out << m_name << "\n";

  return out;
}

bool MJ_PluginDbTreeNodeName::filter(MJ_PluginFilter *f)
{
  return f->accept(m_entry);
}

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

ulong MJ_PluginDbTreeNodeMany::nofNodes() const
{
  return m_nodes.count();
}

QTextStream & MJ_PluginDbTreeNodeMany::print(QTextStream &out, 
					     int indentLevel,
					     bool html)
{
  for(int i=0; i < indentLevel; i++) {
    out << "  ";
  }
  
  if(html && indentLevel) out << "<LI> ";

  out << m_name << "\n";

  if(html) {
    for(int i=0; i < indentLevel; i++) {
      out << "  ";
    }
    out << "<UL>\n";
  }

  QListIterator<MJ_PluginDbTreeNode> iterator(m_nodes);
  for(;iterator.current(); ++iterator)
    iterator.current()->print(out, indentLevel + 1, html);

  if(html) {
    for(int i=0; i < indentLevel; i++) {
      out << "  ";
    }
    out << "</UL>\n";
  }

  return out;
}


bool MJ_PluginDbTreeNodeMany::addNode(MJ_PluginDbTreeNode *node)
{
  if(!node) {
    MJ_error(MJ_ERR_ARGUMENT, 
	  "MJ_PluginDbTreeNodeMany::addNode","NULL node");
    return false;
  }
  else {
    m_nodes.append(node);
    return true;
  }
}

bool MJ_PluginDbTreeNodeMany::formTree(QList<MJ_PluginDbeBase> &source,
				       int iDepth)
{
  unsigned i;

  if(!source.count())
    return false;
  
  QString str0 = source.at(0)->menuPosition();
  int fields = MJ_fields(str0, MJ_pluginSeparator);
  
  // cout << " SOURCE = " << str0 << " FIELDS = " << fields << endl;

  if(fields == iDepth+1) {
    for(i=0; i < source.count(); i++) {
      
      MJ_PluginDbeBase *entry = source.at(i);
      MJ_PluginDbTreeNodeName *tmp = new MJ_PluginDbTreeNodeName();

      if(!tmp) MJ_fatal(MJ_ERR_MEMORY, "MJ_PluginDbTreeNodeMany::formTree");

      /* MJ_String name;
	 name.qString() = entry->menuPositionConst().copy();
	 
	 tmp->setName(name.nthLevel(iDepth)); */
      
      QString str1, str2(entry->menuPosition().copy());

      str1 = MJ_field(str2, iDepth, MJ_pluginSeparator);

      tmp->setName(str1);

      tmp->setEntry(entry);
      m_nodes.append(tmp);
    }
  }
  else {
    
    QListIterator<MJ_PluginDbeBase> iterator(source);
    MJ_PluginDbeBase * entry = 0;
    QList<MJ_PluginDbeBase> levelEntrys;

    // QString currLevelName = str.nthLevel(iDepth);
    QString currLevelName = MJ_field(str0, iDepth, MJ_pluginSeparator);
    QString currMenuPath;

    for(;(entry = iterator.current()); ++iterator) {
      /* MJ_String name(entry->menuPosition().data());
	 QString levelName = name.nthLevel(iDepth); */

      QString tmp = entry->menuPosition();
      QString levelName = MJ_field(tmp, iDepth, MJ_pluginSeparator);

      // Here is the bug: RRRRRRRRRRRRRRR !!!!!!!!!!!!!!!!!
      if(levelName != currLevelName) {

	if(levelEntrys.count() > 1 ||  
	   MJ_fields(currMenuPath, MJ_pluginSeparator) > iDepth + 1) {

	  MJ_PluginDbTreeNodeMany *node = new MJ_PluginDbTreeNodeMany();
	  node->setName(currLevelName);
	  
	  m_nodes.append(node);
	  node->formTree(levelEntrys, iDepth+1);
	  levelEntrys.clear();
	  
	  currLevelName = levelName;
	}
	else if(levelEntrys.count() == 1) {

	  MJ_PluginDbeBase *entry = levelEntrys.at(0);
	  MJ_PluginDbTreeNodeName *tmp = new MJ_PluginDbTreeNodeName();
	  
	  QString str1, str2(entry->menuPosition().copy());
	  
	  str1 = MJ_field(str2, iDepth, MJ_pluginSeparator);
	  
	  tmp->setName(str1);
	  
	  tmp->setEntry(entry);
	  m_nodes.append(tmp);
	  
	  levelEntrys.clear();
	  currLevelName = levelName;
	}
      }      
      
      levelEntrys.append(entry);
      currMenuPath = tmp;
    }
    // Get the last ones:
    if(levelEntrys.count() > 1 ||  
       MJ_fields(currMenuPath, MJ_pluginSeparator) > iDepth + 1) {
      MJ_PluginDbTreeNodeMany *node = new MJ_PluginDbTreeNodeMany();
      node->setName(currLevelName);
      m_nodes.append(node);
      node->formTree(levelEntrys, iDepth+1);
    }
    else if(levelEntrys.count() == 1) {

      MJ_PluginDbeBase *entry = levelEntrys.at(0);
      MJ_PluginDbTreeNodeName *tmp = new MJ_PluginDbTreeNodeName();

      QString str1, str2(entry->menuPosition().copy());

      str1 = MJ_field(str2, iDepth, MJ_pluginSeparator);

      tmp->setName(str1);

      tmp->setEntry(entry);
      m_nodes.append(tmp);
      
    }
  }
  return true;
}

bool MJ_PluginDbTreeNodeMany::filter(MJ_PluginFilter *f)
{
  bool ok = false;

  uint count = m_nodes.count();

  for(uint i=0; i < count; i++)
    ok = ok || m_nodes.at(i)->filter(f);
  
  return ok;
}
