/* COPYRIGHT
 *
 * This file is part of Mustajuuri LGPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. Extra clauses can be found from the file
 * MUSTAJUURI_LGPL and further explanations from the file
 * MUSTAJUURI_LICENSE. The license text can be found in file
 * "LGPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *    
 */

#include <mj_plugin_menu.h>

#include <mj_dsp_plugin.h>
#include <mj_dsp_plugin_loader.h>
#include <mj_dsp_plugin_filters.h>
#include <mj_error.h>
#include <mj_str_remap.h>
#include <mj_tr.h>

#include <qpopupmenu.h>
#include <qkeycode.h>
#include <qapplication.h>
#include <qmessagebox.h>

#include <typeinfo>

bool MJ_PluginDbTree_GQ::formTree(QObject         *callObj,
				  const char      *callSlot,
				  QPopupMenu      *menu,
				  MJ_PluginFilter *f)
{
  /* MJ_trace2("MJ_PluginDbTree_GQ::formTree # %s %p", 
     m_node->name().ascii(), this); */

  MJ_DspPluginFilterMetaFunc 
    fdefault(MJ_DspPluginMetaFunctionalityKeys[MJ_DPMF_CHAINABLE]);
  
  if(!f) f = & fdefault;

  if(!m_node) {
    MJ_error(MJ_ERR_UNINITIALIZED, "MJ_PluginDbTree_GQ::formTree");
    return false;
  }
  if(!menu) {
    MJ_error(MJ_ERR_ARGUMENT, "MJ_PluginDbTree_GQ::formTree","Need menu");
    return false;
  }

  if(dynamic_cast<MJ_PluginDbTreeNodeName *> (m_node)) {
    MJ_PluginDbTreeNodeName *node = (MJ_PluginDbTreeNodeName *)m_node;

    MJ_StrRemap *remap = 
      new MJ_StrRemap(node->dbe()->menuPosition());
    
    connect(remap, SIGNAL(emitCall(const QString &)), 
	    callObj, callSlot);

    
    callObj->insertChild(remap);

    int id = menu->insertItem(node->name(), remap, SLOT(call()));
    const QString & about = node->dbe()->parameter(MJ_PP_ABOUT);
    
    if(!about.isEmpty())
      menu->setWhatsThis(id, about);
  }
  else {
    MJ_PluginDbTreeNodeMany *node = (MJ_PluginDbTreeNodeMany *)m_node;
    
    if(node->filter(f)) {

      QPopupMenu *new_menu = new QPopupMenu(menu);
      
      uint i, filtered = 0;
      uint maxPerMenu = 30;
      uint minPerMenu = 5;

      filtered = filteredNodesLeft(node, 0, f);
      
      /* for(i=0; i < node->nofNodes(); i++) {

	if(node->getNode(i)->filter(f))
	  filtered++;
	  }*/
      
      // MJ_trace2("MJ_PluginDbTree_GQ::formTree # %u accepted nodes", filtered);

      if(filtered > maxPerMenu + minPerMenu) {
	
	QPopupMenu *new_menu2 = new QPopupMenu(new_menu);

	QString name = node->getNode(0)->name();
	QString endName;

	for(i=0; i < node->nofNodes(); i++) {

	  MJ_PluginDbTreeNode *new_node = node->getNode(i);
	  
	  if(!new_node->filter(f))
	    continue;

	  if(name.isEmpty())
	    name = new_node->name();

	  MJ_PluginDbTree_GQ *new_gui = new MJ_PluginDbTree_GQ;
	  new_gui->setNode(new_node);
	  new_gui->formTree(callObj, callSlot, new_menu2, f);
	  callObj->insertChild(new_gui);

	  endName = new_node->name();

	  if((((i + 1) % maxPerMenu) == 0) && 
	     (filteredNodesLeft(node, i, f) > minPerMenu)) {

	    if(name != endName)
	      name += " - " + endName;

	    new_menu->insertItem(name, new_menu2);
	    new_menu2 = new QPopupMenu(new_menu);

	    /* MJ_trace2("MJ_PluginDbTree_GQ::formTree # Added \"%s\"", 
	       name.ascii()); */
	    name.truncate(0);
	  }
	}
	// Get the final entries:
	if(new_menu2->count()) {
	  if(name != endName)
	    name += " - " + endName;

	  new_menu->insertItem(name, new_menu2);
	}

	// menu->insertItem(name, new_menu);
      }
      else {
	
	for(i=0; i < node->nofNodes(); i++) {
	  
	  MJ_PluginDbTreeNode *new_node = node->getNode(i);
	  
	  if(!new_node->filter(f))
	    continue;

	  MJ_PluginDbTree_GQ *new_gui = new MJ_PluginDbTree_GQ;
	  new_gui->setNode(new_node);
	  new_gui->formTree(callObj, callSlot, new_menu, f);
	  callObj->insertChild(new_gui);
	}
      }
      menu->insertItem(m_node->name(), new_menu);
      // callObj->insertChild(new_menu); // Should we do this ???????????
    }
  }

  /* MJ_trace2("MJ_PluginDbTree_GQ::formTree # EXIT %s %p", 
     m_node->name(). ascii(), this); */

  return true;
}

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

MJ_PluginMenu::MJ_PluginMenu( QWidget *parent, const char *name,
			      MJ_PluginDbeDspLoader *xpLoader )
  : QWidget( parent, name ),
    M_xpDB(0),
    M_xpLoader(xpLoader)
{
  const char * fname = "MJ_PluginMenu::MJ_PluginMenu";

  if(M_xpLoader)
    M_xpDB = &M_xpLoader->entryList();

  M_xObjs.setAutoDelete(TRUE);

  QPopupMenu *file = new QPopupMenu(parent);
  CHECK_PTR( file );
  M_xObjs.append(file);

  if(!M_xpDB) {
    MJ_error(MJ_ERR_UNKNOWN, fname, "Bad DB");
    return;
  }
  
  MJ_PluginDbTreeNodeMany tree;
  tree.formTree(xpLoader->entryList());
  tree.setName("Blocks");
  
  MJ_PluginDbTree_GQ tree_menu;
  tree_menu.setNode(&tree);

  tree_menu.formTree(this, SLOT(call(const QString &)), file);

  file->insertSeparator();
  file->insertItem( MJ_Tr::tr("E&xit"),  qApp, SLOT(quit()), CTRL+Key_Q );

  menu = new QMenuBar( this );
  CHECK_PTR( menu );
  M_xObjs.append(menu);
  menu->insertItem( MJ_Tr::tr("&File"), file );

  setMinimumSize( 50, 20 );
}

MJ_PluginMenu::~MJ_PluginMenu()
{
  MJ_trace("MJ_PluginMenu::~MJ_PluginMenu");
}

void MJ_PluginMenu::call(const QString &xCall)
{
  const char * fname = "MJ_PluginMenu::call";

  MJ_trace(fname, xCall);

  if(!signalsBlocked()) {
    emit emitName(xCall);
  }
  else {
    MJ_PluginDbeBase *dbe = 
      M_xpLoader->entryList().findDbe(xCall, MJ_PP_MENU_POSITION);
    
    if(!dbe) {
      MJ_error(MJ_ERR_INTERNAL, fname, "No entry");
      return;
    }
    // }*/
    
    MJ_PluginDbeDsp *dbeDsp = dynamic_cast<MJ_PluginDbeDsp *> (dbe);
    // MJ_PluginDbeDsp *dbeDsp = (MJ_PluginDbeDsp *) (dbe);
    
    if(!dbeDsp) {
      MJ_error(MJ_ERR_INTERNAL, fname, "Bad dbe type");
      return;
    }
    
    // cout << " \n Found entry. Creating block..." << flush;
    MJ_ModuleDsp *new_block = dbeDsp->newPlugin();
    
    if(!new_block) return;
  
    new_block->toCout();
    // cout << "One block worth of memory leak (!)" << endl;
  }
}

