/* COPYRIGHT
 *
 * This file is part of Mustajuuri LGPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. Extra clauses can be found from the file
 * MUSTAJUURI_LGPL and further explanations from the file
 * MUSTAJUURI_LICENSE. The license text can be found in file
 * "LGPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *    
 */

#include <mj_prefs.h>

#include <mj_error.h>
#include <mj_error_streams.h>
#include <mj_global.h>
#include <mj_misc.h>
#include <mj_strs.h>

#include <qfile.h> 
#include <qstring.h>
#include <qtextstream.h> 

#include <stdlib.h>
#include <string.h>

const char * MJ_PrefItemKeys[MJ_PREF_SIZEOF]  = {
  "allow_fpe",
  "try_rt",
  "trace_file",
  "all_to_trace_file",
  "frame_size",
  "no_watch_dog",
  "no_gui",
  "no_gui_warnings"
};

MJ_Prefs MJ_prefs;

MJ_Prefs::MJ_Prefs()
  : m_tryRtPrio(true),
    m_allowFpe(true),
    m_noWatchDog(false),
    m_noGuiWarnings(false),
    m_noGui(false),
    m_minimumLatency(50.0),
    m_maximumLatency(100.0),
    m_frameSize(128)
{
  if(this == &MJ_prefs) {

    const char * str = getenv(MJ_Strs::MUSTAJUURI_FRAME_SIZE);

    if(str) {
      m_frameSize = atol(str);
      if(m_frameSize < 1) m_frameSize = 1;
      else if(m_frameSize > 8192) m_frameSize = 8192;
    }
  }
}

void MJ_Prefs::scanArgs(int & argc, char ** argv)
{
  const char * fname = "MJ_Prefs::scanArgs";
  
  QCString fileName;

  // QFile *file;

  for(int i=0; i < argc; i++) {

    const char *str = argv[i];

    if(strncmp(str, "--", 2) == 0) {
      
      int item = MJ_findStr(&str[2], MJ_PrefItemKeys, MJ_PREF_SIZEOF);

      switch(item) {

      case MJ_PREF_ALLOW_FPE:
	
	m_allowFpe = true;

	MJ_shiftBack(i, 1, &argc, argv);
	break;

      case MJ_PREF_TRY_RT:

	if((i+1) < argc) {
	  m_tryRtPrio = atoi(argv[i+1]) != 0;
	  MJ_shiftBack(i, 2, &argc, argv);
	}
	else {
	  m_tryRtPrio = true;
	  MJ_shiftBack(i, 1, &argc, argv);
	}
	i--;
	
	MJ_trace(fname, (int) m_tryRtPrio);

	break;
	
      case MJ_PREF_TRACE_FILE:


	if((i+1) < argc) {
	  fileName = argv[i+1];
	  MJ_shiftBack(i, 2, &argc, argv);
	}
	else
	  MJ_shiftBack(i, 1, &argc, argv);

	if(fileName.isEmpty())
	  fileName = "mustajuuri.log";
	
	/* file = new QFile(fileName);
	   file->open(IO_WriteOnly);*/
	
	MJ_traceFile = fopen(fileName, "w");
	
	i--;
	break;
	
      case MJ_PREF_ALL_TO_TRACE_FILE:
	
	MJ_shiftBack(i, 1, &argc, argv);
	MJ_errorsToFile = true;
	break;
	
      case MJ_PREF_FRAME_SIZE:

	if((i+1) < argc) {
	  m_frameSize = atol(argv[i+1]);
	  MJ_shiftBack(i, 2, &argc, argv);
	}
	else {
	  MJ_shiftBack(i, 1, &argc, argv);
	}
	i--;
	
      case MJ_PREF_NO_WATCH_DOG:
	
	MJ_shiftBack(i, 1, &argc, argv);
	m_noWatchDog = true;
	break;
	
      case MJ_PREF_NO_GUI_WARNINGS:
	
	MJ_shiftBack(i, 1, &argc, argv);
	m_noGuiWarnings = true;
	break;
	
      case MJ_PREF_SIZEOF:
      default:
	break;
      }
    }
  }
}
