/* COPYRIGHT
 *
 * This file is part of Mustajuuri LGPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. Extra clauses can be found from the file
 * MUSTAJUURI_LGPL and further explanations from the file
 * MUSTAJUURI_LICENSE. The license text can be found in file
 * "LGPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *    
 */

#ifndef MJ_PREFS_H
#define MJ_PREFS_H

enum MJ_PrefItem {
  MJ_PREF_ALLOW_FPE,
  MJ_PREF_TRY_RT,
  MJ_PREF_TRACE_FILE,
  MJ_PREF_ALL_TO_TRACE_FILE,
  MJ_PREF_FRAME_SIZE,
  MJ_PREF_NO_WATCH_DOG,
  MJ_PREF_NO_GUI,
  MJ_PREF_NO_GUI_WARNINGS,
  MJ_PREF_SIZEOF
};

extern const char * MJ_PrefItemKeys[MJ_PREF_SIZEOF];

/// Mustajuuri general preferences
class MJ_Prefs
{
public:

  MJ_Prefs();

  ~MJ_Prefs() {}
  
  void scanArgs(int & argc, char ** argv);

  inline bool tryRtPrio() const { return m_tryRtPrio; }
  void setTryRtPrio(bool b) { m_tryRtPrio = b; }

  /// Should we allow FPEs to happen.
  inline bool allowFpe() { return m_allowFpe; }

  void setLatencies(float minLat, float maxLat)
  { m_minimumLatency = minLat; m_maximumLatency = maxLat; }

  float minimumLatency() { return m_minimumLatency; }
  float maximumLatency() { return m_maximumLatency; }

  int frameSize() const { return m_frameSize; }

  bool noWatchDog() const { return m_noWatchDog; }
  bool noGuiWarnings() const { return m_noGuiWarnings; }
  bool noGui() const { return m_noGuiWarnings; }

protected:
  
  bool m_tryRtPrio;
  bool m_allowFpe;
  bool m_noWatchDog;
  bool m_noGuiWarnings;
  bool m_noGui;
  
  float m_minimumLatency, m_maximumLatency;

  int m_frameSize;
};

extern MJ_Prefs MJ_prefs;

#endif
