/* COPYRIGHT
 *
 * This file is part of Mustajuuri LGPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. Extra clauses can be found from the file
 * MUSTAJUURI_LGPL and further explanations from the file
 * MUSTAJUURI_LICENSE. The license text can be found in file
 * "LGPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *    
 */

#include <mj_realtime_scheduling.h>
#ifndef WIN32

#include <unistd.h>
#include <sched.h>

#include <stdio.h>
#include <stdlib.h>

#include <di_types.h>

/// Call system's real-time scheduling facilities
bool MJ_realTimeScheduling(int priority, int pid)
{
  pid_t xPid;
  int iSchedPolicy;
  
  if(pid <= 0) xPid = 0;
  else xPid = (pid_t) pid;
  
  if(priority <= 0) iSchedPolicy = SCHED_OTHER;
  else iSchedPolicy = SCHED_FIFO;
  
  if(iSchedPolicy != SCHED_OTHER && (priority <= 0 || priority > 99)) {
    fprintf(stderr, "Need priority in range 1-99, requested: %d", priority);
    fflush(0);
    return false;
  }
  
  sched_param xParam;
  xParam.sched_priority = priority;

  if(sched_setscheduler(xPid, iSchedPolicy, &xParam) == -1)
    return false;

  return true;
}
#else
#include <windows.h>
bool MJ_realTimeScheduling(int priority, int pid)
{
	SetThreadPriority(GetCurrentThread(),priority);
	return true;
}

#endif		//#ifndef WIN32
