/* COPYRIGHT
 *
 * This file is part of Mustajuuri LGPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. Extra clauses can be found from the file
 * MUSTAJUURI_LGPL and further explanations from the file
 * MUSTAJUURI_LICENSE. The license text can be found in file
 * "LGPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *    
 */

#ifdef __OS_LINUX__

#include <sched.h>
#include <linux/version.h>

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 4, 0)
#include <linux/rtc.h>
#else 
#include <linux/mc146818rtc.h>
#endif

#include <sys/ioctl.h>
#include <sys/time.h>
#include <fcntl.h>
#include <stdio.h>
#include <unistd.h>

int MJ_rtRtcTestInt()
{
  puts("MJ_rtRtcTestInt");

  int fd = open("/dev/rtc", O_RDONLY);
  int hz = 2048;
  int secs = 3600 * 5;
  int loop = 1;
  ulong maxDiff = 0, tmp;
  int count;

  unsigned long data;
  struct timeval tv1, tv2, tv3;

  if(fd == -1) return 0;

  if(ioctl(fd, RTC_IRQP_READ, &hz) == -1) return 0;
  if(ioctl(fd, RTC_PIE_ON, 0) == -1) return 0;

  gettimeofday(&tv3, 0);

  pid_t pid = getpid();
  sched_param sp;
  sp.sched_priority = 80;
  
  if(sched_setscheduler(pid, SCHED_FIFO, &sp) == -1)
    return 0;

  puts("Enter test loop");
  
  for(count = 0; loop; count++) {
    gettimeofday(&tv1, 0);

    read(fd, &data, sizeof(unsigned long));
    read(fd, &data, sizeof(unsigned long));

    gettimeofday(&tv2, 0);
    tmp = (tv2.tv_sec - tv1.tv_sec) * 1000000 + 
      (tv2.tv_usec - tv1.tv_usec);

    if(tmp > maxDiff) {
      printf("New max = %d\n", (int) tmp);
      fflush(0);
      maxDiff = tmp;
    }

    loop = (tv2.tv_sec - tv3.tv_sec) < secs;
    
    if(count % (hz * 5) == 0) {
      printf("x"); 
      fflush(0);
    }
  }

  printf("Max diff = %d\n", (int) maxDiff);
  fflush(0);
  return 1;
}

int MJ_rtRtcTest(void *)
{
  if(!MJ_rtRtcTestInt())
    puts("MJ_rtRtcTestInt FAILED");

  return 0;
}

#endif
