/* COPYRIGHT
 *
 * This file is part of Mustajuuri LGPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. Extra clauses can be found from the file
 * MUSTAJUURI_LGPL and further explanations from the file
 * MUSTAJUURI_LICENSE. The license text can be found in file
 * "LGPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *    
 */

#include <mj_signal_frame_gui.h>

#include <mj_gui_utils.h> 
#include <mj_line_edit.h> 
#include <mj_tr.h>


#include <qcombobox.h> 
#include <qlabel.h> 
#include <qlayout.h> 
#include <qslider.h> 


#define MJ_SAMPLING_RATES 9

static const int MJ_SignalRates[MJ_SAMPLING_RATES] = {
  8000,
  11025,
  16000,
  22050,
  32000,
  44100,
  48000,
  88200,
  96000
};

MJ_SignalFrameGui::MJ_SignalFrameGui(int samplingRate,
				     int frameSize,
				     int maxFrame,
				     MJ_RtControl *rtParent,
				     QWidget *parent,
				     const char *name)
  : QWidget(parent, name),
    MJ_RtControl(rtParent),
    m_maxFrame(maxFrame)
{
  if(!parent) {
    QString str(MJ_Tr::tr("Signal configuration"));
    setCaption(str);
    setIconText(str);    
  }

  int i, j;
  
  QGridLayout *grid = new QGridLayout(this, 3, 2, 10, 5);
  MJ_equalStretch(grid);
  grid->setColStretch(1, 60);
  
  // Sampling rate:

  grid->addWidget(new QLabel(MJ_Tr::tr("Sampling rate"), this), 0, 0);
  
  QComboBox *combo = new QComboBox(this);
  
  QString str;
  
  for(i=0; (j = indexToRate(i)) != 0; i++) {
    str.setNum(((float) j * (float) 0.001), 'g', 5);
    str += " " + MJ_Tr::tr("kHz");
    combo->insertItem(str);
  }

  combo->setCurrentItem(rateToIndex(samplingRate));
  m_combo = combo;

  grid->addWidget(combo, 0, 1);

  connect(combo, SIGNAL(activated(int)), this, SLOT(selectRate(int)));

  // Frame size

  m_is.createObjs(this);

  m_is.toGridHorizontal(grid, 1, 0);
  m_is.label()->setText(MJ_Tr::tr("Cycle size (samples)"));
  m_is.set(frameSize, 1, maxFrame);

  connect(& m_is, SIGNAL(emitValue(int)), this, SLOT(setFrameSize(int)));
  connect(& m_is, SIGNAL(emitGetValue(int *)), 
	  this, SLOT(getFrameSize(int *)));

  realTimeChange(isRealTime());
}

int MJ_SignalFrameGui::indexToRate(int i)
{
  if(i >= MJ_SAMPLING_RATES) return 0;
  
  return MJ_SignalRates[i];
}

int MJ_SignalFrameGui::rateToIndex(int i)
{
  for(int j=0; j < MJ_SAMPLING_RATES; j++) {
    if(MJ_SignalRates[j] == i) return j;
  }
  return -1;
}

void MJ_SignalFrameGui::selectRate(int i)
{
  emit masterRate(indexToRate(i));
}

void  MJ_SignalFrameGui::setFrameSize(int s)
{
  if(s < 1) s = 1;
  else if(s > m_maxFrame) s = m_maxFrame;

  emit emitFrameSize(s);
}

void  MJ_SignalFrameGui::getFrameSize(int *s)
{
  emit emitGetFrameSize(s);
}

void MJ_SignalFrameGui::realTimeChange (bool nowRt)
{
  bool enable = !nowRt;
  m_combo->setEnabled(enable);
  m_is.slider()->setEnabled(enable);
  m_is.edit()->setEnabled(enable);
}
