/* COPYRIGHT
 *
 * This file is part of Mustajuuri LGPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. Extra clauses can be found from the file
 * MUSTAJUURI_LGPL and further explanations from the file
 * MUSTAJUURI_LICENSE. The license text can be found in file
 * "LGPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *    
 */

#include <mj_sleep.h>

#include <mj_mutex.h>
#include <mj_time_stamp.h>

#ifndef WIN32
long MJ_SleepSync::sleepTo(const DI_TimeStamp *stamp, MJ_Mutex *mutex)
{
  gettimeofday(&m_tmp, 0);
  m_timing.tv_sec = (long) stamp->secondsUl();
  m_timing.tv_usec = (long) stamp->subSecondsUs();

  MJ_addTime(& m_timing, & m_initial);

  long diffUs = MJ_timeDiffUs(&m_tmp, &m_timing);
  if(diffUs < 0) return diffUs;

#ifdef __OS_LINUX__
  // Minimum 11 ms sleep.
  if(diffUs < 11000) diffUs = 11000;
#endif
  
  if(mutex) mutex->unlock();
  sleepUs(diffUs);
  if(mutex) mutex->lock();

  gettimeofday(&m_tmp, 0);
  return MJ_timeDiffUs(&m_tmp, &m_timing);
}

long MJ_SleepSync::sleepSynchroUs(long us)
{
  MJ_addTimeUs(&m_timing, us);
  gettimeofday(&m_tmp, 0);

  long diffUs = MJ_timeDiffUs(&m_tmp, &m_timing);
  if(diffUs < 0) return diffUs;

  sleepUs(diffUs);
  gettimeofday(&m_tmp, 0);
  return MJ_timeDiffUs(&m_tmp, &m_timing);
}
#endif
