/* COPYRIGHT
 *
 * This file is part of Mustajuuri LGPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. Extra clauses can be found from the file
 * MUSTAJUURI_LGPL and further explanations from the file
 * MUSTAJUURI_LICENSE. The license text can be found in file
 * "LGPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *    
 */

#include <dlfcn.h>

#include <mj_error.h>

#include <mj_so_ref_count.h>

bool MJ_SoRefCount::m_debug = false;

int MJ_SoRefCount::m_objects = 0;

MJ_SoRefCount::MJ_SoRefCount(void *handle, const QString &file)
  : m_plugins(0), m_fileName(file), m_handle(handle) 
{
  m_objects++;

  if(m_debug)
    MJ_trace2("MJ_SoRefCount::MJ_SoRefCount %p %p %s", 
	      this, handle, (const char *) file);
}

MJ_SoRefCount::~MJ_SoRefCount()
{
  const char * fname = "MJ_SoRefCount::~MJ_SoRefCount";

  m_objects--;


  if(m_debug)
    MJ_trace2("%s # %p %d", fname, this, m_plugins);

  if(m_objects < 0) {
    MJ_fatal(MJ_ERR_INTERNAL, fname);
  }
  
  if(m_plugins) {
    
    MJ_error(MJ_ERR_INTERNAL, fname,
	     "Still references left, will not close shared library",
	     m_fileName);
  }
  else {
    /* The handle was received in the "Read" or "Read2" method. 
       It is deleted here. */

    if(m_debug)
      MJ_trace2("%s # Unloading \"%s\"", 
		fname, (const char *) m_fileName);
  

    if(m_handle) {
      if(dlclose(m_handle)) {
	MJ_error2(MJ_ERR_UNKNOWN, 
		  "%s # Error closing dynamic library %s handle: %s", 
		  fname, (const char *) m_fileName, 
		  dlerror());
      }
      else if(m_debug)
	MJ_trace2("%s # Successfully closed %s", 
		  fname, (const char *) m_fileName);
    }
  }
}
