/* COPYRIGHT
 *
 * This file is part of Mustajuuri LGPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. Extra clauses can be found from the file
 * MUSTAJUURI_LGPL and further explanations from the file
 * MUSTAJUURI_LICENSE. The license text can be found in file
 * "LGPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *    
 */

#include <mj_error.h>
#include <mj_stream_utils.h>

#include <qtextstream.h> 

/** Tries to move QTextStream IO device pointer by delta bytes.  Can
    be called with NULL argument.  */
bool MJ_moveTextStream(QTextStream *s, int delta)
{
  if(!s) return false;
  return MJ_moveIODevice(s->device(), delta);
}

/** Tries to move IO device pointer by delta bytes. Can be called with
    NULL argument. */

bool MJ_moveIODevice(QIODevice *d, int delta)
{
  if(!d) return false;
  return d->at(d->at() + delta);
}

/** Returns the current line number of (ascii) device.

    If search fails, -1 is returned.
 */

int MJ_currentLine(QIODevice *d)
{
  int line = 0;

  uint p = d->at();

  if(!d->at(0)) return -1;

  QTextStream s(d);

  while((uint) d->at() < p) {
    s.readLine();
    line++;
  }

  d->at(p);
  return line;
}


void MJ_errorNearLine(QIODevice *d, const char *fname)
{
  int line = MJ_currentLine(d);

  QCString str;

  if(fname) {
    str.resize(strlen(fname) + 64);
    str.sprintf("%s # Error near line %d", fname, line);
  }
  else
    str.sprintf("Error near line %d", line);

  MJ_error(MJ_ERR_UNKNOWN, str.data());
}
