/* COPYRIGHT
 *
 * This file is part of Mustajuuri LGPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. Extra clauses can be found from the file
 * MUSTAJUURI_LGPL and further explanations from the file
 * MUSTAJUURI_LICENSE. The license text can be found in file
 * "LGPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *    
 */

#include <mj_to_ladspa_plugin.h>

#include <mj_messages.h>

#include <stdlib.h>

MJ_ModuleAdjustMessage::MJ_ModuleAdjustMessage(const char *id, MJ_Message *msg)
{
  m_msg.setId(id);
  m_msg.setMessage(msg);
}

MJ_ModuleAdjustMessage::~MJ_ModuleAdjustMessage()
{}
  
bool MJ_ModuleAdjustMessage::apply(MJ_ModuleDsp *m)
{
  return m->msgProcess(m_msg.id(), m_msg.message());
}

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////


MJ_ModuleAdjustChannels::~MJ_ModuleAdjustChannels()
{}
  
bool MJ_ModuleAdjustChannels::apply(MJ_ModuleDsp *m)
{
  return m->dspSetInChannels(m_in) && m->dspSetOutChannels(m_out);
}

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

MJ_LadspaData::MJ_LadspaData(ulong id, const char *idStr, const char * name)
  : m_id(id), m_idStr(strdup(idStr)), m_name(name ? strdup(name) : 0)
{
  m_adjustments.setAutoDelete(true);
}

MJ_LadspaData::~MJ_LadspaData()
{
  if(m_idStr) free(m_idStr);
  if(m_name)  free(m_name);
}

bool MJ_LadspaData::apply(MJ_ModuleDsp *m)
{
  bool ok = true;

  uint i, count = m_adjustments.count();

  for(i=0; i < count; i++) {
    ok = m_adjustments.at(i)->apply(m);
  }

  return ok;
}

void MJ_LadspaData::addMsgDouble(const char *id, double v)
{ 
  m_adjustments.
    append(new MJ_ModuleAdjustMessage(id, new MJ_MessageDouble(v))); 
}

void MJ_LadspaData::addMsgInt(const char *id, int v)
{ 
  m_adjustments.
    append(new MJ_ModuleAdjustMessage(id, new MJ_MessageInt(v))); 
}

void MJ_LadspaData::setChannels(uint in, uint out)
{
  m_adjustments.append(new MJ_ModuleAdjustChannels(in, out)); 
}


/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
  
MJ_LadspaDatas::~MJ_LadspaDatas()
{}
  
MJ_LadspaData *MJ_LadspaDatas::newElem
(ulong id, const char *idStr, const char * name)
{ 
  MJ_LadspaData *ld = new MJ_LadspaData(id, idStr, name); 
  if(ld) m_datas.append(ld);
    
  return ld;
}

/** Add a new LADSPA element. This method also sets the input-and
    output channels of the plugin. This method returns pointer to the
    new LADSPA element. */

MJ_LadspaData *MJ_LadspaDatas::newElem
(ulong id, const char *idStr, const char * name, uint inchans, uint outchans)
{ 
  MJ_LadspaData *ld = new MJ_LadspaData(id, idStr, name); 

  if(ld) {
    m_datas.append(ld);
    ld->setChannels(inchans, outchans);
  }

  return ld;
}

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

// Instantiate a few templates:

template class QList<MJ_ModuleAdjust>;
template class QList<MJ_LadspaData>;
