/* COPYRIGHT
 *
 * This file is part of Mustajuuri LGPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. Extra clauses can be found from the file
 * MUSTAJUURI_LGPL and further explanations from the file
 * MUSTAJUURI_LICENSE. The license text can be found in file
 * "LGPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *    
 */

#include <mj_version.h>

#include <mj_error.h>
#include <mj_iostream.h>
#include <mj_misc.h>

#include <qfile.h> 
#include <qtextstream.h> 

const char *MJ_Version::m_label = "version";

MJ_Version::~MJ_Version()
{}

bool MJ_Version::exceedsThis(const MJ_Version &v) const
{
  if(v.m_major > m_major) return true;
  if(v.m_minor > m_minor) return true;
  if(v.m_subminor > m_subminor) return true;
  return false;
}

bool MJ_Version::equals(const MJ_Version &v) const
{
  return (v.m_major == m_major &&
	  v.m_minor == m_minor && 
	  v.m_subminor == m_subminor);
}

bool MJ_Version::write(QTextStream *s)
{
  (*s) << m_label << " " 
       << m_major << "." << m_minor << "." << m_subminor 
       << " " << m_info << "\n";
  return true;
}

bool MJ_Version::read(QTextStream *s)
{
  QString str, str2, str3;
  bool ok = true;

  str = s->readLine();
  
  str3 = MJ_field(str, 0);
  
  if(str3 !=  m_label) {
    MJ_error(MJ_ERR_ARGUMENT, "MJ_Version::read");
    return false;
  }
  
  str2 = MJ_field(str, 1);
  
  m_major = MJ_field(str2, 0, ".").toInt(&ok);
  m_minor = MJ_field(str2, 1, ".").toInt(&ok);
  m_subminor = MJ_field(str2, 2, ".").toInt(&ok);

  int startInfo = str3.length() + str2.length();
  m_info = str;
  m_info.remove(0, startInfo); 
  return ok;
}

void MJ_Version::test()
{
  const char *filename = "foo.version";
  
  MJ_Version v2, v1(67, 1, 345, "beta1 pre 7");
  
  {
    QFile f1(filename);
    f1.open(IO_WriteOnly);
    QTextStream s1(&f1);
    v1.write(&s1);
    v1.write(& MJ_cout());
  }
  MJ_cout() << "\nWRITTEN 1\n";
  fflush(0);
  {
    QFile f2(filename);
    f2.open(IO_ReadOnly);
    QTextStream s2(&f2);
    v2.read(&s2);
    v2.write(& MJ_cout());
  }
  MJ_cout() << "\nWRITTEN 2\n";
  fflush(0);

  if(v1.equals(v2))
    MJ_cout() << "SUCCESS\n";
  else
    MJ_cout() << "FAILURE\n";
  fflush(0);
}
