/* COPYRIGHT
 *
 * This file is part of Mustajuuri LGPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. Extra clauses can be found from the file
 * MUSTAJUURI_LGPL and further explanations from the file
 * MUSTAJUURI_LICENSE. The license text can be found in file
 * "LGPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *    
 */

#include <mj_watch_dog.h>

#include <mj_error.h>
#include <mj_global.h>
#include <mj_kernel.h>
#include <mj_root_powers.h> 
#include <mj_sleep.h> 
#include <mj_realtime_scheduling.h>

MJ_WatchDog * MJ_WatchDog::m_dog = 0;

MJ_WatchDog::MJ_WatchDog() 
  : m_guard(0), 
  m_guard2(0), 
  m_loop(true) 
{ 
  if(m_dog) MJ_fatal(MJ_ERR_INTERNAL, 
		     "MJ_WatchDog::MJ_WatchDog # THERE CAN ONLY BE ONE");
  m_dog = this; 
}

MJ_WatchDog::~MJ_WatchDog() 
{
  m_dog = 0; 
}

void MJ_WatchDog::childLoop()
{
  const char * fname = "MJ_WatchDog::childLoop";

  MJ_realTimeScheduling(MJ_RTP_WATCHDOG);

  MJ_Sleep s;

  const uint sleepMs = 200;
  const uint guardCheckInterval = 8000; // Check interval in ms.
  const uint checkMod = guardCheckInterval / sleepMs + 1;
  uint count = checkMod;
  uint count2 = checkMod;

  if(m_guard) * m_guard = false;

  while(m_loop) {
    s.sleepMs(sleepMs);

    // First variable
    if(m_guard && !count) {
      if(!*m_guard) {
	if(MJ_kernel) {
	  MJ_error(MJ_ERR_INTERNAL, fname);
	  MJ_kernel->panic();
	}
	MJ_fatal(MJ_ERR_INTERNAL, fname);
      }
      *m_guard = false;
      count = checkMod;
    }

    // Second variable
    if(m_guard2 && !count2) {
      if(!*m_guard2) {
	if(MJ_kernel) {
	  MJ_error(MJ_ERR_INTERNAL, fname, 2);
	  MJ_kernel->panic();
	}
	MJ_fatal(MJ_ERR_INTERNAL, fname);
      }
      *m_guard2 = false;
      count2 = checkMod;
    }

    count--;    
    count2--;    
  }

  // MJ_trace(fname, "EXIT");
}
