/* COPYRIGHT
 *
 * This file is part of Mustajuuri LGPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. Extra clauses can be found from the file
 * MUSTAJUURI_LGPL and further explanations from the file
 * MUSTAJUURI_LICENSE. The license text can be found in file
 * "LGPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *    
 */

#include <mj_message_numeric_tmpl.C>

#include <di_data_buffer.h>

// template <class double>
template<> MJ_MessageType MJ_MessageNumeric<double>::messageType() const
{
  return MJ_MSG_DOUBLE;
}

template<> 
bool MJ_MessageNumeric<double>::write2(DI_DataBuffer2 *b)
{
  if(!MJ_Message::write2(b)) return false;
  
  b->writeFloat64(m_value);

  return true;
}

template<> 
bool MJ_MessageNumeric<double>::read2(DI_DataBuffer2 *b, int readFields)
{
  if(!MJ_Message::read2(b, readFields)) return false;

  m_value = b->readFloat64();
  
  return true;
}

template<> 
std::string MJ_MessageNumeric<double>::describe()
{
  std::string tmp;
  tmp.resize(32);
  sprintf(& * tmp.begin(), "%5.4f", m_value);
  return tmp;
}

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

template<> MJ_MessageType MJ_MessageNumeric<int>::messageType() const
{
  return MJ_MSG_INT;
}

template<> 
bool MJ_MessageNumeric<int>::write2(DI_DataBuffer2 *b)
{
  if(!MJ_Message::write2(b)) return false;
  
  b->writeInt32(m_value);

  return true;
}

template<> 
bool MJ_MessageNumeric<int>::read2(DI_DataBuffer2 *b, int readFields)
{
  if(!MJ_Message::read2(b, readFields)) return false;

  m_value = b->readInt32();
  
  return true;
}

template<> 
std::string MJ_MessageNumeric<int>::describe()
{
  std::string tmp;
  tmp.resize(16);
  sprintf(& * tmp.begin(), "%d", m_value);
  return tmp;
}

template class MJ_MessageNumeric<double>;
template class MJ_MessageNumeric<int>;
