/* COPYRIGHT
 *
 * This file is part of Mustajuuri LGPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. Extra clauses can be found from the file
 * MUSTAJUURI_LGPL and further explanations from the file
 * MUSTAJUURI_LICENSE. The license text can be found in file
 * "LGPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *    
 */

#include <mj_messages.h>

#include <mj_error.h>
// #include <mj_tr.h>

#include <di_data_buffer.h>

#include <stdio.h>

/*#include <qapplication.h> 
  #include <qtextstream.h> */


MJ_MessageStlString::MJ_MessageStlString(const MJ_MessageStlString &s)
  : MJ_Message(s)
{
  m_string = s.m_string;
}

MJ_MessageStlString::MJ_MessageStlString(const std::string &s)
  : m_string(s)
{}


MJ_MessageType MJ_MessageStlString::messageType() const
{
  return MJ_MSG_STL_STRING;
}

MJ_Message *MJ_MessageStlString::duplicate()
{
  return new MJ_MessageStlString(*this);
}

double MJ_MessageStlString::toDouble(bool *ok)
{
  if(!m_string.size()) {
    if(ok) *ok = false;
    return 0.0;
  }

  double tmp = 0.0;

  if(sscanf(& * m_string.begin(), "%lf", & tmp) != 1 && ok)
    (*ok) = false;

  return tmp;
}

int MJ_MessageStlString::toInt(bool *ok)
{
  if(!m_string.size()) {
    if(ok) *ok = false;
    return 0;
  }

  int tmp = 0;

  if(sscanf(& * m_string.begin(), "%d", & tmp) != 1 && ok)
    (*ok) = false;

  return tmp;
}


bool MJ_MessageStlString::write2(DI_DataBuffer2 *b)
{
  if(!MJ_Message::write2(b)) return false;
  
  b->writeString(& m_string);

  return true;
}

bool MJ_MessageStlString::read2(DI_DataBuffer2 *b, int readFields)
{
  if(!MJ_Message::read2(b, readFields)) return false;

  b->readString(& m_string);
  
  return true;
}

std::string MJ_MessageStlString::describe()
{
  return m_string;
}

MJ_Message *MJ_MessageStlString::createNew()
{
  return new MJ_MessageStlString;
}

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

MJ_MessageType MJ_MessageBool::messageType() const
{
  return MJ_MSG_BOOL;
}


MJ_Message *MJ_MessageBool::duplicate()
{
  return new MJ_MessageBool(*this);
}

double MJ_MessageBool::toDouble(bool *ok)
{
  if(ok) *ok = true;
  return m_val;
}

int    MJ_MessageBool::toInt(bool *ok)
{
  if(ok) *ok = true;
  return m_val;
}

bool MJ_MessageBool::toBool(bool *ok)
{
  if(ok) *ok = true;
  return m_val;
}

bool MJ_MessageBool::write2(DI_DataBuffer2 *b)
{
  if(!MJ_Message::write2(b)) return false;
  
  b->writeByte(m_val);

  return true;
}

bool MJ_MessageBool::read2(DI_DataBuffer2 *b, int readFields)
{
  if(!MJ_Message::read2(b, readFields)) return false;

  m_val = b->readByte();
  
  return true;
}

std::string MJ_MessageBool::describe()
{
  const char * v = m_val ? "true" : "false";

  /* if(qApp) {
    return MJ_Tr::tr(v);
    }*/

  return v;
}
  
MJ_Message *MJ_MessageBool::createNew()
{
  return new MJ_MessageBool;
}

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

MJ_MessageType MJ_MessageMidiEvent::messageType() const
{
  return MJ_MSG_MIDI_EVENT;
}

MJ_Message *MJ_MessageMidiEvent::duplicate()
{
  return new MJ_MessageMidiEvent(*this);
}

bool MJ_MessageMidiEvent::write2(DI_DataBuffer2 *b)
{
  if(!MJ_Message::write2(b)) return false;
  
  return m_msg.write(b);
}

bool MJ_MessageMidiEvent::read2(DI_DataBuffer2 *b, int readFields)
{
  if(!MJ_Message::read2(b, readFields)) return false;

  return m_msg.read(b);
}

std::string MJ_MessageMidiEvent::describe()
{
  std::string tmp("MIDI event");
  return tmp;
}

MJ_Message *MJ_MessageMidiEvent::createNew()
{
  return new MJ_MessageMidiEvent;
}
