/* COPYRIGHT
 *
 * This file is part of Mustajuuri LGPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. Extra clauses can be found from the file
 * MUSTAJUURI_LGPL and further explanations from the file
 * MUSTAJUURI_LICENSE. The license text can be found in file
 * "LGPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *    
 */

#include <mj_delay_line.h>

/** Process many samples. This method is internally optimized to
    eliminate delay time interpolation if it is not needed. */

template <class T> 
void MJ_DelayLine<T>::process(T *target, const T *source, unsigned n)
{
  if(m_delaySamples.isActive()) {
    
    for(const T *sentinel = source + n; source < sentinel; ) {
      
      m_delaySamples.advanceLinear();
      double delay = m_delaySamples.get();
      m_delay.put(*source++);
      
      //*target++ = m_delay.getNewest((unsigned) delay);
      *target++ = m_delay.getInterpolatedNewest1D(delay);
    }
  }
  else {
    unsigned d = (unsigned) (m_delaySamples.get() + 0.5);
    
    for(const T *sentinel = source + n; source < sentinel; ) {
      m_delay.put(*source++);
      *target++ = m_delay.getNewest(d);
    }
  }
}

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

/** Process many samples. This method is internally optimized to
    eliminate delay time interpolation if it is not needed. */

template <class T> 
void MJ_DelayLine2<T>::process(T *target, const T *source, unsigned n)
{
  if(m_delaySamples.isActive()) {
    
    for(const T *sentinel = source + n; source < sentinel; ) {
      
      m_delaySamples.advanceLinear();
      double delay = m_delaySamples.get();
      m_delay.put(*source++);
      
      //*target++ = m_delay.getNewest((unsigned) delay);
      m_gain.advanceLinear();
      *target++ = m_delay.getInterpolatedNewest1D(delay) * m_gain.get();
    }
  }
  else {
    unsigned d = (unsigned) (m_delaySamples.get() + 0.5);
    
    if(m_gain.isActive()) {
    
      for(const T *sentinel = source + n; source < sentinel; ) {
	m_delay.put(*source++);
	m_gain.advanceLinear();
	*target++ = m_delay.getNewest(d) * m_gain.get();
      }
    }
    else {
      for(const T *sentinel = source + n; source < sentinel; ) {
	m_delay.put(*source++);
	*target++ = m_delay.getNewest(d) * m_gain.get();
      }
    }
  }
}
