/* COPYRIGHT
 *
 * This file is part of Mustajuuri LGPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. Extra clauses can be found from the file
 * MUSTAJUURI_LGPL and further explanations from the file
 * MUSTAJUURI_LICENSE. The license text can be found in file
 * "LGPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *    
 */

#include <mj_math.h>

/* We used to define mathematical functions here, but those are
   inlined now. */

/** Finds the divider of the number. If the number is a prime number,
    then zero is returned. */ 
unsigned MJ_findDivider(unsigned l)
{
  unsigned sentinel = (unsigned) (sqrt((double) l)) + 1;

  for(unsigned i = 2; i < sentinel; i++)
    if((l / i) * i == l) return i;

  return 0;
}

/** Find the next prime number, starting from l. */

unsigned MJ_nextPrime(unsigned l)
{
  for(unsigned i=l; i != 0xffffffff; i++)
    if(MJ_isPrime(i)) return i;
  return 0;
}

/** Find the previous prime number, starting from l. 

    @return This function returns either the previous prime number, or
    zero if no prime number could be found.
*/

unsigned MJ_previousPrime(unsigned l)
{
  for(unsigned i=l; i; i--)
    if(MJ_isPrime(i)) return i;

  return 0;
}
