/* COPYRIGHT
 *
 * This file is part of Mustajuuri LGPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. Extra clauses can be found from the file
 * MUSTAJUURI_LGPL and further explanations from the file
 * MUSTAJUURI_LICENSE. The license text can be found in file
 * "LGPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *    
 */

#include <mj_resampler.h>

#include <stdio.h>

template <class TElem>
void MJ_Resampler<TElem>::setParameters(TElem frequencyLimit,
					uint upsample, 
					uint filterOrder)
{
  m_filter.setOrder(filterOrder);
  
  m_filter.coeffs().designLowPassNormalized(frequencyLimit / (TElem) upsample,
					    MJ_WINDOW_HAMMING);
  m_filter.resetMemory();

  m_buffer.resize(upsample + 2);  // Need extra space due to interpolation
  m_buffer.setAll(0.0);

  m_multiply = upsample;
  m_lastInput = 0.0;

  m_currPos = 0.0;
}

template <class TElem>
void MJ_Resampler<TElem>::put(TElem v)
{
  TElem scale = 1.0 / (TElem) m_multiply;

  for(uint i = 1; i <= m_multiply; i++) {
    TElem scaled = i * scale;

    TElem tmp = m_lastInput * (1.0 - scaled) + v * scaled;

    m_filter.put(tmp);
    // m_buffer.advance();
    m_buffer.put(m_filter.get());
    // printf("buffer got %3.3f\n", m_buffer.getNewest());
  }

  m_lastInput = v;
  m_currPos += m_multiply;
}

/* template <class TElem>
TElem MJ_Resampler<TElem>::getResampled(double advance)
{
  TElem ret = m_buffer.getInterpolatedNewest3DFast(m_currPos);
  m_currPos -= advance;
  return ret;
  }*/
