/* COPYRIGHT
 *
 * This file is part of Mustajuuri GPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * Tommi.Ilmonen(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the General Public License (GPL),
 * versions 2. Extra clauses can be found from the file MUSTAJUURI_GPL
 * and further explanations from the file MUSTAJUURI_LICENSE. The
 * license text can be found in file "GPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *  
 */


#include <mj_debug.h>
#include <mj_error.h>
#include <mj_kernel.h>
#include <mj_wave_editor.h>
#include <mj_prefs.h>
#include <mj_root_powers.h>
#include <mj_watch_dog.h>

#include <qawl_view.h>

#include <qapplication.h>

#include <errno.h>

#ifndef WIN32
#include <sys/mman.h>
#endif

int main(int argc, char **argv)
{
  // const char * fname = "ZEB";

  const char * file = 0;

  QApplication * qa = new QApplication(argc, argv);

#ifdef __OS_LINUX__
  MJ_RootPowers::getCapabilities();

  if(!mlockall(MCL_CURRENT | MCL_FUTURE)) {
    // This may succeed on IRIX even without extra privileges

    // We seem to have super-high priorities :-)
    MJ_trace("MLOCKALL SUCCESSFUL, HAS REALTIME CAPABILITIES, "
	     "THIS IS GOOD, CONGRATULATIONS");
    MJ_prefs.setLatencies(15.0, 20.0);
  }
  else {
    errno = 0;
  }
#endif

  MJ_Kernel::setEnv();

  MJ_prefs.scanArgs(argc, argv);

  for(int i = 1; i < argc; i++) {

    if(strcmp(argv[i], "-f") == 0 && (i + 1 < argc)) {
      file = argv[++i];
    }
    if(strcmp(argv[i], "--debug") == 0) {

      if(i + 1 == argc) MJ_Debug::debugType(MJ_DEBUG_ALL);
      else if(argv[i + 1][0] == '-') {
	MJ_Debug::debugType(MJ_DEBUG_ALL);
      }
      else {
	int tmp = 1;
	
	while (++i < argc && tmp) {
	  
	  if(argv[i][0] == '-') { // If this is a new option, break loop.
	    
	    i -= 2;
	    tmp = 0;
	  }
	  else MJ_Debug::debugName(argv[i]);
	}
      }
    }
    else {
      fprintf(stderr, "Unrecognized option \"%s\"\n", argv[i]);
      fflush(0);
    }
  }

  qDebug("ZEB: init");
  
  MJ_WatchDog *wd;

  if(MJ_prefs.noWatchDog())
    wd = 0; 
  else {
    wd = new MJ_WatchDog;
    wd->run(true);
  }
  
  MJ_Kernel *k = new MJ_Kernel;

  MJ_WaveEditor * editor = new MJ_WaveEditor(0);

  if(file)
      editor->setFile(file);
    
  editor->resize(600, 400);

  qa->setMainWidget(editor);
  editor->show();

  MJ_KernelGui *kg = new MJ_KernelGui(k, 0, 0, false);

  qa->exec();

  delete kg;
  // delete editor; This is deleted elsewhere.
  delete k;
  delete qa;

  if(wd){
    wd->requestEnd();
    wd->waitEnd();
    delete wd;
  }

  return 0;
}
