/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of Mustajuuri GPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * Tommi.Ilmonen(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the General Public License (GPL),
 * versions 2. Extra clauses can be found from the file MUSTAJUURI_GPL
 * and further explanations from the file MUSTAJUURI_LICENSE. The
 * license text can be found in file "GPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *  
 */

#ifndef MJ_WAVE_EDITOR_H
#define MJ_WAVE_EDITOR_H

#include <mj_module_dsp_ui.h>

#include <qawl_keyboard_handler.h>

#include <qmainwindow.h> 


class MJ_WavePlayer;

class QAWL_View2;

class QPopupMenu;
class QButton;

class MJ_WaveEditor : public QWidget, // QMainWindow, 
		      public QAWL_KeyboardHandler,
		      public MJ_ModuleDspUi
{
  Q_OBJECT
public:
  MJ_WaveEditor(QWidget * parent, const char * name = 0);
  virtual ~MJ_WaveEditor();


  virtual void viewKeyPressEvent (QAWL_View *, QKeyEvent * );
  virtual void viewKeyReleaseEvent (QAWL_View *, QKeyEvent * e );

  virtual bool close ( bool alsoDelete );

public slots:
  void setFile(const QString &fileName);

  void open();
  void saveAs();
  void saveSelectionAs();
  void quit();

  void record();

  void selectionAvailable(bool);

  void zoomIn();
  void zoomOut();
  void zoomSelection();

  void magnifyIn();
  void magnifyOut();

  void selectAll();
  void cropToSelection();

  void togglePlay();

  void selectOutput(int);
  void selectInput(int);
  void selectFileSampleFormat(int);

  void update();

protected:

  
  enum Tool {
    Record,
    MagnifyIn,
    MagnifyOut,
    ZoomIn,
    ZoomOut,
    ZoomSelection,
    ToolCount
  };

  enum State {
    Inactive,
    Recording,
    Playing,
    StateCount
  };

  void addMruItem(const QString &);
  void redoMruMenu();

  // Override from MJ_RtControl (MJ_ModuleDspUi)
  virtual void realTimeChange(bool nowRt);
  void endRecording();

  // QString     m_tmpFileName;

  class Internal;
  Internal *m_internal;
};

#endif
