/* COPYRIGHT
 *
 * This file is part of Mustajuuri GPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * Tommi.Ilmonen(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the General Public License (GPL),
 * versions 2. Extra clauses can be found from the file MUSTAJUURI_GPL
 * and further explanations from the file MUSTAJUURI_LICENSE. The
 * license text can be found in file "GPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *  
 */

#include <mj_wave_operations.h>

#include <mj_error.h>

#include <di_audio_file.h>

#include <vector>

bool MJ_WaveOperations::copySegment(const QString &oldFile, 
				    const QString &newFile, 
				    long firstFrame, 
				    long lastFrame,
				    Target target,
				    DI_AudioSampleFormat preferFormat)
{
  const char * fname = "MJ_WaveOperations::copySegment";

  DI_AudioFile in, out;

  long framesPerBlock = 10000;
  
  std::vector<int> tmpBuf;

  if(!in.open(oldFile.ascii(), 
	      DI_IO_INPUT, DI_ASF_INT32, framesPerBlock)) {
    MJ_error2(MJ_ERR_ARGUMENT, "%s # Failed to open input file \"%s\"", 
	      fname, oldFile.ascii());
    return false;
  }
  
  uint chans = in.inChannels();

  tmpBuf.resize(chans * framesPerBlock);

  // DI_AudioSampleFormat fileSample;
  DI_AudioSampleFormat userSample;

  if(target == TmpFile) {

    out.setFileFormat(DI_AFF_WAVE);
    out.setFileSampleFormat(DI_ASF_FLOAT32);
    userSample = DI_ASF_FLOAT32;
  }
  else {
    out.setFileFormat(in.getFileFormat());
    /* out.setSampleWidth(in.getSampleWidth());
       out.setSampleFormat(in.getSampleFormat()); */
    
    out.setFileSampleFormat(preferFormat);
    out.setByteOrder(in.getByteOrder());
    userSample = DI_ASF_INT32;
  }

  out.setSampleRate(in.getSampleRate());
  out.setChannels(in.inChannels());

  in.setUserSampleFormat(userSample);

  if(!out.open(newFile.ascii(), DI_IO_OUTPUT, userSample, framesPerBlock)) {
    MJ_error2(MJ_ERR_ARGUMENT, "%s # Failed to open output file \"%s\"", 
	      fname, newFile.ascii());
    return false;
  }

  in.seekFrame(firstFrame);

  long currFrame = firstFrame;
  if(lastFrame < 0 || lastFrame > in.frameCount()) lastFrame = in.frameCount();
  
  
  bool ok = true;

  /* QCursor curs = cursor();

  setCursor(Qt::WaitCursor);*/

  while(currFrame < lastFrame && ok) {

    long needFrames = lastFrame - currFrame;
    long readFrames = needFrames < framesPerBlock ? 
      needFrames : framesPerBlock;

    long r = in.readFrames( & * tmpBuf.begin(), readFrames);
    
    if(r != readFrames) {
      ok = false;
      break;
    }
    
    out.writeFrames( & * tmpBuf.begin(), r);
    currFrame += r;
  }

  // setCursor(curs);
  
  MJ_trace2("%s # Wrote stuff to a file - %ld frames (%s)", fname, currFrame, 
	    ok ? "success" : "failed");

  return ok;
}
