/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of Mustajuuri GPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * Tommi.Ilmonen(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the General Public License (GPL),
 * versions 2. Extra clauses can be found from the file MUSTAJUURI_GPL
 * and further explanations from the file MUSTAJUURI_LICENSE. The
 * license text can be found in file "GPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *  
 */

#ifndef MJ_WAVE_PLAYER_H
#define MJ_WAVE_PLAYER_H

#include <mj_module_dsp_group.h>
#include <mj_ramp.h>

#include <list>

class MJ_ModuleLevel;
class MJ_WaveRecorder;

class MJ_WavePlayer : public MJ_ModuleDspGroup
{
public:
  MJ_WavePlayer();
  virtual ~MJ_WavePlayer();
    
  // virtual bool dspSetTimer(MJ_TimeStamp *t);

  virtual bool dspPrepare();
  virtual void dspProcessPeriod();

  virtual bool msgProcess(const char *id, MJ_Message *);

  bool connectPlayItems();
  
  void setPlayBack(bool playBack) { m_playBack = playBack; }
  void setOutputDevice(int id) { m_audioDevId = id; }

  MJ_WaveRecorder *recorder() { return m_recorder; }

  MJ_ModuleLevel *inLevel() { return m_inLevel; }
  MJ_ModuleLevel *outLevel() { return m_outLevel; }

  class DspItem
  {
  public:
    DspItem() : m_wetDry(1.0), m_module(0) {}

    inline void dspProcessPeriod();
    MJ_ModuleDsp *module() { return m_module; }
    
    void prepare() { m_wetDryRamp.reset(m_wetDry); }

  protected:

    MJ_Ramp<MJ_Sample> m_wetDryRamp;
    /// 1.0 -> fully wet, 0.0 -> fully dry
    MJ_Sample          m_wetDry;
    MJ_ModuleDsp      *m_module;
  };

protected:
  
  
  std::list<DspItem> m_dspItems;

  typedef std::list<DspItem>::iterator dspIterator;
  
  // For recording:
  MJ_WaveRecorder *m_recorder;

  // For playback:
  MJ_ModuleDsp *m_fileReader;
  MJ_ModuleDsp *m_gain; 
  MJ_ModuleDsp *m_audioOut; 

  MJ_ModuleLevel *m_inLevel;
  MJ_ModuleLevel *m_outLevel;

  QString m_fileName;

  int m_audioDevId;
  int m_recordChannels;

  bool m_playBack;

  MJ_TimeStamp m_timer;

  static bool m_debug;
};

#endif
