/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of Mustajuuri GPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * Tommi.Ilmonen(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the General Public License (GPL),
 * versions 2. Extra clauses can be found from the file MUSTAJUURI_GPL
 * and further explanations from the file MUSTAJUURI_LICENSE. The
 * license text can be found in file "GPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *  
 */

#ifndef MJ_WAVE_RECORDER_H
#define MJ_WAVE_RECORDER_H

#include <mj_module_rtaudio.h>

#include <di_audio_port.h>
#include <di_audio_file_threaded.h>
#include <mj_level_meter.h>
#include <mj_math.h>

#include <vector>

/** A plugin for recording material directly to a file. 

    Funny features: Always stores the audio data as 32-bit ints. This
    is to guarantee that no bits are lost.
 */

class MJ_WaveRecorder : public MJ_ModuleRtAudio
{
public:
  MJ_WaveRecorder();
  virtual ~MJ_WaveRecorder();
    
  virtual int  bufferLength();

  virtual bool dspAcceptIoChans(int *, int *);

  virtual bool dspPrepare();
  virtual bool dspPrepareFast();
  virtual void dspProcessPeriod();
  virtual bool dspStop();

  virtual bool msgProcess(const char *id, MJ_Message *);

  void setInputDevice(int id);
  
  void setFileName(const QString &name) { m_fileName = name; }

  void setChannels(int chans) { m_channels = chans; }

  int fileChans();

protected:


  DI_AudioPort         m_input;

  DI_AudioFileThreaded m_file;

  QString              m_device;
  int                  m_deviceId;
  QString              m_fileName;
  int                  m_channels;
  int                  m_fileChannels;

  std::vector<int>     m_buffer;
  std::vector<int>     m_buffer2;

  int m_audioDevId;

  static bool m_debug;
};

#endif
