/* COPYRIGHT
 *
 * This file is part of Mustajuuri GPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the General Public License (GPL),
 * versions 2. Extra clauses can be found from the file MUSTAJUURI_GPL
 * and further explanations from the file MUSTAJUURI_LICENSE. The
 * license text can be found in file "GPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *  
 */

#include <mj_midi_thru.h>


#include <mj_dsp_plugin.h>
#include <mj_error.h>
#include <mj_global.h>
#include <mj_kernel.h>
#include <mj_message_string.h>
#include <mj_module_dsp_row_gui.h>
#include <mj_strs.h>

#include <qapplication.h>

/* This function creates a MIDI thru system. 

   Building new module groups via C++ code is not exactly very
   intelligent (scripting or GUI approach would be superior), but that
   is what is done here...  */

void MJ_MidiThru(QApplication *qa, int argc, char **argv)
{
  
  const char *fname = "MJ_MidiThru";

  MJ_Kernel k;
  
  const char *inDevice=0, *outDevice=0;
  
  
  for(int i=0; i < argc; i++) {
    if(strcmp(argv[i], "--indevice") == 0 && i+1 < argc)
      inDevice = argv[++i];
    if(strcmp(argv[i], "--outdevice") == 0 && i+1 < argc)
      outDevice = argv[++i];
  }
  

  MJ_ModuleDsp *in, *out;
  
  in = MJ_kernel->newFunctionalModule
    (QString(MJ_DspPluginFunctionalityKeys[MJ_DPF_MIDI_IN]));
  out = MJ_kernel->newFunctionalModule
    (QString(MJ_DspPluginFunctionalityKeys[MJ_DPF_MIDI_OUT]));

  if(!in || !out)
    MJ_fatal(MJ_ERR_INTERNAL, fname, "No plugins");

  MJ_MessageString ms;

  if(inDevice) {
    ms.string() = inDevice;
    in->msgProcess(MJ_Strs::MIDIdevice, &ms);
  }
  if(outDevice) {
    ms.string() = outDevice;
    out->msgProcess(MJ_Strs::MIDIdevice, &ms);
  }
  
  MJ_ModuleDspRow dr;
  
  dr.appendModule(in);
  dr.appendModule(out);
  dr.connectModules();
  // out->connectInputMessage(in->id(), 0, 0);
  
  MJ_ModuleDspRow_QG rg(0, &dr, 0,"minisynth test");
  rg.setTestMode(true);
  
  qa->setMainWidget(&rg);
  qa->exec();
}
