/* COPYRIGHT
 *
 * This file is part of Mustajuuri GPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the General Public License (GPL),
 * versions 2. Extra clauses can be found from the file MUSTAJUURI_GPL
 * and further explanations from the file MUSTAJUURI_LICENSE. The
 * license text can be found in file "GPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *  
 */

#ifndef MJ_MIXER_STRIP_QG_H
#define MJ_MIXER_STRIP_QG_H

#include <mj_module_dsp_gui.h>
#include <mj_module_mixer.h>
#include <mj_undo_center.h>

class QVBoxLayout;
class QScrollView;

/** Postcard level-of-detail view for a single mixer strip. */

class MJ_MixerStrip_QG : public MJ_ModuleDspGui
{
  Q_OBJECT
public:
  MJ_MixerStrip_QG(MJ_RtControl *rtparent,
		   MJ_ModuleDsp *module,
		   uint          strip,
		   MJ_ModuleMixer_QG *mixerGui,
		   QWidget      *parent = 0,
		   const char   *name = 0);
  
  virtual ~MJ_MixerStrip_QG();

  void hide();

  virtual void moduleModified4(int,int,int);

  uint strip() const { return m_strip; }

public slots:

  /** Removes a child widget. May leave the mixer strip in rather
      inconsistent state. You should call "update(int,int)" soon after
      calling this one.*/
  void remove(int,int);

  /** Updates the GUI (builds new widgets as necessary). */
  void update(int,int);

  /** Deletes this object. */
  void kill();


protected:
  void resizeEvent(QResizeEvent *);

  void limitSize(int rows);
  void expandTo(int rows);

  /** Fill a slot, the slot must not be filled before this is
      called. */
  void fillSlot(uint rowInd);

  /** Builds the user interface from scratch. */

  void draw();

  /** Rebuilds the layout. The old layout is deleted and new one is
      created in its place. Nothing is done to the child widgets. */
  void redoLayout();

  void determineSizes();

  uint            m_strip;

  MJ_ModuleMixer    *m_mixer;
  MJ_ModuleMixer_QG *m_mixerGui;

  QList<QWidget>  m_children;
  
  QVBoxLayout    *m_layout1, *m_layout2;

  QScrollView    *m_scroll;
  QWidget        *m_base;

  MJ_UndoCenter    m_undoControl;

  static int m_debug;
};

#endif
