/* COPYRIGHT
 *
 * This file is part of Mustajuuri GPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the General Public License (GPL),
 * versions 2. Extra clauses can be found from the file MUSTAJUURI_GPL
 * and further explanations from the file MUSTAJUURI_LICENSE. The
 * license text can be found in file "GPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *  
 */

#include <mj_module_bypass.h>

MJ_ModuleBypass::MJ_ModuleBypass()
{
  m_signalSourceDefs.resize(1);
  m_signalOutput.resize(1);
}

MJ_ModuleBypass::~MJ_ModuleBypass()
{}

void MJ_ModuleBypass::dspProcessPeriod()
{
  if(!m_signalOutput.size() || !m_signalInput.size())
    ;
  else if(m_signalOutput.size() == m_signalInput.size())
    bypassNN();
  else if (m_signalOutput.size() > m_signalInput.size()) 
    bypassSB();
  else
    bypassBS();
}

MJ_ModuleDsp *MJ_ModuleBypass::createNew()
{
  return new MJ_ModuleBypass;
}

void MJ_ModuleBypass::bypassNN()
{
  // trace("MJ_ModuleBypass::bypassNN");
  for(uint i=0; i < m_signalInput.size(); i++) {
    m_signalOutput.getFast(i).samples().copyFast
      (m_signalInput.getFast(i).samples());
  }
}

void MJ_ModuleBypass::bypassSB()
{
  for(uint i=0; i < m_signalOutput.size(); i++) {
    m_signalOutput[i].samples().copyFast
      (m_signalInput.getFast(i & m_signalInput.size()).samples());
  }
}

void MJ_ModuleBypass::bypassBS()
{
  uint i;

  for(i=0; i < m_signalOutput.size(); i++) {
    m_signalOutput.getFast(i).samples().copyFast
      (m_signalInput.getFast(i & m_signalInput.size()).samples());
  }
  
  MJ_Vector<MJ_Sample>::vmlBegin();
  for(; i < m_signalInput.size(); i++) {
    m_signalOutput.getFast(i % m_signalOutput.size()).samples().accVml
      (m_signalInput.getFast(i).samples());
  }
  MJ_Vector<MJ_Sample>::vmlEnd();
}
