/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of Mustajuuri GPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the General Public License (GPL),
 * versions 2. Extra clauses can be found from the file MUSTAJUURI_GPL
 * and further explanations from the file MUSTAJUURI_LICENSE. The
 * license text can be found in file "GPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *  
 */

#ifndef MJ_MODULE_COMPRESSOR_H
#define MJ_MODULE_COMPRESSOR_H

#include <mj_module_dsp.h>

#include <mj_filter_ap1.h>

/** Base class for modules that implement some kind of compression
    effect (compressors and peak limiters). 
    
    The proper way to use this class is to manipulate the
    m_compressGains -variable so that current per-channel compression
    gains (linear gain factor) are in the vector.
*/

class MJ_CompressorBase 
{
public:
  virtual ~MJ_CompressorBase() {}

  MJ_Vector<MJ_Sample> &compressGains() { return m_compressGains; }

protected:
  
  MJ_Vector<MJ_Sample>                m_compressGains;
};

/// Parameters for compressor
enum MJ_ParamsCompressor {
  MJ_PARCOMP_TRESHOLD,
  MJ_PARCOMP_RATIO,
  MJ_PARCOMP_ATTACK,
  MJ_PARCOMP_SIZEOF
};

extern const char * MJ_ParamsCompressorKeys[MJ_PARCOMP_SIZEOF];

/** Compressor module. 
    
    Not too efficient (computationally or acoustically). 

    Knows both independent multi-channel compression and synchronized
    multi-channel compression.  */

class MJ_ModuleCompressor : public MJ_ModuleDsp, public MJ_CompressorBase
{
public:
  MJ_ModuleCompressor();
  virtual ~MJ_ModuleCompressor();
  
  virtual bool dspPrepare();
  virtual void dspProcessPeriod();
  virtual bool dspStop();

  virtual bool msgProcess(const char *, MJ_Message *);
  virtual MJ_Parameter * msgGetInput(const char *);
  virtual QCString msgGetInputName(int);
  
protected:
  
  void processSync();

  void processNonSync();
  
  void redesignFilters();

  MJ_Array<MJ_FilterAP1<MJ_Sample> > m_levelFilters;

  MJ_Sample m_threshold, m_ratio, m_attack, m_diffMul;

  bool m_syncChannels;

  MJ_Vector<MJ_Sample>               m_lastSamples;
  MJ_Vector<MJ_Sample>               m_compressVect;
};


#endif
