/* COPYRIGHT
 *
 * This file is part of Mustajuuri GPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the General Public License (GPL),
 * versions 2. Extra clauses can be found from the file MUSTAJUURI_GPL
 * and further explanations from the file MUSTAJUURI_LICENSE. The
 * license text can be found in file "GPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *  
 */

#include <mj_module_compressor_gui.h>

#include <mj_display_type.h>
#include <mj_error.h>
#include <mj_level_display.h>
#include <mj_math.h>
#include <mj_module_compressor.h>

#include <qlayout.h> 
#include <stdio.h> 

MJ_ModuleCompressorGui::MJ_ModuleCompressorGui(MJ_RtControl *rtparent,
					       MJ_ModuleDsp *module,
					       QWidget      *parent,
					       const char   *name)
  : MJ_ModuleDspGuiPostCard(rtparent, module, true,
			    (uint) -1, 
			    parent ? MJ_GUI_STAMP : MJ_GUI_OWN_WINDOW,
			    parent, name)
{
  const char * fname = "MJ_ModuleCompressorGui::MJ_ModuleCompressorGui";

  connect(&m_timer, SIGNAL(timeout()), this, SLOT(update()));

  QVBoxLayout *mainLayout = dynamic_cast<QVBoxLayout *> 
    (child("mainlayout"));
  
  MJ_cpf(mainLayout, fname);

  m_ldpy = new MJ_LevelDisplay(this);
  m_ldpy->setMinimumSize(50, 50);
  mainLayout->addWidget(m_ldpy, 80);
  m_ldpy->show();
  realTimeChange(isRealTime());
}

MJ_ModuleCompressorGui::~MJ_ModuleCompressorGui()
{}

MJ_ModuleDspGui *MJ_ModuleCompressorGui::createNew(MJ_RtControl *rtparent,
						   MJ_ModuleDsp *module,
						   QWidget *parent, 
						   const char *name)
{
  return new MJ_ModuleCompressorGui(rtparent, module, parent, name);
}

void MJ_ModuleCompressorGui::update()
{
  MJ_ModuleDspGuiPostCard::update();

  MJ_CompressorBase *cmp = dynamic_cast<MJ_CompressorBase *> (m_module);
  
  MJ_cpf(cmp, "MJ_ModuleCompressorGui::update");

  lock();
  uint gains = cmp->compressGains().size();
  unlock();
  
  if(gains != m_gains.size()) {
    m_gains.resize(gains);
    /* char buf[128];
    sprintf(buf, "MJ_ModuleCompressorGui::update %d != %d", 
	    gains, m_gains.size());
	    MJ_fatal(MJ_ERR_INTERNAL, buf);*/
  }
  
  lock();
  for(uint i=0; i < gains; i++) {
    float tmp = cmp->compressGains().get(i);
    m_gains.get(i) = MJ_absToDb(tmp) / -20.0;
  }
  unlock();


  if(gains) {
    m_ldpy->setNofBars(gains);
    m_ldpy->set(m_gains.data(), 0);
  }
}

void MJ_ModuleCompressorGui::realTimeChange(bool nowRt)
{
  int chans = m_module->dspInChannels();

  m_ldpy->setNofBars(chans);

  m_gains.resize(chans);

  int updateInterval = MJ_DisplayType::localDisplay() ? 30 : 60;
    
  if(nowRt) m_timer.start(updateInterval);
  else m_timer.stop();
}

