/* COPYRIGHT
 *
 * This file is part of Mustajuuri GPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the General Public License (GPL),
 * versions 2. Extra clauses can be found from the file MUSTAJUURI_GPL
 * and further explanations from the file MUSTAJUURI_LICENSE. The
 * license text can be found in file "GPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *  
 */

#ifndef MJ_MODULE_DELAY_H
#define MJ_MODULE_DELAY_H

#include <mj_module_dsp.h>
#include <mj_ringbuffer.h>

/** Simple multichannel delay module. */
class MJ_ModuleDelay : public MJ_ModuleDsp
{
public:

  MJ_ModuleDelay();
  virtual ~MJ_ModuleDelay();

  virtual bool dspPrepare();
  virtual void dspProcessPeriod();
  virtual bool dspStop();

  virtual bool msgProcess(const char *, MJ_Message *);
  virtual MJ_Parameter * msgGetInput(const char *);
  virtual QCString msgGetInputName(int);

  float maxTimeMs() { return m_maxDelayMs; }
  
protected:

  void reset() 
  {
    m_timeSamples = m_targetTimeSamples; 
    m_wetRatio = m_targetWetRatio;
    m_dryRatio = m_targetDryRatio;
    m_feedBack = m_targetFeedBack;
  }

  void cleanMemory()
  {
    for(uint i=0; i < m_delayBuffers.size(); i++) 
      m_delayBuffers.get(i).resize(0);
  }

  MJ_Array<MJ_RingBuffer2<float> > m_delayBuffers;
  
  uint                  m_timeSamples; // Milliseconds

  double                m_wetRatio;
  double                m_dryRatio;
  double                m_feedBack;
  double                m_newWeight;

  uint              m_targetTimeSamples;
  float             m_targetTime; 
  float             m_targetWetRatio;
  float             m_targetDryRatio;
  float             m_targetFeedBack;
  
  float             m_maxDelayMs;

  ulong             m_interpSamples;
};

#endif
