/* COPYRIGHT
 *
 * This file is part of Mustajuuri GPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the General Public License (GPL),
 * versions 2. Extra clauses can be found from the file MUSTAJUURI_GPL
 * and further explanations from the file MUSTAJUURI_LICENSE. The
 * license text can be found in file "GPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *  
 */

#include <mj_error.h>
#include <mj_module_dsp_group.h>
#include <mj_prefs.h>
#include <mj_rt_driver.h>
#include <mj_stream_utils.h>

#include <qbuffer.h> 
#include <qfile.h> 
#include <qtextstream.h> 

bool MJ_ModuleDspGroup::m_groupDebug = false;

MJ_ModuleDspGroup::MJ_ModuleDspGroup()
  : m_myTimer(true)
{
  if(m_groupDebug) trace("MJ_ModuleDspGroup::MJ_ModuleDspGroup");

  m_currentTime = new MJ_TimeStamp(MJ_prefs.frameSize());
}

MJ_ModuleDspGroup::~MJ_ModuleDspGroup()
{
  if(m_groupDebug) trace("MJ_ModuleDspGroup::~MJ_ModuleDspGroup");
  clearTimer();
}

bool MJ_ModuleDspGroup::dspSetTimer(MJ_TimeStamp *t)
{ 
  clearTimer();

  if(!t) {
    t = new MJ_TimeStamp;
    m_myTimer = true;
  }
  else
    m_myTimer = false;

  m_currentTime = t; 

  return MJ_ModuleDspArray::dspSetTimer(t);
}

/* bool MJ_ModuleDspGroup::dspPrepare()
{
  if()
  }*/

bool MJ_ModuleDspGroup::writeParameters(QTextStream *s)
{
  (*s) << MJ_ModuleKeyWords[MJ_MKW_SIZE] 
       << " " << m_array.size() << "\n";

  // Write modules:

  MJ_ModuleDsp *m;

  for(uint i=0; i < m_array.size(); i++) {

    m = m_array.get(i);

    // Grab the data to a buffer.
    QBuffer b;
    b.open(IO_ReadWrite);
    QTextStream stmp(&b);
    m->write(&stmp);
    // Write size of the data block:
    (*s) << b.at() << "\n";  
    // Actually write the data to the stream.
    QByteArray ba = b.buffer();
    s->writeRawBytes(ba, b.at());
    
  }

  return true;
}

bool MJ_ModuleDspGroup::readParameters(QTextStream *s)
{
  const char *fname ="MJ_ModuleDspGroup::readParameters";

  bool ok = true;

  QIODevice *device = s->device();

  QCString tmp;
  QString str;
  (*s) >> str;

  if(str != MJ_ModuleKeyWords[MJ_MKW_SIZE]) 
    return false;

  str = s->readLine();
  uint modules = (uint) str.toInt(&ok);  
  if(!ok) return false;

  trace("MODULES", (int)modules);

  int p;

  for(uint i=0; i < modules; i++) {

    str = s->readLine();
    int blockLen = (uint) str.toInt(&ok);  
    p = (int) device->at();

    if(!ok) {
      tmp.resize(str.length() + 128);
      tmp.sprintf("%s # Around line %d \"%s\"", 
		  fname, MJ_currentLine(device), (const char *) str);
      
      error(MJ_ERR_ARGUMENT, tmp);
      return false;
    }

    MJ_ModuleDsp *m = MJ_ModuleDsp::readFromStream(s);
    if(m) appendModule(m);
    else {
      MJ_error(MJ_ERR_ARGUMENT, 
	       "MJ_ModuleDspGroup::readParameters - Unidentified plugin type");
      ok = false;
    }
    
    if(m_groupDebug) {
      trace("MJ_ModuleDspGroup::readParameters NEW MODULE done", i);
    }
    // Make sure we are in good position for the next module.
    // device->at(p + blockLen);  
    // s->setDevice(device);
  }
  return ok;
}


void MJ_ModuleDspGroup::runShort(int rounds)
{
  MJ_RtDriver d(this);
  if(!d.runPeriods(rounds))
    error(MJ_ERR_INTERNAL, "MJ_ModuleDspGroup::runShort");
}

void MJ_ModuleDspGroup::clearTimer()
{
  if(m_myTimer && m_currentTime)
    delete m_currentTime;
  m_currentTime = 0;
}

