/* COPYRIGHT
 *
 * This file is part of Mustajuuri GPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the General Public License (GPL),
 * versions 2. Extra clauses can be found from the file MUSTAJUURI_GPL
 * and further explanations from the file MUSTAJUURI_LICENSE. The
 * license text can be found in file "GPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *  
 */

#include <mj_module_dsp_row_gui.h>

#include <mj_error.h>
#include <mj_global.h>
#include <mj_kernel.h>
#include <mj_rt_driver.h>
#include <mj_rt_thread.h>
#include <mj_tr.h>

#include <qapplication.h>
#include <qbuffer.h> 
#include <qfile.h> 
#include <qfiledialog.h>
#include <qlayout.h> 
#include <qmenubar.h> 
#include <qpopupmenu.h>
#include <qtextstream.h> 


#include <stdio.h>

static MJ_RtThread *s_thread = 0;
static MJ_RtDriver *s_driver = 0;
static MJ_Kernel   *s_kernel = 0;

///
MJ_ModuleDspRow_QG::MJ_ModuleDspRow_QG(MJ_RtControl *rtparent,
				       MJ_ModuleDsp *module,
				       QWidget      *parent,
				       const char   *name)
  : MJ_ModuleDspGui(rtparent, module, parent, name),
    m_layout(0),
    m_dspRow(0),
    m_testMode(true)
{
  MJ_ModuleDspRow *mr = dynamic_cast<MJ_ModuleDspRow *> (module);
  
  if(!mr) {
    MJ_fatal(MJ_ERR_ARGUMENT, " MJ_ModuleDspRow_QG::MJ_ModuleDspRow_QG");
    return;
  }
  m_dspRow = mr;

  draw();
}

MJ_ModuleDspRow_QG::~MJ_ModuleDspRow_QG()
{}


void MJ_ModuleDspRow_QG::test(QApplication *xpQA, int iArgc, char **cppArgv)
{
  const char *fname = "MJ_ModuleDspRow_QG::test";

  MJ_Kernel k;

  int testNum = 0;
  QString tmp;
  
  const char *options[] = { "--test" };
  
  for(int i=1; i < iArgc; i++) {
    int option = MJ_findStr(cppArgv[i], options, 1);

    switch(option) {
    case -1:
      break;
      
    case 0:
      if(i+1 < iArgc) {
	tmp = cppArgv[++i];
	testNum = tmp.toInt();
	
      }
    default:
      break;
    }
  }
  
  MJ_ModuleDspRow m;

  MJ_trace("MJ_ModuleDspRow_QG::test # testNum = ", testNum);


  if(testNum == 0) m.testRow(1);
  else if(testNum == 1) m.playbackRow();
  else MJ_fatal(MJ_ERR_ARGUMENT, fname, "Bad test number");

  MJ_ModuleDspRow_QG mg(0, &m, 0,"row gui test");
  mg.m_testMode = true;
  
  xpQA->setMainWidget(&mg);
  xpQA->exec();
}

MJ_ModuleDspGui *MJ_ModuleDspRow_QG::createNew(MJ_RtControl *rtparent,
					       MJ_ModuleDsp *module,
					       QWidget *parent, 
					       const char *name)
{
  return new MJ_ModuleDspRow_QG(rtparent, module, parent, name);
}

void MJ_ModuleDspRow_QG::runShort()
{
  if(s_thread) {
    error(MJ_ERR_INTERNAL, "MJ_ModuleDspRow_QG::runShort", "Already active");
    return;
  }
  m_dspRow->runShort(20000);
}

void MJ_ModuleDspRow_QG::runShort2()
{
  if(s_thread) {
    error(MJ_ERR_INTERNAL, "MJ_ModuleDspRow_QG::runShort", "Already active");
    return;
  }
  if(MJ_kernel) {
     MJ_kernel->playModule(this, MJ_KERNEL_NOTHREAD_PLAY);
     return;
  }

  if(!s_kernel) s_kernel = new MJ_Kernel;

  s_kernel->playModule(this, MJ_KERNEL_NOTHREAD_PLAY);
}

void MJ_ModuleDspRow_QG::stopShort2()
{
  if(MJ_kernel) {
    if(!MJ_kernel->stopPlaying())
      error(MJ_ERR_INTERNAL, "MJ_ModuleDspRow_QG::dspStopShort2 0");
  }
  else if(s_kernel) {
    if(!s_kernel->stopPlaying())
      error(MJ_ERR_INTERNAL, "MJ_ModuleDspRow_QG::dspStopShort2 1");
  }
  else error(MJ_ERR_INTERNAL, "MJ_ModuleDspRow_QG::dspStopShort2 2");
}

void MJ_ModuleDspRow_QG::writeCout()
{
  m_dspRow->toCout();
}

void MJ_ModuleDspRow_QG::saveAs()
{ 
  QString f = QFileDialog::getSaveFileName( 0, "*.mj", this );

  if(f.isEmpty()) return;

  saveModule(f);
}

void MJ_ModuleDspRow_QG::startThreaded()
{
  const char *fname = "MJ_ModuleDspRow_QG::startThreaded";

  trace(fname);

  if(s_thread) {
    error(MJ_ERR_INTERNAL, fname, "Already active");
    return;
  }
  if(!s_driver)
    s_driver = new MJ_RtDriver;
  s_driver->setModule(m_module);

  s_thread = new MJ_RtThread;
  s_thread->setDriver(s_driver);

  rtStatusChange(MJ_RT_THREADED_RT, &s_thread->moduleAccesMutex());
  
  s_thread->run();

  if(!s_thread->start()) {
    MJ_fatal(MJ_ERR_UNKNOWN, fname, "start failed");
  }
  MJ_trace(fname, " thread started");
}

void MJ_ModuleDspRow_QG::stopThreaded()
{
  const char *fname = "MJ_ModuleDspRow_QG::dspStopThreaded";
  trace(fname);

  if(!s_thread) {
    error(MJ_ERR_INTERNAL, fname, "NOT active");
    return;
  }
  // cout << fname << " trying to stop" << endl;

  if(!s_thread->stop()) {
    MJ_fatal(MJ_ERR_UNKNOWN, fname, "stop failed");
  }
  // cout << fname << " s_thread->requestEnd" << endl;
  s_thread->requestEnd();
  // cout << fname << " s_thread->waitEnd" << endl;
  s_thread->waitEnd();
  // cout << fname << " delete s_thread" << endl;
  rtStatusChange(MJ_RT_NOT, 0);

  s_driver->setModule(0);

  delete s_thread;
  s_thread = 0;
  delete s_driver;
  s_driver = 0;
}


void MJ_ModuleDspRow_QG::newModule(const QString &s)
{
  MJ_ModuleDsp * m = MJ_Kernel::newDspModule(s, MJ_PP_MENU_POSITION);
  
  m_dspRow->appendModule(m);
  append(m);
  m_layout->activate();
  m_dspRow->connectModules();
}

void MJ_ModuleDspRow_QG::draw()
{
  m_layout = new QVBoxLayout(this, 5);

  QPopupMenu *tests = new QPopupMenu(this);

  if(m_testMode) {
    tests->insertItem( MJ_Tr::tr("Run short"), this, SLOT(runShort()));
    tests->insertItem( MJ_Tr::tr("&Start non-threaded"), 
		       this, SLOT(runShort2()));
    tests->insertItem( MJ_Tr::tr("&Stop non-threaded"), 
		       this, SLOT(stopShort2()));

    tests->insertItem( MJ_Tr::tr("Start threaded"), 
		       this, SLOT(startThreaded()));
    tests->insertItem( MJ_Tr::tr("Stop threaded"), 
		       this, SLOT(stopThreaded()));
  }

  tests->insertItem( MJ_Tr::tr("Write"), this, SLOT(writeCout()));
  tests->insertItem( MJ_Tr::tr("Save as..."), this, SLOT(saveAs()));

  if(m_testMode) 
    tests->insertItem( MJ_Tr::tr("&Quit"), qApp, SLOT(quit()));

  m_edit = new QPopupMenu(this);

  m_undoControl.setMenu(m_edit);

  if(MJ_kernel)
    MJ_KernelGui::moduleMenu(this, SLOT(newModule(const QString &)), m_edit);

  QMenuBar *menu = new QMenuBar( this );

  if(m_testMode)
    menu->insertItem( MJ_Tr::tr("&Tests"), tests );
  else
    menu->insertItem( MJ_Tr::tr("&File"), tests );

  menu->insertItem( MJ_Tr::tr("&Edit"), m_edit );

  m_layout->setMenuBar(menu);

  MJ_Array<MJ_ModuleDsp *> *modules = &m_dspRow->moduleArray();

  for(uint i=0; i < modules->size(); i++) {
    
    // cout << "New round v1 " << i << endl;

    lock();
    MJ_ModuleDsp *m = modules->get(i);
    unlock();

    append(m);
  }
  m_layout->activate();
  if(!parent()) show();
}

void MJ_ModuleDspRow_QG::append(MJ_ModuleDsp *m)
{
  MJ_ModuleDspGui *tmp = m->newGui(this, MJ_GUI_POSTCARD, this);

  if(!tmp) MJ_fatal(MJ_ERR_INTERNAL, "MJ_ModuleDspRow_QG::draw");

  m_undoControl.addSatellite(m->satellite());
    
  tmp->setFrameStyle( QFrame::Box | QFrame::Raised );
  tmp->setLineWidth( 1 );

  m_layout->addWidget(tmp);
  tmp->show();
}
