/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of Mustajuuri GPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the General Public License (GPL),
 * versions 2. Extra clauses can be found from the file MUSTAJUURI_GPL
 * and further explanations from the file MUSTAJUURI_LICENSE. The
 * license text can be found in file "GPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *  
 */

#ifndef MJ_MODULE_GAIN_H
#define MJ_MODULE_GAIN_H

// Include necessary definitions:
#include <mj_module_dsp.h>

/** This plugin controls signal volume. It simply multiplies signal
    values with given coefficient.
   
 */
class MJ_ModuleGain : public MJ_ModuleDsp
{
public:
  MJ_ModuleGain();
  virtual ~MJ_ModuleGain();

  /* This method is called by the host module before any signal
     processing takes place. Initializations can be performed here. */
  virtual bool dspPrepare();
  /* This method does the actual processing. The parent module calls this
     method repeatedly. Any practical plugin should redefine this method. */
  virtual void dspProcessPeriod();
  
  /* This method is used to receive input messages (or events).  The
     various child classes of MJ_Message make it possible to pass
     virtually any information via this interface. */
  virtual bool msgProcess(const char *, MJ_Message *);
  /* The types of input messages are declared here. It is hoped all
     modules would tell all of their input message types in this
     method.  */
  virtual MJ_Parameter * msgGetInput(const char *);

  virtual QCString msgGetInputName(int);

  void setGain(MJ_Sample gain) { m_gain = gain; }

  /** Tests stream IO. This is a utility test method. */
  static void testStreamIo();

  static bool *debugPtr() { return & m_debug; }

protected:
  MJ_Sample m_gain;

  long      m_targetTime;
  MJ_Sample m_targetGain;

  MJ_Vector<MJ_Sample> m_ramp;

  static bool m_debug;
};

#endif
