/* COPYRIGHT
 *
 * This file is part of Mustajuuri GPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the General Public License (GPL),
 * versions 2. Extra clauses can be found from the file MUSTAJUURI_GPL
 * and further explanations from the file MUSTAJUURI_LICENSE. The
 * license text can be found in file "GPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *  
 */


#include <mj_module_impulse.h>
#include <mj_tr.h>

#include <qlayout.h> 
#include <qpushbutton.h> 

MJ_ModuleImpulse::MJ_ModuleImpulse()
  : m_impulseTime(-1)
{
  dspSetOutChannels(1);
}

MJ_ModuleImpulse::~MJ_ModuleImpulse()
{}
  
bool MJ_ModuleImpulse::dspPrepare()
{
  m_impulseTime = -1;
  
  return m_signalOutput.size() == 1;
}

void MJ_ModuleImpulse::dspProcessPeriod()
{
  MJ_Vector<MJ_Sample> *out = & m_signalOutput.getFast(0).samples();

  if(m_impulseTime == -1 ||
     (m_impulseTime - (long) m_currentTime->stamp()) >= (long) out->size()) 
    out->setAll(0.0);
  else if(m_impulseTime == 0) {
    out->setAll(0.0);
    out->get(0) = 1.0;
    m_impulseTime = -1;
  }
  else {
    out->setAll(0.0);
    out->get(m_impulseTime - (long) m_currentTime->stamp()) = 1.0;
    m_impulseTime = -1;
  }
}

void MJ_ModuleImpulse::impulse(long sample)
{
  m_impulseTime = sample;

  if(m_currentTime) 
    if(m_currentTime->isRealTime() && sample < (long) m_currentTime->stamp())
      m_impulseTime = 0;
}

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

MJ_ModuleImpulse_QG::MJ_ModuleImpulse_QG(MJ_RtControl *rtparent,
					 MJ_ModuleDsp *module,
					 QWidget      *parent,
					 const char   *name)
  : MJ_ModuleDspGui(rtparent, module, parent, name)
{
  draw();
}

MJ_ModuleImpulse_QG::~MJ_ModuleImpulse_QG()
{}

MJ_ModuleDspGui *MJ_ModuleImpulse_QG::createNew(MJ_RtControl *rtparent,
						MJ_ModuleDsp *module,
						QWidget *parent, 
						const char *name)
{
  return new MJ_ModuleImpulse_QG(rtparent, module, parent, name);
}

void MJ_ModuleImpulse_QG::impulse()
{
  ((MJ_ModuleImpulse *) m_module)->impulse();
}

void MJ_ModuleImpulse_QG::draw()
{
  QGridLayout * grid = new QGridLayout(this, 3, 3);
  
  QPushButton *pb = new QPushButton(MJ_Tr::tr("Impulse"), 
				    this, "impulsebutton");
  if(!pb) return;
  
  connect(pb, SIGNAL(pressed()), this, SLOT(impulse()));

  grid->addWidget(pb, 1, 1);
  pb->show();
}
