/* COPYRIGHT
 *
 * This file is part of Mustajuuri GPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the General Public License (GPL),
 * versions 2. Extra clauses can be found from the file MUSTAJUURI_GPL
 * and further explanations from the file MUSTAJUURI_LICENSE. The
 * license text can be found in file "GPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *  
 */

#include <mj_module_impulsetrain.h>

#include <mj_parameters.h>
#include <mj_strs.h>

MJ_ModuleImpulseTrain::MJ_ModuleImpulseTrain()
  : m_interval(99),
    m_counter(1),
    m_amplitude(0.25)
{
  m_signalOutput.resize(1);
}

MJ_ModuleImpulseTrain::~MJ_ModuleImpulseTrain()
{}

bool MJ_ModuleImpulseTrain::dspPrepare()
{
  if(m_signalOutput.size() != 1) return false;

  m_counter = 1;  
  return true;
}

void MJ_ModuleImpulseTrain::dspProcessPeriod()
{
  MJ_Vector<MJ_Sample> *out = & m_signalOutput[0].samples();

  ulong counter = m_counter;
  const MJ_Sample zero = 0.0;
  for(uint i = 0; i < out->size(); i++) {
    counter--;

    if(!counter) {
      counter = m_interval;
      out->getFast(i) = m_amplitude;
    }
    else 
      out->getFast(i) = zero;
  }
  m_counter = counter;
}

void MJ_ModuleImpulseTrain::setHz(double freq)
{
  ulong interval = 
    (ulong) ((m_signalOutput[0].sampleRateConst().rate()) / freq);
  setInterval(interval);
}


bool MJ_ModuleImpulseTrain::msgProcess(const char *id, MJ_Message *m)
{
  bool ok = true;
    
  if(strcmp(id, MJ_Strs::interval) == 0) {
    int i = m->toInt(&ok);
    if(!ok || i <= 0) return false;

    setInterval(i);

    return true;
  }
  else if(strcmp(id, MJ_Strs::amplitude) == 0) {
    double v = m->toDouble(&ok);
    if(!ok || v < 0.0) return false;

    m_amplitude = v;

    return true;
  }
  return false;
}

MJ_Parameter * MJ_ModuleImpulseTrain::msgGetInput(const char *id)
{
  if(strcmp(id, MJ_Strs::interval) == 0) 
    return new MJ_ParameterInt(MJ_Strs::Interval, MJ_Strs::interval,
			       (int) m_interval, 1, 10000, MJ_UNIT_SAMPLES);
  else if(strcmp(id, MJ_Strs::amplitude) == 0) 
    return new MJ_ParameterDouble(MJ_Strs::Amplitude, MJ_Strs::amplitude,
				  m_amplitude, 0.0, 1.0);
  return 0;
}

QCString MJ_ModuleImpulseTrain::msgGetInputName(int i)
{
  QCString s;
  if(i == 0) s = MJ_Strs::interval;
  else if(i == 1) s = MJ_Strs::amplitude;
  return s;  
}

