/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of Mustajuuri GPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the General Public License (GPL),
 * versions 2. Extra clauses can be found from the file MUSTAJUURI_GPL
 * and further explanations from the file MUSTAJUURI_LICENSE. The
 * license text can be found in file "GPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *  
 */

#ifndef MJ_MODULE_LEVEL_H
#define MJ_MODULE_LEVEL_H

#include <mj_level_display.h>
#include <mj_level_meter.h>
#include <mj_module_dsp.h>
#include <mj_module_dsp_gui.h>
#include <mj_filter_ap1.h>

#include <qtimer.h> 

class QHBoxLayout;
class QVBoxLayout;
class QLabel;
class QPushButton;


/** Signal level measurement plugin. */ 

class MJ_ModuleLevel : public MJ_ModuleDsp
{
public:
  MJ_ModuleLevel();
  virtual ~MJ_ModuleLevel();
  
  virtual bool dspPrepare();
  virtual void dspProcessPeriod();
  virtual bool dspStop();

  virtual MJ_ModuleDspGui *newGui
  (MJ_RtControl *rtparent, MJ_GuiSize s, QWidget *parent, const char *name);

  class Level
  {
  public:

    Level() { reset(); }
    
    void reset() { m_peak = 0.0; m_levelLong.reset(); m_levelShort.reset(); }

    inline void put(MJ_Sample v, uint shortHold, uint longHold)
    {
      v = MJ_abs(v);

      m_levelLong.put(v, longHold);
      m_levelShort.put(v, shortHold);

      if(m_peak < v) m_peak = v;
    }

    MJ_LevelMeter<MJ_Sample> m_levelLong;
    MJ_LevelMeter<MJ_Sample> m_levelShort;
    float                    m_peak;
  };


  inline MJ_Array<Level> & levels() { return m_levels; }


  MJ_Sample peak() const 
  { 
    MJ_Sample p = 0.0;
    
    for(uint i=0; i < m_levels.size(); i++) {
      float tmp = m_levels.getFastConst(i).m_peak;
      if(p < tmp) p = tmp;
    }
    return p; 
  }
  
  void resetPeak() 
  { 
    for(uint i=0; i < m_levels.size(); i++)
      m_levels[i].m_peak = 0.0;
  }
  
protected:


  void zeroLevels();
  
  
  MJ_Array<Level>        m_levels;

  // MJ_Array<MJ_LevelMeter<MJ_Sample> > m_levels;

  // MJ_Sample               m_peak;
  /// Hold time in millisecs
  MJ_Sample               m_timeMs;
  MJ_Sample               m_peakHoldMs;
  uint                    m_timeSamples;
  uint                    m_peakHoldSamples;
  
};


/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////


/// GUI for MJ_ModuleLevel
class MJ_ModuleLevel_QG : public MJ_ModuleDspGui
{
  Q_OBJECT

public:
  MJ_ModuleLevel_QG(MJ_RtControl *rtparent,
		    MJ_GuiSize    size,
		    MJ_ModuleDsp *module,
		    QWidget      *parent = 0,
		    const char   *name = 0);
  virtual ~MJ_ModuleLevel_QG();

  virtual int weight();

  static void setDebug(bool b) { m_debug = b; }

public slots:
  virtual void update();

  void resetPeak();
protected:

  class Level
  {
  public:
    Level() { reset(); }

    void reset(MJ_Sample level = 0.0) 
    { m_peak = m_level = m_currentPeak = level; }

    void copyPeaks(const MJ_ModuleLevel::Level &l)
    { 
      m_peak = l.m_peak;
      m_level = l.m_levelLong.peak();
      m_currentPeak = l.m_levelShort.peak();
    }

    MJ_Sample m_peak;
    MJ_Sample m_level;
    MJ_Sample m_currentPeak;
  };

  virtual void realTimeChange (bool nowRt); 

  void resizeDisplays(uint);
  void draw();

  QTimer                    m_timer;

  
  MJ_GuiSize                m_size;
  MJ_LevelDisplay          *m_ldpy;

  // QHBoxLayout              *m_box;
  // QVBoxLayout              *m_vbox;
  QLabel                   *m_peak;
  QPushButton              *m_reset;

  QArray<MJ_LevelDisplay *> m_displays;
  MJ_Array<Level>           m_levels;
  MJ_Array<float>           m_levelDb;

  static bool               m_debug;
};


#endif
