/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of Mustajuuri GPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the General Public License (GPL),
 * versions 2. Extra clauses can be found from the file MUSTAJUURI_GPL
 * and further explanations from the file MUSTAJUURI_LICENSE. The
 * license text can be found in file "GPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *  
 */

#ifndef MJ_MODULE_MIDISEQ_H
#define MJ_MODULE_MIDISEQ_H

#include <mj_messages.h>
#include <mj_midi_sequence.h>
#include <mj_module_dsp.h>
#include <mj_module_dsp_gui.h>
#include <mj_satellite.h>

#define MJ_MODULE_MSEQ_OPEN_FILE 0

class QApplication;

/** Prints any input signals to the screen. */
class MJ_ModuleMidiseq : public MJ_ModuleDsp
{
public:

  MJ_ModuleMidiseq();
  virtual ~MJ_ModuleMidiseq();

  virtual bool dspPrepare();
  virtual void dspProcessPeriod();

  virtual bool msgProcess(const char *, MJ_Message *);
  virtual MJ_Parameter *msgGetInput(const char *);
  virtual QCString msgGetInputName(int);

  virtual MJ_Parameter *msgGetOutput(const char *);
  virtual QCString msgGetOutputName(int);

  virtual MJ_ModuleDspGui *newGui
  (MJ_RtControl *rtparent, MJ_GuiSize s, QWidget *parent, const char *name);

  bool readFile(const QString &);

  MJ_MidiSequence & sequence() { return m_sequence; }

  static bool *debugPtr() { return & m_debug; }

protected:

  MJ_MidiSequence m_sequence;
  QList<MJ_MidiEvent> m_currEvents;

  MJ_MessageMidiEvent m_eventMsg;

  static bool m_debug;
};

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

/** Simple MIDI file viewer class. */
class MJ_MidiView : public QWidget, public MJ_RtControl
{
public:
  MJ_MidiView(QWidget      *parent = 0,
	      const char   *name = 0);
  virtual ~MJ_MidiView();
  
  void setSequence(MJ_MidiSequence *s);

  void draw();

protected:
  void paintEvent(QPaintEvent *);

  MJ_MidiSequence *m_midiSeq;
};

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

/** Gui for MJ_ModuleMidiseq. @see MJ_ModuleMidiseq */
class MJ_ModuleMidiseq_QG : public MJ_ModuleDspGui
{
  Q_OBJECT
public:
  MJ_ModuleMidiseq_QG(MJ_RtControl *rtparent,
		      MJ_ModuleDsp *module,
		      QWidget      *parent = 0,
		      const char   *name = 0);
  virtual ~MJ_ModuleMidiseq_QG();

  virtual void moduleModified(const char *);

  /// Self-test
  static void test(QApplication *xpQA, int iArgc, char **cppArgv);

  static MJ_ModuleDspGui *createNew(MJ_RtControl *rtparent,
				    MJ_ModuleDsp *module,
				    QWidget *parent, 
				    const char *name);

public slots:
  void open();

protected:
  void draw();

  MJ_ModuleMidiseq *m_midiseq;

  MJ_MidiView      *m_midiView;
};

#endif
