/* COPYRIGHT
 *
 * This file is part of Mustajuuri GPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the General Public License (GPL),
 * versions 2. Extra clauses can be found from the file MUSTAJUURI_GPL
 * and further explanations from the file MUSTAJUURI_LICENSE. The
 * license text can be found in file "GPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *  
 */

#ifndef MJ_MODULE_MINISYNTH_H
#define MJ_MODULE_MINISYNTH_H

#include <mj_gui_numeric.h>
#include <mj_midi.h>
#include <mj_module_dsp.h>
#include <mj_module_dsp_gui.h>
#include <mj_ringbuffer.h>

#include <qtimer.h> 

class QLCDNumber;

/// Maximum polyphony for the minisynth.
#define MJ_MINISYNTH_MAX_POLY 128

/** A class that contain parameters for a simple note. 
    
    Helps MJ_ModuleMiniSynth.
 */
class MJ_SimpleNote
{
public:
  friend class MJ_ModuleMiniSynth;

  MJ_SimpleNote();
  ~MJ_SimpleNote() {}

protected:

  MJ_MidiEvent m_noteOn;
  MJ_Sample    m_gain;

  float        m_phase;
  float        m_phaseVel;

  int          m_channel;
  int          m_keyNumber;
};

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

/// Minimalistic synthesizer
class MJ_ModuleMiniSynth : public MJ_ModuleDsp
{
public:
  MJ_ModuleMiniSynth();
  virtual ~MJ_ModuleMiniSynth();

  virtual bool dspPrepare();
  virtual void dspProcessPeriod();
  virtual bool dspStop();

  virtual bool msgProcess(const char *, MJ_Message *);
  virtual MJ_Parameter *msgGetInput(const char *);

  virtual QCString msgGetInputName(int);

  virtual MJ_ModuleDspGui *newGui
  (MJ_RtControl *rtparent, MJ_GuiSize s, QWidget *parent, const char *name);

  int activeNotes() const { return m_activeNotes; }
  void allNotesOff() { m_activeNotes = 0; }

  double gainDb() const;
  void setGainDb(double);  

  static bool * debugPtr() { return & m_debug; }
protected:

  void noteOn(MJ_MidiEvent *);
  void noteOff(MJ_MidiEvent *);

  void removeNote(int);

  int findNote(MJ_MidiEvent *);

  MJ_RingBuffer2<MJ_Sample> m_sampleBuffer;

  int                       m_activeNotes;
  ulong                     m_eventCount;
  MJ_Sample                 m_gain;

  MJ_SimpleNote             m_notes[MJ_MINISYNTH_MAX_POLY];

  static bool               m_debug;
};

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

/** Gui for MJ_ModuleMiniSynth. 
    
    Displays the number of active notes.

    @see MJ_ModuleMiniSynth */
class MJ_ModuleMiniSynth_QG : public MJ_ModuleDspGui
{
  Q_OBJECT
public:
  MJ_ModuleMiniSynth_QG(MJ_RtControl *rtparent,
			MJ_ModuleDsp *module,
			QWidget      *parent = 0,
			const char   *name = 0);

  virtual ~MJ_ModuleMiniSynth_QG();

  static void test(QApplication *xpQA, int iArgc, char **cppArgv);

  static MJ_ModuleDspGui *createNew(MJ_RtControl *rtparent,
				    MJ_ModuleDsp *module,
				    QWidget *parent, 
				    const char *name);


public slots:
  void update();
  void allNotesOff();
  void setVolume(double);
  void getVolume(double *);
protected:

  void draw();

  QTimer             m_timer;
  
  QLCDNumber        *m_lcd;
  MJ_GuiDoubleSlider m_gainSlider;

  MJ_ModuleMiniSynth *m_synth;
};

#endif
