/* COPYRIGHT
 *
 * This file is part of Mustajuuri GPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the General Public License (GPL),
 * versions 2. Extra clauses can be found from the file MUSTAJUURI_GPL
 * and further explanations from the file MUSTAJUURI_LICENSE. The
 * license text can be found in file "GPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *  
 */


#include <mj_module_mixer_bus.h>

#include <qtextstream.h> 

static const char *MJ_busesStr = "BUSES";

MJ_ModuleMixerBus::MJ_ModuleMixerBus()
{
  m_mappingDefs.setAutoDelete(true);
}

MJ_ModuleMixerBus::~MJ_ModuleMixerBus()
{}

bool MJ_ModuleMixerBus::dspPrepare()
{
  if(!MJ_ModuleWeightedSum::dspPrepare()) return false;

  formArray();
  return checkChannelRanges();
}

void MJ_ModuleMixerBus::dspProcessPeriod()
{
  dspClearOutputs();

  MJ_IntIntSample *iis = m_mappings.data();
  const uint max = m_mappings.size();

  for(uint i=0; i < max; i++) {
    MJ_IntIntSample *map = & iis[i];

    MJ_Vector<MJ_Sample> &in = m_signalInput[map->m_i1].samples();
    MJ_Vector<MJ_Sample> &out = m_signalOutput[map->m_i2].samples();

    out.mac(in, map->m_s);
  }
}

void MJ_ModuleMixerBus::clearMappings() 
{ 
  m_mappingDefs.clear(); 
  dspSetInChannels(0);
}

void MJ_ModuleMixerBus::addMapping(const char * sourceId, 
				   int sourceChannel,
				   int targetOutput,
				   MJ_Sample gain)
{ 
  int targetInput = (int) m_mappingDefs.count();

  dspSetInChannels(targetInput + 1);

  dspConnectInput(sourceId, sourceChannel, targetInput);

  MJ_IntIntSample *iis = new MJ_IntIntSample();
  iis->m_i1 = targetInput;
  iis->m_i2 = targetOutput;
  iis->m_s = gain;
  m_mappingDefs.append(iis); 
}

bool MJ_ModuleMixerBus::checkChannelRanges()
{
  uint count = m_mappings.size();

  int ic = dspInChannels();
  int oc = dspOutChannels();
  for(uint i=0; i < count; i++) {
    const MJ_IntIntSample *iis = & m_mappings.getFastConst(i);
    if(iis->m_i1 > ic || iis->m_i2 > oc) {
      error(MJ_ERR_INTERNAL, "MJ_ModuleMixerBus::checkChannelRanges");
      return false;
    }
  }
  return true;
}

bool MJ_ModuleMixerBus::writeParameters(QTextStream *s)
{
  (*s) << MJ_busesStr << "  " << m_mappings.size() << "\n";

  for(uint i=0; i < m_mappings.size(); i++) {
    MJ_IntIntSample *map = & m_mappings.getFast(i);
    (*s) << map->m_i1 << "  " << map->m_i2 << "  " << map->m_s << "\n";
  }
  return true; 
}

bool MJ_ModuleMixerBus::readParameters(QTextStream *s)
{
  const char * fname = "MJ_ModuleMixerBus::readParameters";

  QCString str;
  uint siz;
  bool ok = true;

  // trace(fname);

  (*s) >> str;
  if(str != MJ_busesStr) {
    error(MJ_ERR_DATA_CORRUPTED, fname, 1);
    return false;
  }
  
  (*s) >> str;
  siz = str.toUInt(&ok);

  if(!ok) {
    error(MJ_ERR_DATA_CORRUPTED, fname, 2);
    return false;
  }

  m_mappings.resize(siz); 

  for(uint i=0; i < siz; i++) {    
    MJ_IntIntSample *map = & m_mappings.getFast(i);
    (*s) >> map->m_i1 >> map->m_i2 >>  map->m_s;
  }

  s->readLine();

  return true;
}

void MJ_ModuleMixerBus::formArray()
{
  uint count = m_mappingDefs.count();
  m_mappings.resize(count);
  for(uint i=0; i < count; i++) {
    m_mappings.getFast(i) = * m_mappingDefs.at(i);
  }
}

