/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of Mustajuuri GPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the General Public License (GPL),
 * versions 2. Extra clauses can be found from the file MUSTAJUURI_GPL
 * and further explanations from the file MUSTAJUURI_LICENSE. The
 * license text can be found in file "GPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *  
 */

#ifndef MJ_MODULE_MIXER_GUI_H
#define MJ_MODULE_MIXER_GUI_H

#include <mj_module_mixer.h>

#include <mj_module_dsp_gui.h>
#include <mj_rect_layout.h>
#include <mj_undo_center.h>

#include <qlabel.h> 
#include <qgroupbox.h> 

class QComboBox;
class QScrollView;


class MJ_AudioIoGui;
class MJ_MidiConfGui;
class MJ_ModuleMenubar;
class MJ_ModuleMixer_QG;
class MJ_PluginDbTree_GQ;

extern int MJ_mixerNonlinearPower(int);

/** GUI for the blank mixer parts. Usually mixer does not have all of
    its slots full. This is the widget that fills the unused space.

    @see MJ_ModuleMixer_QG */
class MJ_MixerBlank_QG : public QLabel
{
  Q_OBJECT
public:

  MJ_MixerBlank_QG(MJ_ModuleMixer_QG *host,
		   uint strip, 
		   uint row,
		   QWidget      *parent = 0,
		   const char   *name = 0);
  
  ~MJ_MixerBlank_QG();

protected:

  void mousePressEvent(QMouseEvent *);


  MJ_ModuleMixer_QG *m_host;

  uint m_strip;

  uint m_row;

  QPopupMenu  *m_menu;

  static bool m_debug;
};

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

/** Stamp GUI for mixer sends. */
class MJ_MixerSendStamp_QG : public QWidget
{
  Q_OBJECT
public:
  ///
  MJ_MixerSendStamp_QG(QWidget      *parent = 0,
			const char   *name = 0);
  
  
protected:
  ///
  QGridLayout       *m_grid;
};

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

/** Resizer GUI for mixer. */

class MJ_ModuleMixerResize : public QGroupBox
{
  Q_OBJECT
public:
  ///
  MJ_ModuleMixerResize(MJ_ModuleMixer *mixer,
		    MJ_Mutex       *lock = 0,
		    QWidget        *parent = 0,
		    const char     *name = 0);
  virtual ~MJ_ModuleMixerResize();

public slots:
  void setStrips(int);
  void setRows(int);
  void setBuses(int);

  void apply();
signals:
  void emitMixerConf(int,int,int);
protected:
  int m_strips, m_rows, m_buses;
};

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

/// Strip header gui for mixer.
class MJ_ModuleStripHeader : public QWidget, public MJ_RtControl
{
  Q_OBJECT
public:
  MJ_ModuleStripHeader(int             strip,
		       MJ_ModuleMixer_QG *mixer,
		       MJ_RtControl   *rtParent,
		       QWidget        *parent = 0,
		       const char     *name = 0);
  
  virtual ~MJ_ModuleStripHeader();

  void addModule(const QString &name, int number);
  void clearHeader();

  void setStrip(int s) { m_strip = s; }
  void rebuild();

public slots:
  void deleteModule(int);
  void moduleDeleted(int,int);
  void viewStrip() { emit emitViewStrip(m_strip); }
  void viewModule(int);

  void clear();
  void defaultStrip();

signals:

  void emitDelete(int,int);
  void emitViewStrip(int);
  void emitViewModule(int,int);

protected:
  virtual void realTimeChange(bool nowRt);

  int            m_strip,   m_deleteId, m_viewModuleId, m_clearId, m_defId;

  QPopupMenu    *m_delete, *m_viewModule, *m_menu;

  MJ_ModuleMixer_QG *m_mixer;

  /** Remapping objects. */
  QList<QObject> m_remaps;

  static bool m_debug;
};

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

/** Gui for MJ_ModuleMixer. @see MJ_ModuleMixer */
class MJ_ModuleMixer_QG : public MJ_ModuleDspGui
{
  Q_OBJECT
public:

  friend class MJ_ModuleStripHeader;

  MJ_ModuleMixer_QG(MJ_RtControl *rtparent,
		    MJ_ModuleDsp *module,
		    QWidget      *parent = 0,
		    const char   *name = 0);
  
  ///
  virtual ~MJ_ModuleMixer_QG();

  MJ_ModuleMixer *mixer() { return m_mixer; }
  void sendBusChanged(int bus, int s, int r);

  /** Places a new module to the mixer. Potential old modules are
      deleted. */
  void placeModule(MJ_ModuleDsp *m, uint strip, uint row);

  static void test(QApplication *xpQA, int iArgc, char **cppArgv);

  static MJ_ModuleDspGui *createNew(MJ_RtControl *rtparent,
				    MJ_ModuleDsp *module,
				    QWidget *parent, 
				    const char *name);

  virtual void hide();

  void setActive(int strip, int row)
  { m_activeStrip = strip; m_activeRow = row; }
  
  MJ_ModuleMenubar *menuBar() { return m_menuBar; }


  QPopupMenu *pluginMenu(uint row);

  QWidget *childParent();

  static bool *debugPtr() { return &m_debug; }

  virtual bool loadModule(const QString &);

  /** @name SLOTS */
  //@{
public slots:

  void moduleModified2(int);
  
  void moduleModified4(int,int,int);

  void writeCout();

  void runShort();

  void runShort2();

  void stopShort2();

  void resizeMixer();

  void mixerConf(int,int,int);

  void removeModule(int,int);

  void viewStrip(int);

  void viewModule(int,int);

  void forgetModule(int,int);

  void updateSlot(int,int);

  void clear();

  void startThreaded();
  void stopThreaded();

  void viewAudioIo();
  void enableAudioIo(bool,int,bool);
  void viewMidiIo();
  void enableMidiIo(bool,int,bool);

  /// Create a new module.
  void newModule(const QString &);
  /// A new mixer send.
  void mixerSend();

  /// A new mixer source.
  void mixerSource();

  void forgetChild(QWidget *);
  void showModule(MJ_ModuleDsp *);

  void toggleMMC();

  //@}

signals:
  /// 
  void moduleDeleted(int,int);
  /// 
  void reload();
protected:

  /// Overrides QWidget::paintEvent
  virtual void paintEvent(QPaintEvent *e);
  virtual void showEvent(QShowEvent *e);
  /** Draw the vertical bars that separate the mixer strips. */
  void drawVertSeparators();

  /// 
  void realTimeChange(bool nowRt);

  /// 
  enum MJ_MixerRtChange {
    MJ_MIXER_RT_START_THREADED,
    MJ_MIXER_RT_STOP_THREADED,
    MJ_MIXER_RT_SIZEOF
  };

  /// 
  void draw();
  /// 
  void redraw(int, int);
  
  /** Resize the grid to incorporate. The actual grid will have one
      row more than the number of elements in a strip since the strip
      header menu takes some space.  */
  void resizeGrid(int strips, int rows);

  void showAll();

  void relayout();
  void buildStripHeaders();
  void rebuildStripHeader(uint);
  void updateAudioConf();
  void updateMidiConf();

  /** Delete the DSP GUI widgets that have been launched to their own
      windows. */
  void deleteChildren();
  void clearInternal();

  /** Given an object places a GUI for that object in the mixer. */
  QWidget * placeGui(MJ_MixerUnit *u, uint strip, uint row);
  
  MJ_ModuleMixer *m_mixer;

  QScrollView    *m_scrollView;

  MJ_RectLayout   m_layoutEngine;

  MJ_ModuleMenubar *m_menuBar;
  QPopupMenu       *m_playMenu;

  MJ_AudioIoGui    *m_audioIoGui;
  MJ_MidiConfGui   *m_midiConfGui;

  /** Menu of the plugins. By building these only once we cut down the
      instantation speed and memory usage. */

  QPopupMenu     *m_pluginMenuFirstRow;
  QPopupMenu     *m_pluginMenu;
  QPopupMenu     *m_pluginMenuLastRow;
  
  MJ_Array2D<QWidget *> m_moduleWindowGuis;
  MJ_Array2D<QWidget *> m_moduleGuis;

  MJ_Array<QWidget *> m_stripHeaders;

  /// True type is MJ_MixerStrip_QG
  QList<QWidget>  m_stripWindowGuis;

  int             m_menuIds[MJ_MIXER_RT_SIZEOF];

  int             m_activeStrip, m_activeRow;

  int             m_action;
  int             m_mmcId;

  /** Distance of the members in the grid. */
  static const int m_gdist;// = 8;
  static const int m_gbord;// = 4;

  static bool     m_debug;
};

#endif
