/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of Mustajuuri GPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the General Public License (GPL),
 * versions 2. Extra clauses can be found from the file MUSTAJUURI_GPL
 * and further explanations from the file MUSTAJUURI_LICENSE. The
 * license text can be found in file "GPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *  
 */

#ifndef MJ_MIXER_SEND_H
#define MJ_MIXER_SEND_H

#include <mj_module_dsp_gui_default.h>
#include <mj_module_gain.h>
#include <mj_config.h>
#include <mj_connect_view.h>
#include <mj_rt_control.h>

class MJ_ModuleMixer;
class MJ_Satellite;

class QScrollView;
class QTextStream;

/// Mixer send module
class MJ_ModuleMixerSend : public MJ_ModuleGain
{
public:
  static const char m_keyWord[];
  static const char *m_params[];
  
  enum {
    DO_ADD,
    DO_REMOVE,
    DO_SIZEOF
  };

  MJ_ModuleMixerSend();
  virtual ~MJ_ModuleMixerSend();

  MJ_Array<MJ_3Int> & mappings() { return m_mappings; }

  uint nofMappings() const { return m_mappings.size(); }

  void resizeMappings(uint size) { m_mappings.resize(size); }

  bool addMappings(int i)
  { return m_mappings.addSize(i, false); }

  void setMapping(uint ind, int type, int sourceChannel, int targetChannel);
  const MJ_3Int &mapping(uint i) { return m_mappings.get(i); }

  void forgetMapping(int ind) { m_mappings.removeFast(ind); }

  bool isActiveMapping(uint ind, MJ_ModuleMixer *mixer);

  /* MJ_Sample gainDb() const { return m_gainDb; }
     void setGainDb(MJ_Sample gain) { m_gainDb = gain; } */

  /* uint channels() { return m_channels; }
     void setChannels(uint c) { m_channels = c; } */
  
  bool makeLegal(MJ_ModuleMixer *mixer);

  virtual bool writeParameters(QTextStream *s);
  virtual bool readParameters(QTextStream *s);

protected:

  /** The mappings that define where signal gets from this
      module. Each mapping element has three variables:

      <OL>
      <LI> Type of mapping  - MJ_MixerSourceType
      <LI> Source channel - Channel of this module
      <LI> Target channel - Channel in the target module
      </OL>
  
  */

  MJ_Array<MJ_3Int> m_mappings;
};


/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

/** GUI For MJ_MixerSend 

    @see MJ_MixerSend
*/
class MJ_ModuleMixerSend_QG : public MJ_ModuleDspGuiPostCard
{
  Q_OBJECT
public:

  enum Changes {
    CHANGE_CONNECTION,
    CHANGE_SIZEOF
  };

  MJ_ModuleMixerSend_QG(MJ_ModuleMixerSend *send,
			MJ_ModuleMixer     *mixer,
			MJ_RtControl       *rtparent,
			QWidget      *parent = 0,
			const char   *name = 0);
  
  virtual ~MJ_ModuleMixerSend_QG();
  
  void realTimeChange(bool nowRt);

  static bool * debugPtr() { return & m_debug; }

  static MJ_ModuleDspGui *createNew(MJ_RtControl *rtparent,
				    MJ_ModuleDsp *module,
				    QWidget *parent, 
				    const char *name);
  virtual void moduleIoModified(uint);
public slots:
 
  void mixerModified(int);
  void sendModified(const char *);

  void newConnection(int,int);
  void deleteConnection(int);

signals:
  void emitDestroyed(QWidget *);
protected:

  virtual void resizeEvent(QResizeEvent *);

  void updateView();

  MJ_ModuleMixerSend   *m_mixerSend;
  MJ_ModuleMixer *m_mixer;

  MJ_ConnectView *m_mapper;
  QScrollView    *m_scroll;

  static bool     m_debug;
};

#endif
