/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of Mustajuuri GPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the General Public License (GPL),
 * versions 2. Extra clauses can be found from the file MUSTAJUURI_GPL
 * and further explanations from the file MUSTAJUURI_LICENSE. The
 * license text can be found in file "GPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *  
 */

#ifndef MJ_MODULE_MIXER_SOURCE_H
#define MJ_MODULE_MIXER_SOURCE_H

#include <mj_connect_view.h>
#include <mj_module_bypass.h>
#include <mj_module_dsp_gui.h>

#include <qpainter.h> 

#define MJ_MOD_MIXER_SOURCE_MAX_CHANNELS 64

class MJ_ModuleMixer;

class QComboBox;
class QButtonGroup;
class QLabel;
class QScrollView;
class QSlider;

enum MJ_MixerSourceType {
  MJ_MIXERSOURCE_AUDIO_INPUT,
  MJ_MIXERSOURCE_AUDIO_OUTPUT,
  MJ_MIXERSOURCE_BUS,
  MJ_MIXERSOURCE_SIZEOF
};

extern const char *MJ_MixerSourceTypes[];

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

/** Helper for class MJ_ModuleMixer. 
    Takes care of the first input row.
 */
class MJ_ModuleMixerSource : public MJ_ModuleBypass
{
public:

  MJ_ModuleMixerSource();
  virtual ~MJ_ModuleMixerSource();

  virtual bool dspPrepare();
  virtual void dspProcessPeriod();

  virtual bool writeParameters(QTextStream *s);
  virtual bool readParameters(QTextStream *s);

  bool setMappings(uint i)
  { return m_mappings.resize(i, false); }

  bool addMappings(int i)
  { return m_mappings.addSize(i, false); }

  void setMapping(uint ind, int type, int sourceChannel, int targetChannel);
  inline uint nofMappings() { return m_mappings.size(); }

  MJ_Array<MJ_3Int> & sourceDefs() { return  m_mappings; }

  /// Internal mappings.
  MJ_Array<MJ_2Int> & mappings2() { return  m_mappings2; }

  void forgetMapping(int);

  void setGain(MJ_Sample g) { m_gain = g; }
  
  /** Makes this a legal source - connections may be removed. 
      
      If the module is modified the return value is true. Else the
      return value is false.  */
  bool makeLegal();

protected:

  /** The mappings that define where signal gets to this
      module. Each mapping element has three variables:

      <OL>
      <LI> Type of mapping  - MJ_MixerSourceType
      <LI> Source channel - Channel of source module
      <LI> Target channel - Target channel in this module
      </OL>
  
  */

  MJ_Array<MJ_3Int>  m_mappings;
  MJ_Array<MJ_2Int>  m_mappings2;

  MJ_Sample          m_gain;   
  
};

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

/** A GUI for MJ_ModuleMixerSource. */
class MJ_ModuleMixerSource_QG : public MJ_ModuleDspGui
{
  Q_OBJECT
public:

  enum Changes {
    CHANGE_CONNECTION,
    CHANGE_SIZEOF
  };

  MJ_ModuleMixerSource_QG(MJ_RtControl *rtparent,
			  MJ_ModuleDsp *module,
			  QWidget      *parent = 0,
			  const char   *name = 0);

  virtual ~MJ_ModuleMixerSource_QG();

  static MJ_ModuleDspGui *createNew(MJ_RtControl *rtparent,
				    MJ_ModuleDsp *module,
				    QWidget *parent, 
				    const char *name);

  virtual void moduleModified2(int);

public slots:

  void setChannels(int);
  void updateChannels();

  void mixerModified(int);

  void newConnection(int,int);
  void deleteConnection(int);

protected:

  void realTimeChange(bool nowRt);

  virtual void resizeEvent(QResizeEvent *);

  void updateView();

  void draw();
  
  uint audioChansIn();

  MJ_ModuleMixerSource *m_mixerSource;
  MJ_ModuleMixer       *m_mixer;

  QSlider              *m_chanSlider;
  QLabel               *m_chanLabel;

  int                   m_newChans;

  MJ_ConnectView *m_mapper;
  QScrollView    *m_scroll;
};

#endif
