/* COPYRIGHT
 *
 * This file is part of Mustajuuri GPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the General Public License (GPL),
 * versions 2. Extra clauses can be found from the file MUSTAJUURI_GPL
 * and further explanations from the file MUSTAJUURI_LICENSE. The
 * license text can be found in file "GPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *  
 */

#include <mj_module_oscillator.h>

#include <mj_mapper.h> 
#include <mj_math.h>
#include <mj_messages.h>
#include <mj_parameters.h>
#include <mj_strs.h>


MJ_ModuleOscillator::MJ_ModuleOscillator()
  : m_gain(0.5),
    m_phase(0.0),
    m_phaseStep(0.001),
    m_freq(440)
{
  m_signalOutput.resize(1);
}

MJ_ModuleOscillator::~MJ_ModuleOscillator()
{}

bool MJ_ModuleOscillator::dspPrepare()
{
  m_phase = 0.0;
  updateFreq();
  return m_signalOutput.size() == 1;
}

void MJ_ModuleOscillator::dspProcessPeriod()
{
  MJ_Vector<MJ_Sample> *outVect = & m_signalOutput[0].samples();

  MJ_Sample * out = outVect->data();
  uint n = outVect->size();

  for(uint i = 0; i < n; i++) {
    m_phase += m_phaseStep;
    *out++ = (MJ_Sample) sin(m_phase) * m_gain;
    
    if(m_phase > MJ_2pi) m_phase -= MJ_2pi;
  }
}

bool MJ_ModuleOscillator::msgProcess(const char * id, MJ_Message *m)
{
  bool ok = true;

  double tmp = m->toDouble(&ok);

  if(!ok) return false;

  if(!strcmp(MJ_Strs::frequency, id)) {
    m_freq = tmp;
    updateFreq();
    return true;
  }
  if(!strcmp(MJ_Strs::gain, id)) {
    m_gain = tmp;
    return true;
  }
  return false;
}

MJ_Parameter * MJ_ModuleOscillator::msgGetInput(const char * i)
{
  if(!strcmp(MJ_Strs::frequency, i)) {
    MJ_ParameterDouble *pd = 
      new MJ_ParameterDouble(MJ_Strs::Frequency, i, 
			     m_freq, 0.1, 22000, 
			     MJ_UNIT_HERTZ);
    pd->setMapper(new MJ_MapperLog);
    return pd;
  }
  if(!strcmp(MJ_Strs::gain, i))
    return new MJ_ParameterDouble(MJ_Strs::Gain, i, 
				  m_gain, 0.0, 1.0, 
				  MJ_UNIT_DECIBEL);
  return 0;
}

QCString MJ_ModuleOscillator::msgGetInputName(int i)
{
  QCString r;
  if(i == 0) r = MJ_Strs::frequency;
  else if(i == 1) r = MJ_Strs::gain;
  
  return r;
}
  
void MJ_ModuleOscillator::updateFreq()
{
  if(!m_signalOutput.size()) return;

  double sr = dspSamplingRate();
  m_phaseStep = m_freq * MJ_2pi / sr;
}
