/* COPYRIGHT
 *
 * This file is part of Mustajuuri GPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the General Public License (GPL),
 * versions 2. Extra clauses can be found from the file MUSTAJUURI_GPL
 * and further explanations from the file MUSTAJUURI_LICENSE. The
 * license text can be found in file "GPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *  
 */

#ifndef MJ_MODULE_OSCILLATOR_H
#define MJ_MODULE_OSCILLATOR_H

#include <mj_module_dsp.h>

/** Sinusoid oscillator class. 

    The value of sin is calculated with standard libm function {\em
    sin}.

    The oscillator frequency is modified with a parameter.  */
class MJ_ModuleOscillator : public MJ_ModuleDsp
{
public:
  MJ_ModuleOscillator();
  virtual ~MJ_ModuleOscillator();

  virtual bool dspPrepare();
  virtual void dspProcessPeriod();
  virtual bool msgProcess(const char *, MJ_Message *);
  virtual MJ_Parameter * msgGetInput(const char *);
  virtual QCString msgGetInputName(int);
  
protected:
  void updateFreq();

  MJ_Sample m_gain;
  /// Current oscillator phase
  double m_phase;
  /// How much the oscillator phase is advanced in each step.
  double m_phaseStep;
  /// Frequency in Hz.
  double m_freq;
};

#endif
