/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of Mustajuuri GPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the General Public License (GPL),
 * versions 2. Extra clauses can be found from the file MUSTAJUURI_GPL
 * and further explanations from the file MUSTAJUURI_LICENSE. The
 * license text can be found in file "GPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *  
 */

#ifndef MJ_MODULE_PHASE_DISPLAY_H
#define MJ_MODULE_PHASE_DISPLAY_H

#include <mj_filter_ap1.h>
#include <mj_module_dsp.h>
#include <mj_module_dsp_gui_default.h>
#include <mj_multi_sample.h>
#include <mj_ringbuffer.h>

#include <qgl.h> 
#include <qtimer.h> 

class QHBoxLayout;
class QVBoxLayout;
class QLabel;
class QPushButton;


/** Signal phase measurement plugin. This DSP-module basically just
    collects the sample data into a local ring buffer. */ 

class MJ_ModulePhaseDisplay : public MJ_ModuleDsp
{
public:
  friend class MJ_PhaseWidget;

  MJ_ModulePhaseDisplay();
  virtual ~MJ_ModulePhaseDisplay();
  
  virtual bool dspPrepare();
  virtual void dspProcessPeriod();

  virtual bool msgProcess(const char *, MJ_Message *);
  virtual MJ_Parameter * msgGetInput(const char *);
  virtual QCString msgGetInputName(int);

protected:
  
  MJ_RingBufferDelay<MJ_DualSample> m_buffer;

  float m_gain;
};


/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

class MJ_ModulePhaseDisplay_QG;

class MJ_PhaseWidget : public QGLWidget, public MJ_RtControl
{

public:
  MJ_PhaseWidget(const QGLFormat & format,
		 MJ_RtControl *rtparent,
		 MJ_ModulePhaseDisplay * phase = 0,
		 QWidget      *parent = 0,
		 const char   *name = 0);
  
  virtual ~MJ_PhaseWidget();

  void clearGl();


protected:

  virtual void realTimeChange (bool nowRt); 

  virtual void initializeGL ();
  virtual void resizeGL ( int width, int height );
  virtual void paintGL ();

  QTimer                    m_timer;

  MJ_ModulePhaseDisplay    * m_phase;

  MJ_RingBufferDelay<MJ_DualSample> m_buffer;
};

/// GUI for MJ_ModulePhaseDisplay
class MJ_ModulePhaseDisplayGui : public MJ_ModuleDspGuiPostCard
{
public:
  MJ_ModulePhaseDisplayGui(MJ_RtControl *rtparent,
			   MJ_ModuleDsp *module,
			   QWidget      *parent = 0,
			   const char   *name = 0);

  virtual ~MJ_ModulePhaseDisplayGui();

  static MJ_ModuleDspGui *createNew(MJ_RtControl *rtparent,
				    MJ_ModuleDsp *module,
				    QWidget *parent, 
				    const char *name);
};


#endif
