/* COPYRIGHT
 *
 * This file is part of Mustajuuri GPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the General Public License (GPL),
 * versions 2. Extra clauses can be found from the file MUSTAJUURI_GPL
 * and further explanations from the file MUSTAJUURI_LICENSE. The
 * license text can be found in file "GPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *  
 */

#include <mj_module_print_messages.h>

#include <mj_module_blank_messages.h>
#include <mj_module_dsp_row.h>

#include <mj_messages.h>
#include <mj_message_factory.h>
#include <mj_strs.h>
#include <mj_error.h>

#include <stdlib.h>

MJ_ModulePrintMessages::MJ_ModulePrintMessages()
  : m_file("foo.msgs"),
    m_stream(&m_file),
    m_silent(false)
{
  m_file.open(IO_WriteOnly);
}

MJ_ModulePrintMessages::~MJ_ModulePrintMessages()
{}
    
void MJ_ModulePrintMessages::dspProcessPeriod()
{}

bool MJ_ModulePrintMessages::msgProcess(const char *, MJ_Message *m)
{
  if(!m_silent)
    return MJ_MessageFactory::write(&m_stream, m);
  return true;
}

void MJ_ModulePrintMessages::test(int ac, const char **av)
{
  MJ_trace("MJ_ModulePrintMessages::test");

  bool benchMark = false;
  
  for(int i = 1; i< ac; i++) {
    if(strcmp(av[i], "--benchmark") == 0)
      benchMark = true;
  }


  MJ_ModuleDspRow dr;
  
  // Creates "double" messages:
  MJ_ModuleBlankMessages *bm = new MJ_ModuleBlankMessages;
  // Prints the messages:
  MJ_ModulePrintMessages *pm = new MJ_ModulePrintMessages;
  
  dr.appendModule(bm);
  dr.appendModule(pm);

  MJ_trace("Connecting");
  
  bool ok = pm->msgConnectInput(bm->id(), 
				bm->msgGetOutputName(0), 
				MJ_Strs::foo);
  
  if(!ok) {
    MJ_fatal(MJ_ERR_UNKNOWN, "Failed to connect");
  }
  
  if(!benchMark)
    dr.runShort(50);
  else {
    pm->silent(true);
    dr.runShort(200000);
  }
}
