/* COPYRIGHT
 *
 * This file is part of Mustajuuri GPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the General Public License (GPL),
 * versions 2. Extra clauses can be found from the file MUSTAJUURI_GPL
 * and further explanations from the file MUSTAJUURI_LICENSE. The
 * license text can be found in file "GPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *  
 */

#include <mj_module_printer.h>

#include <mj_error.h>
#include <mj_gui_utils.h>
#include <mj_iostream.h>

#include <qapplication.h>
#include <qcheckbox.h> 
#include <qlabel.h> 
#include <qlayout.h> 


MJ_ModulePrinter::MJ_ModulePrinter()
  : m_stream(&MJ_cout()),
    m_silence(false)
{
  m_signalSourceDefs.resize(1);
}

MJ_ModulePrinter::~MJ_ModulePrinter()
{}

void MJ_ModulePrinter::dspProcessPeriod()
{
  if(m_silence) return;

  (*m_stream) << "\nMJ_ModulePrinter::dspProcessPeriod";

  for(uint i = 0; i < m_signalInput.size(); i++) {
    (*m_stream) << " CH: " << i << " DATA: ";
    
    for(uint j = 0; j < m_signalInput[i].samples().size(); j++) {
      (*m_stream) << " " << m_signalInput[i].samples()(j);
    }
  }
}

MJ_ModuleDspGui *MJ_ModulePrinter::newGui
(MJ_RtControl *rtparent, MJ_GuiSize s, QWidget *parent, const char *name)
{
  QWidget *p = 0;

  if(s != MJ_GUI_OWN_WINDOW)
    p = parent;
  
  return new MJ_ModulePrinter_QG(rtparent, this, p, name);
}

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

MJ_ModulePrinter_QG::MJ_ModulePrinter_QG(MJ_RtControl *rtparent,
					 MJ_ModuleDsp *module,
					 QWidget      *parent,
					 const char   *name)
  : MJ_ModuleDspGui(rtparent, module, parent, name),
    m_button(0)
{
  draw();
  if(!parent) show();

  MJ_ModulePrinter *mp = dynamic_cast<MJ_ModulePrinter *> (module);

  if(!mp) {
    error(MJ_ERR_ARGUMENT, "MJ_ModulePrinter_QG::MJ_ModulePrinter_QG");
    return;
  }
  m_printer = mp;
  
  lock();
  bool b = m_printer->silence();
  unlock();
  m_button->setChecked(b);
}

MJ_ModulePrinter_QG::~MJ_ModulePrinter_QG()
{}

void MJ_ModulePrinter_QG::test(QApplication *xpQA, int iArgc, char **cppArgv)
{
  (void) iArgc;
  (void) cppArgv;

  MJ_ModulePrinter p;
  MJ_ModulePrinter_QG pg(0, &p);

  xpQA->setMainWidget(&pg);
  xpQA->exec();
}

MJ_ModuleDspGui *MJ_ModulePrinter_QG::createNew(MJ_RtControl *rtparent, 
						MJ_ModuleDsp *module, 
						QWidget *parent, 
						const char *name)
{
  return new MJ_ModulePrinter_QG(rtparent, module, parent, name);
}

void MJ_ModulePrinter_QG::silence(bool b)
{
  if(!m_printer) return;

  MJ_trace("MJ_ModulePrinter_QG::silence", (int) b);
  
  lock();
  m_printer->setSilence(b);
  unlock();
}

void MJ_ModulePrinter_QG::draw()
{
  QGridLayout *l = new QGridLayout(this, 4,3, 5, 0);
  MJ_equalStretch(l);

  QLabel *label = new QLabel(this, "printlabel");
  label->setText("Printer");
  l->addWidget(label,1,1);

  m_button = new QCheckBox(this, "silencer");
  m_button->setText("silence");
  connect(m_button, SIGNAL(toggled(bool)), this, SLOT(silence(bool)));
  l->addWidget(m_button,2,1);

  MJ_limMinSize(label);
  MJ_limMinSize(m_button);

}
