/* COPYRIGHT
 *
 * This file is part of Mustajuuri GPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the General Public License (GPL),
 * versions 2. Extra clauses can be found from the file MUSTAJUURI_GPL
 * and further explanations from the file MUSTAJUURI_LICENSE. The
 * license text can be found in file "GPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *  
 */

#ifndef MJ_MODULE_PRINTER_H
#define MJ_MODULE_PRINTER_H


#include <mj_module_dsp.h>
#include <mj_module_dsp_gui.h>

#include <mj_ringbuffer.h>

class QApplication;
class QCheckBox;

class QTextStream;

/** Prints any input signals to the screen. */
class MJ_ModulePrinter : public MJ_ModuleDsp
{
public:
  MJ_ModulePrinter();
  virtual ~MJ_ModulePrinter();

  virtual void dspProcessPeriod();

  /// Silence (=switch off) the printing
  void setSilence(bool s) { m_silence = s; }
  bool silence() const { return m_silence; }

  virtual MJ_ModuleDspGui *newGui
  (MJ_RtControl *rtparent, MJ_GuiSize s, QWidget *parent, const char *name);

protected:
  
  QTextStream *m_stream;
  bool         m_silence;
};

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

/** Gui for MJ_ModulePrinter. @see MJ_ModulePrinter */
class MJ_ModulePrinter_QG : public MJ_ModuleDspGui
{
  Q_OBJECT
public:
  MJ_ModulePrinter_QG(MJ_RtControl *rtparent,
		      MJ_ModuleDsp *module,
		      QWidget      *parent = 0,
		      const char   *name = 0);

  virtual ~MJ_ModulePrinter_QG();

  static void test(QApplication *xpQA, int iArgc, char **cppArgv);

  static MJ_ModuleDspGui *createNew(MJ_RtControl *rtparent, 
				    MJ_ModuleDsp *module, 
				    QWidget *parent, 
				    const char *name);

public slots:
  void silence(bool);

protected:
  void draw();

  QCheckBox *m_button;

  MJ_ModulePrinter *m_printer;
};

#endif
