/* COPYRIGHT
 *
 * This file is part of Mustajuuri GPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the General Public License (GPL),
 * versions 2. Extra clauses can be found from the file MUSTAJUURI_GPL
 * and further explanations from the file MUSTAJUURI_LICENSE. The
 * license text can be found in file "GPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *  
 */

#include <mj_module_protector.h>
#include <mj_math.h>
#include <mj_misc.h>
#include <mj_error.h>
#include <mj_parameters.h>
#include <mj_strs.h>

#include <mj_module_dsp_gui_default.h>

#include <qapplication.h>


bool MJ_ModuleProtector::m_debug = true;

MJ_ModuleProtector::MJ_ModuleProtector()
  : m_ratio(1.0),
    m_maxAmpl(0.0),
    m_allowedPeak(0.55)
{}

MJ_ModuleProtector::~MJ_ModuleProtector()
{}

bool MJ_ModuleProtector::dspPrepare()
{
  if(m_signalInput.size() != m_signalOutput.size())
    return false;

  m_ratio   = 10.0;
  m_maxAmpl = 0.0;
  return true;
}

void MJ_ModuleProtector::dspProcessPeriod()
{
  // Iterate over channels
  for(uint i=0; i < m_signalInput.size(); i++) {

    const MJ_Vector<MJ_Sample> *in = & m_signalInput[i].samples();
    MJ_Vector<MJ_Sample> *out = & m_signalOutput[i].samples();
    
    // iterate over samples.
    for(uint j=0; j < out->size(); j++) {
      MJ_Sample curr = MJ_abs(in->getConst(i));
      
      if(curr > m_maxAmpl) {
	m_maxAmpl = curr;

	if(curr > m_allowedPeak) {
	  m_ratio = m_allowedPeak / curr;
	  // cout << "Ratio change: " << m_ratio;
	}
      }
      out->get(i) = in->getConst(i) * m_ratio;
    }
  }
}


bool MJ_ModuleProtector::msgProcess(const char * id, MJ_Message *m)
{
  const char *fname = "MJ_ModuleProtector::msgProcess";

  if(m_debug) trace(fname);
  
  bool ok = true;
  
  if(!strcmp(id, MJ_Strs::threshold)) { 

    double tmp = m->toDouble(&ok);
    if(ok) {
      if(m_debug) { 
	trace(fname, "Success 1");
	// cout << "Value: " << MJ_dbToAbs(tmp) << endl; 
      }

      m_allowedPeak = MJ_dbToAbs(tmp);
    }
  }

  if(!ok && m_debug) {
    error(MJ_ERR_ARGUMENT, fname);
  }

  return ok;
}

MJ_Parameter * MJ_ModuleProtector::msgGetInput(const char * i)
{
  if(m_debug) trace("MJ_ModuleProtector::msgGetInputs");
  
  if(!strcmp(i, MJ_Strs::threshold)) { 
    // Peak:
    MJ_ParameterDouble *pd = new MJ_ParameterDouble;
    pd->set(MJ_Strs::Threshold, i,
	    MJ_absToDb(m_allowedPeak), -80, 20, MJ_UNIT_BLANK);

#if 0
    for(int i=1; i < 10; i++) {
      pd->addAllowed( i * 0.1);
    }
#endif

    return pd;
  }
  return 0;
}

QCString MJ_ModuleProtector::msgGetInputName(int i)
{
  return MJ_copyIfZero(MJ_Strs::threshold, i);
}

void MJ_ModuleProtector::test(QApplication *xpQA, int iArgc, char **cppArgv)
{
  (void) iArgc;
  (void) cppArgv;

  MJ_ModuleProtector p;
  MJ_ModuleDspGuiPostCard pg(0, &p), pg2(0, &p);

  xpQA->setMainWidget(&pg);

  pg.show();
  xpQA->exec();
}
