/* COPYRIGHT
 *
 * This file is part of Mustajuuri GPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the General Public License (GPL),
 * versions 2. Extra clauses can be found from the file MUSTAJUURI_GPL
 * and further explanations from the file MUSTAJUURI_LICENSE. The
 * license text can be found in file "GPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *  
 */
/* -*- C++ -*- */

#ifndef MJ_MODULE_PROTECTOR_H
#define MJ_MODULE_PROTECTOR_H

#include <mj_module_dsp.h>

#include <mj_ringbuffer.h>

class QApplication;

/** Hearing protector. 

    This class limits the amplitude of the signal
    to avoid hardware clipping.
 */
class MJ_ModuleProtector : public MJ_ModuleDsp
{

public:

  MJ_ModuleProtector();
  virtual ~MJ_ModuleProtector();

  virtual bool dspPrepare();
  virtual void dspProcessPeriod();
  
  bool limits() const { return m_ratio > 1.0; }

  virtual bool msgProcess(const char *, MJ_Message *);
  virtual MJ_Parameter * msgGetInput(const char *);
  virtual QCString msgGetInputName(int);

  static void test(QApplication *xpQA, int iArgc, char **cppArgv);

protected:

  /// Multiplier.
  MJ_Sample                 m_ratio;
  /// The biggest amplitude so far.
  MJ_Sample                 m_maxAmpl;
  /// The maximum amplitude before clipping
  MJ_Sample                 m_allowedPeak;

  static bool m_debug;
};

#endif
