/* COPYRIGHT
 *
 * This file is part of Mustajuuri GPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the General Public License (GPL),
 * versions 2. Extra clauses can be found from the file MUSTAJUURI_GPL
 * and further explanations from the file MUSTAJUURI_LICENSE. The
 * license text can be found in file "GPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *  
 */

#ifndef MJ_MODULE_EXAMPLE0_H
#define MJ_MODULE_EXAMPLE0_H

#include <mj_module_dsp.h>

/** This file contain headers for a number of simple arithmetic plugins. */

/** This plugin multiplies two input signals.
 */
class MJ_ModuleMul2 : public MJ_ModuleDsp
{
public:
  MJ_ModuleMul2() {}
  virtual ~MJ_ModuleMul2() {}

  virtual bool dspPrepare();
  virtual void dspProcessPeriod();
};

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

/** This plugin sums two input signals. There is no need to define
    dspPrepare, since the parent class has an implementation that
    checks that the number of inputs and outputs is legal.
 */
class MJ_ModuleAdd2 : public MJ_ModuleMul2
{
public:
  MJ_ModuleAdd2() {}
  virtual ~MJ_ModuleAdd2() {}
  virtual void dspProcessPeriod();
};

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

/** This plugin subtracts two input signals.
 */
class MJ_ModuleSub2 : public MJ_ModuleMul2
{
public:
  MJ_ModuleSub2() {}
  virtual ~MJ_ModuleSub2() {}
  virtual void dspProcessPeriod();
};


/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

/** This plugin takes the absolute values of all input signals.
 */
class MJ_ModuleAbs : public MJ_ModuleDsp
{
public:
  MJ_ModuleAbs() {}
  virtual ~MJ_ModuleAbs() {}

  virtual bool dspPrepare();
  virtual void dspProcessPeriod();
};


/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

/** This plugin takes squares all input signals.
 */
class MJ_ModuleSquare : public MJ_ModuleAbs
{
public:
  MJ_ModuleSquare() {}
  virtual ~MJ_ModuleSquare() {}
  virtual void dspProcessPeriod();
};

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

/** This plugin takes signedSquare of all input signals.
 */
class MJ_ModuleSignedSquareRoot : public MJ_ModuleAbs
{
public:
  MJ_ModuleSignedSquareRoot() {}
  virtual ~MJ_ModuleSignedSquareRoot() {}
  virtual void dspProcessPeriod();
};

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

/** This plugin takes signedSquare of all input signals.
 */
class MJ_ModuleConst : public MJ_ModuleDsp
{
public:
  MJ_ModuleConst() {}
  virtual ~MJ_ModuleConst();

  virtual bool dspPrepare();
  virtual void dspProcessPeriod();

  virtual bool msgProcess(const char *, MJ_Message *);
  virtual MJ_Parameter * msgGetInput(const char *);
  virtual QCString msgGetInputName(int);

protected:
  MJ_Sample m_value;
};

#endif
