/* COPYRIGHT
 *
 * This file is part of Mustajuuri GPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the General Public License (GPL),
 * versions 2. Extra clauses can be found from the file MUSTAJUURI_GPL
 * and further explanations from the file MUSTAJUURI_LICENSE. The
 * license text can be found in file "GPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *  
 */

#include <mj_module_automation.h>
#include <mj_module_blank_messages.h>
#include <mj_module_bypass.h>
#include <mj_module_compressor.h>
#include <mj_module_compressor_gui.h>
#include <mj_module_delay.h>
#include <mj_module_dsp_row_gui.h>
#include <mj_module_gain.h>
#include <mj_module_gain_multi.h>
#include <mj_module_impulse.h>
#include <mj_module_impulsetrain.h>
#include <mj_module_level.h>
#include <mj_module_midiseq.h>
#include <mj_module_minisynth.h>
#include <mj_module_mixer_gui.h>
#include <mj_module_mixer_send.h>
#include <mj_module_mixer_source.h>
#include <mj_module_monodelay.h>
#include <mj_module_noise.h>
#include <mj_module_oscillator.h>
#include <mj_module_phase_display.h>
#include <mj_module_print_messages.h>
#include <mj_module_printer.h>
#include <mj_module_protector.h>
#include <mj_module_span.h>
#include <mj_module_wsum.h>
#include <mj_modules_arithmetic.h>

#include <mj_debug.h>
#include <mj_error.h>
#include <mj_dsp_plugin.h>
#include <mj_module_keys.h>
#include <mj_strs.h>
#include <mj_to_ladspa_plugin.h>
#include <mj_version.h>

static bool MJ_exportDebug = false;

// Bypass

MJ_ModuleDsp *MJ_createNew_ModuleBypass()
{
  return new MJ_ModuleBypass;
}

MJ_PluginDbeDsp *MJ_ModuleBypass_Dbe()
{
  if(MJ_exportDebug)
   MJ_trace("Enter MJ_ModuleBypass_Dbe");
  
  MJ_PluginDbeDsp *dbe = new MJ_PluginDbeDsp;

  dbe->setAuthors(MJ_Strs::authorTommi);
  dbe->setMenuPosition("_Audio_Routing_Bypass");
  dbe->setAbout("Signal bypass/sum");
  dbe->setCreator(MJ_createNew_ModuleBypass);
  dbe->setIdentifier("_routing_bypass_1");
  dbe->pluginVersion()->set(0,0,1);
  if(MJ_exportDebug)
 MJ_trace("Exit MJ_Bypass_Dbe");

  return dbe;
}

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

// Impulse train

MJ_ModuleDsp *MJ_createNew_ModuleImpulseTrain()
{
  return new MJ_ModuleImpulseTrain;
}

MJ_PluginDbeDsp *MJ_ModuleImpulseTrain_Dbe()
{
  if(MJ_exportDebug)
   MJ_trace("MJ_ModuleImpulseTrain_Dbe");
  
  MJ_PluginDbeDsp *dbe = new MJ_PluginDbeDsp;

  dbe->setAuthors(MJ_Strs::authorTommi);
  dbe->setMenuPosition("_Audio_Source_Impulse train");
  dbe->setAbout("Generates a steady series of impulses");
  dbe->setCreator(MJ_createNew_ModuleImpulseTrain);
  dbe->setIdentifier("_source_impulse_train_1");
  dbe->pluginVersion()->set(0,0,1);

  dbe->options()->defaultChans2(0,1);


  return dbe;
}

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////


MJ_ModuleDsp *MJ_createNew_ModuleMonoDelay()
{
  return new MJ_ModuleMonoDelay;
}

MJ_PluginDbeDsp *MJ_ModuleMonoDelay_Dbe()
{
  if(MJ_exportDebug)
   MJ_trace("MJ_ImpulseTrain_Dbe");
  
  MJ_PluginDbeDsp *dbe = new MJ_PluginDbeDsp;

  dbe->setAuthors(MJ_Strs::authorTommi);
  dbe->setMenuPosition("_Audio_Delay_Mono delay");
  dbe->setAbout("Single channel delay");
  dbe->setCreator(MJ_createNew_ModuleMonoDelay);
  dbe->setIdentifier(MJ_monoDelayIdentifier);
  dbe->pluginVersion()->set(0,0,1);

  dbe->options()->defaultChans(1);

  return dbe;
}

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

MJ_ModuleDsp *MJ_createNew_ModulePrinter()
{
  return new MJ_ModulePrinter;
}

MJ_PluginDbeDsp *MJ_ModulePrinter_Dbe()
{
  if(MJ_exportDebug)
   MJ_trace("MJ_ModulePrinter_Dbe");
  
  MJ_PluginDbeDsp *dbe = new MJ_PluginDbeDsp;

  dbe->setAuthors(MJ_Strs::authorTommi);
  dbe->setMenuPosition("_Audio_Sink_Printer");
  dbe->setAbout("Prints audio signal to the screen");
  dbe->setCreator(MJ_createNew_ModulePrinter);
  dbe->setIdentifier("_sink_printer_1");
  dbe->pluginVersion()->set(0,0,1);
  dbe->options()->setChans(1, 1, 0, 0);
  dbe->options()->undefine(MJ_DSP_MAX_CHANNELS_OUT); // No upper bound

  dbe->setGuiFactory(MJ_GUI_STAMP, MJ_ModulePrinter_QG::createNew);

  return dbe;
}

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

MJ_ModuleDsp *MJ_createNew_ModuleDspRow()
{
  return new MJ_ModuleDspRow;
}

MJ_PluginDbeDsp *MJ_ModuleDspRow_Dbe()
{
  if(MJ_exportDebug)
   MJ_trace("MJ_ModuleDspRow_Dbe");
  
  MJ_PluginDbeDsp *dbe = new MJ_PluginDbeDsp;

  dbe->setAuthors(MJ_Strs::authorTommi);
  dbe->setMenuPosition("_Combo_Row");
  dbe->setAbout("An effects rack");
  dbe->setCreator(MJ_createNew_ModuleDspRow);
  dbe->setIdentifier(MJ_dspRowIdentifier);
  dbe->setGuiFactory(MJ_GUI_OWN_WINDOW, MJ_ModuleDspRow_QG::createNew);
  dbe->pluginVersion()->set(0,0,1);

  dbe->setGuiFactory(MJ_GUI_POSTCARD, MJ_ModuleDspRow_QG::createNew);

  dbe->setParameter(MJ_PP_METAFUNCTIONALITY,
		    MJ_DspPluginMetaFunctionalityKeys[MJ_DPMF_STAND_ALONE]);

  return dbe;
}

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

MJ_ModuleDsp *MJ_createNew_ModuleGain()
{
  return new MJ_ModuleGain;
}

MJ_PluginDbeDsp *MJ_ModuleGain_Dbe()
{
  if(MJ_exportDebug)
   MJ_trace("MJ_ModuleGain_Dbe");
  
  MJ_PluginDbeDsp *dbe = new MJ_PluginDbeDsp;

  dbe->setAuthors(MJ_Strs::authorTommi);
  dbe->setMenuPosition("_Audio_Gain_Gain");
  dbe->setAbout("Volume controller");
  dbe->setCreator(MJ_createNew_ModuleGain);
  dbe->setIdentifier(MJ_dspGainIdentifier);
  dbe->pluginVersion()->set(0,0,1);
  dbe->options()->setValue(MJ_DSP_EQUAL_CHANNEL_IO, 1);
  dbe->options()->setValue(MJ_DSP_MIN_CHANNELS_IN, 1);
  dbe->options()->setValue(MJ_DSP_MIN_CHANNELS_OUT, 1);
  dbe->setPluginDebug(MJ_ModuleGain::debugPtr());

  // LADSPA specific stuff:

  // CONSUME LADSPA IDS 1343-1344

  MJ_LadspaDatas *lds = new MJ_LadspaDatas;
  dbe->setLadspaDatas(lds);
  // Mono version:
  lds->newElem(1343, "mjgain-mono", "Mustajuuri Gain (mono)", 1, 1);
  // Stereo version:
  lds->newElem(1344, "mjgain-stereo", "Mustajuuri Gain (stereo)", 2, 2);

  return dbe;
}
/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

MJ_ModuleDsp *MJ_createNew_ModuleProtector()
{
  return new MJ_ModuleProtector;
}

MJ_PluginDbeDsp *MJ_ModuleProtector_Dbe()
{
  if(MJ_exportDebug)
   MJ_trace("MJ_ModuleProtector_Dbe");
  
  MJ_PluginDbeDsp *dbe = new MJ_PluginDbeDsp;

  dbe->setAuthors(MJ_Strs::authorTommi);
  dbe->setMenuPosition("_Audio_Dynamics_Protector");
  dbe->setCreator(MJ_createNew_ModuleProtector);
  dbe->setIdentifier("_dynamics_protector_1");
  dbe->pluginVersion()->set(0,0,1);

  dbe->options()->setValue(MJ_DSP_MIN_CHANNELS_IN, 1);
  dbe->options()->setValue(MJ_DSP_MIN_CHANNELS_OUT, 1);

  dbe->options()->setValue(MJ_DSP_EQUAL_CHANNEL_IO, 1);

  return dbe;
}

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

MJ_ModuleDsp *MJ_createNew_ModuleBlankMessages()
{
  return new MJ_ModuleBlankMessages;
}

MJ_PluginDbeDsp *MJ_ModuleBlankMessages_Dbe()
{
  if(MJ_exportDebug)
   MJ_trace("MJ_ModuleBlankMessages_Dbe");
  
  MJ_PluginDbeDsp *dbe = new MJ_PluginDbeDsp;

  dbe->setAuthors(MJ_Strs::authorTommi);
  dbe->setMenuPosition("_Messages_Source_Blank messages");
  dbe->setCreator(MJ_createNew_ModuleBlankMessages);
  dbe->setIdentifier("_source_BlankMessages_1");
  dbe->pluginVersion()->set(0,0,1);
  dbe->options()->defaultChans(0);

  return dbe;
}

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

MJ_ModuleDsp *MJ_createNew_ModulePrintMessages()
{
  return new MJ_ModulePrintMessages;
}

MJ_PluginDbeDsp *MJ_ModulePrintMessages_Dbe()
{
  if(MJ_exportDebug)
   MJ_trace("MJ_ModulePrintMessages_Dbe");
  
  MJ_PluginDbeDsp *dbe = new MJ_PluginDbeDsp;

  dbe->setAuthors(MJ_Strs::authorTommi);
  dbe->setMenuPosition("_Messages_Sink_Message printer");
  dbe->setAbout("Prints messages to the screen");
  dbe->setCreator(MJ_createNew_ModulePrintMessages);
  dbe->setIdentifier("_sink_PrintMessages_1");
  dbe->pluginVersion()->set(0,0,1);
  dbe->options()->defaultChans(0);

  return dbe;
}

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

MJ_ModuleDsp *MJ_createNew_ModuleMiniSynth()
{
  return new MJ_ModuleMiniSynth;
}

MJ_PluginDbeDsp *MJ_ModuleMiniSynth_Dbe()
{
  if(MJ_exportDebug)
   MJ_trace("MJ_ModuleMiniSynth_Dbe");
  
  MJ_PluginDbeDsp *dbe = new MJ_PluginDbeDsp;

  dbe->setAuthors(MJ_Strs::authorTommi);
  dbe->setMenuPosition("_Audio_Synthesis_Minisynth");
  dbe->setAbout("Minimalistic synthesizer");
  dbe->setCreator(MJ_createNew_ModuleMiniSynth);
  dbe->setIdentifier("_synthesis_MiniSynth_1");
  dbe->pluginVersion()->set(0,0,1);
  dbe->options()->defaultChans2(0, 1);
  dbe->setPluginDebug(MJ_ModuleMiniSynth::debugPtr());
  return dbe;
}

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

MJ_ModuleDsp *MJ_createNew_ModuleMidiseq()
{
  return new MJ_ModuleMidiseq;
}

MJ_PluginDbeDsp *MJ_ModuleMidiseq_Dbe()
{
  if(MJ_exportDebug)
   MJ_trace("MJ_ModuleMidiseq_Dbe");
  
  MJ_PluginDbeDsp *dbe = new MJ_PluginDbeDsp;

  dbe->setAuthors(MJ_Strs::authorTommi);
  dbe->setMenuPosition("_Messages_MIDI_Minisequencer");
  dbe->setAbout("Minimalistic MIDI sequencer");
  dbe->setCreator(MJ_createNew_ModuleMidiseq);
  dbe->setIdentifier("_midi_MiniSequencer_1");
  dbe->pluginVersion()->set(0,0,1);
  dbe->options()->defaultChans(0);

  dbe->setPluginDebug(MJ_ModuleMidiseq::debugPtr());

  dbe->setGuiFactory(MJ_GUI_POSTCARD, MJ_ModuleMidiseq_QG::createNew);

  return dbe;
}

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

MJ_ModuleDsp *MJ_createNew_ModuleWeightedSum()
{
  return new MJ_ModuleWeightedSum;
}

MJ_PluginDbeDsp *MJ_ModuleWeightedSum_Dbe()
{
  if(MJ_exportDebug)
   MJ_trace("MJ_ModuleWeightedSum_Dbe");
  
  MJ_PluginDbeDsp *dbe = new MJ_PluginDbeDsp;

  dbe->setAuthors(MJ_Strs::authorTommi);
  dbe->setMenuPosition("_Audio_Routing_Weighted sum");
  dbe->setCreator(MJ_createNew_ModuleWeightedSum);
  dbe->setIdentifier("_routing_weighted_sum_1");
  dbe->pluginVersion()->set(0,0,1);
  dbe->options()->setValue(MJ_DSP_MIN_CHANNELS_OUT, 1);

  return dbe;
}

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////


MJ_ModuleDsp *MJ_createNew_ModuleNoise()
{
  return new MJ_ModuleNoise;
}

MJ_PluginDbeDsp *MJ_ModuleNoise_Dbe()
{
  if(MJ_exportDebug)
   MJ_trace("MJ_ModuleNoise_Dbe");
  
  MJ_PluginDbeDsp *dbe = new MJ_PluginDbeDsp;

  dbe->setAuthors(MJ_Strs::authorTommi);
  dbe->setMenuPosition("_Audio_Source_Noise");
  dbe->setCreator(MJ_createNew_ModuleNoise);
  dbe->setIdentifier("_source_noise_1");
  dbe->pluginVersion()->set(0,0,1);
  dbe->options()->setChans(0, 0, 1, 100000);
  
  return dbe;
}

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

MJ_ModuleDsp *MJ_createNew_ModuleLevel()
{
  return new MJ_ModuleLevel;
}

MJ_PluginDbeDsp *MJ_ModuleLevel_Dbe()
{
  if(MJ_exportDebug)
   MJ_trace("MJ_ModuleLevel_Dbe");
  
  MJ_PluginDbeDsp *dbe = new MJ_PluginDbeDsp;

  dbe->setAuthors(MJ_Strs::authorTommi);
  dbe->setMenuPosition("_Audio_Visual_Level meter");
  dbe->setCreator(MJ_createNew_ModuleLevel);
  dbe->setIdentifier(MJ_dspVisualIdentifier);
  dbe->pluginVersion()->set(0,0,1);
  dbe->options()->setValue(MJ_DSP_MIN_CHANNELS_IN, 1);
  dbe->options()->setValue(MJ_DSP_MAX_CHANNELS_OUT, 0);

  if(MJ_Debug::debugAll())
    MJ_ModuleLevel_QG::setDebug(true);

  return dbe;
}

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

MJ_ModuleDsp *MJ_createNew_ModuleStereoPan()
{
  return new MJ_ModuleStereoPan;
}

MJ_PluginDbeDsp *MJ_ModuleStereoPan_Dbe()
{
  if(MJ_exportDebug)
   MJ_trace("MJ_ModuleStereoPan_Dbe");
  
  MJ_PluginDbeDsp *dbe = new MJ_PluginDbeDsp;

  dbe->setAuthors(MJ_Strs::authorTommi);
  dbe->setMenuPosition("_Audio_Panning_Stereo pan");
  dbe->setCreator(MJ_createNew_ModuleStereoPan);
  dbe->setIdentifier("_panning_stereo_pan_1");
  dbe->pluginVersion()->set(0,0,1);
  dbe->options()->setChans(1, 2, 2, 2);

  return dbe;
}

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////


MJ_ModuleDsp *MJ_createNew_ModuleMixer()
{
  return new MJ_ModuleMixer;
}

MJ_PluginDbeDsp *MJ_ModuleMixer_Dbe()
{
  const char *id = "_combo_mixer";

  // Put the group debugging on if necessary.

  if(MJ_Debug::debugAll() || MJ_Debug::debugNeeded(id)) {
    * MJ_ModuleDspGroup::groupDebug() = true;
    MJ_MixerUnit::setDebug(1);
    * MJ_ModuleMixer_QG::debugPtr() = true;

    * MJ_ModuleMixerStrip::debugPtr() = true;
  }
  

  if(MJ_exportDebug)
   MJ_trace("MJ_ModuleMixer_Dbe");

  MJ_PluginDbeDsp *dbe = new MJ_PluginDbeDsp;

  dbe->setAuthors(MJ_Strs::authorTommi);
  dbe->setMenuPosition("_Combo_Mixer");
  dbe->setAbout("Traditional mixing console");
  
  dbe->setCreator(MJ_createNew_ModuleMixer);
  dbe->setIdentifier(id);
  dbe->setGuiFactory(MJ_GUI_OWN_WINDOW, MJ_ModuleMixer_QG::createNew);
  dbe->pluginVersion()->set(0,0,1);

  dbe->setGuiFactory(MJ_GUI_POSTCARD, MJ_ModuleMixer_QG::createNew);

  dbe->setParameter(MJ_PP_METAFUNCTIONALITY,
		    MJ_DspPluginMetaFunctionalityKeys[MJ_DPMF_STAND_ALONE]);
  
  dbe->setPluginDebug(MJ_ModuleMixer::debugPtr());

  return dbe;
}

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

MJ_ModuleDsp *MJ_createNew_ModuleMixerSource()
{
  return new MJ_ModuleMixerSource;
}

MJ_PluginDbeDsp *MJ_ModuleMixerSource_Dbe()
{
  if(MJ_exportDebug)
   MJ_trace("MJ_ModuleMixerSource_Dbe");
  
  MJ_PluginDbeDsp *dbe = new MJ_PluginDbeDsp;

  dbe->setAuthors(MJ_Strs::authorTommi);
  dbe->setMenuPosition("_Audio_Routing_Mixer input");
  dbe->setCreator(MJ_createNew_ModuleMixerSource);
  dbe->setIdentifier(MJ_mixerSourceIdentifier);
  dbe->pluginVersion()->set(0,0,1);
  dbe->options()->setValue(MJ_DSP_MIN_CHANNELS_OUT, 1);

  dbe->setGuiFactory(MJ_GUI_POSTCARD, MJ_ModuleMixerSource_QG::createNew);
  
  /* Zero out the metafunctionality. After this the module won't be
     visible in most menus.*/
  dbe->setParameter(MJ_PP_METAFUNCTIONALITY, "");
		    

  return dbe;
}

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

MJ_ModuleDsp *MJ_createNew_ModuleMixerSend()
{
  return new MJ_ModuleMixerSend;
}

MJ_PluginDbeDsp *MJ_ModuleMixerSend_Dbe()
{
  if(MJ_exportDebug)
   MJ_trace("MJ_ModuleMixerSend_Dbe");
  
  MJ_PluginDbeDsp *dbe = new MJ_PluginDbeDsp;

  dbe->setAuthors(MJ_Strs::authorTommi);
  dbe->setMenuPosition("_Audio_Routing_Mixer send");
  dbe->setCreator(MJ_createNew_ModuleMixerSend);
  dbe->setIdentifier(MJ_mixerSendIdentifier);
  dbe->pluginVersion()->set(0,0,1);
  dbe->options()->setValue(MJ_DSP_MIN_CHANNELS_OUT, 1);

  dbe->setGuiFactory(MJ_GUI_POSTCARD, MJ_ModuleMixerSend_QG::createNew);

  dbe->options()->setValue(MJ_DSP_EQUAL_CHANNEL_IO, 1);
  dbe->options()->setValue(MJ_DSP_MIN_CHANNELS_IN, 1);
  dbe->options()->setValue(MJ_DSP_MIN_CHANNELS_OUT, 1);
  
  /* Zero out the metafunctionality. After this the module won't be
     visible in most menus.*/
  dbe->setParameter(MJ_PP_METAFUNCTIONALITY, "");

  return dbe;
}

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

MJ_ModuleDsp *MJ_createNew_ModuleMixerBus()
{
  return new MJ_ModuleMixerBus;
}

MJ_PluginDbeDsp *MJ_ModuleMixerBus_Dbe()
{
  if(MJ_exportDebug)
   MJ_trace("MJ_ModuleMixerBus_Dbe");
  
  MJ_PluginDbeDsp *dbe = new MJ_PluginDbeDsp;

  dbe->setAuthors(MJ_Strs::authorTommi);
  dbe->setMenuPosition("_Audio_Routing_Mixer bus");
  dbe->setCreator(MJ_createNew_ModuleMixerBus);
  dbe->setIdentifier(MJ_busIdentifier);
  dbe->pluginVersion()->set(0,0,1);

  /* Zero out the metafunctionality. After this the module won't be
     visible in most menus.*/
  dbe->setParameter(MJ_PP_METAFUNCTIONALITY, "");

  return dbe;
}

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

MJ_ModuleDsp *MJ_createNew_ModuleOscillator()
{
  return new MJ_ModuleOscillator;
}

MJ_PluginDbeDsp *MJ_ModuleOscillator_Dbe()
{
  if(MJ_exportDebug)
   MJ_trace("MJ_ModuleOscillator_Dbe");
  
  MJ_PluginDbeDsp *dbe = new MJ_PluginDbeDsp;

  dbe->setAuthors(MJ_Strs::authorTommi);
  dbe->setMenuPosition("_Audio_Source_Oscillator");
  dbe->setCreator(MJ_createNew_ModuleOscillator);
  dbe->setIdentifier(MJ_dspOscillatorIdentifier);
  dbe->pluginVersion()->set(0,0,1);

  dbe->options()->defaultChans2(0,1);

  return dbe;
}

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

MJ_ModuleDsp *MJ_createNew_ModuleImpulse()
{
  return new MJ_ModuleImpulse;
}

MJ_PluginDbeDsp *MJ_ModuleImpulse_Dbe()
{
  if(MJ_exportDebug)
   MJ_trace("MJ_ModuleImpulse_Dbe");
  
  MJ_PluginDbeDsp *dbe = new MJ_PluginDbeDsp;

  dbe->setAuthors(MJ_Strs::authorTommi);
  dbe->setMenuPosition("_Audio_Source_Impulse");
  dbe->setCreator(MJ_createNew_ModuleImpulse);
  dbe->setGuiFactory(MJ_GUI_STAMP, MJ_ModuleImpulse_QG::createNew);
  dbe->setGuiFactory(MJ_GUI_POSTCARD, MJ_ModuleImpulse_QG::createNew);
  dbe->setIdentifier(MJ_dspImpulseIdentifier);
  dbe->pluginVersion()->set(0,5,0);

  dbe->options()->defaultChans2(0,1);

  return dbe;
}

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

MJ_ModuleDsp *MJ_createNew_ModuleCompressor()
{
  return new MJ_ModuleCompressor;
}

MJ_PluginDbeDsp *MJ_ModuleCompressor_Dbe()
{
  MJ_PluginDbeDsp *dbe = new MJ_PluginDbeDsp;

  dbe->setAuthors(MJ_Strs::authorTommi);
  dbe->setMenuPosition("_Audio_Dynamics_Simple Compressor");
  dbe->setCreator(MJ_createNew_ModuleCompressor);
  dbe->setGuiFactory(MJ_GUI_POSTCARD, MJ_ModuleCompressorGui::createNew);
  dbe->setIdentifier(MJ_dspCompressorIdentifier);
  dbe->pluginVersion()->set(0,0,1);

  dbe->options()->setValue(MJ_DSP_EQUAL_CHANNEL_IO, 1);
  dbe->options()->setValue(MJ_DSP_MIN_CHANNELS_IN, 1);
  dbe->options()->setValue(MJ_DSP_MIN_CHANNELS_OUT, 1);

  MJ_LadspaDatas *lds = new MJ_LadspaDatas;
  dbe->setLadspaDatas(lds);
  // Mono version:
  lds->newElem(1345, "mjcompressor1-mono", "Compressor 1 (mono)", 1, 1);
  // Stereo version:
  lds->newElem(1346, "mjcompressor1-stereo", "Compressor 1 (stereo)", 2, 2);

  return dbe;
}

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

MJ_ModuleDsp *MJ_createNew_ModuleDelay()
{
  return new MJ_ModuleDelay;
}

MJ_PluginDbeDsp *MJ_ModuleDelay_Dbe()
{
  MJ_PluginDbeDsp *dbe = new MJ_PluginDbeDsp;

  dbe->setAuthors(MJ_Strs::authorTommi);
  dbe->setMenuPosition("_Audio_Delay_Delay");
  dbe->setCreator(MJ_createNew_ModuleDelay);
  dbe->setIdentifier(MJ_dspDelayIdentifier);
  dbe->pluginVersion()->set(0,0,1);

  dbe->options()->setValue(MJ_DSP_EQUAL_CHANNEL_IO, 1);
  dbe->options()->setValue(MJ_DSP_MIN_CHANNELS_IN, 1);
  dbe->options()->setValue(MJ_DSP_MIN_CHANNELS_OUT, 1);

  return dbe;
}

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

MJ_ModuleDsp *MJ_createNew_ModuleMul2()
{
  return new MJ_ModuleMul2;
}

MJ_PluginDbeDsp *MJ_ModuleMul2_Dbe()
{
  if(MJ_exportDebug)
    MJ_trace("MJ_ModuleMul2_Dbe");
  
  MJ_PluginDbeDsp *dbe = new MJ_PluginDbeDsp;

  dbe->setAuthors(MJ_Strs::authorTommi);
  dbe->setMenuPosition("_Audio_Arithmetic_Dual multiply");
  dbe->setCreator(MJ_createNew_ModuleMul2);
  dbe->setIdentifier(MJ_dspMul2Identifier);
  dbe->pluginVersion()->set(1,0,0);

  dbe->options()->defaultChans2(2,1);

  return dbe;
}

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

MJ_ModuleDsp *MJ_createNew_ModuleAdd2()
{
  return new MJ_ModuleAdd2;
}

MJ_PluginDbeDsp *MJ_ModuleAdd2_Dbe()
{
  if(MJ_exportDebug)
    MJ_trace("MJ_ModuleAdd2_Dbe");
  
  MJ_PluginDbeDsp *dbe = MJ_ModuleMul2_Dbe();

  dbe->setMenuPosition("_Audio_Arithmetic_Dual add");
  dbe->setCreator(MJ_createNew_ModuleAdd2);
  dbe->setIdentifier(MJ_dspAdd2Identifier);

  return dbe;
}

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

MJ_ModuleDsp *MJ_createNew_ModuleSub2()
{
  return new MJ_ModuleSub2;
}

MJ_PluginDbeDsp *MJ_ModuleSub2_Dbe()
{
  if(MJ_exportDebug)
    MJ_trace("MJ_ModuleSub2_Dbe");
  
  MJ_PluginDbeDsp *dbe = MJ_ModuleMul2_Dbe();

  dbe->setMenuPosition("_Audio_Arithmetic_Dual subtract");
  dbe->setCreator(MJ_createNew_ModuleSub2);
  dbe->setIdentifier(MJ_dspSub2Identifier);

  return dbe;
}

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

MJ_ModuleDsp *MJ_createNew_ModuleAbs()
{
  return new MJ_ModuleAbs;
}

MJ_PluginDbeDsp *MJ_ModuleAbs_Dbe()
{
  MJ_PluginDbeDsp *dbe = new MJ_PluginDbeDsp;

  dbe->setAuthors(MJ_Strs::authorTommi);
  dbe->setMenuPosition("_Audio_Arithmetic_Abs");
  dbe->setCreator(MJ_createNew_ModuleAbs);
  dbe->setIdentifier(MJ_dspAbsIdentifier);
  dbe->pluginVersion()->set(1,0,0);

  dbe->options()->setValue(MJ_DSP_EQUAL_CHANNEL_IO, 1);
  dbe->options()->setValue(MJ_DSP_MIN_CHANNELS_IN, 1);
  dbe->options()->setValue(MJ_DSP_MIN_CHANNELS_OUT, 1);

  return dbe;
}

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

MJ_ModuleDsp *MJ_createNew_ModuleSquare()
{
  return new MJ_ModuleSquare;
}

MJ_PluginDbeDsp *MJ_ModuleSquare_Dbe()
{
  MJ_PluginDbeDsp *dbe = MJ_ModuleAbs_Dbe();

  dbe->setMenuPosition("_Audio_Arithmetic_Square");
  dbe->setCreator(MJ_createNew_ModuleSquare);
  dbe->setIdentifier(MJ_dspSquareIdentifier);

  return dbe;
}

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

MJ_ModuleDsp *MJ_createNew_ModuleSignedSquareRoot()
{
  return new MJ_ModuleSignedSquareRoot;
}

MJ_PluginDbeDsp *MJ_ModuleSignedSquareRoot_Dbe()
{
  MJ_PluginDbeDsp *dbe = MJ_ModuleAbs_Dbe();

  dbe->setMenuPosition("_Audio_Arithmetic_Signed square");
  dbe->setCreator(MJ_createNew_ModuleSignedSquareRoot);
  dbe->setIdentifier(MJ_dspSignedSquareRootIdentifier);

  return dbe;
}

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

MJ_ModuleDsp *MJ_createNew_ModuleGainMulti()
{
  return new MJ_ModuleGainMulti;
}

MJ_PluginDbeDsp *MJ_ModuleGainMulti_Dbe()
{
  if(MJ_exportDebug)
   MJ_trace("MJ_ModuleGainMulti_Dbe");
  
  MJ_PluginDbeDsp *dbe = new MJ_PluginDbeDsp;

  dbe->setAuthors(MJ_Strs::authorTommi);
  dbe->setMenuPosition("_Audio_Gain_Gain (channel-specific)");
  dbe->setAbout("Volume controller");
  dbe->setCreator(MJ_createNew_ModuleGainMulti);
  dbe->setIdentifier(MJ_dspGainMultiIdentifier);
  dbe->pluginVersion()->set(0,0,1);
  dbe->options()->setValue(MJ_DSP_EQUAL_CHANNEL_IO, 1);
  dbe->options()->setValue(MJ_DSP_MIN_CHANNELS_IN, 1);
  dbe->options()->setValue(MJ_DSP_MIN_CHANNELS_OUT, 1);
  dbe->setPluginDebug(MJ_ModuleGainMulti::debugPtr());

  return dbe;
}

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

MJ_ModuleDsp *MJ_createNew_ModuleConst()
{
  return new MJ_ModuleConst;
}

MJ_PluginDbeDsp *MJ_ModuleConst_Dbe()
{
  MJ_PluginDbeDsp *dbe = new MJ_PluginDbeDsp;

  dbe->setAuthors(MJ_Strs::authorTommi);
  dbe->setMenuPosition("_Audio_Arithmetic_Constant value");
  dbe->setCreator(MJ_createNew_ModuleConst);
  dbe->setIdentifier(MJ_dspConstIdentifier);
  dbe->options()->defaultChans2(0,1);

  return dbe;
}

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

MJ_ModuleDsp *MJ_createNew_ModuleAutomation()
{
  return new MJ_ModuleAutomation;
}

MJ_PluginDbeDsp *MJ_ModuleAutomation_Dbe()
{
  MJ_PluginDbeDsp *dbe = new MJ_PluginDbeDsp;

  dbe->setAuthors(MJ_Strs::authorTommi);
  dbe->setMenuPosition("_Audio_Automation");
  dbe->setCreator(MJ_createNew_ModuleAutomation);
  dbe->setIdentifier("MJ_AUTOMATION1");
  dbe->options()->defaultChans2(0,0);
  dbe->setGuiFactory(MJ_GUI_POSTCARD, MJ_ModuleAutomation_QG::createNew);

  return dbe;
}
/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

MJ_ModuleDsp *MJ_createNew_ModulePhaseDisplay()
{
  return new MJ_ModulePhaseDisplay;
}

MJ_PluginDbeDsp *MJ_ModulePhaseDisplay_Dbe()
{
  MJ_PluginDbeDsp *dbe = new MJ_PluginDbeDsp;

  dbe->setAuthors(MJ_Strs::authorTommi);
  dbe->setMenuPosition("_Audio_Visual_Phase");
  dbe->setCreator(MJ_createNew_ModulePhaseDisplay);
  dbe->setIdentifier("MJ_PHASEDISPLAY");
  dbe->options()->defaultChans2(2, 0);
  dbe->setGuiFactory(MJ_GUI_POSTCARD, MJ_ModulePhaseDisplayGui::createNew);

  return dbe;
}

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

static MJ_PluginDbeDsp *__MJ_createPlugin(int m)
{
  MJ_exportDebug = MJ_exportDebug || MJ_Debug::debugAll();
  
  if(MJ_exportDebug)
    MJ_trace("Enter libmjmodules_CREATE_DSPMODULE_NTH ");

  switch(m) {
  case 0:
    return MJ_ModuleBypass_Dbe();
  case 1:
    return MJ_ModuleImpulseTrain_Dbe();
  case 2:
    return MJ_ModuleMonoDelay_Dbe();
  case 3:
    return MJ_ModulePrinter_Dbe();
  case 4:
    return MJ_ModuleDspRow_Dbe();
  case 5:
    return MJ_ModuleGain_Dbe();
  case 6:
    return MJ_ModuleProtector_Dbe();
  case 7:
    return MJ_ModuleBlankMessages_Dbe();
  case 8:
    return MJ_ModulePrintMessages_Dbe();
  case 9:
    return MJ_ModuleMiniSynth_Dbe();
  case 10:
    return MJ_ModuleMidiseq_Dbe();
  case 11:
    return MJ_ModuleWeightedSum_Dbe();
  case 12:
    return MJ_ModuleNoise_Dbe();
  case 13:
    return MJ_ModuleLevel_Dbe();
  case 14:
    return MJ_ModuleStereoPan_Dbe();
  case 15:
    return MJ_ModuleMixer_Dbe();
  case 16:
    return MJ_ModuleMixerSource_Dbe();
  case 17:
    return MJ_ModuleMixerBus_Dbe();
  case 18:
    return MJ_ModuleOscillator_Dbe();
  case 19:
    return MJ_ModuleImpulse_Dbe();
  case 20:
    return MJ_ModuleCompressor_Dbe();
  case 21:
    return MJ_ModuleDelay_Dbe();
  case 22:
    return MJ_ModuleMul2_Dbe();
  case 23:
    return MJ_ModuleAdd2_Dbe();
  case 24:
    return MJ_ModuleSub2_Dbe();
  case 25:
    return MJ_ModuleAbs_Dbe();
  case 26:
    return MJ_ModuleSquare_Dbe();
  case 27:
    return MJ_ModuleSignedSquareRoot_Dbe();
  case 28:
    return MJ_ModuleMixerSend_Dbe();
  case 29:
    return MJ_ModuleGainMulti_Dbe();
  case 30:
    return MJ_ModuleConst_Dbe();
  case 31:
    return MJ_ModuleAutomation_Dbe();
  case 32:
    return MJ_ModulePhaseDisplay_Dbe();
  }
  if(MJ_exportDebug)
    MJ_trace("Exit libmjmodules_CREATE_DSPMODULE_NTH ");

  return 0;
}

static QString compileDate(DATETIME);

MJ_PluginDbeDsp *MJ_createPlugin(int m)
{
  MJ_PluginDbeDsp * dbe = __MJ_createPlugin(m);

  if(dbe) dbe->setParameter(MJ_PP_COMPILE_TIME, compileDate);

  return dbe;
}

