/* COPYRIGHT
 *
 * This file is part of Mustajuuri GPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the General Public License (GPL),
 * versions 2. Extra clauses can be found from the file MUSTAJUURI_GPL
 * and further explanations from the file MUSTAJUURI_LICENSE. The
 * license text can be found in file "GPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *  
 */

#include <mj_freq_resp_gui.h>

#include <mj_graphics_area.h>

#include <mj_graphic_axes.h>
#include <mj_graphic_polyline.h>
#include <mj_graphic_polygon.h>
#include <mj_graphic_text.h>
#include <mj_graphics_context_simple.h>
#include <mj_random.h>

#include <qapplication.h>
#include <qcolor.h>
#include <qpen.h>

int main(int argc, char **argv)
{
  QApplication qa(argc, argv);

  MJ_FreqRespGui::test();

  MJ_GraphicsArea ga;
  ga.setBoundingBox(0.0, -0.5, 2.0, 1.5);

  ga.setBackgroundColor(Qt::white);

  qa.setMainWidget(&ga);
  ga.show();

  // Axes
  MJ_GraphicAxes *ax = new MJ_GraphicAxes();
  ax->setGraphicsContext
    (new MJ_GraphicsContextPen(QPen(Qt::black, 1, Qt::DashLine)));

  ga.appendGraphic(ax);

  QArray<float> fa(8);

  // Polygon
  float *data = fa.data();
  data[0] = 0.2;
  data[1] = 0.2;
  data[2] = 0.9;
  data[3] = 0.3;
  data[4] = 0.9;
  data[5] = 0.6;
  data[6] = -.5;
  data[7] = 0.4;

  MJ_GraphicPolygon *gpg = new MJ_GraphicPolygon(fa.copy());
  
  gpg->setGraphicsContext(new MJ_GraphicsContextPenBrush(QPen(Qt::gray, 3),
							 QBrush(Qt::red)));
  
  ga.appendGraphic(gpg);
  

  // Polyline
  MJ_Random r;
  r.getTimeSeed();

  fa.resize(64);
  for(int i=0; i < 64; i++) {
    fa[i] =  r.getDouble0();
  }
  MJ_GraphicPolyLine *gpl = new MJ_GraphicPolyLine(fa);

  gpl->setGraphicsContext(new MJ_GraphicsContextPen(QPen(Qt::darkBlue, 3)));

  ga.appendGraphic(gpl);

  MJ_GraphicText *gt = new MJ_GraphicText("FOO rulez", 1.5, 0.5);
  ga.appendGraphic(gt);
  

  return qa.exec();
}
