/* COPYRIGHT
 *
 * This file is part of Mustajuuri GPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the General Public License (GPL),
 * versions 2. Extra clauses can be found from the file MUSTAJUURI_GPL
 * and further explanations from the file MUSTAJUURI_LICENSE. The
 * license text can be found in file "GPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *  
 */


#ifndef MJ_FREQ_RESP_GUI_H
#define MJ_FREQ_RESP_GUI_H

#include <mj_freq_resp.h>

#include <mj_graphics_area.h>

class MJ_GraphicPolyLine;

/** A DSP filter frequency response widget. Rather crude and
    experimental. 

    By default the widget has two graphic objects - the axes and the
    curve.   */

class MJ_FreqRespGui : public MJ_GraphicsArea
{
  Q_OBJECT

public:
  MJ_FreqRespGui(QWidget      *parent=0, 
		 const char   *name=0,
		 float         widthDb = 20.0,
		 float         minFreq = 0.001,
		 const QColor *graphColor = 0,
		 int           graphWidth = 2);

  MJ_FreqRespGui(int filters, 
		 QWidget      *parent=0, 
		 const char   *name=0,
		 float         widthDb = 20.0,
		 float         minFreq = 0.001);
  
  virtual ~MJ_FreqRespGui();

  void resizeFilters(int filters);

  void clearFilters();

  void setNthFilter(uint filter,
		    const MJ_Sample *b, uint bdegree,
		    const MJ_Sample *a, uint adegree);

  void setNthCurveGC(uint curve, MJ_GraphicsContext *gc);

  void bringToTop(int filter);

  void sumToTop();

  void calculateSum();

  /** Sets the filter parameters. */
  void setFilter(const MJ_Sample *b, uint bdegree,
		 const MJ_Sample *a, uint adegree);

  /// Return the curve object(s)
  MJ_GraphicPolyLine *curve() { return m_curve; }

  /// Returns the number of curve objects
  uint curves() const { return m_curves; }

  static void test();

protected:
  
  void formCurve(MJ_GraphicPolyLine *curve, 
		 const MJ_Sample *b, uint bdegree,
		 const MJ_Sample *a, uint adegree);

  MJ_Array<MJ_Sample> m_b;
  MJ_Array<MJ_Sample> m_a;

  MJ_GraphicPolyLine *m_curve;
  uint                m_curves;
  bool                m_hasSum;
};

#endif
