/* COPYRIGHT
 *
 * This file is part of Mustajuuri LGPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2001.
 * Tommi.Ilmonen@hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. Extra clauses can be found from the file
 * MUSTAJUURI_LGPL and further explanations from the file
 * MUSTAJUURI_LICENSE. The license text can be found in file
 * "LGPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *    
 */

#ifndef MJ_COLOR_H
#define MJ_COLOR_H

#include <qcolor.h>

#include <mj_math.h>

extern const QColor MJ_colorMixedGreen1;
extern const QColor MJ_colorRust;


/** Returns the brightness ("value" in HSV) of a color. */

inline int MJ_colorBrightness(const QColor &c)
{
  int h, s, v;
  c.getHsv(h,s,v);
  return v;
}

/** Multiplies the RGB values of a color. */

inline QColor operator * (const QColor &c, float m)
{
  return QColor(MJ_min((int) (c.red() * m), 0xff), 
		MJ_min((int) (c.green() * m), 0xff),
		MJ_min((int) (c.blue() * m), 0xff));
}

/** Sums the RGB values of two colors. */

inline QColor operator + (const QColor &c1, const QColor &c2)
{
  return QColor(MJ_min((int) ((c1.red() + c2.red())), 0xff), 
		MJ_min((int) ((c1.green() + c2.green())), 0xff), 
		MJ_min((int) ((c1.blue() + c2.blue())), 0xff));
}

#endif
