/* COPYRIGHT
 *
 * This file is part of Mustajuuri LGPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2001.
 * Tommi.Ilmonen@hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. Extra clauses can be found from the file
 * MUSTAJUURI_LGPL and further explanations from the file
 * MUSTAJUURI_LICENSE. The license text can be found in file
 * "LGPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *    
 */

#ifndef MJ_CONNECT_VIEW_H
#define MJ_CONNECT_VIEW_H

#include <mj_multi_int.h>

#include <qwidget.h>
#include <qpointarray.h> 

#include <vector>

/// Defines a label and its location.
class MJ_LabelDef 
{
public:
  MJ_LabelDef() : m_highlighted(false) {}
  QString m_string;
  QRect   m_rect;
  bool    m_highlighted;
};

/** Widget that is used to connect some text entities. */
class MJ_ConnectView : public QWidget
{
  Q_OBJECT
public:
  MJ_ConnectView(QWidget      *parent = 0,
		 const char   *name = 0);
  virtual ~MJ_ConnectView();

  void set(const std::vector<MJ_LabelDef> &leftLabels,
	   const std::vector<MJ_LabelDef> &rightLabels,
	   const std::vector<MJ_2Int> &connections);
  QSize minimumSizeHint() const;

  void drawBase();
  void redraw();
  void addConnection(int,int);

signals:
  void newConnection(int,int);
  void deleteConnection(int);

protected:

  virtual void mousePressEvent(QMouseEvent *); 
  virtual void mouseReleaseEvent(QMouseEvent *);
  virtual void mouseMoveEvent(QMouseEvent *);
  virtual void paintEvent(QPaintEvent *);  
  virtual void keyPressEvent(QKeyEvent *);


  void highlight(bool leftSide, bool makeVisible, uint index);
  void unhighlight();

  bool seekIndex(const QPoint &pos, bool *isleft, int *index);
  void drawConnections();
  void alignLabels();
  void drawLabels();

  bool isConnection(int,int);

  void unhighlightConnection();
  void highlightConnection();

  /** Checks if given point is on top of some connection.
      
      If not, returns -1.
   */

  int onConnection(const QPoint &pos, int range = 3);

  std::vector<MJ_LabelDef> m_sourceLabels;
  std::vector<MJ_LabelDef> m_targetLabels;
  std::vector<MJ_2Int>     m_connections;
  
  QPointArray m_points;

  QSize       m_minimumSize;  

  bool        m_startLeft;
  int         m_startIndex;

  bool        m_lastLeft;
  int         m_lastIndex;

  int         m_connectionIndex;

  int         m_leftBorder;
};


#endif
