/* COPYRIGHT
 *
 * This file is part of Mustajuuri LGPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2001.
 * Tommi.Ilmonen@hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. Extra clauses can be found from the file
 * MUSTAJUURI_LGPL and further explanations from the file
 * MUSTAJUURI_LICENSE. The license text can be found in file
 * "LGPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *    
 */

#ifndef MJ_CSTR_REMAP_H
#define MJ_CSTR_REMAP_H

#include <qobject.h>
#include <qstring.h>

#include <mj_config.h>

/** Transform "void" signals to "const char *" signals. Objects of
this type are used to "add" a s string data to an other call */
class MJ_CStrRemap : public  QObject
{
  Q_OBJECT
public:
#ifdef MJ_MEM_DEBUG
  MJ_CStrRemap(const QCString &i="") : m_id(i.copy()) { m_count++; }
  static long count() { return m_count; }
#else
  MJ_CStrRemap(const QCString &i="") : m_id(i.copy()) {}
#endif

  virtual ~MJ_CStrRemap();
  
  void setId(const QCString & i) { m_id = i; }
  const QCString & id() const { return m_id; }

public slots:
  void call() { emit emitCall(m_id); }
  void callBool(bool b) { emit emitCallBool(m_id, b); }
  void callDouble(double d) { emit emitCallDouble(m_id, d); }
  void callDoublePtr(double *d) { emit emitCallDoublePtr(m_id, d); }

signals: 
  void emitCall(const char *); 
  void emitCallBool(const char *,bool); 
  void emitCallDouble(const char *,double); 
  void emitCallDoublePtr(const char *,double *); 

protected:
  QCString m_id;
#ifdef MJ_MEM_DEBUG
  static long m_count;
#endif

private:
  MJ_CStrRemap(const MJ_CStrRemap &) {}
};

#endif
