/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of Mustajuuri LGPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2001.
 * Tommi.Ilmonen@hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. Extra clauses can be found from the file
 * MUSTAJUURI_LGPL and further explanations from the file
 * MUSTAJUURI_LICENSE. The license text can be found in file
 * "LGPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *    
 */

#ifndef MJ_DIAL_H
#define MJ_DIAL_H

#include <qdial.h>

class MJ_Mapper;

/** A dial (or knob) widget with "double" output. This widget expands
    the QDial by transforming the output to double. 

    Additionally this widget can be instructed to not jump from the
    minimum value to the maximum value = the user has to turn the dial
    to get from minimum value to the maximum value. This feature was
    because it is sometimes \b dangerous to jump from minimum value to
    maximum value (for example when changing the volume of audio
    playback).

    @author Tommi Ilmonen   
*/

class MJ_Dial : public QDial
{
  Q_OBJECT
public:
  MJ_Dial(double minValue, double maxValue, double value, 
	  QWidget *parent = 0, const char * name = 0,
	  int iminValue = 0,
	  int imaxValue = 300,
	  int lineStep = 5,
	  int pageStep = 20);

  virtual ~MJ_Dial();

  void setRealRange(double, double);

  void setSliderRange(int, int);

  void setMapper(MJ_Mapper *m);

  double valueDouble() const {return m_value;}

  /** Allow/forbid jumps from minimum value to maximum value. */
  void setAllowJump(bool allow) { m_allowWrap = allow; }

public slots:

  void setValueDouble(double);
  void getValueDouble(double *d)
  {*d = m_value;}

signals:

  void valueChangedDouble(double);

protected:

  virtual void valueChange();
  
  double m_minValue;
  double m_maxValue;
  double m_value;

  int    m_oldVal;
  bool   m_allowWrap;
  MJ_Mapper  *m_mapper;
};

#endif
