/* COPYRIGHT
 *
 * This file is part of Mustajuuri LGPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2001.
 * Tommi.Ilmonen@hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. Extra clauses can be found from the file
 * MUSTAJUURI_LGPL and further explanations from the file
 * MUSTAJUURI_LICENSE. The license text can be found in file
 * "LGPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *    
 */


#ifndef MJ_GRAPHIC_H
#define MJ_GRAPHIC_H

class MJ_Painter;
class MJ_GraphicsContext;

class QWidget;

/** Graphic element for MJ_GraphicsArea.
    
 */
class MJ_Graphic
{
public:
  MJ_Graphic();

  virtual ~MJ_Graphic();

  /** Renders this object with the painter. */

  virtual void render(MJ_Painter *) = 0;

  /** Unrenders this object with the painter. Typiaclly this method
      can simply call "render" (the brush/pen already is in bacjground
      mode). */
  virtual void unrender(MJ_Painter *) = 0;

  /** Sets the id. */
  void setId(int i) { m_id = i; }

  /** Returns id. */
  int id() { return m_id; }
  
  /** Sets the target widget. */
  void setWidget(QWidget *w) { m_widget = w; }

  /** Returns graphics context (pen/brush etc). */
  MJ_GraphicsContext *graphicsContext() { return m_gc; }

  /** Sets the graphics context. */
  void setGraphicsContext(MJ_GraphicsContext *gc, bool deleteOld = true);

protected:
  int m_id;
  QWidget *m_widget;

  MJ_GraphicsContext *m_gc;
};

#endif
