/* COPYRIGHT
 *
 * This file is part of Mustajuuri LGPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2001.
 * Tommi.Ilmonen@hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. Extra clauses can be found from the file
 * MUSTAJUURI_LGPL and further explanations from the file
 * MUSTAJUURI_LICENSE. The license text can be found in file
 * "LGPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *    
 */


#include <mj_graphic_axes.h>
#include <mj_math.h>
#include <mj_painter.h>

#include <qpainter.h>
#include <qwidget.h>

MJ_GraphicAxes::MJ_GraphicAxes()
  : m_axesDist(100)
{}

MJ_GraphicAxes::~MJ_GraphicAxes()
{}

void MJ_GraphicAxes::render(MJ_Painter *p)
{
  drawHoriz(p, true);
  drawHoriz(p, false);
}

void MJ_GraphicAxes::unrender(MJ_Painter *p)
{
  MJ_GraphicAxes::render(p);
}

void MJ_GraphicAxes::drawHoriz(MJ_Painter *p, bool vertical)
{
  int width = m_widget->width();
  int height = m_widget->height();

  double scale;

  int axes;
  
  if(vertical) {
    if(m_axesDist > 0)
      axes = MJ_max(width / m_axesDist, 3);
    else
      axes = - m_axesDist;
    scale = (double) width / (double) (axes - 1);
  }
  else  {
    if(m_axesDist > 0)
      axes = MJ_max(height / m_axesDist, 3);
    else
      axes = - m_axesDist;
    scale = (double) height / (double) (axes - 1);
  }

  for(int i=0; i < axes; i++) {
    int x = (int) (scale * (double) i);
    
    if(vertical) {
      if(x >= width) x = width - 1;    
      p->qpainter().drawLine(x, 0, x, height);
    }
    else {
      if(x >= height) x = height - 1;    
      p->qpainter().drawLine(0, x, width, x);
    }
  }
}
