/* COPYRIGHT
 *
 * This file is part of Mustajuuri LGPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2001.
 * Tommi.Ilmonen@hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. Extra clauses can be found from the file
 * MUSTAJUURI_LGPL and further explanations from the file
 * MUSTAJUURI_LICENSE. The license text can be found in file
 * "LGPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *    
 */


#ifndef MJ_GRAPHIC_AXES_H
#define MJ_GRAPHIC_AXES_H

#include <mj_graphic.h>

/** A graphics object that draws axes to the screen. 

    The axes can be controlled by either setting the number of axes or
    the distance between them.  */

class MJ_GraphicAxes : public MJ_Graphic
{
public:
  MJ_GraphicAxes();
  virtual ~MJ_GraphicAxes();
  
  virtual void render(MJ_Painter *);
  virtual void unrender(MJ_Painter *);

  /** Sets the number of axes to be drawn. The number is same for both
      vertical and horizontal directions. */
  void setAxes(int axes) { m_axesDist = -axes; }

  /** Set the minumum distance between axes. The default value is 100
      pixels. */
  void setAxDistance(int pixels) { m_axesDist = pixels; }

protected:
  void drawHoriz(MJ_Painter *, bool);
  /** Distance of the axes in pixels or number of axes. If this value
      is negative the number of indicates the number of axes to use.*/
  int m_axesDist;
  double m_xmin, m_xmax, m_ymin, m_ymax;
};

#endif
