/* COPYRIGHT
 *
 * This file is part of Mustajuuri LGPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2001.
 * Tommi.Ilmonen@hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. Extra clauses can be found from the file
 * MUSTAJUURI_LGPL and further explanations from the file
 * MUSTAJUURI_LICENSE. The license text can be found in file
 * "LGPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *    
 */

#include <mj_graphic_ellipse.h>

#include <mj_painter.h>

#include <qwidget.h>

#include <math.h>

#define MJ_2pi 6.2831853071795862

MJ_GraphicEllipse::MJ_GraphicEllipse(float left, float bottom, 
				     float right, float top, 
				     int segments)
{
  setEllipse(left, bottom, right, top, segments);
}

MJ_GraphicEllipse::MJ_GraphicEllipse()
{}

MJ_GraphicEllipse::~MJ_GraphicEllipse()
{}

void MJ_GraphicEllipse::setEllipse(float left, float bottom, 
				   float right, float top, 
				   int segments)
{
  m_rect[0] = left;
  m_rect[1] = bottom;
  m_rect[2] = right;
  m_rect[3] = top;

  const float centerX = (left + right) * 0.5;
  const float centerY = (bottom + top) * 0.5;
  const float radX = right - centerX;
  const float radY = top - centerY; 

  float x, y, angle;

  m_points.resize(segments + 1);
  
  double scale = MJ_2pi / (double) (segments - 1);

  for(int i=0; i < segments; i++) {
    angle = (float) ((double) i * scale);
    x = centerX + cosf(angle) * radX;
    y = centerY + sinf(angle) * radY;
    m_points.setPoint(i, x, y);
  }
  m_points.setPoint(segments, m_points.getPoint(0));
}
