
/* COPYRIGHT
 *
 * This file is part of Mustajuuri LGPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2001.
 * Tommi.Ilmonen@hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. Extra clauses can be found from the file
 * MUSTAJUURI_LGPL and further explanations from the file
 * MUSTAJUURI_LICENSE. The license text can be found in file
 * "LGPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *    
 */


#ifndef MJ_GRAPHICS_AREA_H
#define MJ_GRAPHICS_AREA_H

#include <qwidget.h>
#include <qptrlist.h>

#include <mj_graphic.h>

/** A graphics drawing area. This widget was created for with
    scientific visualization in mind. 
    
    MJ_GraphicsArea is widget that can contain graphics objects -
    lines, polygons, axes etc. The coordinates are expressed as
    flaoting point numbers so one can scale the view arbitrarily.

    It may well be useful for other purposes. There is a plan to make
    this a lightweight mini-canvas (proper canvas being too heavy to
    implement).

    @see MJ_Painter
*/

class MJ_GraphicsArea : public QWidget
{
  Q_OBJECT
public:
  MJ_GraphicsArea(QWidget *parent = 0,
		  const char * name = 0, 
		  bool autoDeleteChildren = true);

  virtual ~MJ_GraphicsArea();

  int appendGraphic(MJ_Graphic *);

  bool removeGraphic(int id, bool del = true);

  bool removeGraphicIndex(int index, bool del = true);

  void setBoundingBox(float leftB, float topB, float rightB, float bottomB);

  void setSquarePixels(bool b) { m_squarePixels = b; }
  bool squarePixels() const { return m_squarePixels; }
  

  inline float leftBorder() const { return m_bbox[0]; }
  inline float rightBorder() const { return m_bbox[2]; }

  /// Redraws the graphics area.
  void redraw(bool doubleBuffer = false);

  void drawAll();

  void drawGraphic(MJ_Graphic *);
  
  void undrawGraphic(MJ_Graphic *);
  
  /// Returns nth graphics object
  MJ_Graphic *at(uint n) { return m_graphics.at(n); }

  void clear()
  { m_graphics.clear(); }

signals:

  void mousePress(QMouseEvent *, float, float);
  void mouseRelease(QMouseEvent *, float, float);
  void mouseDoubleClick(QMouseEvent *, float, float);
  void mouseMove(QMouseEvent *, float, float);

protected:

  virtual void mousePressEvent(QMouseEvent *);  
  virtual void mouseReleaseEvent(QMouseEvent *);
  virtual void mouseDoubleClickEvent(QMouseEvent *);
  virtual void mouseMoveEvent(QMouseEvent *);

  virtual void paintEvent(QPaintEvent * ) ;

  int allocateId();
  void paintAll(MJ_Painter *);
  void makeTransform(MJ_Painter *);
  void unmapPoint(int x, int y, float *xf, float *yf);

  QList<MJ_Graphic> m_graphics;

  /** The bounding box covered by this widget. 

      Contents:
      
      <OL>
      <LI> Left border
      <LI> Top border
      <LI> Right border
      <LI> Bottom border
      </OL>
      
   */

  float             m_bbox[4];

  bool              m_squarePixels;
};

#endif
