/* COPYRIGHT
 *
 * This file is part of Mustajuuri LGPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2001.
 * Tommi.Ilmonen@hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. Extra clauses can be found from the file
 * MUSTAJUURI_LGPL and further explanations from the file
 * MUSTAJUURI_LICENSE. The license text can be found in file
 * "LGPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *    
 */

#ifndef MJ_GRAPHICS_CONTEXT_SIMPLE_H
#define MJ_GRAPHICS_CONTEXT_SIMPLE_H

#include <mj_graphics_context.h>

#include <qpen.h> 
#include <qbrush.h> 


/** Pen graphics context. */

class MJ_GraphicsContextPen : public MJ_GraphicsContext
{
public:
  MJ_GraphicsContextPen();
  MJ_GraphicsContextPen(const QPen &);

  virtual ~MJ_GraphicsContextPen();

  virtual void apply(QPainter *);
  virtual MJ_GraphicsContext * clone();

  QPen & pen() { return m_pen; }

protected:
  /// This variable holds the full graphics context.
  QPen m_pen;
};

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

/** Pen and brush graphics context. */

class MJ_GraphicsContextPenBrush : public MJ_GraphicsContext
{
public:
  MJ_GraphicsContextPenBrush();
  MJ_GraphicsContextPenBrush(const QPen &, const QBrush &);

  virtual ~MJ_GraphicsContextPenBrush();

  virtual void apply(QPainter *);
  virtual MJ_GraphicsContext * clone();

  QPen & pen() { return m_pen; }
  QBrush & brush() { return m_brush; }

  void set(const QPen &p, const QBrush &b)
    {m_pen = p;m_brush = b;}
  
protected:
  /// These variables holds the full graphics context.
  QPen   m_pen;
  QBrush m_brush;
};

#endif
