/* COPYRIGHT
 *
 * This file is part of Mustajuuri LGPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2001.
 * Tommi.Ilmonen@hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. Extra clauses can be found from the file
 * MUSTAJUURI_LGPL and further explanations from the file
 * MUSTAJUURI_LICENSE. The license text can be found in file
 * "LGPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *    
 */

#include <mj_gui_numeric.h>

#include <mj_dial.h>
#include <mj_gui_utils.h>
#include <mj_line_edit.h>
#include <mj_slider.h>
#include <mj_error.h>

#include <qcombobox.h> 
#include <qlayout.h> 
#include <qlabel.h> 
#include <qscrollview.h> 



/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

MJ_GuiDouble::MJ_GuiDouble()
{}

MJ_GuiDouble::~MJ_GuiDouble()
{}

void MJ_GuiDouble::setRange(double, double)
{}


/** Emits a signal to that tells that the current value is final. */
void MJ_GuiDouble::done()
{
  if(!m_id)
    emit emitDone();
  else
    emit emitDone2(m_id);
}


/** Used by child classes to emit new value. Returns the updated
    value.*/

double MJ_GuiDouble::emitInternal(double d)
{
  double tmp = 0.0;
  if(!m_id) {
    emit emitValue(d);
    emit emitGetValue(&tmp);
  }
  else {
    emit emitValue2(m_id, d);
    emit emitGetValue2(m_id, &tmp);
  }
  return tmp;
}

/** Emits a signal to get the current parameter value. */
void MJ_GuiDouble::emitGet(double *d)
{
  if(!m_id) 
    emit emitGetValue(d);
  else
    emit emitGetValue2(m_id, d);
}


/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

MJ_GuiDoubleCombo::MJ_GuiDoubleCombo()
  : m_label(0),
    m_combo(0)
{
  m_vals.setAutoDelete(true);
}

MJ_GuiDoubleCombo::MJ_GuiDoubleCombo(QObject *parent)
  : MJ_GuiDouble(parent),
    m_label(0),
    m_combo(0)
{
  m_vals.setAutoDelete(true);
}

MJ_GuiDoubleCombo::~MJ_GuiDoubleCombo()
{}

void MJ_GuiDoubleCombo::setValue(double d)
{
  checkReady();

  int index = 0;

  bool b = m_combo->signalsBlocked();
  m_combo->blockSignals(true);
  
  for(uint i=0; i < m_vals.count(); i++) {
    if(*m_vals.at(i) == d) {
      index = i;
      i = m_vals.count();
    }
  }
  m_combo->setCurrentItem(index);
  m_combo->blockSignals(b);
}


void MJ_GuiDoubleCombo::setEnabled(bool ena)
{
  if(m_combo) m_combo->setEnabled(ena);
}

/** Creates objects relative to a parent. Also connects the widgets'
    signals to this object. Returns false if operations fail (may
    happe when out of memory etc.). */
bool MJ_GuiDoubleCombo::createObjs(QWidget *parent)
{
  m_label = new QLabel(parent);
  m_combo = new QComboBox(parent);

  if(!m_label || !m_combo) return false;

  for(uint i=0; i < m_vals.count(); i++) {
    QString s;
    s.setNum(*m_vals.at(i));
    m_combo->insertItem(s);
  }

  connect(m_combo, SIGNAL(activated(int)),
	  this, SLOT(newInd(int)));

  return true;
}


void MJ_GuiDoubleCombo::addChildren(QScrollView *s)
{
  if(m_label) s->addChild(m_label);
  if(m_combo) s->addChild(m_combo);
}

/** Limits the minimum sizes of the widgets to the minimum size
    implied by their "sizeHint". */
void MJ_GuiDoubleCombo::limMinSizes()
{
  checkReady();
  MJ_limMinSize(m_combo);
  MJ_limMinSize(m_label);
}

/** Deltes objects. */
void MJ_GuiDoubleCombo::deleteObjs()
{
  if(m_label) delete m_label; m_label = 0;
  if(m_combo) delete m_combo; m_combo = 0;
}

/** Puts the objects to a horizontal grid. First label, then the
    combo. If you need more detailed control over widgets' location,
    you must do it by hand.*/
void MJ_GuiDoubleCombo::toGridHorizontal(QGridLayout *g, int row, int firstCol)
{
  checkReady();
  g->addWidget(m_label, row, firstCol);
  g->addWidget(m_combo, row, firstCol + 1, AlignLeft);
}

/** Receives signals from the combo. */
void MJ_GuiDoubleCombo::newInd(int i)
{
  emitInternal(* m_vals.at(i));
  done();
}


void MJ_GuiDoubleCombo::show()
{
  if(m_label) m_label->show();
  if(m_combo) m_combo->show();
}

/** Check that the necessary child objects do exist. If the objects do
    not exist, fatal error is generated.*/

void MJ_GuiDoubleCombo::checkReady()
{
  if(!m_label || !m_combo) 
    qDebug("MJ_GuiDoubleCombo::checkReady NULL pointer(s)");
}

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

MJ_GuiDoubleSlider::MJ_GuiDoubleSlider()
  : m_label(0), m_slider(0), m_edit(0)
{}

MJ_GuiDoubleSlider::MJ_GuiDoubleSlider(QObject *parent)
  : MJ_GuiDouble(parent), m_label(0), m_slider(0), m_edit(0)
{}

MJ_GuiDoubleSlider::~MJ_GuiDoubleSlider()
{}

MJ_GuiDoubleSlider &MJ_GuiDoubleSlider::operator = (const MJ_GuiDoubleSlider &)
{
  qDebug("MJ_GuiDoubleSlider::operator = DO NOT CALL");
  return *this;
}

void MJ_GuiDoubleSlider::setValue(double d)
{
  // qDebug("MJ_GuiDoubleSlider::setValue %f", d);
  
  m_slider->setValueDouble(d);
  if(m_edit) m_edit->setValueDouble(d);
}

void MJ_GuiDoubleSlider::setEnabled(bool ena)
{
  if(m_slider) m_slider->setEnabled(ena);
  if(m_edit) m_edit->setEnabled(ena);
}



bool MJ_GuiDoubleSlider::createObjs(QWidget *parent)
{
  return createObjs(parent, true);
}

/** Creates and connects child widgets. */
bool MJ_GuiDoubleSlider::createObjs(QWidget *parent, bool withLineEdit)
{
  m_label  = new QLabel(parent);
  m_slider = new MJ_Slider(0,1,0,parent);
  if(withLineEdit) m_edit   = new MJ_LineEdit(parent);

  if(!m_label || !m_slider || (!m_edit && withLineEdit)) return false;
  
  connectObjs();
  return true;
}

void MJ_GuiDoubleSlider::addChildren(QScrollView *s)
{
  if(m_label) s->addChild(m_label);
  if(m_slider) s->addChild(m_slider);
  if(m_edit) s->addChild(m_edit);
}

/** Connects child widgets. */
void MJ_GuiDoubleSlider::connectObjs()
{
  checkReady();

  connect(m_slider, SIGNAL(valueChangedDouble(double)),
	  this, SLOT(transmitValue(double)));

  connect(m_slider, SIGNAL(emitDone()),
	  this, SLOT(done()));

  if(m_edit) {
    connect(m_slider, SIGNAL(valueChangedDouble(double)), 
	    m_edit, SLOT(setValue(double)));

    connect(m_edit, SIGNAL(emitValue(double)), 
	    m_slider, SLOT(setValueDouble(double)));
    
    connect(m_edit, SIGNAL(emitValue(double)), 
	    this, SLOT(transmitValue(double)));
    
    connect(m_edit, SIGNAL(emitGetValueDouble(double *)), 
	    this, SLOT(emitGet(double *)));
    
    connect(m_edit, SIGNAL(emitDone()),
	    this, SLOT(done()));
  }
}


/** Limits minimum size of the child objects to that implied by
    "sizeHint".*/
void MJ_GuiDoubleSlider::limMinSizes()
{
  checkReady();
  MJ_limMinSize(m_label);
  MJ_limMinSize(m_slider);
  if(m_edit) {
    MJ_limMinSize(m_edit);
    /* qDebug("MJ_GuiDoubleSlider::limMinSizes # mw = %d mh = %d", 
       m_edit->minimumWidth(), m_edit->minimumHeight());*/
  }
}

/** Deletes child objects. */
void MJ_GuiDoubleSlider::deleteObjs()
{
  if(m_label)  delete m_label;  m_label = 0;
  if(m_slider) delete m_slider; m_slider = 0;
  if(m_edit)   delete m_edit;   m_edit = 0;
}

/** Sets the range of the slider.  */
void MJ_GuiDoubleSlider::setRange(double minimum, double maximum)
{
  checkReady();
  m_slider->setRealRange(minimum, maximum);
}

/** Puts objects to grid. First label, then slider and then the text
    box.*/
void MJ_GuiDoubleSlider::toGridHorizontal
(QGridLayout *g, int row, int firstCol)
{
  checkReady();
  g->addWidget(m_label,  row, firstCol);

  if(m_edit) {
    g->addWidget(m_edit,   row, firstCol+2);
    g->addWidget(m_slider, row, firstCol+1);
  }
  else
    g->addMultiCellWidget(m_slider, row, row, firstCol+1, firstCol+2);
}

/** Shown all child objects. */
void MJ_GuiDoubleSlider::show()
{
  if(m_label) m_label->show();
  m_slider->show();
  if(m_edit) m_edit->show();
}

void MJ_GuiDoubleSlider::transmitValue(double d)
{
  emitInternal(d);
}

/** Checks all child objects exist. If they do not exist, fatal error
    is generated. */
void MJ_GuiDoubleSlider::checkReady()
{
  if(!m_label || !m_slider) 
    qDebug("MJ_GuiDoubleSlider::checkReady NULL pointer(s)");
}


/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////


MJ_GuiDoubleDial::MJ_GuiDoubleDial()
  : m_label(0),
    m_dial(0),
    m_display(0)
{}

MJ_GuiDoubleDial::~MJ_GuiDoubleDial()
{}

void MJ_GuiDoubleDial::setValue(double value)
{
  if(!m_dial) return;

  m_dial->setValueDouble(value);
  updateLabeltext();
}

void MJ_GuiDoubleDial::setEnabled(bool ena)
{
  if(m_dial) m_dial->setEnabled(ena);
  if(m_label) m_label->setEnabled(ena);
}

void MJ_GuiDoubleDial::limMinSizes()
{
  MJ_limMinSize(m_label);
  MJ_limMaxSize(m_label);
  m_dial->setMinimumSize(m_dial->minimumSizeHint());
  if(m_display) MJ_limMinSize(m_display);
}

void MJ_GuiDoubleDial::setRange(double minR, double maxR)
{
  m_dial->setRealRange(minR, maxR);
}

void MJ_GuiDoubleDial::deleteObjs()
{
  delete m_label;
  delete m_dial;
  delete m_display;

  m_label = 0;
  m_dial = 0;
  m_display = 0;
}

bool MJ_GuiDoubleDial::createObjs(QWidget *parent)
{
   return createObjs(parent, true);
}

/** Create and connect objects. 

    @arg parent This is the parent of the widgets.

    @arg withDisplay If this variable is true, then the numeric
    display will be created.
*/

bool MJ_GuiDoubleDial::createObjs(QWidget *parent, bool withDisplay)
{
  deleteObjs();

  m_label = new QLabel(parent);
  m_dial = new MJ_Dial(0.0, 1.0, 10.0, parent);

  if(withDisplay) m_display = new QLabel(parent);

  connect(m_dial, SIGNAL(valueChangedDouble(double)), 
	  this, SLOT(transmitValue(double)));

  connect(m_dial, SIGNAL(dialReleased()),
	  this, SLOT(done()));
  
  return true;
}

void MJ_GuiDoubleDial::toGridHorizontal(QGridLayout *g, int row, int firstCol)
{
  g->addWidget(m_label, row, firstCol);
  g->addWidget(m_dial, row, firstCol + 1, AlignLeft);
  if(m_display) g->addWidget(m_display, row, firstCol + 2, AlignRight);
}

void MJ_GuiDoubleDial::show()
{
  if(m_label)   m_label->show();
  if(m_dial)    m_dial->show();
  if(m_display) m_display->show();
}

void MJ_GuiDoubleDial::transmitValue(double d)
{
  emitInternal(d);
  updateLabeltext();
}

void MJ_GuiDoubleDial::updateLabeltext()
{
  if(!m_display) return;

  QString str;
  str.sprintf("%3.3f", m_dial->valueDouble());
  m_display->setText(str);
}
