/* COPYRIGHT
 *
 * This file is part of Mustajuuri LGPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2001.
 * Tommi.Ilmonen@hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. Extra clauses can be found from the file
 * MUSTAJUURI_LGPL and further explanations from the file
 * MUSTAJUURI_LICENSE. The license text can be found in file
 * "LGPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *    
 */

#ifndef MJ_GUI_STRING_H
#define MJ_GUI_STRING_H

#include <qfiledialog.h> // To get the enums...
#include <qobject.h>
 
class QComboBox;
class QLabel;
class QLineEdit;
class QGridLayout;
class QPushButton;
class QScrollView;

/** Packs a label, a text-entry box and optionally a browse-button. 
*/

class MJ_GuiString : public QObject
{
  Q_OBJECT
public:
  MJ_GuiString();
  MJ_GuiString(QObject * parent);
  ~MJ_GuiString();
   
  void deleteObjs();
 
  bool createObjs(QWidget *parent, 
		  const QString &label, 
		  const QString &text);

  void addChildren(QScrollView *);

  bool createBrowseButton(QWidget *parent,
			  const QString &browseText,
			  const QString &filter,
			  QFileDialog::Mode);
 
  void toGridHorizontal(QGridLayout *, 
			int row, 
			int firstCol, 
			int entryWidth = 1);
   
  void setId(const QCString &id) { m_id = id; }

  void show();
   
  /// Returns the label widget
  inline QLabel    * label() { return m_label; }
  /// Returns the line edit widget
  inline QLineEdit * lineEdit() { return m_lineEdit; }
 
public slots:
 
  void activate();
  void browse();
  void sendString(const QString &);
  void sendDir(const QString &);
signals:
 
  void textEmit(const QString &);
  void textEmit(const char *, const QString &);
protected:  
  QLabel      *m_label;
  QLineEdit   *m_lineEdit;

  // File browsing:
  QPushButton *m_button;
  QFileDialog *m_fdialog;
  QFileDialog::Mode m_fmode;
  QString      m_filter;

  QCString     m_id;
};

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
 
/** Packs a label and a combo box.
*/

class MJ_GuiStringSelector : public QObject
{
  Q_OBJECT
public:
  MJ_GuiStringSelector();
  ~MJ_GuiStringSelector();
   
  void deleteObjs();
 
  bool createObjs(QWidget *parent, 
		  const QString &label);

  void addChildren(QScrollView *);

  void toGridHorizontal(QGridLayout *, 
			int row, 
			int firstCol);
   
  void setId(const QCString &id) { m_id = id; }

  void show();
   
  /// Returns the label widget
  inline QLabel    *label() { return m_label; }
  inline QComboBox *box() { return m_box; }
 
public slots:
  void sendString(const QString &);

signals:
 
  void textEmit(const QString &);
  void textEmit(const char *, const QString &);

protected:  
  QLabel      *m_label;
  QComboBox   *m_box;

  QCString     m_id;
};

 
#endif
