/* COPYRIGHT
 *
 * This file is part of Mustajuuri LGPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2001.
 * Tommi.Ilmonen@hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. Extra clauses can be found from the file
 * MUSTAJUURI_LGPL and further explanations from the file
 * MUSTAJUURI_LICENSE. The license text can be found in file
 * "LGPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *    
 */

#include <mj_header_label.h>

MJ_HeaderLabel::MJ_HeaderLabel(QWidget      *parent,
			       const char   *name)
  : QLabel(parent, name),
    m_enaClicks(false)
{
  setAlignment(AlignLeft | AlignTop);
  m_defaultFont = font();
  
  QFont f = m_defaultFont;
  f.setBold(true);
  setFont(f);
}

MJ_HeaderLabel::~MJ_HeaderLabel()
{}

void MJ_HeaderLabel::enableClicking(bool b)
{
  if(b == m_enaClicks) return;
  m_enaClicks = b;

  if(b) {
    setCursor(pointingHandCursor);

    QPalette p = palette();
    QColor linkBlue(0,0,170);
    p.setColor(QColorGroup::Text, linkBlue);
    p.setColor(QColorGroup::Foreground, linkBlue);
    setPalette(p);  

    QFont f = m_defaultFont;

    // Change font to indicate this is a header.
    
    /* We try to change the font somewhat. Preferred changes:
       
       1) underline
       2) boldify
       2) italify
    */

    if(!f.underline()) f.setUnderline(true); 
    else if(!f.bold()) f.setBold(true);
    else f.setItalic(true);
    setFont(f);
    
  }
  else {
    QFont f = m_defaultFont;
    f.setBold(true);
    setFont(f);

    setCursor(arrowCursor);

    QPalette p = palette();
    p.setColor(QColorGroup::Text, QColor(0,0,0));
    setPalette(p);  
  }
}

void MJ_HeaderLabel::mousePressEvent(QMouseEvent *)
{
  // qDebug("MJ_HeaderLabel::mousePressEvent");
  if(m_enaClicks)
    emit pressed();
}
