/* COPYRIGHT
 *
 * This file is part of Mustajuuri LGPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2001.
 * Tommi.Ilmonen@hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. Extra clauses can be found from the file
 * MUSTAJUURI_LGPL and further explanations from the file
 * MUSTAJUURI_LICENSE. The license text can be found in file
 * "LGPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *    
 */

#ifndef MJ_INT_REMAP_H
#define MJ_INT_REMAP_H

#include <qobject.h>

/** Transform "void" signals to "int" signals. Objects of this type
are used to "add" a single integer data to an other call */
class MJ_IntRemap : public  QObject
{
  Q_OBJECT
public:
  MJ_IntRemap(QObject * parent, int i=0) : QObject(parent), m_id(i) {}
  MJ_IntRemap(int i=0) : m_id(i) {}
  virtual ~MJ_IntRemap();
  
  void setId(int i) { m_id = i; }
  int id() const { return m_id; }

public slots:
  /// emits "emitCall(m_id)"
  void call() { emit emitCall(m_id); }
  void callBool(bool b) { emit emitCallBool(m_id, b); }
  void callDouble(double d) { emit emitCallDouble(m_id, d); }
  void callDoublePtr(double *d) { emit emitCallDoublePtr(m_id, d); }

signals: 
  void emitCall(int); 
  void emitCallBool(int,bool); 
  void emitCallDouble(int,double); 
  void emitCallDoublePtr(int,double *); 

protected:
  int m_id;
};

#endif
