/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of Mustajuuri LGPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2001.
 * Tommi.Ilmonen@hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. Extra clauses can be found from the file
 * MUSTAJUURI_LGPL and further explanations from the file
 * MUSTAJUURI_LICENSE. The license text can be found in file
 * "LGPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *    
 */

#ifndef MJ_KNOB_H
#define MJ_KNOB_H

#include <qwidget.h>

class QApplication;

/** Knob widget. Incomplete, but somewhat functional. */

class MJ_Knob : public QWidget
{
  Q_OBJECT

public:
  MJ_Knob(QWidget      *parent=0, 
	  const char   *name=0);

  virtual ~MJ_Knob();

  void incrementStep();
  void decrementStep();

public slots:

  void setValue(double);

signals:
  void emitValue(double);

protected:

  virtual void mousePressEvent(QMouseEvent *);
  virtual void mouseMoveEvent(QMouseEvent *);
  virtual void paintEvent(QPaintEvent *);

  virtual QSize sizeHint () const;
  virtual QSize minimumSizeHint () const;

  void turnTowards();

  void redrawFast();

  void angleToVal();

  // Current value
  double m_val;
  // How big steps to make when people press the mid and right button.
  double m_stepSize;
  // Current and old angle.
  double m_angle, m_oldAngle;
  // Minimum and maximum values of output.
  double m_min, m_max;
  // Minimum and maximum angles.
  double m_minAngle, m_maxAngle;

  int m_centerX, m_centerY, m_diameter;  
};

#endif
