/* COPYRIGHT
 *
 * This file is part of Mustajuuri LGPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2001.
 * Tommi.Ilmonen@hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. Extra clauses can be found from the file
 * MUSTAJUURI_LGPL and further explanations from the file
 * MUSTAJUURI_LICENSE. The license text can be found in file
 * "LGPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *    
 */

#ifndef MJ_LED_H
#define MJ_LED_H

#include <qwidget.h>
#include <qptrlist.h>

/** Led light. 

    To use this:
    Create an MJ_Led object, set the shape, colors, size etc correctly.

    Start changing the colors.
*/
class MJ_Led : public QWidget
{
  Q_OBJECT

public:

  /// Posible shapes for the light
  enum shape {
    SHAPE_ELLIPSE,
    SHAPE_RECTANGLE,
    SHAPE_SIZEOF
  };

  MJ_Led(QWidget      *parent=0, 
	 const char   *name=0);
  virtual ~MJ_Led();

  void clearColors();

  /// Returns the index of this color.
  int addColor(const QColor &);

  /// Number of colors
  int colors();

  /// Returns the index of current color.
  inline int currentColor() const { return M_iColor; }

  /// Set the size of the active area.
  void setLedSize(const QSize &);
  /// Utility for calling {\em setLedSize(const QSize &)}.
  void setLedSize(int);

  /// Create black and green colors.
  void blackAndGreen();

  /** Create a pair of colors. 
      
      Example: colorPair(black, green);
   */
  void colorPair(const QColor &, const QColor &);

  /// Self test.
  static void test(QApplication *xpQA, int iArgc, char **cppArgv);
  

public slots:

  /// Choose a color for display
  void chooseColor(int);

  /// Choose one of the shapes in MJ_Led::shape
  void chooseShape(int);

  /** Increments the color index. 

      When called many times starts to circulate the colors. 
  */
  void incrementColor();

  /** Decrements the color index. 

      When called many times starts to circulate the colors. 
  */
  void decrementColor();

protected:
  void paintEvent(QPaintEvent *);
  void draw();

  /// Current color index
  int     M_iColor;
  /// Pointer to current color.
  QColor *M_xpColor;

  QSize   M_xSize;

  shape   M_xShape;

  /// Colors for the light area.
  QList<QColor> M_xColors;
};

#endif
