/* COPYRIGHT
 *
 * This file is part of Mustajuuri LGPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2001.
 * Tommi.Ilmonen@hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. Extra clauses can be found from the file
 * MUSTAJUURI_LGPL and further explanations from the file
 * MUSTAJUURI_LICENSE. The license text can be found in file
 * "LGPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *    
 */
/* 
   Copyright (C) 1997--1999 Tommi Ilmonen
   This file is part of the Roaring Frog.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. */


#include <qlayout.h> 

#include <mj_ledrow.h>

MJ_LedRow::MJ_LedRow(int          iLeds,
		     bool         bHorizontal,
		     QWidget      *xpParent, 
		     const char   *cpName)
  : QWidget(xpParent, cpName)
{
  createLeds(iLeds, bHorizontal);
  activeTo(0);
}

MJ_LedRow::~MJ_LedRow()
{}

void MJ_LedRow::setLedSize(const QSize &xS)
{
  QListIterator<MJ_Led> xIter(M_xLeds);

  while(xIter.current()) {
    xIter.current()->setLedSize(xS);
    ++xIter;
  }
}

void MJ_LedRow::chooseLedShape(int iShape)
{
  QListIterator<MJ_Led> xIter(M_xLeds);

  while(xIter.current()) {
    xIter.current()->chooseShape(iShape);
    ++xIter;
  }
}

void MJ_LedRow::activeTo(int iFirstInactive)
{
  QListIterator<MJ_Led> xIter(M_xLeds);

  for(uint i=0; i < M_xLeds.count(); i++) {
    int iColor = (uint) iFirstInactive > i;
    xIter.current()->chooseColor(iColor);
    ++xIter;
  }
}

void MJ_LedRow::createLeds(int iLeds, bool bHorizontal)
{
  QBoxLayout * xpBox;
  if(bHorizontal) xpBox = new QHBoxLayout(this, 0, 0);
  else xpBox = new QVBoxLayout(this, 1, 5);
  
  for(int i=0; i < iLeds; i++) {
    MJ_Led *xpLed = new MJ_Led(this);
    xpLed->blackAndGreen();
    xpBox->addWidget(xpLed, 5);
    M_xLeds.append(xpLed);
  }
  xpBox->addStretch(50);  // Squeeze the leds to a group.
  xpBox->activate();
}
