/* COPYRIGHT
 *
 * This file is part of Mustajuuri LGPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2001.
 * Tommi.Ilmonen@hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. Extra clauses can be found from the file
 * MUSTAJUURI_LGPL and further explanations from the file
 * MUSTAJUURI_LICENSE. The license text can be found in file
 * "LGPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *    
 */

#include <stdio.h>
// #include <iostream.h>

#include <mj_line_edit.h>
#include <mj_error.h> 

#include <qfont.h>

static long L_lLineEdits = 0;

MJ_LineEdit::MJ_LineEdit(QWidget      *xpParent, 
		     const char   *cpName)
  : QLineEdit(xpParent, cpName),
    M_iPrecision(3),
    M_bIsDouble(true)
{
  L_lLineEdits++;
  connect(this, SIGNAL(returnPressed()),
	  this, SLOT(exportValue()));
  
}

MJ_LineEdit::~MJ_LineEdit()
{
  L_lLineEdits--;
  if(L_lLineEdits < 0) 
    qDebug("MJ_LineEdit::~MJ_LineEdit Too many deletions");
}

void MJ_LineEdit::setResolution(int iReso)
{
  M_iPrecision = iReso;
  setValue(M_dVal);
}

QSize MJ_LineEdit::sizeHint () const
{
  QSize s1 = QLineEdit::sizeHint();
  // QFont f = font();
  // int needPixels = f.pixelSize() * 4 + 10;
  return QSize(s1.width() / 2, s1.height());
}

void MJ_LineEdit::setValue(double dVal)
{
  if(M_bIsDouble) setValueDouble(dVal);
  else setValueInt((int) dVal);
}

void MJ_LineEdit::setValueDouble(double dVal)
{
  M_dVal = dVal;

  const int iBufSiz = 128;
  char caBuf1[iBufSiz];
  char caBuf2[iBufSiz];
  
  sprintf(caBuf1, "%%.%df", M_iPrecision);
  sprintf(caBuf2, caBuf1, M_dVal);

  setText(caBuf2);
}

void MJ_LineEdit::setValueInt(int lVal)
{
  M_dVal = (double) lVal;

  QString xStr;
  xStr.setNum(lVal);
  setText(xStr);
}

void MJ_LineEdit::exportValue()
{
  if(M_bIsDouble) exportValueDouble();
  else exportValueInt();
}

void MJ_LineEdit::exportValueDouble()
{
  double dVal = 0.0;

  QString xStr(text());
  bool bOk = true;

  dVal = xStr.toDouble(&bOk);

  if(bOk) {
    emit emitValue(dVal);
    // We assume the recipient has new value if it is valid...

    double dCurrentVal = dVal;
    emit emitGetValueDouble(&dCurrentVal);
    if(dCurrentVal != dVal)
      setValueDouble(dCurrentVal);

    emit emitDone();
  }
  else {
    qDebug("MJ_LineEdit::exportValueDouble Bad string in GUI");

    emit emitGetValueDouble(&dVal);
    setValueDouble(dVal);
  }
}

void MJ_LineEdit::exportValueInt()
{
  int lVal = 0;

  QString xStr(text());
  bool bOk = true;

  lVal = xStr.toInt(&bOk);

  if(bOk) {
    emit emitValueInt(lVal);
    emit emitValueInt(lVal);
    // We assume the recipient has new value if it is valid...

    int lCurrentVal;
    emit emitGetValueInt(&lCurrentVal);
    if(lCurrentVal != lVal)
      setValueInt(lCurrentVal);

    emit emitDone();
  }
  else {
    qDebug("MJ_LineEdit::emitNow Bad string in GUI");

    emit emitGetValueInt(&lVal);

    setValueInt(lVal);
  }
}

void MJ_LineEdit::setPrecision(int iPrec)
{
  if(iPrec < 0)
    qDebug("MJ_LineEdit::setPrecision Arg MUST be positive");
  else
    M_iPrecision = iPrec;
}

