/* COPYRIGHT
 *
 * This file is part of Mustajuuri LGPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2001.
 * Tommi.Ilmonen@hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. Extra clauses can be found from the file
 * MUSTAJUURI_LGPL and further explanations from the file
 * MUSTAJUURI_LICENSE. The license text can be found in file
 * "LGPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *    
 */


#ifndef MJ_LINE_EDIT_H
#define MJ_LINE_EDIT_H

#include <qlineedit.h> 

/** A line edit widget with numeric content. */
class MJ_LineEdit : public QLineEdit
{
  Q_OBJECT
public:

  MJ_LineEdit(QWidget      *xpParent=0, 
	    const char   *cpName=0);

  virtual ~MJ_LineEdit();
  
  /** Select either double or int to used. Double is the default. */
  void useDouble(bool bIsDouble) { M_bIsDouble = bIsDouble; }

  /** For double type, set resolution. */
  void setResolution(int iReso);

  virtual QSize sizeHint () const;

public slots:

  /** Set the value of the entry. */
  void setValue(double);
  void setValueDouble(double);
  void setValueInt(int);

  /** Export the current value of entry.
      
      After new value is exported a new value is obtained using
      "emitGetValue". It is assumed someone can proivde a new value.
      If the value emitted was correct this should do nothing.
      If the old value was not correct someone should have corrected it. 
   */

  void exportValue();
  void exportValueDouble();
  void exportValueInt();

  /** Set precision to use when putting values to the number box. */
  void setPrecision(int);

signals:

  void emitValue(double);
  void emitGetValueDouble(double *);

  void emitValueInt(int);
  void emitGetValueInt(int *);

  /** Emitted after the value has been successfully delivered. */
  void emitDone();

protected:

  int    M_iPrecision;
  bool   M_bIsDouble;
  double M_dVal;
};

#endif
