/* COPYRIGHT
 *
 * This file is part of Mustajuuri LGPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2001.
 * Tommi.Ilmonen@hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. Extra clauses can be found from the file
 * MUSTAJUURI_LGPL and further explanations from the file
 * MUSTAJUURI_LICENSE. The license text can be found in file
 * "LGPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *    
 */

#include <mj_mapper.h>

#include <mj_dom.h>
#include <mj_math.h>


MJ_Mapper * MJ_Mapper::create(const char *id)
{
  if(strcmp(id, "MJ_MapperLog") == 0)
    return new MJ_MapperLog;

  if(strcmp(id, "MJ_MapperPow") == 0)
    return new MJ_MapperPow;

  if(strcmp(id, "MJ_MapperMultiplyLimit") == 0)
    return new MJ_MapperMultiplyLimit;

  return 0;
}


/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

double MJ_MapperLog::map(double input)
{
  return log(input);
}

double MJ_MapperLog::unmap(double input)
{
  return exp(input);
}

MJ_Mapper * MJ_MapperLog::duplicate()
{
  return new MJ_MapperLog;
}

QDomElement MJ_MapperLog::writeDom(QDomDocument &doc)
{
  QDomElement elem = doc.createElement("MJ_MapperLog");
  return elem;
}

bool MJ_MapperLog::readDom(QDomElement &elem)
{
  return elem.tagName() == "MJ_MapperLog";
}

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////


double MJ_MapperPow::map(double input)
{
  return pow(input, 1.0 / m_power);
}

double MJ_MapperPow::unmap(double input)
{
  return pow(input, m_power);
}

MJ_Mapper * MJ_MapperPow::duplicate()
{
  return new MJ_MapperPow();
}

QDomElement MJ_MapperPow::writeDom(QDomDocument &doc)
{
  QDomElement elem = doc.createElement("MJ_MapperPow");

  MJ_addTextChild(elem, doc, "power", m_power);

  return elem;
}
 
bool MJ_MapperPow::readDom(QDomElement &elem)
{
  if(elem.tagName() != "MJ_MapperPow")
    return false;

  m_power = MJ_getDouble(elem, "power", m_power);

  return true;
}

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

MJ_MapperMultiplyLimit::~MJ_MapperMultiplyLimit()
{}

double MJ_MapperMultiplyLimit::map(double input)
{
  double tmp = input * m_scale;
  return MJ_clamp(tmp, m_min, m_max);
}

double MJ_MapperMultiplyLimit::unmap(double input)
{
  input = MJ_clamp(input, m_min, m_max);
  return input / m_scale;
}

MJ_Mapper * MJ_MapperMultiplyLimit::duplicate()
{
  return new MJ_MapperMultiplyLimit(*this);
}

QDomElement MJ_MapperMultiplyLimit::writeDom(QDomDocument &doc)
{
  QDomElement elem = doc.createElement("MJ_MapperMultiplyLimit");

  MJ_addTextChild(elem, doc, "scale", m_scale);
  MJ_addTextChild(elem, doc, "min", m_min);
  MJ_addTextChild(elem, doc, "max", m_max);

  return elem;
}

bool MJ_MapperMultiplyLimit::readDom(QDomElement &elem)
{
  if(elem.tagName() != "MJ_MapperMultiplyLimit")
    return false;

  m_scale = MJ_getDouble(elem, "scale", 1.0);
  m_min = MJ_getDouble(elem, "min", -1.0e+10);
  m_max = MJ_getDouble(elem, "max",  1.0e+10);
  
  return true;
}
