/* COPYRIGHT
 *
 * This file is part of Mustajuuri LGPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2001.
 * Tommi.Ilmonen@hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. Extra clauses can be found from the file
 * MUSTAJUURI_LGPL and further explanations from the file
 * MUSTAJUURI_LICENSE. The license text can be found in file
 * "LGPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *    
 */

#ifndef MJ_MAPPER_H
#define MJ_MAPPER_H

#include <mj_config.h>

#include <qdom.h>

/** Base class for one-dimensional to/from mappers. */

class MJ_Mapper
{
public:
  MJ_Mapper() {}
  virtual ~MJ_Mapper() {}

  virtual double map(double input) = 0;
  virtual double unmap(double input) = 0;

  virtual MJ_Mapper * duplicate() = 0;

  MJ_Mapper * clone() { return duplicate(); }

  virtual QDomElement writeDom(QDomDocument &) = 0;
  virtual bool readDom(QDomElement &elem) = 0;

  static MJ_Mapper * create(const char *);

};

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

/// Logarithmic mapper
class MJ_MapperLog : public MJ_Mapper
{
public:
  MJ_MapperLog() {}
  virtual ~MJ_MapperLog() {}

  virtual double map(double input);
  virtual double unmap(double input);
  virtual MJ_Mapper * duplicate();

  virtual QDomElement writeDom(QDomDocument &);
  virtual bool readDom(QDomElement &elem);

};

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

/// Power of something mapper
class MJ_MapperPow : public MJ_Mapper
{
public:
  MJ_MapperPow() : m_power(1.5) {}
  virtual ~MJ_MapperPow() {}

  virtual double map(double input);
  virtual double unmap(double input);
  virtual MJ_Mapper * duplicate();

  virtual QDomElement writeDom(QDomDocument &);
  virtual bool readDom(QDomElement &elem);

protected:
  double m_power;

};

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

/// Multiply and limit the value
class MJ_MapperMultiplyLimit : public MJ_Mapper
{
public:
  MJ_MapperMultiplyLimit(double scale = 1.0,
			 double minval = -1.0e+10, 
			 double maxval =  1.0e+10) 
    : m_scale(scale), m_min(minval), m_max(maxval) {}
  virtual ~MJ_MapperMultiplyLimit();

  virtual double map(double input);
  virtual double unmap(double input);
  virtual MJ_Mapper * duplicate();

  virtual QDomElement writeDom(QDomDocument &);
  virtual bool readDom(QDomElement &elem);

protected:
  double m_scale, m_min, m_max;

};


#endif
