/* COPYRIGHT
 *
 * This file is part of Mustajuuri LGPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2001.
 * Tommi.Ilmonen@hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. Extra clauses can be found from the file
 * MUSTAJUURI_LGPL and further explanations from the file
 * MUSTAJUURI_LICENSE. The license text can be found in file
 * "LGPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *    
 */

#include <mj_multi_info_gui.h>

#include <qlabel.h>
#include <qlayout.h>
#include <qpushbutton.h> 
#include <qscrollview.h> 

MJ_MultiInfoGui::MJ_MultiInfoGui(const QString &buttonText, 
				 const QString &topLabel, 
				 QWidget *parent, 
				 const char * name)
  : QWidget(parent, 
	    name, 
	    parent ? 0 : WDestructiveClose),
    m_elems(0)
{
  m_mainLayout = new QVBoxLayout(this, 5, 5);

  if(!topLabel.isEmpty()) {
    QLabel *label = new QLabel(topLabel, this);

    QFont font = label->font();
    font.setBold(true);

    label->setFont(font);

    m_mainLayout->addWidget(label);
  }

  m_scrollView = new QScrollView(this);
  m_mainLayout->addWidget(m_scrollView);  

  m_childParent2 = new QWidget(m_scrollView);
  m_scrollView->addChild(m_childParent2);
  m_infoLayout = new QVBoxLayout(m_childParent2, 5, 5);
  
  QPushButton *b = new QPushButton(buttonText, this);
  connect(b, SIGNAL(pressed()), this, SLOT(close()));
  m_mainLayout->addWidget(b, 5, AlignHCenter);

  // qDebug("MJ_MultiInfoGui::MJ_MultiInfoGui # %p", this);
}

MJ_MultiInfoGui::~MJ_MultiInfoGui()
{
  // qDebug("MJ_MultiInfoGui::~MJ_MultiInfoGui # %p", this);
}



/** Adds one information field to the display. */
void MJ_MultiInfoGui::addInfo(const QString &str)
{
  m_elems++;

  // qDebug("MJ_MultiInfoGui::addInfo %d", m_elems);

  // if(m_elems == 2) makeScrolled();

  // QLabel *label = new QLabel(str, m_scrollView->viewport());
  // m_scrollView->addChild(label);
  QLabel *label = new QLabel(str, m_childParent2);
  label->setAlignment(label->alignment() | WordBreak);
  label->setFrameStyle( QFrame::Box | QFrame::Raised );
  m_infoLayout->addWidget(label, 5);
  label->show();
  checkSizes();
  // m_scrollView->resize();
}

void MJ_MultiInfoGui::hide()
{
  if(getWFlags() | WDestructiveClose)
    close();
}

void MJ_MultiInfoGui::resizeEvent(QResizeEvent * e)
{
  QWidget::resizeEvent(e);
  checkSizes();
}

void MJ_MultiInfoGui::checkSizes()
{
  QSize s = m_childParent2->size();
  QSize sh = m_childParent2->sizeHint();
  
  // int mw = m_childParent2->minimumWidth();
  int ww = s.width();   // Widget width
  int hw = sh.width();  // Widget hinted width
  int vw = m_scrollView->visibleWidth();  // Visible width

  if(hw > 500) hw = 500;

  // qDebug("MJ_MultiInfoGui::checkSizes %d %d", ww, vw);

  if(ww < vw) {

    QSize newSize(vw, sh.height());

    m_scrollView->resizeContents(newSize.width(), newSize.height());
    m_childParent2->resize(newSize);
    return;
  }
  else if(ww > vw) {

    int newWidht = hw > vw ? hw : vw;

    // qDebug("MJ_MultiInfoGui::checkSizes (ww > vw) %d %d", hw, newWidht);
    QSize newSize(newWidht, sh.height());

    m_scrollView->resizeContents(newSize.width(), newSize.height());
    m_childParent2->resize(newSize);
    
  }

}
