/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of Mustajuuri LGPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2001.
 * Tommi.Ilmonen@hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. Extra clauses can be found from the file
 * MUSTAJUURI_LGPL and further explanations from the file
 * MUSTAJUURI_LICENSE. The license text can be found in file
 * "LGPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *    
 */

#ifndef MJ_MULTI_INFO_GUI_H
#define MJ_MULTI_INFO_GUI_H

#include <qwidget.h> 

class QScrollView;
class QVBoxLayout;

/** Widget that shows multiple warnings to the user.
    
    This class is intended for error reporting. It is useful when you
    want to display many (possibly similar) error reports.

    This widget shows a master label (top label), several information
    strings and a "Close"-button.
*/

class MJ_MultiInfoGui : public QWidget
{
public:
  MJ_MultiInfoGui(const QString &buttonText, 
		  /// The label that comes to the top
		  const QString &topLabel,
		  QWidget *parent = 0, 
		  const char * name = 0);

  virtual ~MJ_MultiInfoGui();

  void addInfo(const QString &str);
  
  /// Number of information fields.
  int elements() const { return m_elems; }

  virtual void hide();

protected:
  virtual void resizeEvent ( QResizeEvent * );

  void checkSizes();

  QScrollView *m_scrollView;
  QWidget     *m_childParent2;
  QVBoxLayout *m_infoLayout;
  QVBoxLayout *m_mainLayout;
  int m_elems;
};

#endif
