
#include <mj_painter.h>

MJ_Painter::MJ_Painter(const QPaintDevice *pd)
  : m_qpainter(pd)
{
  identityTransform();
}

MJ_Painter::~MJ_Painter()
{

}
  
/// Draws a polyline
void MJ_Painter::drawPolyline
(const MJ_PointArray2D &mpa, int index, int npoints)
{
  mpa.copyToQpa(& m_qpoints, transform(), index, npoints);

  if(npoints < 0) npoints = mpa.size();

  m_qpainter.drawPolyline(m_qpoints, 0, npoints);
}

/// Draws a polygon
void MJ_Painter::drawPolygon
(const MJ_PointArray2D &mpa, bool winding, int index, int npoints)
{
  mpa.copyToQpa(& m_qpoints, transform(), index, npoints);

  if(npoints < 0) npoints = mpa.size();

  m_qpainter.drawPolygon(m_qpoints, winding, 0, npoints);
}

/// Draws a line.
void MJ_Painter::drawLine(float x1, float y1, float x2, float y2)
{
  float xu, yu;

  applyTransform(x1, y1, &xu, &yu);
  int ix1((int) xu), iy1((int) yu);
  
  applyTransform(x2, y2, &xu, &yu);
  int ix2((int) xu), iy2((int) yu);
  
  m_qpainter.drawLine(ix1, iy1, ix2, iy2);
}

/** Makes the current transform an identity matrix. */
void MJ_Painter::identityTransform()
{
  for(int i=0; i < 3; i++) {
    for(int j=0; j < 3; j++) {
      int index = i * 3 + j;

      if(i == j) m_transform[index] = 1.0;
      else m_transform[index] = 0.0;
    }
  }
}

/** Scales the current transform in X- and Y-direction. */

void MJ_Painter::scale(float xscale, float yscale)
{
  getCoeff(0,0) *= xscale;
  getCoeff(1,1) *= yscale;
}

/** Makes a scaling transformation. The old transformation matrix is
    totally ignored. */

void MJ_Painter::scale2(float xscale, float yscale)
{
  identityTransform();
  scale(xscale, yscale);
}

/** Translates the current transform. */

void MJ_Painter::translate(float xtrans, float ytrans)
{
  getCoeff(0,2) += xtrans;
  getCoeff(1,2) += ytrans;
  
}

/** Makes a scaling and translation transformation. The old
    transformation matrix is totally ignored.  */

void MJ_Painter::scaleTranslate2
(float xscale, float yscale, float xtrans, float ytrans)
{
  scale2(xscale, yscale);
  translate(xtrans, ytrans);
}

/** Applies the current transformation to a given point. */
void MJ_Painter::applyTransform
(float xin, float yin, float * xout, float * yout)
{
  * xout = xin * getCoeff(0,0) + yin * getCoeff(0,1) + getCoeff(0,2);
  * yout = xin * getCoeff(1,0) + yin * getCoeff(1,1) + getCoeff(1,2);
}
