/* COPYRIGHT
 *
 * This file is part of Mustajuuri LGPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2001.
 * Tommi.Ilmonen@hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. Extra clauses can be found from the file
 * MUSTAJUURI_LGPL and further explanations from the file
 * MUSTAJUURI_LICENSE. The license text can be found in file
 * "LGPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *    
 */


#ifndef MJ_PAINTER_H
#define MJ_PAINTER_H

#include <mj_point_array.h>

#include <qpainter.h> 

/** Paint device for MJ_GraphicsArea. 

    This class has drawing methods that take floating point
    arguments. All argument point values are transformed with the
    current transformation matrix before being drawn.

    It uses QPainter to do the dirty work. The QPainter cannot be used
    directly too often since it takes integer arguments.  */

class MJ_Painter 
{
public:
  MJ_Painter(const QPaintDevice * );
  MJ_Painter() {}
  virtual ~MJ_Painter();
  
  /// Returns the actual QPainter that does the work.
  QPainter &qpainter() { return m_qpainter; }

  void drawPolyline(const MJ_PointArray2D &, int index = 0, int npoints = -1);

  void drawPolygon(const MJ_PointArray2D &, 
		   bool winding = false, 
		   int index = 0, 
		   int npoints = -1);

  void drawLine(float x1, float y1, float x2, float y2);

  /// Returns current transformation
  float * transform() { return (float *) m_transform; }

  void identityTransform();

  void scale(float xscale, float yscale);
  
  void scale2(float xscale, float yscale);

  void translate(float xtrans, float ytrans);

  /** Calculates the transformation matrix. */
  void scaleTranslate2(float xscale, float yscale, float xtrans, float ytrans);
 
  void applyTransform(float xin, float yin, float * xout, float * yout);

  /** Get transformation matrix coefficient. */
  float &getCoeff(uint row, uint col)
  { return m_transform[row*3 + col]; }

protected:

  QPainter     m_qpainter;
  
  /// Temporary storage for QPoints.
  QPointArray  m_qpoints;

  float m_transform[9];
};

#endif
