/* COPYRIGHT
 *
 * This file is part of Mustajuuri LGPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2001.
 * Tommi.Ilmonen@hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. Extra clauses can be found from the file
 * MUSTAJUURI_LGPL and further explanations from the file
 * MUSTAJUURI_LICENSE. The license text can be found in file
 * "LGPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *    
 */


#include <mj_point_array.h>

#include <qpointarray.h> 

void MJ_PointArray2D::resize(int s)
{ 
  uint oldSize = m_a.size();
  uint newSize = (uint) s * 2;

  m_a.resize(newSize); 

  for(uint i=oldSize; i < newSize; i++) 
    m_a[(int)i] = 0;
}

void MJ_PointArray2D::addYValues(const MJ_PointArray2D &add)
{
  if(add.size() != size()) {
    qDebug("MJ_PointArray2D::addYValues : size mismatch");
    return;
  }

  float *data1 = & m_a[0];
  const float *data2 = add.qarrayConst().data();

  uint ps = size();

  for(uint i=0; i < ps; i++)
    data1[i*2 + 1] += data2[i*2 + 1];
}

/* Copies values from this floating point array to Qt's integer point
   array. The points are transformed with given transformation
   matrix.
   
   If the target array is too small it is resized to be large enough.
*/

void MJ_PointArray2D::copyToQpa(QPointArray *target, float *transform, 
				int index, int npoints) const
{
  if(npoints < 0) npoints = size() - index;

  if(npoints <= 0) 
    return;

  int ts = target->size();

  if(ts < npoints) 
    target->resize(npoints);

  const float *points = & m_a[index];
  
  for(int i = 0; i < npoints; i++) {

    float pX = *points++;
    float pY = *points++;
    float x = transform[0] * pX + transform[1] * pY + transform[2];
    float y = transform[3] * pX + transform[4] * pY + transform[5];

    target->setPoint(i, (int) x, (int) y);
  }
}

void MJ_PointArray2D::mirrorY()
{
  float *data = m_a.data();

  int s = size();

  for(int i=0; i < s; i++) {
    float & tmp = data[i*2 + 1];
    tmp = - tmp;
  }
}
