/* COPYRIGHT
 *
 * This file is part of Mustajuuri LGPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2001.
 * Tommi.Ilmonen@hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. Extra clauses can be found from the file
 * MUSTAJUURI_LGPL and further explanations from the file
 * MUSTAJUURI_LICENSE. The license text can be found in file
 * "LGPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *    
 */

#ifndef MJ_POINT_ARRAY_H
#define MJ_POINT_ARRAY_H

#include <qmemarray.h> 

class QPointArray;


/** Array of 2D points.*/
class MJ_PointArray2D 
{
public:

  void resize(int s); 
  
  inline int size() const { return m_a.size() / 2; }

  void setPoint(uint p, float x, float y)
    { float *pa = & m_a[(int)p*2]; pa[0] = x; pa[1] = y; }

  void setPointY(uint p, float y)
    { m_a[(int)p*2 + 1] = y; }

  void setPoint(uint p, float *ptr)
    { float *pa = & m_a[(int)p*2]; pa[0] = ptr[0]; pa[1] = ptr[1]; }

  /** Returns one point. The first float is the X-value and the second
      is the Y-value. */
  float * getPoint(uint i) { return & m_a[(int)i*2]; }
  
  /** Get pointer to the point array. */
  float * points() { return & m_a[0]; }

  float & getX(uint i) { return m_a[(int)i*2]; }
  float & getY(uint i) { return m_a[(int)i*2+1]; }

  void addYValues(const MJ_PointArray2D &);

  void copyToQpa(QPointArray *target, float *transform, 
		 int index = 0, int npoints = -1) const;

  void mirrorY();
  
  QMemArray<float> & qarray() { return m_a; }
  
  const QMemArray<float> & qarrayConst() const { return m_a; }

protected:
  QMemArray<float> m_a;
};

#endif
