/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of Mustajuuri LGPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2001.
 * Tommi.Ilmonen@hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. Extra clauses can be found from the file
 * MUSTAJUURI_LGPL and further explanations from the file
 * MUSTAJUURI_LICENSE. The license text can be found in file
 * "LGPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *    
 */

#ifndef MJ_SLIDER_H
#define MJ_SLIDER_H

#include <qslider.h> 

class MJ_Mapper;

/** Slider with "double" output. The usual QSlider operates on integer
    values. This extension class has double value. */

class MJ_Slider : public QSlider
{
  Q_OBJECT
public:
  MJ_Slider(double dMinValue,
	    double dMaxValue,
	    double dValue,
	    QWidget      *xpParent=0, 
	    const char   *cpName=0,
	    Orientation  xOrientation = QSlider::Horizontal,   
	    int iMinValue = 0, 
	    int iMaxValue = 1000, 
	    int iPageStep = 10);

  virtual ~MJ_Slider();
  
  void setRealRange(double dMin, double dMax);

  void setSliderRange(int iMin, int iMax);
  
  void setMapper(MJ_Mapper *m);

public slots:

  void setValueDouble(double);
  void setValueInt(int);
  void getValueDouble(double *);

signals:

  void valueChangedDouble(double);

  void emitDone();

protected:

  void mouseReleaseEvent(QMouseEvent *);

  double M_dRealMin,   M_dRealMax;
  double M_dValue;
  MJ_Mapper  *m_mapper;
};

#endif
