/* COPYRIGHT
 *
 * This file is part of Mustajuuri GPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the General Public License (GPL),
 * versions 2. Extra clauses can be found from the file MUSTAJUURI_GPL
 * and further explanations from the file MUSTAJUURI_LICENSE. The
 * license text can be found in file "GPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *  
 */

#include <mj_app_action_auctrl.h>

#include <mj_app_actions.h>
#include <mj_error.h>
#include <mj_kernel.h>
#include <mj_module_auctrl.h>

bool MJ_AppActionAuCtrl::m_debug = false;

MJ_AppActionAuCtrl::MJ_AppActionAuCtrl(MJ_ModuleAuCtrl *control)
  : m_socketOpen(false),
    m_dataSocket(0),
    m_aucontrol(control)
{}

MJ_AppActionAuCtrl::~MJ_AppActionAuCtrl()
{
  MJ_trace2("MJ_AppActionAuCtrl::~MJ_AppActionAuCtrl # %p", this);
}

void MJ_AppActionAuCtrl::operate()
{
  const char * fname = "MJ_AppActionAuCtrl::operate";

  m_mutex.lock();

  // MJ_trace(fname, this);

  if(!m_socketOpen || !m_aucontrol || m_dataSocket) {
    m_mutex.unlock();
    return;
  }


  if(m_serverSocket.isPendingInput()) {
    MJ_trace3(m_debug, "%s # Accepting connection", fname);
    m_dataSocket = m_serverSocket.listen();
    MJ_trace3(m_debug, "%s # Got socket %p", fname, m_dataSocket);

    if(m_dataSocket) {
      m_aucontrol->setSocket(m_dataSocket);
      // Start DSP:

      // Find the top module:
      MJ_AppActions::startDsp(m_aucontrol);
    }
  }
  m_mutex.unlock();
}

void MJ_AppActionAuCtrl::newAddress(const QCString &s) 
{ 
  const char * fname = "MJ_AppActionAuCtrl::newAddress";

  if(s.isEmpty()) return;

  m_address = s.copy();

  m_serverSocket.close();

  m_socketOpen = m_serverSocket.open(m_address, true, true);

  if(!m_socketOpen) {
    MJ_error2(MJ_ERR_ARGUMENT, 
	      "%s # Could not open server socket to address \"%s\"",
	      fname, (const char *) m_address);
  }
  
  if(m_dataSocket) {
    MJ_trace(fname, "old data socket still active?");
  }

  m_dataSocket = 0;
}


void MJ_AppActionAuCtrl::dspStop() 
{ 
  const char * fname = "MJ_AppActionAuCtrl::dspStop";
  
  MJ_trace(fname);

  m_mutex.lock();
  m_dataSocket = 0; 
  m_mutex.unlock();

  MJ_trace(fname, "EXIT");
}
