/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of Mustajuuri GPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the General Public License (GPL),
 * versions 2. Extra clauses can be found from the file MUSTAJUURI_GPL
 * and further explanations from the file MUSTAJUURI_LICENSE. The
 * license text can be found in file "GPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *  
 */

#ifndef MJ_MODULE_AUCTRL_H
#define MJ_MODULE_AUCTRL_H

#include <mj_module_dsp.h>

#include <ac_buffer.h>

#include <di_condition.h>
#include <di_mutex.h>
#include <di_socket.h>
#include <di_thread.h>

class MJ_AppActionAuCtrl;
class MJ_ModuleVBAP;
class MJ_ModuleVBAP2;

class MJ_AcBufferBank
{
public:
  MJ_AcBufferBank();
  
  AC_Buffer *get(uint i) { return m_list.at(i); }
  
  uint inUse() const { return m_inUse; }

  AC_Buffer * getUseBuffer();

  void unuseLastBuffer() { m_inUse--; }

  void clearUseCount() { m_inUse = 0; }

protected:
  QList<AC_Buffer> m_list;
  uint m_inUse;
};

/** Auralization control plugin. This class looks a tiny bit like the
    one Lauri Savioja did in 2000. The differences are that this beast
    works with the MJ_ModuleSynth and MJ_ModuleVBAP modules. This one
    also uses threads and what-not to run the synthesis.

    This module works by 1) finding the auralization-related plugins
    that are in use and 2) by routing control messages to those plugins in
    real-time.

    @author Tommi Ilmonen
*/
class MJ_ModuleAuCtrl : public MJ_ModuleDsp, public DI_Thread
{
public:
  MJ_ModuleAuCtrl();

  virtual ~MJ_ModuleAuCtrl();
  
  virtual bool dspPrepare();
  virtual void dspProcessPeriod();
  virtual bool dspStop();

  virtual bool msgProcess(const char *, MJ_Message *);
  virtual MJ_Parameter * msgGetInput(const char *);
  virtual QCString msgGetInputName(int);

  void setSocket(DI_Socket *s);

  static bool *debugPtr() { return &m_debug; }

protected:

  void describeParam(char *buf, const char * name, int i);

  void playSample();
  void playInput();
  void noteOff();
  void changeSourceOrientation();
  void changeSourceLocation();
  void setNoteGain();
  void setNoteRelativePitch();
  void processMjMessage();

  // Thread stuff:
  void stopThread();
  virtual void childLoop();
  bool threadHandleMsg();

  AC_Buffer      *m_currentCmd;
  AC_BufferHeader m_header;
  bool            m_requestStop;

  DI_MutexAuto    m_mutex;
  DI_Condition    m_condition;

  MJ_ModuleVBAP  *m_vbap;
  MJ_ModuleVBAP2 *m_vbap2;
  MJ_ModuleDsp   *m_synth;

  MJ_AppActionAuCtrl *m_appAction;

  static const int AUCTRL_BUFFERS = 3;
  /** Buffer swap area. Three buffer are needed:  */
  MJ_AcBufferBank m_bufferBanks[AUCTRL_BUFFERS];

  uint m_dspBuffer, m_threadBuffer;

  // Thread-space:
  AC_Buffer *m_buffer;

  DI_Socket *m_socket;
  /// Socket address.
  QCString   m_address;
  QString    m_synthPluginId;
  bool       m_threadActive;
  bool       m_endThread;
  bool       m_endRequested;
  bool       m_collectingContiguous;

  static bool m_debug;
};

#endif


