/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of Mustajuuri GPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the General Public License (GPL),
 * versions 2. Extra clauses can be found from the file MUSTAJUURI_GPL
 * and further explanations from the file MUSTAJUURI_LICENSE. The
 * license text can be found in file "GPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *  
 */

#ifndef MJ_MODULE_DIVA_AF_H
#define MJ_MODULE_DIVA_AF_H

#include <mj_module_dsp.h>
#include <mj_module_dsp_gui_default.h>
#include <di_audio_file_threaded.h>

class QApplication;

class MJ_ModuleDivaAudioFile : public MJ_ModuleDsp
{
public:
  MJ_ModuleDivaAudioFile();
  virtual ~MJ_ModuleDivaAudioFile();

  virtual bool dspStop();
  
  virtual bool msgProcess(const char *,MJ_Message *);
  virtual MJ_Parameter *msgGetInput(const char *);
  virtual QCString msgGetInputName(int);

  void setFileName(const QString &name) { m_fileName = name.copy(); }

  static bool *debugPtr() { return &m_debug; }

protected:

  DI_AudioFileThreaded m_file; 

  MJ_Vector<MJ_Sample> m_buffer;

  QString m_fileName;
  
  static bool m_debug;
};

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

/** Audio file output module. This class is mostly based on
    MJ_ModuleDivaAudioFile. As a result it is very compact. */

class MJ_ModuleDivaAudioFileIn : public MJ_ModuleDivaAudioFile
{
public:
  MJ_ModuleDivaAudioFileIn();
  virtual ~MJ_ModuleDivaAudioFileIn();

  virtual bool dspAcceptIoChans(int *, int *);

  virtual bool dspPrepare();
  virtual void dspProcessPeriod();
  
  virtual bool msgProcess(const char *,MJ_Message *);
  virtual MJ_Parameter *msgGetInput(const char *);
  virtual QCString msgGetInputName(int);

protected:

  void stopProcessing();
  bool m_eofEnds;
  bool m_loop;
  long m_endFrame;

  static const char * m_eofEndsStr;
};

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

/** Audio file output module. This class is mostly based on
    MJ_ModuleDivaAudioFile. As a result it is very compact. */

class MJ_ModuleDivaAudioFileOut : public MJ_ModuleDivaAudioFileIn
{
public:
  MJ_ModuleDivaAudioFileOut();
  virtual ~MJ_ModuleDivaAudioFileOut();
  
  virtual bool dspAcceptIoChans(int *, int *);

  virtual bool dspPrepare();
  virtual void dspProcessPeriod();

  virtual bool msgProcess(const char *,MJ_Message *);
  virtual MJ_Parameter *msgGetInput(const char *);
  virtual QCString msgGetInputName(int);

protected:

  QString newWriteFile(const QString &name, bool *modified);

  DI_AudioSampleFormat m_sformat;
  DI_AudioFileFormat   m_format;

  bool                 m_overWriteFiles;
};

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

class MJ_ModuleDivaAudioFileOutGui : public MJ_ModuleDspGuiPostCard
{
  // Q_OBJECT
public:
  MJ_ModuleDivaAudioFileOutGui(MJ_RtControl *rtparent,
			       MJ_ModuleDsp *module,
			       QWidget      *parent = 0,
			       const char   *name = 0);
  
  virtual ~MJ_ModuleDivaAudioFileOutGui();

  static MJ_ModuleDspGui *createNew(MJ_RtControl *rtparent,
				    MJ_ModuleDsp *module,
				    QWidget *parent, 
				    const char *name);
  /// Overdrive from MJ_ModuleDspGuiPostCard
  virtual void moduleModified(const char *);

protected:
  
};

#endif
