/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of Mustajuuri GPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the General Public License (GPL),
 * versions 2. Extra clauses can be found from the file MUSTAJUURI_GPL
 * and further explanations from the file MUSTAJUURI_LICENSE. The
 * license text can be found in file "GPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *  
 */

#ifndef MJ_MODULE_DIVA_AF_REC_PLAY_H
#define MJ_MODULE_DIVA_AF_REC_PLAY_H

#include <mj_module_diva_af.h>

/** A module tha implements the (very usual) record/play/bypass
    functionality in HDR software. */

class MJ_ModuleDivaAudioFileRecPlay : public MJ_ModuleDivaAudioFileOut
{
public:
  enum Mode {
    BYPASS,
    RECORD,
    PLAY,
    MODES
  };

  MJ_ModuleDivaAudioFileRecPlay();
  virtual ~MJ_ModuleDivaAudioFileRecPlay();

  virtual bool dspAcceptIoChans(int *, int *);

  virtual bool dspPrepare();
  virtual void dspProcessPeriod();
  virtual bool dspStop();
  
  virtual bool msgProcess(const char *,MJ_Message *);
  virtual MJ_Parameter *msgGetInput(const char *);
  virtual QCString msgGetInputName(int);

  static int determineMode(const char *str);

protected:

  void bypass() 
  {
    for(uint i=0; i < m_signalInput.size(); i++)
      m_signalOutput.getFast(i).samples().copyFast
	(m_signalInput.getFast(i).samples());
  }

  Mode m_mode;

  static const char * m_modeIds[MODES];
  static const char * m_modeLabels[MODES];
};

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

class MJ_ModuleDivaAudioFileRecPlayGui : public MJ_ModuleDivaAudioFileOutGui
{
public:
  
  MJ_ModuleDivaAudioFileRecPlayGui(MJ_RtControl *rtparent,
				   MJ_ModuleDsp *module,
				   QWidget      *parent = 0,
				   const char   *name = 0);
  
  virtual ~MJ_ModuleDivaAudioFileRecPlayGui();

  static MJ_ModuleDspGui *createNew(MJ_RtControl *rtparent,
				    MJ_ModuleDsp *module,
				    QWidget *parent, 
				    const char *name);

  /// Overdrive from MJ_ModuleDivaAudioFileOutGui
  virtual void moduleModified(const char *);

protected:
  QPalette m_palettes[MJ_ModuleDivaAudioFileRecPlay::MODES];

};

#endif

