/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of Mustajuuri GPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the General Public License (GPL),
 * versions 2. Extra clauses can be found from the file MUSTAJUURI_GPL
 * and further explanations from the file MUSTAJUURI_LICENSE. The
 * license text can be found in file "GPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *  
 */

#ifndef MJ_MODULE_DIVA_AUDIO_H
#define MJ_MODULE_DIVA_AUDIO_H

#include <di_audio_port.h>

#include <mj_module_rtaudio.h>
#include <mj_messages.h>

/** Performs audio IO using the DIVA audio IO classes. 
    
    This class has #ifdefs to accommodate for the alsa-cvs-nov divaio
    wrappers. */

class MJ_ModuleDivaAudioIn : public MJ_ModuleRtAudio
{
public:
  // Suppress a few g++ warnings:
  friend class MJ_ModuleDivaAudioOut;

  MJ_ModuleDivaAudioIn(DI_IoMode mode);
  virtual ~MJ_ModuleDivaAudioIn();

  virtual int  bufferLength();

  /** Checks conflicts with other objects.
      
      Objecs of this class operate on limited resources. They share
      AUDIO ports. On many systems (Linux/OSS) a particular device can
      be opened only once. Therefore one needs to spdread the
      curresponding file id all over the app so all objects that want
      it can use it. 

      The system can use this method to make sure the objects do not
      conflict. Since AUDIO input and output are different objects
      they must be linked (one owns the AUDIO io port while the other
      simply uses it). This may appear to be a bit complex, but it
      makes sure all objects behave the way one expects them them to
      behave.
      
   */

  virtual bool conflicts(MJ_ModuleDsp *);

  virtual bool dspPrepare();
  virtual bool dspPrepareFast();
  virtual void dspProcessPeriod();
  virtual bool dspStop();

  virtual bool msgProcess(const char *, MJ_Message *);

  virtual MJ_Parameter *msgGetInput(const char *);
  virtual QCString msgGetInputName(int);
  
  static bool *debugPtr() { return & m_debug; }

protected:

  bool open();

  bool close();
  
  /** Distributes the IO object to other modules that need it. */
  void distributeObject();

  /** Unlinks the IO object from other modules. */
  void unlinkObject();

  uint chans() const;
  
  QCString             m_portName;
  
  DI_AudioPort        *m_audioIo;
  
  DI_IoMode            m_mode;
  DI_IoMode            m_totalMode;
  
  uint                 m_chans;
  uint                 m_totalChans;

  MJ_Array<MJ_Sample>  m_buffer;

  bool                 m_isOwner;

  /** Friend of the io object. */
  MJ_ModuleDivaAudioIn  *m_friend;
  MJ_ModuleDivaAudioIn  *m_owner;

#ifdef ALSA_IS_0_9
  int  m_inAvail;
  int  m_outAvail;
  uint m_outWritten;
  uint m_inRead;
#endif

  static bool          m_debug;
};

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////


class MJ_ModuleDivaAudioOut : public MJ_ModuleDivaAudioIn
{
public:
  MJ_ModuleDivaAudioOut(DI_IoMode mode)
    : MJ_ModuleDivaAudioIn(mode) {}

  virtual ~MJ_ModuleDivaAudioOut();

  virtual void dspProcessPeriod();
};

#endif
