/* COPYRIGHT
 *
 * This file is part of Mustajuuri GPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the General Public License (GPL),
 * versions 2. Extra clauses can be found from the file MUSTAJUURI_GPL
 * and further explanations from the file MUSTAJUURI_LICENSE. The
 * license text can be found in file "GPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *  
 */

#ifndef MJ_MODULE_DIVA_MIDI_H
#define MJ_MODULE_DIVA_MIDI_H

#include <mj_midi.h>
#include <di_midi_io.h>

#include <mj_module_dsp.h>
#include <mj_messages.h>

#include <poll.h>

/** Performs MIDI IO using the DIVA MIDI IO classes. 
    
 */

class MJ_ModuleDivaMidi : public MJ_ModuleDsp
{
public:
  MJ_ModuleDivaMidi(DI_IoMode mode);
  virtual ~MJ_ModuleDivaMidi();

  /** Checks conflicts with other objects.
      
      Objecs of this class operate on limited resources. They share
      MIDI ports. On many systems (Linux/OSS) a particular device can
      be opened only once. Therefore one needs to spdread the
      curresponding file id all over the app so all objects that want
      it can use it. 

      The system can use this method to make sure the objects do not
      conflict. Since MIDI input and output are different objects they
      must be linked (one owns the MIDI io port while the other simply
      uses it). This may appear bit complex, but it makes sure all
      objects behave the way one expects them them to behave.
      
   */

  virtual bool conflicts(MJ_ModuleDsp *);

  virtual bool dspPrepare();
  virtual bool dspPrepareFast();
  virtual void dspProcessPeriod();
  virtual bool dspStop();

  virtual bool msgProcess(const char *, MJ_Message *);

  virtual MJ_Parameter *msgGetInput(const char *);
  virtual QCString msgGetInputName(int);
  virtual MJ_Parameter *msgGetOutput(const char *);
  virtual QCString msgGetOutputName(int);

protected:

  bool open();

  bool close();
  
  /** Distributes the IO object to other modules that need it. */
  void distributeObject();
  /** Unlinks the IO object from other modules. */
  void unlinkObject();

  /** Makes this an owner object that others can point to. */
  void makeOwner();

  /** Links this object to another. */
  bool linkTo(MJ_ModuleDivaMidi *m);

  QCString             m_portName;

  DI_MidiIo           *m_midiIo;
  MJ_MessageMidiEvent  m_event;

  DI_IoMode            m_mode;
  DI_IoMode            m_totalMode;

  bool                 m_isOwner;
  /** If this is the object that owns the port, this list contains
      objects that link to the same midi IO object. */
  QList<MJ_ModuleDsp>  m_ptrs;
  /** Owner of the io object.  */
  MJ_ModuleDivaMidi   *m_owner;

  static bool          m_debug;
};


#endif
