/* COPYRIGHT
 *
 * This file is part of Mustajuuri GPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the General Public License (GPL),
 * versions 2. Extra clauses can be found from the file MUSTAJUURI_GPL
 * and further explanations from the file MUSTAJUURI_LICENSE. The
 * license text can be found in file "GPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *  
 */
#include <mj_module_file_input.h>
#include <mj_module_dsp_row.h>
#include <mj_module_level.h>

#include <mj_messages.h>

#include <qapplication.h>

#include <string.h>

bool MJ_ModuleFileRead::m_debug = false;

MJ_ModuleFileRead::MJ_ModuleFileRead()
  : m_fileName("test.wav"),
    m_file(0),
    m_processing(false)
{
  if(m_debug) trace("MJ_ModuleFileRead::MJ_ModuleFileRead");
  dspSetOutChannels(1);
}

MJ_ModuleFileRead::~MJ_ModuleFileRead()
{
  if(m_debug) trace("MJ_ModuleFileRead::~MJ_ModuleFileRead");
}

bool MJ_ModuleFileRead::dspPrepare()
{
  bool ok = open();
  if(!ok) return false;
  
  return true;
}

void MJ_ModuleFileRead::dspProcessPeriod()
{
  if(!m_file) return;

  long frames = (long) m_signalOutput[0].samples().size();
  
  long readFrames = m_file->readFrames(m_buffer.data(), frames);
  // long fillZero = frames - readFrames;
  
  for(uint i=0; i < m_signalOutput.size(); i++) {
    // Fill the signal portion
    m_signalOutput[i].samples().copySkipping
      (m_buffer.data(), 
       i,
       readFrames,
       m_signalOutput.size());
    // Fill the rest with zero (EOF).
    m_signalOutput[i].samples().setRange(0.0, readFrames, frames);
  }
}

bool MJ_ModuleFileRead::dspStop()
{
  close();
  return true;
}
  
bool MJ_ModuleFileRead::msgProcess(int i,MJ_Message *m)
{
  if(i == 0) {
    MJ_MessageString *ms = dynamic_cast<MJ_MessageString *> (m);

    if(!ms) return false;
    
    m_fileName = ms->stringConst().copy();
    return true;
  }

  return false;
}

void MJ_ModuleFileRead::test(QApplication *a, int iArgc, char **cppArgv)
{
  const char *filename = "test.wav";

  for(int i=1; i < iArgc; i++) {
    if((strcmp(cppArgv[i], "--file") == 0) && ((i+1) < iArgc))
      filename = cppArgv[++i];
  }

  MJ_ModuleFileRead *mfr = new MJ_ModuleFileRead;
  MJ_ModuleLevel  *ml = new MJ_ModuleLevel;

  mfr->setFileName(filename);
  mfr->dspSetOutChannels(1);

  MJ_ModuleDspRow dr;
  dr.appendModule(mfr);
  dr.appendModule(ml);
  
  ml->dspConnectInput(mfr->id(), 0, 0);

  MJ_ModuleDspRow_QG drg;
  drg.setTestMode(true);
  drg.setModule(&dr);

  a->setMainWidget(&drg);
  a->exec();
}

bool MJ_ModuleFileRead::open()
{
  const char * fname = "MJ_ModuleFileRead::open";

  if(!m_file.open(m_fileName, DI_IO_INPUT, DI_ASF_SAMPLE, 1000)) {
    error(MJ_ERR_INTERNAL, fname);
    return false;
  }
  
  int chans = m_file.getChannels();
  
  if((uint) chans != m_signalOutput.size() || !chans) {
    error(MJ_ERR_ARGUMENT, fname, "bad number of channels");
    return false;
  }

  m_buffer.resize(chans * m_signalOutput[0].samples().size());

  return true;
}

void MJ_ModuleFileRead::close()
{
  if(m_file) {
    m_file->close();
    delete m_file;
    m_file = 0;
  }
}
