/* COPYRIGHT
 *
 * This file is part of Mustajuuri GPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the General Public License (GPL),
 * versions 2. Extra clauses can be found from the file MUSTAJUURI_GPL
 * and further explanations from the file MUSTAJUURI_LICENSE. The
 * license text can be found in file "GPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *  
 */

#include <mj_module_diva_midi.h>
#include <mj_module_diva_af_rec_play.h>
#include <mj_module_diva_audio.h>

#include <mj_debug.h>
#include <mj_error.h>
#include <mj_dsp_plugin.h>
#include <mj_version.h>

#include <mj_divaio_device_support.h>

// Midi:

MJ_ModuleDsp *MJ_createNew_ModuleDivaMidiIn()
{
  return new MJ_ModuleDivaMidi(DI_IO_INPUT);
}

MJ_ModuleDsp *MJ_createNew_ModuleDivaMidiOut()
{
  return new MJ_ModuleDivaMidi(DI_IO_OUTPUT);
}

MJ_PluginDbeDsp *MJ_ModuleDivaMidiIn_Dbe()
{
  MJ_PluginDbeDsp *dbe = new MJ_PluginDbeDsp;

  dbe->setAuthors("Ilmonen Tommi S.");
  dbe->setMenuPosition("_Messages_MIDI_MIDI input");
  dbe->setCreator(MJ_createNew_ModuleDivaMidiIn);
  dbe->setIdentifier("_midi_diva_midi_input_1");
  dbe->pluginVersion()->set(0,0,1);
  dbe->options()->defaultChans(0);
  
  dbe->setParameter(MJ_PP_FUNCTIONALITY,
		    MJ_DspPluginFunctionalityKeys[MJ_DPF_MIDI_IN]);

  /** Notify the system that objects of this kind may conflict with
      others. */
  dbe->setMayConflict(true);

  return dbe;
}

MJ_PluginDbeDsp *MJ_ModuleDivaMidiOut_Dbe()
{
  MJ_PluginDbeDsp *dbe = new MJ_PluginDbeDsp;

  dbe->setAuthors("Ilmonen Tommi S.");
  dbe->setMenuPosition("_Messages_MIDI_MIDI output");
  dbe->setCreator(MJ_createNew_ModuleDivaMidiOut);
  dbe->setIdentifier("_midi_diva_midi_output_1");
  dbe->pluginVersion()->set(0,0,1);
  dbe->options()->defaultChans(0);
  
  dbe->setParameter(MJ_PP_FUNCTIONALITY,
		    MJ_DspPluginFunctionalityKeys[MJ_DPF_MIDI_OUT]);
  
  dbe->setMayConflict(true);

  return dbe;
}


/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

// Audio:

MJ_ModuleDsp *MJ_createNew_ModuleDivaAudioIn()
{
  return new MJ_ModuleDivaAudioIn(DI_IO_INPUT);
}

MJ_ModuleDsp *MJ_createNew_ModuleDivaAudioOut()
{
  return new MJ_ModuleDivaAudioOut(DI_IO_OUTPUT);
}

MJ_PluginDbeDsp *MJ_ModuleDivaAudioIn_Dbe()
{
  MJ_PluginDbeDsp *dbe = new MJ_PluginDbeDsp;

  dbe->setAuthors("Ilmonen Tommi S.");
  dbe->setMenuPosition("_Audio_input/output_Audio input");
  dbe->setCreator(MJ_createNew_ModuleDivaAudioIn);
  dbe->setIdentifier("_audio_diva_audio_input_1");
  dbe->pluginVersion()->set(0,0,1);
  dbe->options()->setChans(0,0,1,128);

  dbe->setParameter(MJ_PP_FUNCTIONALITY,
		    MJ_DspPluginFunctionalityKeys[MJ_DPF_AUDIO_IN]);

  /** Notify the system that objects of this kind may conflict with
      others. */
  dbe->setMayConflict(true);

  dbe->setPluginDebug(MJ_ModuleDivaAudioIn::debugPtr());
  return dbe;
}

MJ_PluginDbeDsp *MJ_ModuleDivaAudioOut_Dbe()
{
  MJ_PluginDbeDsp *dbe = new MJ_PluginDbeDsp;

  dbe->setAuthors("Ilmonen Tommi S.");
  dbe->setMenuPosition("_Audio_input/output_Audio output");
  dbe->setCreator(MJ_createNew_ModuleDivaAudioOut);
  dbe->setIdentifier("_audio_diva_audio_output_1");
  dbe->pluginVersion()->set(0,0,1);
  dbe->options()->setChans(1,128, 0, 0);

  dbe->setParameter(MJ_PP_FUNCTIONALITY,
		    MJ_DspPluginFunctionalityKeys[MJ_DPF_AUDIO_OUT]);

  /** Notify the system that objects of this kind may conflict with
      others. */
  dbe->setMayConflict(true);

  dbe->setPluginDebug(MJ_ModuleDivaAudioIn::debugPtr());
  return dbe;
}

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

MJ_ModuleDsp *MJ_createNew_ModuleDivaAudioFileIn()
{
  return new MJ_ModuleDivaAudioFileIn;
}

MJ_PluginDbeDsp *MJ_ModuleDivaAudioFileIn_Dbe()
{
  MJ_PluginDbeDsp *dbe = new MJ_PluginDbeDsp;

  dbe->setAuthors("Ilmonen Tommi S.");
  dbe->setMenuPosition("_Audio_File_Audio file play");
  dbe->setCreator(MJ_createNew_ModuleDivaAudioFileIn);
  dbe->setIdentifier("_file_audio_file_input1");
  dbe->pluginVersion()->set(0,0,1);
  dbe->options()->setChans(0,0,1,64);
  
  dbe->setParameter(MJ_PP_FUNCTIONALITY,
		    MJ_DspPluginFunctionalityKeys[MJ_DPF_FILE_IN]);

  dbe->setParameter(MJ_PP_TRANSLATION_FILE, "mj_divaio");

  dbe->setPluginDebug(MJ_ModuleDivaAudioFileIn::debugPtr());

  return dbe;
}

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

MJ_ModuleDsp *MJ_createNew_ModuleDivaAudioFileOut()
{
  return new MJ_ModuleDivaAudioFileOut;
}

MJ_PluginDbeDsp *MJ_ModuleDivaAudioFileOut_Dbe()
{
  MJ_PluginDbeDsp *dbe = new MJ_PluginDbeDsp;

  dbe->setAuthors("Ilmonen Tommi S.");
  dbe->setMenuPosition("_Audio_File_Audio file record");
  dbe->setCreator(MJ_createNew_ModuleDivaAudioFileOut);
  dbe->setIdentifier("_file_audio_file_output1");
  dbe->pluginVersion()->set(0,0,1);
  dbe->options()->setChans(1,64,0,0);
  
  dbe->setParameter(MJ_PP_FUNCTIONALITY,
		    MJ_DspPluginFunctionalityKeys[MJ_DPF_FILE_OUT]);

  // dbe->setParameter(MJ_PP_TRANSLATION_FILE, <mj_audio_file1");

  dbe->setPluginDebug(MJ_ModuleDivaAudioFileOut::debugPtr());

  dbe->setGuiFactory(MJ_GUI_POSTCARD, MJ_ModuleDivaAudioFileOutGui::createNew);
  
  return dbe;
}

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

MJ_ModuleDsp *MJ_createNew_ModuleDivaAudioFileRecPlay()
{
  return new MJ_ModuleDivaAudioFileRecPlay;
}

MJ_PluginDbeDsp *MJ_ModuleDivaAudioFileRecPlay_Dbe()
{
  MJ_PluginDbeDsp *dbe = new MJ_PluginDbeDsp;

  dbe->setAuthors("Ilmonen Tommi S.");
  dbe->setMenuPosition("_Audio_File_Audio file record/play");
  dbe->setCreator(MJ_createNew_ModuleDivaAudioFileRecPlay);
  dbe->setIdentifier("_diva_file_audio_file_recplay");
  dbe->pluginVersion()->set(0,0,1);
  dbe->options()->setChans(1,64,1,64);
  dbe->options()->setValue(MJ_DSP_EQUAL_CHANNEL_IO, 1);
  
  // dbe->setParameter(MJ_PP_TRANSLATION_FILE, <mj_audio_file1");

  dbe->setPluginDebug(MJ_ModuleDivaAudioFileOut::debugPtr());

  dbe->setGuiFactory(MJ_GUI_POSTCARD, 
		     MJ_ModuleDivaAudioFileRecPlayGui::createNew);
  
  return dbe;
}

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////


static MJ_PluginDbeDsp *__MJ_createPlugin(int m)
{
  // MJ_trace("libmjdiva_CREATE_DSPMODULE_NTH", m);

#ifdef ALSA_IS_0_9
  if(MJ_Debug::debugAll())
    DI_AudioPort::setDebug(true);
#endif  

  switch(m) {
  case 0:

    MJ_DivaBuildEnv();

    return MJ_ModuleDivaMidiOut_Dbe();
    
  case 1:
    return MJ_ModuleDivaMidiIn_Dbe();
    
  case 2:
    return MJ_ModuleDivaAudioIn_Dbe();

  case 3:
    return MJ_ModuleDivaAudioOut_Dbe();

  case 4:
    return MJ_ModuleDivaAudioFileIn_Dbe();

  case 5:
    return MJ_ModuleDivaAudioFileOut_Dbe();

  case 6:
    return MJ_ModuleDivaAudioFileRecPlay_Dbe();

  default:
    break;
  }
  return 0;
}

static QString compileDate(DATETIME);

MJ_PluginDbeDsp *MJ_createPlugin(int m)
{
  MJ_PluginDbeDsp * dbe = __MJ_createPlugin(m);

  if(dbe) dbe->setParameter(MJ_PP_COMPILE_TIME, compileDate);

  return dbe;
}
