/* COPYRIGHT
 *
 * This file is part of Mustajuuri.
 *
 * Author: Tommi Ilmonen, 1999-2000.
 * Tommi.Ilmonen@hut.fi
 *
 * http://www.tcm.hut.fi/~tilmonen/mustajuuri/
 *
 * Mustajuuri is licensed under GNU Lesser General Public License
 * (LGPL). The LGPL conditions can be found in file "LGPL" that is
 * distributed with this source package. The Mustajuuri license
 * interpretation that further clarifes LGPL can be found in file
 * "MUSTAJUURI_LICENSE" that is distributed with the mustajuuri source code.
 *    
 */

/* This file contains the necessary code to automatically generate
   audio configuration information on IRIX.
   
   Not exactly well done, but does the job.  */

#include "mj_array.h"
#include "mj_audio_conf.h"
#include "mj_error.h"

#include <di_audio_port.h> 

#include <qstring.h> 

#include <stdlib.h>
#include <stdio.h>

#ifdef DI_AUDIO_IS_SGI

// Use this macro to get debugging information.
// #define MJ_DEBUG_DIVA

static QCString MJ_divaAudioConfig;

void MJ_DivaBuildEnv()
{
#ifdef MJ_DEBUG_DIVA
  const char * fname = "MJ_DivaBuildEnv";
#endif

  // Do not override the value given by the user.
  const char *ae = getenv(MJ_AudioEnvVarName);
  if(ae) {
    if(strlen(ae))
      return;
  }

  ulong i;
  MJ_Array<ulong> ids;

  char buf[DI_AUDIO_MAX_NAME_LENGTH];
  char buf2[DI_AUDIO_MAX_NAME_LENGTH + 128];

  QCString config;

  // Input devices.
  ulong indevs = DI_AudioDevice::getNumberOfInputDevices();
  ids.resize(indevs);
  DI_AudioDevice::getInputDevices(ids.data());

  for(i=0; i < indevs; i++) {
    ulong id = ids.get(i);

    DI_AudioDevice::getDeviceLabel(id, buf);
    
#ifdef MJ_DEBUG_DIVA
    MJ_trace(fname, buf);
#endif    

    /** First check if it is an ADAT device. */
    DI_AudioDevice d(id, true, 8);

    if(d.isOK() && d.getChannels() == 8) {
      sprintf(buf2, ":%s=%s", MJ_AudioDeviceTypeKeys[MJ_ADT_ADAT_IN], buf);
      config += buf2;
    }
    else {
      /** Check if it is a stereo device. */

      DI_AudioDevice d2(id, true, 2);
    
      if(d2.isOK() && d.getChannels() == 2) {
	sprintf(buf2, ":%s=%s", MJ_AudioDeviceTypeKeys[MJ_ADT_STEREO_IN], buf);
	config += buf2;
      }
    }
  }

  // Output devices.
  ulong outdevs = DI_AudioDevice::getNumberOfOutputDevices();
  ids.resize(outdevs);
  DI_AudioDevice::getOutputDevices(ids.data());
  
  for(i=0; i < outdevs; i++) {
    ulong id = ids.get(i);

    DI_AudioDevice::getDeviceLabel(id, buf);
    
#ifdef MJ_DEBUG_DIVA
    MJ_trace(fname, buf);
#endif    

    /** First check if it is an ADAT device. */
    DI_AudioDevice d(id, false, 8);

    if(d.isOK() && d.getChannels() == 8) {
      sprintf(buf2, ":%s=%s", MJ_AudioDeviceTypeKeys[MJ_ADT_ADAT_OUT], buf);
      config += buf2;
    }
    else {
      /** Check if it is a stereo device. */

      DI_AudioDevice d2(id, false, 2);
      
      if(d2.isOK() && d.getChannels() == 2) {
	sprintf(buf2, ":%s=%s", MJ_AudioDeviceTypeKeys[MJ_ADT_STEREO_OUT], buf);
	config += buf2;
      }
    }
  }
  MJ_divaAudioConfig.resize(config.length() + 64);
  
  MJ_divaAudioConfig.sprintf("%s=%s", MJ_AudioEnvVarName, config.data());
  
  putenv(MJ_divaAudioConfig);
#ifdef MJ_DEBUG_DIVA
  MJ_trace(fname, MJ_divaAudioConfig);
#endif
}

#endif // DI_AUDIO_IS_SGI
