/* COPYRIGHT
 *
 * This file is part of Mustajuuri GPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the General Public License (GPL),
 * versions 2. Extra clauses can be found from the file MUSTAJUURI_GPL
 * and further explanations from the file MUSTAJUURI_LICENSE. The
 * license text can be found in file "GPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *  
 */

#include <mj_module_compressor2.h>
#include <mj_module_compressor3.h>
#include <mj_module_limiter.h>
#include <mj_module_limiter2.h>
#include <mj_module_compressor_gui.h>

#include <mj_dsp_plugin.h>
#include <mj_to_ladspa_plugin.h>
#include <mj_version.h>

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

MJ_ModuleDsp * MJ_createNew_ModuleCompressor2() 
{
  return new MJ_ModuleCompressor2;
}

MJ_PluginDbeDsp * MJ_pluginCompressor2()
{
  MJ_PluginDbeDsp *dbe = new MJ_PluginDbeDsp;

  dbe->setAuthors("Ilmonen Tommi S.");
  dbe->setMenuPosition("_Audio_Dynamics_Compressor 2");
  dbe->setCreator(MJ_createNew_ModuleCompressor2);
  dbe->setGuiFactory(MJ_GUI_POSTCARD, MJ_ModuleCompressorGui::createNew);
  dbe->setIdentifier("Tommi.Ilmone_at_hut.fi_compressor2");

  dbe->options()->setValue(MJ_DSP_EQUAL_CHANNEL_IO, 1);
  dbe->options()->setValue(MJ_DSP_MIN_CHANNELS_IN, 1);
  dbe->options()->setValue(MJ_DSP_MIN_CHANNELS_OUT, 1);

  // dbe->setParameter(MJ_PP_TRANSLATION_FILE, <mj_compressor");
  // dbe->setGuiFactory(MJ_GUI_POSTCARD, MJ_ModuleCompressor1_QG::createNew);
  
  MJ_LadspaDatas *lds = new MJ_LadspaDatas;
  dbe->setLadspaDatas(lds);
  // Mono version:
  lds->newElem(1347, "mjcompressor2-mono", "Compressor 2 (mono) - MJ", 1, 1);
  // Stereo version:
  lds->newElem(1348, "mjcompressor2-stereo", "Compressor 2 (stereo) - MJ", 2, 2);

  return dbe;  
}

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

MJ_ModuleDsp * MJ_createNew_ModuleCompressor3() 
{
  return new MJ_ModuleCompressor3;
}

MJ_PluginDbeDsp * MJ_pluginCompressor3()
{
  MJ_PluginDbeDsp *dbe = new MJ_PluginDbeDsp;

  dbe->setAuthors("Ilmonen Tommi S.");
  dbe->setMenuPosition("_Audio_Dynamics_Compressor 3");
  dbe->setCreator(MJ_createNew_ModuleCompressor3);
  dbe->setGuiFactory(MJ_GUI_POSTCARD, MJ_ModuleCompressorGui::createNew);
  dbe->setIdentifier("Tommi.Ilmone_at_hut.fi_compressor3");

  dbe->options()->setValue(MJ_DSP_EQUAL_CHANNEL_IO, 1);
  dbe->options()->setValue(MJ_DSP_MIN_CHANNELS_IN, 1);
  dbe->options()->setValue(MJ_DSP_MIN_CHANNELS_OUT, 1);

  // dbe->setParameter(MJ_PP_TRANSLATION_FILE, <mj_compressor");
  // dbe->setGuiFactory(MJ_GUI_POSTCARD, MJ_ModuleCompressor1_QG::createNew);
  
  MJ_LadspaDatas *lds = new MJ_LadspaDatas;
  dbe->setLadspaDatas(lds);
  // Mono version:
  lds->newElem(1349, "mjcompressor3-mono", "Compressor 3 (mono) - MJ", 1, 1);
  // Stereo version:
  lds->newElem(1350, "mjcompressor3-stereo", "Compressor 3 (stereo) - MJ", 2, 2);

  return dbe;  
}

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

MJ_ModuleDsp * MJ_createNew_ModuleLimiter() 
{
  return new MJ_ModuleLimiter;
}

MJ_PluginDbeDsp * MJ_pluginLimiter()
{
  MJ_PluginDbeDsp *dbe = new MJ_PluginDbeDsp;

  dbe->setAuthors("Ilmonen Tommi S.");
  dbe->setMenuPosition("_Audio_Dynamics_Peak limiter");
  dbe->setCreator(MJ_createNew_ModuleLimiter);
  dbe->setGuiFactory(MJ_GUI_POSTCARD, MJ_ModuleCompressorGui::createNew);
  dbe->setIdentifier("Tommi.Ilmone_at_hut.fi_limiter");

  dbe->options()->setValue(MJ_DSP_EQUAL_CHANNEL_IO, 1);
  dbe->options()->setValue(MJ_DSP_MIN_CHANNELS_IN, 1);
  dbe->options()->setValue(MJ_DSP_MIN_CHANNELS_OUT, 1);

  // dbe->setParameter(MJ_PP_TRANSLATION_FILE, <mj_limiter");
  // dbe->setGuiFactory(MJ_GUI_POSTCARD, MJ_ModuleLimiter1_QG::createNew);
  
  MJ_LadspaDatas *lds = new MJ_LadspaDatas;
  dbe->setLadspaDatas(lds);
  // Mono version:
  lds->newElem(1351, "mjlimiter1-mono", "Heavy Peak Limiter (mono) - MJ", 1, 1);
  // Stereo version:
  lds->newElem(1352, "mjlimiter1-stereo", "Heavy Peak Limiter (stereo) - MJ", 2, 2);

  return dbe;  
}

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

MJ_ModuleDsp * MJ_createNew_ModuleLimiter2() 
{
  return new MJ_ModuleLimiter2;
}

MJ_PluginDbeDsp * MJ_pluginLimiter2()
{
  MJ_PluginDbeDsp *dbe = new MJ_PluginDbeDsp;

  dbe->setAuthors("Ilmonen Tommi S.");
  dbe->setMenuPosition("_Audio_Dynamics_Peak limiter (fast)");
  dbe->setCreator(MJ_createNew_ModuleLimiter2);
  dbe->setGuiFactory(MJ_GUI_POSTCARD, MJ_ModuleCompressorGui::createNew);
  dbe->setIdentifier("Tommi.Ilmone_at_hut.fi_limiter2");

  dbe->options()->setValue(MJ_DSP_EQUAL_CHANNEL_IO, 1);
  dbe->options()->setValue(MJ_DSP_MIN_CHANNELS_IN, 1);
  dbe->options()->setValue(MJ_DSP_MIN_CHANNELS_OUT, 1);
  dbe->setPluginDebug(MJ_ModuleCompressor3::debugPtr());

  // dbe->setParameter(MJ_PP_TRANSLATION_FILE, <mj_limiter2");
  // dbe->setGuiFactory(MJ_GUI_POSTCARD, MJ_ModuleLimiter21_QG::createNew);
  
  MJ_LadspaDatas *lds = new MJ_LadspaDatas;
  dbe->setLadspaDatas(lds);
  // Mono version:
  lds->newElem(1353, "mjlimiter2-mono", "Fast Peak Limiter (mono) - MJ", 1, 1);
  // Stereo version:
  lds->newElem(1354, "mjlimiter2-stereo", "Fast Peak Limiter (stereo) - MJ", 2, 2);

  return dbe;  
}

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

static MJ_PluginDbeDsp * __MJ_createPlugin(int i)
{
  switch(i) {
  case 0:
    return MJ_pluginCompressor2();
  case 1:
    return MJ_pluginCompressor3();
  case 2:
    return MJ_pluginLimiter();
  case 3:
    return MJ_pluginLimiter2();
  }
  return 0;
}

static QString compileDate(DATETIME);

MJ_PluginDbeDsp *MJ_createPlugin(int m)
{
  MJ_PluginDbeDsp * dbe = __MJ_createPlugin(m);

  if(dbe) dbe->setParameter(MJ_PP_COMPILE_TIME, compileDate);

  return dbe;
}

