/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of Mustajuuri GPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the General Public License (GPL),
 * versions 2. Extra clauses can be found from the file MUSTAJUURI_GPL
 * and further explanations from the file MUSTAJUURI_LICENSE. The
 * license text can be found in file "GPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *  
 */

#ifndef MJ_MODULE_COMPRESSOR2_H
#define MJ_MODULE_COMPRESSOR2_H

#include <mj_filter_ap1.h>
#include <mj_level_meter.h> 
#include <mj_module_compressor.h>
#include <mj_ramp.h>

/// Parameters for compressor2
enum MJ_ParamsCompressor2 {
  MJ_PARCOMP2_THRESHOLD,
  MJ_PARCOMP2_RATIO,
  MJ_PARCOMP2_ATTACK,
  MJ_PARCOMP2_HOLD,
  MJ_PARCOMP2_RELEASE,
  MJ_PARCOMP2_SYNC,
  MJ_PARCOMP2_EXTENDED,
  MJ_PARCOMP2_SIZEOF
};

extern const char * MJ_ParamsCompressor2Keys[MJ_PARCOMP2_SIZEOF];

enum MJ_CompressorState {
  MJ_COMPRESSOR_IDLE,
  MJ_COMPRESSOR_ATTACK,
  MJ_COMPRESSOR_HOLD,
  MJ_COMPRESSOR_DECAY
};

class MJ_ChannelCompressor 
{
public:
  
  MJ_Ramp<double>         & ramp() { return m_ramp; }
  MJ_FilterAP1<MJ_Sample> & rmsFilter() { return m_rmsFilter; }

  void prepare(MJ_Sample thresholdDb) 
  { 
    m_ramp.reset(1.0);
    m_rmsFilter.resetMemory(); 
    m_state = MJ_COMPRESSOR_IDLE;
    m_currentPeak = m_secondaryPeak = thresholdDb;
  }

  void newGain(MJ_Sample gain, uint samples)
  {
    m_ramp.setTarget(samples, gain, MJ_RAMP_EXPONENTIAL);
  }

  MJ_CompressorState state() { return m_state; }
  void setState(const MJ_CompressorState &s) { m_state = s; }
  
  MJ_Sample newGain(MJ_Sample energy,
		    MJ_Sample thresholdDb, 
		    MJ_Sample ratio,
		    uint attackTime,
		    uint holdTime,
		    uint decayTime);

protected:
  /// Use double to handle long release times.
  MJ_Ramp<double>          m_ramp;
  MJ_FilterAP1<MJ_Sample>  m_rmsFilter;
  MJ_CompressorState       m_state;
  MJ_Sample                m_currentPeak;
  MJ_Sample                m_secondaryPeak;
};


/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

/** Compressor module. 

    Knows both independent multi-channel compression and synchronized
    multi-channel compression.  */

class MJ_ModuleCompressor2 : public MJ_ModuleDsp, public MJ_CompressorBase
{
public:
  MJ_ModuleCompressor2();
  virtual ~MJ_ModuleCompressor2();
  
  virtual bool dspPrepare();
  virtual void dspProcessPeriod();
  virtual bool dspStop();

  virtual bool msgProcess(const char *, MJ_Message *);
  virtual MJ_Parameter * msgGetInput(const char *);
  virtual QCString msgGetInputName(int);

protected:
  
  void processSync();

  void processNonSync();
  
  void redesignFilters();

  void calculateTimes();

  MJ_Array<MJ_ChannelCompressor> m_channelCompressors;
  
  MJ_Sample m_threshold, m_ratio, m_attackMs, m_holdMs, m_releaseMs;
  uint      m_attackSamples, m_holdSamples, m_releaseSamples;

  bool m_syncChannels;
  bool m_extendedRanges;

  MJ_Vector<MJ_Sample>               m_compressVect;

};


#endif
