/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of Mustajuuri GPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the General Public License (GPL),
 * versions 2. Extra clauses can be found from the file MUSTAJUURI_GPL
 * and further explanations from the file MUSTAJUURI_LICENSE. The
 * license text can be found in file "GPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *  
 */

#ifndef MJ_MODULE_COMPRESSOR3_H
#define MJ_MODULE_COMPRESSOR3_H

#include <mj_filter_ap1.h>
#include <mj_level_meter.h> 
#include <mj_module_compressor.h>
#include <mj_ramp.h>

#include <vector>

/// Parameters for compressor3 (see MJ_ModuleCompressor3)
enum MJ_ParamsCompressor3 {
  MJ_PARCOMP3_THRESHOLD,
  MJ_PARCOMP3_RATIO,
  MJ_PARCOMP3_ATTACK,
  MJ_PARCOMP3_RELEASE,
  MJ_PARCOMP3_SYNC,
  MJ_PARCOMP3_SIZEOF
};

extern const char * MJ_ParamsCompressor3Keys[MJ_PARCOMP3_SIZEOF];

class MJ_ChannelCompressor3
{
public:
  
  MJ_FilterAP1<double> & attackFilter() { return m_attackFilter; }
  MJ_FilterAP1<double> & releaseFilter() { return m_releaseFilter; }

  void prepare(MJ_Sample tresholdSquare) 
  { 
    m_attackFilter.resetMemory(tresholdSquare); 
    m_releaseFilter.resetMemory(tresholdSquare); 
  }
  
  MJ_Sample newGain(MJ_Sample energy,
		    MJ_Sample thresholdSquare, 
		    MJ_Sample thresholdDb, 
		    MJ_Sample ratio);

protected:
  MJ_FilterAP1<double>  m_attackFilter;
  MJ_FilterAP1<double>  m_releaseFilter;
};


/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

/** Compressor module. 

    Knows both independent multi-channel compression and synchronized
    multi-channel compression.  */

class MJ_ModuleCompressor3 : public MJ_ModuleDsp, public MJ_CompressorBase
{
public:
  MJ_ModuleCompressor3();
  virtual ~MJ_ModuleCompressor3();
  
  virtual bool dspPrepare();
  virtual void dspProcessPeriod();
  virtual bool dspStop();

  virtual bool msgProcess(const char *, MJ_Message *);
  virtual MJ_Parameter * msgGetInput(const char *);
  virtual QCString msgGetInputName(int);

  static bool * debugPtr() {return & m_debug;}
protected:
  
  void processSync();

  void processNonSync();
  
  void redesignFilters();

  std::vector<MJ_ChannelCompressor3> m_channelCompressors;
  
  MJ_Sample m_thresholdDb, m_thresholdSquare, m_ratio, m_attackMs, m_releaseMs;
  uint      m_attackSamples, m_releaseSamples;

  bool m_syncChannels;

  MJ_Vector<MJ_Sample>               m_compressVect;

  static bool m_debug;
};


#endif
