/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of Mustajuuri GPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the General Public License (GPL),
 * versions 2. Extra clauses can be found from the file MUSTAJUURI_GPL
 * and further explanations from the file MUSTAJUURI_LICENSE. The
 * license text can be found in file "GPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *  
 */

#ifndef MJ_MODULE_LIMITER_H
#define MJ_MODULE_LIMITER_H

#include <mj_level_meter.h> 
#include <mj_module_compressor.h>
#include <mj_ringbuffer.h>

enum MJ_ParamsLimiter {
  MJ_PARLIM_THRESHOLD,
  MJ_PARLIM_ATTACK,
  MJ_PARLIM_RELEASE,
  MJ_PARLIM_SIZEOF
};

extern const char * MJ_ParamsLimiterKeys[MJ_PARLIM_SIZEOF];


class MJ_ChannelLimiter
{
public:

  void prepare(MJ_Sample thresholdLog, uint maxDelay)
  {
    m_buffer.resize(maxDelay); 
    m_buffer.setAll(0); 
    m_logBuffer.resize(maxDelay);
    m_logBuffer.setAll(thresholdLog); 
    m_level.reset(thresholdLog);
    m_gain = 0.0;
    m_step  = 0.0;
    m_untilPeak = 0;
  }

  MJ_Sample putGet(MJ_Sample insample,
		   MJ_Sample thresholdAbs,
		   uint attackTime,
		   uint releaseTime);

  MJ_Sample gain() const{return m_gain; }

protected:
  MJ_RingBuffer2<MJ_Sample>     m_buffer;
  MJ_RingBuffer2<MJ_Sample>     m_logBuffer;
  MJ_LevelMeter<MJ_Sample>      m_level;
  MJ_Sample                     m_gain;
  MJ_Sample                     m_step;
  uint                          m_untilPeak;
};

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////


/** Limiter module. Sound fairly nice and hogs a lot of CPU.

    @see MJ_ModuleLimiter2
    
    @author Tommi Ilmonen
 */

class MJ_ModuleLimiter : public MJ_ModuleDsp, public MJ_CompressorBase
{
public:
  MJ_ModuleLimiter();
  virtual ~MJ_ModuleLimiter();
  
  virtual bool dspPrepare();
  virtual void dspProcessPeriod();

  virtual bool msgProcess(const char *, MJ_Message *);
  virtual MJ_Parameter * msgGetInput(const char *);
  virtual QCString msgGetInputName(int);

protected:
    
  void redesignFilters();

  void calculateTimes();

  MJ_Array<MJ_ChannelLimiter> m_channelLimiters;
  
  MJ_Sample m_threshold, m_attackMs, m_releaseMs;
  uint      m_attackSamples, m_releaseSamples;
};

#endif
