/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of Mustajuuri GPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the General Public License (GPL),
 * versions 2. Extra clauses can be found from the file MUSTAJUURI_GPL
 * and further explanations from the file MUSTAJUURI_LICENSE. The
 * license text can be found in file "GPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *  
 */

#ifndef MJ_MODULE_LIMITER2_H
#define MJ_MODULE_LIMITER2_H

#include <mj_filter_ap1.h>
#include <mj_level_meter.h> 
#include <mj_math.h>
#include <mj_module_compressor.h>
#include <mj_ringbuffer.h>

enum MJ_ParamsLimiter2 {
  MJ_PARLIM2_THRESHOLD,
  MJ_PARLIM2_RELEASE,
  MJ_PARLIM2_SIZEOF
};

extern const char * MJ_ParamsLimiter2Keys[MJ_PARLIM2_SIZEOF];

class MJ_ChannelLimiter2
{
public:

  void prepare(MJ_Sample thresholdAbs)
  {
    m_level.reset(thresholdAbs);
    m_absFilter.resetMemory(thresholdAbs);
  }

  MJ_Sample newGain(MJ_Sample insample,
		    MJ_Sample thresholdAbs,
		    uint releaseTime)
  {
    MJ_Sample inAbs = MJ_max(MJ_abs(insample), thresholdAbs);
    
    m_level.put(inAbs, releaseTime);
    
    MJ_Sample gain = thresholdAbs / m_absFilter.putGetSample(inAbs);
    MJ_Sample peak = m_level.peak();
    
    if(peak * gain > thresholdAbs) {
      gain = thresholdAbs / peak;
      m_absFilter.resetMemory(peak);
    }

    return gain;
  }

  MJ_FilterAP1<double> & absFilter() {return m_absFilter; }

protected:
  MJ_LevelMeter<MJ_Sample> m_level;
  MJ_FilterAP1<double>     m_absFilter;
};

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////


/** Fast limiter module. This implementation is extremely fast. As a
    result it is not the most pleasound-sounding limiter around.

    @see MJ_ModuleLimiter
    
    @author Tommi Ilmonen
 */

class MJ_ModuleLimiter2 : public MJ_ModuleDsp, public MJ_CompressorBase
{
public:
  MJ_ModuleLimiter2();
  virtual ~MJ_ModuleLimiter2();
  
  virtual bool dspPrepare();
  virtual void dspProcessPeriod();

  virtual bool msgProcess(const char *, MJ_Message *);
  virtual MJ_Parameter * msgGetInput(const char *);
  virtual QCString msgGetInputName(int);

protected:
    
  void redesignFilters();

  MJ_Array<MJ_ChannelLimiter2> m_channelLimiters;
  
  MJ_Sample m_threshold, m_releaseMs;
  uint      m_releaseSamples;
};

#endif
