/* COPYRIGHT
 *
 * This file is part of Mustajuuri GPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the General Public License (GPL),
 * versions 2. Extra clauses can be found from the file MUSTAJUURI_GPL
 * and further explanations from the file MUSTAJUURI_LICENSE. The
 * license text can be found in file "GPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *  
 */

#include <mj_module_eq2.h>
#include <mj_module_eq1.h>
#include <mj_module_eq1_gui.h>
#include <mj_module_eq3_gui.h>
#include <mj_module_high_pass.h>

#include <mj_dsp_plugin.h>
#include <mj_strs.h>
#include <mj_to_ladspa_plugin.h>
#include <mj_version.h>


MJ_ModuleDsp * MJ_createNew_ModuleEq1() 
{
  return new MJ_ModuleEq1;
}

/// One-band parametric EQ
static MJ_PluginDbeDsp *MJ_Eq1_Dbe() 
{

  MJ_PluginDbeDsp *dbe = new MJ_PluginDbeDsp;
  dbe->setAuthors("Ilmonen Tommi S.");
  dbe->setMenuPosition("_Audio_Equalizers_Single-band Parametric EQ");
  dbe->setCreator(MJ_createNew_ModuleEq1);
  dbe->setIdentifier("equalizers/EQ/1_tommi.ilmonen@hut.fi");

  dbe->pluginVersion()->set(0,0,1);

  dbe->options()->setValue(MJ_DSP_EQUAL_CHANNEL_IO, 1);
  dbe->options()->setChans(1, -1, 1, -1);
  
  // dbe->setParameter(MJ_PP_TRANSLATION_FILE, <mj_example1");
  dbe->setGuiFactory(MJ_GUI_POSTCARD, MJ_ModuleEq1_QG::createNew);
  
  return dbe;  
}

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

MJ_ModuleDsp * MJ_createNew_ModuleHighPass() 
{
  return new MJ_ModuleHighPass;
}

/// One-band parametric EQ
static MJ_PluginDbeDsp *MJ_HighPass_Dbe() 
{

  MJ_PluginDbeDsp *dbe = new MJ_PluginDbeDsp;
  dbe->setAuthors("Ilmonen Tommi S.");
  dbe->setMenuPosition("_Audio_Equalizers_High-pass");
  dbe->setCreator(MJ_createNew_ModuleHighPass);
  dbe->setIdentifier("mj_high_pass_1");

  dbe->pluginVersion()->set(0,0,1);

  dbe->options()->setValue(MJ_DSP_EQUAL_CHANNEL_IO, 1);
  dbe->options()->setChans(1, -1, 1, -1);
    
  return dbe;  
}

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

MJ_ModuleDsp * MJ_createNew_ModuleEq2() 
{
  return new MJ_ModuleEq2(5);
}

/// Parametric EQ
static MJ_PluginDbeDsp *MJ_Eq2_Dbe() 
{

  MJ_PluginDbeDsp *dbe = new MJ_PluginDbeDsp;
  dbe->setAuthors("Ilmonen Tommi S.");
  dbe->setMenuPosition("_Audio_Equalizers_Parametric EQ");
  dbe->setCreator(MJ_createNew_ModuleEq2);
  dbe->setIdentifier("equalizers/EQ/2_tommi.ilmonen@hut.fi");

  dbe->pluginVersion()->set(0,0,1);
  dbe->setParameter(MJ_PP_TRANSLATION_FILE, "mj_eq2");

  dbe->options()->setValue(MJ_DSP_EQUAL_CHANNEL_IO, 1);
  dbe->options()->setChans(1, -1, 1, -1);
  dbe->setGuiFactory(MJ_GUI_STAMP, MJ_ModuleDspGuiStamp::createNew);
  dbe->setGuiFactory(MJ_GUI_POSTCARD, MJ_ModuleEq2_QG::createNew);
  
  // Make the ladspa plugins
  MJ_LadspaDatas *lds = new MJ_LadspaDatas;
  dbe->setLadspaDatas(lds);
  
  QCString ladLabel;
  QCString ladName;

  int bandNums[5] = { 3, 5, 7, 9, 15 };

  // Start allocation of ladspa plugins here.
  ulong ladspaId = 1321;
  
  // CONSUME LADSPA IDS 1321-1330
  
  // Mono and stereo
  for(uint chans = 1; chans <= 2; chans++) {
    int bands;

    // Iterate over bands
    for(int i = 0; i < 5 ; i++) {
      bands = bandNums[i];
      ladLabel.sprintf("Mustajuuri-parametric-EQ-%d-channels-%d-bands)",
		       chans, bands);
      if(chans == 1)
	ladName.sprintf("Mustajuuri %d-band parametric EQ (mono)",
			bands);
      else
	ladName.sprintf("Mustajuuri %d-band parametric EQ (stereo)",
			bands);

      MJ_LadspaData *ld = lds->newElem(ladspaId++, ladLabel, ladName);
      ld->setChannels(chans,chans);
      ld->addMsgInt(MJ_Strs::bands, bands);
      ld->addMsgInt("inladspa", 1);
    }
  }
    
  return dbe;  
}

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

/// Graphical EQ
static MJ_PluginDbeDsp *MJ_Eq3_Dbe() 
{

  MJ_PluginDbeDsp *dbe = new MJ_PluginDbeDsp;
  dbe->setAuthors("Ilmonen Tommi S.");
  dbe->setMenuPosition("_Audio_Equalizers_Graphical EQ");
  dbe->setCreator(MJ_createNew_ModuleEq2);
  dbe->setIdentifier("equalizers/EQ/3_tommi.ilmonen@hut.fi");

  dbe->pluginVersion()->set(0,0,1);
  dbe->setParameter(MJ_PP_TRANSLATION_FILE, "mj_eq3");

  dbe->options()->setValue(MJ_DSP_EQUAL_CHANNEL_IO, 1);
  dbe->options()->setChans(1, -1, 1, -1);
  dbe->setGuiFactory(MJ_GUI_STAMP, MJ_ModuleDspGuiStamp::createNew);
  dbe->setGuiFactory(MJ_GUI_POSTCARD, MJ_ModuleEq3_QG::createNew);

  // Make the ladspa plugins
  MJ_LadspaDatas *lds = new MJ_LadspaDatas;
  dbe->setLadspaDatas(lds);

  QCString ladLabel;
  QCString ladName;

  int bandNums[6] = { 3, 5, 7, 9, 15, 31 };

  // Start allocation of ladspa plugins here.
  ulong ladspaId = 1331;
  
  // CONSUME LADSPA IDS 1331-1342
  
  // Mono and stereo
  for(uint chans = 1; chans <= 2; chans++) {
    int bands;

    // Iterate over bands
    for(int i = 0; i < 6 ; i++) {
      bands = bandNums[i];
      ladLabel.sprintf("Mustajuuri-parametric-EQ-%d-channels-%d-bands)",
		       chans, bands);
      if(chans == 1)
	ladName.sprintf("%d-band graphical EQ (mono) (Mustajuuri)",
			bands);
      else
	ladName.sprintf("%d-band graphical EQ (stereo) (Mustajuuri)",
			bands);

      MJ_LadspaData *ld = lds->newElem(ladspaId++, ladLabel, ladName);
      ld->setChannels(chans,chans);
      ld->addMsgInt(MJ_Strs::bands, bands);
      ld->addMsgInt("inladspa", 1);
    }
  }
    
  return dbe;  
}

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////


static MJ_PluginDbeDsp *__MJ_createPlugin(int i)
{
  switch(i) {
  case 0:
    return MJ_Eq1_Dbe();
  case 1:
    return MJ_Eq2_Dbe();
  case 2:
    return MJ_Eq3_Dbe();
  case 3:
    return MJ_HighPass_Dbe();
  }
  return 0;
}

static QString compileDate(DATETIME);

MJ_PluginDbeDsp *MJ_createPlugin(int m)
{
  MJ_PluginDbeDsp * dbe = __MJ_createPlugin(m);

  if(dbe) dbe->setParameter(MJ_PP_COMPILE_TIME, compileDate);

  return dbe;
}


