/* COPYRIGHT
 *
 * This file is part of Mustajuuri GPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the General Public License (GPL),
 * versions 2. Extra clauses can be found from the file MUSTAJUURI_GPL
 * and further explanations from the file MUSTAJUURI_LICENSE. The
 * license text can be found in file "GPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *  
 */

#ifndef MJ_MODULE_EQ1_H
#define MJ_MODULE_EQ1_H

/* Include base  definitions. */
#include <mj_module_dsp.h>

#include <mj_filter_iir2.h>

extern const char * MJ_MEQ1_params[3];

/* Single-band parametric EQ. */
class MJ_ModuleEq1 : public MJ_ModuleDsp
{
public:
  MJ_ModuleEq1();

  virtual ~MJ_ModuleEq1();

  virtual bool dspPrepare();
  virtual void dspProcessPeriod();
  virtual bool msgProcess(const char *, MJ_Message *);
  virtual MJ_Parameter * msgGetInput(const char *);
  virtual QCString msgGetInputName(int);

  MJ_Sample targetGainDb() const { return m_targetGainDb; }
  MJ_Sample targetCenterHz() const { return m_targetCenterHz; }
  MJ_Sample targetBandWidthOctaves() const { return m_targetBandWidthOctaves; }
  MJ_Sample samplingRate() const { return m_samplingRate; }

protected:
  
  bool updateFilterCoeffs();

  double m_gainDb;
  double m_centerHz;
  double m_bandWidthOctaves;

  MJ_Sample m_targetGainDb;
  MJ_Sample m_targetCenterHz;
  MJ_Sample m_targetBandWidthOctaves;
  MJ_Sample m_samplingRate;

  ulong m_interpSamples;

  MJ_Array<MJ_FilterIIR2<MJ_Sample> > m_filters;

  static bool m_debug;
};


#endif
