/* COPYRIGHT
 *
 * This file is part of Mustajuuri GPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the General Public License (GPL),
 * versions 2. Extra clauses can be found from the file MUSTAJUURI_GPL
 * and further explanations from the file MUSTAJUURI_LICENSE. The
 * license text can be found in file "GPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *  
 */

#include <mj_module_eq1_gui.h>

#include <mj_freq_resp_gui.h> 
#include <mj_module_eq1.h>
#include <mj_strs.h>

#include <qlayout.h> 

MJ_ModuleEq1_QG::MJ_ModuleEq1_QG(MJ_RtControl *rtparent,
				 MJ_ModuleDsp *module,
				 QWidget      *parent,
				 const char   *name)
  : MJ_ModuleDspGuiPostCard(rtparent, module, true,
			    (uint) -1,
			    MJ_GUI_POSTCARD,
			    parent, name),
  m_blockDraw(false)
{
  resize(600, 400);
  draw();
}

MJ_ModuleEq1_QG::~MJ_ModuleEq1_QG()
{}

void MJ_ModuleEq1_QG::moduleModified(const char *id)
{
  MJ_ModuleDspGuiPostCard::moduleModified(id);

  if(!m_blockDraw)
    updateEq();
}

MJ_ModuleDspGui *MJ_ModuleEq1_QG::createNew(MJ_RtControl *rtparent,
					    MJ_ModuleDsp *module,
					    QWidget *parent, 
					    const char *name)
{ 
  return new MJ_ModuleEq1_QG(rtparent, module, parent, name);
}

void MJ_ModuleEq1_QG::grMousePress(QMouseEvent *e, float x, float y)
{
  // qDebug("MJ_ModuleEq1_QG::grMousePress # %f %f", x, y);
  
  if(e->stateAfter())
    setPeak(x, y);
}

void MJ_ModuleEq1_QG::grMouseRelease(QMouseEvent *, float, float)
{
  doneValue(MJ_Strs::frequency);
  doneValue(MJ_Strs::gain);
}

void MJ_ModuleEq1_QG::draw()
{
  int rows = m_layout->numRows();
  int cols = m_layout->numCols();

  m_layout->expand(rows + 1, cols);
  
  m_freqResp = new MJ_FreqRespGui(this, "eqgraph", 20.0, 0.001, &Qt::red);

  m_freqResp->setBackgroundColor(Qt::white);
  m_freqResp->setMinimumHeight(100);

  connect(m_freqResp, SIGNAL(mousePress(QMouseEvent *, float, float)), 
	  this, SLOT(grMousePress(QMouseEvent *, float, float)));

  connect(m_freqResp, SIGNAL(mouseMove(QMouseEvent *, float, float)), 
	  this, SLOT(grMousePress(QMouseEvent *, float, float)));

  connect(m_freqResp, SIGNAL(mouseRelease(QMouseEvent *, float, float)), 
	  this, SLOT(grMouseRelease(QMouseEvent *, float, float)));

  m_layout->addMultiCellWidget(m_freqResp, rows, rows, 0, cols - 1);
  m_layout->setRowStretch(rows, 90);

  updateEq();
  m_freqResp->show();
}

void MJ_ModuleEq1_QG::updateEq()
{
  MJ_FilterIIR2<MJ_Sample> iir;

  MJ_ModuleEq1 *eq = (MJ_ModuleEq1 *) m_module;

  MJ_Sample hz;

  lock();
  hz = eq->targetCenterHz();
  
  iir.designParametricB2(eq->targetCenterHz(),
			 eq->samplingRate(),
			 eq->targetGainDb(),
			 eq->targetBandWidthOctaves());
  unlock();

  qDebug("MJ_ModuleEq1_QG::updateEq # hz = %f", hz);
  
  m_freqResp->setFilter(iir.bptr(), 2,
			iir.aptr(), 2);
}

void MJ_ModuleEq1_QG::setPeak(float x, float y)
{
  MJ_ModuleEq1 *eq = (MJ_ModuleEq1 *) m_module;

  lock();
  float nyquist = eq->samplingRate() * 0.5;
  unlock();

  float logMax = MJ_log(nyquist);
  float logMin = MJ_log(nyquist * 0.001);
  
  float freqBase = (x - 0.001) / (0.999);

  float logFreq = freqBase * (logMax - logMin) + logMin;

  double freqHz = exp(logFreq);
  
  // qDebug("freqHz = %f, freqBase = %f", freqHz, logFreq);
  
  m_blockDraw = true;
  setValue(MJ_Strs::frequency, freqHz);
  setValue(MJ_Strs::gain, y);
  m_blockDraw = false;

  update2(MJ_Strs::frequency);
  update2(MJ_Strs::gain);

  updateEq();
}
