/* COPYRIGHT
 *
 * This file is part of Mustajuuri GPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the General Public License (GPL),
 * versions 2. Extra clauses can be found from the file MUSTAJUURI_GPL
 * and further explanations from the file MUSTAJUURI_LICENSE. The
 * license text can be found in file "GPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *  
 */

#ifndef MJ_MODULE_EQ2_H
#define MJ_MODULE_EQ2_H

/* Include base  definitions. */
#include <mj_module_dsp.h>
#include <mj_array2d.h>

#include <mj_filter_iir2.h>

class MJ_Eq2FilterParams
{
public:
  friend class MJ_ModuleEq2;

  MJ_Eq2FilterParams();
  ~MJ_Eq2FilterParams() {}

  void reset();

  inline void copyTargetValues()
  {    
    m_gainDb = m_targetGainDb;
    m_centerHz = m_targetCenterHz;
    m_bandWidthOctaves = m_targetBandWidthOctaves;
    m_interpSamples = 0;
  }

  void addNewParam(int p, MJ_Sample target, ulong interpSamples);

  void describe(char *);

  double & gainDb() { return m_gainDb; }
  double & centerHz() { return m_centerHz; }
  double & bandWidthOctaves() { return m_bandWidthOctaves; }

  MJ_Sample targetGainDb() const { return m_targetGainDb; }
  MJ_Sample targetCenterHz() const { return m_targetCenterHz; }
  MJ_Sample targetBandWidthOctaves() const { return m_targetBandWidthOctaves; }

  ulong    & interpSamples() { return m_interpSamples; }

  inline MJ_Sample gainRate(double scale) 
  { return (m_targetGainDb - m_gainDb) * scale; }
  inline MJ_Sample centerHzRate(double scale) 
  { return (m_targetCenterHz - m_centerHz) * scale; }
  inline MJ_Sample bwRate(double scale) 
  { return (m_targetBandWidthOctaves - m_bandWidthOctaves) * scale; }

protected:
  double m_gainDb;
  double m_centerHz;
  double m_bandWidthOctaves;

  MJ_Sample m_targetGainDb;
  MJ_Sample m_targetCenterHz;
  MJ_Sample m_targetBandWidthOctaves;

  ulong     m_interpSamples;
};

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

/** Multi-band parametric EQ. 
    
    This same class is used to run graphical EQ DSP side.  */
class MJ_ModuleEq2 : public MJ_ModuleDsp
{
public:
  MJ_ModuleEq2(uint peqBands);

  virtual ~MJ_ModuleEq2();

  virtual bool dspPrepare();
  virtual void dspProcessPeriod();
  virtual bool dspStop();

  virtual void msgGetInputs(MJ_Parameters *);
  virtual bool msgProcess(const char *, MJ_Message *);
  virtual MJ_Parameter * msgGetInput(const char *);

  uint stages() { return m_params.size(); }

  MJ_Array<MJ_Eq2FilterParams> & parameters() { return m_params; }

  uint closestBand(MJ_Sample freqHz);
  uint closestGraphicBand(MJ_Sample freqHz);

  static void formMsgId(char *buf, int type, int index);

  static void determineParams(const char *id, int *type, int *index);

  static MJ_Eq2FilterParams defaultBandParam(uint band, uint bands);

  void resize(uint stages);


protected:
  
  bool updateFilterCoeffs();

  void clearFilterMemories();
  
  MJ_Array<MJ_Eq2FilterParams> m_params;

  /** Matrix of filters. Each row has one kind of filter (for all
      channels). The number of rows is the number of filter bands. The
      number of columns equals the number of channels. */
  
  MJ_Array2D<MJ_FilterIIR2<MJ_Sample> > m_filters;

  bool m_inLadspa;
  
  static bool m_debug;
};


#endif
