/* COPYRIGHT
 *
 * This file is part of Mustajuuri GPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the General Public License (GPL),
 * versions 2. Extra clauses can be found from the file MUSTAJUURI_GPL
 * and further explanations from the file MUSTAJUURI_LICENSE. The
 * license text can be found in file "GPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *  
 */

#ifndef MJ_MODULE_EQ2_GUI_H
#define MJ_MODULE_EQ2_GUI_H

#include <mj_module_dsp_gui_default.h>

class MJ_FreqRespGui;
class MJ_ModuleMenubar;

/** A gui for MJ_ModuleEq2 (parametric EQ). Also base class for using
    graphical EQ. */

class MJ_ModuleEq2_QG : public MJ_ModuleDspGuiPostCard
{
  Q_OBJECT
public:
  MJ_ModuleEq2_QG(MJ_RtControl *rtparent,
		  MJ_ModuleDsp *module,
		  bool          drawSliders,
		  QWidget      *parent = 0,
		  const char   *name = 0);

  virtual ~MJ_ModuleEq2_QG();

  virtual void moduleModified(const char *);

  static MJ_ModuleDspGui *createNew(MJ_RtControl *rtparent,
				    MJ_ModuleDsp *module,
				    QWidget *parent, 
				    const char *name);

  static int menuChoiceToBands(int);

public slots:

  virtual void grMousePress(QMouseEvent *, float, float);
  virtual void grMouseRelease(QMouseEvent *, float, float);

  void flatten();

  virtual void setBands(int);

protected:

  virtual void realTimeChange (bool nowRt);
  
  void drawEq(bool drawSliders);

  void updateEq();

  void updateBand(uint band);

  void seekActiveBand(float x, float y);

  void setPeak(float x, float y);

  float posToFreq(float x);

  virtual void setBand(double Hz, double gainDb);

  void rebuildSliders();

  void resizeFreqResp(uint stages);

  void setActiveBand(uint band);

  

  MJ_FreqRespGui *m_freqResp;

  QWidget        *m_sliderBase;
  QGridLayout    *m_sliderBaseLayout;
  QWidget        *m_sliderWindow;
  /// Objects that need to be deleted.
  QList<QObject>  m_objects;

  MJ_Array<void *>  m_guiComponents;
  MJ_ModuleMenubar *m_menuBar;
  int               m_bandItem;

  uint              m_activeBand;

  /** This variable is used to block multiple graph updates that
      would take place unnecessarily. */
  bool m_blockDraw;

  static bool m_debug;
};

#endif
