/* This is an example plugin for Mustajuuri.
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * Author: Tommi Ilmonen, 1999-2002.
 * Tommi.Ilmonen@hut.fi
 *
 * Do whatever you want with it.
 */

/* Include base  definitions. */

#include <mj_module_dsp.h>
#include <mj_dsp_plugin.h>

/** Module that squares the signal. The implementation could be more
    efficient, but in this case the code is optimized for
    readability. */

class MJ_ModuleMonoSquare : public MJ_ModuleDsp
{
public:  
  virtual ~MJ_ModuleMonoSquare() {}
  virtual void dspProcessPeriod()
  {
    // Get pointers to input and output buffers:
    MJ_Sample *input = m_signalInput.getFast(0).samples().data();
    MJ_Sample *output = m_signalOutput.getFast(0).samples().data();
    int samples = m_signalInput.getFast(0).samples().size();

    // Do the processing:
    for(int i=0; i < samples; i++)
      output[i] = input[i] * input[i];
  }
};

/// Module factory function
MJ_ModuleDsp * MJ_createNew_ModuleExample0() 
{
  return new MJ_ModuleMonoSquare;
}

/// Function that creates the plugin
MJ_PluginDbeDsp *MJ_createPlugin(int i)
{
  if(i != 0) return 0; // This function has already been called once.

  MJ_PluginDbeDsp *dbe = new MJ_PluginDbeDsp;
  dbe->setAuthors("Ilmonen Tommi S.");
  dbe->setMenuPosition("_Audio_Arithmetic_Square (mono)");
  dbe->setCreator(MJ_createNew_ModuleExample0);
  // Identifer string (must not contain blanks):
  dbe->setIdentifier("tommi_ilmonen_at_hut_fi:example0(=square)");
  // One input and one output channel:
  dbe->options()->defaultChans(1);
  return dbe;  
}
