/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of Mustajuuri GPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the General Public License (GPL),
 * versions 2. Extra clauses can be found from the file MUSTAJUURI_GPL
 * and further explanations from the file MUSTAJUURI_LICENSE. The
 * license text can be found in file "GPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *  
 */

#ifndef MJ_MODULE_EXAMPLE1_H
#define MJ_MODULE_EXAMPLE1_H

/* Include base  definitions. */
#include <mj_module_dsp.h>

/* Use ready-made filters. */
#include <mj_filter_ap1.h>

/* This plugin performs single channel low-pass filtering. 

   All plugins must inherit base class MJ_ModuleDsp.  All public
   methods of the class simply override virtual methods de3clared by
   MJ_ModuleDsp. */
class MJ_ModuleLowPass1 : public MJ_ModuleDsp
{
public:
  MJ_ModuleLowPass1();

  virtual ~MJ_ModuleLowPass1();
  
  /* This method is called by the host module before any signal
     processing takes place. Initializations can be performed here. */
  virtual bool dspPrepare();

  /* This method does the actual processing. The parent module calls this
     method repeatedly. Any practical plugin should redefine this method. */
  virtual void dspProcessPeriod();
  
  /* This method is used to receive input messages (or events).  The
     various child classes of MJ_Message make it possible to pass
     virtually any information via this interface. */
  virtual bool msgProcess(const char *, MJ_Message *);

  /* The types of input messages are declared here. It is hoped all
     modules would tell all of their input message types with this
     method.  */
  virtual MJ_Parameter * msgGetInput(const char *);

  /* Returns the name of the parameter (assuming the plugin has linear
     parameter space). */
  virtual QCString msgGetInputName(int);

protected:
  
  /* Tries to update filter coefficients. */
  void updateFilterCoeffs();

  /* The filter cutoff frequency in Hertz. */
  double m_cutOffHz;
  
  /* The filter that will be applied to the signal. Mustajuuri offers
     some sdignal processing primitives that can be used in
     plugins. In this case the most simple possible IIR filter is
     used.
     
     We use double as the filter coefficient sine the user may wish to
     use very low cutoff frequency, resulting in great numeric
     resolution requirements. */
  MJ_FilterAP1<double> m_filter;
  
  /* Static variable that can be used to switch on debugging mode. In
     that case the system starts to write all sorts of debugging
     information to the user (see mj_module_example1.C). */
  static bool m_debug;
};

#endif
