/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of Mustajuuri GPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the General Public License (GPL),
 * versions 2. Extra clauses can be found from the file MUSTAJUURI_GPL
 * and further explanations from the file MUSTAJUURI_LICENSE. The
 * license text can be found in file "GPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *  
 */

#ifndef MJ_MODULE_GRAPH_H
#define MJ_MODULE_GRAPH_H

#include <mj_module_dsp_array.h>

/** Module that implements a free-form DSP graph. The graph is given
    in a file that describes the graph. 

    @author Tommi Ilmonen
*/

class MJ_ModuleGraph : public MJ_ModuleDspArray
{
public:
  MJ_ModuleGraph();

  virtual ~MJ_ModuleGraph();
  
  virtual MJ_ModuleDsp *findModule(const char *id);

  virtual bool dspSetTimer(MJ_TimeStamp *t);

  virtual bool dspAcceptIoChans(int *, int *);

  virtual bool dspPrepare();
  virtual void dspProcessPeriod();
  virtual bool dspStop();

  virtual bool msgProcess(const char *, MJ_Message *);
  virtual MJ_Parameter * msgGetInput(const char *);

  virtual QCString msgGetInputName(int);

  static bool *debugPtr() { return & m_debug; }

protected:
  
  bool loadFile(const QString &file);

  QString m_fileName;
  uint    m_framesPerCycle;
  ulong   m_framesPerCycleNow;

  /// The number of samples processed by this graph.
  ulong   m_doneSamples;
  /// The number of samples given to the host system.
  ulong   m_givenSamples;
  /// The number of samples in the host system.
  ulong   m_externSamples;
  /// The number samples per cycle in the host system.
  ulong   m_externalFramesPerCycle;

  uint    m_inChans;
  uint    m_outChans;

  MJ_ModuleDspNull m_transfer;

  MJ_TimeStamp m_graphTimer;

  static bool m_debug;
};

#endif
