/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of Mustajuuri GPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the General Public License (GPL),
 * versions 2. Extra clauses can be found from the file MUSTAJUURI_GPL
 * and further explanations from the file MUSTAJUURI_LICENSE. The
 * license text can be found in file "GPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *  
 */

#ifndef MJ_MODULE_EXAMPLE1_H
#define MJ_MODULE_EXAMPLE1_H


#include <mj_module_dsp.h>

#include <mj_math.h>
#include <mj_ringbuffer.h>

/** Simple Karplus-Strong implementation.
*/
class MJ_ModuleKS1 : public MJ_ModuleDsp
{
public:
  MJ_ModuleKS1();

  virtual ~MJ_ModuleKS1();
  
  virtual bool dspPrepare();
  virtual void dspProcessPeriod();
  
  virtual bool msgProcess(const char *, MJ_Message *);
  virtual MJ_Parameter * msgGetInput(const char *);

  virtual QCString msgGetInputName(int);

protected:
  
  class Fir2
  {
  public:
    Fir2() { reset(); }

    void reset()
    { m_g0 = 0.5f; m_g1 = 0.5f; m_m = 0.0f; }
    
    float put(float v) { 

#ifdef __OS_LINUX__
      if(MJ_abs(v) < 1e-6f) v = 0.0f;
#endif

      float out = v * m_g0 + m_m * m_g1;
      m_m = v;
      return out;
    }

    float m_g0, m_g1;
    float m_m;
  };


  MJ_RingBufferDelay<float> m_buffer;
  Fir2                      m_filter;
  float                     m_feedback;
  float                     m_delayTime;

  int                       m_note;
};

#endif
