/* COPYRIGHT
 *
 * This file is part of Mustajuuri GPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the General Public License (GPL),
 * versions 2. Extra clauses can be found from the file MUSTAJUURI_GPL
 * and further explanations from the file MUSTAJUURI_LICENSE. The
 * license text can be found in file "GPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *  
 */

#include <mj_ladspa_driver.h>

#include <mj_error.h>
#include <mj_module_ladspa.h>
#include <mj_so_ref_count.h>
#include <mj_strs.h>

#include <set>

MJ_LadspaDriver::MJ_LadspaDriver(const LADSPA_Descriptor * handle, 
				 MJ_SoRefCount *ladspaSo)
  : m_handle (handle), m_ladspaSo(ladspaSo)
{ 
  ladspaSo->incrementReferenceCounter();
  ladspaToMj(); 
}

MJ_LadspaDriver::~MJ_LadspaDriver()
{
  // MJ_trace("MJ_LadspaDriver::~MJ_LadspaDriver", this);
  m_ladspaSo->decrementReferenceCounter();

  if(!m_ladspaSo->count()) delete m_ladspaSo;
}

MJ_ModuleDsp *MJ_LadspaDriver::newPlugin()
{
  // MJ_trace("MJ_LadspaDriver::newDriver");

  MJ_ModuleLadspa *m = new MJ_ModuleLadspa(m_handle);
  exportCreator(m);
  return m;
}

int MJ_LadspaDriver::indexToPort
(const LADSPA_Descriptor * handle, int mask, int index)
{
  if(index >= nofPorts(handle, mask)) return -1;

  int found = 0;

  for(ulong i=0; i < handle->PortCount; i++) {
    
    if((handle->PortDescriptors[i] & mask) == mask) {

      if(found == (int) index) return (int) i;
      
      found++;
    }
  }
  return -1; // Never to go here.
}


int MJ_LadspaDriver::nofPorts(const LADSPA_Descriptor *handle, int mask)
{
  MJ_cpf((void *) handle, "MJ_LadspaDriver::nofPorts");

  ulong ports = handle->PortCount;

  // MJ_trace("MJ_LadspaDriver::nofPorts", (int) ports);

  int valids = 0;

  for(ulong i=0; i < ports; i++)
    if((handle->PortDescriptors[i] & mask) == mask)
      valids++;
  
  // MJ_trace("MJ_LadspaDriver::nofPorts", "Exit");

  return valids;
}


void MJ_LadspaDriver::ladspaToMj()
{
  static std::set<QString> labels;

  // MJ_trace("MJ_LadspaDriver::ladspaToMj");

  MJ_DspOptions *opts = options();

  opts->defaultChans2(nofInputs(m_handle), nofOutputs(m_handle));
  opts->setValue(MJ_DSP_EQUAL_SAMPLE_RATE_IO, 1);

  QString id;
  int index = 0;

  for(bool ok = false; !ok; index++) {
    id = QString("Audio_LADSPA_") + m_handle->Label;
    
    if(index)
      id += " " + QString().setNum(index);
    
    if(labels.find(id) == labels.end())
      ok = true;
  }

  setIdentifier(id);
  setAuthors(m_handle->Maker);

  QString menuPos(QString("Audio_") + MJ_Strs::LADSPA_);
  menuPos += m_handle->Name;
  setMenuPosition(menuPos);
  
  setParameter(MJ_PP_COPYRIGHT, m_handle->Copyright);
}
