/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of Mustajuuri GPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the General Public License (GPL),
 * versions 2. Extra clauses can be found from the file MUSTAJUURI_GPL
 * and further explanations from the file MUSTAJUURI_LICENSE. The
 * license text can be found in file "GPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *  
 */

#ifndef MJ_LADSPA_DRIVER_H
#define MJ_LADSPA_DRIVER_H

#include <mj_dsp_plugin.h>

#include <ladspa.h>

/** LADSPA plugins. LADSPA plugin model does not match Mustajuuri
    model with one-to-one, but fairly well. */

class MJ_LadspaDriver : public MJ_PluginDbeDsp
{
public:

  MJ_LadspaDriver(const LADSPA_Descriptor * handle, 
		  MJ_SoRefCount *ladspaSo);

  virtual ~MJ_LadspaDriver();

  virtual MJ_ModuleDsp *newPlugin();

  const LADSPA_Descriptor * handle() { return m_handle; }

  static int nofInputs(const LADSPA_Descriptor * handle)
  { return nofPorts(handle, LADSPA_PORT_INPUT | LADSPA_PORT_AUDIO); }

  static int nofOutputs(const LADSPA_Descriptor * handle)
  { return nofPorts(handle, LADSPA_PORT_OUTPUT | LADSPA_PORT_AUDIO); }
  
  static int nofControlIns(const LADSPA_Descriptor * handle)
  { return nofPorts(handle, LADSPA_PORT_INPUT | LADSPA_PORT_CONTROL); }
  
  static int indexToPort(const LADSPA_Descriptor * handle, 
			 int mask, 
			 int index);

  static int nofPorts(const LADSPA_Descriptor * handle, int mask);

protected:

  /** Fills Mustajuuri data structures from LADSPA data structures. */

  void ladspaToMj();

  const LADSPA_Descriptor * m_handle;

  MJ_SoRefCount *m_ladspaSo;
};

#endif
