/* COPYRIGHT
 *
 * This file is part of Mustajuuri GPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the General Public License (GPL),
 * versions 2. Extra clauses can be found from the file MUSTAJUURI_GPL
 * and further explanations from the file MUSTAJUURI_LICENSE. The
 * license text can be found in file "GPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *  
 */

#include <mj_ladspa_driver.h>

#include <mj_dsp_plugin_loader.h>
#include <mj_error.h>
#include <mj_paths.h> 
#include <mj_so_ref_count.h>
#include <mj_version.h>

#include <dlfcn.h>

#include <qstringlist.h>

static MJ_PluginDbeList MJ_ladspaToMjList;

/// Load LADSPA plugins in a particular library.
static void MJ_loadLadspaPlugin(const QString &path)
{
  const char * fname = "MJ_loadLadspaPlugin";

  MJ_trace(fname, path);

  void *handle = dlopen(path, RTLD_NOW);
  
  if(!handle) {
    MJ_error2(MJ_ERR_SYSTEM, "%s # dlopen failed %s", fname, dlerror());
    
    return;
  }
  
  LADSPA_Descriptor_Function  descrFunc 
    = (LADSPA_Descriptor_Function) dlsym(handle,
					 "ladspa_descriptor");
  
  if(!descrFunc) {
    MJ_error(MJ_ERR_SYSTEM, fname, "dlsym failed");
    return;
  }
  
  // MJ_trace(fname, "second stage");

  MJ_SoRefCount * socount = new MJ_SoRefCount(handle, path);

  for(ulong i = 0; ; i++) {
    const LADSPA_Descriptor *descr = descrFunc(i);

    if(!descr) 
      return;
    else {

      // MJ_trace(fname, "third stage");
      
      MJ_LadspaDriver *plugin = new MJ_LadspaDriver(descr, socount);
      MJ_cpf(plugin, fname);
      
      MJ_ladspaToMjList.append(plugin);
    }
  }
}

/// Load all LADSPA plugins.
static void MJ_LadspaInit()
{
  const char * fname = "mj_ladspa_import.C:MJ_LadspaInit";

  MJ_trace(fname);

  MJ_Paths paths;
  
  paths.addPathVariable("LADSPA_PATH");

  uint nofPaths = paths.paths();
  
  if(!nofPaths) {
    paths.addPath("/usr/lib/ladspa:/usr/local/lib/ladspa");
    nofPaths = paths.paths();    
  }

  /* if(!nofPaths) {
    MJ_error(MJ_ERR_UNKNOWN, fname, "No paths");
    return;
    }*/

  for(uint i=0; i < nofPaths; i++) {
    QString path = paths.path(i);
  
    QFileInfo pathInfo(path);

    if(!pathInfo.isDir())
      continue;

    QDir d(path);
    
    QStringList files = d.entryList("*.so");

    if(!files.count()) {
      MJ_error2(MJ_ERR_ARGUMENT, 
		"%s # empty directory %s", fname, (const char *) path);
    }
    else {
      for(QStringList::Iterator it = files.begin(); 
	  it != files.end(); ++it )
	MJ_loadLadspaPlugin(path + "/" + (*it));
    }
  }
}

MJ_PluginDbeDsp *MJ_createPlugin(int i)
{
  if(!i) MJ_LadspaInit();
  
  if((uint) i < MJ_ladspaToMjList.count())
    return (MJ_PluginDbeDsp *) MJ_ladspaToMjList.at(i);

  return 0;
}
