/* COPYRIGHT
 *
 * This file is part of Mustajuuri GPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the General Public License (GPL),
 * versions 2. Extra clauses can be found from the file MUSTAJUURI_GPL
 * and further explanations from the file MUSTAJUURI_LICENSE. The
 * license text can be found in file "GPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *  
 */

#ifndef MJ_MODULE_LATENCYTEST_H
#define MJ_MODULE_LATENCYTEST_H

#include <mj_module_dsp.h>

#include <mj_module_dsp_gui_default.h> 
#include <mj_random.h> 

#include <qtimer.h> 

/* This plugin does nothign, instead it is used to test the
   automagical UI generation.
*/
class MJ_ModuleLatencyTest : public MJ_ModuleDsp
{
public:
  MJ_ModuleLatencyTest();

  virtual ~MJ_ModuleLatencyTest();
  
  virtual bool dspPrepare();
  virtual void dspProcessPeriod();
  virtual bool msgProcess(const char *, MJ_Message *);

  virtual MJ_Parameter * msgGetInput(const char *);

  virtual QCString msgGetInputName(int);

  long timeSamples() const { return m_timeSamples; }

protected:
  MJ_Sample m_threshold;
  MJ_Sample m_testIntervalMs;
  ulong     m_testIntervalSamples;
  ulong     m_burstSamples;
  
  long      m_timeSamples;
  bool      m_listen;
  ulong     m_counter;
  ulong     m_sendTime;

  MJ_Random m_noiseSource;

  static const int m_params = 6;
};

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

class QLabel;

class MJ_ModuleLatencyTest_QG : public MJ_ModuleDspGuiPostCard
{
  Q_OBJECT
public:
  MJ_ModuleLatencyTest_QG(MJ_RtControl *rtparent,
			  MJ_ModuleDsp *module,
			  QWidget      *parent = 0,
			  const char   *name = 0);
  
  static MJ_ModuleDspGui *createNew(MJ_RtControl *rtparent,
				    MJ_ModuleDsp *module,
				    QWidget *parent, 
				    const char *name);

public slots:
  void update();

protected:

  virtual void realTimeChange (bool nowRt);

  QLabel *m_display;
  QTimer  m_timer;
};



#endif
