/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of Mustajuuri GPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the General Public License (GPL),
 * versions 2. Extra clauses can be found from the file MUSTAJUURI_GPL
 * and further explanations from the file MUSTAJUURI_LICENSE. The
 * license text can be found in file "GPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *  
 */

#ifndef MJ_MODULE_1TON_H
#define MJ_MODULE_1TON_H

#include <mj_module_dsp.h>
#include <mj_ramp.h>

#include <vector>

/** A module that transfers the single input signal to one of the n
    output signals.

    @author Tommi Ilmonen
 */

class MJ_Module1ToN : public MJ_ModuleDsp
{
public:
  MJ_Module1ToN();
  virtual ~MJ_Module1ToN();
  
  virtual bool dspAcceptIoChans(int *, int *);

  virtual bool dspPrepare();
  virtual void dspProcessPeriod();

  virtual bool msgProcess(const char *, MJ_Message *);
  virtual MJ_Parameter * msgGetInput(const char *);
  virtual QCString msgGetInputName(int);

  static uint maxChans() { return m_maxChans; }

protected:

  std::vector<MJ_Ramp<MJ_Sample> >  m_interpolators;
  
  uint m_outChannels;
  uint m_chosenChannel;
  
  static const uint m_maxChans = 128;
};

#endif
