/* COPYRIGHT
 *
 * This file is part of Mustajuuri GPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the General Public License (GPL),
 * versions 2. Extra clauses can be found from the file MUSTAJUURI_GPL
 * and further explanations from the file MUSTAJUURI_LICENSE. The
 * license text can be found in file "GPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *  
 */

#ifndef MJ_MODULE_OVERECHO_GUI_H
#define MJ_MODULE_OVERECHO_GUI_H

#include <mj_module_dsp_gui_default.h>

#include <mj_graphics_area.h> 
#include <mj_multi_sample.h> 

#include <qgroupbox.h> 

class MJ_ModuleOverecho;
class MJ_ModuleOverecho_QG;
class MJ_GraphicPolygon;
class MJ_GraphicPolyLine;
class MJ_PointArray2D;

class QComboBox;
class QGridLayout;
class QTabWidget;
class QPushButton;

class MJ_OverEchoResize : public QGroupBox, public MJ_RtControl
{
  Q_OBJECT
public:
  MJ_OverEchoResize(MJ_ModuleOverecho_QG *echoGui,
		    MJ_RtControl *rtparent,
		    QWidget      *parent = 0,
		    const char   *name = 0);

  virtual ~MJ_OverEchoResize();

public slots:
  void apply();

  void setPredelays(int);
  void setAllpasses(int);
  void setOutputs(int);

protected:
  virtual void realTimeChange(bool nowRt);

  /// Predelays, allpasses, outputs
  int m_sizes[3];

  QComboBox *m_combos[3];
  QPushButton *m_applyButton;
  MJ_ModuleOverecho_QG *m_echoGui;
};

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

/// Gui for MJ_ModuleOverecho
class MJ_ModuleOverecho_QG : public MJ_ModuleDspGuiPostCard
{
  Q_OBJECT
public:
  MJ_ModuleOverecho_QG(MJ_RtControl *rtparent,
		       MJ_ModuleDsp *module,
		       QWidget      *parent = 0,
		       const char   *name = 0);
  
  virtual ~MJ_ModuleOverecho_QG();

  virtual void moduleModified(const char *);

  static MJ_ModuleDspGui *createNew(MJ_RtControl *rtparent,
				    MJ_ModuleDsp *module,
				    QWidget *parent, 
				    const char *name);

  void forgetResize() { m_resize = 0; }

  MJ_ModuleOverecho *echo() { return m_echo; }

  void resizeEcho(int predelays, int allpasses, int outputs);

public slots:

  void handleParam(const char *,double);
  void resizer();


protected:
  
  void draw();

  void redrawGraphics();

  MJ_GraphicPolyLine *addPolyline(MJ_PointArray2D *points, 
				  MJ_GraphicsContext *gc);

  MJ_GraphicPolygon *addPolygon(MJ_PointArray2D *points, 
				MJ_GraphicsContext *gc);

  void rebuildTw();

  QWidget *predelaySliders();

  void buildPredelay(QGridLayout *l,
		     QWidget *w,
		     MJ_QuadSample *qs, 
		     int row, 
		     int predelay, 
		     int chan);

  QWidget *allpassSliders();

  void buildAllpass(QGridLayout *l,
		    QWidget *w,
		    MJ_TriSample *ts, 
		    int row, 
		    int index,
		    float maxTime);

  QWidget *outputSliders();

  void buildOutput(QGridLayout *l,
		   QWidget *w,
		   MJ_DualSample *ds, 
		   int row, 
		   int index,
		   int channel,
		   float maxTime);

  QWidget *miscellanous();

  QWidget *feedbackParams();

  void sliderConnect(QObject *s, const char * str);

  void buildGui(QWidget *w, const char **paramIds, uint nofIds);

  MJ_ModuleOverecho *m_echo;

  MJ_GraphicsArea m_graphics;

  MJ_OverEchoResize *m_resize;

  QList<QObject>     m_children;
  
  QTabWidget        *m_tw;
  QWidget           *m_twChildren[5];
  static bool m_debug;
};

#endif
