/* COPYRIGHT
 *
 * This file is part of Mustajuuri GPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the General Public License (GPL),
 * versions 2. Extra clauses can be found from the file MUSTAJUURI_GPL
 * and further explanations from the file MUSTAJUURI_LICENSE. The
 * license text can be found in file "GPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *  
 */

#include <mj_module_pitch2.h>

#include <mj_color.h> 
#include <mj_display_type.h> 
#include <mj_error.h>
#include <mj_graphic_polygon.h>
#include <mj_graphics_context_simple.h> 
#include <mj_gui_utils.h> 
#include <mj_parameters.h>
#include <mj_math.h>
#include <mj_strs.h>

#include <qlabel.h> 
#include <qlayout.h> 

MJ_ModulePitch2::MJ_ModulePitch2()
{
  m_freqs.resize(KEEP_TIMES);
}

/* The descturctor does not really need to do anything. All member
   variables destroy themselves. */
MJ_ModulePitch2::~MJ_ModulePitch2()
{}

bool MJ_ModulePitch2::dspPrepare()
{
  m_freqs.reset();

  bool ok = MJ_ModulePitch1::dspPrepare();

  return ok;
}

void MJ_ModulePitch2::dspProcessPeriod()
{
  MJ_ModulePitch1::dspProcessPeriod();
}

bool MJ_ModulePitch2::msgProcess(const char * id, MJ_Message *m)
{
  // const char *fname = "MJ_ModulePitch2::msgProcess";

  /* if(!strcmp(id, MJ_Strs::frequency)||
     !strcmp(id, MJ_Strs::threshold))  */
  return MJ_ModulePitch1::msgProcess(id, m);

  // return false;
}

MJ_Parameter * MJ_ModulePitch2::msgGetInput(const char * id)
{
  if(m_debug) trace("MJ_ModulePitch2::msgGetInput", id);
  
  /* if(!strcmp(id, MJ_Strs::frequency)||
     !strcmp(id, MJ_Strs::threshold))  */
  return MJ_ModulePitch1::msgGetInput(id);
  
  // return 0;
}

QCString MJ_ModulePitch2::msgGetInputName(int i)
{
  /* QCString s;
  
  switch(i) {
  case 0:
    s = MJ_Strs::frequency;
    break;
    
  case 1:
    s = MJ_Strs::threshold;
    break;
    } */
    
  return MJ_ModulePitch1::msgGetInputName(i);
}

void MJ_ModulePitch2::newFreq(ulong sampleInd, float freq)
{
  m_freqs.advance();
  MJ_ULongFloat *spec = & m_freqs.getNewest();
  spec->m_ulong = sampleInd;
  spec->m_float = freq;
}



/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

MJ_ModulePitch2_QG::MJ_ModulePitch2_QG(MJ_RtControl *rtparent,
				       MJ_ModuleDsp *module,
				       QWidget      *parent,
				       const char   *name)
  : MJ_ModulePitch1_QG(rtparent, module, parent, name),
    m_tuningCenter(440.0),
    m_graphics(this),
    m_needle(new MJ_GraphicPolygon()),
    m_noteHolder(0),
    m_currentNote(0)
{
  const uint nofNotes = MJ_NOF_NOTES;

  const char * fname = "MJ_ModulePitch2_QG::MJ_ModulePitch2_QG";

  if(!dynamic_cast<MJ_ModulePitch2 *> (module)) 
    MJ_fatal(MJ_ERR_ARGUMENT, fname);

  m_graphics.resize(100, 100);
  m_graphics.setMinimumSize(100, 100);
  m_graphics.setBoundingBox(-0.5, 0.5, 0.5, -0.5);
  m_graphics.appendGraphic(m_needle);
  m_graphics.setBackgroundColor(Qt::black);

  MJ_GraphicsContextPenBrush *gcpb = 
    new MJ_GraphicsContextPenBrush(QPen(Qt::red), QBrush(Qt::red));
  m_needle->setGraphicsContext(gcpb);

  int rows = m_layout->numRows();
  m_layout->expand(rows + 2, m_layout->numCols());

  // Note names:
  m_noteHolder = new QWidget(this);

  m_layout->addMultiCellWidget(m_noteHolder, rows, rows, 0, 2);
  m_noteHolder->show();
  
  m_holderLayout = new QHBoxLayout(m_noteHolder, 2, 2);
  m_noteNames.resize(nofNotes);

  QFont f = font();
  if(f.pixelSize() < 14)
    f.setPixelSize(14);

  for(uint i=0; i < nofNotes; i++) {
    QLabel *label = new QLabel(MJ_Strs::notes[i], m_noteHolder);
    label->setBackgroundColor(Qt::gray);
    label->setAlignment(AlignHCenter | AlignVCenter);
    label->setFont(f);
    label->setFrameShape(QFrame::Panel);
    label->setFrameShadow(QFrame::Plain);
    label->setLineWidth(2);
    label->setMargin(3);
    MJ_limMinSize(label);
    m_holderLayout->addWidget(label);
    m_noteNames.getFast(i) = label;
  }
  
  // The needle that shows offset:
  rows++;
  m_layout->addMultiCellWidget(&m_graphics, rows, rows, 0, 2);
  m_layout->setRowStretch(rows, 80);

  for(uint i=0; i < nofNotes; i++) {
    m_noteNames.getFast(i)->show();
  }

  m_graphics.setSquarePixels(true);
  m_graphics.show();

  m_holder.set(0, 0.5);
  
  if(isRealTime()) {
    realTimeChange(true);
  }
}
  
void MJ_ModulePitch2_QG::update()
{
  // The frequency display
  MJ_ModulePitch1_QG::update();

  lock();
  float freq = ((MJ_ModulePitch1 *) m_module)->freq();
  unlock();
  
  // 1) First the note
  float closestFreq;
  float offset;
  int noteInd;
  
  closestNoteFreq(freq, &closestFreq, &offset, &noteInd);

  m_holder.putCyclic(offset, 50.0);
  // m_holder.putLinear(offset);
  offset = m_holder.get();
  
  int octave = 0;

  while(noteInd < 0) {
    noteInd += MJ_NOF_NOTES;
    octave --;
  }
  while(noteInd >= MJ_NOF_NOTES) {
    noteInd -= MJ_NOF_NOTES;
    octave ++;
  }

  if(m_currentNote != noteInd) {
    if(m_currentNote >= 0) {
      m_noteNames.get(m_currentNote)->setBackgroundColor(Qt::gray);
    }
    m_noteNames.get(noteInd)->setBackgroundColor(Qt::red);
    m_currentNote = noteInd;
  }

  // 2) Graphics:

  MJ_PointArray2D pa;

  float angle = - offset / 50.0 * MJ_pi + MJ_pi2;

  float as = MJ_sin(angle);
  float ac = MJ_cos(angle);

  float tipY = as * 0.4;
  float tipX = ac * 0.4;

  pa.resize(3);
  pa.setPoint(0, tipX, tipY);
  tipY *= 0.2;
  tipX *= 0.2;
  pa.setPoint(1, tipY, -tipX);
  pa.setPoint(2, -tipY, tipX);

  m_needle->setGraph(pa.qarray());

  MJ_GraphicsContextPenBrush *gcpb = (MJ_GraphicsContextPenBrush *)
    m_needle->graphicsContext();
  
  QColor color;

  float mix = offset / 50;

  if(mix < 0) {
    mix = -mix;
    color = (Qt::blue * mix + Qt::white * (1.0 - mix));
  }
  else {
    color = (Qt::red * mix + Qt::white * (1.0 - mix));
  }

  gcpb->pen().setColor(color);
  gcpb->brush().setColor(color);
  
  m_graphics.redraw(MJ_DisplayType::localDisplay());
}

MJ_ModuleDspGui *MJ_ModulePitch2_QG::createNew(MJ_RtControl *rtparent,
						    MJ_ModuleDsp *module,
						    QWidget *parent, 
						    const char *name)
{
  return new MJ_ModulePitch2_QG(rtparent, module, parent, name);
}

void MJ_ModulePitch2_QG::closestNoteFreq(float freq, 
					 float *closest, 
					 float *offset, 
					 int *ind)
{
  double dfreqRel = freq / m_tuningCenter;

  double semitone = pow(2.0, 1.0 / 12.0);

  const int range = 12 * 5;
  int best = 0;
  double bestDiff = 1e+10;
  double bestFreq = 0.0;

  for(int i= -range; i < range; i++) {

    double tmp = pow(semitone, (double) i);
    double diff;

    if(tmp < dfreqRel)
      diff = dfreqRel / tmp - 1.0;
    else
      diff = tmp / dfreqRel - 1.0;
    
    if (diff < bestDiff) {
      best = i;
      bestDiff = diff;
      bestFreq = tmp;
    }
  }

  *offset = MJ_mulToCent(dfreqRel / bestFreq);
  
  bestFreq *= m_tuningCenter;

  *closest = bestFreq;
  *ind = best;
}

