/* COPYRIGHT
 *
 * This file is part of Mustajuuri GPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the General Public License (GPL),
 * versions 2. Extra clauses can be found from the file MUSTAJUURI_GPL
 * and further explanations from the file MUSTAJUURI_LICENSE. The
 * license text can be found in file "GPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *  
 */

#ifndef MJ_MODULE_PITCH2_H
#define MJ_MODULE_PITCH2_H

#include <mj_module_pitch1.h>

#include <mj_graphics_area.h>

#include <mj_value_hold.h>

class QHBoxLayout;

// This should be a configurable parameter.
#define MJ_NOF_NOTES 12

class MJ_ULongFloat 
{
 public:
  MJ_ULongFloat() : m_ulong(0), m_float(0.0) {}
  MJ_ULongFloat(int x) : m_ulong(x), m_float(0.0) {}

  MJ_ULongFloat & operator = (const int &x)
  {
    m_ulong = x;
    m_float = 0.0;
    return *this;
  }

  ulong m_ulong;
  float m_float;
};

/* This plugin performs single channel low-pass filtering. 

   All plugins must inherit base class MJ_ModuleDsp.  All public
   methods of the class simply override virtual methods de3clared by
   MJ_ModuleDsp. */
class MJ_ModulePitch2 : public MJ_ModulePitch1
{
public:

  MJ_ModulePitch2();

  virtual ~MJ_ModulePitch2();
  
  virtual bool dspPrepare();
  virtual void dspProcessPeriod();
  virtual bool msgProcess(const char *, MJ_Message *);

  virtual MJ_Parameter * msgGetInput(const char *);

  virtual QCString msgGetInputName(int);

protected:

  virtual void newFreq(ulong sampleInd, float freq);

  MJ_RingBufferDelay<MJ_ULongFloat> m_freqs;
};

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

class QLabel;
class MJ_GraphicPolygon;

class MJ_ModulePitch2_QG : public MJ_ModulePitch1_QG
{
  Q_OBJECT
public:
  MJ_ModulePitch2_QG(MJ_RtControl *rtparent,
		     MJ_ModuleDsp *module,
		     QWidget      *parent = 0,
		     const char   *name = 0);
  
  static MJ_ModuleDspGui *createNew(MJ_RtControl *rtparent,
				    MJ_ModuleDsp *module,
				    QWidget *parent, 
				    const char *name);

  virtual void update();

protected:

  void closestNoteFreq(float freq, 
		       float *closest, 
		       float *offset, 
		       int *ind);

  float              m_tuningCenter;

  MJ_GraphicsArea    m_graphics;
  MJ_GraphicPolygon *m_needle;

  QWidget           *m_noteHolder;
  QHBoxLayout       *m_holderLayout;
  MJ_Array<QWidget*> m_noteNames;
  int                m_currentNote;

  MJ_ValueHold<float> m_holder;
};




#endif
