/* COPYRIGHT
 *
 * This file is part of Mustajuuri GPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the General Public License (GPL),
 * versions 2. Extra clauses can be found from the file MUSTAJUURI_GPL
 * and further explanations from the file MUSTAJUURI_LICENSE. The
 * license text can be found in file "GPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *  
 */

#include <mj_module_pitch2.h>

#include <mj_dsp_plugin.h>
#include <mj_version.h>

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

MJ_ModuleDsp * MJ_createNew_ModulePitch1() 
{
  return new MJ_ModulePitch1;
}

MJ_PluginDbeDsp * libmjpitch1_CREATE_DSPMODULE()
{
  MJ_PluginDbeDsp *dbe = new MJ_PluginDbeDsp;

  dbe->setAuthors("Ilmonen Tommi S.");
  dbe->setMenuPosition("_Audio_Analysis_Pitch detector");
  dbe->setCreator(MJ_createNew_ModulePitch1);
  dbe->setIdentifier("Tommi.Ilmonen@hut.fi_analysis_pitch_detector1");
  dbe->pluginVersion()->set(0,0,1);
  dbe->options()->defaultChans2(1, 0);
  dbe->setParameter(MJ_PP_TRANSLATION_FILE, "mj_pitch");
  dbe->setGuiFactory(MJ_GUI_POSTCARD, MJ_ModulePitch1_QG::createNew);
  
  return dbe;  
}

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

MJ_ModuleDsp * MJ_createNew_ModulePitchAc() 
{
  return new MJ_ModulePitchAc;
}

MJ_PluginDbeDsp * libmjpitchAc_CREATE_DSPMODULE()
{
  MJ_PluginDbeDsp *dbe = new MJ_PluginDbeDsp;

  dbe->setAuthors("Ilmonen Tommi S.");
  dbe->setMenuPosition("_Audio_Analysis_Pitch detector (autocorrelation)");
  dbe->setCreator(MJ_createNew_ModulePitchAc);
  dbe->setIdentifier("Tommi.Ilmonen@hut.fi_analysis_pitch_detector_ac");
  dbe->pluginVersion()->set(0,0,1);
  dbe->options()->defaultChans2(1, 0);
  // dbe->setParameter(MJ_PP_TRANSLATION_FILE, <mj_pitch");
  dbe->setGuiFactory(MJ_GUI_POSTCARD, MJ_ModulePitch1_QG::createNew);
  
  return dbe;  
}


/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

MJ_ModuleDsp * MJ_createNew_ModulePitch2() 
{
  return new MJ_ModulePitch2;
}

MJ_PluginDbeDsp * libmjpitch2_CREATE_DSPMODULE()
{
  MJ_PluginDbeDsp *dbe = new MJ_PluginDbeDsp;

  dbe->setAuthors("Ilmonen Tommi S.");
  dbe->setMenuPosition("_Audio_Analysis_Tuning meter");
  dbe->setCreator(MJ_createNew_ModulePitch2);
  dbe->setIdentifier("Tommi.Ilmonen@hut.fi_analysis_tuning_meter");
  dbe->pluginVersion()->set(0,0,1);
  dbe->options()->defaultChans2(1, 0);
  // dbe->setParameter(MJ_PP_TRANSLATION_FILE, <mj_pitch2");
  dbe->setGuiFactory(MJ_GUI_POSTCARD, MJ_ModulePitch2_QG::createNew);
  
  return dbe;  
}

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

static MJ_PluginDbeDsp * __MJ_createPlugin(int i)
{
  switch(i) {
  case 0:
    return libmjpitch1_CREATE_DSPMODULE();
  case 1:
    return libmjpitch2_CREATE_DSPMODULE();
  case 2:
    return libmjpitchAc_CREATE_DSPMODULE();
  }
  return 0;
}

static QString compileDate(DATETIME);

MJ_PluginDbeDsp *MJ_createPlugin(int m)
{
  MJ_PluginDbeDsp * dbe = __MJ_createPlugin(m);

  if(dbe) dbe->setParameter(MJ_PP_COMPILE_TIME, compileDate);

  return dbe;
}

