/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of Mustajuuri GPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the General Public License (GPL),
 * versions 2. Extra clauses can be found from the file MUSTAJUURI_GPL
 * and further explanations from the file MUSTAJUURI_LICENSE. The
 * license text can be found in file "GPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *  
 */

#ifndef MJ_MODULE_REVERB1_H
#define MJ_MODULE_REVERB1_H

#include <mj_filter_ap1.h>
#include <mj_module_dsp.h>
#include <mj_all_pass_operator.h>

#include <vector>

enum MJ_ParamsReverb1 {
  MJ_PARREV_PREDELAY_TIME,
  MJ_PARREV_PREDELAY_SPREAD_TIME,
  MJ_PARREV_PREDELAY_SPREAD,
  MJ_PARREV_PREDELAY_COLOR,
  MJ_PARREV_TAIL_TIME,
  MJ_PARREV_TAIL_SPREAD,
  MJ_PARREV_TAIL_COLOR,
  MJ_PARREV_TAIL_FEEDBACK,
  MJ_PARREV_DRY_WET,
  MJ_PARREV_SIZEOF
};

extern const char * MJ_ParamsReverb1Keys[MJ_PARREV_SIZEOF];

class MJ_ReverbTailNode
{
public:
  MJ_ReverbTailNode() : m_predelayTime(100), m_delayTime(100), m_val(0.0) {}

  float putGet(MJ_RingBufferDelay<float> *preDelBuf, 
	       float v2, float feedBack) {
    m_feedbackFilter.put(v2 * feedBack + preDelBuf->getNewest(m_predelayTime));
    float tmp = m_feedbackFilter.get();
    m_delay.put(tmp);
    m_operators[0].operate(m_delay);
    m_operators[1].operate(m_delay);
    m_operators[2].operate(m_delay);
    m_val = m_delay.getNewest(m_delayTime);
    return m_val;
  }
  
  float get() { return m_val; }

  MJ_RingBufferDelay<float> m_delay;
  MJ_AllPassOperator<float> m_operators[3];
  MJ_FilterAP1<float>       m_feedbackFilter;

  uint  m_predelayTime;
  uint  m_delayTime;
  float m_val;
};

/** Reverb module. 

    @author Tommi Ilmonen
 */

class MJ_ModuleReverb1 : public MJ_ModuleDsp
{
public:
  MJ_ModuleReverb1();
  virtual ~MJ_ModuleReverb1();
  
  virtual bool dspPrepare();
  virtual void dspProcessPeriod();

  virtual bool msgProcess(const char *, MJ_Message *);
  virtual MJ_Parameter * msgGetInput(const char *);
  virtual QCString msgGetInputName(int);

protected:

  void allocateTails(uint);
  void calculateTails();

  MJ_RingBufferDelay<float>      m_predelayBuffer;

  std::vector<MJ_ReverbTailNode> m_tailNodes;

  MJ_FilterAP1<float>            m_feedbackFilterHighPass;

  float m_predelayMs;
  float m_predelaySpreadTimeMs;
  float m_predelaySpread;
  float m_predelayColor;

  float m_tailSpreadTimeMs;
  float m_tailSpread;
  float m_tailColor;
  float m_tailFeedback;  

  float m_wetDryRatio;
};

#endif
