/* COPYRIGHT
 *
 * This file is part of Mustajuuri GPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the General Public License (GPL),
 * versions 2. Extra clauses can be found from the file MUSTAJUURI_GPL
 * and further explanations from the file MUSTAJUURI_LICENSE. The
 * license text can be found in file "GPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *  
 */

#ifndef MJ_MODULE_SCOPE1_H
#define MJ_MODULE_SCOPE1_H

#include <mj_graphics_area.h>
#include <mj_module_dsp.h>
#include <mj_module_dsp_gui_default.h>
#include <mj_oscillosscope.h>

#include <qtimer.h>  

/** Scope detector. */
class MJ_ModuleScope1 : public MJ_ModuleDsp
{
public:
  MJ_ModuleScope1();

  virtual ~MJ_ModuleScope1();
  
  virtual bool dspPrepare();
  virtual void dspProcessPeriod();
  virtual bool msgProcess(const char *, MJ_Message *);

  virtual MJ_Parameter * msgGetInput(const char *);

  virtual QCString msgGetInputName(int);

  ulong getWave(MJ_Array<float> *);

  ulong news() const {return m_news;}

protected:
  
  void updateFilterCoeffs();
  
  MJ_Oscilloscope m_scope;

  MJ_Sample m_cutOffHz;
  
  ulong m_news;

  static bool m_debug;
};

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

class MJ_GraphicPolyLine;

class MJ_ModuleScope1_QG : public MJ_ModuleDspGuiPostCard
{
  Q_OBJECT
public:
  MJ_ModuleScope1_QG(MJ_RtControl *rtparent,
		     MJ_ModuleDsp *module,
		     QWidget      *parent = 0,
		     const char   *name = 0);
  
  static MJ_ModuleDspGui *createNew(MJ_RtControl *rtparent,
				    MJ_ModuleDsp *module,
				    QWidget *parent, 
				    const char *name);

public slots:

  virtual void update();

protected:

  virtual void realTimeChange (bool nowRt);
  
  MJ_Vector<float> m_waveData;

  MJ_GraphicsArea     m_graphics;
  MJ_GraphicPolyLine *m_waveForm;

  QTimer  m_timer;

  ulong m_news;
};

#endif
