/* COPYRIGHT
 *
 * This file is part of Mustajuuri GPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the General Public License (GPL),
 * versions 2. Extra clauses can be found from the file MUSTAJUURI_GPL
 * and further explanations from the file MUSTAJUURI_LICENSE. The
 * license text can be found in file "GPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *  
 */

#include <mj_oscillosscope.h>

MJ_Oscilloscope::MJ_Oscilloscope(uint maxCycle)
  : m_threshold(0.2)
{
  m_data.resize(maxCycle);
  m_data.setAll(0.0);
  designFilter(0.1);
  clear();
}

/** Put many samples to the scope. This function returns true if new
    wave(s) was detected. The data in input samples is saved to
    the internal buffer. */
bool MJ_Oscilloscope::putSamples(const MJ_Sample *s, uint count)
{
  float oldS = m_filter.get();

  MJ_ScopeState state = m_state;

  bool hit = false;

  while(count) {
    
    float newS = m_filter.putGetSample(*s);

    if(newS > m_threshold) {
      state = MJ_SCOSTA_ARMED;
    }
    
    if((state == MJ_SCOSTA_ARMED) && (newS > 0) && (oldS <= 0)) {
      
      float diff = newS - oldS;
      float fraction = newS / diff;

      m_lastStart = m_lastEnd;
      m_lastEnd = (m_data.getSampleCount() << 8) + (ulong) (fraction * 256);
      
      state = MJ_SCOSTA_WAIT;
      hit = true;
    }

    m_data.advance();
    m_data.getNewest() = *s++;

    oldS = newS;
    count--;
  }
  m_state = state;

  return hit;
}
  
void MJ_Oscilloscope::clear()
{
  m_state = MJ_SCOSTA_WAIT;
  m_lastStart = 0;
  m_lastEnd = 0;
  m_data.setAll(0);
  m_filter.resetMemory();
}
