/* COPYRIGHT
 *
 * This file is part of Mustajuuri GPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the General Public License (GPL),
 * versions 2. Extra clauses can be found from the file MUSTAJUURI_GPL
 * and further explanations from the file MUSTAJUURI_LICENSE. The
 * license text can be found in file "GPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *  
 */

#ifndef MJ_OSCILLOSSCOPE_H
#define MJ_OSCILLOSSCOPE_H

#include <mj_filter_iir2.h>
#include <mj_ringbuffer.h>

#include <mj_config.h>

/** Simple oscillosscope class.
    
    The start and stop times are recorded with sub-sample
    accuracy. Thus the time information has its 8 LSBs indicating the
    fractional part and the MSBs indicating integer values. */

class MJ_Oscilloscope
{
public: 
  /**  Constructs an oscillosscope object. The internal delay buffer
       will have at least maxCycle elements. */
  MJ_Oscilloscope(uint maxCycle = 128);

  ~MJ_Oscilloscope() {}

  bool putSamples(const MJ_Sample *s, uint count);
  
  MJ_RingBuffer2<float> & buffer() {return m_data;}
  
  void designFilter(float omega) 
    { m_filter.designLowPass1(omega, 1.0); }

  void setThreshold(float threshold) { m_threshold = threshold; }
  float threshold() const {return m_threshold;}

  ulong waveStartSamples() const { return m_lastStart >> 8;}
  ulong waveEndSamples() const { return m_lastEnd >> 8;}

  void clear();

protected:

  enum MJ_ScopeState {
    MJ_SCOSTA_WAIT,
    MJ_SCOSTA_ARMED    
  };

  MJ_RingBuffer2<float>        m_data;
  MJ_FilterIIR2<MJ_Sample>     m_filter;
  float                        m_threshold;
  
  /// Start and end times of the last wave.
  ulong                 m_lastStart, m_lastEnd;
  MJ_ScopeState         m_state;
};


#endif
