/* COPYRIGHT
 *
 * This file is part of Mustajuuri GPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the General Public License (GPL),
 * versions 2. Extra clauses can be found from the file MUSTAJUURI_GPL
 * and further explanations from the file MUSTAJUURI_LICENSE. The
 * license text can be found in file "GPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *  
 */

#ifndef MJ_MODULE_SPECTROMETER_H
#define MJ_MODULE_SPECTROMETER_H

#include <mj_array2d.h> 
#include <mj_filter_iir2.h> 
#include <mj_filter_ap1.h> 
#include <mj_level_meter.h> 
#include <mj_module_dsp.h>
#include <mj_module_dsp_gui_default.h>

#include <qtimer.h>  

class QBoxLayout;

/// Helper for MJ_ModuleSpectrometer
class MJ_FilterAndLevels
{
public:  
  void resetMemory() 
    { m_filter.resetMemory(); m_absFilter.resetMemory(); m_level.reset(); }

  MJ_FilterIIR2<MJ_Sample> & filter() { return m_filter; }
  MJ_FilterAP1<double> & absFilter() { return m_absFilter; } 
  MJ_LevelMeter<MJ_Sample> & level() { return m_level; } 

protected:

  MJ_FilterIIR2<MJ_Sample> m_filter;
  MJ_FilterAP1<double>     m_absFilter;
  MJ_LevelMeter<MJ_Sample> m_level;
};

/** Spectrometer plugin. Uses second order IIR filters and usual level
    meters to do the dirty work. 

    @author Tommi Ilmonen
*/
class MJ_ModuleSpectrometer : public MJ_ModuleDsp
{
public:
  MJ_ModuleSpectrometer();
  virtual ~MJ_ModuleSpectrometer();
  
  virtual bool dspPrepare();
  virtual void dspProcessPeriod();
  virtual bool msgProcess(const char *, MJ_Message *);

  virtual MJ_Parameter * msgGetInput(const char *);
  virtual QCString msgGetInputName(int);

  uint bands() { return m_levels.cols(); }
  uint upcomingBands() { return m_bands; }

  MJ_FilterAndLevels & level(uint chan, uint band)
  { return m_levels.get(chan, band); }

protected:
  
  void updateFilterCoeffs();
  
  MJ_Sample whiteNoiseGain(MJ_FilterIIR2<MJ_Sample> *filter, 
			   uint samples = 50000);

  ulong m_peakTime, m_peakHoldTime;

  MJ_Array2D<MJ_FilterAndLevels> m_levels;

  uint m_bands;

  static bool m_debug;
};

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////


/** GUI for MJ_ModuleSpectrometer.
   
    @author Tommi Ilmonen 
 */
class MJ_ModuleSpectrometer_QG : public MJ_ModuleDspGuiPostCard
{
  Q_OBJECT
public:
  MJ_ModuleSpectrometer_QG(MJ_RtControl *rtparent,
			   MJ_ModuleDsp *module,
			   QWidget      *parent = 0,
			   const char   *name = 0);

  virtual ~MJ_ModuleSpectrometer_QG();
  
  static MJ_ModuleDspGui *createNew(MJ_RtControl *rtparent,
				    MJ_ModuleDsp *module,
				    QWidget *parent, 
				    const char *name);

public slots:

  virtual void update();

protected:

  virtual void realTimeChange (bool nowRt);

  void draw();

  QWidget        *m_table;
  /// List of MJ_LevelDisplay objects
  QList<QWidget>  m_graphs;
  QBoxLayout     *m_graphLayout;

  MJ_Array<float> m_tmpBuffer;
  
  bool    m_vertical;

  QTimer  m_timer;
};

#endif
