/* COPYRIGHT
 *
 * This file is part of Mustajuuri GPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the General Public License (GPL),
 * versions 2. Extra clauses can be found from the file MUSTAJUURI_GPL
 * and further explanations from the file MUSTAJUURI_LICENSE. The
 * license text can be found in file "GPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *  
 */

#include <mj_additive_voice.h>
#include <mj_arraytmpl.C>

float MJ_Sinusoid::m_table[m_tableElems];

MJ_Sinusoid::MJ_Sinusoid()
{
  clear();
}

void MJ_Sinusoid::synthesize
(MJ_Sample *buf, uint samples, MJ_AdditiveVoice *voice)
{
  if(!m_frequency.isActive()) updateParams(voice);

  for(uint i=0; i < samples; i++) {

    buf[i] += fastCos(m_phase) * m_amplitude.get();

    m_frequency.advance();
    m_amplitude.advance();

    m_phase += m_frequency.get();
    if(m_phase > cycle()) m_phase -= cycle();
  }
}

void MJ_Sinusoid::clear()
{
  m_phase = 0;
  m_segmentEnter = 0;

  m_frequency.clear();
  m_amplitude.clear();
}

/** This method initializes the lookup-table. */

void MJ_Sinusoid::tableInit()
{
  float scale = MJ_2pi / (float) m_tableElems;

  for(uint i=0; i < m_tableElems; i++) {
    m_table[i] = MJ_cos((float) i * scale);
  }
}

void MJ_Sinusoid::updateParams(MJ_AdditiveVoice *voice)
{
  MJ_SinusoidParams sp;
  
  voice->calculateParams(*this, &sp);
}

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

/// Auto-caller for MJ_Sinusoid::tableInit
class MJ_SinusoidInit 
{
public:
  MJ_SinusoidInit() { MJ_Sinusoid::tableInit(); }
};

static MJ_SinusoidInit MJ_sinusoidInit;

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////


MJ_AdditiveVoice::MJ_AdditiveVoice()
{}

MJ_AdditiveVoice::~MJ_AdditiveVoice()
{}

void MJ_AdditiveVoice::synthesize(MJ_Sample *buf, uint samples)
{
  for(uint i=0; i < m_sinusoids.size(); i++) {
    m_sinusoids.getFast(i).synthesize(buf, samples, this);
  }
}


MJ_SynthVoiceStatus MJ_AdditiveVoice::status()
{
  return MJ_SVS_INACTIVE;
}

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

template class MJ_Array<MJ_Sinusoid>;
