/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of Mustajuuri GPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the General Public License (GPL),
 * versions 2. Extra clauses can be found from the file MUSTAJUURI_GPL
 * and further explanations from the file MUSTAJUURI_LICENSE. The
 * license text can be found in file "GPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *  
 */

#ifndef MJ_GRANULAR_VOICE_H
#define MJ_GRANULAR_VOICE_H

#include <mj_synth_voice.h>

#include <vector>

class MJ_AudioSample;

class MJ_GranularVoice : public MJ_SynthVoice
{
public:
  MJ_GranularVoice();
  virtual ~MJ_GranularVoice();

  virtual void synthesize(MJ_Sample *buf, uint samples);
  virtual bool msgProcess(const char *id, MJ_Message *, MJ_ModuleSynth *);
  virtual MJ_SynthVoiceStatus status();
  
protected:
  
  class Grain
  {
  public:
    Grain() : m_sampleCount(0.0), m_gain(0.0), 
	      m_pitchRelative(1.0), m_active(false) {}

    void init(MJ_Sample gain, MJ_Sample pitchRelative)
    {
       m_sampleCount = 0.0; 
       m_gain = gain; 
       m_pitchRelative = pitchRelative; 
       m_active = true;
    };

    void synthesize(MJ_Sample *buf, uint samples, const MJ_AudioSample *);

    bool isActive() const { return m_active; }

    double    m_sampleCount;
    MJ_Sample m_gain;
    MJ_Sample m_pitchRelative;
    bool      m_active;
  };

  Grain * findFreeGrain();
  void clearGrains();

  MJ_AudioSample      *m_sample;

  float                m_gain;
  float                m_pitch;
  float                m_pitchDistribution;
  float                m_rate;
  float                m_interval;
  // float                m_sinceLast;

  uint                 m_activeCount;
  int                  m_untilNext;
  
  std::vector<Grain> m_grains;

  MJ_SynthVoiceStatus m_status;
};


/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////


/// Granular voice factory
class MJ_GranularVoiceFactory : public MJ_SynthVoiceFactory
{
public:
  MJ_GranularVoiceFactory() {}
  virtual ~MJ_GranularVoiceFactory() {}

  virtual const char * type();
  virtual MJ_SynthVoice *create();

  static MJ_GranularVoiceFactory * instance() { return m_instance; }

  static void init()     { m_instance = new MJ_GranularVoiceFactory;}
  static void shutdown() { delete m_instance; m_instance = 0; }

protected:
  static MJ_GranularVoiceFactory * m_instance;
};


#endif
