/* COPYRIGHT
 *
 * This file is part of Mustajuuri GPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the General Public License (GPL),
 * versions 2. Extra clauses can be found from the file MUSTAJUURI_GPL
 * and further explanations from the file MUSTAJUURI_LICENSE. The
 * license text can be found in file "GPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *  
 */

#include <mj_input_voice.h>

#include <mj_error.h>
#include <mj_message_multi.h>
#include <mj_module_synth.h>
#include <mj_strs.h>

MJ_InputVoice::MJ_InputVoice()
  : m_source(0),
    m_ending(true)
{}

MJ_InputVoice::~MJ_InputVoice()
{}

void MJ_InputVoice::synthesize(MJ_Sample *buf, uint samples)
{
  /* MJ_trace2("MJ_InputVoice::synthesize # 0x%p 0x%p %d", 
     m_source, buf, samples); */

  const MJ_Sample *source = m_source;
  const MJ_Sample *sentinel = source + samples;

  while(source < sentinel) {
    m_ramp.advanceLinear();
    *buf++ += (*source++ * m_ramp.get());
  }
}


bool MJ_InputVoice::msgProcess
(const char *id, MJ_Message *msg, MJ_ModuleSynth *synth)
{
  // MJ_trace2("MJ_InputVoice::msgProcess # 0x%p, \"%s\"", this, id);

  if(!strcmp(id, MJ_Strs::noteon)) {
    double gain;
    
    MJ_MessageMulti *mm = dynamic_cast<MJ_MessageMulti *> (msg);

    int source;
    
    if(mm) {
      gain = mm->getDouble(MJ_Strs::gain, 1.0);
      source = mm->getInt("source", 0);
    }
    else {
      gain = 1.0;
      source = 0;
    }

    m_ramp.reset(gain);
    m_ending = false;

    if(source >= synth->dspInChannels()) {
      MJ_error2(MJ_ERR_ARGUMENT, "MJ_InputVoice::msgProcess # No channel %d",
		source);
      return false;
    }

    m_source = synth->dspInSignal(source).samples().data();

    return true;
  }
  else if(!strcmp(id, MJ_Strs::noteoff)) {
    uint interpSamples = 1000;
    m_ramp.setTarget(interpSamples, 0.0, MJ_RAMP_LINEAR);
    m_ending = true;
    return true;
  }
  else if(!strcmp(id, MJ_Strs::noteoffnow)) {
    m_ramp.reset(0.0);
    m_source = 0;
    m_ending = true;
    return true;
  }

  return false;
}

MJ_SynthVoiceStatus MJ_InputVoice::status()
{
  if(!m_source || (m_ending && m_ramp.get() == 0.0))
    return MJ_SVS_INACTIVE;
    
  return MJ_SVS_ACTIVE;
}

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

MJ_InputVoiceFactory * MJ_InputVoiceFactory::m_instance = 0;

const char * MJ_InputVoiceFactory::type()
{
  return "input";
}

MJ_SynthVoice *MJ_InputVoiceFactory::create()
{
  return new MJ_InputVoice;
}
