/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of Mustajuuri GPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the General Public License (GPL),
 * versions 2. Extra clauses can be found from the file MUSTAJUURI_GPL
 * and further explanations from the file MUSTAJUURI_LICENSE. The
 * license text can be found in file "GPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *  
 */

#ifndef MJ_INPUT_VOICE_H
#define MJ_INPUT_VOICE_H

#include <mj_synth_voice.h>

#include <mj_ramp.h>

/** This voice type is used to read data from the input of the
    synth. */

class MJ_InputVoice : public MJ_SynthVoice
{
public:
  MJ_InputVoice();
  virtual ~MJ_InputVoice();

  virtual void synthesize(MJ_Sample *buf, uint samples);
  virtual bool msgProcess(const char *id, MJ_Message *, MJ_ModuleSynth *);
  // virtual void command(MJ_SynthVoiceCommand cmd, float value);
  virtual MJ_SynthVoiceStatus status();
  
  void setSource(const MJ_Sample *source) { m_source = source; }
  
protected:
  
  const MJ_Sample         *m_source;

  bool                     m_ending;

  MJ_Ramp<MJ_Sample>       m_ramp;
};

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

/// Input voice factory
class MJ_InputVoiceFactory : public MJ_SynthVoiceFactory
{
public:
  MJ_InputVoiceFactory() {}
  virtual ~MJ_InputVoiceFactory() {}

  virtual const char * type();
  virtual MJ_SynthVoice *create();

  static MJ_InputVoiceFactory * instance() { return m_instance; }

  static void init()     { m_instance = new MJ_InputVoiceFactory;}
  static void shutdown() { delete m_instance; m_instance = 0; }

protected:
  static MJ_InputVoiceFactory * m_instance;
};

#endif
