/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of Mustajuuri GPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the General Public License (GPL),
 * versions 2. Extra clauses can be found from the file MUSTAJUURI_GPL
 * and further explanations from the file MUSTAJUURI_LICENSE. The
 * license text can be found in file "GPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *  
 */

#ifndef MJ_NOISE_VOICE_H
#define MJ_NOISE_VOICE_H

#include <mj_filter_iir2.h>
#include <mj_ramp.h>
#include <mj_random.h>
#include <mj_synth_voice.h>

class MJ_NoiseVoice : public MJ_SynthVoice
{
public:
  MJ_NoiseVoice();
  virtual ~MJ_NoiseVoice();

  virtual void synthesize(MJ_Sample *buf, uint samples);
  virtual bool msgProcess(const char *id, MJ_Message *, MJ_ModuleSynth *);
  virtual MJ_SynthVoiceStatus status();
  
protected:

  inline void redesignFilter()
  {
    m_filter.designBandPassB2(m_pitch.get(), m_sr, m_octaves.get());
  }

  uint                 m_location;
  uint                 m_endLocation;

  float                m_sr;
  bool                 m_ending;

  MJ_Random                m_rand;
  MJ_FilterIIR2<MJ_Sample> m_filter;
  MJ_RampLinear<double>    m_gain;
  MJ_Ramp<double>          m_pitch;
  MJ_Ramp<double>          m_octaves;
};

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

/// Noise voice factory
class MJ_NoiseVoiceFactory : public MJ_SynthVoiceFactory
{
public:
  MJ_NoiseVoiceFactory();
  virtual ~MJ_NoiseVoiceFactory();

  virtual const char * type();
  virtual MJ_SynthVoice *create();
  // virtual MJ_SynthMidiConverter * midiConverter();

  static MJ_NoiseVoiceFactory * instance() { return m_instance; }

  static void init() { m_instance = new MJ_NoiseVoiceFactory;}
  static void shutdown() { delete m_instance; m_instance = 0; }

protected:
  static MJ_NoiseVoiceFactory * m_instance;
};

#endif
