/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of Mustajuuri GPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the General Public License (GPL),
 * versions 2. Extra clauses can be found from the file MUSTAJUURI_GPL
 * and further explanations from the file MUSTAJUURI_LICENSE. The
 * license text can be found in file "GPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *  
 */

#ifndef MJ_SAMPLE_VOICE_H
#define MJ_SAMPLE_VOICE_H

#include <mj_synth_voice.h>

#include <mj_ramp.h>
#include <mj_resampler.h>

class MJ_AudioSample;

/** Voice that uses audio samples as waveform. This class cannot do
    pitch shifting or any other really intelligent operations. It
    implements a simple sample player.
    
    This class must be used with care - there are <b>few safety
    checks</b>.
*/

class MJ_SampleVoice : public MJ_SynthVoice
{
public:
  MJ_SampleVoice();
  virtual ~MJ_SampleVoice();

  virtual void synthesize(MJ_Sample *buf, uint samples);
  virtual bool msgProcess(const char *id, MJ_Message *, MJ_ModuleSynth *);
  virtual MJ_SynthVoiceStatus status();


  void setSample(MJ_AudioSample *s) { m_sample = s; rewind(); }
  void setLoop(bool loop) { m_loop = loop; }
  void rewind() { m_location = m_startLocation; }

protected:
  MJ_AudioSample      *m_sample;

  uint                 m_startLocation;
  uint                 m_location;
  uint                 m_endLocation;
  uint                 m_loopInterleave;

  bool                 m_loop;
  bool                 m_ending;

  MJ_Sample            m_gain;
  MJ_Ramp<float>       m_ramp;
};


/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

/// Sample voice factory
class MJ_SampleVoiceFactory : public MJ_SynthVoiceFactory
{
public:
  MJ_SampleVoiceFactory();
  virtual ~MJ_SampleVoiceFactory();

  virtual const char * type();
  virtual MJ_SynthVoice *create();
  virtual MJ_SynthMidiConverter * midiConverter();

  static MJ_SampleVoiceFactory * instance() { return m_instance; }

  static void init() { m_instance = new MJ_SampleVoiceFactory;}
  static void shutdown() { delete m_instance; m_instance = 0; }
protected:
  static MJ_SampleVoiceFactory * m_instance;
};

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

class MJ_SampleVoicePitched : public MJ_SampleVoice
{
public:
  MJ_SampleVoicePitched();
  virtual ~MJ_SampleVoicePitched();
  
  virtual void synthesize(MJ_Sample *buf, uint samples);
  virtual bool msgProcess(const char *id, MJ_Message *, MJ_ModuleSynth *);

protected:
  /// Relative pitch (compared to the original sample)
  MJ_Ramp<double>         m_relativePitch;
  MJ_Resampler<MJ_Sample> m_resampler;

  bool m_done;
};

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

/// Sample voice factory
class MJ_SampleVoicePitchedFactory : public MJ_SynthVoiceFactory
{
public:
  MJ_SampleVoicePitchedFactory() {}
  virtual ~MJ_SampleVoicePitchedFactory() {}

  virtual const char * type();
  virtual MJ_SynthVoice *create();

  static MJ_SampleVoicePitchedFactory * instance() { return m_instance; }

  static void init() { m_instance = new MJ_SampleVoicePitchedFactory;}
  static void shutdown() { delete m_instance; m_instance = 0; }
    
protected:
  static MJ_SampleVoicePitchedFactory * m_instance;
};

#endif
