/* COPYRIGHT
 *
 * This file is part of Mustajuuri GPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the General Public License (GPL),
 * versions 2. Extra clauses can be found from the file MUSTAJUURI_GPL
 * and further explanations from the file MUSTAJUURI_LICENSE. The
 * license text can be found in file "GPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *  
 */

#include <mj_stk_shakers.h>

#include <mj_message.h>
#include <mj_random.h>
#include <mj_strs.h>

static MJ_Random __MJ_shakeRandom;

MJ_StkVoiceShakers2::MJ_StkVoiceShakers2()
  : m_frequency(220.0f),
    m_amplitude(1.0),
    m_rate(10.0f),
    m_sr(44100.0f),
    m_untilNext(0)
{}

MJ_StkVoiceShakers2::~MJ_StkVoiceShakers2()
{}

void MJ_StkVoiceShakers2::synthesize(MJ_Sample *buf, uint samples)
{
  MJ_StkVoice<Shakers>::synthesize(buf, samples);

  if(m_untilEnd >= 0)
    return;

  m_untilNext -= samples;

  while(m_untilNext < 0) {
    float tmp = m_sr / m_rate;
    m_untilNext += tmp * 0.3 + __MJ_shakeRandom.getFloat0() * 0.7;

    m_instrument.noteOn(m_frequency, m_amplitude);
  }
}

bool MJ_StkVoiceShakers2::msgProcess
(const char *id, MJ_Message *m, MJ_ModuleSynth *s)
{
  if(strcmp(id, MJ_Strs::rate) == 0) {
    bool ok = true;
    double value = m->toDouble(&ok);

    if(ok)
      setRate(value);

    return ok;
  }

  return MJ_StkVoice<Shakers>::msgProcess(id, m, s);
}

MJ_SynthVoice * MJ_StkVoiceShakers2::create()
{
  return new MJ_StkVoiceShakers2;
}
