/* COPYRIGHT
 *
 * This file is part of Mustajuuri GPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the General Public License (GPL),
 * versions 2. Extra clauses can be found from the file MUSTAJUURI_GPL
 * and further explanations from the file MUSTAJUURI_LICENSE. The
 * license text can be found in file "GPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *  
 */

#include <mj_stk_voice_tmpl.C>
#include <mj_stk_shakers.h>

#include <di_midi_event.h>

#include <BandedWG.h> 
#include <Bowed.h> 
#include <StifKarp.h> 
#include <BlowBotl.h> 
#include <Shakers.h> 
#include <Flute.h> 
#include <Sitar.h> 

#include <vector>

#ifndef MJ_THIS_MACRO_IS_FOR_BUGGY_MIPSPRO_COMPILER
#define MJ_THIS_MACRO_IS_FOR_BUGGY_MIPSPRO_COMPILER

static std::vector<MJ_StkVoiceFactory *> MJ_stkFactories;

MJ_StkVoiceFactory::MJ_StkVoiceFactory
(MJ_SynthVoice * (*v)(), const char * name)
  : m_voiceFunc(v),
    m_name(name)
{
  MJ_stkFactories.push_back(this);
}

MJ_StkVoiceFactory::~MJ_StkVoiceFactory()
{}

const char * MJ_StkVoiceFactory::type()
{
  return m_name.c_str();
}

MJ_SynthVoice *MJ_StkVoiceFactory::create()
{
  return m_voiceFunc ? (*m_voiceFunc)() : 0;
}

MJ_StkVoiceFactory * MJ_StkVoiceFactory::getInstance(int i)
{
  if((uint) i < MJ_stkFactories.size()) 
    return MJ_stkFactories[i];

  return 0;
}

void MJ_StkVoiceFactory::addInstance(MJ_SynthVoice * (*v)(), const char *name)
{
  new MJ_StkVoiceFactory(v, name);
}

bool MJ_StkVoiceFactory::dspPrepare(MJ_ModuleSynth *s)
{
  MY_FLOAT sr = s->dspSamplingRate();

  Stk::setSampleRate(sr);

  return true;
}

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////


/* Section 2: Specializations and template invocations. */

// Specializations:
template <> 
bool MJ_StkVoice<Shakers>::noteOn(MJ_MessageMulti *mm, MJ_ModuleSynth *s)
{
  MY_FLOAT instrument = mm->getDouble(MJ_Strs::instrument, 0.0);
  MY_FLOAT amplitude = mm->getDouble(MJ_Strs::amplitude, 1.0);
  
  MY_FLOAT frequency = DI_MidiEvent::keyNumberToHz(instrument);
  m_instrument.Shakers::noteOn(frequency, amplitude);

  MJ_StkVoiceShakers2 *s2 = dynamic_cast<MJ_StkVoiceShakers2 *> (this);

  if(s2) {
    s2->setRate(mm->getDouble(MJ_Strs::rate, 10.0));
    s2->setSampleRate(s->dspSamplingRate());
    s2->setAmplitude(amplitude);
    s2->setFrequency(frequency);
  }

  return true;
}

// Templates that can be instantiedted without any trouble:
template class MJ_StkVoice<BandedWG>;
template class MJ_StkVoice<BlowBotl>;
template class MJ_StkVoice<Shakers>;


// Types that have one argument in the bloody constructor ;-)
#define MJ_STK_TEMPLATE_WITH_CONTRUCTOR_VALUE(T,V) \
  template <> MJ_StkVoice<T>::MJ_StkVoice() \
    : m_instrument(V), m_status(MJ_SVS_INACTIVE), m_untilEnd(-1) {} \
  template class MJ_StkVoice<T>;

MJ_STK_TEMPLATE_WITH_CONTRUCTOR_VALUE(Bowed, 50)
MJ_STK_TEMPLATE_WITH_CONTRUCTOR_VALUE(Flute, 50)
MJ_STK_TEMPLATE_WITH_CONTRUCTOR_VALUE(Sitar, 50)
MJ_STK_TEMPLATE_WITH_CONTRUCTOR_VALUE(StifKarp, 50)

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////


/* Section 3: Type initializations */

class MJ_StkInit
{
public:
  MJ_StkInit();
  ~MJ_StkInit();
};

static MJ_StkInit __MJ_stkInit;

MJ_StkInit::MJ_StkInit()
{
  MJ_StkVoiceFactory::addInstance(MJ_StkVoice<Bowed>::create, "bandedwg-stk");
  MJ_StkVoiceFactory::addInstance(MJ_StkVoice<Bowed>::create, "bowed-stk");
  MJ_StkVoiceFactory::addInstance(MJ_StkVoice<Shakers>::create, "shakers-stk");
  MJ_StkVoiceFactory::addInstance(MJ_StkVoiceShakers2::create, "shakers2-stk");
  MJ_StkVoiceFactory::addInstance(MJ_StkVoice<Flute>::create, "flute-stk");
  MJ_StkVoiceFactory::addInstance(MJ_StkVoice<BlowBotl>::create, "blowbottle-stk");
  MJ_StkVoiceFactory::addInstance(MJ_StkVoice<Sitar>::create, "sitar-stk");
  MJ_StkVoiceFactory::addInstance(MJ_StkVoice<StifKarp>::create, "stifkarp-stk");
}

MJ_StkInit::~MJ_StkInit()
{
  for(uint i=0; i < MJ_stkFactories.size(); i++)
    delete MJ_stkFactories[i];

  MJ_stkFactories.resize(0);
}


#endif  // MJ_THIS_MACRO_IS_FOR_BUGGY_MIPSPRO_COMPILER
