/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of Mustajuuri GPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the General Public License (GPL),
 * versions 2. Extra clauses can be found from the file MUSTAJUURI_GPL
 * and further explanations from the file MUSTAJUURI_LICENSE. The
 * license text can be found in file "GPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *  
 */

#ifndef MJ_STK_VOICE_H
#define MJ_STK_VOICE_H

#include <mj_synth_voice.h>

#include <string>

class MJ_MessageMulti;

template <class T>
class MJ_StkVoice : public MJ_SynthVoice
{
public:
  MJ_StkVoice();
  virtual ~MJ_StkVoice();

  virtual void synthesize(MJ_Sample *buf, uint samples);
  virtual bool msgProcess(const char *id, MJ_Message *, MJ_ModuleSynth *);
  virtual MJ_SynthVoiceStatus status();
  // virtual MJ_SynthVoice * clone() const;

  bool noteOn(MJ_MessageMulti *mm, MJ_ModuleSynth *);

  static MJ_SynthVoice * create();

protected:

  bool internalMsgProcess(const char *id, MJ_Message *, MJ_ModuleSynth *);

  T m_instrument;
  MJ_SynthVoiceStatus m_status;
  int m_untilEnd;
};

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

/// Stk voice factory
class MJ_StkVoiceFactory : public MJ_SynthVoiceFactory
{
public:
  MJ_StkVoiceFactory(MJ_SynthVoice * (*)(), const char * name);
  virtual ~MJ_StkVoiceFactory();

  virtual const char * type();
  virtual MJ_SynthVoice *create();

  static MJ_StkVoiceFactory * getInstance(int);
  static void addInstance(MJ_SynthVoice * (*)(), const char *);
  
  virtual bool dspPrepare(MJ_ModuleSynth *);

protected:
  MJ_SynthVoice * (*m_voiceFunc)();
  std::string m_name;
};

#endif
