/* COPYRIGHT
 *
 * This file is part of Mustajuuri GPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the General Public License (GPL),
 * versions 2. Extra clauses can be found from the file MUSTAJUURI_GPL
 * and further explanations from the file MUSTAJUURI_LICENSE. The
 * license text can be found in file "GPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *  
 */

#include <mj_synth_midi.h>

#include <mj_error.h>
#include <mj_messages.h>
#include <mj_module_synth.h>

#include <string.h>

bool MJ_SynthMidiConverter::convert
(const DI_MidiEvent *e, MJ_ModuleSynth *synth)
{
  if(e->isNoteOff()) {

    int index = synth->midiFindNote(e->keyNumber(), e->channel());

    if(index >= 0) {
      MJ_SynthVoiceHolder *svp = synth->getActiveVoice(index);
      MJ_MessageDouble m(e->noteVelocity() / 128.0);
      
      char buf[32];
      sprintf(buf, "note/%ld/noteoff", svp->id());
      synth->msgProcess(buf, &m);
    }
    return true;
  }
  return false;
}

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

MJ_SynthMidiSetup::MJ_SynthMidiSetup()
{
  memset(m_channelSetups, 0, 
	 sizeof(MJ_SynthMidiConverter *) * DI_MIDI_CHANNELS);
}

MJ_SynthMidiSetup::~MJ_SynthMidiSetup()
{
  for(uint i=0; i < DI_MIDI_CHANNELS; i++) 
    delete m_channelSetups[i];
}

uint MJ_SynthMidiSetup::maximumOutputChannel() const
{
  uint v = 0;

  for(uint i=0; i < DI_MIDI_CHANNELS; i++) {
    
    MJ_SynthMidiConverter * converter = m_channelSetups[i];
    
    if(converter)
      if(v < converter->maxChannel())
	v = converter->maxChannel();
  }

  return v;
}


bool MJ_SynthMidiSetup::convert(const DI_MidiEvent *e, MJ_ModuleSynth *synth)
{
  uint channel = e->channel();

  MJ_SynthMidiConverter * converter = m_channelSetups[channel];

  if(!converter) {
    MJ_error2(MJ_ERR_INTERNAL, 
	      "MJ_SynthMidiSetup::convert # No converter for channel %u", 
	      channel);
    return false;
  }
  return converter->convert(e, synth);
}
