/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of Mustajuuri GPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the General Public License (GPL),
 * versions 2. Extra clauses can be found from the file MUSTAJUURI_GPL
 * and further explanations from the file MUSTAJUURI_LICENSE. The
 * license text can be found in file "GPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *  
 */

#ifndef MJ_SYNTH_VOICE_H
#define MJ_SYNTH_VOICE_H

#include <mj_config.h>

#include <di_types.h>


/** Status of a synthesis voice.

    @see MJ_SynthVoice.
*/
enum MJ_SynthVoiceStatus {
  /// The voice is active.
  MJ_SVS_ACTIVE,
  /// The voice is at end.
  MJ_SVS_AT_END,
  /// The voice is inactive.
  MJ_SVS_INACTIVE,
  /// Number of states.
  MJ_SVS_SIZEOF
};

class MJ_Message;
class MJ_ModuleSynth;

/** Base class for synthesizer voices. Different voice types need to
    override the virtual functions to get going.
    
    @see MJ_ModuleSynth
*/

class MJ_SynthVoice
{
public:
  MJ_SynthVoice() {}
  virtual ~MJ_SynthVoice() {}

  /** Synthesize some sound to the target buffer. The operation
      <B>must be adding</B> - ie. the synthesized sound is added to
      the buffer.
  */
  virtual void synthesize(MJ_Sample *buf, uint samples) = 0;

  virtual bool msgProcess(const char *id, MJ_Message *, MJ_ModuleSynth *) = 0;

  virtual MJ_SynthVoiceStatus status() = 0;

  // virtual MJ_SynthVoice * clone() const { return 0; }
};

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

class MJ_SynthMidiConverter;

class MJ_SynthVoiceFactory
{
public:
  MJ_SynthVoiceFactory() {}
  virtual ~MJ_SynthVoiceFactory() {}

  virtual const char * type() = 0;
  virtual MJ_SynthVoice *create() = 0;
  virtual MJ_SynthMidiConverter * midiConverter() { return 0; }
  virtual bool dspPrepare(MJ_ModuleSynth *) { return true; }
};

extern bool MJ_synthDebug;

#endif
