
#include <mj_module_uitest.h>

#include <mj_dsp_plugin.h>
#include <mj_iostream.h>
#include <mj_message_string.h>
#include <mj_messages.h>
#include <mj_parameters.h>

#include <qfiledialog.h> 

// The number of these parameters = MJ_ModuleUiTest::m_params.
static const char *MJ_UiTestIds[] = {
  "intslider",
  "intlist",
  "doubleslider",
  "doublelist",
  "string",
  "filename"
};

// The number of these parameters = MJ_ModuleUiTest::m_params.
static const char *MJ_UiTestLabels[] = {
  "Number of universes",
  "Chosen universe",
  "Vulgarity coefficient",
  "Speed of thought",
  "Meaning of life",
  "Source of wisdom"
};

MJ_ModuleUiTest::MJ_ModuleUiTest()
  : m_i1(0), 
  m_i2(0),
  m_d1(0), 
  m_d2(0),
  m_str1("Poppanajuures"),
  m_str2("ugh.C")
{
  trace("MJ_ModuleUiTest::MJ_ModuleUiTest");
}

MJ_ModuleUiTest::~MJ_ModuleUiTest()
{
  trace("MJ_ModuleUiTest::~MJ_ModuleUiTest");
}
  

void MJ_ModuleUiTest::dspProcessPeriod()
{}

  
bool MJ_ModuleUiTest::msgProcess(const char *id, MJ_Message *m)
{
  const char * fname = "MJ_ModuleUiTest::msgProcess";

  trace(fname, id);

  int ind = MJ_findStr(id, MJ_UiTestIds, m_params);

  if(ind < 0) {
    error(MJ_ERR_ARGUMENT, fname, "Bad id");
    return false;
  }

  bool ok = true;
  
  MJ_cout() << m->describe().c_str() << "\n";
  MJ_cout().device()->flush();
  
  if(ind == 0) {
    m_i1 = m->toInt(&ok);
  }
  else if(ind == 1) {
    m_i2 = m->toInt(&ok);
  }
  else if(ind == 2) {
    m_d1 = m->toDouble(&ok);
  }
  else if(ind == 3) {
    m_d2 = m->toDouble(&ok);
  }
  else {
    MJ_MessageString *ms = dynamic_cast<MJ_MessageString *> (m);
    if(!ms) {
      ok = false;
      error(MJ_ERR_ARGUMENT, fname, "NOT MJ_MessageString");
    }
    else if(ind == 4)
      m_str1 = ms->string();
    else if(ind == 5)
      m_str2 = ms->string();
  }
  return ok;
}

MJ_Parameter * MJ_ModuleUiTest::msgGetInput(const char *id)
{
  int ind = MJ_findStr(id, MJ_UiTestIds, m_params);

  if(ind == 0) {
    return new MJ_ParameterInt(MJ_UiTestLabels[0], id, m_i1, -20, 100);
  }
  else if(ind == 1) {
    return new MJ_ParameterInt(MJ_UiTestLabels[1], id, m_i2, -20, 100);
  }
  else if(ind == 2) {
    return new MJ_ParameterDouble(MJ_UiTestLabels[2], id, m_d1, 0.0, 1314.0);
  }
  else if(ind == 3) {
    MJ_ParameterDouble *pd = 
      new MJ_ParameterDouble(MJ_UiTestLabels[3], id, m_d2, 0.0, 1314.0);
    
    pd->addAllowed(0.4);
    pd->addAllowed(1.12);
    pd->addAllowed(2.2323);
    pd->addAllowed(17.43);
    pd->addAllowed(189.0);
    pd->addAllowed(1025.0435 );
    return pd;
  }
  else if(ind == 4) {
    return new MJ_ParameterString(m_str1, MJ_UiTestLabels[4], id);
  }
  
  else if(ind == 5) {
    return new MJ_ParameterFilename(m_str2, MJ_UiTestLabels[5], id,
				    "*.[hC]", QFileDialog::AnyFile);
  }
  
  error(MJ_ERR_ARGUMENT, "MJ_ModuleUiTest::msgGetInput");
  return 0;
}


QCString MJ_ModuleUiTest::msgGetInputName(int i)
{
  QCString r;

  if(i <  m_params) r = MJ_UiTestIds[i];

  return r;
}

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

MJ_ModuleDsp * MJ_createNew_ModuleExample1() 
{
  return new MJ_ModuleUiTest;
}

MJ_PluginDbeDsp *MJ_createPlugin(int i)
{
  if(i != 0) return 0;

  MJ_PluginDbeDsp *dbe = new MJ_PluginDbeDsp;
  dbe->setAuthors("Ilmonen Tommi S.");
  dbe->setMenuPosition("_Audio_Tests_User interface test");
  dbe->setCreator(MJ_createNew_ModuleExample1);
  dbe->setIdentifier("tcm.hut.fi_uitest1");
  dbe->setParameter(MJ_PP_TRANSLATION_FILE, "mj_uitest");
  dbe->setParameter(MJ_PP_COMPILE_TIME, DATETIME);

  return dbe;  
}
