/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of Mustajuuri GPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the General Public License (GPL),
 * versions 2. Extra clauses can be found from the file MUSTAJUURI_GPL
 * and further explanations from the file MUSTAJUURI_LICENSE. The
 * license text can be found in file "GPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *  
 */

#ifndef MJ_MODULE_VBAP_H
#define MJ_MODULE_VBAP_H

#include <mj_file_load_info.h>
#include <mj_module_dsp.h>
#include <mj_vbap_voice.h>

#include <vbap_setup.h>

extern bool MJ_debugVBAP;

/** A VBAP panner. 

    <B>NOTE:</B> The messages that this class receives can be timed in
    three ways:

    <OL>
    <LI> timestamp = -1 -> Default interpolation period
    <LI> timestamp = -2 -> Immeddiate source repositioning
    <li> timestamp > 0  -> Use available interpolation times
    </OL>
*/
class MJ_ModuleVBAP : public MJ_ModuleDsp
{
public:
  MJ_ModuleVBAP();
  virtual ~MJ_ModuleVBAP();
  
  virtual bool dspAcceptIoChans(int * in, int * out);

  virtual bool dspPrepare();
  virtual void dspProcessPeriod();
  virtual bool dspStop();

  virtual bool msgProcess(const char *, MJ_Message *);
  virtual MJ_Parameter * msgGetInput(const char *);
  virtual QCString msgGetInputName(int);

protected:

  void initPanners();

  void updatePanner(uint i, uint interpolateSamples);

  void angleToCart(float azi, float ele, double *res);

  void VBAPGains(double g[3], int ls[3], float azi, float ele, uint dim);
  
  bool loadSetup();

  uint voicesInUse();

  MJ_Array<MJ_VBAPVoiceGroup> m_panners;
  VBAP_Setup                 *m_vbap;

  QString                     m_fileName;

  MJ_FileLoadInfo             m_vbapLoadInfo;
};

#endif
