/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of Mustajuuri GPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the General Public License (GPL),
 * versions 2. Extra clauses can be found from the file MUSTAJUURI_GPL
 * and further explanations from the file MUSTAJUURI_LICENSE. The
 * license text can be found in file "GPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *  
 */

#ifndef MJ_MODULE_VBAP2_H
#define MJ_MODULE_VBAP2_H

#include <mj_module_vbap.h>

#include <mj_delay_line.h>

/** VBAP panner with distance delay and attenuation. */

class MJ_ModuleVBAP2 : public MJ_ModuleVBAP
{
public:

  MJ_ModuleVBAP2();
  virtual ~MJ_ModuleVBAP2();
  
  virtual bool dspPrepare();
  virtual void dspProcessPeriod();
  virtual bool dspStop();

  virtual bool msgProcess(const char *, MJ_Message *);
  virtual MJ_Parameter * msgGetInput(const char *);
  virtual QCString msgGetInputName(int);

protected:
  
  class Preprocess {
  public:
    Preprocess() : m_delayTime(0.0) { m_line.setGain(1.0); }

    void prepare(float maxDelay, float sr) 
    { m_line.prepare((uint) (maxDelay * sr), 0.0);}

    void setDelay(float d, float sr)
    { m_delayTime = d; m_line.setDelay(d * sr); }
    void interpolateDelay(float d, float sr, uint n)
    { m_delayTime = d; m_line.interpolateDelay(d * sr, n); }

    void setGain(float d) { m_line.setGain(d); }
    void interpolateGain(float d, uint n) { m_line.interpolateGain(d, n); }

    void updateSr(float sr)
    { m_line.setDelay(m_delayTime * sr); }

    MJ_DelayLine2<MJ_Sample> &line() { return m_line; }
    float delayTime() const { return m_delayTime; }
    float gainTarget() const { return m_line.gainTarget(); }

    MJ_DelayLine2<MJ_Sample> m_line;
    /// Delay time in seconds
    float m_delayTime;
  };


  MJ_Array<Preprocess> m_preprocessors;
  MJ_Array<MJ_Sample>  m_tmpBuffer;
};

#endif
