/* COPYRIGHT
 *
 * This file is part of Mustajuuri GPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the General Public License (GPL),
 * versions 2. Extra clauses can be found from the file MUSTAJUURI_GPL
 * and further explanations from the file MUSTAJUURI_LICENSE. The
 * license text can be found in file "GPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *  
 */

#include <mj_vbap_voice.h>

#include <mj_error.h>

#include <stdio.h>
#include <string.h>     
#include <strings.h>

void MJ_VBAPVoice::clear()
{
  m_ramp.clear();

  m_target = 0;
  m_goingOff = false;
}

void MJ_VBAPVoice::voiceOn(uint interpolationTime,
			   float gain,
			   MJ_Sample *target)
{
  m_goingOff = false;

  m_ramp.set(interpolationTime, 0.0, gain, MJ_RAMP_LINEAR);
  m_target = target;
  // MJ_trace2("MJ_VBAPVoice::voiceOn # %g %g", m_ramp.get(), m_ramp.step());
}

void MJ_VBAPVoice::voiceRenew(uint interpolationTime, MJ_Sample gain)
{
  m_ramp.setTarget(interpolationTime, gain, MJ_RAMP_LINEAR);
  m_goingOff = false;

  //MJ_trace2("MJ_VBAPVoice::voiceRenew # %g %g", m_ramp.get(), m_ramp.step());
}

void MJ_VBAPVoice::voiceOff(uint interpolationTime)
{
  if(m_goingOff) return;

  m_goingOff = true;
  
  m_ramp.setTarget(interpolationTime, 0.0, MJ_RAMP_LINEAR);
  // MJ_trace2("MJ_VBAPVoice::voiceOff # %g %g", m_ramp.get(), m_ramp.step());
}

#if 0
void MJ_VBAPVoice::process(const MJ_Sample *source, uint samples)
{
#if 0
  /* MJ_trace2("MJ_VBAPVoice::process this = %p s = %p t = %p %d", 
     this, source, m_target, samples); */
  
  MJ_Sample *target = m_target;
  const MJ_Sample *sentinel = target + samples;
    
  while(target < sentinel) {
    m_ramp.advanceLinear();
    // target++;
    // m_ramp.decrementCounter();
    *target++ += (*source++ * m_ramp.get());
    /* if(!finite(*(target-1))) {
       MJ_fatal(MJ_ERR_INTERNAL, "MJ_VBAPVoice::process");
       }*/
  }
  //printf(" [ %f %f %f %d]", *(source-1), m_ramp.get(), *(target-1), samples);
#elsif 1

  m_ramp.advanceLinearMac(m_target, source, samples);

#else

  for(uint i=0; i < samples; i++) {
    m_ramp.advanceLinear();
    // m_target[i] += (source[i] * m_ramp.get());
    float tmp = (source[i] * m_ramp.get());
    float tmp2 = m_target[i] + tmp;
    if(!finite(tmp2))
      MJ_fatal(MJ_ERR_INTERNAL, "!finite(tmp2)");
    m_target[i] = tmp2;
  }
#endif
}
#endif
/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

MJ_VBAPVoiceGroup::MJ_VBAPVoiceGroup()
  : m_azi(0),
    m_ele(0),
    m_activeVoices(0)
{}

MJ_VBAPVoiceGroup::~MJ_VBAPVoiceGroup()
{}

void MJ_VBAPVoiceGroup::process(const MJ_Sample *source, uint samples)
{
  // MJ_trace2("MJ_VBAPVoiceGroup::process %p %d", m_source, samples);

  uint i;
  
  // Process stuff to output:
  //for(i=0; i < m_activeVoices; i++)
  for(i=0; i < m_activeVoices; i++)
    m_voices[i].process(source, samples);

  if(!m_activeVoices) return;

  // MJ_trace2("MJ_VBAPVoiceGroup::process HALF");

  // Remove the inactive voices, if necessary
  
  for(i=0; i < m_activeVoices; ) {
    
    if(!m_voices[i].isActive()) {

      m_activeVoices--;

      for(uint j = i; j < m_activeVoices; j++)
	m_voices[j] = m_voices[j + 1];
      
      
      m_voices[m_activeVoices].clear();

      /* MJ_trace2("MJ_VBAPVoiceGroup::process removed one %d %d", 
		i, m_activeVoices); 

      for(uint k = 0; k < m_activeVoices; k++) 
	printf(" %d: g = %f t = %f", k, m_voices[k].gain(), 
	       m_voices[k].target());

	       fflush(0);*/
    }
    else 
      i++;
  }
}

void MJ_VBAPVoiceGroup::newTarget(float azi,
				  float ele,
				  uint interpolationTime,
				  uint dimension,
				  double *gains,
				  MJ_Sample **targets)
{
  uint i;

#if 0
  //Shit code:
  m_azi = azi;
  m_ele = ele;
  m_activeVoices = dimension;
  
  for(i=0; i < m_activeVoices; i++) {
    m_voices[i].voiceOn(interpolationTime, gains[i], targets[i]);
  }

  return;
#endif

  // Old (and more real) code:

  m_azi = azi;
  m_ele = ele;

  // MJ_trace2("MJ_VBAPVoiceGroup::newTarget ENTER %p", m_source);

  // Downgrade multiple declarations to unique declarations:
  if(dimension == 3 && targets[1] == targets[2]) {
    dimension--;
    gains[1] += gains[2];
  }
  if(targets[0] == targets[1]) {
    dimension--;
    gains[0] += gains[1];
    gains[1] = gains[2];
    targets[1] = targets[2];
  }

  /// Make all voices die.
  for(i=0; i < m_activeVoices; i++) {
    m_voices[i].voiceOff(interpolationTime);
  }
  
  /// Now bring back to life the ones that deserve it:
  for(i = 0; i < dimension; i++) {

    int index = allocateVoice(targets[i]);

    MJ_Sample gain = gains[i];
    MJ_Sample * target = targets[i];
    MJ_VBAPVoice *vv = & m_voices[index];

    

    /* MJ_trace2("MJ_VBAPVoiceGroup::newTarget %f %d %d %d # %d %d %p %p",
	      gain, index, i, m_activeVoices, 
	      m_maxVoices, interpolationTime, target, vv);*/

    if(vv->targetConst() == target) 
      vv->voiceRenew(interpolationTime, gain);
    else 
      vv->voiceOn(interpolationTime, gain, targets[i]);
  }
  // MJ_trace2("MJ_VBAPVoiceGroup::newTarget EXIT %p", m_source);
}

void MJ_VBAPVoiceGroup::start()
{
  // MJ_trace("MJ_VBAPVoiceGroup::start");

  for(uint i=0; i < m_maxVoices; i++)
    m_voices[i].clear();

  m_activeVoices = 0;
}

int MJ_VBAPVoiceGroup::allocateVoice(const MJ_Sample *target)
{
  uint i;
  
  if(m_activeVoices >= m_maxVoices) {   
    // Scan for any dying voice:
    for(i = 0; i < (int) m_maxVoices; i++) {
      if(m_voices[i].goingOff()) {
	return i;
      }
    }
    MJ_fatal(MJ_ERR_INTERNAL, "MJ_VBAPVoiceGroup::allocateVoice FIRST");
  }


  // MJ_trace2("MJ_VBAPVoiceGroup::allocateVoice searching %p", target);

  // Scan for a voice that is already handling this target
  for(i = 0; i < m_activeVoices; i++) {

    const MJ_VBAPVoice *vv = & m_voices[i];

    /* printf("%d target = %p %d %g\n", 
       i, vv->targetConst(), (int) vv->isActive(), vv->gain()); */

    if(vv->targetConst() == target && vv->isActive())
      return i;
  }  
  
  /* MJ_trace2("MJ_VBAPVoiceGroup::allocateVoice no first rate match %p",
     target);*/
  
  // Scan for any free voice:
  if(!m_voices[m_activeVoices].isActive()) {
    m_activeVoices++;
    return i;
  }  
  
  // MJ_trace("MJ_VBAPVoiceGroup::allocateVoice no second rate match");

  // Scan for any dying voice:
  for(i = 0; i <  m_maxVoices; i++) {
    /* MJ_trace2("MJ_VBAPVoiceGroup::allocateVoice # %d isactive %d",
       i, (int) m_voices[i].isActive()); */

    if(m_voices[i].goingOff()) {
      return i;
    }
  }
  
  MJ_fatal(MJ_ERR_INTERNAL, "MJ_VBAPVoiceGroup::allocateVoice");

  return 0;
}
