/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of Mustajuuri GPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the General Public License (GPL),
 * versions 2. Extra clauses can be found from the file MUSTAJUURI_GPL
 * and further explanations from the file MUSTAJUURI_LICENSE. The
 * license text can be found in file "GPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *  
 */

#ifndef MJ_VBAP_VOICE_H
#define MJ_VBAP_VOICE_H

#include <mj_config.h>
#include <mj_ramp.h>

#include <di_types.h>

#define MJ_VBAP_MAX_DIM 3

class MJ_VBAPVoice
{
public:
  MJ_VBAPVoice() : m_target(0) {}
  ~MJ_VBAPVoice() {}

  void clear();

  void voiceOn(uint interpolationTime,
	       MJ_Sample gain,
	       MJ_Sample *target);
  
  void voiceRenew(uint interpolationTime, MJ_Sample gain);

  void voiceOff(uint interpolationTime);

  void process(const MJ_Sample *source, uint samples)
  { m_ramp.advanceLinearMac(m_target, source, samples); }

  bool isActive() const
  { return m_ramp.isActive() || m_ramp.get() != 0.0; }

  bool goingOff() const { return m_goingOff; }


  const MJ_Sample *targetConst() const { return m_target; }

  MJ_Sample gain() const { return m_ramp.get(); }
  MJ_Sample target() const { return m_ramp.target(); }

protected:
  MJ_Ramp<MJ_Sample> m_ramp;
  MJ_Sample *m_target;
  bool m_goingOff;
};

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

class MJ_VBAPVoiceGroup
{
public:
  MJ_VBAPVoiceGroup();
  ~MJ_VBAPVoiceGroup();

  void process(const MJ_Sample *source, uint samples);

  void newTarget(float azi,
		 float ele,
		 uint interpolationTime,
		 uint dimension,
		 double *gains,
		 MJ_Sample **targets);

  void start();

  void setAzi(float azi) { m_azi = azi; }
  void setEle(float ele) { m_ele = ele; }

  float azi() const { return m_azi; }
  float ele() const { return m_ele; }

  uint activeVoices() const { return m_activeVoices; }
protected:

  int allocateVoice(const MJ_Sample *target);

  static const uint m_maxVoices = 9;
  
  float        m_azi;
  float        m_ele;

  MJ_VBAPVoice m_voices[m_maxVoices];

  uint m_activeVoices;
};

#endif
