/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of Mustajuuri LGPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. Extra clauses can be found from the file
 * MUSTAJUURI_LGPL and further explanations from the file
 * MUSTAJUURI_LICENSE. The license text can be found in file
 * "LGPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *    
 */

#ifndef QAWL_FILE_CACHE_H
#define QAWL_FILE_CACHE_H

#include <qawl_min_max.h>

#include <di_audio_file.h>

#include <qstring.h>

#include <vector>

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

/** Audio (preview) file cache. */
class QAWL_FileCache
{
public:
  
  QAWL_FileCache();
  ~QAWL_FileCache();
  
  bool setFile(const QString &, uint scale);

  bool loadRealFrames(ulong first, ulong last)
  { return loadFrames(first/m_scale, last / m_scale + 100); }

  bool loadFrames(ulong first, ulong last);

  ulong firstFrame() const { return m_firstFrame; }
  ulong lastFrame() const { return m_lastFrame; }
  
  uint channels() const { return m_chans; }
  uint scale() const { return m_scale; }
  
  QAWL_MinMax getMinMax(ulong firstRealFrame, 
			ulong lastRealFrame, 
			uint channel);

protected:

  bool fullReload(ulong first, ulong last);
  bool fullReloadOrig(ulong first, ulong last);


  QAWL_MinMax &getSample(ulong frame, uint channel)
  { return m_data[frame * m_chans + channel]; }

  std::vector<QAWL_MinMax> m_data;

  ulong m_firstFrame;
  ulong m_lastFrame;

  ulong m_chans;
  ulong m_fileFrames;

  /// The number of frames to tolerate before discarding the unused frames.
  ulong m_discardAfter;

  uint  m_scale;
  // uint  m_trueScale;

  QString m_fileName;

  DI_AudioFile m_infile;
  std::vector<int> m_tmpBuf;
};

#endif
