/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of Mustajuuri LGPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. Extra clauses can be found from the file
 * MUSTAJUURI_LGPL and further explanations from the file
 * MUSTAJUURI_LICENSE. The license text can be found in file
 * "LGPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *    
 */

#ifndef QAWL_MIN_MAX_H
#define QAWL_MIN_MAX_H


/** This class holds a pair of short integer values. The purpose of
    the class is to offer an easy way to manipulate minimum-maximum
    paris that are needed in the audio file preview and rendering
    operations. 

    This class also has a series of methods that operate on 32-bit
    integers (to & from). The idea is that objects of this class can
    be easily converted to/from 32-bit integers. This feature is handy
    when one wants to serialize these objects to/from a file.
*/

class QAWL_MinMax
{
public:
  typedef unsigned short usho;

  QAWL_MinMax() {}
  QAWL_MinMax(short mi, short ma) : min(mi), max(ma) {}
  
  short min, max;

  short average() const { return ((int) min + (int) max) / 2; }

  void expand(short v)
  {
    if(min > v) min = v;
    if(max < v) max = v;
  }

  void limitZeroPassing()
  {
    if(min > 0) min = 0;
    if(max < 0) max = 0;
  }

  /** @name 32-bit integer operations */
  //@{
  static inline short getMin(unsigned i32) { return (usho) i32; }
  static inline short getMax(unsigned i32) { return (usho) (i32 >> 16); }


  void setInt32(unsigned i32) { max = getMax(i32); min = getMin(i32); }
  void setInt16(short i16) { max = i16; min = i16; }

  unsigned getInt32() const 
  { return ((unsigned) (usho) max) << 16 | (unsigned) (usho) min; }

  void expandInt32(int i32)
  {
    short m = getMin(i32);
    if(min > m) min = m;

    m = getMax(i32);    
    if(max < m) max = m;
  }
  //@}
};

#endif
