
/* COPYRIGHT
 *
 * This file is part of Mustajuuri LGPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. Extra clauses can be found from the file
 * MUSTAJUURI_LGPL and further explanations from the file
 * MUSTAJUURI_LICENSE. The license text can be found in file
 * "LGPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *    
 */

#include <qawl_preview_test.h>

#include <qawl_file_cache.h>
#include <qawl_preview_calculator.h>
#include <qawl_renderer.h>

#include <qpainter.h> 
#include <qpushbutton.h> 

static QAWL_PreviewCalculator *pvc;

QAWL_PreviewTest::QAWL_PreviewTest()
{
  QPushButton *pb = new QPushButton("Test", this);

  connect(pb, SIGNAL(clicked()), this, SLOT(test()));

  resize(700, 300);
}

QAWL_PreviewTest::~QAWL_PreviewTest()
{}

void QAWL_PreviewTest::test()
{
  qDebug("QAWL_PreviewTest::test");

  if(!pvc) {
    pvc = new QAWL_PreviewCalculator;
    
    connect(pvc, SIGNAL(previewAvailable(const QString &, uint)),
	    this, SLOT(fileAvailable(const QString &, uint)));
  }

  // pvc->requestPreview(m_file, 25);
  pvc->requestPreview(m_file, 150);
  pvc->requestPreview(m_file, 10002);

  pvc->requestNormalPreviews(m_file);
  // pvc->requestPreview(m_file, 1000);
  // pvc->requestPreview(m_file, 10000);
}

void QAWL_PreviewTest::fileAvailable(const QString &f, uint scale)
{
  qDebug("File \"%s\" has preview available with scale %d",
	 (const char *) f, scale);

  QAWL_FileCache fcache;

  fcache.setFile(f, scale);

  qDebug("fcache.setFile DONE");
  
  fcache.loadFrames(100, 500);
  fcache.loadFrames(10, 500);
  fcache.loadFrames(0, 1000);
  fcache.loadFrames(0, 1200);
  fcache.loadFrames(0, 1400);
  fcache.loadFrames(0, 2000);
  fcache.loadFrames(0, 3000);

  qDebug("DONE LOADING");
  
  if(scale == 150) {

    QAWL_Renderer renderer;
  
    QPainter p(this);
    QPointArray pa;
    
    renderer.render(&p, &pa, &fcache, 0, 1800000,
		    0, 0, width(), height() / 2, 
		    height() / 4,height() / 4);  }
}
