
/* COPYRIGHT
 *
 * This file is part of Mustajuuri LGPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2004.
 * e-mail: firstname.lastname(at)hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the Lesser General Public License
 * (LGPL) - version 2.1. Extra clauses can be found from the file
 * MUSTAJUURI_LGPL and further explanations from the file
 * MUSTAJUURI_LICENSE. The license text can be found in file
 * "LGPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *    
 */

#include <qawl_preview_test2.h>

#include <qawl_view.h>

#include <qcombobox.h> 
#include <qfiledialog.h>
#include <qlayout.h> 
#include <qmenubar.h>
#include <qpopupmenu.h>
#include <qslider.h>

#include <math.h>

QAWL_PreviewTest2::QAWL_PreviewTest2(int test)
  : m_scale(1500.0)
{
  QVBoxLayout *layout = new QVBoxLayout(this);

  QMenuBar *menuBar = new QMenuBar(this);
  layout->setMenuBar(menuBar);

  QPopupMenu *fmenu = new QPopupMenu(menuBar);
  menuBar->insertItem("File", fmenu);

  fmenu->insertItem("Open...", this, SLOT(open()));

  m_slider = new QSlider(1, 1000, 1, (int) m_scale, 
			 Qt::Horizontal, this, "scales-slider");

  connect(m_slider, SIGNAL(valueChanged(int)), this, SLOT(setScale(int)));

  layout->addWidget(m_slider);

  QSlider * magnify = new QSlider(0, 1000, 10, 0, 
				  Qt::Horizontal, this, "magnify-slider");

  connect(magnify, SIGNAL(valueChanged(int)), this, SLOT(setMagnify(int)));
  layout->addWidget(magnify);

  QComboBox * combo = new QComboBox(this);

  combo->insertItem("Minutes");
  combo->insertItem("Frames");
  connect(combo, SIGNAL(activated(int)), this, SLOT(setScaleType(int)));

  layout->addWidget(combo);
  
  
  if(test == 0)
    m_view = new QAWL_View(this);
  else {
    m_view = new QAWL_View2(this);
  }

  connect(this, SIGNAL(emitScaleType(int)), m_view, SLOT(setScaleType(int)));

  layout->addWidget(m_view);
}

QAWL_PreviewTest2::~QAWL_PreviewTest2()
{
  qDebug("QAWL_PreviewTest2::~QAWL_PreviewTest2");
}

void QAWL_PreviewTest2::open()
{
  QString fileName = QFileDialog::getOpenFileName
    (QString::null, "*.wav *.aiff", this, "Select audio file");

  if(fileName.isEmpty()) return;

  m_view->setFile(fileName);
}

void QAWL_PreviewTest2::setScale(int scale)
{
  double logScale = exp(scale / 100.0);

  double logMax = exp(m_slider->maxValue() / 100.0);
  double logMin = exp(m_slider->minValue() / 100.0);

  const double realMax = 400000.0;
  const double realMin = 0.3;

  double dscale = (logScale - logMin) / (logMax - logMin);
  dscale = (dscale * (realMax - realMin)) + realMin;

  if(dscale == m_scale) return;

  /* qDebug("QAWL_PreviewTest2::setScale # (%3.3lf %3.3lf %3.3lf) %lf -> %lf", 
     logScale, logMin, logMax, m_scale, dscale); */

  m_scale = dscale;

  m_view->setScale(dscale);
}

void QAWL_PreviewTest2::setMagnify(int m)
{
  double magnify = m / 1000.0 * 10.0 + 1.0;

  m_view->setMagnification(magnify);
}

void QAWL_PreviewTest2::setScaleType(int t)
{
  emit emitScaleType(t);
}
