/*  A test application to see how capabilities work in practice.
   
    g++ -Wall capgive.C -o capgive -lcap

    AS ROOT:
    
    chown root capgive; chmod 4755 capgive
    
    ./capgive 
 */

#include <errno.h>
#include <sched.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#undef _POSIX_SOURCE
#include <sys/capability.h>

#include <di_types.h>
#include <unistd.h>

int main(int argc, char **argv)
{
  cap_value_t cap = CAP_SYS_NICE;
  cap_t caps;
  cap_t caps2;

  pid_t ppid = getppid();

  if(argc > 1)
    ppid = atol(argv[1]);

  printf("%s %d\n", argv[0], ppid);
  fflush(0);

  caps = cap_init();
  cap_clear(caps);

  cap_set_flag(caps, CAP_EFFECTIVE, 1, &cap , CAP_SET);
  cap_set_flag(caps, CAP_INHERITABLE, 1, &cap , CAP_SET);
  cap_set_flag(caps, CAP_PERMITTED, 1, &cap , CAP_SET);

  cap = CAP_SETPCAP;

  caps2 = cap_init();
  cap_clear(caps2);
  cap_set_flag(caps2, CAP_EFFECTIVE, 1, &cap , CAP_SET);
  cap_set_flag(caps2, CAP_INHERITABLE, 1, &cap , CAP_SET);
  cap_set_flag(caps2, CAP_PERMITTED, 1, &cap , CAP_SET);
  int err = cap_set_proc(caps2);

  if(err) {
    fprintf(stderr, "cap_set_proc FAILED %s\n", strerror(errno));
  }

  printf("Setting caps for process %d\n", ppid);
  
  err = capsetp(ppid, caps);

  if(err) {
    fprintf(stderr, "capsetp FAILED: %s\n", strerror(errno));
  }
  err = capgetp(ppid, caps2);

  if(err) {
    fprintf(stderr, "capgetp FAILED: %s\n", strerror(errno));
  }

  printf("Two caps: \n\t\"%s\"\n", cap_to_text(caps, 0));
  printf("\t\"%s\"\n",  cap_to_text(caps2, 0));
  fflush(0);

  return 0;
}
