/*  A test application to see how capabilities work in practice.
   
   gcc -Wall captest.c -o captest -lcap

   AS ROOT:

   chown root captest; chmod 4755 captest

   AS USER:
   ./captest
   The following does not drop root privileges with setuid
   ./captest -n

 */

#include <errno.h>
#include <sched.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#undef _POSIX_SOURCE
#include <sys/capability.h>

#include <di_types.h>
#include <unistd.h>

int main(int argc, char **argv)
{
  cap_value_t cap = CAP_SYS_NICE;
  
  int dropsuid = 1;
  int useCapgive = 1;
  uid_t uid;
  uid_t euid;
  pid_t pid;
  cap_t caps;
  char *capStr;
  int policy;
  int i;
  struct sched_param scp;

  char buf[128];

  FILE *dangerFile = 0;
  const char * rootFile = "/root/captest.foo";

  caps = cap_init();
  cap_clear(caps);

  cap_set_flag(caps, CAP_EFFECTIVE, 1, &cap , CAP_SET);
  cap_set_flag(caps, CAP_INHERITABLE, 1, &cap , CAP_SET);
  cap_set_flag(caps, CAP_PERMITTED, 1, &cap , CAP_SET);

  /* cap = CAP_SETPCAP;
  cap_set_flag(caps, CAP_EFFECTIVE, 1, &cap , CAP_SET);
  cap_set_flag(caps, CAP_INHERITABLE, 1, &cap , CAP_SET);
  cap_set_flag(caps, CAP_PERMITTED, 1, &cap , CAP_SET); */


  cap_set_proc(caps); /* This should get us the capabilities*/
  capsetp(pid, caps); /* So should this. */

  capStr = cap_to_text(cap_get_proc(), 0);
 
  printf("Capabilities \"%s\"\n", capStr);

  uid = getuid();
  euid = geteuid();

  if(!(uid && !euid))
    fprintf(stderr, "BINARY NOT SUID !!\n");
  
  for(i=0; i < argc; i++) {
    if(strcmp(argv[i], "-n") == 0)
      dropsuid = 0;
    if(strcmp(argv[i], "-N") == 0)
      useCapgive = 0;
  }

  if(dropsuid)
    seteuid(uid); /* Drop suid privileges. */
    // setreuid(0, uid); /* Drop suid privileges. */
  else
    puts("Keeping all suid powers");

  pid = getpid();
  
  if(useCapgive) {
    sprintf(buf, "./capgive %d", pid);
    printf("Running \"%s\"\n", buf);
    system(buf);
  }
  
  scp.sched_priority = 50;

  i = sched_setscheduler(pid, SCHED_FIFO, &scp);

  if(i) {
    fprintf(stderr, "sched_setscheduler failed : \"%s\"\n", 
	    strerror(errno));    
  }
  
  policy = sched_getscheduler(pid);

  if(policy != SCHED_FIFO) 
    fprintf(stderr, "Sched policy not FIFO !!\n");
  else
    fprintf(stdout, "Sched policy *IS* FIFO !!\n");

  /* Print the current capabilities. These should include
     cap_sys_nice. */

  capStr = cap_to_text(cap_get_proc(), 0);
  printf("Capabilities at end %s\n", capStr);


  dangerFile = fopen(rootFile, "w");
  
  if(dangerFile)
    printf("Managed to open :\"%s\"\n", rootFile);

  return 0;
}
