/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of Mustajuuri GPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2001.
 * Tommi.Ilmonen@hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the General Public License (GPL),
 * versions 2. Extra clauses can be found from the file MUSTAJUURI_GPL
 * and further explanations from the file MUSTAJUURI_LICENSE. The
 * license text can be found in file "GPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *  
 */

#ifndef MJ_AUDIO_SAMPLE_H
#define MJ_AUDIO_SAMPLE_H

#include <mj_vector.h>

#include <mj_file_load_info.h>

/** Audio sample data. This class is used to read audio files into
    application memory. It is basically just a wrapper around
    MJ_Vector<float> that knows how to load audio files.

    @author Tommi Ilmonen
*/
class MJ_AudioSample
{
public:
  MJ_AudioSample();
  ~MJ_AudioSample();

  bool loadFile(const QString &fileName, 
		bool forceRead);

  const QString &fileName() { return m_fileLoadInfo.file(); }
  
  MJ_Vector<float> & wave() { return m_wave; }
  const MJ_Vector<float> & wave() const { return m_wave; }

  void clear() { m_wave.resize(0); m_fileLoadInfo.clear(); }

protected:
  MJ_Vector<float> m_wave;

  MJ_FileLoadInfo  m_fileLoadInfo;

  static bool      m_debug;
};

#endif
