/* COPYRIGHT
 *
 * This file is part of Mustajuuri GPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2001.
 * Tommi.Ilmonen@hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
  
 * This file is licensed under the GNU Public License (GPL).

 *  
 */


#include <di_error.h>
#include <di_sleep.h>
#include <di_time.h>
#include <di_thread.h>

#include <vector>

#include <unistd.h>
#include <sched.h>
#include <stdio.h>
#include <stdlib.h>
#include <strings.h>

static bool done = false;

static bool ok = true;

class WatchDog : public DI_Thread
{
public:
  WatchDog() {}
  virtual ~WatchDog() {}

protected:
  
  virtual void childLoop() 
  {
    system("/usr/local/bin/givertcap");
    
    sched_param sp;
    sp.sched_priority = 99;
    
    if(!ok) {
      DI_fatal(DI_ERR_UNKNOWN, "Something is blocked. System hang????");
      ok = false;

      DI_Sleep::sleepMs(500);
    }
  }

};

class HighPrioMemuser : public DI_Thread
{
public:
  HighPrioMemuser() {}
  virtual ~HighPrioMemuser() {}

protected:
  
  std::vector<void *> m_membuffers;

  virtual void childLoop() 
  {
    system("/usr/local/bin/givertcap");
    
    sched_param sp;
    sp.sched_priority = 60;

    DI_Time t1, t2, begint, endt, lastcheck;
    
    if(sched_setscheduler(0, SCHED_FIFO, &sp) == -1) {
      puts("FAILED TO GET HIGH PRIORITY");
    }
    else 
      puts("GOT RT PRIORITY");

    m_membuffers.resize(10000);
    bzero(m_membuffers.begin(), m_membuffers.size() * sizeof(void *));

    const uint cases = 7;
    int sizesToGet[cases] = {16, 48, 128, 300, 500, 1000, 1011 };

    for(uint j=0; j < cases; j++) {

      for(uint i=0; i < m_membuffers.size(); i++) {
	free(m_membuffers[i]);
	m_membuffers[i] = malloc(sizesToGet[j]);
      }
    }

    uint count = 0;

    // Free latencies
    double minFLatency = 1e+10;
    double maxFLatency = 0.0;
    double maxMaxFLatency = 0.0;
    // Malloc latencies
    double minMLatency = 1e+10;
    double maxMLatency = 0.0;
    double maxMaxMLatency = 0.0;

    begint.getTimeOfDay();
    lastcheck = begint;

    puts("Testing memory management");

    for(endt.getTimeOfDay(); begint.secsTo(endt) < 60.0; endt.getTimeOfDay()) {

      DI_Sleep::sleepMs(15);

      t2.getTimeOfDay();

      if(lastcheck.secsTo(t2) > 3.0) {

	if(maxMaxMLatency < maxMLatency) 
	  maxMaxMLatency = maxMLatency;

	if(maxMaxFLatency < maxFLatency) 
	  maxMaxFLatency = maxFLatency;

	printf("Current free latency was in range  %3.3f-%3.3f (%3.3f) ms\n", 
	       minFLatency, maxFLatency, maxMaxFLatency);
	printf("Current alloc latency was in range %3.3f-%3.3f (%3.3f) ms\n",
	       minMLatency, maxMLatency, maxMaxMLatency);
	fflush(0);
	lastcheck = t2;

	maxMLatency = maxFLatency = 0.0;
      }

      for(uint j=0; j < 100; j++) {
	uint target = count++ % m_membuffers.size();

	t1.getTimeOfDay();
	free(m_membuffers[target]);
	t2.getTimeOfDay();

	double delta = t1.secsTo(t2) * 1000.0;

	if(maxFLatency < delta) maxFLatency = delta;
	if(minFLatency > delta) minFLatency = delta;
	
	m_membuffers[target] = calloc(1, rand() % 1024);
	
	t1.getTimeOfDay();
	delta = t2.secsTo(t1) * 1000.0;
	
	if(maxMLatency < delta) maxMLatency = delta;
	if(minMLatency > delta) minMLatency = delta;
	
      }
    }
    
    printf("Free latency was in range %f-%f ms\n", minFLatency, maxFLatency);
    printf("Alloc latency was in range %f-%f ms\n", minMLatency, maxMLatency);

    done = true;
    
  }

};

void memoryHog()
{
  system("/usr/local/bin/givertcap");
    
  sched_param sp;
  sp.sched_priority = 50;

  DI_Time t1, t2, begint, endt, lastcheck;
    
  if(sched_setscheduler(0, SCHED_FIFO, &sp) == -1) {
    puts("HOG FAILED TO GET HIGH PRIORITY");
  }
  else 
    puts("HOG GOT RT PRIORITY");

  std::vector<void *> membuffers;
  
  membuffers.resize(2000);

  bzero(membuffers.begin(), membuffers.size() * sizeof(void *));

  uint count = 0;

  while(!done) {

    DI_Sleep::sleepMs(20);

    for(uint j=0; j < 10000; j++) {

      uint target = count++ % membuffers.size();
    
      free(membuffers[target]);
      membuffers[target] = calloc(1, rand() % 2048);
    }
  }
}

int main()
{
  WatchDog dog;

  HighPrioMemuser thread;

  dog.run();
  DI_Sleep::sleepMs(50);
  thread.run(true);
  
  memoryHog();
}
