/* COPYRIGHT
 *
 * This file is part of Mustajuuri GPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2001.
 * Tommi.Ilmonen@hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
 * This file is licensed under the General Public License (GPL),
 * versions 2. Extra clauses can be found from the file MUSTAJUURI_GPL
 * and further explanations from the file MUSTAJUURI_LICENSE. The
 * license text can be found in file "GPL.txt".
 *
 * If you want a parallel license (for commercial reasons for example),
 * you should negotiate the matter with the author(s).
 *  
 */

#include <qfile.h> 
#include <qtextstream.h> 

const QString licenseStart("/* COPYRIGHT");
const QString licenseEnd("*/");

void relicenseFile(const QString &fileName, const QString &newLicense)
{
  QFile input(fileName);

  bool ok = input.open(IO_ReadOnly);

  if(!ok) return;

  QTextStream inStream(&input);

  QString buffer;
  QTextStream tmpStream(&buffer, IO_WriteOnly);
  QString tmp;

  bool onLicense = false;

  while (!input.atEnd()) {
    tmp = inStream.readLine();
    
    if(!onLicense) {
      if(tmp.find(licenseStart) == 0) {
	onLicense = true;
	tmpStream << newLicense;
      }
      else
	tmpStream << tmp << "\n";
    }
    else if(tmp.find(licenseEnd) >= 0) {
      onLicense = false;
    }
  }

  input.close();

  
  QFile output(fileName);
  ok = output.open(IO_WriteOnly);
  QTextStream outStream(&output);

  outStream << buffer;
  printf("Done file \"%s\"\n", (const char *) fileName);
}

int main(int argc, char **argv)
{
  if(argc < 3) {
    fprintf(stderr, "Usage:\n%s license-file file1 [file2 file3...]\n", *argv);
    fflush(0);
    return 0;
  }

  
  QFile licenseFile(argv[1]);
  bool ok = licenseFile.open(IO_ReadOnly);
  
  if(!ok) {
    fprintf(stderr, "Failed to open license file.");
    fflush(0);
    return 0;
  }

  QTextStream tmpStream(&licenseFile);
  
  QString license;
  
  while(!licenseFile.atEnd()) {
    QString tmp(tmpStream.readLine());
    license += tmp + "\n";
  }

  for(int i=2; i < argc; i++) 
    relicenseFile(argv[i], license);
}
