/* 
   This software is only useful on Linux.

   Compile command:

   g++ -Wall main.C mj_rtc_hz.C -o rtchz

 */

#include <mj_rtc_hz.h>

#include <stdlib.h>
#include <unistd.h>
#include <iostream.h>

/* If this is non-zero, the user is allowed to choose the device name.
 
   Since we expect this application is run as root, it may be a good
   idea to not allow this choice. Otherwise users may be able to
   control (disturb) devices that they have no right to. */
#define MJ_RTC_ALLOW_DEVICE_CHOICE 0

/* 
   rtchz
   
   Controls Real-Time Clock (RTC).
   
   Make this software "sudo" to be able to control the RTC frequency
   in any way you want.
   
   Arguments:
   
   s:       Get status (frequency)
   r +arg:  Set frequency
   g: Switch debugging on.  */

// Maximum RTC frequency in Hz.
static const int maxHz = 8192;

void MJ_helper()
{
  cerr << "Options: \n" 
       << "\t -h      \tThis help\n"
       << "\t -s      \tQuery status\n"
       << "\t -r +arg \tSet frequency         - default: 64 Hz\n"
       << "\t -g      \tEnter debugging mode (does very little)\n"
       << endl;
  exit(-1);
}

int main(int argc, char **argv)
{
  int hz = 64;
  bool verbose = true;
  bool success;
  bool getStatus = false;

  const char *device = 0;

  int c;

#if MJ_RTC_ALLOW_DEVICE_CHOICE
  while ((c = getopt(argc, argv, "d:hsgr:")) != -1) { 
#else
  while	((c = getopt(argc, argv, "hsgr:")) != -1) {
#endif

    switch (c) {

#if MJ_RTC_ALLOW_DEVICE_CHOICE
    case 'd':
      device = optarg;
      break; 
#endif
      
    case 'g':
      verbose = true;
      break;
      
    case 'h':
      MJ_helper();
      break;

    case 'r':
      hz = atoi(optarg);
      break;

    case 's':
      getStatus = true;
      break;

    }
  }
  
  MJ_RtcHz rtc;
  if(!rtc.open(device)) {
    cerr << "Failed to  init RTC control" << endl;
    return 0;
  }

  if(getStatus) {
    cout << "RTC frequency = " << rtc.getHz() << " Hz" << endl;
  }
  else {

    if(hz <= 0) {
      cerr << "No frequnecy" << endl;
      return 0;
    }

    if(hz > maxHz) {
      cerr << "Too high frequency" << endl;
      return 0;      
    }

    if(!(success = rtc.setHz(hz))) {
      cerr << "Failed to set RTC frequency to " << hz << endl;
    }
    else if(verbose) {
      cout << "Set RTC frequency to " << hz << endl;
    }
  }

  return 0;
}
