
#include "mj_rtc_hz.h"

#include <linux/mc146818rtc.h>

#include <sys/ioctl.h>
#include <fcntl.h>
#include <unistd.h>

MJ_RtcHz::MJ_RtcHz()
  : m_fd(-1)
{}

MJ_RtcHz::~MJ_RtcHz()
{
  if(m_fd != -1) ::close(m_fd);
}

bool MJ_RtcHz::open(const char *device = "/dev/rtc")
{
  return ((m_fd = ::open(device, O_RDONLY)) != -1);
}

bool MJ_RtcHz::close()
{
  bool ok = true;

  if(m_fd == -1) return false;
  ok = (::close(m_fd) != -1);
  
  m_fd = -1;

  return ok; // Not open
}

bool MJ_RtcHz::setHz(int hz)
{
  if(ioctl(m_fd, RTC_IRQP_SET, hz) == -1)
    return false;

  return (ioctl(m_fd, RTC_PIE_ON, 0) != -1);
}

int MJ_RtcHz::getHz()
{
  unsigned long hz;

  if(ioctl(m_fd, RTC_IRQP_READ, &hz) == -1)
    return -1;
  return hz;
}

bool MJ_RtcHz::waitTicks(int n)
{
  unsigned long data;

  bool ok = true;

  for(int i=0; i < n; i++) {
    if(read(m_fd, &data, sizeof(unsigned long)) == -1) {
      return false;
    }
    if(i) {
      unsigned long ticks = (data >> 8) & 0xffffff;
      if(ticks != 1) {
	i += ticks - 1;  // Update the loop counter.
	ok = false;
      }
    }
  }

  return ok;
}
