/* -*- C++ -*- */

#ifndef MJ_RTC_HZ_H
#define MJ_RTC_HZ_H

/** Sets the frequency of the real-time clock (RTC). 

    Only useful on Linux. 

    For documentation see file "rtc.txt" in linux kernel
    documentation.  */

class MJ_RtcHz 
{
public:
  MJ_RtcHz();
  ~MJ_RtcHz();
  
  /** Inits the device (opens for reading). */
  bool open(const char *device = "/dev/rtc");

  /** Closes the device. */
  bool close();

  /** Returns true if the device is open. */
  bool isOpen() { return (m_fd != -1); }

  /** You may/will need root privileges to set the frequency. */
  bool setHz(int Hz);

  /** Returns current frequency. Upon error, returns -1. */
  int getHz();

  /** Wait for n ticks. Uses "read" to block to the device. */
  bool waitTicks(int n);

protected:
  int m_fd;
};

#endif
