/* COPYRIGHT
 *
 * This file is part of Mustajuuri GPL modules.
 *
 * Author: Tommi Ilmonen, 1999-2001.
 * Tommi.Ilmonen@hut.fi
 *
 * http://www.tml.hut.fi/~tilmonen/mustajuuri/
 *
  
 * This file is licensed under the GNU Public License (GPL).

 *  
 */

#include <di_thread.h>
#include <di_error.h>
#include <di_sleep.h>

#include <qdatetime.h> 

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

class MJ_CpuEater : public DI_Thread
{
public:
  MJ_CpuEater() {}
  virtual ~MJ_CpuEater() {}
  void setEndTime(const QTime &t) { m_endTime = t; }

protected:
  virtual void childLoop();
  QTime m_endTime;
};


void MJ_CpuEater::childLoop()
{
  puts("MJ_CpuEater::childLoop");

  uint loops = 500000;

  while(true) {
    if(QTime::currentTime() > m_endTime) {
      printf("Thread is done %p\n", this);
      fflush(0);
      return;
    }
    for(uint i=0; i < loops; i++)
      rand();
  }

}

int main(int argc, char **argv)
{
  uint i, nthreads = 4;

  uint secs = 10;

  if(argc > 1)
    nthreads = atoi(argv[1]);

  if(argc > 2)
    secs = atoi(argv[2]);

  if(nthreads > 30 || !nthreads) {
    fprintf(stderr, "Bad number of threads required: %d\n", nthreads);
    fflush(0);
    return 0;
  }

  QTime t(QTime::currentTime());
  t =t.addSecs(secs);

  MJ_CpuEater *eaters = new MJ_CpuEater[nthreads];

  for(i=0; i < nthreads; i++) {
    eaters[i].setEndTime(t);
    eaters[i].run(true);
  }
  
  sleep(secs);

  for(i=0; i < nthreads; i++)
    eaters[i].waitEnd();
}


