
#include <viena_application.h>

#include <mj_error.h>

#include <di_video_input.h>
#include <di_video_avcodec.h>

#include <qtimer.h>

#include <assert.h>

#include <GL/glu.h>

VienaApplication::VienaApplication()
{
  bzero(m_camera, sizeof(m_camera));
  resize(640 , 480 * (CAMERAS + 1));
}

VienaApplication::~VienaApplication()
{}
 
bool VienaApplication::init(const char * configFile)
{
  DI_VideoAvcodecInput * movieIn = new DI_VideoAvcodecInput();

  const char * file = 
    "/home/tilmonen/scratchtilmonen/movies/canon-s1-MVI_0134.AVI";

  if(!movieIn->open(file))
    return false;

  m_camera[0] = movieIn;

  int w = m_camera[0]->width();
  int h = m_camera[0]->height();

  m_image.resize(h, w);
  m_image.setAll(0);

  QTimer * t = new QTimer(this);
  
  connect(t, SIGNAL(timeout()), this, SLOT(updateGL()));

  t->start(100);

  return true;
}

void VienaApplication::initializeGL ()
{
  glGenTextures(CAMERAS, m_textures);

  // Texture size
  int s = 1024;
  
  for(uint i=0; i < 2; i++) {

    glBindTexture(GL_TEXTURE_2D, m_textures[i]);

    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP);
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP);
    glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
    
    glTexImage2D(GL_TEXTURE_2D, 0, GL_LUMINANCE, s, s,
		 0, GL_LUMINANCE, GL_UNSIGNED_BYTE, 0);
  }
}

void VienaApplication::resizeGL ( int, int )
{
  
}

void VienaApplication::paintGL ()
{
  // puts("paintGL");

  estimateCandleLocation();
  

  // Now the display part:

  glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
  
  glViewport(0, 0, width(), height());
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  gluOrtho2D(0, 640, 0, 960);

  float xt = 640.0 / 1024.0;
  float yt = 480.0 / 1024.0;

  for(int i = 0; i < CAMERAS; i++) {
    
    glEnable(GL_TEXTURE_2D);
    glBindTexture(GL_TEXTURE_2D, m_textures[i]);
    
    glDisable(GL_LIGHTING);
    glColor3f(1, 1, 1);
    
    float y = i * 640;

    glBegin(GL_QUADS);

    glTexCoord2f(0.0, yt);
    glVertex2f(0, y);

    glTexCoord2f(xt, yt);
    glVertex2f(640, y);

    glTexCoord2f(xt, 0);
    glVertex2f(640, y + 480);

    glTexCoord2f(0, 0);
    glVertex2f(0, y + 480);

    glEnd();
  }

  glColor3f(0, 1, 1);
  glLineWidth(3);

  float s = 30.0;

  glBegin(GL_LINES);

  float y = 480.0f - m_candleY;

  glVertex2f(m_candleX, y - s);
  glVertex2f(m_candleX, y + s);

  glVertex2f(m_candleX - s, y);
  glVertex2f(m_candleX + s, y);

  glEnd();

  /* glDisable(GL_TEXTURE_2D);

  glBegin(GL_TRIANGLES);

  glColor3f(1, 1, 1);
  glVertex2f(0, 0);
  glVertex2f(600, 50);
  glVertex2f(300, 300);

  glEnd(); */
}

void VienaApplication::estimateCandleLocation()
{
  uint i;

  float x1, x2, y1, y2;

  DI_Image grayscale(DI_IMAGE_GRAYSCALE);
  grayscale.m_planes[0].m_data = m_image.begin();

  for(i = 0; i < CAMERAS; i++) {

    const DI_Image * image = m_camera[i]->captureImage();
    assert(image);

    bool ok = DI_ImageConversion::convert(image, & grayscale);
    
    if(!ok) {
      MJ_fatal
	(MJ_ERR_UNKNOWN, 
	 "VienaApplication::estimateCandleLocation # Could not get image");
    }
    
    int x, y;
    
    findImagePeak( & x, & y);
    
    glBindTexture(GL_TEXTURE_2D, m_textures[i]);
    glTexSubImage2D(GL_TEXTURE_2D, 0, 0, 0, 640, 480,
		    GL_LUMINANCE, GL_UNSIGNED_BYTE, m_image.begin());

    m_candleX = x;
    m_candleY = y;
  }
}

/** Seeks the pixel that conrtains most luminance in the image. */

void VienaApplication::findImagePeak(int * x, int * y)
{
  const uchar * begin = m_image.begin();
  const uchar * ptr = begin;
  const uchar * end = m_image.end();

  uchar peak = 0;

  int best = 0;

  for(; ptr < end; ptr++) {
    if(*ptr > peak) {
      peak = *ptr;
      best = ptr - begin;
    }
  }
  
  * x = best % m_image.cols();
  * y = best / m_image.cols();

}
