/* COPYRIGHT
 *
 * This file is part of the Visual Effects Engine - VEE
 *
 * Read the "VEE-LICENSE" file for the license.
 *
 * Authors & Copyright:   
 *
 * Tommi Ilmonen, Tuukka Heikura, Marko Myllymaa and 
 * Janne Kontkanen 2001-2004
 *
 * Additional copyrights: Tekes 2003-2004
 *
 * firstname.lastname@hut.fi
 *
 */

#include <vee_error.h>

#include <stdarg.h>
#include <stdio.h>

static void (*VEE_traceOut)(const char *) = 0;
static void (*VEE_errorOut)(const char *) = 0;
static void (*VEE_fatalOut)(const char *) = 0;
static void (*VEE_streamLock)(bool) = 0;

static void VEE_lockDefault(bool)
{
}

static void VEE_traceDefault(const char *buf)
{
  if(!VEE_streamLock) VEE_streamLock = & VEE_lockDefault;

  (*VEE_streamLock)(true);

  printf("VEE TRACE # %s\n", buf);
  fflush(0);  

  (*VEE_streamLock)(false);
}

static void VEE_errorDefault(const char *buf)
{
  if(!VEE_streamLock) VEE_streamLock = & VEE_lockDefault;

  (*VEE_streamLock)(true);

  fprintf(stderr, "VEE ERROR # %s\n", buf);
  fflush(0);  

  (*VEE_streamLock)(false);
}

static void VEE_fatalDefault(const char *buf)
{
  if(!VEE_streamLock) VEE_streamLock = & VEE_lockDefault;

  (*VEE_streamLock)(true);

  fprintf(stderr, "VEE FATAL # %s\nINTENTIONAL SEGMENTATION FAULT !!\n", buf);
  fflush(0);  

  (*VEE_streamLock)(false);
}


void VEE_setTraceStreamLock(void (*function)(bool))
{
  VEE_streamLock = function;
}

/// Print trace information to the user
extern void VEE_trace(const char *msg, ...)
{
  if(!VEE_traceOut) VEE_traceOut = &VEE_traceDefault;

  char buf[8196];
  va_list args;
  va_start(args, msg);
  vsprintf(buf, msg, args);

  (*VEE_traceOut)(buf);
  // printf("VEE TRACE # %s\n", buf);
  // fflush(0);
  va_end(args);
}

/// Print error information to the user
extern void VEE_error(const char *msg, ...)
{
  if(!VEE_errorOut) VEE_errorOut = &VEE_errorDefault;

  char buf[8196];
  va_list args;
  va_start(args, msg);
  vsprintf(buf, msg, args);

  (*VEE_errorOut)(buf);

  // fprintf(stderr, "VEE ERROR # %s\n", buf);
  fflush(0);
  va_end(args);
}


/** Print error information to the user. Once the information has been
    written the application will be killed with segmentation fault. */
extern void VEE_fatal(const char *msg, ...)
{
  if(!VEE_fatalOut) VEE_fatalOut = &VEE_fatalDefault;

  char buf[8196];
  va_list args;
  va_start(args, msg);
  vsprintf(buf, msg, args);

  (*VEE_fatalOut)(buf);

  /* fprintf(stderr, "VEE FATAL ERROR # %s\nINTENTIONAL SEGMENTATION FAULT !!\n",
	  buf);

	  fflush(0); */

  int *badAddress = 0;
  *badAddress = 1; // Segmentation fault
}

