/* COPYRIGHT
 *
 * This file is part of the Visual Effects Engine - VEE
 *
 * Read the "VEE-LICENSE" file for the license.
 *
 * Authors & Copyright:   
 *
 * Tommi Ilmonen, Tuukka Heikura, Marko Myllymaa and 
 * Janne Kontkanen 2001-2004
 *
 * Additional copyrights: Tekes 2003-2004
 *
 * firstname.lastname@hut.fi
 *
 */

#include <vee_color_table.h>
#include <vee_force_plane.h>
#include <vee_gravitate.h>
#include <vee_gravity.h>
#include <vee_looks_color_table.h>
#include <vee_volume_ellipsoid.h>
#include <vee_vortex.h>

int main()
{
  VEE_RgbaTable colors;
  VEE_LooksColorTable looks(&colors);

  VEE_SceneBox *vps = new VEE_SceneBox(0, VEE_Vector3(-100.0f,-100.0f,-100.0f)
                                        , VEE_Vector3(100.0f,100.0f,100.0f) );
  VEE_Gravity *grav = new VEE_Gravity(VEE_Vector3(0.0f, -0.01f, 0.0f));
  VEE_VolumeEllipsoid *area = 
    new VEE_VolumeEllipsoid(VEE_Vector3(1.0f,1.0f,1.0f));
  VEE_Gravitate *pull = new VEE_Gravitate(VEE_Vector3(2.0f, -5.0f, 5.0f),
					  VEE_Vector3(0.0f, 0.0f, 0.0f),
					  10000.0,
					  0.05,
					  true);
  VEE_Vortex *vortex = new VEE_Vortex(VEE_Vector3(0.0f, -1.0f, 0.0f),
				   VEE_Vector3(0.0f, 0.0f, 0.0f),
				   0.005, 11);
  VEE_ForcePlane *fplane = new VEE_ForcePlane(VEE_Vector3(0.0f, -20.0f, 0.0f),
					      VEE_Vector3ZERO,
					      VEE_Vector3(0.0f, 1.0f, 0.0f),
					      0.5);

  vps->addObject(grav);
  vps->addObject(pull);
  vps->addObject(fplane);

  while(grav->age()<10.0)
  {
    vps->update(10, 5, 5);
  }

  vps->write(&std::cout);

  delete area;
  delete grav;
  delete pull;
  delete vortex;
  delete vps;
}

