/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of the Visual Effects Engine - VEE
 *
 * Read the "VEE-LICENSE" file for the license.
 *
 * Authors & Copyright:   
 *
 * Tommi Ilmonen, Tuukka Heikura, Marko Myllymaa and 
 * Janne Kontkanen 2001-2004
 *
 * Additional copyrights: Tekes 2003-2004
 *
 * firstname.lastname@hut.fi
 *
 */

#ifndef VEE_OPERATOR_LIST_H
#define VEE_OPERATOR_LIST_H

#include <vector>
#include <string>

class VEE_Object;
class VEE_Operator;
class VEE_Value;

class VEE_OperatorList
{
public:
  VEE_OperatorList();
  VEE_OperatorList(const VEE_OperatorList &l) { *this = l; }
  ~VEE_OperatorList();
  
  void appendOperator(VEE_Operator *o, const char * name = 0);
  void appendOperator2(VEE_Object *, VEE_Operator *o, const char * name = 0);
  bool removeOperator(const char *name, bool alsoDelete = true);

  void operate(VEE_Object *, float);
  void operate2(float);
  void clear();

  bool setValue(const char *name, VEE_Value *);

  VEE_Operator *get(int);

  VEE_OperatorList & operator = (const VEE_OperatorList &);

  uint size() const { return m_operators.size(); }

protected:

  class Holder {
  public:
    
    VEE_Operator *oper() { return m_operator; }

    std::string   m_name;
    VEE_Operator *m_operator;
    VEE_Object   *m_target;
  };

  std::vector<Holder> m_operators;
  typedef std::vector<Holder>::iterator holderIter_t;

};

#endif
