/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of the Visual Effects Engine - VEE
 *
 * Read the "VEE-LICENSE" file for the license.
 *
 * Authors & Copyright:   
 *
 * Tommi Ilmonen, Tuukka Heikura, Marko Myllymaa and 
 * Janne Kontkanen 2001-2004
 *
 * Additional copyrights: Tekes 2003-2004
 *
 * firstname.lastname@hut.fi
 *
 */
#ifndef VEE_WIND_H
#define VEE_WIND_H

#include <vee_clonable_pointer.h>
#include <vee_object.h>
#include <vee_scene.h>
#include <vee_volume.h>

#include <mj_filter_ap1.h>

class VEE_Volume;

/** Wind affects other particles with a constant global force
    along a defined vector.
*/
class VEE_Wind : public VEE_ForceObject
{
public:
  // VEE_Wind(const VEE_Wind &that) : m_volume(0) { *this = that; }
  VEE_Wind(const VEE_Vector3 &dir = VEE_Vector3(1.0f, 0.0f, 0.0f),
	   float strength = 1.0,
	   float gustiness = 0.0f,
	   float gustfreq = 0.5f);
  ~VEE_Wind();

  virtual bool setValue(const char *, VEE_Value *);
  virtual VEE_Variable getVariable(const char *) const;
  virtual VEE_Object * clone() const;

  virtual void update(float);

  void getForce(VEE_Vector3 *force,
		const VEE_Vector3 &,
                const VEE_Vector3 &);
  void boundingBoxOfForce(VEE_BoundingBox *box);

  static VEE_Object * factory();

  // VEE_Wind & operator = (const VEE_Wind &);

  VEE_Volume * volume() { return m_volume.ptr(); }

protected:

  void redesignFilter();
  void recalculateWind();

  VEE_Vector3 m_wind;
  VEE_Vector3 m_direction;
  float       m_strength;
  float       m_gustStrength;
  float       m_gustFreq;
  float       m_sinceLast;
  MJ_FilterAP1<float> m_gustFilter;

  VEE_ClonablePointer<VEE_Volume> m_volume;
};

#endif
