/* COPYRIGHT
 *
 * This file is part of the Visual Effects Engine - VEE
 *
 * Read the "VEE-LICENSE" file for the license.
 *
 * Authors & Copyright:   
 *
 * Tommi Ilmonen, Tuukka Heikura, Marko Myllymaa and 
 * Janne Kontkanen 2001-2004
 *
 * Additional copyrights: Tekes 2003-2004
 *
 * firstname.lastname@hut.fi
 *
 */

#include <vee_bgl_app.h>

#include <gee_group.h>
#include <gee_solar_conversion.h>
#include <gee_world.h>

#include <vee_error.h>

#include <Solar/Object.hpp>

#include <SDL/SDL.h>

#include <GL/gl.h>
#include <GL/glu.h>


int main(int argc, char **argv)
{
  int i;

  std::vector<const char *> filenames;
  const char * materials = "default.mtl";  
  const char * vrs = 0;
  const char * bgl = 0;

  VEE_SdlApp a;
  VEE_BglApp ba(&a);

  for(i = 1; i < argc; i++) {
    if(strcmp(argv[i], "--vrs") == 0 && i + 1 < argc)
      vrs = argv[++i];
    else if(strcmp(argv[i], "--bgl") == 0 && i + 1 < argc)
      bgl = argv[++i];
    else if(strcmp(argv[i], "--obj") == 0 && i + 1 < argc)
      filenames.push_back(argv[++i]);
    else if(strcmp(argv[i], "--mtl") == 0 && i + 1 < argc)
      materials = argv[++i];
    else if(strcmp(argv[i], "--nolists") == 0)
      a.useLists(false);
    else if(strcmp(argv[i], "--maxframes") == 0)
      a.setMaxFrames(atoi(argv[++i]));
    else if(strcmp(argv[i], "--videowall") == 0)
      a.setAppType(VEE_SdlApp::TEXTURE_STREAM);
    else {
      VEE_error("%s # Undefined command line options \"%s\"", argv[0],
		argv[i]);
      return 0;
    }
  }

  a.setMtlFile(materials);

  a.init();

  for(i = 0; i < (int) filenames.size(); i++) {

    GEE_World world;
    GEE_Group * g = new GEE_Group();
    
    world.addGeometry(g);
    
    if(!g->loadModel(filenames[i]))
      continue;

    g->convertPolygonsToTriangles();
    
    Solar::Model * model = new Solar::Model();

    GEE_ToSolar::convert(model, g);
    
    Solar::Object * solarobj = new Solar::Object();
      
    solarobj->setModel(model);

    model->normalize(20.0, 1);
    
    a.addModel(model);
    a.addObject(solarobj);
  }

  VEE_Qt::initImageIO();

  if(!vrs)
    VEE_fatal("%s # Must have a VRS setup file", argv[0]);

  ba.loadConfigs(vrs, bgl);

  // printf("App type = %d\n", (int) a.appType());
  
  ba.exec();

  return 0;
}



