/* COPYRIGHT
 *
 * This file is part of the Visual Effects Engine - VEE
 *
 * Read the "VEE-LICENSE" file for the license.
 *
 * Authors & Copyright:   
 *
 * Tommi Ilmonen, Tuukka Heikura, Marko Myllymaa and 
 * Janne Kontkanen 2001-2004
 *
 * Additional copyrights: Tekes 2003-2004
 *
 * firstname.lastname@hut.fi
 *
 */

#include <vee_polygon_clouds.h>

#include <vee_looks_color_table.h>
#include <vee_polygon.h>

VEE_PolygonClouds::VEE_PolygonClouds(const VEE_Vector3 & location, 
				     const VEE_Vector3 & velocity,
				     float size,
				     float age,
				     float cloudInterval, 
				     float magnitude)
  : VEE_VortexEffect(location, velocity, magnitude, age, size, 
		    5.0 / cloudInterval, 4.0 / cloudInterval),
    m_cloudInterval(cloudInterval),
    m_sinceLastCloud(0.0)
{
  VEE_Rgba myGray(0.3, 0.3, 0.3, 0.7);
  m_colorTable.appendColor(VEE_Rgba::Black);
  m_colorTable.appendColor(myGray);
  m_colorTable.appendColor(myGray);
  m_colorTable.appendColor(myGray);  
  m_colorTable.appendColor(VEE_Rgba::Black);
}

VEE_PolygonClouds::~VEE_PolygonClouds()
{}

void VEE_PolygonClouds::update(float dt)
{
  VEE_VortexEffect::update(dt);

  m_sinceLastCloud += dt;

  while(m_sinceLastCloud > m_cloudInterval) {
    
    emitCloud();

    m_sinceLastCloud -= m_cloudInterval;
  }
}

void VEE_PolygonClouds::emitCloud()
{
  float width = m_range * (m_rand.rand01() + 0.1);
  float height = m_range * (m_rand.rand01() + 0.1);

  VEE_Vector3 loc(m_rand.rand11() * m_range,
		  0.0f,
		  m_rand.rand11() * m_range);

  
  loc += m_location;
  
  // The polygon is the cloud
  VEE_Polygon *poly = VEE_Polygon::createNewMesh(2, 2, width, height);

  loc[0] -= width  * 0.5;
  loc[2] -= height * 0.5;

  poly->move(loc);
  poly->setLooks(new VEE_LooksColorTable(&m_colorTable));
  poly->setAgeLimit(m_cloudInterval * 5.0);
  poly->setStiffness(100.0);

  m_sceneBox->addObject(poly);
}
