/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of the Visual Effects Engine - VEE
 *
 * Read the "VEE-LICENSE" file for the license.
 *
 * Authors & Copyright:   
 *
 * Tommi Ilmonen, Tuukka Heikura, Marko Myllymaa and 
 * Janne Kontkanen 2001-2004
 *
 * Additional copyrights: Tekes 2003-2004
 *
 * firstname.lastname@hut.fi
 *
 */

#ifndef GEE_PATH_TO_PARTICLES_H
#define GEE_PATH_TO_PARTICLES_H

#include <gee_param_writer.h>
#include <gee_particle_object.h>
#include <gee_path_to_x.h>

#include <vee_random.h>

class GEE_PathToParticles : public GEE_PathToX
{
public:
  GEE_PathToParticles(GEE_id_t pathId = -1, 
		      GEE_id_t particleId = -1);
  virtual ~GEE_PathToParticles();

  virtual bool start(GEE_World *, const GEE_TimeStamp &);
  virtual bool stop(GEE_World *, const GEE_TimeStamp &);

  virtual bool read(GEE_Io *);
  virtual bool write(GEE_Io *);
  virtual const char * type() { return "GEE_PathToParticles"; }

  static GEE_Operator * create();

  void setRate(float r)   { m_rate = r; }
  void setRadius(float r) { m_radius = r; }
  void setRadiusExaggerate(float e) { m_exaggerateRadius = e; }
  void setRadiusNorm(float n) { m_radiusNorm = n; }
  
  void setUseNapra(bool use) { m_useNapra = use; }

  GEE_ParticleData & particle() { return m_particle; }

  void setTextureName(const std::string &);

protected:

  virtual bool handleNewVertex(GEE_Path3D::iterator &);

  GEE_ParamWriter      m_writer;

  GEE_id_t             m_particleId;

  VEE_Vector3          m_prevVertex;
  float                m_rate;
  float                m_radius;
  float                m_exaggerateRadius;
  float                m_radiusNorm;
  /** Rotation range in radians. */
  float                m_rotationRange;
  uint                 m_randomSeed;
  float                m_remaining;
  float                m_prevTime;
  VEE_RandomUniform    m_rand;
 
  bool                 m_useNapra;

  GEE_ParticleData     m_particle;
  VEE_RefPtr<GEE_Geometry>  m_particles;
};

#endif
