/* COPYRIGHT
 *
 * This file is part of the Visual Effects Engine - VEE
 *
 * Read the "VEE-LICENSE" file for the license.
 *
 * Authors & Copyright:   
 *
 * Tommi Ilmonen, Tuukka Heikura, Marko Myllymaa and 
 * Janne Kontkanen 2001-2004
 *
 * Additional copyrights: Tekes 2003-2004
 *
 * firstname.lastname@hut.fi
 *
 */

#ifndef VEE_GL_DISPLAY_LISTS_HPP
#define VEE_GL_DISPLAY_LISTS_HPP

#include <gee_renderer.h>

#include <vee_reference_object.h>

#include <GL/gl.h>

#include <map>
#include <set>

class VEE_GarbageBin;

class GEE_GlDisplayList
{
 public:
  GEE_GlDisplayList() : m_list(0), m_generation(-1) {}
      
  bool hasList() { return m_list != 0; }
  void freeList() 
    { if(m_list) glDeleteLists(m_list, 1); m_list = 0; m_generation = -1;}

  bool needsUpdate(int generation) 
    { return m_generation != generation; }
      
  GLuint listId()
    { if(!m_list) m_list = glGenLists(1); return m_list; }

  void setGeneration(long generation) { m_generation = generation; }

  long generation() const { return m_generation; }

 protected:

  GLuint m_list;
  long    m_generation;
};

class GEE_GlDisplayLists 
{
public:

  GEE_GlDisplayLists();
  ~GEE_GlDisplayLists();

  void freeLists(VEE_GarbageBin *items);

  GEE_GlDisplayList & getList(void *id);
  GEE_GlDisplayList * getListPtr(void *id);

  uint size() const { return m_storage.size(); }

protected:

  typedef std::map<void *, VEE_ReferenceObject<GEE_GlDisplayList> >::iterator iterator;

  std::map<void *, VEE_ReferenceObject<GEE_GlDisplayList> > m_storage;

};

#endif
