/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of the Visual Effects Engine - VEE
 *
 * Read the "VEE-LICENSE" file for the license.
 *
 * Authors & Copyright:   
 *
 * Tommi Ilmonen, Tuukka Heikura, Marko Myllymaa and 
 * Janne Kontkanen 2001-2004
 *
 * Additional copyrights: Tekes 2003-2004
 *
 * firstname.lastname@hut.fi
 *
 */

#ifndef GEE_GL_LIGHT_H
#define GEE_GL_LIGHT_H

#include <ConfigReaderEmpty.hpp>

#include <vee_vector4.h>

#include <GL/gl.h>

#include <map>
#include <vector>

/** OpenGL light source.


    @author Tommi Ilmonen
*/

class GEE_GlLight {
public:
  GEE_GlLight();
  
  void apply(GLenum id);
  
  void set(ConfigReader::Chunk *values);
  
  VEE_Vector4 m_location;
  VEE_Vector4 m_ambient;
  VEE_Vector4 m_diffuse;
  VEE_Vector4 m_specular;
};

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

/** A group of OpenGL lights. */

class GEE_GlLights 
{
public:
  GEE_GlLights();
  ~GEE_GlLights();

  void set(ConfigReader::Config *config);

  void clear();

  void apply(bool on, bool reset = true);

  void set(GLenum lightId, const GEE_GlLight & light);

  uint size() const { return m_lights.size(); }

protected:

  typedef std::map<GLenum, GEE_GlLight>::iterator iterator;

  std::map<GLenum, GEE_GlLight> m_lights;

  VEE_Vector4 m_sceneAmbient;  
};

#endif
