/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of the Visual Effects Engine - VEE
 *
 * Read the "VEE-LICENSE" file for the license.
 *
 * Authors & Copyright:   
 *
 * Tommi Ilmonen, Tuukka Heikura, Marko Myllymaa and 
 * Janne Kontkanen 2001-2004
 *
 * Additional copyrights: Tekes 2003-2004
 *
 * firstname.lastname@hut.fi
 *
 */

/** @file vee_cg_gl_renderer.h

    Cg/OpenGL rendering for project VEE.

*/

#ifndef VEE_CG_GL_RENDERER_H
#define VEE_CG_GL_RENDERER_H

#include <vee_gl_renderer.h>

#include <vee_reference_object.h>

#include <Cg/cgGL.h>

class VEE_CgGlRenderer;

enum VEE_CgParam {
  /// Camera location (Vector4)
  VEE_CGP_MODELVIEW_PROJECTION,
  /// Camera location (Vector4)
  VEE_CGP_CAMPOS,
  /// Age (float)
  VEE_CGP_AGE,
  /// Color (Vector4)
  VEE_CGP_COLOR,
  /// Color additivity (float)
  VEE_CGP_ADDITIVE,
  /// Center (Vector4)
  VEE_CGP_LOCATION,
  /// Velocity (Vector4)
  VEE_CGP_VELOCITY,
  /** Corner + uv (Vector4) */
  VEE_CGP_CORNER_UV,
  /** Corner + uv (Vector4) */
  VEE_CGP_SCREEN_SCALE,
  VEE_CGP_SIZEOF
};

extern const char * VEE_CgParamNames[];

/** Stores pointers to a series of Cg parameters for the shaders. */

class VEE_CgGlDriver
{
public:
  VEE_CgGlDriver();
  virtual ~VEE_CgGlDriver();

  bool createProgram(const char *program, 
		     CGcontext context, 
		     /// Vertex profile
		     CGprofile vp, 
		     /// Fragment profile
		     CGprofile fp);

  virtual bool bindProgram(VEE_CgGlRenderer *);

  virtual void renderParticle(VEE_Particle *, VEE_CgGlRenderer *);


  CGparameter m_params[VEE_CGP_SIZEOF];

  CGprogram m_vertexProgram;
  CGprogram m_fragmentProgram;

  std::string m_programName;
};

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

class VEE_CgGlRenderer : public VEE_GlRenderer
{
public:
  VEE_CgGlRenderer(VEE_SceneBox *box = 0);
  virtual ~VEE_CgGlRenderer();
  
  virtual void initializeGL();
  virtual void endGlMode();
  virtual void renderParticle(VEE_Particle *p);

  void disableCg();
  void enableCg();

  void loadShaders();

  static bool debug() { return m_debug; }
  
  CGprofile cgVertexProfile() { return m_cgVertexProfile; }
  CGprofile cgFragmentProfile() { return m_cgFragmentProfile; }

protected:

  CGcontext m_cgContext;
  CGprofile m_cgVertexProfile, m_cgFragmentProfile;

  std::map<int, VEE_SafePtr<VEE_CgGlDriver> > m_drivers;
  typedef std::map<int, VEE_SafePtr<VEE_CgGlDriver> >::iterator driver_iter;

  VEE_CgGlDriver * m_currentDriver;

  int         m_currentShader;

  static bool m_debug;
};

#endif
