/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of the Visual Effects Engine - VEE
 *
 * Read the "VEE-LICENSE" file for the license.
 *
 * Authors & Copyright:   
 *
 * Tommi Ilmonen, Tuukka Heikura, Marko Myllymaa and 
 * Janne Kontkanen 2001-2004
 *
 * Additional copyrights: Tekes 2003-2004
 *
 * firstname.lastname@hut.fi
 *
 */

#ifndef VEE_TEXTURE_H
#define VEE_TEXTURE_H

#include <GL/gl.h>

#include <map>
#include <string>

class VEE_Image;

/** OpenGL texture manager class. Each class instance can be used to
    hold information about a particular texture. */

class VEE_GlTexture2D
{
public:
  VEE_GlTexture2D();
  ~VEE_GlTexture2D();

  bool genTexture1D(const unsigned char *, unsigned n, 
		    GLenum format, bool mipmaps);
  bool genTexture(const VEE_Image *, bool mipmaps);

  void allocateId();
  void unallocateId();
  
  GLuint id() const { return m_glId; }

  bool isDefined() const { return m_glId != (GLuint) -1; }
  
  /// Forgets the texture id without freeing the texture.
  void forgetId() { m_glId = (GLuint) -1; }

  void use(GLenum target = GL_TEXTURE_2D) 
  { 
    if(!isDefined()) {
      glDisable(target);
      return; 
    }
    else {
      glEnable(target);
      glBindTexture(target, m_glId);
    }
  }

  void use1D() { use(GL_TEXTURE_1D); }
  void use2D() { use(GL_TEXTURE_2D); }
  void use3D() { use(GL_TEXTURE_3D); }

  void allocateArea(int minW, int minH, GLenum format);

  int width()  const { return m_width; }
  int height() const { return m_height; }

protected:

  /** ID of the OpenGL texture. */
  GLuint m_glId;

  int   m_width, m_height;
  
private:
  // VEE_GlTexture2D(const VEE_GlTexture2D & ) {}
  
};

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

class VEE_GlTexture2DBank
{
public:
  VEE_GlTexture2DBank();
  
  /// DO NOT CALL!
  VEE_GlTexture2DBank(const VEE_GlTexture2DBank &) {}

  ~VEE_GlTexture2DBank();

  /// DO NOT CALL!
  VEE_GlTexture2DBank & operator = (const VEE_GlTexture2DBank &)
  { return *this; }

  VEE_GlTexture2D *getTexture(const std::string &);
  VEE_GlTexture2D *buildTexture(const std::string &);

  void addTexture(const std::string &, const VEE_GlTexture2D &);

  void clear();

  uint size() { return m_textures.size(); }

  void loadTextures(const std::string & path,
		    const std::string & name,
		    const std::string & suffix = ".jpg",
		    int number = 20);

  bool loadTexture(const std::string & filename,
		   const std::string & name);
  
  typedef std::map<std::string, VEE_GlTexture2D>::iterator iterator;

  iterator textureBegin() { return m_textures.begin(); }
  iterator textureEnd()   { return m_textures.end(); }

protected:
  std::map<std::string, VEE_GlTexture2D> m_textures;
  std::map<std::string, std::string>     m_fileTextures;
};

#endif
