
# Usual initializations:

from vee_init import *
from vee import *
import math

qa = QApplication_newApp()
scene = VEE_SceneBox()
VEE_ObjectFactory_init()

vq = VEE_Qt(scene)
# Make force invisible:
# q2 = VEE_TestQt(vq, scene, "", 640, 480, vq)
# vq.setGlWidget(q2);

world = vq.getGlWidget().geeEngine().world();

agelimit = 5.0
gravity = 5.0

# object = "cat.lwo";
object = "/usr/share/Performer/data/cow.obj";
# object = "cupbase.lwo";
# object = "/usr/share/Performer/data/teapot.dxf";
objectscale = 1.0;

collider = VEE_OpcodeCollision()

collider.loadModel(object)

scene.addNewObject(collider)

group = GEE_Group();
group.thisown = 0
world.addData(group)
# g.loadModel("/usr/share/Performer/data/dumptruck.obj");
group.loadModel(object);
# g.loadModel("hakkyra.obj");

trans = VEE_Vector4()
trans.make(0.0, 20.0, 0.0, 1.0)
mat = VEE_Matrix4()
mat.identity()
mat.addToColumn(3, trans);

group.setTransformation(mat)


#############################################################################

# Build a particle with nice colors, lifespan etc:

# p = VEE_Particle()
p = VEE_TailParticle()
p.thisown = 0

# p.setAgeLimit(agelimit)

rgbatable = VEE_RgbaTable()
color = VEE_Rgba()

color.set(0.7, 0.0, 0.0, 1.0)
rgbatable.appendColor(color)
color.set(0.7, 0.7, 0.3, 1.0)
rgbatable.appendColor(color)
color.set(0.3, 0.3, 0.3, 1.0)
rgbatable.appendColor(color)
color.set(0.0, 0.0, 0.0, 1.0)
rgbatable.appendColor(color)

looks = VEE_LooksColorTable(rgbatable)
looks.thisown = 0

valueLooks = VEE_ValueLooks()
valueLooks.setLooks(looks)

p.setValue("looks", valueLooks)


#############################################################################

# Now for the flame creator
creator =  VEE_Emitter(p)
creator.thisown = 0;

r = p.setValue("looks", valueLooks)

v1 = VEE_Vector3()
v2 = VEE_Vector3()

range = 0.2

v1.make(-range, -0.5 * range , -range)
v2.make( range,  0.5 * range,  range)

rv = VEE_ValueRandomVector3()

rv.setRanges(v1, v2)

vo = VEE_ValueOperator()

vo.set("addvelocity", rv.clone())

creator.appendOperator(vo.clone())

vo.set("addlocation", rv.clone())
creator.appendOperator(vo.clone())

rf = VEE_ValueRandomFloat()
rf.set(0.5, 1.0)

vo.set("mass", rf.clone())
creator.appendOperator(vo.clone())

scene.addNewObject(creator)

#############################################################################

# Build a second particle with ugly colors, lifespan etc:

p2 = VEE_Particle()
p2.thisown = 0
p2.setAgeLimit(2);

# p.setAgeLimit(agelimit)

# rgbatable2 = VEE_RgbaTable()
# color2 = VEE_Rgba()

# color2.set(0.0, 0.0, 0.5, 1.0)
# rgbatable2.appendColor(color2)
#color2.set(0.2, 0.0, 0.7, 1.0)
#rgbatable2.appendColor(color2)
#color2.set(0.3, 0.0, 0.3, 1.0)
#rgbatable2.appendColor(color2)
#color2.set(0.0, 0.0, 0.0, 1.0)
#rgbatable2.appendColor(color2)

looks2 = VEE_LooksColorTable(rgbatable)
looks2.thisown = 0

valueLooks2 = VEE_ValueLooks()
valueLooks2.setLooks(looks2)

p2.setValue("looks", valueLooks2)



#############################################################################

# vortex = VEE_ObjectFactory_create("vortex")
# scene.addNewObject(vortex)
# vortex.setAgeLimit(2000.0)

#############################################################################

# Create a vortex generator:

vortex = VEE_ObjectFactory_create("vortex")
vortex.setAgeLimit(1.5);
vortex.setValueFloat("range", 1.0);
vortex.setValueFloat("magnitude", 0.5);
vortex.setVelocityUpdate(1)

screator = VEE_Emitter(vortex)
screator.thisown = 0;
screator.setValueInt("visible", 0);

# Random direction:
vo.set("axes", rv.clone())
screator.appendOperator(vo.clone())
# Randomize location:
vo.set("addlocation", rv.clone())
screator.appendOperator(vo.clone())
screator.setRate(1.0);

scene.addNewObject(screator)


#############################################################################

# Create a few forces:

g = VEE_ObjectFactory_create("gravity")
scene.addNewObject(g)
g.setValueInt("visible", 0);

damp = VEE_ObjectFactory_create("damping")
scene.addNewObject(damp)
damp.setValueInt("visible", 0);


#grav = VEE_ObjectFactory_create("gravitate")
#grav.setValueFloat("range", 2.0);
#grav.setValueFloat("magnitude", -200); # Push away from object
#grav.setValueFloat("maxforce", 10);
#grav.setLocation3(1.0, 4.0, 0.0);
#scene.addNewObject(grav)

# scene.addNewObject(VEE_ObjectFactory_create("blow"))

#############################################################################

# Make GUI mappings for a couple a few parameters:

paramView = VEE_ValueWindow()

paramView.addFloatParameter("Gravity", "gravity", g, -5, -20.0, 20.0,
                            VEE_MAP_LINEAR);

paramView.addFloatParameter("Grid bounciness", "bounce", collider,
                            0.5, 0.0, 1.0,
                            VEE_MAP_LINEAR);

paramView.addFloatParameter("Grid drag", "drag", collider,
                            0.0, 0.0, 1.0,
                            VEE_MAP_LINEAR);

paramView.addFloatParameter("Flame agelimit", "agelimit", p, 8, .5, 10.0,
                            VEE_MAP_LINEAR);

paramView.addFloatParameter("Flame tail", "tailsize", p, 20, 4, 200,
                            VEE_MAP_LOGARITHMIC);

paramView.addFloatParameter("Flame tail thresh", "threshold", p, 0.2, 0.0, 2.0,
                            VEE_MAP_LINEAR);

paramView.addFloatParameter("Flame rate", "rate", creator, 50.0, 0.0, 200.0,
                            VEE_MAP_LINEAR);

paramView.addVector3Parameter("Flame location", "location", creator,
                              1.0, -7.0, 7.0,
                              3.0, -3.0, 7.0,
                              0.0, -7.0, 7.0);

paramView.addFloatParameter("Vortex rate", "rate", screator, 0.0, 0.0, 3.0,
                            VEE_MAP_LINEAR);

paramView.addFloatParameter("Vortex range", "range", vortex, 2.0, 0.1, 20.0,
                            VEE_MAP_LOGARITHMIC);

paramView.addFloatParameter("Vortex magnitude", "magnitude", vortex, 3.0, 0.1, 20.0,
                            VEE_MAP_LOGARITHMIC);

paramView.addFloatParameter("Vortex maxforce", "maxforce", vortex, 10.0, 0.1, 20.0,
                            VEE_MAP_LOGARITHMIC);

paramView.addFloatParameter("Damping", "magnitude", damp, 0.5, -2, 6,
                            VEE_MAP_LINEAR);

#############################################################################

# Make it visible:

# q2.setMtlFile("vee_materials.mtl");

vq.toggleForceMode();
qa.setMainWidget(vq)
vq.show()
# vq.setRenderOrder("1");
vq.setValueFloat("GL/size", 0.04);
vq.setValueInt("GL/subtexture", 5);

vq.setFrameFile("frames/cow-%.5d.jpg")

kp = vq.addKeyPoint2(-0.000, 1.090, 3.762, 3.142, -0.300);
vq.selectKeyPoint2(kp);

paramView.bangValues()

paramView.show()

qa.execute()

