%module(directors="1") vee
%feature("director") VEE_HasValues;
%feature("director") VEE_Operator;

%feature("director") VEE_Data;
%feature("director") VEE_Group;

%feature("director") GEE_Action;

%{

/* COPYRIGHT
 *
 * This file is part of the Visual Effects Engine - VEE
 *
 * Read the "VEE-LICENSE" file for the license.
 *
 * Authors & Copyright:   
 *
 * Tommi Ilmonen, Tuukka Heikura, Marko Myllymaa and 
 * Janne Kontkanen 2001-2003
 *
 * firstname.lastname@hut.fi
 *
 */

#include <Opcode.h>

#include <vee_color.h>
#include <vee_curve_particle.h>
#include <vee_emitter.h>
#include <vee_force_object.h>
#include <vee_gl_renderer.h>
#include <vee_grid_force.h>
#include <vee_image.h>
#include <vee_looks_color_table.h>
#include <vee_math.h>
#include <vee_object_factory.h>
#include <vee_operator_dt.h>
#include <vee_operator_force_gradient.h>
#include <vee_operator_list.h>
#include <vee_operator_math.h>
#include <vee_operator_object.h>
#include <vee_operator_random.h>
#include <vee_operator_texture_loop.h>
#include <vee_parameters.h>
#include <vee_polygon.h>
#include <vee_qt.h>
#include <vee_rand2d.h>
#include <vee_rate_control.h>
#include <vee_render_class.h>
#include <vee_rigid_particle.h>
#include <vee_scene.h>
#include <vee_tail_particle.h>
#include <vee_test_qt.h>
#include <vee_volume_box.h>
#include <vee_volume_grid.h>
#include <vee_values.h>
#include <vee_value_curve.h>
#include <vee_value_gui.h>
#include <vee_value_gui_float.h>
#include <vee_value_gui_vector3.h>
#include <vee_value_random.h>
#include <vee_value_operator.h>
#include <vee_volume_sphere.h>


#include <gee_operator.h>
#include <gee_geometry.h>
#include <gee_debug.h>
#include <gee_engine.h>
#include <gee_erase_around_path.h>
#include <gee_color_around_path.h>
#include <gee_copy_geometry.h>
#include <gee_move_around_path.h>
#include <gee_scale_around_path.h>
#include <gee_gl_light.h>
#include <gee_gl_material.h>
#include <gee_group.h>
#include <gee_id_select.h>
#include <gee_io.h>
#include <gee_lines.h>
#include <gee_mover.h>
#include <gee_time_stamp.h>
#include <gee_triangle_mesh.h>
#include <gee_particle_rotate.h>
#include <gee_path.h>
#include <gee_path_insert.h>
#include <gee_path_multiply.h>
#include <gee_path_to_lines.h>
#include <gee_path_to_mesh.h>
#include <gee_path_to_particles.h>
#include <gee_sphere.h>
#include <gee_solar_conversion.h>
#include <gee_texture_around_path.h>

#include <GL/gl.h>

#include <qapplication.h> 
#include <qimage.h> 

  class VEE_Looks;
  class VEE_Particle;

#include <Solar/Loaders.hpp>
#include <Solar/LwoConvert.hpp>
#include <Solar/Model.hpp>
#include <Solar/Object.hpp>
  
  // Needs to be after Qt has been included (!)
#include <vee_opcode_collision.h>

  typedef Solar::Model  Solar_Model;
  typedef Solar::Object Solar_Object;
  typedef Solar::Loaders Solar_Loaders;
  typedef Solar::LwoConvert Solar_LwoConvert;

  using namespace Fluid;

%}

#define VEE_PI         3.1415926535897931
#define VEE_TWO_PI     6.2831853071795862
#define VEE_HALF_PI    1.57079632679489660
#define VEE_QUARTER_PI 0.78539816339744828

typedef int uint;
typedef long long llong;

/* enum VEE_ForceType {
  VEE_FORCE_GRAVITY,
  VEE_FORCE_DRAG,
  VEE_FORCE_HIDDEN,
  VEE_FORCE_SIZEOF
  };*/

// %include "vee_object.h"

%include "std_string.i"

%include "di_io_defs.h"


%include "Fluid/Vector2.hpp"
%include "Fluid/Vector3.hpp"
%include "Fluid/Vector4.hpp"
%include "Fluid/Matrix3.hpp"
%include "Fluid/Matrix4.hpp"
%include "Fluid/TimeStamp.hpp"

%include "vee_vector2.h"
%include "vee_vector3.h"
%include "vee_vector4.h"
%include "vee_matrix3.h"
%include "vee_matrix4.h"
%include "gee_time_stamp.h"

%template(VEE_Vector2)  Fluid::Vector2T<float>;
%template(VEE_Vector2d) Fluid::Vector2T<double>;
%template(VEE_Vector3)  Fluid::Vector3T<float>;
%template(VEE_Vector3d) Fluid::Vector3T<double>;
%template(VEE_Vector4)  Fluid::Vector4T<float>;
%template(VEE_Vector4d) Fluid::Vector4T<double>;

%template(VEE_Matrix3)  Fluid::Matrix3T<float>;
%template(VEE_Matrix3d) Fluid::Matrix3T<double>;

%template(VEE_Matrix4)  Fluid::Matrix4T<float>;
%template(VEE_Matrix4d) Fluid::Matrix4T<double>;

%rename(__add__) VEE_Vector3::operator+;
%rename(__sub__) VEE_Vector3::operator-;
%rename(__neg__) VEE_Vector3::operator-();

float dot(VEE_Vector3 , VEE_Vector3);
VEE_Vector3 cross(VEE_Vector3 , VEE_Vector3);

%include "vee_config.h"
%include "vee_garbage.h"
%include "vee_has_values.h"
%include "vee_has_variables.h"
%include "vee_bounding_box.h"
%include "vee_color.h"
%include "vee_render_class.h"

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

enum VEE_VariableType {
  VEE_VAR_INVALID,
  VEE_VAR_INT,
  VEE_VAR_INT_PTR,
  VEE_VAR_FLOAT,
  VEE_VAR_FLOAT_PTR,
  VEE_VAR_RGBA,
  VEE_VAR_VECTOR3,
  VEE_VAR_VECTOR3_PTR,
  VEE_VAR_MATRIX3,
  VEE_VAR_SIZEOF
};

class VEE_Variable
{
public:
  VEE_Variable();
  ~VEE_Variable();

  bool isNumber() const ;
  VEE_VariableType type();

  inline float getFloat(float def) const;
  inline int   getInt(int def) const;

  const VEE_Rgba & getRgba(const VEE_Rgba &def);
  const VEE_Vector3 & getVector3(const VEE_Vector3 &def);
  const VEE_Matrix3 & getMatrix3(const VEE_Matrix3 &def);

  %extend {
    float getFloatX() const
      { return self->getFloat(); }
    int   getIntX() const
      { return self->getInt(); }

    const VEE_Rgba & getRgbaX() { return self->getRgba(); }
    const VEE_Vector3 & getVector3X() { return self->getVector3(); }
    const VEE_Matrix3 & getMatrix3X() { return self->getMatrix3(); }
  }

  void setFloat(float v);
  void setInt(int v);
  void setRgba(const VEE_Rgba &v);
  void setVector3(const VEE_Vector3 &v);
  void setMatrix3(const VEE_Matrix3 &v);
};

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

class VEE_Value : public VEE_HasValues
{
public:

  VEE_Value() {}
  virtual ~VEE_Value() {}

  virtual VEE_Value * clone() const;

  virtual bool getFloat(float *);
  virtual bool getInt(int *);
  virtual bool getVector3(VEE_Vector3 *);
  virtual bool getMatrix3(VEE_Matrix3 *);  

};

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////


/// Value that carries one floating point number.
class VEE_ValueFloat : public VEE_Value
{
public:
  VEE_ValueFloat() {}
  virtual ~VEE_ValueFloat() {}

  // virtual bool getFloat(float *);
  // virtual bool getInt(int *);

  virtual void setValue2(float v);
  float getValue() const;

  %extend {
    static VEE_ValueFloat *cast(VEE_Value *v)
      { return dynamic_cast<VEE_ValueFloat *> (v); }
  }

};

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

class VEE_ValueVector3 : public VEE_Value
{
public:
  VEE_ValueVector3(const VEE_Vector3 &v = VEE_Vector3ZERO) : m_val(v) {}
  virtual ~VEE_ValueVector3() {}

  void setValue2(const VEE_Vector3 &v) { m_val = v; }
  VEE_Vector3  getValue() const { return m_val; }
  
  %extend {
    static VEE_ValueVector3 *cast(VEE_Value *v)
      { return dynamic_cast<VEE_ValueVector3 *> (v); }

    void setValue3(float x,float y, float z)
      { self->setValue2(VEE_Vector3(x, y, z)); }
  }

protected:
  VEE_Vector3 m_val;
};

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

class VEE_ValueMatrix3 : public VEE_Value
{
public:
  VEE_ValueMatrix3(const VEE_Matrix3 &v = VEE_Matrix3IDENTITY) : m_val(v) {}
  virtual ~VEE_ValueMatrix3() {}

  void setValue2(const VEE_Matrix3 &v) { m_val = v; }
  const VEE_Matrix3  getValue() const { return m_val; }

  %extend {
    static VEE_ValueMatrix3 *cast(VEE_Value *v)
      { return dynamic_cast<VEE_ValueMatrix3 *> (v); }
  }

protected:
  VEE_Matrix3 m_val;
};

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

class VEE_ValueRandomVector3 : public VEE_Value
{
public:
  VEE_ValueRandomVector3();
  virtual ~VEE_ValueRandomVector3();

  void setRanges(const VEE_Vector3 &minVect, const VEE_Vector3 &maxVect);
  void setRotation(const VEE_Matrix3 &rot);

  %extend {
    static VEE_ValueRandomVector3 *cast(VEE_Value *v)
      { return dynamic_cast<VEE_ValueRandomVector3 *> (v); }
  }

};

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

/// Random rotation matrix producer.
class VEE_ValueRandomRotation : public VEE_Value
{
public:
  VEE_ValueRandomRotation();
  virtual ~VEE_ValueRandomRotation();
  
  void setAxes(const VEE_Vector3 &axes);
  void setMaxRotation(float v);

  %extend {
    static VEE_ValueRandomRotation *cast(VEE_Value *v)
      { return dynamic_cast<VEE_ValueRandomRotation *> (v); }
  }
};

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

class VEE_ValueRandomFloat : public VEE_Value
{
public:
  VEE_ValueRandomFloat();
  virtual ~VEE_ValueRandomFloat();
  
  void set(float fmin, float fmax);

  %extend {
    static VEE_ValueRandomFloat *cast(VEE_Value *v)
      { return dynamic_cast<VEE_ValueRandomFloat *> (v); }
  }
};

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

class VEE_ValueLooks : public VEE_Value
{
public:
  VEE_ValueLooks();
  virtual ~VEE_ValueLooks();

  VEE_Looks *looks();
  
  void setLooks(VEE_Looks *);
};


/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

enum VEE_ForceType {
  VEE_FORCE_GRAVITY,
  VEE_FORCE_DRAG,
  VEE_FORCE_HIDDEN,
  VEE_FORCE_SIZEOF
};

class VEE_Object : public VEE_HasValues, public VEE_HasVariables
{
public:

  virtual ~VEE_Object();

  virtual VEE_Object * clone() const = 0;

  const VEE_Rgba & color() const { return m_color; }
  void setColor(const VEE_Rgba & c) { m_color = c; }

  void setLocation(const VEE_Vector3 &l);
  void setLocation3(float x, float y, float z);
  void setVelocity(const VEE_Vector3 &v);
  void setVelocity3(float x, float y, float z);

  VEE_Vector3  location() { return m_location; }
  VEE_Vector3  velocity() { return m_velocity; }
  
  virtual void update(float);
  virtual void updateVelocity(float);

  void setVelocityUpdate(bool p);
  
  float age();
  void setAge(float age);

  float ageLimit();
  void setAgeLimit(float ageLimit);
 
  void setMass(float m);
  void setArea(float a);
  float mass();
  float area();

  int  texture() const { return m_texture; }
  void setTexture(int id) { m_texture = id; }

  float size() const { return m_size; }
  void setSize(float s) { m_size = s; }

  float rotation() const { return m_rotationRadians; }
  void setRotation(float radians) { m_rotationRadians = radians; }

  float omega() const { return m_omega; }
  void setOmega(float omega) { m_omega = omega; }

  bool done();

  const char * className() const;
  
  void setRenderClass(VEE_RenderClass *renderClass) ;
};

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

class VEE_ObjectFactory
{
public:
  static VEE_Object * create(const char *type);
  static void init();

  /* %extend {
    static void begin() { VEE_ObjectFactory::init(); }
    }*/
};

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

%include "vee_looks.h"
%include "vee_looks_color_table.h"
%include "vee_color_table.h"

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

class VEE_Particle : public VEE_Object
{
public:

  VEE_Particle();
  virtual ~VEE_Particle();

  virtual VEE_Object *clone() const;

  void setLooks(VEE_Looks *l);
  VEE_Looks *looks();
  
  void setVisible(bool visible);
  bool visible() const;

  %extend {
    static VEE_Particle *cast(VEE_Object *v)
      { return dynamic_cast<VEE_Particle *> (v); }
  }
};

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

class VEE_Particle2 : public VEE_Particle
{
public:

  VEE_Particle2();
  virtual ~VEE_Particle2();

  void appendSelfOperator(VEE_Operator *o);
  void appendSelfOperator2(VEE_Operator *o, const char * name);

  %extend {
    static VEE_Particle2 *cast(VEE_Object *v)
      { return dynamic_cast<VEE_Particle2 *> (v); }
  }
};

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

class VEE_RigidParticle : public VEE_Particle2
{
public:
  VEE_RigidParticle();
  virtual ~VEE_RigidParticle();
};

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

class VEE_TailParticle : public VEE_Particle
{
public:
  VEE_TailParticle();
  virtual ~VEE_TailParticle();
  void resizeTail(int);
};

class VEE_TailParticle2 : public VEE_TailParticle
{
 public:
  
  VEE_TailParticle2();
  ~VEE_TailParticle2();
};

class VEE_TailParticle3 : public VEE_TailParticle
{
 public:
  
  VEE_TailParticle3();
  ~VEE_TailParticle3();
};
/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

class VEE_CurveParticle : public VEE_Particle
{
public:
  VEE_CurveParticle();
  virtual ~VEE_CurveParticle();
  void resizeCurve(int);
};

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

class VEE_OperatorObject : public VEE_Particle
{
public:
  VEE_OperatorObject() {}
  virtual ~VEE_OperatorObject();
  
  void appendOperator(VEE_Object *target, VEE_Operator *o);
  void appendOperator2(VEE_Object *target, VEE_Operator *o, const char * name);
};
  
/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

class VEE_Polygon : public VEE_Particle
{
public:
  VEE_Polygon();
  virtual ~VEE_Polygon();

  static VEE_Polygon * createNewMesh(int horizontalDivisions,
				     int verticalDivisions,
				     float width,
				     float height,
				     float ageLimit);

  static VEE_Polygon * createNewQuadMesh(const uint horizontalDivisions,
					 const uint verticalDivisions,
					 float width,
					 float height,
					 float ageLimit);
  
  void move(const VEE_Vector3 &);
  virtual void setStiffness(float s);
  int triangleCount() const;

  void setMaterial(int m);
  int getMaterial();

  %extend {
    static VEE_Polygon *cast(VEE_Object *v)
      { return dynamic_cast<VEE_Polygon *> (v); }
  }
};


/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

class VEE_Emitter : public VEE_Particle2
{
public:
  
  VEE_Emitter(VEE_Object *type);
  virtual ~VEE_Emitter();
  
  VEE_OperatorList & operators();
  void appendOperator(VEE_Operator *o);
  void appendOperator2(VEE_Operator *o, const char * name);

  void setRate(float hz);
  void setInheritVelocity(bool inherit);

  void setType(VEE_Object *o);

  %extend {
    static VEE_Emitter *cast(VEE_Object *v)
      { return dynamic_cast<VEE_Emitter *> (v); }
  }
};

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

class VEE_ForceObject : public VEE_Particle2
{
public:
  
  %extend {
    static VEE_ForceObject *cast(VEE_Object *v)
      { return dynamic_cast<VEE_ForceObject *> (v); }
  }
protected:
  VEE_ForceObject();
  virtual ~VEE_ForceObject();
};

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

%include "vee_grid_force.h"

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

%include "vee_opcode_collision.h"

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

class VEE_Operator : public VEE_HasValues
{
public:
  VEE_Operator() {}
  virtual ~VEE_Operator() {}
  virtual void operate(VEE_Object *, float);
  virtual VEE_Operator * clone() const;
};

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

class VEE_ValueOperator : public VEE_Operator
{
public:
  VEE_ValueOperator();
  virtual ~VEE_ValueOperator();

  void set(const char *, VEE_Value *);
};

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////


class VEE_OperatorDt : public VEE_Operator
{
public:
  VEE_OperatorDt();
  virtual ~VEE_OperatorDt();

  void set(float addDt, bool moveBack, bool allowAgeChange);
  void setMaxStep(float step);
};

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

/* 
%{
  static void VEE_PythonOperatorCallback(VEE_Object *, float, void *)
    {   
      PyObject *func, *arglist;
      PyObject *result;
      func = (PyObject *) clientdata;
    }
%}

class VEE_OperatorCallback : public VEE_Operator
{
public:
  VEE_OperatorCallback();
  virtual ~VEE_OperatorCallback();
  
  %extend {
    void setPythonCallback(PyObject *pyfunc) {
      self->setCallback();
      Py_INCREF(pyfunc);
    }
  }
};
*/

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

class VEE_RateControl : public VEE_Operator
{
public:
  VEE_RateControl();
  virtual ~VEE_RateControl();

  void defaults();

  virtual void operate(VEE_Object *, float);
  virtual VEE_Operator * clone() const;
  void setId(const char *id);
  void setRadius(float r);
  void setHeat(float h);
  void setHeatCoeff(float c);
  void setRequireHeat(float r);
  void setMaxRate(float r);
  void dieAfter(float d);
};


/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////


class VEE_ValueCurve : public VEE_Operator
{
public:
  VEE_ValueCurve();
  virtual ~VEE_ValueCurve();

  void setId(const char *id);

  void appendPoint(float v, float t);
  
  void clearCurve();
};

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

enum VEE_TimeScale {
  VEE_TS_LINEAR,
  VEE_TS_AGE_MATCH,
  VEE_TS_PERIODIC,
  VEE_TS_SIZEOF
};

class VEE_ValueCurve2 : public VEE_Operator
{
public:
  VEE_ValueCurve2();
  virtual ~VEE_ValueCurve2();

  void setId(const char *id);

  void appendPoint(float v, float t);
  
  void clearCurve();
  void setScaling(int scaleMethod);
  void scaleValues(float scale);
  void scaleTimes(float scale);
};

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

class VEE_ValueCurveMulti : public VEE_Operator
{
public:
  VEE_ValueCurveMulti();
  virtual ~VEE_ValueCurveMulti();

  void resize(int curves);

  void setId(const char *name, int n);

  void appendPoint(float v, float t, int n);
  
  void clearCurves();
};

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

class VEE_OperatorForceGradient : public VEE_Operator
{
public:
  VEE_OperatorForceGradient();
  virtual ~VEE_OperatorForceGradient();

  void resize(int curves);

  void setId(const char *name, int n);

  void appendPoint(float v, float t, int n);

  void clearCurves();
};

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

class VEE_RandOnMesh2D : public VEE_Operator
{
public:
  VEE_RandOnMesh2D();
  virtual ~VEE_RandOnMesh2D();

  void resize(int rows, int cols);
  void clear();
  void setDecay(float d);
  void setThreshold(float t);

  void setId(const char *id);

  VEE_Vector3 newVector();
};

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

class VEE_OperatorRandomSphereVector : public VEE_Operator
{
public:
  VEE_OperatorRandomSphereVector();
  virtual ~VEE_OperatorRandomSphereVector();

  void setId(const char *);  
  void setRadius(float r);
};

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

enum VEE_MathOperation {
  VEE_MATH_NONE,
  VEE_MATH_ADD,
  VEE_MATH_SUBTRACT,
  VEE_MATH_MULTIPLY,
  VEE_MATH_DIVIDE,
  VEE_MATH_SIZEOF
};

class VEE_OperatorRandomFloat : public VEE_Operator
{
public:
  VEE_OperatorRandomFloat();
  virtual ~VEE_OperatorRandomFloat();

  void setId(const char * id) { m_id = id; }
  void setRadius(float r) { m_radius = r; }
  void setOperation(VEE_MathOperation op) { m_operation = op; }
};

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

class VEE_OperatorMathFloat : public VEE_Operator
{
public:
  VEE_OperatorMathFloat();
  ~VEE_OperatorMathFloat();

  void set(float) ;
  void setOperation(VEE_MathOperation op);
  void setId(const char * id) { m_id = id; }
};

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

class VEE_OperatorList
{
public:
  VEE_OperatorList();
  void appendOperator(VEE_Operator *o);
  void operate(VEE_Object *, float);
  void clear();
};

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

class VEE_OperatorTextureLoop : public VEE_Operator
{
public:
  VEE_OperatorTextureLoop();
  virtual ~VEE_OperatorTextureLoop();

  void addTexture(int t);
  void setTimeScale(VEE_TimeScale scale);
  void setLoopTime(float time);
};


/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

class VEE_MetaBalls
{
public:
  VEE_MetaBalls();
  ~VEE_MetaBalls();

  void addBall(const VEE_Vector3 & center, float radius);
  float presence(const VEE_Vector3 &loc, float rfull);
  void clear();

};


/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

class VEE_SceneBox
{
public:

  VEE_SceneBox();

  virtual ~VEE_SceneBox();

  void setArea(const VEE_Vector3 &low,
	       const VEE_Vector3 &high);

  bool addNewObject(VEE_Object *);
  bool addNewForce(VEE_ForceObject *);

  int totalChildren();
  int totalForces();

  void update(int splitTreshold, 
	      int collapseTreshold, 
	      int allowRecursionLevels,
	      float dt);

  bool isSplit() const;
  uint numChildBoxes();
};

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

class VEE_GlRenderer
{
public:
  VEE_GlRenderer();
  virtual ~VEE_GlRenderer();

  int subTextureCount() const ;
  int currentSubTexture() const;
  
  void setCameraPosition(const VEE_Vector3 &pos);
  void setCameraPosition3f(float x, float y, float z);
  void setCameraAngles(float azi, float ele);
  int  textureNameToId(const char *name);
  int  shaderNameToId(const char *name);
  void setRange(float minDist, float fadeDist, float maxDist);
};

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

%include "gee_config.h"
%include "gee_id.h"
%include "gee_bounded.h"
%include "gee_containment.h"
%include "gee_sphere_group.h"
%include "gee_debug.h"
%include "gee_enums.h"
%include "gee_operator.h"
%include "gee_material.h"
%include "gee_geometry.h"
%include "gee_engine.h"
%include "gee_gallery.h"
%include "gee_gl_light.h"
%include "gee_gl_material.h"
%include "gee_group.h"
%include "gee_interval.h"
%include "gee_io.h"
%include "gee_lines.h"
%include "gee_mover.h"
%include "gee_operator.h"
%include "gee_path.h"
%include "gee_path_to_x.h"
%include "gee_recolor.h"
%include "gee_id_select.h"
%include "gee_copy_geometry.h"
%include "gee_color_around_path.h"
%include "gee_erase_around_path.h"
%include "gee_move_around_path.h"
%include "gee_scale_around_path.h"
%include "gee_texture_around_path.h"
%include "gee_path_insert.h"
%include "gee_path_multiply.h"
%include "gee_path_to_lines.h"
%include "gee_path_to_mesh.h"
%include "gee_path_to_particles.h"
%include "gee_particle_rotate.h"
%include "gee_profile.h"
%include "gee_sphere.h"
%include "gee_triangle_mesh.h"
%include "gee_world.h"

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

class QImage
{
 public:
  
  int width () const;
  int height () const;
  
  int depth () const;
  int numColors () const;

  QImage convertDepth ( int depth ) const;
  QImage copy ( int x, int y, int w, int h);

  int load ( const char * fileName);

};

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

class QWidget
{
public:
  QWidget();
  virtual ~QWidget();

  void show();
  void resize(int, int);
  void move(int, int);

  %extend {
    void raiseWidget() { self->raise(); }
  }
};

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

class QApplication
{
public:
  QApplication(int & argc, char ** argv);
  ~QApplication();

  void setMainWidget(QWidget *);
  void processEvents ( int maxtime );
  void processOneEvent ();
  bool hasPendingEvents ();
  
  %extend {
    // QApplication() { int argc = 0; const char **argv = 0; 
    static QApplication * newApp()
      { 
	int argc = 0; 
	char * tmp = "";
	char **argv = & tmp; 
	return new QApplication(argc, argv); 
      }
    
    int execute() { return self->exec();}
  }
};

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

/* %include "vee_qt.h" */

class VEE_Qt;
class VEE_ValueWindow;

enum VEE_GlRendererType {
  /// VEE_GlRenderer
  VEE_GL_PLAIN,
  /// VEE_GlSlRenderer
  VEE_GL_SL,
  /// VEE_CgGlRenderer
  VEE_GL_CG
};

class VEE_QtHidden
{
 public:
  VEE_QtHidden(VEE_Qt * host,
	       VEE_SceneBox *box);

  virtual ~VEE_QtHidden();  
  GEE_Engine * geeEngine();
  GEE_GlMaterialBank * geeMaterials();
};

class VEE_Qt : public QWidget , public VEE_HasValues
{
public:
  VEE_Qt(VEE_SceneBox *box, 
	 VEE_GlRendererType type = VEE_GL_SL,
	 QWidget *parent=0, 
	 const char *name=0 , 
	 bool createGlWidget = true);
  virtual ~VEE_Qt();

  VEE_GlRenderer *renderer();
  void setGlWidget(VEE_QtHidden *);
  VEE_QtHidden * getGlWidget();

  void setFrameFile(const char *);
  void setScreenShotCount(int);
  void setScreenShotOverSample(int);
  void setValueWindow(VEE_ValueWindow *vw) { m_valueWindow = vw; }
  
  static void initImageIO();

  void setFrameRate(float rate);
  void setTreeThresholds(int, int);

  void resizeGlWidget(int w, int h);
  int addKeyPoint2(float x, float y, float z, float azi, float ele);

  void toggleNObj();
  void togglePause();
  void toggleVisualUpdate();
  void toggleCalculusUpdate();
  void toggleShowBoxes();
  void toggleBlend();
  void togglePolygonMode();
  void toggleForceMode();
  void toggleForceBoxMode();

  void useQuadTextures();
  void useTriTextures();
  void usePoints();
  void useLines();
  void useNothing();

  void paramsVisible();

  void nofade();
  void fadeLittle();
  void fadeMore();
  void fadeExtreme();

  void white();
  void black();
  void gray();

  void additive();
  void subtractive();
  void usual();

  void setRenderOrder(const char *);

  void bgColorDialog();
  void selectKeyPoint(const char *);
  void selectKeyPoint2(int);

};

class VEE_TestQt : public VEE_QtHidden
{
public:
  VEE_TestQt(VEE_Qt * host,
	     VEE_SceneBox *box, 
	     const char *rendermanFile = "",
	     uint width = 640, 
	     uint height = 480,
	     VEE_GlRendererType type = VEE_GL_SL,
	     QWidget *parent=0, 
	     const char *name=0);
  virtual ~VEE_TestQt();

  void deleteSolarObjectsAndModels();
  
  void addModel(Solar_Model *model) { m_models.push_back(model); }
  void addObject(Solar_Object *object) { m_objects.push_back(object); }
  
  Solar::GlMaterialBank & materials() { return m_materials; }
  
  void setMtlFile(const std::string &f) { m_mtlFile = f; }

  static void loadMaterials(const std::string     &mtlFile, 
			    VEE_GlTexture2DBank   *textures,
			    Solar::GlMaterialBank *materials);
};

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

class VEE_Parameter
{
public:
  VEE_Parameter() {}
  virtual ~VEE_Parameter() {}

  virtual VEE_Parameter * clone() const;
  virtual bool setValue(VEE_HasValues *);

  %extend {

    const char * name() const { return self->name().c_str(); }
    void setName(const char * name) { self->setName(name); }
  }
};

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

enum VEE_Mapping {
  VEE_MAP_LINEAR = 0,
  VEE_MAP_LOGARITHMIC = 1
};


class VEE_ParameterFloat : public VEE_Parameter
{
public:
  VEE_ParameterFloat() : m_value(0.0), m_min(0.0), m_max(1.0) {}
  virtual ~VEE_ParameterFloat() {}
  
  void set(float value, float mi, float ma);

  void setFloat(float v);
    
  float value() const;
  float minValue()   const;
  float maxValue()   const;
  
  void setMapping(VEE_Mapping m) { m_mapping = m; }
  VEE_Mapping mapping() const { return m_mapping; }

};

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////


class VEE_Volume : public VEE_HasValues
{
public:
  VEE_Volume();
  virtual ~VEE_Volume();
  
  virtual float inside(const VEE_Vector3 &);
  virtual void getBoundingBox(VEE_BoundingBox *);
  virtual VEE_Vector3 newRandomPoint();

  virtual VEE_Volume * clone() const;
};

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

class VEE_VolumeSphere : public VEE_Volume
{
public:
  VEE_VolumeSphere();

  virtual ~VEE_VolumeSphere();

  virtual float inside(const VEE_Vector3 &);
  virtual void getBoundingBox(VEE_BoundingBox *);

  virtual VEE_Volume * clone() const;

  void set(float inner, float outer);
};

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

class VEE_VolumeBox : public VEE_Volume
{
public:
  VEE_VolumeBox();
  virtual ~VEE_VolumeBox();
  
  void setAxes(float a1, float a2, float a3);
  void setRotation(VEE_Matrix3 &rot) { m_rotation = rot; }
  
};

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

class VEE_ValueVolume : public VEE_Value
{
public:
  VEE_ValueVolume();
  virtual ~VEE_ValueVolume();

  VEE_Volume *object();
  
  void setObject(VEE_Volume *);
};


/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

class VEE_VolumeGrid2 : public VEE_Volume
{
public:

  VEE_VolumeGrid2();
  virtual ~VEE_VolumeGrid2();
  
  void setVoxelSize(float s);
  float voxelSize() const;

  void addBall(const VEE_Vector3 &loc, float radius);
  void triangulate();
  int triangleCount() const;
  int byteCount() const;
  int gridCount() const;
  

};
  

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

class VEE_ParamTargets
{
public:
  VEE_ParamTargets();
  ~VEE_ParamTargets();

  void clear();
  void append(VEE_HasValues *target, const char *id);
  void appendMany(VEE_ParamTargets *pt);
  int size() const ;
};

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

class VEE_ValueGui
{
public:
  VEE_ValueGui(QObject * parent = 0,
	       const char * name = 0);
  
  virtual ~VEE_ValueGui();

  virtual void bangValue();

  void addTarget(VEE_HasValues *target, const char * id);
  void addTargets(VEE_ParamTargets *pt);
};

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

class VEE_ValueGuiFloat : public VEE_ValueGui
{
public:
  VEE_ValueGuiFloat();
  virtual ~VEE_ValueGuiFloat();

  void setValue(double);
  void getValue(double *);
};

class VEE_ValueGuiVector3 : public VEE_ValueGui
{
public:
  VEE_ValueGuiVector3();
  virtual ~VEE_ValueGuiVector3();

};

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

/// Window for displaying values
class VEE_ValueWindow : public QWidget
{
public:
  VEE_ValueWindow();
  virtual ~VEE_ValueWindow();
  VEE_ValueGui * addParameter(VEE_Parameter *, VEE_HasValues *);
  
  VEE_ValueGuiFloat * addFloatParameter(const char *, const char *, 
					VEE_HasValues *,
					float, float, float,
					VEE_Mapping);

  VEE_ValueGuiVector3 * addVector3Parameter(const char *,
					    const char *, VEE_HasValues *,
					    float xv, float xmin, float xmax,
					    float yv, float ymin, float ymax,
					    float zv, float zmin, float zmax);
  void bangValues();
};

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

// SOLAR SOLAR SOLAR SOLAR SOLAR SOLAR SOLAR SOLAR SOLAR SOLAR SOLAR SOLAR
// SOLAR SOLAR SOLAR SOLAR SOLAR SOLAR SOLAR SOLAR SOLAR SOLAR SOLAR SOLAR

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

class Solar_Model
{
public:
  Solar_Model();
  ~Solar_Model();

  void scale(float s);
  void clear();
  
  /// Returns the number of sub-objects
    int subObjectCount() const;

  void calculateBoundingBox();

  void normalize(float maximumSpan, int center);

  void centralize();

  void createPyramid(float size, const char *baseMaterial, 
		     int shareMaterials);
  void createCube(float size, const char *baseMaterial,
		  int shareMaterials);
  void createFromHeightMap(const unsigned char *map, 
			   unsigned w, unsigned h,
			   const char *);

  %extend {
    void createFromHeightMapQI(const QImage &i, const char *material) {
      self->createFromHeightMap(i.bits(), i.width(), i.height(), material);
    }
  }

};

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

class Solar_Object
{
public:
  Solar_Object();
  virtual ~Solar_Object();
  
  virtual void update(float dt);
  
  void setModel(Solar_Model *m);
  Solar_Model *getModel();

  void setLocation(const VEE_Vector3 &);
  void setRotation(const VEE_Matrix3 &);
};

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////


class Solar_LwoConvert 
{
public:
  static int load(Solar_Model *, const char *);
};

class Solar_Loaders 
{
public:
  static int load(Solar_Model *, const char *);
};

%include "gee_solar_conversion.h"

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

class VEE_Image
{
public:
  VEE_Image();
  ~VEE_Image();

  int w() const;
  int h() const;
  int load(const char *fileName);
  int save(const char *fileName);
  void addAlpha();
  void alphaFromImageLuminance(const VEE_Image &img);

  void resize(int, int, int);
  void randomValues();
  void perlinNoise(int, double att, int octaves, double, double);

  void checkTexture(int horizontalInterval,
		    int verticalInterval);

  void skotchTexture(int stripeWidth,
		     int stripeInterval);

  void channelCopy(int targetChannel, int sourceChannel);
  void channelSwap(int c1, int c2);
  void boostRgb(int boost);
};

