/* COPYRIGHT
 *
 * This file is part of the Visual Effects Engine - VEE
 *
 * Read the "VEE-LICENSE" file for the license.
 *
 * Authors & Copyright:   
 *
 * Tommi Ilmonen, Tuukka Heikura, Marko Myllymaa and 
 * Janne Kontkanen 2001-2004
 *
 * Additional copyrights: Tekes 2003-2004
 *
 * firstname.lastname@hut.fi
 *
 */

#include <vee_value_gui.h>

#include <vee_error.h>
#include <vee_value_gui_float.h>
#include <vee_value_gui_vector3.h>

#include <qlayout.h>

VEE_ValueWindow::VEE_ValueWindow(QWidget * parent,
				 const char * name)
  : QWidget(parent, name)
{
  m_grid = new QGridLayout(this);
  setCaption("VEE parameters");
}

VEE_ValueWindow::~VEE_ValueWindow()
{
  VEE_trace("VEE_ValueWindow::~VEE_ValueWindow");
}

QWidget *VEE_ValueWindow::childParent()
{
  return this;
}

VEE_ValueGui * VEE_ValueWindow::addParameter
(VEE_Parameter *p, VEE_HasValues *o)
{
  VEE_ParameterFloat *pf = dynamic_cast<VEE_ParameterFloat *> (p);

  if(pf) {
    VEE_ValueGuiFloat *vgf = new VEE_ValueGuiFloat(this);
    vgf->build(this, o, *pf);
    m_guis.push_back(vgf);
    return vgf;
  }

  VEE_ParameterVector3 *pv3 = dynamic_cast<VEE_ParameterVector3 *> (p);

  if(pv3) {
    VEE_ValueGuiVector3 *vgv = new VEE_ValueGuiVector3(this);
    vgv->build(this, o, *pv3);
    m_guis.push_back(vgv);
    return vgv;
  }
  return 0;
}

VEE_ValueGuiFloat * VEE_ValueWindow::addFloatParameter
(const char *name,
 const char *id, VEE_HasValues *o,
 float v, float rmin, float rmax,
 VEE_Mapping m)
{
  VEE_ParameterFloat *pf = new VEE_ParameterFloat(v, rmin, rmax, m);
  pf->setName(name);
  pf->setId(id);
  return dynamic_cast<VEE_ValueGuiFloat *> (addParameter(pf, o));
}

VEE_ValueGuiVector3 * VEE_ValueWindow::addVector3Parameter
(const char *name,
 const char *id, VEE_HasValues *o,
 float xv, float xmin, float xmax,
 float yv, float ymin, float ymax,
 float zv, float zmin, float zmax)
{
  VEE_ParameterVector3 *pv = new VEE_ParameterVector3;
  
  pv->boundingBox().low().make(xmin, ymin, zmin);
  pv->boundingBox().high().make(xmax, ymax, zmax);

  VEE_Vector3 vect(xv, yv, zv);
  pv->setVector(vect);

  pv->setName(name);
  pv->setId(id);

  return dynamic_cast<VEE_ValueGuiVector3 *> (addParameter(pv, o));
}

void VEE_ValueWindow::bangValues()
{
  for(std::list<VEE_ValueGui *>::iterator it = m_guis.begin();
      it != m_guis.end(); it++) {
    (*it)->bangValue();
  }
}

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

VEE_ValueGui::VEE_ValueGui(QObject * parent,
			   const char * name)
  : QObject(parent, name) 
{}
  
VEE_ValueGui::~VEE_ValueGui() 
{}

void VEE_ValueGui::bangValue()
{}
