/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of the Visual Effects Engine - VEE
 *
 * Read the "VEE-LICENSE" file for the license.
 *
 * Authors & Copyright:   
 *
 * Tommi Ilmonen, Tuukka Heikura, Marko Myllymaa and 
 * Janne Kontkanen 2001-2004
 *
 * Additional copyrights: Tekes 2003-2004
 *
 * firstname.lastname@hut.fi
 *
 */

#ifndef VEE_VALUE_GUI_H
#define VEE_VALUE_GUI_H

#include <vee_parameters.h>

#include <qwidget.h>

#include <algorithm>
#include <iterator>
#include <list>
#include <string>
#include <vector>

class VEE_HasValues;
class VEE_Parameter;
class VEE_ValueGui;
class VEE_ValueGuiFloat;
class VEE_ValueGuiVector3;

class QGridLayout;

class VEE_ValueWindow : public QWidget
{
public:
  VEE_ValueWindow(QWidget * parent = 0,
		  const char * name = 0);
  virtual ~VEE_ValueWindow();

  QGridLayout *grid() { return m_grid; }

  QWidget *childParent();

  VEE_ValueGui * addParameter(VEE_Parameter *, VEE_HasValues *);

  VEE_ValueGuiFloat * addFloatParameter(const char *,
					const char *, VEE_HasValues *,
					float v, float rmin, float rmax,
					VEE_Mapping m = VEE_MAP_LINEAR);

  VEE_ValueGuiVector3 * addVector3Parameter(const char *,
					    const char *, VEE_HasValues *,
					    float xv, float xmin, float xmax,
					    float yv, float ymin, float ymax,
					    float zv, float zmin, float zmax);

  void bangValues();
protected:
  QGridLayout *m_grid;

  std::list<VEE_ValueGui *> m_guis;
};

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

class VEE_ParamTargets
{
public:
  VEE_ParamTargets() {}
  ~VEE_ParamTargets() {}

  void clear() { m_targets.clear(); }
  void append(VEE_HasValues *target, const char *id)
  { 
    Internal tmp(target, id);
    m_targets.push_back(tmp);
  }
  void append(VEE_HasValues *target, const std::string & id)
  { 
    Internal tmp(target, id);
    m_targets.push_back(tmp);
  }

  void appendMany(VEE_ParamTargets *pt)
  {
    /*std::copy(m_targets.begin(), m_targets.end(),
      std::back_insert_iterator<std::vector<Internal> >
      (pt->m_targets));*/
    for(uint i=0; i < pt->m_targets.size(); i++)
      m_targets.push_back(pt->m_targets[i]);
  }

  uint size() const { return m_targets.size(); }
  
  VEE_HasValues *target(uint i) { return m_targets[i].m_target; }
  const std::string & id(uint i) { return m_targets[i].m_id; }

protected:

  class Internal
  {
  public:
    Internal(VEE_HasValues *target, const char * id) 
      : m_target(target), m_id(id ? id : "") {}
    
    Internal(VEE_HasValues *target, const std::string & id) 
      : m_target(target), m_id(id) {}
    
    VEE_HasValues *m_target;
    std::string    m_id;
  };

  std::vector<Internal> m_targets;
};

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

class VEE_ValueGui : public QObject
{
public:
  VEE_ValueGui(QObject * parent = 0,
	       const char * name = 0);
  
  virtual ~VEE_ValueGui();

  virtual void bangValue();

  void addTarget(VEE_HasValues *target, const char * id)
  { m_targets.append(target, id); }
  void addTargets(VEE_ParamTargets *pt)
  { m_targets.appendMany(pt); }
  
protected:
  VEE_ParamTargets m_targets;
};

#endif
