/* COPYRIGHT
 *
 * This file is part of the Visual Effects Engine - VEE
 *
 * Read the "VEE-LICENSE" file for the license.
 *
 * Authors & Copyright:   
 *
 * Tommi Ilmonen, Tuukka Heikura, Marko Myllymaa and 
 * Janne Kontkanen 2001-2004
 *
 * Additional copyrights: Tekes 2003-2004
 *
 * firstname.lastname@hut.fi
 *
 */

#include <vee_value_gui_float.h>

#include <vee_error.h>
#include <vee_has_values.h>

#include <mj_gui_numeric.h> 
#include <mj_mapper.h> 
#include <mj_slider.h> 

#include <qlabel.h>
#include <qlayout.h>

#include <typeinfo>

VEE_ValueGuiFloat::VEE_ValueGuiFloat(QObject * parent,
				     const char * name)
  : VEE_ValueGui(parent, name)
{}

VEE_ValueGuiFloat::~VEE_ValueGuiFloat()
{}

void VEE_ValueGuiFloat::build(VEE_ValueWindow *host,
			      VEE_HasValues  *target,
			      const VEE_ParameterFloat &parameter)
{
  m_targets.append(target, parameter.id());
  m_value = parameter;

  QGridLayout *grid = host->grid();

  int row = grid->numRows();

  QWidget *childParent = host->childParent();
  
  MJ_GuiDoubleSlider *gs = new MJ_GuiDoubleSlider(this);
  gs->createObjs(childParent, true);
  gs->set(m_value.value(), m_value.minValue(),  m_value.maxValue());
  
  connect(gs, SIGNAL(emitValue(double)), this, SLOT(setValue(double)));
  connect(gs, SIGNAL(emitGetValue(double*)), this, SLOT(getValue(double*)));

  gs->label()->setText(m_value.name().c_str());
  
  gs->toGridHorizontal(grid, row, 0);
  gs->limMinSizes();

  if(parameter.mapping() == VEE_MAP_LOGARITHMIC)
    gs->slider()->setMapper(new MJ_MapperLog);
}

void VEE_ValueGuiFloat::bangValue()
{
  for(uint i=0; i < m_targets.size(); i++)
    m_targets.target(i)->setValueFloat
      (m_targets.id(i).c_str(), m_value.value());

  /* if(m_target)
     m_value.setValue(m_target); */
}

void VEE_ValueGuiFloat::setValue(double v)
{
  /* VEE_trace("VEE_ValueGuiFloat::setValue # \"%s\" %lf for \"%s\"", 
     m_value.name().c_str(), v, typeid(*m_target).name()); */

  if(v == m_value.value()) return;
  
  m_value.setFloat(v);

  for(uint i=0; i < m_targets.size(); i++) {

    bool ok = m_targets.target(i)->setValueFloat
      (m_targets.id(i).c_str(), m_value.value());

    if(!ok) 
      VEE_error("VEE_ValueGuiFloat::setValue # Failed to set \"%s\" for %s (%u / %u)",
		m_targets.id(i).c_str(), typeid(*m_targets.target(i)).name(), 
		i, (uint) m_targets.size());
  }
}

void VEE_ValueGuiFloat::getValue(double *v)
{
  *v = m_value.value();
}
