/* COPYRIGHT
 *
 * This file is part of the Visual Effects Engine - VEE
 *
 * Read the "VEE-LICENSE" file for the license.
 *
 * Authors & Copyright:   
 *
 * Tommi Ilmonen, Tuukka Heikura, Marko Myllymaa and 
 * Janne Kontkanen 2001-2004
 *
 * Additional copyrights: Tekes 2003-2004
 *
 * firstname.lastname@hut.fi
 *
 */

#include <vee_value_gui_vector3.h>

#include <vee_error.h>
#include <vee_has_values.h>

#include <mj_gui_numeric.h> 
#include <mj_mapper.h> 
#include <mj_slider.h> 

#include <qlabel.h>
#include <qlayout.h>

#include <typeinfo>

VEE_ValueGuiVector3::VEE_ValueGuiVector3(QObject * parent,
					 const char * name)
  : VEE_ValueGui(parent, name)
{}

VEE_ValueGuiVector3::~VEE_ValueGuiVector3()
{}

void VEE_ValueGuiVector3::build(VEE_ValueWindow *host,
				VEE_HasValues  *target,
				const VEE_ParameterVector3 &parameter)
{
  m_targets.append(target, parameter.id());
  m_value = parameter;

  QGridLayout *grid = host->grid();

  QWidget *childParent = host->childParent();

  const char * getNames[3] = {
    SLOT(getValue1(double *)),
    SLOT(getValue2(double *)),
    SLOT(getValue3(double *))
  };

  const char * setNames[3] = {
    SLOT(setValue1(double)),
    SLOT(setValue2(double)),
    SLOT(setValue3(double))
  };

  for(int i=0; i < 3; i++) {

    int row = grid->numRows();
    
    MJ_GuiDoubleSlider *gs = new MJ_GuiDoubleSlider(this);
    gs->createObjs(childParent, true);
    gs->set(m_value.vector()[i], 
	    m_value.boundingBox().low()[i],
	    m_value.boundingBox().high()[i]);
    
    connect(gs, SIGNAL(emitValue(double)), this, setNames[i]);
    connect(gs, SIGNAL(emitGetValue(double*)), this, getNames[i]);
    
    QString tmp;
    tmp.sprintf(" (%d)", i + 1);

    QString tmp2(m_value.name().c_str());

    gs->label()->setText(tmp2 + tmp);
  
    gs->toGridHorizontal(grid, row, 0);
    gs->limMinSizes();

    /* if(parameter.mapping() == VEE_MAP_LOGARITHMIC)
       gs->slider()->setMapper(new MJ_MapperLog); */
  }
}

void VEE_ValueGuiVector3::bangValue()
{
  for(uint i=0; i < m_targets.size(); i++)
    m_targets.target(i)->setValueVector3
      (m_targets.id(i).c_str(), m_value.vector());
}

void VEE_ValueGuiVector3::setValue1(double v)
{
  setValue(0, v);
}
void VEE_ValueGuiVector3::setValue2(double v)
{
  setValue(1, v);
}
void VEE_ValueGuiVector3::setValue3(double v)
{
  setValue(2, v);
}

void VEE_ValueGuiVector3::getValue1(double * v)
{
  getValue(0, v);
}
void VEE_ValueGuiVector3::getValue2(double * v)
{
  getValue(1, v);
}
void VEE_ValueGuiVector3::getValue3(double * v)
{
  getValue(2, v);
}

void VEE_ValueGuiVector3::setValue(int i, double v)
{
  VEE_Vector3 vect = m_value.vector();
  vect[i] = v;
  m_value.setVector(vect);
  
  for(uint i=0; i < m_targets.size(); i++) {

    bool ok = m_targets.target(i)->setValueVector3
      (m_targets.id(i).c_str(), m_value.vector());

    if(!ok) 
      VEE_error("VEE_ValueGuiVector3::setValue # Failed to set \"%s\"",
		m_targets.id(i).c_str());
  }
}

void VEE_ValueGuiVector3::getValue(int i, double *v)
{
  *v = m_value.vector()[i];
}
