/* COPYRIGHT
 *
 * This file is part of the Visual Effects Engine - VEE
 *
 * Read the "VEE-LICENSE" file for the license.
 *
 * Authors & Copyright:   
 *
 * Tommi Ilmonen, Tuukka Heikura, Marko Myllymaa and 
 * Janne Kontkanen 2001-2004
 *
 * Additional copyrights: Tekes 2003-2004
 *
 * firstname.lastname@hut.fi
 *
 */

#ifndef VEE_RMAN_RENDERER_H
#define VEE_RMAN_RENDERER_H

#include <vee_renderer.h>

#include <vee_vector3.h>

// #include <vector>

#include <string>

class VEE_SceneBox;

/** Renderman back-end. Generates RIB-files (Renderman Interface
    Bytestream).

    @author Tommi Ilmonen
*/

class VEE_RManRenderer : public VEE_Renderer
{
public:
  VEE_RManRenderer(float frameDuration);
  virtual ~VEE_RManRenderer();

  virtual void initRenderState();
  virtual void closeRenderState();

  virtual void renderParticle(VEE_Particle *);
  virtual void renderPolygon(VEE_Polygon *);
  virtual void renderForceObject(VEE_ForceObject *);

  void paintRecursive(VEE_SceneBox *);

  void setFrameDur(float dur) 
  { m_frameDuration = dur; }

  static void aimZ(const VEE_Vector3 &);
  
  static void placeCamera(const VEE_Vector3 &pos,
			  const VEE_Vector3 &dir,
			  float roll = 0.0);

protected:
  float m_particleSize;
  float m_frameDuration;

  std::string m_particleMaterial;
  std::string m_polygonMaterial;
  
  // std::vector<VEE_Vector3> m_buffer;
};

#endif
