/* COPYRIGHT
 *
 * This file is part of the Visual Effects Engine - VEE
 *
 * Read the "VEE-LICENSE" file for the license.
 *
 * Authors & Copyright:   
 *
 * Tommi Ilmonen, Tuukka Heikura, Marko Myllymaa and 
 * Janne Kontkanen 2001-2004
 *
 * Additional copyrights: Tekes 2003-2004
 *
 * firstname.lastname@hut.fi
 *
 */

#ifndef VEE_SDL_APP_H
#define VEE_SDL_APP_H

#include <Solar/GlMaterial.hpp>
#include <Solar/GlRenderer.hpp>
#include <Solar/GlDisplayLists.hpp>
#include <Solar/Model.hpp>

#include <vee_test_qt.h>

#include <di_time.h>

class VEE_SdlApp
{
public:

  enum Type {
    OBJECT_DISPLAY,
    TEXTURE_STREAM
  };

  class GlData
  {
  public:
    GlData() : m_renderer(&m_materials) {}

    void initGl(const std::string & mtlFile, bool useLists);

    VEE_GlTexture2DBank   m_textures;
    Solar::GlMaterialBank m_materials;
    Solar::GlRenderer     m_renderer;
    Solar::GlDisplayLists m_lists;
  };

  VEE_SdlApp();
  ~VEE_SdlApp();

  void setMtlFile(const std::string &f) { m_mtlFile = f; }

  void addModel(Solar::Model *model) { m_models.push_back(model); }
  void addObject(Solar::Object *object) { m_objects.push_back(object); }
  void useLists(bool use) { m_withLists = use; }

  void setMaxFrames(uint frames) { m_maxFrames = frames; }

  void setAppType(Type t) { m_type = t; }
  Type appType() const { return m_type; }

  void init();
  void initGl(GlData *);
  void update();
  void draw(GlData *);

  bool isDone();
  
protected:  

  std::vector<Solar::Model *>  m_models;
  std::vector<Solar::Object *> m_objects;

  std::string           m_mtlFile;
  
  bool m_withLists;

  uint m_frame;
  uint m_maxFrames;
  float m_maxTime; // Max time in seconds

  Type m_type;

  /// Time of the constructor call
  DI_Time m_constructionTime;
  /// Time of the second frame
  DI_Time m_startTime;
};


#endif

