/* COPYRIGHT
 *
 * This file is part of the Visual Effects Engine - VEE
 *
 * Read the "VEE-LICENSE" file for the license.
 *
 * Authors & Copyright:   
 *
 * Tommi Ilmonen, Tuukka Heikura, Marko Myllymaa and 
 * Janne Kontkanen 2001-2004
 *
 * Additional copyrights: Tekes 2003-2004
 *
 * firstname.lastname@hut.fi
 *
 */

#include <vee_vrs_app.h>

static DI_MutexAuto mutex;

VEE_VrsApp::VEE_VrsApp(VEE_SdlApp * app)
  : m_app(app)
{}

VEE_VrsApp::~VEE_VrsApp()
{}

void VEE_VrsApp::contextInit(VRS_Window * context)
{
  VEE_SdlApp::GlData * gld = new VEE_SdlApp::GlData();

  m_app->initGl(gld);

  mutex.lock();
  m_glData[context] = gld;
  mutex.unlock();
}

void VEE_VrsApp::contextCleanup(VRS_Window * context)
{
  iterator it = m_glData.find(context);

  if(it == m_glData.end())
    return;

  VEE_SdlApp::GlData * gld = (*it).second;

  delete gld;

  mutex.lock();
  m_glData.erase(it);
  mutex.unlock();
}

void VEE_VrsApp::draw(VRS_Window * context, int eye)
{
  glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

  glColor3f(1, 1, 1);

  iterator it = m_glData.find(context);

  if(it == m_glData.end())
    return;

  m_app->draw((*it).second);
}


void VEE_VrsApp::preFrame()
{
  m_app->update();
  
  if(m_app->isDone())
    quit(true);
}
