/* COPYRIGHT
 *
 * This file is part of the Simple Object Loader and Renderer - SOLAR
 *
 * Read the file "SOLAR-LICENSE" for the license.
 *
 * Authors & Copyright:
 *
 * Tommi Ilmonen, 2001-2003, except LWO-code by RJ, 2001-2003
 *
 * Tommi's email: firstname.lastname@hut.fi
 *
 */

#include <Solar/AsciiRenderer.hpp>

#include <Solar/Error.hpp>
#include <Solar/GraphicTriangles.hpp>
#include <Solar/GraphicTriangleMesh.hpp>
#include <Solar/GraphicTriangleStrips.hpp>
#include <Solar/Model.hpp>
#include <Solar/Stream.hpp>

#include <iostream>

namespace Solar {

AsciiRenderer::AsciiRenderer() 
  : m_stream(0)
{}

AsciiRenderer::AsciiRenderer(std::ostream *s) 
  : m_stream(s)
{}

void AsciiRenderer::prepareToRender()
{}

void AsciiRenderer::begin(Model *m)
{
  if(!m_stream) fatal("AsciiRenderer::begin # m_stream is NULL");

  (*m_stream) << "MODEL {\nSUB-OBJECTS " << m->subObjectCount() << "\n";
}

void AsciiRenderer::end()
{
  (*m_stream) << "}\n";
}

void AsciiRenderer::renderTriangles(GraphicTriangles *triangles)
{
  preRender(triangles);

  triangles->write(m_stream);
}

void AsciiRenderer::renderTriangleMesh(GraphicTriangleMesh *mesh)
{
  preRender(mesh);

  mesh->write(m_stream);
}

void AsciiRenderer::renderTriangleStrips(GraphicTriangleStrips *strips)
{
  uint n = strips->nofStrips() ;

  (*m_stream) << "TRIANGLE_STRIPS " << n << "\n";
  
  for(uint i=0; i < n; i++) {
    GraphicTriangleStrips::Strip strip = strips->strip(i);
    
    (*m_stream) << "STRIP " << strip.m_length << "\n";
    
    for(uint j=0; j < strip.m_length; j++) {

      GraphicTriangleStrips::Vertex v = strips->vertex(i, j);

      (*m_stream) << "loc = " << v.m_location
		  << "n = " << v.m_normal
		  << "uv = " << v.m_uv << "\n";
    }
  }
}

void AsciiRenderer::preRender(Graphic *gr)
{
  (*m_stream) << "MATERIAL = \"" << gr->material() << "\"\n";
}

void AsciiRenderer::pushTransform(const Fluid::Matrix3 &rot,
				  const Fluid::Vector3 &loc,
				  float scale)
{  
  (*m_stream) << "ROTATE = ";  
  Write(m_stream, rot);

  (*m_stream) << "\nTRANSLATE = ";
  Write(m_stream, loc);

  (*m_stream) << "\nSCALE = ";
  (*m_stream) <<  scale;
  (*m_stream) << "\n{\n";
  
}

void AsciiRenderer::popTransform()
{
  (*m_stream) << "}\n";
}



} // namespace Solar
