/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of the Simple Object Loader and Renderer - SOLAR
 *
 * Read the file "SOLAR-LICENSE" for the license.
 *
 * Authors & Copyright:
 *
 * Tommi Ilmonen, 2001-2003, except LWO-code by RJ, 2001-2003
 *
 * Tommi's email: firstname.lastname@hut.fi
 *
 */

#ifndef SOLAR_ASCII_RENDERER_HPP
#define SOLAR_ASCII_RENDERER_HPP

#include <Solar/Renderer.hpp>

#include <iosfwd>

namespace Solar {

class GlMaterialBank;
class Graphic;
class GraphicTriangles;
class Model;

/** Class that can render Solar objects with OpenGL. 

\code
  Solar::GlRenderer renderer(getMyMaterials());
  renderer.initGlState();
  Solar::Object *object = getMyObject();
  object->render(&renderer);
\endcode

    @author Tommi Ilmonen
*/

class AsciiRenderer : public Renderer
{
public:

  AsciiRenderer();
  AsciiRenderer(std::ostream *);

  ~AsciiRenderer() {}
  virtual void prepareToRender();
  
  virtual void begin(Model *);
  virtual void end();

  virtual void renderTriangles(GraphicTriangles *);
  virtual void renderTriangleMesh(GraphicTriangleMesh *);
  virtual void renderTriangleStrips(GraphicTriangleStrips *);

  virtual void pushTransform(const Fluid::Matrix3 &,
			     const Fluid::Vector3 &,
			     float scale = 1.0);
  virtual void popTransform();

  void setStream(std::ostream *s) { m_stream = s; }

protected:

  virtual void preRender(Graphic *);

  std::ostream *m_stream;
};

} // namespace Solar

#endif
