/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of the Simple Object Loader and Renderer - SOLAR
 *
 * Read the file "SOLAR-LICENSE" for the license.
 *
 * Authors & Copyright:
 *
 * Tommi Ilmonen, 2001-2003, except LWO-code by RJ, 2001-2003
 *
 * Tommi's email: firstname.lastname@hut.fi
 *
 */

#ifndef SOLAR_GRAPHIC_HPP
#define SOLAR_GRAPHIC_HPP

#include <Fluid/Vector3.hpp>
#include <Fluid/Matrix3.hpp>

#include <iosfwd>
#include <string>

#include <Solar/BoundingBox.hpp>
#include <Solar/Garbage.hpp>

namespace Solar {

  // class BoundingBox;
class Renderer;

/** Visual sub-object in a model. Child classes match the normal
    hardware/software graphics rendering primitives. */

  class Graphic : public Garbage
  {
  public:

  enum Type {
    Triangles,
    TriangleMesh,
    TriangleStrips,
    Extension
  };

  Graphic() : m_generation(0) {}
  virtual ~Graphic() {}

  /// This method returns the type of the object.
  virtual Type type() const = 0;
  /// The bounding box of the object in local coordinates.
  virtual void getBoundingBox(BoundingBox *bbox) = 0;

  virtual int nofPrimitives() const = 0;

  virtual void render(Renderer *) = 0;
  virtual void scale(const Fluid::Vector3 &) = 0;
  virtual void move(const Fluid::Vector3 &) = 0;
  virtual void transform(const Fluid::Matrix3 &) = 0;

  virtual void write(std::ostream *) = 0;

  const std::string & material() const { return m_material; }
  void setMaterial(const std::string &material) { m_material = material; }
  void setMaterial(const char *material) { m_material = material; }

  int generation() const { return m_generation; }
  void newGeneration() { m_generation++; }

private:
  std::string m_material;
  int m_generation;
};

} // namespace Solar


#endif
