/* COPYRIGHT
 *
 * This file is part of the Simple Object Loader and Renderer - SOLAR
 *
 * Read the file "SOLAR-LICENSE" for the license.
 *
 * Authors & Copyright:
 *
 * Tommi Ilmonen, 2001-2003, except LWO-code by RJ, 2001-2003
 *
 * Tommi's email: firstname.lastname@hut.fi
 *
 */

#ifndef SOLAR_GRAPHIC_TRIANGLES_HPP
#define SOLAR_GRAPHIC_TRIANGLES_HPP

#include <Fluid/Vector2.hpp>

#include <Solar/BoundingBox.hpp>
#include <Solar/Graphic.hpp>

#include <vector>

namespace Solar {

class GraphicTriangles : public Graphic
{
public:
  GraphicTriangles();
  virtual ~GraphicTriangles();

  class Vertex {
  public:
    Fluid::Vector3 m_vertex;
    Fluid::Vector3 m_normal;
    Fluid::Vector2 m_uv;
  };

  class Triangle {
  public:

    void scale(const Fluid::Vector3 &v)
    {
      for(uint i=0; i < 3; i++)
	m_vertices[i].m_vertex.scale(v);
    }

    void move(const Fluid::Vector3 &v)
    {
      for(uint i=0; i < 3; i++)
	m_vertices[i].m_vertex += v;
    }

    void transform(const Fluid::Matrix3 &m)
    {
      for(uint i=0; i < 3; i++) {
	m_vertices[i].m_vertex = m * m_vertices[i].m_vertex;
	m_vertices[i].m_normal = m * m_vertices[i].m_normal;
      }
    }

    /// Set all normals
    void setNormals(const Fluid::Vector3 &v)
    {
      for(uint i=0; i < 3; i++)
	m_vertices[i].m_normal = v;
    }

    Vertex m_vertices[3];

    void clear();
    void write(std::ostream *);
  };

  virtual Type type() const;
  virtual void getBoundingBox(BoundingBox *bbox);
  virtual int nofPrimitives() const;
  virtual void render(Renderer *);
  virtual void scale(const Fluid::Vector3 &);
  virtual void move(const Fluid::Vector3 &);
  virtual void transform(const Fluid::Matrix3 &);
  virtual void write(std::ostream *);

  void clear();

  void addTriangle(const Triangle &);

  uint nofTriangles() const { return m_triangles.size(); }
  
  const Triangle &triangle(uint i) const { return m_triangles[i]; }

  void recalculateBoundingBox();

  uint bytes() const { return m_triangles.size() * sizeof(Triangle); }

protected:

  BoundingBox m_boundingBox;

  std::vector<Triangle> m_triangles;
};

} // namespace Solar

#endif
