
/* COPYRIGHT
 *
 * This file is part of the Simple Object Loader and Renderer - SOLAR
 *
 * Read the file "SOLAR-LICENSE" for the license.
 *
 * Authors & Copyright:
 *
 * Tommi Ilmonen, 2001-2003, except LWO-code by RJ, 2001-2003
 *
 * Tommi's email: firstname.lastname@hut.fi
 *
 */

#include <Solar/Object.hpp>

#include <Solar/Model.hpp>
#include <Solar/Renderer.hpp>

namespace Solar {

  Object::Object() 
    : m_visible(true),
      m_model(0),
      m_scale(1.0f)
  {
    m_rotation.identity();
    m_location.clear();
  }

  Object::Object(Model *model) 
    : m_visible(true),
      m_model(model),
      m_scale(1.0f)
  {
    m_rotation.identity();
    m_location.clear();
  }

  void Object::update(float)
  {}

  void Object::render(Renderer *r)
  { 
    if(!m_visible)
      return;

    if(m_model) {
      r->pushTransform(m_rotation, m_location, m_scale);
      m_model->render(r); 
      r->popTransform();
    }
  }

} // namespace Solar
