/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of the Simple Object Loader and Renderer - SOLAR
 *
 * Read the file "SOLAR-LICENSE" for the license.
 *
 * Authors & Copyright:
 *
 * Tommi Ilmonen, 2001-2003, except LWO-code by RJ, 2001-2003
 *
 * Tommi's email: firstname.lastname@hut.fi
 *
 */

#ifndef SOLAR_OBJECT_HPP
#define SOLAR_OBJECT_HPP

#include <Fluid/Matrix3.hpp>
#include <Fluid/Vector3.hpp>

namespace Solar {

  class Model;
  class Renderer;

  /** An object class. 

  */

  class Object
  {
  public:
    Object();
    Object(Model *model);

    virtual ~Object() {}

    virtual void update(float dt);

    void render(Renderer *r);
  
    void setModel(Model *m) { m_model = m; }
    Model *getModel() { return m_model; }

    Fluid::Matrix3 & getRotation() { return m_rotation; }
    const Fluid::Matrix3 & getRotation() const { return m_rotation; }
    void setRotation(const Fluid::Matrix3 &v) { m_rotation = v; }

    Fluid::Vector3 & getLocation() { return m_location; }
    const Fluid::Vector3 & getLocation() const { return m_location; }
    void setLocation(const Fluid::Vector3 &v) { m_location = v; }
  
    bool isVisible() const { return m_visible; }
    void setVisible(bool v) { m_visible = v; }

    float getScale() const { return m_scale; }
    void setScale(float s) { m_scale = s; }

  protected:
    bool   m_visible;
    Model *m_model;

    Fluid::Matrix3 m_rotation;
    Fluid::Vector3 m_location;
    float m_scale;
  };

} // namespace Solar

#endif
