/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of the Simple Object Loader and Renderer - SOLAR
 *
 * Read the file "SOLAR-LICENSE" for the license.
 *
 * Authors & Copyright:
 *
 * Tommi Ilmonen, 2001-2003, except LWO-code by RJ, 2001-2003
 *
 * Tommi's email: firstname.lastname@hut.fi
 *
 */

#ifndef SOLAR_RENDERER_HPP
#define SOLAR_RENDERER_HPP

#include <Fluid/Matrix3.hpp>
#include <Fluid/Vector3.hpp>

namespace Solar {

  class Model;
  class GraphicTriangles;
  class GraphicTriangleMesh;
  class GraphicTriangleStrips;

  /** Base class for renderers that can handle Model objects. 

  A different renderer should be made for each graphics rendering
  method (OpenGL, Renderman, D3D). The virtual methods match all
  Graphic child classes.

  @author Tommi Ilmonen
  */

  class Renderer
  {
  public:
    Renderer() {}
    virtual ~Renderer() {}

    virtual void prepareToRender() = 0;

    /** Begins to draw a model. */
    virtual void begin(Model *) = 0;
    /** End drawing the current model. */
    virtual void end() = 0;

    /** Render the triangles. */
    virtual void renderTriangles(GraphicTriangles *) = 0;
    /** Render the triangles. */
    virtual void renderTriangleMesh(GraphicTriangleMesh *) = 0;
    /** Render the triangles. */
    virtual void renderTriangleStrips(GraphicTriangleStrips *) = 0;
  
    /** Pushes a new transform to the system. */
    virtual void pushTransform(const Fluid::Matrix3 &,
			       const Fluid::Vector3 &,
			       float scale = 1.0) = 0;
    /** Removes a trnasform from the transform stack. */
    virtual void popTransform() = 0;

  };

} // namespace Solar

#endif
