/* COPYRIGHT
 *
 * This file is part of the Simple Object Loader and Renderer - SOLAR
 *
 * Read the file "SOLAR-LICENSE" for the license.
 *
 * Authors & Copyright:
 *
 * Tommi Ilmonen, 2001-2003, except LWO-code by RJ, 2001-2003
 *
 * Tommi's email: firstname.lastname@hut.fi
 *
 */

#include <Solar/GlDisplayLists.hpp>

#include <Solar/GlRenderer.hpp>

#include <Solar/Garbage.hpp>

namespace Solar {
 
  void GlDisplayLists::List::freeList() 
  { 
    if(m_list) {
      glDeleteLists(m_list, 1); 
      m_list = 0; 
    }
#ifdef HAVE_GL_BUFFERS_ARB
    if(m_buffer) {
      glDeleteBuffersARB(1, & m_buffer); 
      m_list = 0; 
    }
#endif
    m_generation = -1;
  }

  GLuint GlDisplayLists::List::listId()
  { 
    if(!m_list) 
      m_list = glGenLists(1); 
    return m_list; 
  }

  GLuint GlDisplayLists::List::bufferId()
  {
#ifdef HAVE_GL_BUFFERS_ARB
    if(!m_buffer)
      glGenBuffersARB(1, & m_buffer);    

    return m_buffer;
#else
    return 0;
#endif
  }

  GlDisplayLists::GlDisplayLists()
  {}

  GlDisplayLists::~GlDisplayLists()
  {}

  void GlDisplayLists::freeLists(GarbageBin *items)
  {
    for(GarbageBin::iterator it = items->m_items.begin();
	it != items->m_items.end(); it++) {

      iterator myIt = m_storage.find(*it);

      if(myIt != m_storage.end()) {
	(*myIt).second.ref().freeList();
	m_storage.erase(myIt);
      }
	
    }
  }

  GlDisplayLists::List & GlDisplayLists::getList(void *id)
  {
    return m_storage[id].ref();
  }

}
