/* COPYRIGHT
 *
 * This file is part of the Simple Object Loader and Renderer - SOLAR
 *
 * Read the file "SOLAR-LICENSE" for the license.
 *
 * Authors & Copyright:
 *
 * Tommi Ilmonen, 2001-2003, except LWO-code by RJ, 2001-2003
 *
 * Tommi's email: firstname.lastname@hut.fi
 *
 */

#ifndef SOLAR_GL_DISPLAY_LISTS_HPP
#define SOLAR_GL_DISPLAY_LISTS_HPP

#include <Solar/Renderer.hpp>

#include <vee_reference_object.h>

#include <GL/gl.h>

#include <map>
#include <set>

namespace Solar {

  class GarbageBin;

  class GlDisplayLists 
  {
  public:
    
    class List
    {
    public:
      List() : m_list(0), m_buffer(0), m_generation(-1) {}
      
      bool hasList() { return m_list != 0; }
      void freeList();

      bool needsUpdate(int generation) 
      { return m_generation != generation; }
      
      GLuint listId();
      GLuint bufferId();

      void setGeneration(int generation) { m_generation = generation; }

    protected:

      GLuint m_list;
      GLuint m_buffer;
      int    m_generation;
    };

    GlDisplayLists();
    ~GlDisplayLists();

    void freeLists(GarbageBin *items);

    List & getList(void *id);

    uint size() const { return m_storage.size(); }

  protected:

    typedef std::map<void *, VEE_ReferenceObject<List> >::iterator iterator;

    std::map<void *, VEE_ReferenceObject<List> > m_storage;

  };
  

}

#endif
