/* COPYRIGHT
 *
 * This file is part of the Simple Object Loader and Renderer - SOLAR
 *
 * Read the file "SOLAR-LICENSE" for the license.
 *
 * Authors & Copyright:
 *
 * Tommi Ilmonen, 2001-2003, except LWO-code by RJ, 2001-2003
 *
 * Tommi's email: firstname.lastname@hut.fi
 *
 */

#ifndef SOLAR_GL_RENDERER_HPP
#define SOLAR_GL_RENDERER_HPP

#include <GL/gl.h>

#ifdef __OS_LINUX__

// NVidia fixes...
#define GL_GLEXT_PROTOTYPES 1

#ifndef GLAPI
#define GLAPI 
#endif

#ifdef APIENTRY
#undef APIENTRY
#endif

#include <GL/glext.h>

#endif

#include <Solar/GlDisplayLists.hpp>
#include <Solar/Renderer.hpp>

namespace Solar {

  class GlDisplayLists;
  class GlMaterialBank;
  class GlMaterial;
  class Graphic;
  class GraphicTriangles;
  class Model;

  /** Class that can render Solar objects with OpenGL. 

  \code
  Solar::GlRenderer renderer(getMyMaterials());
  renderer.initGlState();
  Solar::Object *object = getMyObject();
  object->render(&renderer);
  \endcode

  @author Tommi Ilmonen
  */

  class GlRenderer : public Renderer
  {
  public:

    GlRenderer(GlMaterialBank *materials);
    GlRenderer();

    virtual ~GlRenderer();
  
    virtual void prepareToRender();

    /// Sets the materials without deleting the old materials.
    void setMaterials(GlMaterialBank *materials) { m_materials = materials; }

    virtual void begin(Model *);
    virtual void end();

    virtual void renderTriangles(GraphicTriangles *);
    virtual void renderTriangleMesh(GraphicTriangleMesh *);
    virtual void renderTriangleStrips(GraphicTriangleStrips *);

    virtual void pushTransform(const Fluid::Matrix3 &,
			       const Fluid::Vector3 &,
			       float scale);
    virtual void popTransform();

    void initGlState();
    void setDisplayLists(GlDisplayLists * lists) { m_displayLists = lists; }

    void applyGlTexture2D(GLuint tex);
    void applyGlBlend(bool blend);
    void applyGlLighting(bool lighting);
    void applyGlSmooth(bool smooth);
    void applyGlTwoSided(bool twoSided);

    long listRenderCount() const { return m_listRenderCount; }

  protected:

    enum ListMode {
      RENDERED_LIST,
      GENERATE_LIST,
      NO_LIST
    };

    virtual void preRender(Graphic *);
    ListMode renderDisplayList(Graphic *);

    GlMaterialBank *m_materials;
    const GlMaterial * m_currentMaterial;

    GlDisplayLists *m_displayLists;
    GlDisplayLists::List *m_currentList;

    GLuint          m_currentTexture;
    bool            m_blend;
    bool            m_lighting;
    bool            m_smooth;
    bool            m_twoSided;
    long            m_listRenderCount;
  };

} // namespace Solar

#endif
