/* COPYRIGHT
 *
 * This file is part of the Simple Object Loader and Renderer - SOLAR
 *
 * Read the file "SOLAR-LICENSE" for the license.
 *
 * Authors & Copyright:
 *
 * Tommi Ilmonen, 2001-2003, except LWO-code by RJ, 2001-2003
 *
 * Tommi's email: firstname.lastname@hut.fi
 *
 */

#include <Solar/MtlParser.hpp>

#include <Solar/Error.hpp>

#include <fstream>

#include <strings.h>

namespace Solar {

  bool MtlParser::load(ConfigReader::Config *conf, const char * filename)
  {
    const char * fname = "MtlParser::load";

    std::ifstream in(filename);
    std::string line;
    
    int count = 0;

    std::string currentMaterial, s1, s2;

    ConfigReader::Chunk mtl;

    if(!in.good()) {
      error("%s # Failed to open file \"%s\"\n", fname, filename);
      return false;
    }

    while(in.good() && !in.eof()) {

      line.resize(2048);
      bzero(&line[0], line.size());
      in.getline((char *) line.c_str(), 2048);
      line = simplify(line);

      s1 = "";
      s2 = "";
      
      split(&s1, &s2, line);

      /* trace("s1 = \"%s\" n = %u, s2 = \"%s\" line = \"%s\"", 
	 s1.c_str(), s1.size(), s2.c_str(), line.c_str()); */
      
      if(!s1.size() || !s2.size() || !line.size())
	;
      else if(s1 == "newmtl") {

	if(mtl.size()) {
	  conf->set(currentMaterial, mtl);
	  mtl.clear();
	  count ++;
	}
	currentMaterial = s2;
      }
      else {
	mtl.set(s1, s2);
      }
    }

    if(mtl.size()) {
      conf->set(currentMaterial, mtl);
      count ++;
    }

    return count;
  }

  std::string MtlParser::simplify(const std::string & s)
  {
    std::string str = s;
    
    uint i;
    int j;

    // Strip leading white-spaces:
    for(i=0; i < str.size();) {
      if(isspace(str[i]))
	str.erase(str.begin() + i);
      else
	i = str.size();
    }

    // Strip tailing white-spaces:
    for(j = str.size() - 1; j >= 0; j--) {
      if(isspace(str[j]))
	str.erase(str.begin() + j);
      else
	j = -1;
    }

    // Strip comments:
    for(i=0; i < str.size(); i++) {
      if(str[i] == '#')
	str.erase(str.begin() + i, str.end());
    }
    
    return str.c_str();
  }

  void MtlParser::split(std::string * header, 
			std::string * rest, 
			const std::string & name)
  {    
    std::string tmp = name;
    
    tmp = simplify(tmp);
    
    *header = tmp;

    uint i;

    for(i=0; i < tmp.size(); i++) {
      if(isspace((*header)[i])) {
	header->erase(header->begin() + i, header->end());
	*header = simplify(*header);
	*rest = tmp;
	rest->erase(rest->begin(), rest->begin() + i);
	*rest = simplify(*rest);
	return;
      }
    }
    
    header->clear();
    rest->clear();
  }
  
} // namespace Solar
