/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of the Visual Effects Engine - VEE
 *
 * Read the "VEE-LICENSE" file for the license.
 *
 * Authors & Copyright:   
 *
 * Tommi Ilmonen, Tuukka Heikura, Marko Myllymaa and 
 * Janne Kontkanen 2001-2004
 *
 * Additional copyrights: Tekes 2003-2004
 *
 * firstname.lastname@hut.fi
 *
 */

#ifndef VEE_TEST_QT_H
#define VEE_TEST_QT_H

#include <vee_gl_texture.h>
#include <vee_qt.h>
#include <vee_time.h>

#include <Solar/GlMaterial.hpp>
#include <Solar/GlRenderer.hpp>
#include <Solar/GlDisplayLists.hpp>

#include <vector>

class VEE_RManRenderer;

namespace Solar {
  class Model;
  class Object;

} // namespace Solar

class VEE_TestQt : public VEE_QtHidden
{
public:
  VEE_TestQt(VEE_Qt * host,
	     VEE_SceneBox *box, 
	     const char *rendermanFile,
	     uint width, 
	     uint height,
	     VEE_GlRendererType type = VEE_GL_SL,
	     QWidget *parent=0, 
	     const char *name=0);
  virtual ~VEE_TestQt();

  void deleteSolarObjectsAndModels();
  
  void addModel(Solar::Model *model) { m_models.push_back(model); }
  void addObject(Solar::Object *object) { m_objects.push_back(object); }
  
  Solar::GlMaterialBank & materials() { return m_materials; }
  
  void setMtlFile(const std::string &f) { m_mtlFile = f; }

  static void loadMaterials(const std::string     &mtlFile, 
			    VEE_GlTexture2DBank   *textures,
			    Solar::GlMaterialBank *materials);

protected:
  
  virtual void initializeGL();

  virtual void veeRender(bool clear = true);

  VEE_Time m_lastUpdate;

  std::vector<Solar::Model *>  m_models;
  std::vector<Solar::Object *> m_objects;

  VEE_GlTexture2DBank   m_textures;
  Solar::GlMaterialBank m_materials;
  Solar::GlRenderer     m_renderer;
  Solar::GlDisplayLists m_lists;

  std::string           m_mtlFile;

  VEE_RManRenderer *m_rmanRenderer;

  /* private:
     VEE_TestQt(const VEE_TestQt &) {} */
};

#endif
