/* -*- C++ -*- */

/* COPYRIGHT
 *
 * This file is part of the Visual Effects Engine - VEE
 *
 * Read the "VEE-LICENSE" file for the license.
 *
 * Authors & Copyright:   
 *
 * Tommi Ilmonen, Tuukka Heikura, Marko Myllymaa and 
 * Janne Kontkanen 2001-2004
 *
 * Additional copyrights: Tekes 2003-2004
 *
 * firstname.lastname@hut.fi
 *
 */


#ifndef VEE_CALCULUS_THREAD_H
#define VEE_CALCULUS_THREAD_H

#include <di_condition.h>
#include <di_thread.h>

#include <vector>

class VEE_SceneBox;

class VEE_CalculusThread : public DI_Thread
{
public:
  enum State {
    IDLE,
    VELOCITY,
    VELOCITYDONE,
    LOCATION,
    LOCATIONDONE
  };

  VEE_CalculusThread();
  virtual ~VEE_CalculusThread();

  void start(VEE_SceneBox * scene,
	     int calculusThreshold,
	     int beginBox,
	     int endBox,
	     float dt);
	     

  void doVelocityUpdate();
  void waitVelocityUpdate();
  void doLocationUpdate();
  void waitLocationUpdate();
  
  void setDt(float dt) { m_dt = dt; }

  void stop();

protected:

  virtual void childLoop();

  DI_Condition m_cond;
  DI_MutexAuto m_mutex;

  VEE_SceneBox * m_scene;

  int m_calculusThreshold;
  int m_beginBox;
  int m_endBox;
  float m_dt;

  State m_allowed;
  State m_current;

  bool  m_continue;
};

/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////

class VEE_CalculusThreads
{
public:
  VEE_CalculusThreads();
  ~VEE_CalculusThreads();

  void setScene(VEE_SceneBox * scene) { m_scene = scene; }

  void addThread(int calculusThreshold,
		 int beginBox,
		 int endBox);

  void setDt(float dt);

  void update();

  void stop();

  void setTreeParams(uint splitAt,
		     uint collapseAt,
		     uint maximumRecursion)
  {
    m_splitAt = splitAt;
    m_collapseAt = collapseAt;
    m_maximumRecursion = maximumRecursion;
  }

  void init5(int calculusThreshold, bool linearSplit);

protected:
  uint m_splitAt;
  uint m_collapseAt;
  uint m_maximumRecursion;

  int  m_method;
  float m_dt;

  VEE_SceneBox * m_scene;  

  std::vector<VEE_CalculusThread *> m_threads;
};

#endif
